/*----------------------------------------------------------
#
#	NewsWatcher	- Macintosh NNTP Client Application
#
#	Written by Steven Falkenburg
#	1990 Apple Computer, Inc.
#
#-----------------------------------------------------------
#
#	filter.c
#
#	This module handles the reduction filter.
#
#	The reduction filter feature was removed in version 1.3d6.
#	This source code is being kept just in case we need it some
#	day - it is no longer compiled or linked or part of the project.
#
#-----------------------------------------------------------*/

#pragma segment filter
#pragma load ":objects:MacHeaders"

#include <string.h>
#include <ctype.h>

#include "glob.h"
#include "draw.h"
#include "filter.h"
#include "open.h"
#include "resize.h"
#include "search.h"
#include "util.h"


/*	ForceUpdate is called by the narrowing filter routines to
	invalidate the list area, forcing an update event to be
	generated for the area.
*/

static void ForceUpdate (WindowPtr wind)
{
	GrafPtr savePort;

	GetPort(&savePort);
	SetPort(wind);
	InvalRect(&wind->portRect);
	SetPort(savePort);
	HandleUpdate(wind);
}


/*	EndFilter is called when a narrowing filter has been completed.
*/

static void EndFilter (WindowPtr wind)
{
	TWindow **info;
	ListHandle doneList;
	GrafPtr savePort;

	info = (TWindow**)GetWRefCon(wind);
	doneList = (**info).theList;
	LActivate(true,(**info).fullList);
	LDispose(doneList);
	(**info).theList = (**info).fullList;
	(**info).fullList = nil;
	*(**info).filter = 0;
	(**info).filterLen = 0;
	GetPort(&savePort);
	SetPort(wind);
	SizeContents(wind,
		wind->portRect.right - wind->portRect.left,
		wind->portRect.bottom - wind->portRect.top);
	SetPort(savePort);
	ForceUpdate(wind);
}


/*	RebuildFilterList rebuilds the filter list from the full list, using
	the filter string.
*/

static void RebuildFilterList (WindowPtr wind)
{
	TWindow **info;
	ListHandle theList, fullList;
	short kind, filterLen, cellData, cellDataLen, numCells;
	TGroup **groupArray;
	TSubject **subjectArray;
	CStr255 filter, theString;
	Cell theCell, newCell;
	Handle strings;
	
	info = (TWindow**)GetWRefCon(wind);
	
	if ((**info).filterLen == 0) {
		EndFilter(wind);
		return;
	}
	
	kind = (**info).kind;
	theList = (**info).theList;
	fullList = (**info).fullList;
	
	if (fullList == nil) {
		NewList(wind);
		fullList = (**info).fullList = theList;
		theList = (**info).theList;
		LActivate(false, fullList);
		(**theList).lClikLoop = (**fullList).lClikLoop;
	} else {
		LDoDraw(false, theList);
		LDelRow(0, 0, theList);
	}
		
	filterLen = (**info).filterLen;
	strncpy(filter, (**info).filter, filterLen);
	
	strings = (**info).strings;
	if (kind == cSubject) {
		subjectArray = (**info).subjectArray;
	} else {
		groupArray = (**info).groupArray;
	}
	
	SetPt(&newCell,0,0);
	numCells = (**fullList).dataBounds.bottom;
	theCell.h = 0;
	for (theCell.v = 0; theCell.v < numCells; theCell.v++) {
		cellDataLen = 2;
		LGetCell(&cellData, &cellDataLen, theCell, fullList);
		if (kind == cSubject) {
			strcpy(theString, *strings + (*subjectArray)[cellData].subjectOffset);
		} else {
			strcpy(theString, *strings + (*groupArray)[cellData].nameOffset);
		}
		if (MySubstringSearch(theString, filter, strlen(theString), filterLen) == 0) {
			LAddRow(1, newCell.v, theList);
			cellDataLen = 2;
			LGetCell(&cellData, &cellDataLen, theCell, fullList);
			LSetCell(&cellData, 2, newCell, theList);
			newCell.v++;
		}
	}
	LDoDraw(true, theList);
	
	ForceUpdate(wind);
}


/*	NarrowFilterList is called by the narrowing search filter when
	a non-backspace key is pressed in the dialog.  It searches
	the secondary list for entries which may be removed.
*/

static void NarrowFilterList (WindowPtr wind)
{
	TWindow **info;
	Cell theCell;
	short cellData, cellDataLen, numCells, kind, filterLen;
	ListHandle theList;
	TGroup **groupArray;
	TSubject **subjectArray;
	CStr255 theString, filter;
	Handle strings;
	
	info = (TWindow**)GetWRefCon(wind);
	kind = (**info).kind;
	theList = (**info).theList;

	filterLen = (**info).filterLen;
	strncpy(filter, (**info).filter, filterLen);
	
	strings = (**info).strings;
	if (kind == cSubject) {
		subjectArray = (**info).subjectArray;
	} else {
		groupArray = (**info).groupArray;
	}
	
	LDoDraw(false,theList);
	
	numCells = (**theList).dataBounds.bottom;
	SetPt(&theCell,0,0);
	while (theCell.v < numCells) {
		cellDataLen = 2;
		LGetCell(&cellData, &cellDataLen, theCell, theList);
		if (kind == cSubject) {
			strcpy(theString, *strings + (*subjectArray)[cellData].subjectOffset);
		} else {
			strcpy(theString, *strings + (*groupArray)[cellData].nameOffset);
		}
		if (MySubstringSearch(theString, filter, strlen(theString), filterLen) == 0) {
			theCell.v++;
		} else {
			LDelRow(1, theCell.v, theList);
			numCells--;
		}
	}

	LDoDraw(true,theList);
	ForceUpdate(wind);
}


/*	NarrowDialogFilter is the dialog filter for the narrow selection
	search dialog box.
*/

static pascal Boolean NarrowDialogFilter (DialogPtr theDialog,
	EventRecord *theEvent,short *itemHit)
{
	WindowPtr wind;
	TWindow **info;
	char theChar;
	unsigned long saveBits;
	
	if (DialogFilter(theDialog, theEvent, itemHit)) return true;
	if (theEvent->what == keyDown || theEvent->what == autoKey) {
		saveBits = (theEvent->message & 0xFFFFFF00);
		theChar = toupper(theEvent->message & charCodeMask);
		theEvent->message = saveBits | theChar;
		SelIText(theDialog,3,32768,32768);
		wind = (WindowPtr)((WindowPeek)FrontWindow())->nextWindow;
		info = (TWindow**)GetWRefCon(wind);
		switch (theChar) {
			case deleteKey:
				if ((**info).filterLen > 0) {
					(**info).filterLen--;
					if ((**info).filterLen > 0) {
						RebuildFilterList(wind);
					} else {
						EndFilter(wind);
					}
				}
				break;
			default:
				(**info).filter[(**info).filterLen++] = theChar;
				if ((**info).filterLen == 1) {
					RebuildFilterList(wind);
				} else {
					NarrowFilterList(wind);
				}
				break;
		}
	}
	return false;
}


/*	DoReductionFilter is called in response to the Reduction Filter
	menu command.  This routine displays a dialog box, where the
	user enters a string which must be contained in all of the
	entries in a group or subject list.
*/

void DoReductionFilter (WindowPtr wind)
{
	DialogPtr theDlg;
	short item;
	short iType;
	Handle iHndl;
	Rect iRect;
	Str255 theText;
	TWindow **info;
	CStr255 filterCopy;
	short filterCopyLen;
	
	CStr255 msg;
	
	strcpy(msg, "The Reduction Filter command is no longer implemented in version 1.3d4. ");
	strcat(msg, "We may revive it, or we may get rid of it altogether. ");
	strcat(msg, "See the Mod History for more information.");
	ErrorMessage(msg);
	return;
	
	info = (TWindow**)GetWRefCon(wind);
	filterCopyLen = (**info).filterLen;
	strncpy(filterCopy, (**info).filter, filterCopyLen);
	
	theDlg = MyGetNewDialog(kNarrowDlg);
	
	if ((**info).filterLen > 0) {
		BlockMove((**info).filter,theText+1,(**info).filterLen);
		*theText = (**info).filterLen;
		GetDItem(theDlg,3,&iType,&iHndl,&iRect);
		SetIText(iHndl,theText);
		SelIText(theDlg,3,32768,32768);
	}
	ShowWindow(theDlg);

	do {
		MyModalDialog((ModalFilterProcPtr)NarrowDialogFilter,&item,true,true);
	} while (item != ok && item != cancel);

	if (item == cancel) {
		if (filterCopyLen == 0) {
			if ((**info).filterLen > 0) EndFilter(wind);
		} else {
			(**info).filterLen = filterCopyLen;
			strncpy((**info).filter, filterCopy, filterCopyLen);
			RebuildFilterList(wind);
		}
	}
	
	DisposDialog(theDlg);
}
