#ifdef MPW
	typedef pascal void (*StdTextProc)(short count, Ptr text, Point numer, Point denom);
	#define STDTEXTPROC(count, text, numer, denom) \
		((StdTextProc)thePort->grafProcs->textProc)(count, text, numer, denom)
#else /* ThinkC Version */
	#define STDTEXTPROC(count, text, numer, denom) \
		CallPascal(count, text, numer, denom, thePort->grafProcs->textProc)
#endif

#include <types.h>
#include <quickdraw.h>
#include <qdoffscreen.h>
#include <windows.h>
#include <resources.h>
#include <memory.h>
#include <events.h>
#include <fonts.h>
#include <fixmath.h>
#include <osutils.h>
#include <toolutils.h>

#define kPString			"\pOutline"

typedef enum boxTextOptions {
	constrainH = 1,
	constrainV = 2
} boxTextOptions;

void GetTextBounds( char* text, Point numer, Point denom, Rect* bounds );
void GetTextRect( char* text, Rect* bounds );
void BoxText( char* myPString, Rect *dst, boxTextOptions options );
void FastRubberBandText(char* pstr, Point p, Rect *r, boxTextOptions opt );
void SlowRubberBandText(char* pstr, Point achor, Rect *r, boxTextOptions opt );
GWorldPtr CreateTextMask( char* text, Rect* myRect, boxTextOptions options );
void ConstrainRect( Rect* src, Rect* dst, Rect* result, boxTextOptions constraints );
void JustShowOff(Point anchorPt);

void GetTextBounds( char* text, Point numer, Point denom,
						  Rect* bounds )
{
	FontInfo info;
	Fixed hScale = FixDiv(numer.h, denom.h);
	Fixed vScale = FixDiv(numer.v, denom.v);
	GetFontInfo( &info );
	SetRect( bounds, 0, FixMul( -info.ascent, vScale),
				FixMul( StringWidth( text ), hScale),
				FixMul(info.descent + info.leading, vScale) );
	OffsetRect( bounds, qd.thePort->pnLoc.h, qd.thePort->pnLoc.v );
}

void GetTextRect( char* text, Rect* bounds )
{
	Point identity;
	
	SetPt(&identity, 1, 1);
	GetTextBounds( text, identity, identity, bounds );
}

void BoxText( char* myPString, Rect *dst, boxTextOptions options )
{
	Point		numer, denom;
	short 	txSize;
	Rect		src;
	if (!options || options & ~(constrainH | constrainV)) {
		DrawString( myPString );
		return;
	}
	txSize = qd.thePort->txSize;
	TextSize( 100 );	/* Pick a big size so our source metrics are precise. */
	MoveTo(0, 0);
	GetTextRect( myPString, &src );
	switch (options) {
		case constrainH:
			numer.h = numer.v = dst->right - dst->left;
			denom.h = denom.v = src.right - src.left;
			break;
		case constrainV:
			numer.h = numer.v = dst->bottom - dst->top;
			denom.h = denom.v = src.bottom - src.top;
			break;
		case (constrainH | constrainV):
			SetPt(&numer, dst->right - dst->left, dst->bottom - dst->top);
			SetPt(&denom, src.right - src.left, src.bottom - src.top);
			break;
	}
	if (denom.h && denom.v) {
		short baseline = FixRound(FixMul(Long2Fix(-src.top),FixDiv(numer.v,denom.v)));
		MoveTo( dst->left, dst->top + baseline );
		StdText( *myPString, myPString+1, numer, denom );
	}
	TextSize( txSize );
}

void FastRubberBandText( char* myPString, Point p, Rect *theRect,
	boxTextOptions constraints )
{
	Rect 			srcRect, dstRect, origRect;
	GDHandle		oldGD;
	GWorldPtr		oldGW;
	GWorldPtr		myOffGWorld;
	Point			oldPoint;
	Point			newPoint;
	PixMapHandle	myPixMapHandle;
	short			theFont = qd.thePort->txFont;
	short			theFace = qd.thePort->txFace;
	short			err;

	GetTextRect( myPString, &origRect );
	srcRect = origRect;
	OffsetRect( &srcRect, -srcRect.left, -srcRect.top );
	srcRect.right <<= 3;
	srcRect.bottom <<= 3;
	/* Take a ride on the GWorld allocation loop... */
	do
		if (err = NewGWorld( &myOffGWorld, 1, &srcRect, 0, 0, useTempMem ))
			if (err = NewGWorld( &myOffGWorld, 1, &srcRect, 0, 0, 0 )) {
				srcRect.right >>= 1;
				srcRect.bottom >>= 1;
			}
	while (err && srcRect.right >= origRect.right);
	if (!err)
	{
		GetGWorld(&oldGW,&oldGD);
		SetGWorld(myOffGWorld,0);
		TextFont( theFont );			/* Use font from the current port. */
		TextFace( theFace );			/* Ditto. */
		myPixMapHandle = (PixMapHandle)GetGWorldPixMap( myOffGWorld ); 
		LockPixels( myPixMapHandle );
		EraseRect( &srcRect );
		BoxText( myPString, &srcRect, constraints );
		SetGWorld( oldGW, oldGD );
		PenMode( patXor );
		TextMode( srcXor );
		do {
			GetMouse( &oldPoint );
			Pt2Rect( oldPoint, p, &dstRect );
			ConstrainRect( &srcRect, &dstRect, theRect, constraints );
			FrameRect( &dstRect );					/* Draw it. */
			CopyBits( (BitMap *)*myPixMapHandle, &qd.thePort->portBits, &srcRect, theRect, srcXor, 0 );
			newPoint = oldPoint;
			while ( EqualPt(newPoint, oldPoint) && Button() )
				GetMouse( &newPoint );
			FrameRect( &dstRect );					/* Erase it. */
			CopyBits( (BitMap *)*myPixMapHandle, &qd.thePort->portBits, &srcRect, theRect, srcXor, 0 );
		} while( Button() );
		UnlockPixels( myPixMapHandle );
		DisposeGWorld( myOffGWorld );
	} else
		SlowRubberBandText( myPString, p, theRect, constraints );
}

void SlowRubberBandText(char* myPString,Point achorPoint,
	Rect *theRect, boxTextOptions options )
{
	Point	 oldPoint, newPoint;
	PenMode( patXor );
	TextMode( srcXor );
	SetRect(theRect, 0, 0, 0, 0);
	do {
		GetMouse( &oldPoint );
		Pt2Rect( oldPoint, achorPoint, theRect ); 
		FrameRect( theRect );					/* Draw it. */
		BoxText( myPString, theRect, options );
		newPoint = oldPoint;
		while( EqualPt(newPoint, oldPoint) && Button() )
			GetMouse( &newPoint );
		FrameRect( theRect );					/* Erase it. */
		BoxText( myPString, theRect, options);
	} while( Button() );
}

GWorldPtr CreateTextMask( char* text, Rect* myRect, boxTextOptions options)
{
	PixMapHandle	bigPixMap;
	GDHandle		oldGD;
	GWorldPtr 		oldGW, maskWorld, bigWorld;
	short			theFont = qd.thePort->txFont;
	short			theFace = qd.thePort->txFace;
	Rect			myRectBig;

	/* Create the 4-bit maskWorld. */
	{	CTabHandle ctab = GetCTable( 4+32 );
		if (NewGWorld( &maskWorld, 4, myRect, ctab, 0, useTempMem ) != noErr) 
			if((NewGWorld( &maskWorld, 4, myRect, ctab, 0, 0 ) != noErr)) {
			DisposHandle( (Handle) ctab );
			return 0;
		}
	}
		
	/*	Create the super-sample bigWorld. */
	{	short zoom = 4;
		short width = myRect->right-myRect->left;
		short height = myRect->bottom-myRect->top;
		do {
			SetRect( &myRectBig, 0, 0, width * zoom, height * zoom);
			if (NewGWorld( &bigWorld, 1, &myRectBig, 0, 0, useTempMem ) == noErr)
				break;
			else
			if (NewGWorld( &bigWorld, 1, &myRectBig, 0, 0, 0 ) == noErr)
				break;
			zoom--;
		} while (zoom > 1);
		if (zoom == 1) {
			DisposeGWorld( maskWorld );
			return 0;
		}
	}

	/* Draw the text into bigWorld. */
	GetGWorld(&oldGW,&oldGD);
	SetGWorld(bigWorld, 0);
	TextFont( theFont );
	TextFace( theFace );
	LockPixels( bigPixMap = GetGWorldPixMap( bigWorld ));
	EraseRect( &myRectBig );
	BoxText( text, &myRectBig, options );

	/* Image into destination 4-bit maskWorld. */
	{	PixMapHandle maskPixMap = GetGWorldPixMap( maskWorld );
		LockPixels( maskPixMap );
		SetGWorld( maskWorld, 0 );
		CopyBits( (BitMap *)*bigPixMap, (BitMap *)*maskPixMap, &myRectBig, myRect, ditherPix+srcCopy, 0L );
		UnlockPixels( maskPixMap );
	}
	DisposeGWorld( bigWorld );
	SetGWorld(oldGW, oldGD);
	return maskWorld;
}

void ConstrainRect( Rect* src, Rect* dst, Rect* result, boxTextOptions constraints )
{
	Fixed ratio;

	*result = *dst;
	switch (constraints) {
		case constrainH:
			ratio = FixDiv( src->bottom - src->top, src->right - src->left );
			result->bottom = dst->top + FixRound(FixMul( dst->right - dst->left, ratio ));
			break;
		case constrainV:
			ratio = FixDiv( src->right - src->left, src->bottom - src->top );
			result->right = dst->left + FixRound(FixMul( dst->bottom - dst->top, ratio ));
			break;
	}
}

void JustShowOff(Point anchorPt)
{
#define		kBoxOptions	 constrainH | constrainV
	Rect 			myRect;
	GWorldPtr 	mask = 0;
	char			*text = "\pCoolgeek!";
	FastRubberBandText( text, anchorPt, &myRect,  kBoxOptions );
	if (mask = CreateTextMask( text, &myRect, kBoxOptions )) {
		GWorldPtr		oldGW, src;
		PicHandle		pic;
		PixMapHandle	maskBits, srcBits;
		GDHandle			oldGD;
		if (NewGWorld( &src, 8, &myRect, 0, 0, useTempMem ) != noErr)
			if (NewGWorld( &src, 8, &myRect, 0, 0, 0 ) != noErr)
				goto EXIT;
		if (!(pic = GetPicture( 1001 )))	/* Assumes the PICT is marked purgeable. */
			goto EXIT;
		GetGWorld( &oldGW, &oldGD );
		SetGWorld( src, 0);					/* Set to draw into offscreen 8-bit. */
		DrawPicture( pic, &myRect );		/* Stretch picture to fill user's rect. */
		LockPixels( maskBits = GetGWorldPixMap( mask ));
		LockPixels( srcBits = GetGWorldPixMap( src ));
		SetGWorld( oldGW, oldGD );
		CopyDeepMask( (BitMap *)*srcBits, (BitMap *)*maskBits, 
					&qd.thePort->portBits, &myRect, &myRect, &myRect, srcCopy, 0L);
		DisposeGWorld( src );
	}
EXIT:
	if (mask)	DisposeGWorld( mask );
}
