#ifdef MPW
	typedef pascal void (*StdTextProc)(short count, Ptr text, Point numer, Point denom);
	#define STDTEXTPROC(count, text, numer, denom) \
		((StdTextProc)thePort->grafProcs->textProc)(count, text, numer, denom)
#else /* ThinkC Version */
	#define STDTEXTPROC(count, text, numer, denom) \
		CallPascal(count, text, numer, denom, thePort->grafProcs->textProc)
#endif

#define ditherCopy			0x40
#define kBoxOptions 		(constrainH | constrainV)
#define kPString			"\pOutline"

typedef enum boxTextOptions {
	constrainH = 1,
	constrainV = 2
} boxTextOptions;

void GetTextBounds( char*, Point numer, Point denom, Rect* );
void GetTextRect( char*, Rect* );
void BoxText(char*, Rect*, boxTextOptions);
void FastRubberBandText( char*, Point p, Rect *theRect, boxTextOptions options );
void SlowRubberBandText( char*, Point p, Rect *theRect, boxTextOptions options );
GWorldPtr CreateTextMask( char* text, Rect* myRect, boxTextOptions options);
void ConstrainRect( Rect* src, Rect* dst, Rect* result, boxTextOptions constraints );
void JustShowOff(Point anchorPt);

void GetTextBounds( char* text, Point numer, Point denom, Rect* bounds )
{
	FontInfo info;
	Fixed hScale = FixDiv(numer.h, denom.h);
	Fixed vScale = FixDiv(numer.v, denom.v);

	GetFontInfo( &info );
	SetRect( bounds, 0, FixMul( -info.ascent, vScale), FixMul( StringWidth( text ), hScale), FixMul(info.descent + info.leading, vScale) );
	OffsetRect( bounds, thePort->pnLoc.h, thePort->pnLoc.v );
}

void GetTextRect( char* text, Rect* bounds )
{
	Point identity;
	
	SetPt(&identity, 1, 1);
	GetTextBounds( text, identity, identity, bounds );
}

void BoxText( char* myPString, Rect *dst, boxTextOptions options )
{
	Point		numer, denom;
	short 	txSize;
	Rect		src;

	if (!options || options & ~(constrainH | constrainV)) {
		DrawString( myPString );
		return;
	}

	txSize = thePort->txSize;
	TextSize( 100 );		/* pick a big size so our source metrics are precise */
	MoveTo(0, 0);
	GetTextRect( myPString, &src );
	
	switch (options) {
		case constrainH:
			numer.h = numer.v = dst->right - dst->left;
			denom.h = denom.v = src.right - src.left;
			break;
		case constrainV:
			numer.h = numer.v = dst->bottom - dst->top;
			denom.h = denom.v = src.bottom - src.top;
			break;
		case (constrainH | constrainV):
			SetPt(&numer, dst->right - dst->left, dst->bottom - dst->top);
			SetPt(&denom, src.right - src.left, src.bottom - src.top);
			break;
	}
	
	if (denom.h && denom.v) {
		short baseline = FixMul(-src.top, FixDiv(numer.v, denom.v));
		MoveTo( dst->left, dst->top + baseline );
		StdText( *myPString, myPString+1, numer, denom );
	}
	TextSize( txSize );
}

void FastRubberBandText( char* myPString, Point p, Rect *theRect, boxTextOptions constraints )
{
Rect srcRect, dstRect, origRect;
GDHandle	oldGD;
GWorldPtr	oldGW;
GWorldPtr	myOffGWorld;
Point		oldPoint;
Point		newPoint;
PixMapHandle	myPixMapHandle;
short	theFont = thePort->txFont;
short	theFace = thePort->txFace;
short	err;

	GetTextRect( myPString, &origRect );
	srcRect = origRect;
	OffsetRect( &srcRect, -srcRect.left, -srcRect.top );
	srcRect.right <<= 3;
	srcRect.bottom <<= 3;

	/* Take a ride on the GWorld allocation loop...
	*/
	do
		if (err = NewGWorld( &myOffGWorld, 1, &srcRect, 0, 0, useTempMem ))
			if (err = NewGWorld( &myOffGWorld, 1, &srcRect, 0, 0, 0 )) {
				srcRect.right >>= 1;
				srcRect.bottom >>= 1;
			}
	while (err && srcRect.right >= origRect.right);

	if (!err)
	{
		GetGWorld(&oldGW,&oldGD);
		SetGWorld(myOffGWorld,0);
		TextFont( theFont );			/* use font from the current port */
		TextFace( theFace );			/* ditto */
		myPixMapHandle = GetGWorldPixMap( myOffGWorld ); 
		LockPixels( myPixMapHandle );
		EraseRect( &srcRect );
		BoxText( myPString, &srcRect, constraints );
		SetGWorld( oldGW, oldGD );
		PenMode( patXor );
		TextMode( srcXor );
		do {
			GetMouse( &oldPoint );
			Pt2Rect( oldPoint, p, &dstRect );
			ConstrainRect( &srcRect, &dstRect, theRect, constraints );
			FrameRect( &dstRect );					/* draw it */
			CopyBits( *myPixMapHandle, &thePort->portBits, &srcRect, theRect, srcXor, 0 );
			newPoint = oldPoint;
			while ( EqualPt(newPoint, oldPoint) && Button() )
				GetMouse( &newPoint );
			FrameRect( &dstRect );					/* Erase it */
			CopyBits( *myPixMapHandle, &thePort->portBits, &srcRect, theRect, srcXor, 0 );
		} while( Button() );
		UnlockPixels( myPixMapHandle );
		DisposeGWorld( myOffGWorld );
	} else
		SlowRubberBandText( myPString, p, theRect, constraints );
}

void SlowRubberBandText( char* myPString, Point p, Rect *theRect, boxTextOptions options )
{
	Point	 oldPoint, newPoint;

	PenMode( patXor );
	TextMode( srcXor );
	SetRect(theRect, 0, 0, 0, 0);
	do {
		GetMouse( &oldPoint );
		Pt2Rect( oldPoint, p, theRect ); 
		FrameRect( theRect );					/* draw it */
		BoxText( myPString, theRect, options );
		newPoint = oldPoint;
		while( EqualPt(newPoint, oldPoint) && Button() )
			GetMouse( &newPoint );
		FrameRect( theRect );					/* Erase it */
		BoxText( myPString, theRect, options);
	} while( Button() );
}

GWorldPtr CreateTextMask( char* text, Rect* myRect, boxTextOptions options)
{
PixMapHandle	bigPixMap;
GDHandle	oldGD;
GWorldPtr oldGW, maskWorld, bigWorld;
short	theFont = thePort->txFont;
short	theFace = thePort->txFace;
Rect		myRectBig;

/*	Create the 4-bit maskWorld
*/

	{	CTabHandle ctab = GetCTable( 4+32 );
		if (NewGWorld( &maskWorld, 4, myRect, ctab, 0, useTempMem ) != noErr) 
			if((NewGWorld( &maskWorld, 4, myRect, ctab, 0, 0 ) != noErr)) {
			DisposHandle( ctab );
			return 0;
		}
	}
		
/*	Create the super-sample bigWorld
*/
	{	short zoom = 4;
		short width = myRect->right-myRect->left;
		short height = myRect->bottom-myRect->top;
		do {
			SetRect( &myRectBig, 0, 0, width * zoom, height * zoom);
			if (NewGWorld( &bigWorld, 1, &myRectBig, 0, 0, useTempMem ) == noErr)
				break;
			else
			if (NewGWorld( &bigWorld, 1, &myRectBig, 0, 0, 0 ) == noErr)
				break;
			zoom--;
		} while (zoom > 1);
		if (zoom == 1) {
			DisposeGWorld( maskWorld );
			return 0;
		}
	}

/*	Draw the text into bigWorld
*/
	GetGWorld(&oldGW,&oldGD);
	SetGWorld(bigWorld, 0);
	TextFont( theFont );
	TextFace( theFace );
	LockPixels( bigPixMap = GetGWorldPixMap( bigWorld ));
	EraseRect( &myRectBig );
	BoxText( text, &myRectBig, options );

/*	Image into destination 4-bit maskWorld
*/
	{	PixMapHandle maskPixMap = GetGWorldPixMap( maskWorld );
		LockPixels( maskPixMap );
		SetGWorld( maskWorld, 0 );
		CopyBits( *bigPixMap, *maskPixMap, &myRectBig, myRect, ditherCopy+srcCopy, 0L );
		UnlockPixels( maskPixMap );
	}
	DisposeGWorld( bigWorld );
	SetGWorld(oldGW, oldGD);

	return maskWorld;
}

void ConstrainRect( Rect* src, Rect* dst, Rect* result, boxTextOptions constraints )
{
	Fixed ratio;

	*result = *dst;
	switch (constraints) {
		case constrainH:
			ratio = FixDiv( src->bottom - src->top, src->right - src->left );
			result->bottom = dst->top + FixMul( dst->right - dst->left, ratio );
			break;
		case constrainV:
			ratio = FixDiv( src->right - src->left, src->bottom - src->top );
			result->right = dst->left + FixMul( dst->bottom - dst->top, ratio );
			break;
	}
}

void JustShowOff(Point anchorPt)
{
	Rect myRect;
	GWorldPtr mask = 0;
	char* text = "\pHello, World";
	CursHandle		Curs;

	FastRubberBandText( text, anchorPt, &myRect,  kBoxOptions );

	Curs = GetCursor(watchCursor);
	SetCursor(*Curs);
	if (mask = CreateTextMask( text, &myRect, kBoxOptions )) {
		GWorldPtr oldGW, src;
		PicHandle pic;
		PixMapHandle maskBits, srcBits;
		GDHandle oldGD;

		if (NewGWorld( &src, 8, &myRect, 0, 0, useTempMem ) != noErr)
			if (NewGWorld( &src, 8, &myRect, 0, 0, 0 ) != noErr)
			{
				SysBeep(10);
				goto EXIT;
			}
		if (!(pic = GetPicture( 1002 )))	/* assumes the PICT is marked purgeable */
		{
			SysBeep(10);
			goto EXIT;
		}
		GetGWorld( &oldGW, &oldGD );
		SetGWorld( src, 0);			/* set to draw into offscreen 8-bit */
		DrawPicture( pic, &myRect );	/* stretch picture to fill user's rect */
		LockPixels( maskBits = GetGWorldPixMap( mask ));
		LockPixels( srcBits = GetGWorldPixMap( src ));
		SetGWorld( oldGW, oldGD );
		CopyDeepMask( *srcBits, *maskBits, &thePort->portBits, &myRect, &myRect, &myRect, srcCopy, 0L);
		DisposeGWorld( src );
	}
EXIT:
	if (mask)	DisposeGWorld( mask );
	SetCursor(&arrow);
}
