/******************************************************************************
**
**     File Name:	LibAUX.h
**
**   Description:	C & C++ interface to the A/UX Libraries.
**
*******************************************************************************
**
**                     Copyright  1991Apple Computer, Inc.
**                       UNPUBLISHEDALL RIGHTS RESERVED.
**
**  This program is  confidential  and trade secret of  Apple Computer, Inc.
**  The receipt or possession of this program does not convey any rights to
**  reproduce or disclose  its  contents, or to manufacture,  use,  or sell
**  anything that it may discribe, in whole or in part, without the express
**  written consent of Apple Computer, Inc.
**
*******************************************************************************
**                       A U T H O R   I D E N T I T Y
*******************************************************************************
**
**	Initials	Name
**	--------	-----------------------------------------------
**	BRH			Brett R. Halle
**
*******************************************************************************
**                      R E V I S I O N   H I S T O R Y
*******************************************************************************
**
**	  Date		Time	Author	Description
**	--------	-----	------	---------------------------------------------
**	06/24/91	11:10	BRH		Original version
**	07/10/91	15:39	JEM		Add additional definitions
**
******************************************************************************/

#ifndef __LIBAUX__
#define __LIBAUX__

#include <Types.h>

#define __MPW_C__
#define _AUX_SOURCE
#define _BSD_SOURCE
#define _POSIX_SOURCE
#define _SYSV_SOURCE

#ifndef __sys_types_h
#include </:usr:include:sys:types.h>
#endif

#ifndef __rpc_types_h
#include </:usr:include:rpc:types.h>
#endif

#ifndef __sys_errno_h
#include </:usr:include:sys:errno.h>
#endif

#ifndef __sys_signal_h
#include </:usr:include:sys:signal.h>
#endif

#ifndef __sys_time_h
#include </:usr:include:sys:time.h>
#endif

#ifndef __nfs_nfs_h
#include </:usr:include:nfs:nfs.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

struct childinfo {
	int tochild;
	int toparent;
	int pid;
	char *childstack;
	sigfunc_t istat;
	sigfunc_t qstat;
	sigfunc_t cstat;
};

union semun {
	int val;
	struct semid_ds *buf;
	ushort array[1];
};



int auxaccept(int s, struct sockaddr *addr, int *addrlen);
int auxaccess(const char *path, int mode);
int auxacct(char *path);
int auxadjtime(struct timeval *delta, struct timeval *olddelta);
unsigned auxalarm(unsigned sec);
int auxbind(int s, struct sockaddr *name, int namelen);
int auxchdir(const char *path);
int auxchmod(const char *path, mode_t mode);
int auxchown(const char *path, uid_t owner, gid_t group);
int auxchroot(char *path);
int auxclose(int fildes);
int auxconnect(int s, struct sockaddr *name, int namelen);
int auxcreat(const char *path, mode_t mode);
int auxdup(int fildes);
int auxexecl(const char *path, const char *arg, ...);
int auxexecv(const char *path, char *const argv[]);
int auxexecle(const char *path, const char *arg, ...);
int auxexecve(const char *path, char *const argv[], char *const *envp);
int auxexeclp(const char *file, ...);
int auxexecvp(const char *file, char *const argv[]);
void aux_exit(int status);
int auxfchmod(int fd, mode_t mode);
int auxfchown(int fd, int owner, int group);
int auxfcntl(int fildes, int cmd, ...);
int auxflock(int fd, int operation);
pid_t auxfork(char *childstack, int **fake);
int auxfsmount(int type, char *dir, int flags, caddr_t data);
int auxfstat(int fildes, struct stat *buf);
int auxfstatfs(int fildes, struct statfs *buf);
int auxfsync(int fd);
int auxftruncate(int fd, int length);
int auxgetcompat(void);
int auxgetdtablesize(void);
int auxgetdirent(int d, char *buf, int nbytes, long *basep);
int auxgetdomainname(char *name, int namelen);
gid_t auxgetegid(void);
char *auxgetenv(char *name);
uid_t auxgeteuid(void);
gid_t auxgetgid(void);
int auxgetgroups(int gidsetsize, gid_t grouplist[]);
int auxgethostid(void);
int auxgethostname(char *name, int namelen);
int auxgetitimer(int which, struct itimerval *value);
int auxgetpeername(int s, struct sockaddr *name, int *namelen);
pid_t auxgetpgrp(void);
pid_t auxgetpid(void);
pid_t auxgetppid(void);
int auxgetsockname(int s, struct sockaddr *name, int *namelen);
int auxgetsockopt(int s, int level, int optname, char *optval, int *optlen);
int aux_gettimeofday(struct timeval *tp, struct timezone *tzp);
uid_t auxgetuid(void);
int auxioctl(int fildes, int request, ...);
int auxkill(int pid, int sig);
int auxlink(const char *existing, const char *newlink);
int auxlisten(int s, int backlog);
int auxlocking(int fildes, int mode, int size);
off_t auxlseek(int fildes, off_t offset, int whence);
int auxlstat(const char *path, struct stat *buf);
int auxmkdir(const char *path, mode_t mode);
int auxmknod(const char *path, mode_t mode, int dev);
int auxmsgctl(int msqid, int cmd, struct msqid_ds *buf);
int auxmsgget(key_t key, int msgflg);
int auxmsgrcv(int msqid, struct msgbuf *msgp, int msgsz, long msgtyp, int msgflg);
int auxmsgsnd(int msqid, struct msgbuf *msgp, int msgz, int msgflg);
int auxnfs_getfh(int fildes, fhandle_t *fhp);
int auxnfssvc(int sock);
int auxopen(const char *path, int oflag, ...);
int auxpause(void);
int auxpipe(int fildes[2]);
int auxplock(int op);
int auxread(int fildes, void *buf, unsigned int nbyte);
int auxreadlink(char *path, char *buf, int bufsiz);
int auxreadv(int fildes, struct iovec *iov, int iovcnt);
int auxrecv(int s, char *buf, int len, int flags);
int auxrecvfrom(int s, char *buf, int len, int flags, struct sockaddr *from, int *fromlen);
int auxrecvmsg(int s, struct msghdr msg[], int flags);
int auxrename(const char *oldname, const char *newname);
int auxrmdir(const char *path);
int auxselect(int nfds, int *readfds, int *writefds, int *execptfds, struct timeval *timeout);
int auxsemctl(int semid, int semnum, int cmd, union semun arg);
int auxsemget(key_t key, int nsems, int semflg);
int auxsemop(int semid, struct sembuf (*sops)[], int nsops);
int auxsend(int s, char *msg, int len, int flags);
int auxsendmsg(int s, struct msghdr msg[], int flags);
int auxsendto(int s, char *msg, int len, int flags, struct sockaddr *to, int tolen);
int auxset42sig(void);
int auxsetdomainname(char *name, int namelen);
int auxsetegid(int egid);
int auxseteuid(int euid);
int auxsetgid(int gid);
int auxsetgroups(int ngroups, int *gidset);
int auxsethostid(int hostid);
int auxsethostname(char *name, int namelen);
int auxsetitimer(int which, struct itimerval *value, struct itimerval *ovalue);
int auxsetpgid(pid_t pid, pid_t pgid);
int auxsetpgrp(int pid, int pgrp);
int auxsetregid(int rgid, int egid);
int auxsetreuid(int ruid, int euid);
pid_t auxsetsid(void);
int auxsetsockopt(int s, int level, int optname, char *optval, int *optlen);
int aux_settimeofday(struct timeval *tp, struct timezone *tzp);
int auxsetuid(int uid);
char *auxshmat(int shmid, char *shmaddr, int shmflg);
int auxshmctl(int shmid, int cmd, struct shmid_ds *buf);
int auxshmdt(char *shmaddr);
int auxshmget(key_t key, int size, int shmflg);
int auxshutdown(int s, int how);
int auxsigblock(int sigmask);
sigfunc_t auxsignal(int sig, sigfunc_t func);
int auxsigpause(int mask);
int auxsigpending(sigset_t *set);
int auxsigsetmask(int mask);
int auxsigstack(struct sigstack *ss, struct sigstack *oss);
int auxsigvec(int sig, struct sigvec *vec, struct sigvec *ovec);
int auxsocket(int af, int type, int protocol);
int auxsocketpair(int d, int type, int protcol, int sv[2]);
int auxstat(const char *path, struct stat *buf);
int auxstatfs(const char *path, struct statfs *buf);
int auxstime(long *tp);
int auxsymlink(char *name1, char *name2);
void auxsync(void);
time_t auxtime(time_t *timer);
clock_t auxtimes(struct tms *buffer);
int auxtruncate(const char *path, int length);
long auxulimit(int cmd, long newlimit);
mode_t auxumask(mode_t cmask);
int auxumount(char *spec);
int auxuname(struct utsname *name);
int auxunlink(const char *path);
int auxunmount(char *name);
int auxustat(int dev, struct ustat *buf);
int auxutime(const char *path, struct utimbuf *times);
int auxuvar(struct var *v);
pid_t auxwait(int *statloc);
int auxwait3(union wait *status, int options, const int zero);
int auxwrite(int fildes, const void *buf, unsigned int nbyte);
int auxwritev(int fildes, struct iovec *iov, int ioveclen);


short AUXisRunning(void);
long GetAUXErrno(void);
void SetAUXErrno(long err);

Handle auxfork_pipe(int toparent, int tochild, void (*childtask) (), void *childarg);
int cleanup_auxfork_pipe(Handle globals);
int auxsystem(char *command);
char *auxfgets(char *buf, int count, int file, int timeout);

#ifdef __cplusplus
}
#endif

#endif
