#include	"Brushes.h"
#include	"DialogUtils.h"

#define kThumbnailItem 	4 		/* A tiny version of of the source picture, implemented
								as a user item in the dialog. This is its id */
#define kRadialItem 	5
#define kCircumfItem 	6

#define kMyID 			202

typedef struct
{
	BrushParams	*params;
	Point		localCtr;
	Rect		localRect;
}ThumbStuff;

typedef struct
{
	Point		ctr;
	Boolean		isRadial;
}CenterSpecs;


/* Prototypes */
void 			Stroke(BrushParams *params);
void 			DoConfigDialog(BrushParams *params);
pascal void 	ThumbnailItem(DialogPtr dptr, short item);
pascal Boolean 	ThumbnailFilter(DialogPtr dptr, EventRecord *event, short *item);
void 			DrawCross(Point pt);
Boolean			LockWorld(GWorldPtr world);
void 			UnlockWorld(GWorldPtr world);
void			ScaleRectAndPoint(Rect *srcRect, Point *pt, Rect *dstRect);
void 			CrossHairIt(WindowPtr dwind, short itemNum);

pascal short main(short msg, BrushParams *params)
{
	short	rslt = noErr;
	
	switch(msg)
	{
		case kStartBrush:
			params->storage = (long)NewHandle(sizeof(CenterSpecs));
			if(params->storage == 0)
				rslt = kCantRun;
			else
			{
				CenterSpecs *specs;
				
				HLock((Handle)params->storage);
				specs = (CenterSpecs *)(*(Handle)params->storage);
				
				/* Find the center */
				specs->ctr.h = ((params->rect.right - params->rect.left) / 2) + params->rect.left;
				specs->ctr.v = ((params->rect.bottom - params->rect.top) / 2) + params->rect.top;
				specs->isRadial = true;
				HUnlock((Handle)params->storage);
				rslt = kConfig;
			}
			break;	
		
		case kConfigure:
			DoConfigDialog(params);
			break;
		
		case kDoStroke:
			Stroke(params);
			break;
			
		case kStopBrush:
			DisposHandle((Handle)params->storage);
			params->storage = 0;
			break;
	}
	return rslt;
}

void Stroke(BrushParams *params)
{
	CenterSpecs 	specs;
	
	/* Make a local copy of the specs */
	specs = *((CenterSpecs *)(*(Handle)params->storage));
	
	/* Set the color */
	RGBForeColor(&params->color);
	
	/* Stroke */
	if(specs.isRadial == true)
	{
		MoveTo(params->pt.h, params->pt.v);
		Line((params->pt.h - specs.ctr.h) / 4, (params->pt.v - specs.ctr.v) / 4);
	}
	else
	{
		short		x, y;
		
		/* These are the x and y distances of the stroke point from the center */
		x = params->pt.h - specs.ctr.h;
		y = params->pt.v - specs.ctr.v;
		
		/* In these calculations, note that the x delta uses y, and vice versa. */
		MoveTo(params->pt.h - y / (2 * 4), params->pt.v + x / (2 * 4));
		Line(y / 4, -x / 4);
	}
}

void DoConfigDialog(BrushParams *params)
{
	DialogPtr			dptr;
	short				hit, kind;
	long				rslt;
	Handle				itmhndl;
	Rect				itemRect, worldRect;
	CenterSpecs			*specs;
	ThumbStuff			stuff;
	pascal void			BtnItem();
	
	dptr = GetNewDialog(kMyID, nil, (Ptr)(-1));
	if(dptr == nil)
	{
		SysBeep(10);
		return;
	}
	
	/* OK button Outline */
	GetDItem(dptr, kOKOutline, &kind, &itmhndl, &itemRect);
	SetDItem(dptr, kOKOutline, userItem, (Handle)BtnItem, &itemRect);
	
	/* Get a hold of the specs */
	HLock((Handle)params->storage);
	specs = (CenterSpecs *)(*(Handle)params->storage);
	
	/* Radio Buttons */
	GetDItem(dptr, kRadialItem, &kind, &itmhndl, &itemRect);
	SetCtlValue(itmhndl, (specs->isRadial ? 1 : 0));
	GetDItem(dptr, kCircumfItem, &kind, &itmhndl, &itemRect);
	SetCtlValue(itmhndl, (specs->isRadial ? 0 : 1));
		
	/* Thumbnail item to click in */
	GetDItem(dptr, kThumbnailItem, &kind, &itmhndl, &itemRect);
	SetDItem(dptr, kThumbnailItem, userItem, (Handle)ThumbnailItem, &itemRect);
	
	/* Calculate the ThumbStuff and store it for the userItem Proc */
	stuff.params = params;
	InsetRect(&itemRect, 3, 3);
	stuff.localRect = params->theSource->portRect;
	stuff.localCtr = specs->ctr;
	ScaleRectAndPoint(&stuff.localRect, &stuff.localCtr, &itemRect);
	SetWRefCon(dptr, (long)&stuff);
	
	/* Do it... */
	ShowWindow(dptr);
	while(true)									/* Loop until break...*/
	{
		Boolean 	out = false;
		
		ModalDialog(ThumbnailFilter, &hit);
		switch(hit)
		{
			case kOKButton:						/* OK button hit...*/
				/* Scale everything back out to the source size and record the point */
				ScaleRectAndPoint(&stuff.localRect, &stuff.localCtr, &params->theSource->portRect);
				specs->ctr = stuff.localCtr;
				GetDItem(dptr, kRadialItem, &kind, &itmhndl, &itemRect);
				specs->isRadial = (GetCtlValue(itmhndl) ? true : false);
				out = true;
				break;
	
			case kCancelButton:						/* Cancel button hit...*/
				out = true;
				break;

			case kRadialItem:					/* Radio button hit...*/
			case kCircumfItem:
				GetDItem(dptr, hit, &kind, &itmhndl, &itemRect);
				SetCtlValue(itmhndl, (GetCtlValue(itmhndl) ? 0 : 1));
				GetDItem(dptr, (hit == kRadialItem ? kCircumfItem : kRadialItem),
					&kind, &itmhndl, &itemRect);
				SetCtlValue(itmhndl, (GetCtlValue(itmhndl) ? 0 : 1));
				continue;
			
			default:					/* Anything else hit... */
				continue;
		}
		if(out)
			break;
	}
	InitCursor();
	DisposDialog(dptr);
	HUnlock((Handle)params->storage);
}

pascal void ThumbnailItem(DialogPtr dptr, short item)
{
	short			type;
	Rect			itemRect, worldRect;
	Point			thePt;
	Handle			hndl;
	PenState		old;
	ThumbStuff		*stuff;
		
	/* Get the stuff */
	stuff = (ThumbStuff *)GetWRefCon(dptr);
	
	/* Copy from world to Dialog */
	if(LockWorld(stuff->params->theSource))
	{
		ForeColor(blackColor);
		worldRect = stuff->localRect;
		CopyBits(*GetGWorldPixMap(stuff->params->theSource), &dptr->portBits,
				&stuff->params->theSource->portRect, &worldRect, srcCopy, nil);
		UnlockWorld(stuff->params->theSource);
	}
	
	/* Frame the picture */
	InsetRect(&worldRect, -3, -3);
	PenSize(2, 2);
	FrameRect(&worldRect);
	PenNormal();
	
	/* Draw the cross Hair */
	DrawCross(stuff->localCtr);
}	

void	ScaleRectAndPoint(Rect *srcRect, Point *pt, Rect *dstRect)
{
	register short	dstWid, dstHt;
	Point			ctr;
	Fixed			scaler;
	
	/* First home the Rect and the point... */
	pt->h -= srcRect->left;
	pt->v -= srcRect->top;
	OffsetRect(srcRect, -srcRect->left, -srcRect->top);
	
	/* ...then get the width and height and center */
	dstWid = dstRect->right - dstRect->left;
	dstHt = dstRect->bottom - dstRect->top;
	ctr.h = dstRect->left + (dstWid / 2);
	ctr.v = dstRect->top + (dstHt / 2);
	
	/* Get the scale value... */
	if(srcRect->bottom > srcRect->right)
		scaler = FixDiv(dstHt, srcRect->bottom);
	else
		scaler = FixDiv(dstWid, srcRect->right);
	
	/* Scale the rect... */
	srcRect->right = FixRound(scaler * srcRect->right);
	srcRect->bottom = FixRound(scaler * srcRect->bottom);
	
	/* Scale the point... */
	pt->h = FixRound(scaler * pt->h);
	pt->v = FixRound(scaler * pt->v);

	/* ...then center it over ctr */
	ctr.h = ctr.h - (srcRect->right / 2);
	ctr.v = ctr.v - (srcRect->bottom / 2);
	OffsetRect(srcRect, ctr.h, ctr.v);
	pt->h += ctr.h;
	pt->v += ctr.v;
}

pascal Boolean ThumbnailFilter(DialogPtr dptr, EventRecord *event, short *item)
{
    WindowPtr 	temp;
    Boolean		returnVal = false;
	ThumbStuff	*stuff;
		
    GetPort(&temp);
    SetPort(dptr);
    
    /* Change the cursor to a cross hair if it's over the Thumbnail item */
    CrossHairIt(dptr, kThumbnailItem);
    
    /* Standard key filtering */
    returnVal = StdKeyFilter(dptr, event, item);
    
	/* if that didn't handle it... */
	if(returnVal == false)
	{
		/* Get the stuff */
		stuff = (ThumbStuff *)GetWRefCon(dptr);
    
		/* Looking for clicks in the thumbnail item */
		if(event->what == mouseDown)
		{
			Point 	where;
			Rect 	rect;
			
			where = event->where;
			GlobalToLocal(&where);
			rect = stuff->localRect;
    		if(PtInRect(where, &rect)) /* It's a click we care about */
    		{
				ThumbStuff	*stuff;
					
				/* Get the stuff */
				stuff = (ThumbStuff *)GetWRefCon(dptr);
				
				/* Erase old and draw new point */
				DrawCross(stuff->localCtr);
				DrawCross(where);
				stuff->localCtr = where;
				
				while(StillDown())
				{
					/* Get new mouse position */
					GetMouse(&where);
					
					/* If it's changed, but still in the rect, redraw and store point */
					if((where.h != stuff->localCtr.h || where.v != stuff->localCtr.v) &&
						PtInRect(where, &rect))
					{
						/* Erase old and draw new point */
						DrawCross(stuff->localCtr);
						DrawCross(where);
						stuff->localCtr = where;
					}
				}
				returnVal = true;
			}
		}
	}
	SetPort(temp);
	return(returnVal);
}

void DrawCross(Point pt)
{
	PenMode(srcXor);
	PenSize(3, 3);
	MoveTo(pt.h - 8, pt.v - 1);
	Line(14, 0);
	MoveTo(pt.h - 1, pt.v - 8);
	Line(0, 14);
	PenNormal();
}

void CrossHairIt(WindowPtr dptr, short itemNum)
{
    Point 		thePt;
    Rect		theRect;
	ThumbStuff	*stuff;

	/* Get the stuff */
	stuff = (ThumbStuff *)GetWRefCon(dptr);
    
    /* See if mouse is in the rect */
    GetMouse(&thePt);
    theRect = stuff->localRect;
    if (PtInRect(thePt, &theRect))
    {
        SetCursor(*(GetCursor(crossCursor)));
    }
    else
    {
        InitCursor();
    }
}
	
Boolean LockWorld(GWorldPtr world)
{
	PixMapHandle	pix;
	
	pix = GetGWorldPixMap(world);
	return(LockPixels(pix));
}
	
void UnlockWorld(GWorldPtr world)
{
	PixMapHandle	pix;
	
	pix = GetGWorldPixMap(world);
	UnlockPixels(pix);
}
