#include	"Brushes.h"
#include 	"DialogUtils.h"

typedef struct
{
	short	length;
	Boolean	horLines;
	Boolean	verLines;
}ParallelSpecs;

/* Prototypes */
void Stroke(BrushParams *params);
void DoConfigDialog(ParallelSpecs *specs);

pascal short main(short msg, BrushParams *params)
{
	short	rslt = noErr;
	
	switch(msg)
	{
		case kStartBrush:
			params->storage = (long)NewHandle(sizeof(ParallelSpecs));
			if(params->storage == 0)
				rslt = kCantRun;
			else
			{
				ParallelSpecs *specs;
				
				HLock((Handle)params->storage);
				specs = (ParallelSpecs *)(*(Handle)params->storage);
				specs->length = 32;
				specs->horLines = false;
				specs->verLines = true;
				HUnlock((Handle)params->storage);
				rslt = kConfig;
			}
			break;	
		
		case kConfigure:
			HLock((Handle)params->storage);
			DoConfigDialog((ParallelSpecs *)(*(Handle)params->storage));
			HUnlock((Handle)params->storage);
			break;
		
		case kDoStroke:
			Stroke(params);
			break;
			
		case kStopBrush:
			DisposHandle((Handle)params->storage);
			params->storage = 0;
			break;
	}
	return rslt;
}

void Stroke(BrushParams *params)
{
	ParallelSpecs 	specs;
	short			startX, startY, endX, endY;
	
	/* Make a local copy of the specs */
	specs = *((ParallelSpecs *)(*(Handle)params->storage));
	
	/* Set the color */
	RGBForeColor(&params->color);
	
	/* Divide the spacing and length by 2 for later use */
	specs.length >>= 1;
	
	/* Draw appropriate line. If both horizontal and vertical are true, pick one */
	if(specs.horLines && specs.verLines)
	{
		if(Random() < 0)
			specs.horLines = false;
		else
			specs.verLines = false;
	}
	
	/* Horizontal. Remember that length and spacing are half the real value */
	if(specs.horLines)
	{
		startX = params->pt.h - specs.length + (Random() % specs.length);
		endX = params->pt.h + specs.length + (Random() % specs.length);
		startY = endY = params->pt.v;
	}
	else
	{
		startY = params->pt.v - specs.length + (Random() % specs.length);
		endY = params->pt.v + specs.length + (Random() % specs.length);
		startX = endX = params->pt.h;
	}
	
	MoveTo(startX, startY);
	LineTo(endX, endY);
}


void DoConfigDialog(ParallelSpecs *specs)
{
	DialogPtr			dptr;
	short				hit, kind;
	long				rslt;
	Handle				itmhndl;
	Rect				rect;
	pascal void			BtnItem();
	
	#define kLengthItem 	4
	#define kHorizItem 		5
	#define kVertItem 		6
	#define kBothItem 		7
	
	dptr = GetNewDialog(205, nil, (Ptr)(-1));
	if(dptr == nil)
	{
		SysBeep(10);
		return;
	}
	
	/* OK button Outline */
	GetDItem(dptr, kOKOutline, &kind, &itmhndl, &rect);
	SetDItem(dptr, kOKOutline, userItem, (Handle)BtnItem, &rect);
	
	/* Set Values... */
	ShortToDlog(specs->length, dptr, kLengthItem);
	
	if(specs->horLines && specs->verLines)
	{
		SetCtlValue(SnatchHandle(dptr, kBothItem), 1);
		SetCtlValue(SnatchHandle(dptr, kHorizItem), 0);
		SetCtlValue(SnatchHandle(dptr, kVertItem), 0);
	}
	else
	{
		SetCtlValue(SnatchHandle(dptr, kBothItem), 0);
		SetCtlValue(SnatchHandle(dptr, kHorizItem), (specs->horLines ? 1 : 0) );
		SetCtlValue(SnatchHandle(dptr, kVertItem),  (specs->verLines ? 1 : 0) );
	}
				
	/* Do it... */
	SelIText(dptr, kLengthItem, 0, 32767);
	ShowWindow(dptr);
	while(true)									/* Loop until break...*/
	{
		Boolean 	out = false;
		
		ModalDialog(NumFilter, &hit);
		switch(hit)
		{
			case kOKButton:						/* OK button hit...*/
				if(InRange(dptr, kLengthItem, 1, 100))
				{
					specs->length = DlogToShort(dptr, kLengthItem);
					if(GetCtlValue(SnatchHandle(dptr, kBothItem)))
					{
						specs->horLines = true;
						specs->verLines = true;
					}
					else
					{
						specs->horLines = 
							GetCtlValue(SnatchHandle(dptr, kHorizItem));
						specs->verLines =
							GetCtlValue(SnatchHandle(dptr, kVertItem));
					}
					out = true;
				}
				break;
	
			case kCancelButton:						/* Cancel button hit...*/
				out = true;
				break;
	
			case kHorizItem:					/* Radio button hit...*/
			case kVertItem:
			case kBothItem:
				GetDItem(dptr, hit, &kind, &itmhndl, &rect);
				if(GetCtlValue(itmhndl) != 1)
				{
					SetCtlValue(itmhndl, 1);
					switch(hit)
					{
						case kHorizItem:
							SetCtlValue(SnatchHandle(dptr, kVertItem), 0);
							SetCtlValue(SnatchHandle(dptr, kBothItem), 0);
							break;
						
						case kVertItem:
							SetCtlValue(SnatchHandle(dptr, kHorizItem), 0);
							SetCtlValue(SnatchHandle(dptr, kBothItem), 0);
							break;
						
						case kBothItem:
							SetCtlValue(SnatchHandle(dptr, kHorizItem), 0);
							SetCtlValue(SnatchHandle(dptr, kVertItem), 0);
							break;
					}
				}
				break;
						
			default:					/* Anything else hit... */
				break;
		}
		if(out)
			break;
	}
	InitCursor();
	DisposDialog(dptr);
}

