#include	"Brushes.h"
#include 	"DialogUtils.h"

typedef struct
{
	short	width;
	short	height;
	Boolean	filled;
	Boolean	isOval;
}RectSpecs;

/* Prototypes */
void Stroke(BrushParams *params);
void DoConfigDialog(RectSpecs *specs);
void CenterRect(Rect *theRect, Point *thePt);

pascal short main(short msg, BrushParams *params)
{
	short	rslt = noErr;
	
	switch(msg)
	{
		case kStartBrush:
			params->storage = (long)NewHandle(sizeof(RectSpecs));
			if(params->storage == 0)
				rslt = kCantRun;
			else
			{
				RectSpecs *specs;
				
				HLock((Handle)params->storage);
				specs = (RectSpecs *)(*(Handle)params->storage);
				specs->width = 4;
				specs->height = 4;
				specs->filled = false;
				specs->isOval = true;
				HUnlock((Handle)params->storage);
				rslt = kConfig;
			}
			break;	
		
		case kConfigure:
			HLock((Handle)params->storage);
			DoConfigDialog((RectSpecs *)(*(Handle)params->storage));
			HUnlock((Handle)params->storage);
			break;
		
		case kDoStroke:
			Stroke(params);
			break;
			
		case kStopBrush:
			DisposHandle((Handle)params->storage);
			params->storage = 0;
			break;
	}
	return rslt;
}

void Stroke(BrushParams *params)
{
	RectSpecs 	specs;
	Rect		rect;
	
	/* Make a local copy of the specs */
	specs = *((RectSpecs *)(*(Handle)params->storage));
	
	/* Set the color */
	RGBForeColor(&params->color);
	
	/* Make and position the rect */
	SetRect(&rect, 0, 0, specs.width, specs.height);
	CenterRect(&rect, &params->pt);
	
	/* Draw appropriate shape */
	if(specs.isOval)
		if(specs.filled)
			PaintOval(&rect);
		else
			FrameOval(&rect);
	else
		if(specs.filled)
			PaintRect(&rect);
		else
			FrameRect(&rect);
}


void DoConfigDialog(RectSpecs *specs)
{
	DialogPtr			dptr;
	short				hit, kind;
	long				rslt;
	Handle				itmhndl;
	Rect				rect;
	pascal void			BtnItem();
	
	#define kWidthItem 	4
	#define kHeightItem 5
	#define kFilledItem 6
	#define kOvalItem 	7
	#define kRectItem 	8
	
	dptr = GetNewDialog(200, nil, (Ptr)(-1));
	if(dptr == nil)
	{
		SysBeep(10);
		return;
	}
	
	/* OK button Outline */
	GetDItem(dptr, kOKOutline, &kind, &itmhndl, &rect);
	SetDItem(dptr, kOKOutline, userItem, (Handle)BtnItem, &rect);
	
	/* Set Values... */
	ShortToDlog(specs->width, dptr, kWidthItem);
	ShortToDlog(specs->height, dptr, kHeightItem);
	
	GetDItem(dptr, kFilledItem, &kind, &itmhndl, &rect);
	SetCtlValue(itmhndl, (specs->filled ? 1 : 0));
	
	GetDItem(dptr, kOvalItem, &kind, &itmhndl, &rect);
	SetCtlValue(itmhndl, (specs->isOval ? 1 : 0));
		
	GetDItem(dptr, kRectItem, &kind, &itmhndl, &rect);
	SetCtlValue(itmhndl, (specs->isOval ? 0 : 1));
		
	/* Do it... */
	SelIText(dptr, kWidthItem, 0, 32767);
	ShowWindow(dptr);
	while(true)									/* Loop until break...*/
	{
		Boolean 	out = false;
		
		ModalDialog(NumFilter, &hit);
		switch(hit)
		{
			case kOKButton:						/* OK button hit...*/
				if(InRange(dptr, kWidthItem, 1, 100))
				{
					if(InRange(dptr, kHeightItem, 1, 100))
					{
						specs->width = DlogToShort(dptr, kWidthItem);
						specs->height = DlogToShort(dptr, kHeightItem);
						GetDItem(dptr, kFilledItem, &kind, &itmhndl, &rect);
						specs->filled = (GetCtlValue(itmhndl) ? true : false);
						GetDItem(dptr, kOvalItem, &kind, &itmhndl, &rect);
						specs->isOval = (GetCtlValue(itmhndl) ? true : false);
						out = true;
					}
				}
				break;
	
			case kCancelButton:						/* Cancel button hit...*/
				out = true;
				break;
	
			case kFilledItem:					/* Check Box hit...*/
				GetDItem(dptr, hit, &kind, &itmhndl, &rect);
				SetCtlValue(itmhndl, (GetCtlValue(itmhndl) ? 0 : 1));
				continue;

			case kOvalItem:					/* Radio button hit...*/
			case kRectItem:
				GetDItem(dptr, hit, &kind, &itmhndl, &rect);
				SetCtlValue(itmhndl, (GetCtlValue(itmhndl) ? 0 : 1));
				GetDItem(dptr, (hit == kOvalItem ? kRectItem : kOvalItem), &kind, &itmhndl, &rect);
				SetCtlValue(itmhndl, (GetCtlValue(itmhndl) ? 0 : 1));
				continue;

			default:					/* Anything else hit... */
				continue;
		}
		if(out)
			break;
	}
	InitCursor();
	DisposDialog(dptr);
}

void CenterRect(Rect *theRect, Point *thePt)
{
	OffsetRect(theRect, -theRect->left, -theRect->top);
	OffsetRect(theRect, thePt->h - (theRect->right / 2), thePt->v - (theRect->bottom / 2));
}

