void 	FastCalculateDeltas( GWorldPtr src, GWorldPtr dst );
void 	OffToOff(GWorldPtr src, GWorldPtr dst);
Boolean LockWorld(GWorldPtr world);
void 	UnlockWorld(GWorldPtr world);

pascal OSErr main(GWorldPtr world)
{
	short				rslt = -1;
	PixMapHandle		srcPixMap;

	srcPixMap = GetGWorldPixMap(world);
	if(srcPixMap != nil)
	{
		/* If world is 32 bit, go ahead. Else beep */
		if((*srcPixMap)->pixelSize == 32)
		{
			GWorldPtr	grayGWorld;
			CTabHandle	myGrayCTable;
		
			myGrayCTable = GetCTable( 40 );		/* Defaullt 256 gray table */
			if( !NewGWorld( &grayGWorld, 8, &world->portRect, myGrayCTable, 0, 0 ) )
			{
				FastCalculateDeltas( world, grayGWorld );
				OffToOff(grayGWorld, world);
				DisposeGWorld( grayGWorld );
				rslt = noErr;		
			}
			else
				rslt = memFullErr;
		}
		else
			SysBeep(10);
	}
	return rslt;
}

/* This routine  by Konstantin Othmer and Mike Reed, for develop #10 */
void FastCalculateDeltas( GWorldPtr src, GWorldPtr dst )
{
PixMapHandle	srcPixMap, dstPixMap;
short		srcRowBytes, dstRowBytes;
long			*srcBaseAddr, *dstBaseAddr, *dstAddr;
unsigned char	*srcAddr1;
char			mmuMode;
short		row, column;
unsigned char	lum1,lum2;
unsigned long	dstLong;
short		width, height;
GDHandle		oldGD;
GWorldPtr		oldGW;

	srcPixMap = GetGWorldPixMap ( src );
	dstPixMap = GetGWorldPixMap ( dst );

	if ( LockPixels ( srcPixMap ) && LockPixels( dstPixMap) ) 
	{	/* lock the pixmaps */
		GetGWorld(&oldGW,&oldGD);
		SetGWorld(dst,nil);
		CopyBits( (BitMap*)*srcPixMap, (BitMap*)*dstPixMap, &(**srcPixMap).bounds, &(**dstPixMap).bounds, srcCopy, 0L );
		SetGWorld( oldGW, oldGD );

		srcBaseAddr = (long *) GetPixBaseAddr ( srcPixMap );	/* get the address of the pixmap */
		srcRowBytes = (**srcPixMap).rowBytes & 0x7fff;			/* get the row increment */
		dstBaseAddr = (long *) GetPixBaseAddr ( dstPixMap );	/* get the address of the pixmap */
		dstRowBytes = (**dstPixMap).rowBytes & 0x7fff;			/* get the row increment */
		width = (**srcPixMap).bounds.right-(**srcPixMap).bounds.left;
		height = (**srcPixMap).bounds.bottom-(**srcPixMap).bounds.top;
	
		mmuMode = true32b;
		SwapMMUMode ( &mmuMode );						/* set the MMU to 32-bit mode */
		for ( row = 0; row < height; row++ ) 
		{
			srcAddr1 = (unsigned char *)dstBaseAddr;
			dstAddr = dstBaseAddr;
			lum1 = *srcAddr1++;		/* get luminance of src pixel */
			for ( column = 0; column < ((width-1)/4); column++ ) 
			{
				/* 
				** Do a long in the destination (4 pixels)
				** This is o.k. since memory blocks are always long word aligned. 
				** Thus, we will never write over the right hand edge.
				*/
				dstLong = 0;
				lum2 = *srcAddr1++;
				dstLong = (unsigned char)((0x100 + lum1 - lum2)>>1);
				dstLong = dstLong << 8;
				lum1 = *srcAddr1++;
				dstLong |= (unsigned char)((0x100 + lum2 - lum1)>>1);
				dstLong = dstLong<<8;
				lum2 = *srcAddr1++;
				dstLong |= (unsigned char)((0x100 + lum1 - lum2)>>1);
				dstLong = dstLong << 8;
				lum1 = *srcAddr1++;
				dstLong |= (unsigned char)((0x100 + lum2 - lum1)>>1);
				*dstAddr++ = dstLong;
			}
	
			srcBaseAddr = (long *) ( (char *) srcBaseAddr + srcRowBytes );	/* go to the next row */
			dstBaseAddr = (long *) ( (char *) dstBaseAddr + dstRowBytes );	/* go to the next row */
		}
		SwapMMUMode ( &mmuMode );						/* restore the previous MMU mode */
		UnlockPixels ( srcPixMap );						/* unlock the pixmap */
		UnlockPixels ( dstPixMap );						/* unlock the pixmap */
	}
}

/* Copies one GWorld to another */
void OffToOff(GWorldPtr src, GWorldPtr dst)
{
	CGrafPtr	oldport;
	GDHandle	olddev;
	
	if(src == nil || dst == nil)
		return;

	GetGWorld(&oldport, &olddev);
	SetGWorld(dst, nil);
	if(LockWorld(src))
	{
		if(LockWorld(dst))
		{
			ForeColor(blackColor);
			BackColor(whiteColor);
			CopyBits(*GetGWorldPixMap(src), *GetGWorldPixMap(dst),
					&src->portRect, &dst->portRect, srcCopy, nil);
			UnlockWorld(dst);
		}
		UnlockWorld(src);
	}
	SetGWorld(oldport, olddev);
}

/* Locks a GWorld's Pixels */
Boolean LockWorld(GWorldPtr world)
{
	PixMapHandle	pix;
	
	pix = GetGWorldPixMap(world);
	return(LockPixels(pix));
}

/* Unlocks a GWorld's Pixels */
void UnlockWorld(GWorldPtr world)
{
	PixMapHandle	pix;
	
	pix = GetGWorldPixMap(world);
	UnlockPixels(pix);
}
