/*
	File: DummySave.c

	Copyright 1990 by Thomas Knoll.

	MPW C source file for DummySave example.
*/

#include <Types.h>
#include <Memory.h>
#include <Resources.h>
#include <QuickDraw.h>
#include <Dialogs.h>
#include <OSUtils.h>
#include <Packages.h>
#include <Errors.h>
#include <ToolUtils.h>

#include "ExportInterface.h"

long gMyA5;
long gHostA5;

short gResult;
ExportRecordPtr gStuff;

short gFRefNum;
short gVRefNum;

/*****************************************************************************/

/* All calls to the plug-in module come through this routine. It must be
   placed first in the resource. To achieve this, most development systems
   require that this be the first routine in the source. */

pascal void ENTRYPOINT (selector, stuff, data, result)

short selector;
ExportRecordPtr stuff;
long *data;
short *result;

	{
	void Dispatch ();

	Dispatch (selector, stuff, data, result);
	}

/*****************************************************************************/

/* The following routines are based on Macintosh Technical Note #256. */

extern long A5Size (void);
extern void A5Init (Ptr myA5);

long ComputeA5 (A5Ref)
Handle A5Ref;
	{
	return ((long) *A5Ref) + A5Size () - 32;
	}

void MakeA5World (A5Ref)
Handle *A5Ref;
	{
	*A5Ref = NewHandle (A5Size ());
	if (*A5Ref)
		{
		HLock (*A5Ref);
		A5Init ((Ptr) ComputeA5 (*A5Ref));
		HUnlock (*A5Ref);
		}
	}

long SetA5World (A5Ref)
Handle A5Ref;
	{
	HLock (A5Ref);
	return SetA5 (ComputeA5 (A5Ref));
	}

void RestoreA5World (oldA5, A5Ref)
long oldA5;
Handle A5Ref;
	{
	(void) SetA5 (oldA5);
	HUnlock (A5Ref);
	}

/*****************************************************************************/

pascal Boolean DoTestAbort (ProcPtr codeAddress)
	= {0x205F, 0x4E90};

/* Calls the host's TestAbort function */

Boolean TestAbort (void)

	{
	long ourA5;
	Boolean result;
	ProcPtr address;

	address = gStuff->abortProc;

	ourA5 = SetA5 (gHostA5);

	result = DoTestAbort (address);

	ourA5 = SetA5 (ourA5);

	return result;
	}

/*****************************************************************************/

pascal void DoUpdateProgress (long done, long total, ProcPtr codeAddress)
	= {0x205F, 0x4E90};

/* Calls the host's UpdateProgress procedure */

void UpdateProgress (done, total)

long done;
long total;

	{
	long ourA5;
	ProcPtr address;

	address = gStuff->progressProc;

	ourA5 = SetA5 (gHostA5);

	DoUpdateProgress (done, total, address);

	ourA5 = SetA5 (ourA5);
	}

/*****************************************************************************/

/* Sets the global variables to their default values. */

void InitGlobals (void)

	{
	}

/*****************************************************************************/

/* Centers a dialog template 1/3 of the way down on the main screen. */

void CenterDialog (dt)

DialogTHndl dt;

#define menuHeight 20

	{
	Rect r;
	short width;
	short height;

	width  = qd.screenBits.bounds.right;
	height = qd.screenBits.bounds.bottom;

	r = (**dt).boundsRect;
	OffsetRect (&r, -r.left, -r.top);
	OffsetRect (&r, (width - r.right) / 2,
					(height - r.bottom - menuHeight) / 3 + menuHeight);
	(**dt).boundsRect = r;
	}

#undef menuHeight

/*****************************************************************************/

/* Displays the about dialog box for the plug-in module. */

void DoAbout (void)

#define dialogID 17000

	{
	long ourA5;
	short item;
	DialogPtr dp;
	DialogTHndl dt;

	dt = (DialogTHndl) GetResource ('DLOG', dialogID);
	HNoPurge ((Handle) dt);
	CenterDialog (dt);

	dp = GetNewDialog (dialogID, nil, (WindowPtr) -1);
	
	ourA5 = SetA5 (gHostA5);

	ModalDialog (nil, &item);
	
	ourA5 = SetA5 (ourA5);

	DisposDialog (dp);
	HPurge ((Handle) dt);
	}

#undef dialogID

/*****************************************************************************/

/* Prepare to export an image.	If the plug-in module needs only a limited
   amount of memory, it can lower the value of the 'maxData' field. */

void DoPrepare (void)

	{
	if (gStuff->maxData > 0x80000)
		gStuff->maxData = 0x80000;
	}

/*****************************************************************************/

/* Request next block of the image */

void RequestNext (top)

short top;

	{
	long count;

	/* Compute maximum number of rows we should request at once */

	count = gStuff->maxData / gStuff->imageSize.h / gStuff->planes;

	if (count < 1) count = 1;

	/* Request next block of the image */

	gStuff->loPlane = 0;
	gStuff->hiPlane = gStuff->planes - 1;

	gStuff->theRect.left  = 0;
	gStuff->theRect.right = gStuff->imageSize.h;
	gStuff->theRect.top   = top;

	if (top + count > gStuff->imageSize.v)
		gStuff->theRect.bottom = gStuff->imageSize.v;
	else
		gStuff->theRect.bottom = top + count;

	}

/*****************************************************************************/

/* Verifies that the image mode is supported.  Prompts the user for
   a file name to save the image to, and requests the first block of
   of image data. */

void DoStart (void)

#define kStringsID 17000

	{
	OSErr err;
	long ourA5;
	Point where;
	Str255 prompt;
	SFReply reply;
	StringPtr name;
	DialogTHndl dt;

	/* This plug-in does not support bitmap images */

	if (gStuff->imageMode == exportModeBitmap)
		{
		gResult = exportBadMode;
		return;
		}

	/* Get prompt string */

	GetIndString (prompt, kStringsID, 1);

	/* Center dialog */

	dt = (DialogTHndl) GetResource ('DLOG', putDlgID);
	HNoPurge ((Handle) dt);

	CenterDialog (dt);
	where.v = (**dt).boundsRect.top;
	where.h = (**dt).boundsRect.left;

	HPurge ((Handle) dt);

	/* Ask the user */
	
	name = &gStuff->filename;
	
	ourA5 = SetA5 (gHostA5);

	SFPutFile (where, prompt, name, nil, &reply);
	
	ourA5 = SetA5 (ourA5);

	if (!reply.good)
		{
		gResult = 1;
		return;
		}

	gVRefNum = reply.vRefNum;

	err = FSDelete (reply.fName, gVRefNum);

	err = Create (reply.fName, gVRefNum, '????', '????');

	if (err != noErr)
		{
		gResult = err;
		return;
		}

	err = FSOpen (reply.fName, gVRefNum, &gFRefNum);

	if (err != noErr)
		{
		gResult = err;
		return;
		}

	/* Request first block of image */

	RequestNext (0);

	}

#undef kStringsID

/*****************************************************************************/

/* Processes the next chunk of the image. */

void DoContinue (void)

	{
	OSErr err;
	short row;
	long count;

	for (row = gStuff->theRect.top; row < gStuff->theRect.bottom; row++)
		{

		/* See if user was aborted the operation */

		if (TestAbort ())
			{
			gResult = 1;
			return;
			}

		/* Update the progress indicator */

		UpdateProgress ((long) row, (long) gStuff->imageSize.v);

		/* Write out this row of the image */

		count = gStuff->planes * (long) gStuff->imageSize.h;

		err = FSWrite (gFRefNum,
					   &count,
					   (Ptr) ((long) (gStuff->data) + gStuff->rowBytes *
										(row - gStuff->theRect.top)));

		if (err != noErr)
			{
			gResult = err;
			return;
			}

		}

	/* Request next block of image */

	RequestNext (gStuff->theRect.bottom);

	}

/*****************************************************************************/

/* This routine will always be called if DoStart does not return an error
   (even if DoContinue returns an error or the user aborts the operation).
   This allows the module to perform any needed cleanup. */

void DoFinish (void)

	{
	OSErr err;

	/* Close the file */

	err = FSClose (gFRefNum);

	if (err != noErr)
		{
		gResult = err;
		return;
		}

	/* Flush the volume */

	err = FlushVol (nil, gVRefNum);

	if (err != noErr)
		{
		gResult = err;
		return;
		}

	/* Clear the image's unsaved changes flag */

	gStuff->dirty = false;

	}

/*****************************************************************************/

/* Main dispatching routine.  Initializes and sets up the global variables,
   and performs the operation specified by the selector. */

void Dispatch (selector, stuff, data, result)

short selector;
ExportRecordPtr stuff;
long *data;
short *result;

	{
	Boolean firstTime;

	/* See if this is the first time called */

	firstTime = (*data == nil);

	/* If so, allocate and initialize a global data area */

	if (firstTime)
		{
		MakeA5World (data);
		if (!*data)
			{
			*result = memFullErr;
			return;
			}
		}

	/* Switch in our global variables */

	gHostA5 = SetA5World (*data);
	gMyA5	= ComputeA5 (*data);

	if (firstTime)
		{

		/* Make a copy of the host's quickdraw globals */

		BlockMove ((Ptr) (*((long *) gHostA5) - 126),
				   (Ptr) &qd.randSeed,
				   130);

		/* Initialize the plug-in global values */

		InitGlobals ();

		}

	/* Perform the requested operation */

	gStuff	= stuff;
	gResult = noErr;

	switch (selector)
		{

		case exportSelectorAbout:
			DoAbout ();
			break;

		case exportSelectorPrepare:
			DoPrepare ();
			break;

		case exportSelectorStart:
			DoStart ();
			break;

		case exportSelectorContinue:
			DoContinue ();
			break;

		case exportSelectorFinish:
			DoFinish ();
			break;

		default:
			gResult = exportBadParameters;

		}

	*result = gResult;

	/* Restore the application's globals */

	RestoreA5World (gHostA5, *data);

	}
