{
	File: DummyScan.p

	Copyright 1990 by Thomas Knoll.

	MPW Pascal source file for DummyScan example.
}

UNIT DummyScan;

INTERFACE

USES
	MemTypes, QuickDraw, OSIntf, ToolIntf, PackIntf,
	AcquireInterface;

PROCEDURE ENTRYPOINT (selector: INTEGER;
					  stuff: AcquireRecordPtr;
					  VAR data: Handle;
					  VAR result: INTEGER);

IMPLEMENTATION

VAR
	gMyA5: LONGINT;
	gHostA5: LONGINT;

	gResult: INTEGER;
	gStuff: AcquireRecordPtr;

	gLastRows: INTEGER;
	gLastCols: INTEGER;
	gLastMode: INTEGER;

	gNextRow: INTEGER;
	gNextPlane: INTEGER;

PROCEDURE Dispatch (selector: INTEGER;
					stuff: AcquireRecordPtr;
					VAR data: Handle;
					VAR result: INTEGER); FORWARD;

{*****************************************************************************}

PROCEDURE ENTRYPOINT (selector: INTEGER;
					  stuff: AcquireRecordPtr;
					  VAR data: Handle;
					  VAR result: INTEGER);

{ All calls to the plug-in module come through this routine. It must be
  placed first in the resource. To achieve this, most development systems
  require that this be the first routine in the source. }

	BEGIN
	Dispatch (selector, stuff, data, result)
	END;

{*****************************************************************************}

{ The following routines are based on Macintosh Technical Note #256. }

FUNCTION A5Size: LONGINT; C; EXTERNAL;

PROCEDURE A5Init (myA5: Ptr); C; EXTERNAL;

FUNCTION ComputeA5 (A5Ref: Handle): LONGINT;
	BEGIN
	ComputeA5 := ORD4 (A5Ref^) + A5Size - 32
	END;

PROCEDURE MakeA5World (VAR A5Ref: Handle);
	BEGIN
	A5Ref := NewHandle (A5Size);
	IF A5Ref <> NIL THEN
		BEGIN
		HLock (A5Ref);
		A5Init (Ptr (ComputeA5 (A5Ref)));
		HUnlock (A5Ref)
		END
	END;

FUNCTION SetA5World (A5Ref: Handle): LONGINT;
	BEGIN
	HLock (A5Ref);
	SetA5World := SetA5 (ComputeA5 (A5Ref))
	END;

PROCEDURE RestoreA5World (oldA5: LONGINT; A5Ref: Handle);

	VAR
		ignore: LONGINT;

	BEGIN
	ignore := SetA5 (oldA5);
	HUnlock (A5Ref)
	END;

{*****************************************************************************}

FUNCTION TestAbort: BOOLEAN;

{ Calls the host's TestAbort function }

	VAR
		ourA5: LONGINT;
		address: ProcPtr;

	FUNCTION DoTestAbort (codeAddress: ProcPtr): BOOLEAN;
		INLINE $205F, $4E90;

	BEGIN

	address := gStuff^.abortProc;

	ourA5 := SetA5 (gHostA5);

	TestAbort := DoTestAbort (address);

	ourA5 := SetA5 (ourA5)

	END;

{*****************************************************************************}

PROCEDURE UpdateProgress (done, total: LONGINT);

{ Calls the host's UpdateProgress procedure }

	VAR
		ourA5: LONGINT;
		address: ProcPtr;

	PROCEDURE DoUpdateProgress (done, total: LONGINT; codeAddress: ProcPtr);
		INLINE $205F, $4E90;

	BEGIN

	address := gStuff^.progressProc;

	ourA5 := SetA5 (gHostA5);

	DoUpdateProgress (done, total, address);

	ourA5 := SetA5 (ourA5)

	END;

{*****************************************************************************}

PROCEDURE InitGlobals;

{ Sets the global variables to their default values. }

	BEGIN

	gLastRows := 256;
	gLastCols := 256;
	gLastMode := acquireModeRGBColor

	END;

{*****************************************************************************}

PROCEDURE CenterDialog (dt: DialogTHndl);

{ Centers a dialog template 1/3 of the way down on the main screen. }

	CONST
		menuHeight = 20;

	VAR
		r: Rect;
		width: INTEGER;
		height: INTEGER;

	BEGIN

	width  := screenBits.bounds.right;
	height := screenBits.bounds.bottom;

	r := dt^^.boundsRect;
	OffsetRect (r, -r.left, -r.top);
	OffsetRect (r, (width - r.right) DIV 2,
				   (height - r.bottom - menuHeight) DIV 3 + menuHeight);
	dt^^.boundsRect := r

	END;

{*****************************************************************************}

PROCEDURE DoAbout;

{ Displays the about dialog box for the plug-in module. }

	CONST
		dialogID  = 16000;

	VAR
		dp: DialogPtr;
		item: INTEGER;
		ourA5: LONGINT;
		dt: DialogTHndl;

	BEGIN

	dt := DialogTHndl (GetResource ('DLOG', dialogID));
	HNoPurge (Handle (dt));
	CenterDialog (dt);

	dp := GetNewDialog (dialogID, NIL, WindowPtr (-1));
	
	ourA5 := SetA5 (gHostA5);

	ModalDialog (NIL, item);

	ourA5 := SetA5 (ourA5);
	
	DisposDialog (dp);
	HPurge (Handle (dt))

	END;

{*****************************************************************************}

PROCEDURE DoPrepare;

{ Prepare to acquire an image.	If the plug-in module needs only a limited
  amount of memory, it can lower the value of the 'maxData' field. }

	BEGIN

	IF gStuff^.maxData > $80000 THEN
		gStuff^.maxData := $80000

	END;

{*****************************************************************************}

PROCEDURE OutlineOK (dp: DialogPtr; item: INTEGER);

{ UserItem to outline the OK button in a dialog box. }

	VAR
		r: Rect;
		h: Handle;
		newA5: LONGINT;
		oldA5: LONGINT;
		itemType: INTEGER;

	BEGIN

	newA5 := GetWRefCon (dp);
	oldA5 := SetA5 (newA5);
	
	GetDItem (dp, ok, itemType, h, r);
	
	PenNormal;
	PenSize (3, 3);

	InsetRect (r, -4, -4);
	FrameRoundRect (r, 16, 16);

	PenNormal;

	oldA5 := SetA5 (oldA5)

	END;

{*****************************************************************************}

FUNCTION ConvertString (s: Str255; VAR n: INTEGER): BOOLEAN;

{ Converts a string to a number between 1 and 30000.
  Returns false if not a valid number or out of range. }

	VAR
		j: INTEGER;
		x: LONGINT;

	BEGIN

	ConvertString := FALSE;

	FOR j := 1 TO LENGTH (s) DO
		IF (s[j] < '0') OR (s[j] > '9') THEN
			EXIT (ConvertString);

	StringToNum (s, x);

	IF (x < 1) OR (x > 30000) THEN
		EXIT (ConvertString);

	ConvertString := TRUE;

	n := x

	END;

{*****************************************************************************}

FUNCTION GetParameters (VAR rows, cols, mode: INTEGER): BOOLEAN;

{ Prompt the user for the image parameters.
  Returns false if the user cancels. }

	CONST
		dialogID  = 16001;
		hookItem  = 3;
		rowsItem  = 4;
		colsItem  = 5;
		bModeItem = 6;
		gModeItem = 7;
		iModeItem = 8;
		cModeItem = 9;

	VAR
		r: Rect;
		s: Str255;
		h: Handle;
		dp: DialogPtr;
		item: INTEGER;
		ourA5: LONGINT;
		dt: DialogTHndl;
		rowsText: Handle;
		colsText: Handle;
		itemType: INTEGER;
		bButton: ControlHandle;
		gButton: ControlHandle;
		iButton: ControlHandle;
		cButton: ControlHandle;

	BEGIN

	dt := DialogTHndl (GetResource ('DLOG', dialogID));
	HNoPurge (Handle (dt));
	CenterDialog (dt);

	dp := GetNewDialog (dialogID, NIL, WindowPtr (-1));

	SetWRefCon (dp, gMyA5);
	
	GetDItem (dp, hookItem, itemType, h 				 , r);
	SetDItem (dp, hookItem, itemType, Handle (@OutlineOK), r);

	GetDItem (dp, rowsItem, itemType, rowsText, r);
	GetDItem (dp, colsItem, itemType, colsText, r);

	NumToString (rows, s);
	SetIText (rowsText, s);

	NumToString (cols, s);
	SetIText (colsText, s);

	SelIText (dp, rowsItem, 0, 32767);

	GetDItem (dp, bModeItem, itemType, Handle (bButton), r);
	GetDItem (dp, gModeItem, itemType, Handle (gButton), r);
	GetDItem (dp, iModeItem, itemType, Handle (iButton), r);
	GetDItem (dp, cModeItem, itemType, Handle (cButton), r);

		REPEAT

		SetCtlValue (bButton, ORD (mode = acquireModeBitmap));
		SetCtlValue (gButton, ORD (mode = acquireModeGrayScale));
		SetCtlValue (iButton, ORD (mode = acquireModeIndexedColor));
		SetCtlValue (cButton, ORD (mode = acquireModeRGBColor));

		ourA5 := SetA5 (gHostA5);
	
		ModalDialog (NIL, item);
	
		ourA5 := SetA5 (ourA5);

		IF (item >= bModeItem) AND (item <= cModeItem) THEN
			mode := item - bModeItem;

		IF item = ok THEN
			BEGIN

			GetIText (rowsText, s);

			IF NOT ConvertString (s, rows) THEN
				BEGIN
				SelIText (dp, rowsItem, 0, 32767);
				SysBeep (1);
				item := 0
				END

			END;

		IF item = ok THEN
			BEGIN

			GetIText (colsText, s);

			IF NOT ConvertString (s, cols) THEN
				BEGIN
				SelIText (dp, colsItem, 0, 32767);
				SysBeep (1);
				item := 0
				END

			END

		UNTIL (item = ok) OR (item = cancel);

	DisposDialog (dp);
	HPurge (Handle (dt));

	GetParameters := (item = ok)

	END;

{*****************************************************************************}

PROCEDURE DoStart;

{ Asks the user and the returns to the calling program the image parmaters. }

	VAR
		j: INTEGER;

	BEGIN

	WITH gStuff^ DO
		BEGIN

		imageSize.v := gLastRows;
		imageSize.h := gLastCols;
		imageMode	:= gLastMode;

		IF NOT GetParameters (imageSize.v, imageSize.h, imageMode) THEN
			BEGIN
			gResult := 1;
			EXIT (DoStart)
			END;

		gLastRows := imageSize.v;
		gLastCols := imageSize.h;
		gLastMode := imageMode;

		gNextRow   := 0;
		gNextPlane := 0;

		IF imageMode = acquireModeBitmap THEN
			depth := 1
		ELSE
			depth := 8;

		IF imageMode = acquireModeRGBColor THEN
			planes := 3
		ELSE
			planes := 1;

		IF depth = 1 THEN
			BEGIN
			imageHRes := FixRatio (300, 1);
			imageVRes := FixRatio (300, 1)
			END
		ELSE
			BEGIN
			imageHRes := FixRatio (72, 1);
			imageVRes := FixRatio (72, 1)
			END;

		IF imageMode = acquireModeIndexedColor THEN
			FOR j := 0 TO 255 DO
				BEGIN
				redLUT	 [j] := CHR (j);
				greenLUT [j] := CHR (255 - j);
				blueLUT  [j] := CHR (j)
				END;

		data := NIL

		END

	END;

{*****************************************************************************}

PROCEDURE DoContinue;

{ Returns the next hunk of the image to the calling program. }

	VAR
		p: Ptr;
		row: INTEGER;
		col: INTEGER;
		count: LONGINT;

	BEGIN

	WITH gStuff^ DO
		BEGIN

		IF data <> NIL THEN
			BEGIN
			DisposPtr (data);
			data := NIL
			END;

		IF gNextPlane <> planes THEN
			BEGIN

			colBytes := 1;

			IF depth = 8 THEN
				rowBytes := imageSize.h
			ELSE
				rowBytes := BSR (imageSize.h + 7, 3);

			loPlane := gNextPlane;
			hiPlane := gNextPlane;

			count := maxData DIV rowBytes;

			IF count < 1 THEN
				BEGIN
				gResult := memFullErr;
				EXIT (DoContinue)
				END;

			IF count > imageSize.v - gNextRow THEN
				count := imageSize.v - gNextRow;

			data := NewPtr (count * rowBytes);

			IF data = NIL THEN
				BEGIN
				gResult := memFullErr;
				EXIT (DoContinue)
				END;

			SetRect (theRect, 0, gNextRow, imageSize.h, gNextRow + count);

			p := data;

			FOR row := gNextRow TO gNextRow + count - 1 DO
				BEGIN

				IF TestAbort THEN
					BEGIN
					gResult := 1;
					EXIT (DoContinue)
					END;

				UpdateProgress (gNextPlane * ORD4 (imageSize.v) + row,
								planes * ORD4 (imageSize.v));

				FOR col := 0 TO rowBytes - 1 DO
					BEGIN

					{$PUSH}
					{$R-}

					IF depth = 1 THEN
						p^ := BSL ($FF, BAND (row, 7))

					ELSE IF planes = 1 THEN
						p^ := BAND (row + col, $FF)

					ELSE
						CASE gNextPlane OF
						0:	p^ := BAND (col, $FF);
						1:	p^ := BAND (row, $FF);
						2:	p^ := BAND (row + col, $FF)
						END;

					{$POP}

					p := Ptr (ORD4 (p) + 1)

					END

				END;

			gNextRow := gNextRow + count;

			IF gNextRow = imageSize.v THEN
				BEGIN
				gNextRow   := 0;
				gNextPlane := gNextPlane + 1
				END

			END

		END

	END;

{*****************************************************************************}

PROCEDURE DoFinish;

{ This routine will always be called if DoStart does not return an error
  (even if DoContinue returns an error or the user aborts the operation).
  This allows the module to perform any needed cleanup. }

	BEGIN

	WITH gStuff^ DO
		IF data <> NIL THEN
			BEGIN
			DisposPtr (data);
			data := NIL
			END

	END;

{*****************************************************************************}

PROCEDURE Dispatch (selector: INTEGER;
					stuff: AcquireRecordPtr;
					VAR data: Handle;
					VAR result: INTEGER);

{ Main dispatching routine.  Initializes and sets up the global variables,
  and performs the operation specified by the selector. }

	VAR
		firstTime: BOOLEAN;

	BEGIN

	{ See if this is the first time called }

	firstTime := (data = NIL);

	{ If so, allocate and initialize a global data area }

	IF firstTime THEN
		BEGIN

		MakeA5World (data);

		IF data = NIL THEN
			BEGIN
			result := memFullErr;
			EXIT (Dispatch)
			END

		END;

	{ Switch in our global variables }

	gHostA5 := SetA5World (data);
	gMyA5	:= ComputeA5 (data);

	IF firstTime THEN
		BEGIN

		{ Make a copy of the host's quickdraw globals }

		BlockMove (Ptr (ORD4 (Handle (gHostA5)^) - 126), @randSeed, 130);

		{ Initialize the plug-in global values }

		InitGlobals

		END;

	{ Perform the requested operation }

	gStuff	:= stuff;
	gResult := noErr;

		CASE selector OF

		acquireSelectorAbout:
			DoAbout;

		acquireSelectorPrepare:
			DoPrepare;

		acquireSelectorStart:
			DoStart;

		acquireSelectorContinue:
			DoContinue;

		acquireSelectorFinish:
			DoFinish;

		OTHERWISE
			gResult := acquireBadParameters

		END;

	result := gResult;

	{ Restore the application's globals }

	RestoreA5World (gHostA5, data)

	END;

{*****************************************************************************}

END.
