{
	File: DummySave.p

	Copyright 1990 by Thomas Knoll.

	MPW Pascal source file for DummySave example.
}

UNIT DummySave;

INTERFACE

USES
	MemTypes, QuickDraw, OSIntf, ToolIntf, PackIntf,
	ExportInterface;

PROCEDURE ENTRYPOINT (selector: INTEGER;
					  stuff: ExportRecordPtr;
					  VAR data: Handle;
					  VAR result: INTEGER);

IMPLEMENTATION

VAR
	gMyA5: LONGINT;
	gHostA5: LONGINT;

	gResult: INTEGER;
	gStuff: ExportRecordPtr;

	gFRefNum: INTEGER;
	gVRefNum: INTEGER;

PROCEDURE Dispatch (selector: INTEGER;
					stuff: ExportRecordPtr;
					VAR data: Handle;
					VAR result: INTEGER); FORWARD;

{*****************************************************************************}

PROCEDURE ENTRYPOINT (selector: INTEGER;
					  stuff: ExportRecordPtr;
					  VAR data: Handle;
					  VAR result: INTEGER);

{ All calls to the plug-in module come through this routine. It must be
  placed first in the resource. To achieve this, most development systems
  require that this be the first routine in the source. }

	BEGIN
	Dispatch (selector, stuff, data, result)
	END;

{*****************************************************************************}

{ The following routines are based on Macintosh Technical Note #256. }

FUNCTION A5Size: LONGINT; C; EXTERNAL;

PROCEDURE A5Init (myA5: Ptr); C; EXTERNAL;

FUNCTION ComputeA5 (A5Ref: Handle): LONGINT;
	BEGIN
	ComputeA5 := ORD4 (A5Ref^) + A5Size - 32
	END;

PROCEDURE MakeA5World (VAR A5Ref: Handle);
	BEGIN
	A5Ref := NewHandle (A5Size);
	IF A5Ref <> NIL THEN
		BEGIN
		HLock (A5Ref);
		A5Init (Ptr (ComputeA5 (A5Ref)));
		HUnlock (A5Ref)
		END
	END;

FUNCTION SetA5World (A5Ref: Handle): LONGINT;
	BEGIN
	HLock (A5Ref);
	SetA5World := SetA5 (ComputeA5 (A5Ref))
	END;

PROCEDURE RestoreA5World (oldA5: LONGINT; A5Ref: Handle);

	VAR
		ignore: LONGINT;

	BEGIN
	ignore := SetA5 (oldA5);
	HUnlock (A5Ref)
	END;

{*****************************************************************************}

FUNCTION TestAbort: BOOLEAN;

{ Calls the host's TestAbort function }

	VAR
		ourA5: LONGINT;
		address: ProcPtr;

	FUNCTION DoTestAbort (codeAddress: ProcPtr): BOOLEAN;
		INLINE $205F, $4E90;

	BEGIN

	address := gStuff^.abortProc;

	ourA5 := SetA5 (gHostA5);

	TestAbort := DoTestAbort (address);

	ourA5 := SetA5 (ourA5)

	END;

{*****************************************************************************}

PROCEDURE UpdateProgress (done, total: LONGINT);

{ Calls the host's UpdateProgress procedure }

	VAR
		ourA5: LONGINT;
		address: ProcPtr;

	PROCEDURE DoUpdateProgress (done, total: LONGINT; codeAddress: ProcPtr);
		INLINE $205F, $4E90;

	BEGIN

	address := gStuff^.progressProc;

	ourA5 := SetA5 (gHostA5);

	DoUpdateProgress (done, total, address);

	ourA5 := SetA5 (ourA5)

	END;

{*****************************************************************************}

PROCEDURE InitGlobals;

{ Sets the global variables to their default values. }

	BEGIN
	END;

{*****************************************************************************}

PROCEDURE CenterDialog (dt: DialogTHndl);

{ Centers a dialog template 1/3 of the way down on the main screen. }

	CONST
		menuHeight = 20;

	VAR
		r: Rect;
		width: INTEGER;
		height: INTEGER;

	BEGIN

	width  := screenBits.bounds.right;
	height := screenBits.bounds.bottom;

	r := dt^^.boundsRect;
	OffsetRect (r, -r.left, -r.top);
	OffsetRect (r, (width - r.right) DIV 2,
				   (height - r.bottom - menuHeight) DIV 3 + menuHeight);
	dt^^.boundsRect := r

	END;

{*****************************************************************************}

PROCEDURE DoAbout;

{ Displays the about dialog box for the plug-in module. }

	CONST
		dialogID = 17000;

	VAR
		dp: DialogPtr;
		item: INTEGER;
		ourA5: LONGINT;
		dt: DialogTHndl;

	BEGIN

	dt := DialogTHndl (GetResource ('DLOG', dialogID));
	HNoPurge (Handle (dt));
	CenterDialog (dt);

	dp := GetNewDialog (dialogID, NIL, WindowPtr (-1));
	
	ourA5 := SetA5 (gHostA5);

	ModalDialog (NIL, item);
	
	ourA5 := SetA5 (ourA5);

	DisposDialog (dp);
	HPurge (Handle (dt))

	END;

{*****************************************************************************}

PROCEDURE DoPrepare;

{ Prepare to export an image.  If the plug-in module needs only a limited
  amount of memory, it can lower the value of the 'maxData' field. }

	BEGIN

	IF gStuff^.maxData > $80000 THEN
		gStuff^.maxData := $80000

	END;

{*****************************************************************************}

PROCEDURE RequestNext (top: INTEGER);

{ Request next block of the image }

	VAR
		count: LONGINT;

	BEGIN

	WITH gStuff^ DO
		BEGIN

		{ Compute maximum number of rows we should request at once }

		count := maxData DIV imageSize.h DIV planes;

		IF count < 1 THEN count := 1;

		{ Request next block of the image }

		loPlane := 0;
		hiPlane := planes - 1;

		theRect.left  := 0;
		theRect.right := imageSize.h;
		theRect.top   := top;

		IF top + count > imageSize.v THEN
			theRect.bottom := imageSize.v
		ELSE
			theRect.bottom := top + count

		END

	END;

{*****************************************************************************}

PROCEDURE DoStart;

{ Verifies that the image mode is supported.  Prompts the user for
  a file name to save the image to, and requests the first block of
  of image data. }

	CONST
		kStringsID = 17000;

	VAR
		err: OSErr;
		where: Point;
		name: Str255;
		ourA5: LONGINT;
		reply: SFReply;
		prompt: Str255;
		dt: DialogTHndl;

	BEGIN

	{ This plug-in does not support bitmap images }

	IF gStuff^.imageMode = exportModeBitmap THEN
		BEGIN
		gResult := exportBadMode;
		EXIT (DoStart)
		END;

	{ Get prompt string }

	GetIndString (prompt, kStringsID, 1);

	{ Center dialog }

	dt := DialogTHndl (GetResource ('DLOG', putDlgID));
	HNoPurge (Handle (dt));

	CenterDialog (dt);
	where := dt^^.boundsRect.topLeft;

	HPurge (Handle (dt));

	{ Ask the user }
	
	name := gStuff^.fileName;
	
	ourA5 := SetA5 (gHostA5);

	SFPutFile (where, prompt, name, NIL, reply);
	
	ourA5 := SetA5 (ourA5);

	IF NOT reply.good THEN
		BEGIN
		gResult := 1;
		EXIT (DoStart)
		END;

	gVRefNum := reply.vRefNum;

	err := FSDelete (reply.fName, gVRefNum);

	err := Create (reply.fName, gVRefNum, '????', '????');

	IF err <> noErr THEN
		BEGIN
		gResult := err;
		EXIT (DoStart)
		END;

	err := FSOpen (reply.fName, gVRefNum, gFRefNum);

	IF err <> noErr THEN
		BEGIN
		gResult := err;
		EXIT (DoStart)
		END;

	{ Request first block of image }

	RequestNext (0)

	END;

{*****************************************************************************}

PROCEDURE DoContinue;

{ Processes the next chunk of the image. }

	VAR
		err: OSErr;
		row: INTEGER;
		count: LONGINT;

	BEGIN

	WITH gStuff^ DO
		BEGIN

		FOR row := theRect.top TO theRect.bottom - 1 DO
			BEGIN

			{ See if user was aborted the operation }

			IF TestAbort THEN
				BEGIN
				gResult := 1;
				EXIT (DoContinue)
				END;

			{ Update the progress indicator }

			UpdateProgress (row, imageSize.v);

			{ Write out this row of the image }

			count := planes * ORD4 (imageSize.h);

			err := FSWrite (gFRefNum,
							count,
							Ptr (ORD4 (data) +
								 rowBytes * (row - theRect.top)));

			IF err <> noErr THEN
				BEGIN
				gResult := err;
				EXIT (DoContinue)
				END

			END;

		{ Request next block of image }

		RequestNext (theRect.bottom)

		END

	END;

{*****************************************************************************}

PROCEDURE DoFinish;

{ This routine will always be called if DoStart does not return an error
  (even if DoContinue returns an error or the user aborts the operation).
  This allows the module to perform any needed cleanup. }

	VAR
		err: OSErr;

	BEGIN

	{ Close the file }

	err := FSClose (gFRefNum);

	IF err <> noErr THEN
		BEGIN
		gResult := err;
		EXIT (DoFinish)
		END;

	{ Flush the volume }

	err := FlushVol (NIL, gVRefNum);

	IF err <> noErr THEN
		BEGIN
		gResult := err;
		EXIT (DoFinish)
		END;

	{ Clear the image's unsaved changes flag }

	gStuff^.dirty := FALSE

	END;

{*****************************************************************************}

PROCEDURE Dispatch (selector: INTEGER;
					stuff: ExportRecordPtr;
					VAR data: Handle;
					VAR result: INTEGER);

{ Main dispatching routine.  Initializes and sets up the global variables,
  and performs the operation specified by the selector. }

	VAR
		firstTime: BOOLEAN;

	BEGIN

	{ See if this is the first time called }

	firstTime := (data = NIL);

	{ If so, allocate and initialize a global data area }

	IF firstTime THEN
		BEGIN

		MakeA5World (data);

		IF data = NIL THEN
			BEGIN
			result := memFullErr;
			EXIT (Dispatch)
			END

		END;

	{ Switch in our global variables }

	gHostA5 := SetA5World (data);
	gMyA5	:= ComputeA5 (data);

	IF firstTime THEN
		BEGIN

		{ Make a copy of the host's quickdraw globals }

		BlockMove (Ptr (ORD4 (Handle (gHostA5)^) - 126), @randSeed, 130);

		{ Initialize the plug-in global values }

		InitGlobals

		END;

	{ Perform the requested operation }

	gStuff	:= stuff;
	gResult := noErr;

		CASE selector OF

		exportSelectorAbout:
			DoAbout;

		exportSelectorPrepare:
			DoPrepare;

		exportSelectorStart:
			DoStart;

		exportSelectorContinue:
			DoContinue;

		exportSelectorFinish:
			DoFinish;

		OTHERWISE
			gResult := exportBadParameters

		END;

	result := gResult;

	{ Restore the application's globals }

	RestoreA5World (gHostA5, data)

	END;

{*****************************************************************************}

END.
