{
	File: Dissolve.p

	Copyright 1990-91 by Thomas Knoll.

	MPW Pascal source file for Dissolve example.
}

UNIT Dissolve;

INTERFACE

USES
	MemTypes, QuickDraw, OSIntf, ToolIntf, PackIntf,
	FilterInterface;

PROCEDURE ENTRYPOINT (selector: INTEGER;
					  stuff: FilterRecordPtr;
					  VAR data: Handle;
					  VAR result: INTEGER);

IMPLEMENTATION

TYPE
	TParameters = RECORD
		percent: INTEGER
		END;

	PParameters = ^TParameters;
	HParameters = ^PParameters;

VAR
	gMyA5: LONGINT;
	gHostA5: LONGINT;

	gResult: INTEGER;
	gStuff: FilterRecordPtr;

PROCEDURE Dispatch (selector: INTEGER;
					stuff: FilterRecordPtr;
					VAR data: Handle;
					VAR result: INTEGER); FORWARD;

{*****************************************************************************}

PROCEDURE ENTRYPOINT (selector: INTEGER;
					  stuff: FilterRecordPtr;
					  VAR data: Handle;
					  VAR result: INTEGER);

{ All calls to the plug-in module come through this routine. It must be
  placed first in the resource. To achieve this, most development systems
  require that this be the first routine in the source. }

	BEGIN
	Dispatch (selector, stuff, data, result)
	END;

{*****************************************************************************}

{ The following routines are based on Macintosh Technical Note #256. }

FUNCTION A5Size: LONGINT; C; EXTERNAL;

PROCEDURE A5Init (myA5: Ptr); C; EXTERNAL;

FUNCTION ComputeA5 (A5Ref: Handle): LONGINT;
	BEGIN
	ComputeA5 := ORD4 (A5Ref^) + A5Size - 32
	END;

PROCEDURE MakeA5World (VAR A5Ref: Handle);
	BEGIN
	A5Ref := NewHandle (A5Size);
	IF A5Ref <> NIL THEN
		BEGIN
		HLock (A5Ref);
		A5Init (Ptr (ComputeA5 (A5Ref)));
		HUnlock (A5Ref)
		END
	END;

FUNCTION SetA5World (A5Ref: Handle): LONGINT;
	BEGIN
	HLock (A5Ref);
	SetA5World := SetA5 (ComputeA5 (A5Ref))
	END;

PROCEDURE RestoreA5World (oldA5: LONGINT; A5Ref: Handle);

	VAR
		ignore: LONGINT;

	BEGIN
	ignore := SetA5 (oldA5);
	HUnlock (A5Ref)
	END;

{*****************************************************************************}

FUNCTION TestAbort: BOOLEAN;

{ Calls the host's TestAbort function }

	VAR
		ourA5: LONGINT;
		address: ProcPtr;

	FUNCTION DoTestAbort (codeAddress: ProcPtr): BOOLEAN;
		INLINE $205F, $4E90;

	BEGIN

	address := gStuff^.abortProc;

	ourA5 := SetA5 (gHostA5);

	TestAbort := DoTestAbort (address);

	ourA5 := SetA5 (ourA5)

	END;

{*****************************************************************************}

PROCEDURE UpdateProgress (done, total: LONGINT);

{ Calls the host's UpdateProgress procedure }

	VAR
		ourA5: LONGINT;
		address: ProcPtr;

	PROCEDURE DoUpdateProgress (done, total: LONGINT; codeAddress: ProcPtr);
		INLINE $205F, $4E90;

	BEGIN

	address := gStuff^.progressProc;

	ourA5 := SetA5 (gHostA5);

	DoUpdateProgress (done, total, address);

	ourA5 := SetA5 (ourA5)

	END;

{*****************************************************************************}

PROCEDURE CenterDialog (dt: DialogTHndl);

{ Centers a dialog template 1/3 of the way down on the main screen. }

	CONST
		menuHeight = 20;

	VAR
		r: Rect;
		width: INTEGER;
		height: INTEGER;

	BEGIN

	width  := screenBits.bounds.right;
	height := screenBits.bounds.bottom;

	r := dt^^.boundsRect;
	OffsetRect (r, -r.left, -r.top);
	OffsetRect (r, (width - r.right) DIV 2,
				   (height - r.bottom - menuHeight) DIV 3 + menuHeight);
	dt^^.boundsRect := r

	END;

{*****************************************************************************}

PROCEDURE DoAbout;

{ Displays the about dialog box for the plug-in module. }

	CONST
		dialogID  = 16000;

	VAR
		dp: DialogPtr;
		item: INTEGER;
		ourA5: LONGINT;
		dt: DialogTHndl;

	BEGIN

	dt := DialogTHndl (GetResource ('DLOG', dialogID));
	HNoPurge (Handle (dt));
	CenterDialog (dt);

	dp := GetNewDialog (dialogID, NIL, WindowPtr (-1));
	
	ourA5 := SetA5 (gHostA5);

	ModalDialog (NIL, item);

	ourA5 := SetA5 (ourA5);

	DisposDialog (dp);
	HPurge (Handle (dt))

	END;

{*****************************************************************************}

PROCEDURE OutlineOK (dp: DialogPtr; item: INTEGER);

{ UserItem to outline the OK button in a dialog box. }

	VAR
		r: Rect;
		h: Handle;
		newA5: LONGINT;
		oldA5: LONGINT;
		itemType: INTEGER;

	BEGIN

	newA5 := GetWRefCon (dp);
	oldA5 := SetA5 (newA5);
	
	GetDItem (dp, ok, itemType, h, r);
	
	PenNormal;
	PenSize (3, 3);
	
	InsetRect (r, -4, -4);
	FrameRoundRect (r, 16, 16);

	PenNormal;

	oldA5 := SetA5 (oldA5)

	END;

{*****************************************************************************}

PROCEDURE DoParameters;

{ Asks the user for the plug-in filter module's parameters. Note that
  the image size information is not yet defined at this point. Also, do
  not assume that the calling program will call this routine every time the
  filter is run (it may save the data held by the parameters handle in
  a macro file). }

	CONST
		dialogID	= 16001;
		hookItem	= 3;
		percentItem = 4;

	VAR
		r: Rect;
		s: Str255;
		h: Handle;
		j: INTEGER;
		x: LONGINT;
		dp: DialogPtr;
		item: INTEGER;
		ourA5: LONGINT;
		dt: DialogTHndl;
		itemType: INTEGER;
		percentText: Handle;

	BEGIN

	WITH gStuff^ DO
		BEGIN

		IF parameters = NIL THEN
			BEGIN

			parameters := NewHandle (SIZEOF (TParameters));

			IF parameters = NIL THEN
				BEGIN
				gResult := memFullErr;
				EXIT (DoParameters)
				END;

			HParameters (parameters)^^.percent := 50

			END;

		dt := DialogTHndl (GetResource ('DLOG', dialogID));
		HNoPurge (Handle (dt));
		CenterDialog (dt);

		dp := GetNewDialog (dialogID, NIL, WindowPtr (-1));

		SetWRefCon (dp, gMyA5);
	
		GetDItem (dp, hookItem, itemType, h 				 , r);
		SetDItem (dp, hookItem, itemType, Handle (@OutlineOK), r);

		GetDItem (dp, percentItem, itemType, percentText, r);

		NumToString (HParameters (parameters)^^.percent, s);
		SetIText (percentText, s);

			REPEAT

			SelIText (dp, percentItem, 0, 32767);

			ourA5 := SetA5 (gHostA5);
		
			ModalDialog (NIL, item);
		
			ourA5 := SetA5 (ourA5);

			IF item = ok THEN
				BEGIN

				GetIText (percentText, s);

				StringToNum (s, x);

				IF (x < 1) OR (x > 99) THEN
					item := 0;

				FOR j := 1 TO LENGTH (s) DO
					IF (s[j] < '0') OR (s[j] > '9') THEN
						item := 0;

				IF item = 0 THEN
					SysBeep (1)

				END

			UNTIL (item = ok) OR (item = cancel);

		DisposDialog (dp);
		HPurge (Handle (dt));

		IF item = cancel THEN
			BEGIN
			gResult := 1;
			EXIT (DoParameters)
			END;

		HParameters (parameters)^^.percent := x

		END

	END;

{*****************************************************************************}

PROCEDURE DoPrepare;

{ Prepare to filter an image.  If the plug-in filter needs a large amount
  of buffer memory, this routine should set the bufferSpace field to the
  number of bytes required. }

	BEGIN
	END;

{*****************************************************************************}

PROCEDURE DoStart;

{ Requests pointer to the first part of the image to be filtered. }

	BEGIN

	WITH gStuff^ DO
		BEGIN

		inRect		  := filterRect;
		inRect.bottom := inRect.top + 1;

		inLoPlane := 0;
		
			CASE imageMode OF
			
			filterModeGrayScale:
				inHiPlane := 0;
				
			filterModeRGBColor,
			filterModeHSBColor,
			filterModeHSLColor:
				inHiPlane := 2;
				
			filterModeCMYKColor:
				inHiPlane := 3;
				
			OTHERWISE
				gResult := filterBadMode
				
			END;

		outRect := inRect;

		outLoPlane := inLoPlane;
		outHiPlane := inHiPlane

		END

	END;

{*****************************************************************************}

PROCEDURE DoContinue;

{ Filters the area and requests the next area. }

	VAR
		srcPtr: Ptr;
		dstPtr: Ptr;
		count: LONGINT;
		plane: INTEGER;
		percent: INTEGER;

	BEGIN

	WITH gStuff^ DO
		BEGIN

		IF TestAbort THEN
			BEGIN
			gResult := 1;
			EXIT (DoContinue)
			END;

		UpdateProgress (inRect.top - filterRect.top,
						filterRect.bottom - filterRect.top);

		percent := HParameters (parameters)^^.percent;

		srcPtr := inData;
		dstPtr := outData;

		count := filterRect.right - filterRect.left;

		WHILE count > 0 DO
			BEGIN

			{$PUSH}
			{$R-}

			IF ABS (Random) MOD 100 < percent THEN
				BEGIN

				FOR plane := 0 TO inHiPlane DO
					BEGIN
					dstPtr^ := ORD (backColor [plane]);
					dstPtr	:= Ptr (ORD4 (dstPtr) + 1)
					END;

				srcPtr := Ptr (ORD4 (srcPtr) + planes)

				END

			ELSE
				FOR plane := 0 TO inHiPlane DO
					BEGIN
					dstPtr^ := srcPtr^;
					dstPtr	:= Ptr (ORD4 (dstPtr) + 1);
					srcPtr	:= Ptr (ORD4 (srcPtr) + 1)
					END;

			{$POP}

			count := count - 1

			END;

		inRect.top	  := inRect.top    + 1;
		inRect.bottom := inRect.bottom + 1;

		IF inRect.top >= filterRect.bottom THEN
			SetRect (inRect, 0, 0, 0, 0);

		outRect := inRect

		END

	END;

{*****************************************************************************}

PROCEDURE DoFinish;

{ This routine will always be called if DoStart does not return an error
  (even if DoContinue returns an error or the user aborts the operation).
  This allows the module to perform any needed cleanup.  None is required
  in this example. }

	BEGIN
	END;

{*****************************************************************************}

PROCEDURE Dispatch (selector: INTEGER;
					stuff: FilterRecordPtr;
					VAR data: Handle;
					VAR result: INTEGER);

{ Main dispatching routine.  Initializes and sets up the global variables,
  and performs the operation specified by the selector. }

	VAR
		firstTime: BOOLEAN;

	BEGIN

	{ See if this is the first time called }

	firstTime := (data = NIL);

	{ If so, allocate and initialize a global data area }

	IF firstTime THEN
		BEGIN

		MakeA5World (data);

		IF data = NIL THEN
			BEGIN
			result := memFullErr;
			EXIT (Dispatch)
			END

		END;

	{ Switch in our global variables }

	gHostA5 := SetA5World (data);
	gMyA5	:= ComputeA5 (data);

	{ Make a copy of the host's quickdraw globals }

	IF firstTime THEN
		BlockMove (Ptr (ORD4 (Handle (gHostA5)^) - 126), @randSeed, 130);

	{ Perform the requested operation }

	gStuff	:= stuff;
	gResult := noErr;

		CASE selector OF

		filterSelectorAbout:
			DoAbout;

		filterSelectorParameters:
			DoParameters;

		filterSelectorPrepare:
			DoPrepare;

		filterSelectorStart:
			DoStart;

		filterSelectorContinue:
			DoContinue;

		filterSelectorFinish:
			DoFinish;

		OTHERWISE
			gResult := filterBadParameters

		END;

	result := gResult;

	{ Restore the application's globals }

	RestoreA5World (gHostA5, data)

	END;

{*****************************************************************************}

END.
