/*********************************************************************

	Color IW Headers.h
	
	This file contains headers for the C & Rez code of the
	"Adventures in Color Printing" color ImageWriter sample,
	"Color IW Adventures."
	
	Dave Hersey, Apple Developer Technical Support.
	
	 Edit Trail 
	
	creation/reorg/rebirth:						 6/29/92  - dmh
	sync'd with other samples:					10/22/92  - dmh
	modified for error notification:			10/22/92  - dmh
	pre-final clean up on election day  8) :	11/03/92  - dmh
	Final clean up and sync with other samples:	11/29/92  - dmh

*********************************************************************/

#ifndef COLORIW_ADVENTURE_HDRS
#define COLORIW_ADVENTURE_HDRS


/* Constants:	*/

// Basic defines used by resources:

#define AppName				"Color ImageWriter Adventures"
#define AppVers				"1.0.1"
#define FinderInfo			"develop issue #13 sample"
#define ApplCreator			'cIWa'

// Misc. app. defines:

#define kMaxPICTHeapPrcnt	.5		/* If a picture to load requires more	*/
									/* this amount of the largest block of	*/
									/* memory available, we spool it.		*/

#define kMaxNumCursors		20		/* Max number of spinning cursors we	*/
									/* allocate space for.					*/
#define kMinCursorTicks		10		/* Min number of ticks that must pass	*/
									/* between spinning cursor changes.		*/

#define kDITop				100		/* Top coordinate for file dialogs.		*/
#define kDILeft				100		/* Left coordinate for file dialogs.	*/

#define kArrowCursor		0		/* Pseudo resource ID for arrow cursor	*/
									/* which is actually a QD global.  We	*/
									/* use this in TearDownCursors.			*/

#define kPadAmount			8		/* Number of pixels to pad image with	*/
									/* when banding a dithered image.  See	*/
									/* BandPicture for more information.	*/

#define n_dlogLevel			.25		/* Level to display our dialogs at as a	*/
									/* percentage of the way down the		*/
									/* screen.								*/


// Application-defined error codes used by resources and AlertUser:

#define unknownErr			0		/* OK to use 0, since it's noErr.		*/
#define no32BQDErr			31		/* OK to use 31, it's obsolete.			*/
#define noPrinterChosenErr	42		/* OK to use 42, it's obsolete.			*/

// all other resource IDs are the same as their error codes.


// Misc. resource defines:

#define r_signature			0		/* ID of our signature resource.		*/
#define r_bundle			128		/* ID of our 'BNDL' resource.			*/
#define r_refAPPL			128		/* ID of our APPL 'FREF' resource.		*/
#define r_refPICT			129		/* ID of our PICT 'FREF' resource.		*/

#define kMinSize			512		/* Min app size for 'SIZE' resource.	*/
#define kPrefSize			3000	/* Preferred app size for 'SIZE' res.	*/

#define r_iconAPPL			128		/* ID of our app icon resource.			*/
#define r_iconPICT			129		/* ID of our 'PICT' icon resource.		*/

#define r_aboutBox			128		/* ID of our About dialog.				*/

#define r_statDlog			1262	/* ID of our status dialog.				*/
#define r_dlogIcon			240		/* ID of our status dialog icon.		*/
#define t_statusItem		2		/* Item# of status dialog's text field.	*/
#define d_cancelItem		3		/* Item# of status dialog's "Press		*/
									/* command-period to cancel." text.		*/

#define r_errorDialog		130		/* ID of our error dialog.				*/
#define r_errorIcon			130		/* ID of our error dialog icon.			*/
#define d_OKButton			1		/* Item# of error dialog's OK button.	*/
#define d_errMsg			3		/* Item# of error dialog's error text.	*/
#define d_errNum			4		/* Item# of error dialog's error# text.	*/

#define r_printHdlType		'PHDL'	/* Res. type of our saved print handles.*/
#define r_optType			'OPTS'	/* Resource type of our saved options.	*/
#define r_settingsID		3		/* Resource ID of our saved options.	*/

#define r_spoolSpinCursor	120		/* ID of our spooling 'acur' resource.	*/
#define r_spoolCurs1		140		/* IDs of the cursors referenced by the	*/
#define r_spoolCurs2		141		/* above 'acur'.						*/
#define r_spoolCurs3		142
#define r_spoolCurs4		143

#define r_stdSpinCursor		121		/* ID of our regular 'acur' resource.	*/
#define r_stdCurs1			150		/* IDs of the cursors referenced by the	*/
#define r_stdCurs2			151		/* above 'acur'.						*/
#define r_stdCurs3			152
#define r_stdCurs4			153


// Menu resources

#define r_menuBar			128
#define mApple				128
#define iAbout				1

#define mFile				129
#define iOpen				1
#define iClose				2
#define iPageSetup			4
#define iPrint				5
#define iQuit				7

#define mEdit				130
#define iUndo				1
#define iCut				3
#define iCopy				4
#define iPaste				5
#define iClear				6

#define mOptions			131
#define iditherCopy			1
#define isrcCopy			2
#define ihighRes			4

#define AllItems		0b1111111111111111111111111111111
#define NoItems			0b0000000000000000000000000000000
#define MenuItem1		0b0000000000000000000000000000001
#define MenuItem2		0b0000000000000000000000000000010
#define MenuItem3		0b0000000000000000000000000000100
#define MenuItem4		0b0000000000000000000000000001000
#define MenuItem5		0b0000000000000000000000000010000
#define MenuItem6		0b0000000000000000000000000100000
#define MenuItem7		0b0000000000000000000000001000000
#define MenuItem8		0b0000000000000000000000010000000
#define MenuItem9		0b0000000000000000000000100000000
#define MenuItem10		0b0000000000000000000001000000000
#define MenuItem11		0b0000000000000000000010000000000
#define MenuItem12		0b0000000000000000000100000000000
#define MenuItem13		0b0000000000000000001000000000000
#define MenuItem14		0b0000000000000000010000000000000



#ifndef COLORIW_REZZING

/* Required include files for C sources: */

#include <AppleEvents.h>
#include <Desk.h>
#include <Dialogs.h>
#include <DiskInit.h>
#include <Editions.h>
#include <Errors.h>
#include <Events.h>
#include <Files.h>
#include <FixMath.h>
#include <Fonts.h>
#include <GestaltEqu.h>
#include <Menus.h>
#include <OSEvents.h>
#include <Packages.h>
#include <Printing.h>
#include <QDOffscreen.h>
#include <Resources.h>
#include <Scrap.h>
#include <script.h>
#include <SegLoad.h>
#include <StandardFile.h>
#include <stdio.h>
#include <Strings.h>
#include <SysEqu.h>
#include <ToolUtils.h>
#include <Traps.h>



/* Macros: */

#define ABS(x)     ((x < 0)? -x:x)
#define MAX(x, y)  ((x < y)? y:x)
#define MIN(x, y)  ((x > y)? y:x)



/* Typedefs: */

// ACurRec -- This is just a C structure that parallels
// the 'acur' animated cursor resource.  The animated
// cursor routines use it.

typedef struct ACurRec {
       short	numCursors;		// Number of cursors.
       short	nextFrame;		// Next cursor to display.
       long		cursID[];		// IDs of crsr or CURS resources.
} ACurRec, *ACurPtr, **ACurHdl;



// PictureHeader -- This structure is just a C
// representation of a version 2 picture's header
// listed in Inside Macintosh VI (17-23)

typedef struct PictureHeader {
       short		version;	// Header version.
       short		reserved1;
	   Fixed		hRes;		// PICT horizontal resolution (if extended v2 pict).
	   Fixed		vRes;		// PICT vertical resolution (if extended v2 pict).
	   Rect			srcRect;	// PICT native bounds at hRes,vRes (if extended v2 pict).
	   long			reserved2;
} PictureHeader;



// PartialPicture -- This is the structure we use to
// hold a picture's version and version 2 header, if
// it has one.

typedef struct PartialPicture {
	   short			pictVersion;	// PICT version.
       PictureHeader	pictHeader;		// PICT header.
} PartialPicture, *PartialPicturePtr, **PartialPictureHdl;



// WindowInfoRec -- This structure is what ties an
// image's file and picture information to the
// window displaying it.  Every window we create
// has a handle to one of these structures in its
// window refCon field.

typedef struct WindowInfoRec {
		Boolean			loaded;			// Picture loaded?  (No means spool.)
		PicHandle		windowPict;		// Handle to picture for window.
		PartialPicture	partialPict;	// Version and hdr. info for picture.
		SFReply			reply;			// Pre-7.0 file info for picture.
		FSSpec			oFSSpec;		// System 7.0 file info for picture.
} WindowInfoRec, *WindowInfoPtr, **WindowInfoHdl;



/* Global variables: */

extern Boolean		gQuitting;			// True if ready to quit.
extern Boolean		gInBackground;		// True if running in the background.
extern Boolean		gSystemSeven;		// True if running under system 7 or later.
extern Boolean		gHaveNewSFCalls;	// True if we have the new Standard File calls.
extern Boolean		g8BQD;				// True if we have "regular" Color QuickDraw.
extern Boolean		g32BQD;				// True if we also have 32-Bit Color QuickDraw.
extern RgnHandle	gCursorRgn;			// Mouse-moved region used by event loop.
extern THPrint		ghPrint;			// Global print record.
extern short		gAppResRefNum;		// RefNum of application's resource file.



/* Function prototypes & file locations: */

// Color IW Main.c

void			main(void);


// Color IW Initialize.c

void			CheckConfig(void);
void			Initialize(void);


// Color IW Event Stuff.c

OSErr			CheckAEParams(AppleEvent *theAppleEvent);
void			DoAEInstallation(void);
void			DoEvent(EventRecord *event);
void			EventLoop(void);
pascal OSErr	HandleOAPP(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon);
pascal OSErr	HandleODOC(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon);
pascal OSErr	HandlePDOC(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon);
pascal OSErr	HandleQUIT(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon);


// Color IW Windows & Menus.c

void			AdjustMenus(void);
void			AlertUser(OSErr err);
void			CleanUp(void);
OSErr			CreatePictWindow(FSSpecPtr opFSSpec, SFReply *reply);
void			DoAnyUpdates(void);
void			DoCloseWindow(WindowPtr wp);
void			DoMenuCommand(long menuResult);
void			DoUpdate(WindowPtr wp);
PixMapHandle	GetGWPixMap(GWorldPtr aGWorld);
void			HideStatusMsg(DialogPtr *statDlog);
void			PositionWindow(WindowPtr windPtr, Boolean showIt, float vertPercent);
void			SetUpMenus(void);
void			ShowStatusMsg(DialogPtr *statDlog, Str255 msg1, Str255 docTitle,
							  Str255 msg2, Boolean showCancelMsg);

// Color IW File.c

void			BumpCursor(void);
void			GetOpenName(StandardFileReply *toReply7, SFReply *toReply);
OSErr			LoadAPICT(WindowInfoHdl windInfo, Rect *pictRect);
void			LoadOrPrintFiles(void);
void			SetUpCursors(short acurID, short finalCursorID);
OSErr			SpoolInstall(WindowInfoHdl windInfo, short cursorToUseWhenDone);
pascal void		SpoolPicProc(Ptr buffer, short amtToRead);
OSErr			SpoolPictToWindow(WindowPtr wp);
OSErr			SpoolRemove(void);
void			TearDownCursors(void);


// Color IW Printing.c

OSErr			BandPicture(WindowPtr wp, Rect *srcRect, GWorldPtr destGWorld,
							Boolean useDitherCopy);
void			CenterRect(Rect *srcRect, Rect *destRect, Rect *ctrRect);
pascal void		ColorBitsProc(BitMap *srcBits, Rect *srcRect, Rect *dstRect, short mode,
							  RgnHandle maskRgn);
OSErr			CommandPeriod(void);
OSErr			DoJobDlog(THPrint hPrint);
OSErr			DoPrintLoop(THPrint hPrint, PicHandle *pict, Rect *pictFrame);
OSErr			DoStyleDlog(THPrint hPrint);
void			GetBestDPI(short *pxDPI, short *pyDPI, short xDPI_ceiling,
						   short yDPI_ceiling, Boolean wantSquareDPI);
OSErr			GetImageRes(WindowInfoHdl windInfo, short *ixDPI, short *iyDPI,
							Rect *srcRect);
OSErr			GetPixMapRes(WindowInfoHdl windInfo, Fixed *hRes, Fixed *vRes,
							 Rect *srcRect);
void			GetResIDString(THPrint	hPrint, Str255	resIDStr);
pascal void		IdleProc(void);
OSErr			LoadPHdl(THPrint *hPrint, short resRefNum);
OSErr			PrintOne(WindowPtr whichWindow);
pascal Boolean	OQDSearch(RGBColor *anRGB, long *position);
OSErr			SavePHdl(THPrint hPrint, short resRefNum);
void			SetPrinterPortDPI(THPrint hPrint, short *pxDPI, short *pyDPI);
OSErr			SetupPrintHdl(THPrint *hPrint);
void			SetUpQDSearch(void);


#endif COLORIW_REZZING
#endif COLORIW_ADVENTURE_HDRS
