/*********************************************************************

	Color IW Printing.c
	
	This file contains printing functions for the
	"Adventures in Color Printing" color ImageWriter sample,
	"Color IW Adventures."
	
	Dave Hersey, Apple Developer Technical Support.
	
	 Edit Trail 
	
	creation/reorg/rebirth:						 6/29/92  - dmh
	cleaned up the mess:						 8/10/92  - dmh
	added support for picts of any dpi:			 9/22/92  - dmh
	added error notification:					10/22/92  - dmh
	cleaned up the mess (again):				10/23/92  - dmh
	added calls to picture spooling rtns:		10/27/92  - dmh
	pre-final clean up while dressed
	in my Halloween costume:					10/31/92  - dmh
	Corrected poor assumption about picFrame
	& image bounds alignment in ColorBitsProc:	11/28/92  - dmh
	final clean up and sync with other samples:	11/29/92  - dmh

	Hint: If you start looking at "PrintOne," you'll be able
	      to follow the chain of command easier.
	
	* Related functions are grouped together in the Mark menu. *
	
*********************************************************************/

#pragma segment Printing

#include "Color IW Headers.h"

static CTabHandle	gOrgQDCTab = nil;	// Storage for original QD CTable.
static char			gCurColor;			// Index to above for current
										// color we're searching for.

static Boolean		gRectNotSetYet;		// Flag used by ColorBitsProc.
static Fixed		gPMHRes;			// Horizontal resolution,
static Fixed		gPMVRes;			// vertical resolution,
static Rect			gSrcRect;			// and bounds of PixMap currently being
										// examined in ColorBitsProc.

/************************************************************
  PrintOne - This routine starts the printing process for
  the indicated window.  We take care of pre-printing
  rendering (in this case making color separations of the
  image) as well as setting resolutions, the document title,
  displaying the job dialog and so forth.  If everything
  succeeds, we pass control on to the DoPrintLoop routine.

*************************************************************/

OSErr PrintOne(WindowPtr whichWindow)
{
	QDErr				err;
	WindowInfoHdl		windInfo;
	GDHandle			savedGDH;
	GWorldPtr			savedGW, OQDGWorld = nil, colorSep = nil;
	PixMapHandle		srcPix, destPix;
	Boolean				useDitherCopy, printAtHighRes;
	DialogPtr			printStatDlog = nil;
	Str255				tempStr, orgTitle;
	OpenCPicParams		cPicParams;
	PicHandle			sepsPICT = nil;
	Rect				pictFrame, rPage, pageFrame;
	Fixed				scale;
	short				pxDPI, pyDPI, ixDPI, iyDPI, itemMark, numChar,
						QDColor[7] = {blackColor, yellowColor, magentaColor,
									  redColor, cyanColor, greenColor, blueColor};

// Get this window's picture and title.

	windInfo = (WindowInfoHdl) GetWRefCon(whichWindow);
	GetWTitle(whichWindow, orgTitle);


// Save the current GWorld and GDevice.  Set the appropriate flags if the
// user has selected "use ditherCopy" or "print at high resolution" from the
// options menu.

	GetGWorld(&savedGW, &savedGDH);
	GetItemMark(GetMHandle(mOptions), iditherCopy, &itemMark);
	useDitherCopy = (itemMark != noMark);
	GetItemMark(GetMHandle(mOptions), ihighRes, &itemMark);
	printAtHighRes = (itemMark != noMark);


// Get the image's native resolution and bounds.

	err = GetImageRes(windInfo, &ixDPI, &iyDPI, &pictFrame);
	if (err) return err;


// If we're printing at high resolution, get the highest resolution
// possible at or below 300 x 300 dpi.  We also request "square"
// resolutions (where the horizontal and vertical DPIs are
// equal).  If we're not using high resolution printing, specify
// the default (72 x 72 dpi) resolution.

	if (printAtHighRes)
	{
		pxDPI = pyDPI = 300;
		GetBestDPI(&pxDPI, &pyDPI, 300, 300, true);
	}
	else
		pxDPI = pyDPI = 72;


// Handle the print job dialog and, if there are no errors, save the
// print handle.  We set the printer resolution here (rather than above)
// because the DoJobDlog call will reset it to 72 dpi.  For more info on
// why it does this, see DoJobDlog.  We could have set the resolution
// before the dialogs (and changed the dialog code slightly), but I've
// done it this way to maintain consistency with the final sample, which
// MUST do things this way.
//
// Tell the user what we're doing, check for command-
// period, and set things up for our search proc.

	err = DoJobDlog(ghPrint);

	if (!err)
	{
		SetCursor(*GetCursor(watchCursor));
		SavePHdl(ghPrint, gAppResRefNum);
		SetPrinterPortDPI(ghPrint, &pxDPI, &pyDPI);


// For convenience in calculations, align the top-left of the
// picture's bounds with (0,0).  If the printer driver's
// resolution is not the same as the image's resolution, scale
// the image's picFrame appropriately so that our printed output
// has the same dimensions as the original.

		OffsetRect(&pictFrame, -pictFrame.left, -pictFrame.top);
	
		if ((pxDPI != ixDPI) || (pyDPI != iyDPI))
		{
			scale = FixRatio(pxDPI, ixDPI);
			pictFrame.right = FixMul(scale, (long) pictFrame.right <<16) >>16;
			scale = FixRatio(pyDPI, iyDPI);
			pictFrame.bottom = FixMul(scale, (long) pictFrame.bottom <<16) >>16;
		}

		numChar = sprintf(&tempStr[0], " %s %s.",
						  "to original QuickDraw colors using",
						  (useDitherCopy)? "ditherCopy": "srcCopy");
		
		tempStr[numChar] = '\0';
		
		ShowStatusMsg(&printStatDlog, "Mapping ", orgTitle, tempStr, true);
		err = CommandPeriod();
		SetUpQDSearch();
		

// Create a 4-bit GWorld which uses the original 8 QuickDraw colors.  If
// there are no errors, Band the picture, using ditherCopy if desired.
// The result is a representation of the image in the original 8 colors.

		rPage = (*ghPrint)->prInfo.rPage;
		SectRect(&pictFrame, &rPage, &pageFrame);

		err = NewGWorld(&OQDGWorld, 4, &pageFrame, gOrgQDCTab, nil, 0);
		
		if (!err)
			err = BandPicture(whichWindow, &pictFrame, OQDGWorld, useDitherCopy);


// Create a new 1-bit GWorld to draw the separations into.

		if (!err)
		{
			err = NewGWorld(&colorSep,1,&pageFrame, nil,nil,0);


// Start up our spinning cursors.  We set the final cursor to r_stdCurs4 so that
// when the printing loop starts spinning its cursors, they mesh with ours.
// Otherwise, you'd see the cursor flash to a watch or whatever in between.  Set
// the current GWorld to the 1-bit GWorld, and create a picture.  Note that this
// means that the picture is tied to the 1-bit GWorld.  Only when that GWorld is
// current will data be recorded into the picture.

			if (!err)
			{
				SetUpCursors(r_stdSpinCursor, r_stdCurs4);

				SetGWorld(colorSep, nil);
				srcPix = GetGWPixMap(OQDGWorld);
				LockPixels(srcPix);
				destPix = GetGWPixMap(colorSep);
				LockPixels(destPix);
	
				ClipRect(&pageFrame);
	
				cPicParams.srcRect = pageFrame;
				cPicParams.hRes = Long2Fix((long) pxDPI);
				cPicParams.vRes = Long2Fix((long) pyDPI);
				cPicParams.version = -2;

				sepsPICT = OpenCPicture(&cPicParams);

// Go through each of the 8 original colors (except white) and create a
// separation.  To do this, we set the current GWorld to whatever it was
// when we came into this routine.  Since this will not be the 1-bit
// GWorld, this turns off our picture's recording.  Next, we tell the
// user what we're doing, install our searchProc, and CopyBits from
// the 8-bit GWorld to the 1-bit one.  As mentioned in the comments
// for OQDSearch, this gives us black bits only where the color in the
// original matches the current separation color.  With the separation
// complete, we delete the search proc and set our current GWorld
// back to the 1-bit one.  This re-enables recording into our picture,
// and we record the foreground color for the current separation,
// followed by the bitmap using srcOr mode.  After all 7 passes have
// been completed, this will give us a picture with 7 separations in it,
// all overlaying each other to make the "original QuickDraw"
// represenation.  We use srcOr mode so that the white is transparent,
// otherwise the white for each layer would overwrite the color from
// the previous layer.

				for (gCurColor = 0; (gCurColor < 7) && !err; gCurColor++)
				{
					BumpCursor();					// Spin.
					SetGWorld(savedGW, savedGDH);
					sprintf(tempStr, "%s%i %s ", "Creating separation #", gCurColor +1, "of");
					ShowStatusMsg(&printStatDlog, tempStr, orgTitle, ".", true);

					err = CommandPeriod();

					BumpCursor();					// Spin.
					AddSearch(OQDSearch);
					CopyBits((BitMap *) *srcPix, (BitMap *) *destPix, &pageFrame, &pageFrame, srcCopy, nil);
					DelSearch(OQDSearch);
					SetGWorld(colorSep, nil);
	
					BumpCursor();					// Spin.
					ForeColor(QDColor[gCurColor]);
					CopyBits((BitMap *) *destPix, (BitMap *) *destPix, &pageFrame, &pageFrame, srcOr, nil);
				}


// Close the picture, restore our saved GWorld/GDevice, stop the cursor from
// spinning, and dispose of our GWorlds.  If the picture's picFrame is empty,
// we had a memory error.

				ClosePicture();
				SetGWorld(savedGW, savedGDH);
				UnlockPixels(destPix);
				UnlockPixels(srcPix);

				TearDownCursors();
			}

			if (!err && EmptyRect(&(*sepsPICT)->picFrame))
				err = memFullErr;

			if (colorSep)
			{
				DisposeGWorld(colorSep);
				colorSep = nil;
			}
			
			if (OQDGWorld)
			{
				DisposeGWorld(OQDGWorld);
				OQDGWorld = nil;
			}

// If no errors, tell the user we're about to start printing and
// then go do it.  When finished, hide our status dialog, throw
// away any data we have left over, restore the saved GWorld &
// GDevice and return any error that's occurred.

			if (!err)
			{
				sprintf(tempStr, "%s %i %s %i %s",  " at", pxDPI,
						"dpi horizontally by", pyDPI, "dpi vertically.");

				ShowStatusMsg(&printStatDlog, "Printing ", orgTitle, tempStr, true);
				err = DoPrintLoop(ghPrint, &sepsPICT, &pageFrame);
			}
		}

		HideStatusMsg(&printStatDlog);
	}

	if (gOrgQDCTab) DisposCTable(gOrgQDCTab);
	gOrgQDCTab = nil;

	if (OQDGWorld) DisposeGWorld(OQDGWorld);
	if (colorSep) DisposeGWorld(colorSep);
	if (sepsPICT) KillPicture(sepsPICT);

	SetGWorld(savedGW, savedGDH);
	SetCursor(&qd.arrow);

	return err;
}


/************************************************************
  DoPrintLoop - This routine does the actual printing.  The
  picture to be printed is passed, fully rendered for output.

*************************************************************/

OSErr DoPrintLoop(THPrint hPrint, PicHandle *pict, Rect *pictFrame)
{
	OSErr			err;
	TPPrPort		printPort;
	TPrStatus		theStatus;
	short			copy, iFstPage, iLstPage;
	Rect			rPage, pictBounds;

// Set the cursor spinning (we'll keep it spinning in our idle proc).  Open the
// printer driver and validate the print record so that the Printing Manager
// pulls our document title from the title of the frontmost window.  The reason
// we save and restore the iFstPage and iLstPage fields is that the LaserWriter
// driver (through at least v7.1.2) has a bug in PrValidate which causes these
// values to be reset.  While we aren't using iFstPage or iLstPage below, you
// never know where this code might end up in this crazy cut & paste world.

	SetUpCursors(r_stdSpinCursor, kArrowCursor);

	PrOpen();

	if (!(err = PrError()))
	{
		iFstPage = (*hPrint)->prJob.iFstPage;
		iLstPage = (*hPrint)->prJob.iLstPage;
		
		PrValidate(hPrint);
		
		(*hPrint)->prJob.iFstPage = iFstPage;
		(*hPrint)->prJob.iLstPage = iLstPage;


// Install our idle proc, then loop for the number of copies we need to
// create, open a document and a page, draw the picture centered on the
// page, and close the page and document.

		(*hPrint)->prJob.pIdleProc = (PrIdleProcPtr) IdleProc;

		for (copy = 1; !err && (copy <= (*hPrint)->prJob.iCopies); copy++)
		{
			printPort = PrOpenDoc(hPrint, nil, nil);
	
			if (!(err = PrError()))
			{
				PrOpenPage(printPort, nil);
				err = PrError();
				
				pictBounds = *pictFrame;
				rPage = (*hPrint)->prInfo.rPage;
				CenterRect(&pictBounds, &rPage, &pictBounds);
				DrawPicture(*pict, &pictBounds);

				PrClosePage(printPort);
			}
		
			PrCloseDoc(printPort);
			if (!err) err = PrError();
		}

// When finished, kill the picture, call PrPicFile (if necessary), remove our
// idle proc, stop those silly cursors from spinning, and close the printer driver.
	
		KillPicture(*pict);
		*pict = nil;

		if (!err && ((*hPrint)->prJob.bJDocLoop == bSpoolLoop))
			PrPicFile(hPrint, nil, nil, nil, &theStatus);
	}

	(*hPrint)->prJob.pIdleProc = nil;
	TearDownCursors();
	PrClose();
	return err;
}


/************************************************************
  GetBestDPI - This routine obtains the best horizontal &
  vertical resolutions to use for printing with the
  selected driver.
  
  pxDPI, pyDPI --  The caller places an "ideal" resolution
  				   pair here (what they really want to use).
				   Also, this is where the routine returns
				   the resolutions it decides on.

  xDPI_ceiling,
  yDPI_ceiling --  The caller places the maximum resolution
  				   they want in either direction.  For
				   example, if you didn't want values larger
				   than 300 dpi returned, you would put 300
				   in both of these parameters.

  wantSquareDPI -- If true, only square resolutions (equal
  				   horizontal and vertical components) will
				   be considered.

  The printer driver is expected to be closed upon entry to
  this routine, and is therefore opened and closed around
  the PrGeneral code.  If PrGeneral is not supported by this
  driver, or if an error occurs, the routine returns
  72x72 dpi, which is the default for Macintosh printer
  drivers.
  
  If the "ideal" resolution the caller passes in is
  available, we choose that, ignoring wantSquareDPI,
  xDPI_ceiling and yDPI_ceiling.  We figure that the calling
  routine knows more about the ideal resolution it requests
  than we do.

*************************************************************/

void GetBestDPI(short *pxDPI, short *pyDPI, short xDPI_ceiling,
				short yDPI_ceiling, Boolean wantSquareDPI)
{
	TGetRslBlk		getResRec;
	Boolean			exactMatch = false;
	short			bestResX, bestResY, xDPI, yDPI,
					desiredResX, desiredResY, rec;

// Store the desired resolution that the caller passed to us through
// the pxDPI and pyDPI parameters.  If 72x72 dpi is requested, return
// those values, since that resolution must always be supported by a
// Mac printer driver (even though it may not appear in the PrGeneral
// getRsl list) for some (confused) printer drivers.

	desiredResX = *pxDPI;
	desiredResY = *pyDPI;

	if ((desiredResX == 72) && (desiredResY == 72)) return;


// Open the driver for our PrGeneral call.  Assume that we'll return
// 72x72 DPI until we find otherwise, and ask PrGeneral for the
// resolution records for this driver.

	PrOpen();
	
	bestResX = bestResY = 72;

	if (!PrError())
	{
		getResRec.iOpCode = getRslDataOp;
		PrGeneral((Ptr) &getResRec);

		if ((!ResError()) && (!getResRec.iError))
		{

// First check for the exact resolution pair that the caller
// requested.  We'll start by checking the range of resolutions
// supported to see if the pair is within that.

			if ((getResRec.xRslRg.iMin <= desiredResX) &&
				(getResRec.xRslRg.iMax >= desiredResX) &&
				(getResRec.yRslRg.iMin <= desiredResY) &&
				(getResRec.yRslRg.iMax >= desiredResY))
				exactMatch = true;


// If we didn't find an exact match, check the driver's
// discrete resolutions to see if we have one there.

			for (rec = 0; (!exactMatch) && (rec < getResRec.iRslRecCnt); rec++)
				if ((getResRec.rgRslRec[rec].iXRsl == desiredResX) &&
					(getResRec.rgRslRec[rec].iYRsl == desiredResY))
					exactMatch = true;


// If we found an exact match use it.  Otherwise, loop through
// each resolution record and find the one which best matches
// our criteria.

			if (exactMatch)	
			{
				bestResX = desiredResX;
				bestResY = desiredResY;
			}
			else
				for (rec = 0; (rec < getResRec.iRslRecCnt); rec++)
				{
					xDPI = getResRec.rgRslRec[rec].iXRsl;
					yDPI = getResRec.rgRslRec[rec].iYRsl;
	
					if ((xDPI <= xDPI_ceiling) && (yDPI <= yDPI_ceiling) &&
						(!wantSquareDPI || (xDPI == yDPI)) &&
						((xDPI * yDPI) > (bestResX * bestResY)))
					{
						bestResX = xDPI;
						bestResY = yDPI;
					}
				}
		}
	}
		
// Return the best resolution pair we found and close the driver.
	
	*pxDPI = bestResX;
	*pyDPI = bestResY;
	PrClose();
}


/************************************************************
  SetPrinterPortDPI - This routine sets the resolution of
  the printer port to the passed horizontal and vertical
  resolutions.  hPrint is the current print handle.

  The printer driver is expected to be closed upon entry to
  this routine, and is therefore opened and closed around
  the PrGeneral code.  If the current driver doesn't support
  PrGeneral, or if an error occurs, the passed horizontal
  and vertical resolutions are changed to 72 dpi, which is
  what printer drivers on the Macintosh default to.

*************************************************************/

void SetPrinterPortDPI(THPrint hPrint, short *pxDPI, short *pyDPI)
{
	TSetRslBlk		setResRec;
	OSErr			err1, err2;

	PrOpen();

	if (!PrError())
	{
		setResRec.iOpCode = setRslOp;
		setResRec.hPrint = hPrint;
		setResRec.iXRsl = *pxDPI;
		setResRec.iYRsl = *pxDPI;
		PrGeneral((Ptr) &setResRec);
	
		err1 = ResError();
		err2 = setResRec.iError;
		
		if (err1 || err2)
			*pxDPI = *pyDPI = 72;
	}

	PrClose();
}


/************************************************************
  DoStyleDlog - This routine handles the style (Page Setup)
  dialog.

  The printer driver is expected to be closed upon entry,
  and is therefore opened and closed in this routine.

  This routine assures that when the handle is returned,
  it is at 72 dpi, regardless of what it was before.  For
  reasons mentioned below, this is important.

*************************************************************/

OSErr DoStyleDlog(THPrint hPrint)
{
	OSErr		err;
	Boolean		doIt;
	short		resetThedpi;

// Make sure we're working with a default resolution print record, and open
// the printer driver.  Since (in the final sample) we don't know what resolution
// we'll be setting the printer port to until after the dialogs, it's best to
// use a default dpi handle now so that options (which may be ignored when we
// print at high resolutions) are at least available in case they *can* be used.
//
// What makes things tricky is that different printer drivers support
// different ways of resetting the printer port resolution to 72 dpi.  You 
// always use PrGeneral, but the ImageWriter driver, for example, goes to "true"
// default resolution mode only when you try to set an invalid resolution such
// as 0x0 dpi, and the LaserWriter makes you pass 72x72 to get to 72 dpi.
// Neither respects the other's way of doing things.  (For an example, try setting
// the ImageWriter's print handle to 72x72 dpi with PrGeneral, and then doing a
// style dialog with that.)
// 
// One solution which I've found to work well is to first try to set the port to
// 72x72 dpi, and then try to set it to 0x0 dpi.  Drivers which reset when passed
// 72x72 ignore the 0x0 setting, and drivers that need to be passed 0x0, are happy
// as well.  Remember that the order is important here.  If you tell the IW driver
// that you want 72x72 dpi, it will disable options in the print dialog that
// shouldn't be.  Only when it tries to set the bogus 0x0 dpi will it clean house.
//
// Unfortunately, this is one of the design short-comings of the current
// printing architecture, or more specifically, the current printer drivers.
// If drivers don't disable options at different resolutions, we have no
// problem at all.

	resetThedpi = 72;				// Pass 72x72 dpi to force the defaults.
	SetPrinterPortDPI(hPrint, &resetThedpi, &resetThedpi);

	resetThedpi = 0;				// Pass a bogus pair to force the defaults.
	SetPrinterPortDPI(hPrint, &resetThedpi, &resetThedpi);
									// One of those should work.

// Handle the style dialog, close the driver and clean up any windows that
// need updating.

	PrOpen();

	err = PrError();
	
	if (err == fnfErr)	// Here, "file not found" really means "no printer chosen."
		err = noPrinterChosenErr;
	
	if (!err)
	{
		doIt = PrStlDialog(hPrint);
		err = PrError();
		if (!err && !doIt) err = iPrAbort;
	}

	PrClose();
	DoAnyUpdates();
	return err;
}


/************************************************************
  DoJobDlog - This routine handles the print job dialog.

  The printer driver is expected to be closed upon entry,
  and is therefore opened and closed in this routine.

  This routine assures that when the handle is returned,
  it is at 72 dpi, regardless of what it was before.  For
  reasons mentioned below, this is important.

*************************************************************/

OSErr DoJobDlog(THPrint hPrint)
{
	OSErr		err;
	Boolean		doIt;
	short		resetThedpi;

// Make sure we're working with a default resolution print record, and open
// the printer driver.  Since (in the final sample) we don't know what resolution
// we'll be setting the printer port to until after the dialogs, it's best to
// use a default dpi handle now so that options (which may be ignored when we
// print at high resolutions) are at least available in case they *can* be used.
//
// What makes things tricky is that different printer drivers support
// different ways of resetting the printer port resolution to 72 dpi.  You 
// always use PrGeneral, but the ImageWriter driver, for example, goes to "true"
// default resolution mode only when you try to set an invalid resolution such
// as 0x0 dpi, and the LaserWriter makes you pass 72x72 to get to 72 dpi.
// Neither respects the other's way of doing things.  (For an example, try setting
// the ImageWriter's print handle to 72x72 dpi with PrGeneral, and then doing a
// style dialog with that.)
// 
// One solution which I've found to work well is to first try to set the port to
// 72x72 dpi, and then try to set it to 0x0 dpi.  Drivers which reset when passed
// 72x72 ignore the 0x0 setting, and drivers that need to be passed 0x0, are happy
// as well.  Remember that the order is important here.  If you tell the IW driver
// that you want 72x72 dpi, it will disable options in the print dialog that
// shouldn't be.  Only when it tries to set the bogus 0x0 dpi will it clean house.
//
// Unfortunately, this is one of the design short-comings of the current
// printing architecture, or more specifically, the current printer drivers.
// If drivers don't disable options at different resolutions, we have no
// problem at all.

	resetThedpi = 72;				// Pass 72x72 dpi to force the defaults.
	SetPrinterPortDPI(hPrint, &resetThedpi, &resetThedpi);

	resetThedpi = 0;				// Pass a bogus pair to force the defaults.
	SetPrinterPortDPI(hPrint, &resetThedpi, &resetThedpi);
									// One of those should work.

// Handle the job dialog, close the driver and clean up any windows that
// need updating.

	PrOpen();

	err = PrError();
	
	if (err == fnfErr)	// Here, "file not found" really means "no printer chosen."
		err = noPrinterChosenErr;
	
	if (!err)
	{
		doIt = PrJobDialog(hPrint);
		err = PrError();
		if (!err && !doIt) err = iPrAbort;
	}

	PrClose();
	DoAnyUpdates();
	return err;
}


/************************************************************
  SetupPrintHdl - This routine returns a default print
  handle for the current printer driver.

  The printer driver is expected to be closed upon entry,
  and is therefore opened and closed in this routine.

*************************************************************/

OSErr SetupPrintHdl(THPrint *hPrint)
{
	OSErr err;
	
// Create a handle, open the printer driver, set the
// default values for the handle, and close the driver.
// Be on the lookout for errors.

	*hPrint = (THPrint) NewHandle(sizeof(TPrint));

	if (*hPrint != nil)
	{
		PrOpen();

		err = PrError();
		
		if (err == fnfErr)	// Here, "file not found" really means "no printer chosen."
			err = noPrinterChosenErr;
		
		if (err) return err;

		PrintDefault(*hPrint);
		err = PrError();

		PrClose();
	}
	else
		err = MemError();
	
	return err;
}


/************************************************************
  GetResIDString - This routine creates a unique resource
  ID by adding (the driver version * 10000) and the high
  byte of the wDev.  All values are derived from the print
  handle passed and the currently opened printer driver.
  Once we have the ID, it's converted to a string to use
  for the resource's name.  (Our ID might be 8 digits long,
  so we can't use it as a normal resource ID which is only 5
  digits long.)
  
  Because we call PrDrvrVers, this function must be called
  between PrOpen & PrClose.

*************************************************************/

void GetResIDString(THPrint hPrint, Str255 resIDStr)
{
	short		wDev;
	long		drvrVers, codeID;

	wDev = (*hPrint)->prStl.wDev >> 8;
	drvrVers = PrDrvrVers();
	codeID = (drvrVers * 10000) + wDev;
	NumToString(codeID, resIDStr);
}


/************************************************************
  SavePHdl - This routine saves the passed print handle in
  the indicated resource fork, using a name which identifies
  it uniquely for the printer driver and version being used.
  If there is already a print record resource with this name,
  it's deleted and replaced.

  The printer driver is expected to be closed upon entry,
  and is therefore opened and closed in this routine.

*************************************************************/

OSErr SavePHdl(THPrint hPrint, short resRefNum)
{
	short		savedRefNum, uniqID;
	OSErr		err;
	Handle		oldHdl;
	Str255		resIDStr;

// Save the current resource file refNum, open the printer
// driver, get our unique name for the passed print handle
// and close the driver.

	savedRefNum = CurResFile();
	PrOpen();
	err = PrError();
	
	if (err == fnfErr)	// Here, "file not found" really means "no printer chosen."
		err = noPrinterChosenErr;
	
	if (err) return err;

	if (!err) GetResIDString(hPrint, resIDStr);
	PrClose();
	

// If there's already a resource with this name in the passed
// resource file, delete it.

	UseResFile(resRefNum);
	
	if (!err)
	{
		oldHdl = GetNamedResource(r_printHdlType, resIDStr);

		if (oldHdl)
		{
			RmveResource(oldHdl);
			DisposHandle(oldHdl);
		}

// Save the new resource with a unique ID and our coded name.
	
		uniqID = UniqueID(r_printHdlType);
		AddResource((Handle) hPrint, r_printHdlType, uniqID, resIDStr);
		WriteResource((Handle) hPrint);
		err = ResError();


// Detach the resource, update the resource file and switch back
// to the original resource file.
	

		if (!err)
			DetachResource((Handle) hPrint);
	}
	
	UpdateResFile(resRefNum);
	UseResFile(savedRefNum);
	return err;
}


/************************************************************
  LoadPHdl - This routine loads a print handle resource
  which matches the name we uniquely encode.  If there is
  no print record resource in the indicated resource file
  which matches our uniquely encoded name, the function
  returns resNotFound.
  
  Upon entry, hPrint is expected to be a valid print handle
  or nil.

  The printer driver is expected to be closed upon entry,
  and is therefore opened and closed in this routine.

*************************************************************/

OSErr LoadPHdl(THPrint *hPrint, short resRefNum)
{
	short		savedRefNum;
	OSErr		err;
	Handle		newHdl;
	Str255		resIDStr;

// Save the current resource file refNum, open the printer driver,
// get the name that we would have encoded this resource with and
// close the driver.

	savedRefNum = CurResFile();
	PrOpen();

	err = PrError();
	
	if (err == fnfErr)	// Here, "file not found" really means "no printer chosen."
		err = noPrinterChosenErr;
	
	if (err) return err;

	GetResIDString(*hPrint, resIDStr);
	PrClose();


// Get our resource, if there is one, out of the indicated resource
// file.  If there isn't one, return resNotFound.  Finally, switch
// back to the original resource file.

	UseResFile(resRefNum);
	
	newHdl = GetNamedResource(r_printHdlType, resIDStr);

	if (newHdl)
	{

// Got one.  Detach it from the resource map.  If we previously had
// a print handle, dispose of that before passing this new one back.

		DetachResource(newHdl);
		if (*hPrint) DisposHandle(*(Handle *) hPrint);
		*hPrint = (THPrint) newHdl;
		err = noErr;
	}
	else
		err = resNotFound;
	
	UseResFile(savedRefNum);
	return err;
}


/************************************************************
  ColorBitsProc - This routine is our QuickDraw bottleneck
  replacement for stdBits.  We use this routine when we're
  trying to determine the resolution of a picture based
  solely on the resolution of its contents.

*************************************************************/

pascal void ColorBitsProc(BitMap *srcBits, Rect *srcRect, Rect *dstRect,
						  short mode, RgnHandle maskRgn)
{
    PixMapPtr  pm;

#pragma unused(srcRect, dstRect, mode, maskRgn);

// If this is a PixMap (rowBytes < 0), store the resolution
// and bounds of the PixMap in our global variables.  Since
// BitMaps don't have resolution fields, ignore those.  We
// keep expanding the bounds everytime we find a pixmap in
// case the image was banded (we want gSrcRect to encompass
// all bands).

	if (srcBits->rowBytes < 0)
	{
		pm = (PixMapPtr) srcBits;
		gPMHRes = MAX(gPMHRes, pm->hRes);
		gPMVRes = MAX(gPMVRes, pm->vRes);
		
		if (gRectNotSetYet)
		{
			gSrcRect = pm->bounds;
			gRectNotSetYet = false;
		}
		else
		{
			gSrcRect.top = MIN(gSrcRect.top, pm->bounds.top);
			gSrcRect.left = MIN(gSrcRect.left, pm->bounds.left);
			gSrcRect.bottom = MAX(gSrcRect.bottom, pm->bounds.bottom);
			gSrcRect.right = MAX(gSrcRect.right, pm->bounds.right);
		}
	}
}


/************************************************************
  GetPixMapRes - This routine returns the resolution of the
  contents of a picture (specifically the resolution of its
  PixMap).  We assume that there will be at most one PixMap
  in our image, and given the type of images we're dealing 
  with, that's a valid assumption.  It's not clear how we
  would determine the correct resolution in a situation
  where multiple PixMaps were involved anyway.
  
  This routine is used if the original picture was not
  created with OpenCPicture.  If it was, we can get the
  resolution by just checking the picture header, as in
  GetImageRes below.

*************************************************************/

OSErr GetPixMapRes(WindowInfoHdl windInfo, Fixed *hRes, Fixed *vRes, Rect *srcRect)
{
	OSErr		err = noErr;
	PicHandle	aPict;
	CQDProcs	bottlenecks;
	GrafPtr		oldPort;
	CGrafPort	cPort;
	Boolean		spoolThisPict;

// Assume the image is 72x72 dpi and that the picture's bounds
// are the same as its picFrame.

    spoolThisPict = !(*windInfo)->loaded;
	aPict = (*windInfo)->windowPict;
	gPMHRes = gPMVRes = Long2Fix(72);
	gSrcRect = (*aPict)->picFrame;
	gRectNotSetYet = true;


// Save off the current port, and open up a new color
// port to work in.  We do this because we *must* be drawing
// into a CGrafPort for us to have PixMaps passed to our
// bottleneck routine without being converted into BitMaps.

	GetPort(&oldPort);
	OpenCPort(&cPort);


// Get the standard CGrafPort bottleneck routines, and add
// our replacement for stdBits.  This replacement routine
// will snoop for a PixMap, and store the resolution and
// bounds of any it finds in our global variables.  If
// none are found, we'll still have the defaults set up
// above.

	if (spoolThisPict)
		err = SpoolInstall(windInfo, kArrowCursor);
	else
	{
		SetStdCProcs(&bottlenecks);
		cPort.grafProcs = &bottlenecks;
	}

	if (!err)
	{
		cPort.grafProcs->bitsProc = (Ptr) ColorBitsProc;


// Draw the picture so our bottleneck routine is called.
// Afterwards, set back to the old port we were using and
// Close the CGrafPort we created.  Finally, return the
// resolution and bounds rectangle for this image.

		DrawPicture(aPict, &(*aPict)->picFrame);

		if (spoolThisPict)
			err = SpoolRemove();
	}

	SetPort(oldPort);
	CloseCPort(&cPort);

	*hRes = gPMHRes;
	*vRes = gPMVRes;
	*srcRect = gSrcRect;

	return err;
}


/************************************************************
  GetImageRes - This routine returns the native resolution
  of a picture, and the appropriate drawing rectangle for
  the image at that resolution.

*************************************************************/

OSErr GetImageRes(WindowInfoHdl windInfo, short *ixDPI, short *iyDPI, Rect *srcRect)
{
	PicHandle			wPICT;
	Fixed				hRes, vRes;
	PartialPicture		partialPict;
	Boolean				extndV2PICT = false;
	OSErr				err = noErr;

	wPICT = (*windInfo)->windowPict;
	HLock((Handle) windInfo);
	BlockMove(&(*windInfo)->partialPict, &partialPict, sizeof(PartialPicture));
	HUnlock((Handle) windInfo);

// If we have a version 2 picture, find out if it's an extended
// version 2 picture.  If so, the version field in the picture's
// header will be -2.  In that case, pull the resolution and bounding
// rect out of the picture header.

	if (partialPict.pictVersion == 2)
		if (partialPict.pictHeader.version == -2)
		{
			extndV2PICT = true;

			hRes = partialPict.pictHeader.hRes;
			vRes = partialPict.pictHeader.vRes;
			*srcRect = partialPict.pictHeader.srcRect;
		}

// If we have a version 1 picture, or a non-extended version 2
// picture, set the resolution and bounding rect for the
// picture based on the PixMap it contains.

	if (!extndV2PICT)
		err = GetPixMapRes(windInfo, &hRes, &vRes, srcRect);

	*ixDPI = FixRound(hRes);
	*iyDPI = FixRound(vRes);
	
	return err;
}


/************************************************************
  SetUpQDSearch - This routine performs initialization
  necessary for the OQDSearch routine.  In particular, it
  loads a color table containing the original 8 QuickDraw
  colors.
  
  (The system Color Table with ID = 127 contains the 8
  original QD colors, as described in IM-V, pp.69 & 81.)

*************************************************************/

void SetUpQDSearch()
{	
	gOrgQDCTab = GetCTable(127);
}


/************************************************************
  OQDSearch - This routine is a color search proc, as
  described on pages 145-147 of Inside Mac V.
  
  The routine compares the passed color to the current entry
  of the "Original 8" color table.  If the color matches the
  color we're currently looking for, we return 1.  Otherwise,
  we return 0.  Because the routine which uses this search
  proc uses a 1-bit deep destination, a 0 signifies the
  background color (white in our case), and 1 signifies the
  foreground color.  This routine is used when we create the
  color separations in the PrintOne routine.

  We always return true to indicate that we've handled the
  color processing.

*************************************************************/

pascal Boolean OQDSearch(RGBColor *anRGB, long *position)
{

// Initially, assume we'll return white.  If the color passed
// in matches the current color we're separating, return the
// position for black.

	*position = 0;

	if ((anRGB->red == (*gOrgQDCTab)->ctTable[gCurColor].rgb.red) &&
		(anRGB->green == (*gOrgQDCTab)->ctTable[gCurColor].rgb.green) &&
		(anRGB->blue == (*gOrgQDCTab)->ctTable[gCurColor].rgb.blue))
		*position = 1;

	return true;
}


/************************************************************
  CenterRect - This routine centers srcRect within destRect
  and returns it in ctrRect.  It's mainly used for centering
  output on the printed page and for centering windows on
  the screen.  The source rectangle should not be larger
  than the destination rectangle in either direction.  If
  it is, the rectangle will not be adjusted in that
  direction.

*************************************************************/

void CenterRect(Rect *srcRect, Rect *destRect, Rect *ctrRect)
{
	Rect	srcR, dstR;
	short	srcWidth, srcHeight, destWidth, destHeight, dx, dy;

	srcR = *srcRect;
	dstR = *destRect;

	OffsetRect(&srcR, dstR.left - srcR.left, dstR.top - srcR.top);

	srcWidth = srcR.right - srcR.left +1;
	srcHeight = srcR.bottom - srcR.top +1;
	destWidth = dstR.right - dstR.left +1;
	destHeight = dstR.bottom - dstR.top +1;
	
	dx = MAX(0, (destWidth-srcWidth)/2);
	dy = MAX(0, (destHeight-srcHeight)/2);

	*ctrRect = srcR;
	OffsetRect(ctrRect, dx, dy);
}


/************************************************************
  IdleProc - This routine is a PrIdleProcPtr that spins our
  cursor and looks for command-period presses during
  printing.

*************************************************************/

pascal void IdleProc()
{
	OSErr		err;
	
	BumpCursor();
	if (err = CommandPeriod()) PrSetError(err);
}


/************************************************************
  CommandPeriod - This routine checks to see if the user has
  pressed command-period.  The Printing Manager does this
  for us, but a routine like this is helpful to allow users
  to cancel in the middle of a rendering routine, rather
  than only Printing Manager operations.
  
  If there's a command-period in the event queue, the
  routine returns iPrAbort just like the Printing Manager.
  Otherwise, it returns noErr.

*************************************************************/

OSErr CommandPeriod()
{
	Boolean		cancel = false;
	EventRecord	evtRec;
	
	while (!cancel && (WaitNextEvent(keyDownMask | autoKeyMask, &evtRec, 0, nil)))
		cancel = (evtRec.modifiers & cmdKey) &&
				 ((evtRec.message & charCodeMask) == '.');

	return ((cancel)? iPrAbort: noErr);
}


/************************************************************
  BandPicture - This routine bands the passed picture into
  the destination GWorld, using ditherCopy if desired.
  
  wp is the window containing the picture.  It has one of
  our WindowInfoHdl handles hanging off the window refCon
  field.  From that structure, we can either locate the
  picture's handle in memory or spool the picture in from
  disk.

*************************************************************/

OSErr BandPicture(WindowPtr	wp, Rect *srcRect,
				  GWorldPtr destGWorld, Boolean useDitherCopy)
{
	OSErr			err = noErr;
	Rect			pictFrame, bandFrame, ditherFrame, ditherRect1, ditherRect2;
	GWorldPtr		curGWorld, bandGWorld = nil, ditherGWorld = nil;
	PixMapHandle	srcPix, destPix, ditherPix;
	PicHandle		srcPict;
	Boolean			moreToDo, spoolThisPict, useSecondGWorld;
	GDHandle		curGDH;
	WindowInfoHdl	windInfo;
	long			bandHeight, bandWidthInBytes, maxBlockSize;
	short			bitsPerPixel, rowWidth, padAmt;

	windInfo = (WindowInfoHdl) GetWRefCon(wp);
	srcPict = (*windInfo)->windowPict;
	spoolThisPict = !(*windInfo)->loaded;
	useSecondGWorld = useDitherCopy;


// Save the current GWorld for restoration later, then lock down the destination
// GWorld's bits and erase them so that no garbage is left in the GWorld.  Lastly,
// get the picture's bounds, putting the top-left corner at (0, 0).

	GetGWorld(&curGWorld, &curGDH);

	destPix = GetGWPixMap(destGWorld);
	LockPixels(destPix);
	
	SetGWorld(destGWorld, nil);
	EraseRect(&(*destPix)->bounds);
	SetGWorld(curGWorld, curGDH);

	pictFrame = *srcRect;
	OffsetRect(&pictFrame, -pictFrame.left, -pictFrame.top);

// Determine the size bands to use.  We first get the maximum contiguous block 
// possible in our heap.  Taking 85% of that value, we figure out how many
// horizontal bands could fit in that.
//
// If we're dithering, we'll need an extra GWorld for dither-distortion correction.
// The process we'll use will require a "pad" of a few extra rows of pixels in
// each GWorld (padAmt).  Therefore, our calculated band size will be:
//
// for srcCopy:    (#of bytes available / number of bytes per row) -padAmt
//
// Remember that the number of bytes per row will be rounded up to the nearest
// long word, since 32-bit QuickDraw makes GWorlds which are 32-bit aligned.
//
// For ditherCopy we do the same calculations as above, but we adjust bitsPerPixel to
// reflect the sum of the pixels in the 32 bit GWorld and those in the additional dither
// GWorld.
//
// If our calculations come up with a band size that's bigger than the original image,
// then just use the height of the original image.

	bandFrame = (*destPix)->bounds;

	padAmt = (useDitherCopy)? kPadAmount: 0;
	maxBlockSize = MaxBlock() * .85;
	bitsPerPixel = (useDitherCopy)? 32 + (*destPix)->pixelSize: 32;
	rowWidth = (bandFrame.right - bandFrame.left +1);
	bandWidthInBytes = (bitsPerPixel * rowWidth)/8;
	bandWidthInBytes += bandWidthInBytes % 4;		// Make sure we're on a LW boundary.

	bandHeight = (maxBlockSize / bandWidthInBytes) -padAmt;
	
	if ((bandHeight +padAmt) >= (bandFrame.bottom - bandFrame.top))
	{
		useSecondGWorld = false;
		padAmt = 0;
		bandHeight = (bandFrame.bottom - bandFrame.top);
	}

// Create the rectangle for the banding GWorld.  The band buffer will be as wide as
// the original image and as tall as either the available band size or the original
// image, whichever is smaller.

	SetRect(&bandFrame, 0, -padAmt, bandFrame.right, bandHeight +padAmt);


// If we're dithering, and using the second GWorld, create the dither GWorld which
// we use to filter our some of the effects of ditherer banding.  This GWorld will
// mirror the destination, except for the bounds we use.

	if (useSecondGWorld)
	{
		ditherFrame = bandFrame;
		err = NewGWorld(&ditherGWorld, (*destPix)->pixelSize, &ditherFrame, (*destPix)->pmTable,
						GetGWorldDevice(destGWorld), 0);
	}


// While there's more to do, band our image.  On every pass we create a 32-bit GWorld
// to band the image through, which we dispose of at the end of the loop.  (UpdateGWorld
// is not used because its behaviour is sometimes questionable when we're only changing
// the bounds of a GWorld.)  After locking down the pixels, we set this as the
// current GWorld and draw the picture into it, clipping to the GWorld's bounds.  This
// creates a band which we then CopyBits to the destination GWorld, or pass through
// ditherGWorld if we're dithering and banding.
//
// Of particular importance are the rectangles we're using:
//
// When we cut the bands out of the picture, we call DrawPicture using pictFrame, and
// clipping to bandFrame.  If we used bandFrame in DrawPicture, every band would be a
// squished up representation of the original image, which obviously isn't what we want.
// bandFrame is updated on each pass so that it drops one band lower, and the DrawPicture
// call copies the next logical band to the 32-bit GWorld.
//
// When dithering and banding, the padAmt is non-zero, and useSecondGWorld is true.  In
// this case, we draw the picture into our 32-bit GWorld and then dither from there to
// our ditherGWorld.  Finally, we srcCopy that image to the destination GWorld, using
// a rectangle that leaves out the top padAmt rows of the dithered image.  By doing this,
// we negate some of the effects that dithering has on the fringes of our bands.  We
// still end up copying all the rows, but we don't copy the pixels that are dithered
// on the top fringe of ditherGWorld between consecutive bands.
//
// We also check for command-period presses while we do this, in case the user gets
// bored and wants to cancel.

	moreToDo = true;
	
	if (!err)
	do
	{
		err = NewGWorld(&bandGWorld, 32, &bandFrame, nil, nil, 0);
		if (!err)
		{
			srcPix = GetGWPixMap(bandGWorld);
			LockPixels(srcPix);

			SetGWorld(bandGWorld, nil);
			ClipRect(&bandFrame);


// If we need to spool this image, install the spooling bottleneck procedure.
 
 			if (spoolThisPict)
				err = SpoolInstall(windInfo, r_stdCurs4);

			if (!err)
			{
				DrawPicture(srcPict, &pictFrame);


// If we spooled this image, remove the spooling bottleneck procedure.
 
				if (spoolThisPict)
					err = SpoolRemove();

				SetGWorld(destGWorld, nil);
				
				if (useSecondGWorld)
				{
					ditherPix = GetGWPixMap(ditherGWorld);
					LockPixels(ditherPix);

					CopyBits((BitMap *) *srcPix, (BitMap *) *ditherPix, &bandFrame,
							 &ditherFrame, ditherCopy, nil);

					ditherRect1 = ditherFrame;	// Offset the rectangles so we don't
					ditherRect1.top += padAmt;	// copy the topmost dithered pixels.
					ditherRect2 = bandFrame;
					ditherRect2.top += padAmt;
	
					CopyBits((BitMap *) *ditherPix, (BitMap *) *destPix, &ditherRect1,
							 &ditherRect2, srcCopy, nil);

					UnlockPixels(ditherPix);
				}
				else
					CopyBits((BitMap *) *srcPix, (BitMap *) *destPix, &bandFrame,
							 &bandFrame, ((useDitherCopy)? ditherCopy: srcCopy), nil);
				
				UnlockPixels(srcPix);
			
				if (useSecondGWorld)
					moreToDo =  (ditherRect2.bottom < pictFrame.bottom);
				else
					moreToDo =  (bandFrame.bottom < pictFrame.bottom);
				
				OffsetRect(&bandFrame, 0, padAmt + MIN(bandHeight, pictFrame.bottom - bandFrame.bottom));
				DisposeGWorld(bandGWorld);
				bandGWorld = nil;
	
				SetGWorld(curGWorld, curGDH);
				if (!err) err = CommandPeriod();
			}
		}
	}
	while (moreToDo && (!err));


// Unlock the destination bits, set the current GWorld to what we started with, and
// dispose of the GWorlds.  Return any errors.

	UnlockPixels(destPix);

	SetGWorld(curGWorld, curGDH);
	if (bandGWorld) DisposeGWorld(bandGWorld);
	if (ditherGWorld) DisposeGWorld(ditherGWorld);

	return err;
}
