/*********************************************************************

	HT Event Stuff.c
	
	This file contains functions for event handling in the
	"Adventures in Color Printing" halftone sample,
	"Halftone Adventures."
	
	Dave Hersey, Apple Developer Technical Support.
	
	 Edit Trail 
	
	creation/reorg/rebirth:						 6/28/92  - dmh
	sync'd with other samples:					11/15/92  - dmh
	final clean up and sync with other samples:	11/28/92  - dmh

*********************************************************************/

#pragma segment Main

#include "HT Headers.h"

static short			gPrevChosenDriverVers;	// Printer driver version when we
												// suspended.  (Used to see if the
												// current driver changed on resume.)

static unsigned char	gPrevChosenPrinterwDev;	// Printer driver wDev when we
												// suspended.  (Used to see if the
												// current driver changed on resume.)

/************************************************************
  EventLoop - This is the app's event loop, which keeps
  everything chugging along.

*************************************************************/

void EventLoop()
{
	Boolean		gotEvent;
	EventRecord	event;
	long		sleepTime;
	
// While not quitting, repeatedly get and dispatch events.  Sleep
// an appropriate amount whether we're running in the foreground
// or background.  Every time through we unload all our segments.
// They won't actually be purged unless we need the memory.

	while (!gQuitting)
	{
		sleepTime = GetDblTime();
		if (gInBackground) sleepTime = -1L;

		gotEvent = WaitNextEvent(everyEvent, &event, sleepTime, gCursorRgn);
		
		if (gotEvent)
		{
			UnloadSeg(DoMenuCommand);
			UnloadSeg(GetOpenName);
			UnloadSeg(PrintOne);

			DoEvent(&event);
		}
	}
}


/************************************************************
  DoEvent - This routine handles whatever events come our
  way.

*************************************************************/

void DoEvent(EventRecord *event)
{
	OSErr		err;
	short		windowPart;
	WindowPtr	window;
	char		key;
	Point		mountPoint;
	Rect		dragRect;
	RgnHandle	grayRgn;

// Set up a rectangle to use for dragging windows around.  Then,
// handle any events.

	grayRgn = GetGrayRgn();
	dragRect = (*grayRgn)->rgnBBox;
	
	switch (event->what)
	{
		case mouseDown:
			windowPart = FindWindow(event->where, &window);
			switch (windowPart)
			{
				case inMenuBar:
					AdjustMenus();
					DoMenuCommand(MenuSelect(event->where));
					break;
				
				case inContent:
					if (window != FrontWindow())
						SelectWindow(window);
					break;
					
				case inDrag:
					DragWindow(window, event->where, &dragRect);
					break;
					
				case inGoAway:
					if (TrackGoAway(window, event->where))
						DoCloseWindow(window);
					break;
			}
			break;

		case keyDown:
		case autoKey:
			key = event->message & charCodeMask;
			if (event->modifiers & cmdKey)
				if (event->what == keyDown)
				{
					AdjustMenus();
					DoMenuCommand(MenuKey(key));
				}
			break;

		case updateEvt:
			DoUpdate((WindowPtr) event->message);
			break;
		
		case diskEvt:
			if ((event->message >> 16) != noErr)
			{
				mountPoint.h = kDILeft;
				mountPoint.v = kDITop;
				err = DIBadMount(mountPoint, event->message);
				AlertUser(err);
			}
			break;
			
		case osEvt:
			switch ((event->message >> 24) & 0x0ff)
			{
				case suspendResumeMessage:
					
					SetCursor(&qd.arrow);
				
					if ((event->message & resumeFlag) == 0)	// suspend 
					{
// When suspending, save the current printer driver's wDev ID and version.

						gInBackground = true;
						gPrevChosenPrinterwDev = (*ghPrint)->prStl.wDev >>8;

						PrOpen();

						if (!(err = PrError()))
							gPrevChosenDriverVers = PrDrvrVers();
						else
							gPrevChosenDriverVers = 0;

						PrClose();
						
						ZeroScrap();
						TEToScrap();
					}
					else
					{										// resume
						unsigned char	curChosenPrinterwDev;
						short			curDriverVers = 0;
						
// When resuming, comapre the current printer driver's wDev ID and version against
// the values we stored when we suspended.  If different, dump the current print
// handle and try to load a previously saved one for the new driver.

						gInBackground = false;
						curChosenPrinterwDev = (*ghPrint)->prStl.wDev >>8;

						PrOpen();

						if (!(err = PrError()))
							curChosenPrinterwDev = PrDrvrVers();

						PrClose();
						
						if ((curChosenPrinterwDev != gPrevChosenPrinterwDev) ||
							(curDriverVers != gPrevChosenDriverVers))
						{
							if (ghPrint)
								DisposeHandle((Handle) ghPrint);

							err = SetupPrintHdl(&ghPrint);
							AlertUser(err);

							if (!err)
							{
								err = LoadPHdl(&ghPrint, gAppResRefNum);
								if (err != resNotFound) AlertUser(err);
							}
						}
						
						if (event->message & convertClipboardFlag)
							TEFromScrap();
					}
					break;
					
				case mouseMovedMessage:
					DisposeRgn(gCursorRgn);
					gCursorRgn = NewRgn();
					SetRectRgn(gCursorRgn, -32768, -32768, 32766, 32766);
					break;
			}
			break;
				
		case kHighLevelEvent:
			AEProcessAppleEvent(event);
			break;
	}
}


/************************************************************
  DoAEInstallation - This routine installs our core
  AppleEvent handlers.

*************************************************************/

void DoAEInstallation()
{
	AEInstallEventHandler(kCoreEventClass,
						  kAEOpenApplication,
						  (EventHandlerProcPtr) HandleOAPP,
						  0, false);

	AEInstallEventHandler(kCoreEventClass,
						  kAEQuitApplication,
						  (EventHandlerProcPtr) HandleQUIT,
						  0, false);

	AEInstallEventHandler(kCoreEventClass,
						  kAEOpenDocuments,
						  (EventHandlerProcPtr) HandleODOC,
						  0, false);

	AEInstallEventHandler(kCoreEventClass,
						  kAEPrintDocuments,
						  (EventHandlerProcPtr) HandlePDOC,
						  0, false);

}


/************************************************************
  HandleOAPP - This is the "open application" AppleEvent
  handler.  We don't do anything more than check out the
  message.

*************************************************************/

pascal OSErr HandleOAPP(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon)
{
	OSErr		err;

#pragma unused (reply, myRefCon)
	
	err = CheckAEParams(theAppleEvent);
	return err;
}


/************************************************************
  HandleQUIT - This is the "quit" AppleEvent handler.  Set
  the gQuitting and the event loop will do the rest.

*************************************************************/

pascal OSErr HandleQUIT(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon)
{
	OSErr	err;

#pragma unused (reply, myRefCon)

	err = CheckAEParams(theAppleEvent);
	if (err) return err;
	
	gQuitting = true;
	return noErr;
}


/************************************************************
  HandleODOC - This is the "open document" AppleEvent
  handler.

*************************************************************/

pascal OSErr HandleODOC(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon)
{
	OSErr		err;
	AEDescList	docList;
	FSSpec		myFSS;
	long		itemsInList;
	AEKeyword	theKeyword;
	DescType	typeCode;
	Size		actualSize;
	long		i;

#pragma unused (reply, myRefCon)

// Get our doc list and count the number of entries in it.

	err = AEGetParamDesc(theAppleEvent, keyDirectObject, typeAEList, &docList);
	if (err) return err;
	
	err = CheckAEParams(theAppleEvent);
	if (err) return err;
	
	err = AECountItems(&docList, &itemsInList);
	if (err) return err;


// For each entry in the document list, open up a new window and
// load the image.

	for (i = 1; i<= itemsInList; i++)
	{
		err = AEGetNthPtr(&docList, i, typeFSS, &theKeyword, &typeCode,
						  (Ptr) &myFSS, sizeof(FSSpec), &actualSize);

		if (err) return err;

		err = CreatePictWindow(&myFSS, (SFReply *) nil);
		AlertUser(err);
	}

// If we made it this far, either there have been no errors or we've
// already handled them-- return noErr.

	return noErr;
}


/************************************************************
  HandlePDOC - This is the "print document" AppleEvent
  handler.

*************************************************************/

pascal OSErr HandlePDOC(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon)
{
	OSErr		err;
	AEDescList	docList;
	FSSpec		myFSS;
	long		itemsInList;
	AEKeyword	theKeyword;
	DescType	typeCode;
	Size		actualSize;
	long		i;
	WindowPtr	wp;

#pragma unused (reply, myRefCon)

// Get our doc list and count the number of entries in it.

	err = AEGetParamDesc(theAppleEvent, keyDirectObject, typeAEList, &docList);
	if (err) return err;
	
	err = CheckAEParams(theAppleEvent);
	if (err) return err;
	
	err = AECountItems(&docList, &itemsInList);
	if (err) return err;


// For each entry in the document list, open up a new window, load
// the image, print it and close its window.  Do updates of any
// windows that might be open (and wonked on) when this window is
// closed.

	for (i = 1; i<= itemsInList; i++)
	{
		err = AEGetNthPtr(&docList, i, typeFSS, &theKeyword, &typeCode, (Ptr) &myFSS, sizeof(FSSpec), &actualSize);
		if (err) return err;

		err = CreatePictWindow(&myFSS, (SFReply *) nil);
		AlertUser(err);
	
		if (!err)
		{
			wp = FrontWindow();
			err = PrintOne(wp);
			AlertUser(err);
			DoCloseWindow(wp);
			DoAnyUpdates();
		}
	}

// Return noErr, since all errors will have been handled by
// this point.

	return noErr;
}


/************************************************************
  CheckAEParams - This routine is used to make sure we've
  received all the parameters for an AppleEvent.  If so, we
  return noErr, otherwise we indicate that the event wasn't
  handled completely.
 
 *************************************************************/

OSErr CheckAEParams(AppleEvent *theAppleEvent)
{
	OSErr		err;
	DescType	typeCode;
	Size		actualSize;
	
	err = AEGetAttributePtr(theAppleEvent, keyMissedKeywordAttr,
							typeWildCard, &typeCode, nil, 0, &actualSize);

	if (err == errAEDescNotFound) return noErr;
	if (err == noErr) return errAEEventNotHandled;
	
	return err;
}
