/*********************************************************************

	Final Color IW Printing.c
	
	This file contains all of the color ImageWriter-specific printing
	functions for the all-in-one "Adventures in Color Printing"
	sample, "The Final Adventure."
	
	Dave Hersey, Apple Developer Technical Support.
	
	 Edit Trail 
	
	creation:									11/24/92  - dmh
	sync'd with other samples:					11/30/92  - dmh
	
*********************************************************************/

#pragma segment Printing

#include "Final Headers.h"

static CTabHandle	gOrgQDCTab = nil;	// Storage for original QD CTable.
static char			gCurColor;			// Index to above for current
										// color we're searching for.

/************************************************************
  DoColorIWPrint - This routine starts the printing process
  for the indicated window.  We take care of pre-printing
  rendering (in this case making color separations of the
  image) as well as setting resolutions, the document title,
  displaying the job dialog and so forth.  If everything
  succeeds, we pass control on to the DoColorIWPrintLoop
  routine.
  
  This function is a minimally-modified version of the
  "Color IW Adventures" sample's PrintOne routine.

*************************************************************/

OSErr DoColorIWPrint(WindowPtr whichWindow, Boolean useHighRes)
{
	QDErr				err;
	WindowInfoHdl		windInfo;
	GDHandle			savedGDH;
	GWorldPtr			savedGW, OQDGWorld = nil, colorSep = nil;
	PixMapHandle		srcPix, destPix;
	Boolean				useDitherCopy, printAtHighRes;
	DialogPtr			printStatDlog = nil;
	Str255				tempStr, orgTitle;
	OpenCPicParams		cPicParams;
	PicHandle			sepsPICT = nil;
	Rect				pictFrame, rPage, pageFrame;
	Fixed				scale;
	short				pxDPI, pyDPI, ixDPI, iyDPI, numChar,
						QDColor[7] = {blackColor, yellowColor, magentaColor,
									  redColor, cyanColor, greenColor, blueColor};

// Get this window's picture and title.

	windInfo = (WindowInfoHdl) GetWRefCon(whichWindow);
	GetWTitle(whichWindow, orgTitle);


// Save the current GWorld and GDevice.  We'll always dither, and if
// we were asked to, we'll render and print at high resolution.

	GetGWorld(&savedGW, &savedGDH);

	useDitherCopy = true;
	printAtHighRes = useHighRes;


// Get the image's native resolution and bounds.

	err = GetImageRes(windInfo, &ixDPI, &iyDPI, &pictFrame);
	if (err) return err;


// If we're printing at high resolution, get the highest resolution
// possible at or below 300 x 300 dpi.  We also request "square"
// resolutions (where the horizontal and vertical DPIs are
// equal).  If we're not using high resolution printing, specify
// the default (72 x 72 dpi) resolution.  Finally, actually set
// the printer port's resolution.

	if (printAtHighRes)
	{
		pxDPI = pyDPI = 300;
		GetBestDPI(&pxDPI, &pyDPI, 300, 300, true);
	}
	else
		pxDPI = pyDPI = 72;

	SetPrinterPortDPI(ghPrint, &pxDPI, &pyDPI);


// For convenience in calculations, align the top-left of the
// picture's bounds with (0,0).  If the printer driver's
// resolution is not the same as the image's resolution, scale
// the image's picFrame appropriately so that our printed output
// has the same dimensions as the original.

	OffsetRect(&pictFrame, -pictFrame.left, -pictFrame.top);

	if ((pxDPI != ixDPI) || (pyDPI != iyDPI))
	{
		scale = FixRatio(pxDPI, ixDPI);
		pictFrame.right = FixMul(scale, (long) pictFrame.right <<16) >>16;
		scale = FixRatio(pyDPI, iyDPI);
		pictFrame.bottom = FixMul(scale, (long) pictFrame.bottom <<16) >>16;
	}


// Tell the user what we're doing, check for command-period, and set
// things up for our search proc.

	SetCursor(*GetCursor(watchCursor));

	numChar = sprintf(&tempStr[0], " %s %s.",
					  "to original QuickDraw colors using",
					  (useDitherCopy)? "ditherCopy": "srcCopy");
	
	tempStr[numChar] = '\0';
	
	ShowStatusMsg(&printStatDlog, "Mapping ", orgTitle, tempStr, true);
	err = CommandPeriod();
	SetUpQDSearch();
	

// Create a 4-bit GWorld which uses the original 8 QuickDraw colors.  If
// there are no errors, Band the picture, using ditherCopy if desired.
// The result is a representation of the image in the original 8 colors.

	rPage = (*ghPrint)->prInfo.rPage;
	SectRect(&pictFrame, &rPage, &pageFrame);

	err = NewGWorld(&OQDGWorld, 4, &pageFrame, gOrgQDCTab, nil, 0);
	
	if (!err)
		err = BandPicture(whichWindow, &pictFrame, OQDGWorld, useDitherCopy);


// Get the PixMap for the original QuickDraw representation and lock
// its pixels.  Create a new 1-bit GWorld to draw the separations into.

	if (!err)
	{
		err = NewGWorld(&colorSep,1,&pageFrame, nil,nil,0);


// Start up our spinning cursors.  We set the final cursor to r_stdCurs4 so that
// when the printing loop starts spinning its cursors, they mesh with ours.
// Otherwise, you'd see the cursor flash to a watch or whatever in between.  Set
// the current GWorld to the 1-bit GWorld, and create a picture.  Note that this
// means that the picture is tied to the 1-bit GWorld.  Only when that GWorld is
// current will data be recorded into the picture.

			if (!err)
			{
				SetUpCursors(r_stdSpinCursor, r_stdCurs4);

				SetGWorld(colorSep, nil);
				srcPix = GetGWPixMap(OQDGWorld);
				LockPixels(srcPix);
				destPix = GetGWPixMap(colorSep);
				LockPixels(destPix);
	
				ClipRect(&pageFrame);
	
				cPicParams.srcRect = pageFrame;
				cPicParams.hRes = Long2Fix((long) pxDPI);
				cPicParams.vRes = Long2Fix((long) pyDPI);
				cPicParams.version = -2;
	
				sepsPICT = OpenCPicture(&cPicParams);

// Go through each of the 8 original colors (except white) and create a
// separation.  To do this, we set the current GWorld to whatever it was
// when we came into this routine.  Since this will not be the 1-bit
// GWorld, this turns off our picture's recording.  Next, we tell the
// user what we're doing, install our searchProc, and CopyBits from
// the 8-bit GWorld to the 1-bit one.  As mentioned in the comments
// for OQDSearch, this gives us black bits only where the color in the
// original matches the current separation color.  With the separation
// complete, we delete the search proc and set our current GWorld
// back to the 1-bit one.  This re-enables recording into our picture,
// and we record the foreground color for the current separation,
// followed by the bitmap using srcOr mode.  After all 7 passes have
// been completed, this will give us a picture with 7 separations in it,
// all overlaying each other to make the "original QuickDraw"
// represenation.  We use srcOr mode so that the white is transparent,
// otherwise the white for each layer would overwrite the color from
// the previous layer.

				for (gCurColor = 0; (gCurColor < 7) && !err; gCurColor++)
				{
					BumpCursor();					// Spin.
					SetGWorld(savedGW, savedGDH);
					sprintf(tempStr, "%s%i %s ", "Creating separation #", gCurColor +1, "of");
					ShowStatusMsg(&printStatDlog, tempStr, orgTitle, ".", true);
	
					err = CommandPeriod();
	
					BumpCursor();					// Spin.
					AddSearch(OQDSearch);
					CopyBits((BitMap *) *srcPix, (BitMap *) *destPix, &pageFrame, &pageFrame, srcCopy, nil);
					DelSearch(OQDSearch);
					SetGWorld(colorSep, nil);
		
					BumpCursor();					// Spin.
					ForeColor(QDColor[gCurColor]);
					CopyBits((BitMap *) *destPix, (BitMap *) *destPix, &pageFrame, &pageFrame, srcOr, nil);
				}


// Close the picture, restore our saved GWorld/GDevice, stop the cursor from
// spinning, and dispose of our GWorlds.  If the picture's picFrame is empty,
// we had a memory error.

				ClosePicture();
				SetGWorld(savedGW, savedGDH);
				UnlockPixels(destPix);
				UnlockPixels(srcPix);
	
				TearDownCursors();
			}

		if (!err && EmptyRect(&(*sepsPICT)->picFrame))
			err = memFullErr;

		if (colorSep)
		{
			DisposeGWorld(colorSep);
			colorSep = nil;
		}
		
		if (OQDGWorld)
		{
			DisposeGWorld(OQDGWorld);
			OQDGWorld = nil;
		}

// If no errors, tell the user we're about to start printing and
// then go do it.  When finished, hide our status dialog, throw
// away any data we have left over, restore the saved GWorld &
// GDevice and return any error that's occurred.

		if (!err)
		{
			sprintf(tempStr, "%s %i %s %i %s",  " at", pxDPI,
					"dpi horizontally by", pyDPI, "dpi vertically.");

			ShowStatusMsg(&printStatDlog, "Printing ", orgTitle, tempStr, true);
			err = DoColorIWPrintLoop(ghPrint, &sepsPICT, &pageFrame);
		}
	}

	HideStatusMsg(&printStatDlog);

	if (gOrgQDCTab) DisposCTable(gOrgQDCTab);
	gOrgQDCTab = nil;

	if (OQDGWorld) DisposeGWorld(OQDGWorld);
	if (colorSep) DisposeGWorld(colorSep);
	if (sepsPICT) KillPicture(sepsPICT);

	SetGWorld(savedGW, savedGDH);
	SetCursor(&qd.arrow);

	return err;
}


/************************************************************
  DoColorIWPrintLoop - This routine does the actual
  printing.  The picture to be printed is passed, fully
  rendered for output.
  
  This function is a minimally-modified version of the
  "Color IW Adventures" sample's DoPrintLoop
  routine.

*************************************************************/

OSErr DoColorIWPrintLoop(THPrint hPrint, PicHandle *pict, Rect *pictFrame)
{
	OSErr			err;
	TPPrPort		printPort;
	TPrStatus		theStatus;
	short			copy, iFstPage, iLstPage;
	Rect			rPage, pictBounds;

// Set the cursor spinning (we'll keep it spinning in our idle proc).  Open the
// printer driver and validate the print record so that the Printing Manager
// pulls our document title from the title of the frontmost window.  The reason
// we save and restore the iFstPage and iLstPage fields is that the LaserWriter
// driver (through at least v7.1.2) has a bug in PrValidate which causes these
// values to be reset.  While we aren't using iFstPage or iLstPage below, you
// never know where this code might end up in this crazy cut & paste world.

	SetUpCursors(r_stdSpinCursor, kArrowCursor);

	PrOpen();

	if (!(err = PrError()))
	{
		iFstPage = (*hPrint)->prJob.iFstPage;
		iLstPage = (*hPrint)->prJob.iLstPage;
		
		PrValidate(hPrint);
		
		(*hPrint)->prJob.iFstPage = iFstPage;
		(*hPrint)->prJob.iLstPage = iLstPage;


// Install our idle proc, then loop for the number of copies we need to
// create, open a document and a page, draw the picture centered on the
// page, and close the page and document.

		(*hPrint)->prJob.pIdleProc = (PrIdleProcPtr) IdleProc;

		for (copy = 1; !err && (copy <= (*hPrint)->prJob.iCopies); copy++)
		{
			printPort = PrOpenDoc(hPrint, nil, nil);
	
			if (!(err = PrError()))
			{
				PrOpenPage(printPort, nil);
				err = PrError();
				
				pictBounds = *pictFrame;
				rPage = (*hPrint)->prInfo.rPage;
				CenterRect(&pictBounds, &rPage, &pictBounds);
				DrawPicture(*pict, &pictBounds);

				PrClosePage(printPort);
			}
		
			PrCloseDoc(printPort);
			if (!err) err = PrError();
		}

// When finished, kill the picture, call PrPicFile (if necessary), remove our
// idle proc, stop those silly cursors from spinning, and close the printer driver.
	
		KillPicture(*pict);
		*pict = nil;

		if (!err && ((*hPrint)->prJob.bJDocLoop == bSpoolLoop))
			PrPicFile(hPrint, nil, nil, nil, &theStatus);
	}

	(*hPrint)->prJob.pIdleProc = nil;
	TearDownCursors();
	PrClose();
	return err;
}


/************************************************************
  SetUpQDSearch - This routine performs initialization
  necessary for the OQDSearch routine.  In particular, it
  loads a color table containing the original 8 QuickDraw
  colors.
  
  (The system Color Table with ID = 127 contains the 8
  original QD colors, as described in IM-V, pp.69 & 81.)

*************************************************************/

void SetUpQDSearch()
{	
	gOrgQDCTab = GetCTable(127);
}


/************************************************************
  OQDSearch - This routine is a color search proc, as
  described on pages 145-147 of Inside Mac V.
  
  The routine compares the passed color to the current entry
  of the "Original 8" color table.  If the color matches the
  color we're currently looking for, we return 1.  Otherwise,
  we return 0.  Because the routine which uses this search
  proc uses a 1-bit deep destination, a 0 signifies the
  background color (white in our case), and 1 signifies the
  foreground color.  This routine is used when we create the
  color separations in the PrintOne routine.

  We always return true to indicate that we've handled the
  color processing.

*************************************************************/

pascal Boolean OQDSearch(RGBColor *anRGB, long *position)
{

// Initially, assume we'll return white.  If the color passed
// in matches the current color we're separating, return the
// position for black.

	*position = 0;

	if ((anRGB->red == (*gOrgQDCTab)->ctTable[gCurColor].rgb.red) &&
		(anRGB->green == (*gOrgQDCTab)->ctTable[gCurColor].rgb.green) &&
		(anRGB->blue == (*gOrgQDCTab)->ctTable[gCurColor].rgb.blue))
		*position = 1;

	return true;
}
