/*********************************************************************

	Final Headers.h
	
	This file contains headers for the C & Rez code of the
	all-in-one "Adventures in Color Printing" sample,
	"The Final Adventure."
	
	Dave Hersey, Apple Developer Technical Support.
	
	 Edit Trail 
	
	creation:									11/24/92  - dmh
	cleaned up and finalized:					11/30/92  - dmh
	
*********************************************************************/

#ifndef FINAL_ADVENTURE_HDRS
#define FINAL_ADVENTURE_HDRS


/* Constants:	*/

// Basic defines used by resources:

#define AppName				"The Final Adventure"
#define AppVers				"1.0.1"
#define FinderInfo			"develop issue #13 sample"
#define ApplCreator			'FiAd'

// Misc. app. defines:

#define kIW_wdevID			1		/* ImageWriter driver's wDev value.		*/	
#define kIWLQ_wdevID		5		/* ImageWriter LQ driver's wDev value.	*/

#define kMaxPICTHeapPrcnt	.5		/* If a picture to load requires more	*/
									/* this amount of the largest block of	*/
									/* memory available, we spool it.		*/

#define kMaxNumCursors		20		/* Max number of spinning cursors we	*/
									/* allocate space for.					*/
#define kMinCursorTicks		10		/* Min number of ticks that must pass	*/
									/* between spinning cursor changes.		*/

#define kDITop				100		/* Top coordinate for file dialogs.		*/
#define kDILeft				100		/* Left coordinate for file dialogs.	*/

#define kArrowCursor		0		/* Pseudo resource ID for arrow cursor	*/
									/* which is actually a QD global.  We	*/
									/* use this in TearDownCursors.			*/

#define kPadAmount			8		/* Number of pixels to pad image with	*/
									/* when banding a dithered image.  See	*/
									/* BandPicture for more information.	*/

#define n_dlogLevel			.25		/* Level to display our dialogs at as a	*/
									/* percentage of the way down the		*/
									/* screen.								*/


// Application-defined error codes used by resources and AlertUser:

#define unknownErr		    0		/* OK to use 0, since it's noErr.		*/
#define no32BQDErr			31		/* OK to use 31, it's obsolete.			*/
#define noPrinterChosenErr	42		/* OK to use 42, it's obsolete.			*/

// all other resource IDs are the same as their error codes.


// Misc. resource defines:

#define r_signature			0		/* ID of our signature resource.		*/
#define r_bundle			128		/* ID of our 'BNDL' resource.			*/
#define r_refAPPL			128		/* ID of our APPL 'FREF' resource.		*/
#define r_refPICT			129		/* ID of our PICT 'FREF' resource.		*/

#define kMinSize			512		/* Min app size for 'SIZE' resource.	*/
#define kPrefSize			3000	/* Preferred app size for 'SIZE' res.	*/

#define r_iconAPPL			128		/* ID of our app icon resource.			*/
#define r_iconPICT			129		/* ID of our 'PICT' icon resource.		*/


#define d_maxDetailItem		1		/* "Max. detail" checkbox DITL item.	*/
									/* This item's number is the same in	*/
									/* the two DITLs below.					*/

#define r_IW_AppendDITL		240 	/* DITL with controls to append to IW	*/
									/* print dialog.						*/

#define d_halftoneItem		2		/* "Halftone" radio button DITL item.	*/
#define d_colorItem			3		/* "Color" radio button DITL item.		*/

#define r_nonIW_AppendDITL	242 	/* DITL with controls to append to		*/
									/* non-IW print dialog.					*/


#define r_aboutBox			128		/* ID of our About dialog.				*/

#define r_statDlog			1262	/* ID of our status dialog.				*/
#define r_dlogIcon			240		/* ID of our status dialog icon.		*/
#define t_statusItem		2		/* Item# of status dialog's text field.	*/
#define d_cancelItem		3		/* Item# of status dialog's "Press		*/
									/* command-period to cancel." text.		*/

#define r_errorDialog		130		/* ID of our error dialog.				*/
#define r_errorIcon			130		/* ID of our error dialog icon.			*/
#define d_OKButton			1		/* Item# of error dialog's OK button.	*/
#define d_errMsg			3		/* Item# of error dialog's error text.	*/
#define d_errNum			4		/* Item# of error dialog's error# text.	*/

#define r_newPrintHdlType	'pSet'	/* Resource type of our saved print		*/
									/* records and print dialog options.	*/
									/* The data structure is different than	*/
									/* what the other 3 samples use, so the */
									/* resource type is also.				*/

#define r_spoolSpinCursor	120		/* ID of our spooling 'acur' resource.	*/
#define r_spoolCurs1		140		/* IDs of the cursors referenced by the	*/
#define r_spoolCurs2		141		/* above 'acur'.						*/
#define r_spoolCurs3		142
#define r_spoolCurs4		143

#define r_stdSpinCursor		121		/* ID of our regular 'acur' resource.	*/
#define r_stdCurs1			150		/* IDs of the cursors referenced by the	*/
#define r_stdCurs2			151		/* above 'acur'.						*/
#define r_stdCurs3			152
#define r_stdCurs4			153


// Menu resources

#define r_menuBar			128
#define mApple				128
#define iAbout				1

#define mFile				129
#define iOpen				1
#define iClose				2
#define iPageSetup			4
#define iPrint				5
#define iQuit				7

#define mEdit				130
#define iUndo				1
#define iCut				3
#define iCopy				4
#define iPaste				5
#define iClear				6

#define AllItems		0b1111111111111111111111111111111
#define NoItems			0b0000000000000000000000000000000
#define MenuItem1		0b0000000000000000000000000000001
#define MenuItem2		0b0000000000000000000000000000010
#define MenuItem3		0b0000000000000000000000000000100
#define MenuItem4		0b0000000000000000000000000001000
#define MenuItem5		0b0000000000000000000000000010000
#define MenuItem6		0b0000000000000000000000000100000
#define MenuItem7		0b0000000000000000000000001000000
#define MenuItem8		0b0000000000000000000000010000000
#define MenuItem9		0b0000000000000000000000100000000
#define MenuItem10		0b0000000000000000000001000000000
#define MenuItem11		0b0000000000000000000010000000000
#define MenuItem12		0b0000000000000000000100000000000
#define MenuItem13		0b0000000000000000001000000000000
#define MenuItem14		0b0000000000000000010000000000000



#ifndef FINAL_REZZING

/* Required include files for C sources: */

#include <AppleEvents.h>
#include <Desk.h>
#include <Dialogs.h>
#include <DiskInit.h>
#include <Editions.h>
#include <Errors.h>
#include <Events.h>
#include <Files.h>
#include <FixMath.h>
#include <Fonts.h>
#include <GestaltEqu.h>
#include <Menus.h>
#include <OSEvents.h>
#include <Packages.h>
#include <Printing.h>
#include <QDOffscreen.h>
#include <Resources.h>
#include <Scrap.h>
#include <script.h>
#include <SegLoad.h>
#include <StandardFile.h>
#include <stdio.h>
#include <Strings.h>
#include <SysEqu.h>
#include <ToolUtils.h>
#include <Traps.h>



/* Macros: */

#define ABS(x)     ((x < 0)? -x:x)
#define MAX(x, y)  ((x < y)? y:x)
#define MIN(x, y)  ((x > y)? y:x)



/* Typedefs: */

// ACurRec -- This is just a C structure that parallels
// the 'acur' animated cursor resource.  The animated
// cursor routines use it.

typedef struct ACurRec {
       short	numCursors;		// Number of cursors.
       short	nextFrame;		// Next cursor to display.
       long		cursID[];		// IDs of crsr or CURS resources.
} ACurRec, *ACurPtr, **ACurHdl;



// PictureHeader -- This structure is just a C
// representation of a version 2 picture's header
// listed in Inside Macintosh VI (17-23)

typedef struct PictureHeader {
       short		version;	// Header version.
       short		reserved1;
	   Fixed		hRes;		// PICT horizontal resolution (if extended v2 pict).
	   Fixed		vRes;		// PICT vertical resolution (if extended v2 pict).
	   Rect			srcRect;	// PICT native bounds at hRes,vRes (if extended v2 pict).
	   long			reserved2;
} PictureHeader;



// PartialPicture -- This is the structure we use to
// hold a picture's version and version 2 header, if
// it has one.

typedef struct PartialPicture {
	   short			pictVersion;	// PICT version.
       PictureHeader	pictHeader;		// PICT header.
} PartialPicture, *PartialPicturePtr, **PartialPictureHdl;



// WindowInfoRec -- This structure is what ties an
// image's file and picture information to the
// window displaying it.  Every window we create
// has a handle to one of these structures in its
// window refCon field.

typedef struct WindowInfoRec {
		Boolean			loaded;			// Picture loaded?  (No means spool.)
		PicHandle		windowPict;		// Handle to picture for window.
		PartialPicture	partialPict;	// Version and hdr. info for picture.
		SFReply			reply;			// Pre-7.0 file info for picture.
		FSSpec			oFSSpec;		// System 7.0 file info for picture.
} WindowInfoRec, *WindowInfoPtr, **WindowInfoHdl;



// PrintSettings -- This structure is used to save our
// print records and appended dialog settings.

typedef struct PrintSettings {
        TPrint		printRec;			// The print record.
        Boolean		printInColor;		// Last value for the "print in color" item.
		Boolean		printMaxDetail;		// Last value for the "print max. detail" item.
    } PrintSettings, *PrintSettingsPtr, **PrintSettingsHdl;



// DialogItem -- This structure is used by Append_DITL.
// It represents one item entry in a DITL.

typedef struct DialogItem {
        Handle		handle;				// Handle or procedure pointer for this item.
        Rect		bounds;				// Display rectangle for this item.
        char		type;				// Item type - 1.
        char		data[1];			// Data length of item.
    } DialogItem, *DialogItemPtr, **DialogItemHandle;



// ItemList -- This structure is used by Append_DITL.
// It represents a DITL.

typedef struct ItemList {
		short		max_index;			// Number of items in DITL - 1.
		DialogItem	items[1];			// First item in the array.
    } ItemList, *ItemListPtr, **ItemListHandle;



// ByteAccess -- This structure is used by Append_DITL.
// It allows a short to be accessed as two bytes.

typedef union ByteAccess {
        short		integer;			// The short
        char		bytes[2];			// The two bytes it coexists as.
    } ByteAccess;



/* Global variables: */

extern Boolean		gQuitting;			// True if ready to quit.
extern Boolean		gInBackground;		// True if running in the background.
extern Boolean		gSystemSeven;		// True if running under system 7 or later.
extern Boolean		gHaveNewSFCalls;	// True if we have the new Standard File calls.
extern Boolean		g8BQD;				// True if we have "regular" Color QuickDraw.
extern Boolean		g32BQD;				// True if we also have 32-Bit Color QuickDraw.
extern RgnHandle	gCursorRgn;			// Mouse-moved region used by event loop.
extern THPrint		ghPrint;			// Global print record.
extern short		gAppResRefNum;		// RefNum of application's resource file.

extern Boolean		gPrintInColor;		// Last used values for the controls we append
extern Boolean		gPrintMaxDetail;	// to the print dialogs.  gPrintInColor is only
										// used by ImageWriter & ImageWriter LQ dialogs.



/* Function prototypes & file locations: */

// Final Main.c

void			main(void);


// Final Initialize.c

void			CheckConfig(void);
void			Initialize(void);


// Final Event Stuff.c

OSErr			CheckAEParams(AppleEvent *theAppleEvent);
void			DoAEInstallation(void);
void			DoEvent(EventRecord *event);
void			EventLoop(void);
pascal OSErr	HandleOAPP(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon);
pascal OSErr	HandleODOC(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon);
pascal OSErr	HandlePDOC(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon);
pascal OSErr	HandleQUIT(AppleEvent *theAppleEvent, AppleEvent *reply, long myRefCon);


// Final Windows & Menus.c

void			AdjustMenus(void);
void			AlertUser(OSErr err);
void			CleanUp(void);
OSErr			CreatePictWindow(FSSpecPtr opFSSpec, SFReply *reply);
void			DoAnyUpdates(void);
void			DoCloseWindow(WindowPtr wp);
void			DoMenuCommand(long menuResult);
void			DoUpdate(WindowPtr wp);
PixMapHandle	GetGWPixMap(GWorldPtr aGWorld);
void			HideStatusMsg(DialogPtr *statDlog);
void			PositionWindow(WindowPtr windPtr, Boolean showIt, float vertPercent);
void			SetUpMenus(void);
void			ShowStatusMsg(DialogPtr *statDlog, Str255 msg1, Str255 docTitle,
							  Str255 msg2, Boolean showCancelMsg);

// Final File.c

void			BumpCursor(void);
void			GetOpenName(StandardFileReply *toReply7, SFReply *toReply);
OSErr			LoadAPICT(WindowInfoHdl windInfo, Rect *pictRect);
void			LoadOrPrintFiles(void);
void			SetUpCursors(short acurID, short finalCursorID);
OSErr			SpoolInstall(WindowInfoHdl windInfo, short cursorToUseWhenDone);
pascal void		SpoolPicProc(Ptr buffer, short amtToRead);
OSErr			SpoolPictToWindow(WindowPtr wp);
OSErr			SpoolRemove(void);
void			TearDownCursors(void);


// Final Printing.c

OSErr			BandPicture(WindowPtr wp, Rect *srcRect, GWorldPtr destGWorld,
							Boolean useDitherCopy);
void			CenterRect(Rect *srcRect, Rect *destRect, Rect *ctrRect);
pascal void		ColorBitsProc(BitMap *srcBits, Rect *srcRect, Rect *dstRect, short mode,
							  RgnHandle maskRgn);
OSErr			CommandPeriod(void);
void			GetBestDPI(short *pxDPI, short *pyDPI, short xDPI_ceiling,
						   short yDPI_ceiling, Boolean wantSquareDPI);
OSErr			GetImageRes(WindowInfoHdl windInfo, short *ixDPI, short *iyDPI,
							Rect *srcRect);
OSErr			GetPixMapRes(WindowInfoHdl windInfo, Fixed *hRes, Fixed *vRes,
							 Rect *srcRect);
short			GetPrinterRes(void);
void			GetResIDString(THPrint	hPrint, Str255	resIDStr);
Boolean			HaveColorPrPort(THPrint hPrint, OSErr *anErr);
pascal void		IdleProc(void);
OSErr			LoadPHdl(THPrint *hPrint, short resRefNum);
OSErr			PrintOne(WindowPtr wp);
OSErr			SavePHdl(THPrint hPrint, short resRefNum);
void			SetPrinterPortDPI(THPrint hPrint, short *pxDPI, short *pyDPI);
OSErr			SetupPrintHdl(THPrint *hPrint);


// Final Print Dialogs.c

short			Append_DITL(TPPrDlg dialog, short aDITLResID);
pascal TPPrDlg	DoJobDlgInit(THPrint hPrint);
OSErr			DoJobDlog(THPrint hPrint, short aDITLResID);
OSErr			DoStyleDlog(THPrint hPrint);
pascal void		HandleJobItems(TPPrDlg theDialog, short itemNo);


// Final Color IW Printing.c

OSErr			DoColorIWPrint(WindowPtr whichWindow, Boolean useHighRes);
OSErr			DoColorIWPrintLoop(THPrint hPrint, PicHandle *pict, Rect *pictFrame);
pascal Boolean	OQDSearch(RGBColor *anRGB, long *position);
void			SetUpQDSearch(void);


// Final Color Printing.c

OSErr			DoColorPrint(WindowPtr whichWindow, Boolean useBanding);
OSErr			DoColorPrintLoop(THPrint hPrint, WindowPtr wp, Rect *destRect,
							  Boolean useDrawPicture);


// Final Halftone Printing.c

OSErr			DoHT0(WindowPtr wp, short pDPI, Rect *destRect);
OSErr			DoHT45(WindowPtr wp, short pDPI, short ixDPI, short iyDPI,
					   Rect *destRect);
OSErr			DoHTPrint(WindowPtr whichWindow, Boolean useTrueHalftone);
OSErr			DoHTPrintLoop(THPrint hPrint, GWorldPtr *srcWorld, Rect *srcRect);
OSErr			HalftonePixMap(GWorldPtr srcGWorld, GWorldPtr *destGWorld, short srcDPI,
							   short destDPI);
long			LUMVAL(Ptr pPixel, PixMapPtr pMap);
OSErr			SimulateHalftone(Rect *srcRect, GWorldPtr srcGWorld, GWorldPtr *htGWorld);


#endif FINAL_REZZING
#endif FINAL_ADVENTURE_HDRS
