/*********************************************************************

	Final Initialize.c
	
	This file contains initialization functions for the all-in-one
	"Adventures in Color Printing" sample, "The Final Adventure."
	Code is used at launch time and then dumped.
	
	Dave Hersey, Apple Developer Technical Support.
	
	 Edit Trail 
	
	creation:									11/24/92  - dmh
	
*********************************************************************/

#pragma segment Initialize

#include "Final Headers.h"


/************************************************************
  Initialize - This routine does necessary toolbox
  initialization and checks our system configuration.

*************************************************************/

void Initialize()
{
	short 	i;
	OSErr	err;

// Standard initialization stuff:

	MaxApplZone();
	for (i=1; i<=4; i++) MoreMasters();
	
	InitGraf((Ptr) &qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
	FlushEvents(everyEvent, 0);
	gAppResRefNum = CurResFile();


// Check our system configuration, set up a default print
// record, and see if there's saved print record to load.

	CheckConfig();

	err = SetupPrintHdl(&ghPrint);
	AlertUser(err);

	if (!err)
	{
		err = LoadPHdl(&ghPrint, gAppResRefNum);
		if (err != resNotFound) AlertUser(err);
	}


// Set up some globals, our menus, and (if we're running
// under system 7 or later) do the required initialization.

	gInBackground = false;
	gQuitting = false;
	gCursorRgn = NewRgn();

	SetUpMenus();
	
	if (gSystemSeven)
	{
		InitEditionPack();	/* initialize Edition Manager */
		DoAEInstallation();	/* install AppleEvent handlers */
	}


// Copy the desk scrap to the TextEdit scrap.  Alert and quit
// if there's no 32-bit QuickDraw (we require it), otherwise
// go do the old document loading/printing stuff if we're not
// running under system 7 or later.

	TEFromScrap();

	if (!g32BQD)
	{
		AlertUser(no32BQDErr);
		gQuitting = true;
	}
	else
		if (!gSystemSeven)
			LoadOrPrintFiles();
}


/************************************************************
  CheckConfig - This routine sets up our global
  configuration variables based on results from Gestalt.
  Note that we don't need to check for Gestalt's existance,
  because MPW 3.2 and later add glue for handling that.

*************************************************************/

void CheckConfig()
{
	long	qdVersion, sysVersion, fileVersion;

// Initially, we don't have anything.  Get the system version
// and see if it's 7.0 or later and determine if we have the
// new, cool Standard File calls or not.  If we don't have the
// new SFGet/SFPut stuff as well as the FSSpec stuff, we just
// use the old calls.

	g8BQD = g32BQD = gSystemSeven = gHaveNewSFCalls = false;
	
	if (Gestalt(gestaltSystemVersion, &sysVersion) == noErr)
		if (sysVersion >= 0x0700) gSystemSeven = true;
	
	if (Gestalt(gestaltStandardFileAttr, &fileVersion) == noErr)
		if ((fileVersion & gestaltStandardFile58) == gestaltStandardFile58)
			if (Gestalt(gestaltFSAttr, &fileVersion) == noErr)
				if ((fileVersion & gestaltHasFSSpecCalls) == gestaltHasFSSpecCalls)
					gHaveNewSFCalls = true;


// Get the QuickDraw version and see if we're using 8-bit or
// 32-bit QuickDraw.

	if (Gestalt(gestaltQuickdrawVersion, &qdVersion) == noErr)
	{
		if (qdVersion >= gestalt32BitQD)
		{
			g32BQD = true;
			g8BQD = true;
		}
		else
			if (qdVersion >= gestalt8BitQD)
				g8BQD = true;
	}
}
