/*********************************************************************

	Final Windows & Menus.c
	
	This file contains functions for window and menu handling in the
	all-in-one "Adventures in Color Printing" sample, "The Final
	Adventure."
	
	Dave Hersey, Apple Developer Technical Support.
	
	 Edit Trail 
	
	creation:									11/24/92  - dmh

	fixed AlertUser so that it no longer
	requires 32 BQD or CQD.  This caused
	problems when the "No 32BQD Available"
	alert was being displayed Version
	changes to 1.0.1:							12/30/92  - dmh

*********************************************************************/

#pragma segment WindowsAndMenus

#include "Final Headers.h"


/************************************************************
  SetUpMenus - This routine puts our menubar together.

*************************************************************/

void SetUpMenus()
{
	Handle		menuBar;

// Create and set up the menubar.

	menuBar = GetNewMBar(r_menuBar);
	SetMenuBar(menuBar);
	DisposHandle(menuBar);
	AddResMenu(GetMHandle(mApple), 'DRVR');
	DrawMenuBar();
}


/************************************************************
  AdjustMenus - This routine handles enabling and disabling
  menu items as required by the current context.

*************************************************************/

void AdjustMenus()
{
	WindowPtr	wp;
	MenuHandle	fileMenu, editMenu;

// The edit menu is always disabled, since we don't use it.

	editMenu = GetMHandle(mEdit);
	
	DisableItem(editMenu, iUndo);
	DisableItem(editMenu, iCut);
	DisableItem(editMenu, iCopy);
	DisableItem(editMenu, iClear);
	DisableItem(editMenu, iPaste);


// If we have a window open, we enable "Close" and "Print,"
// otherwise we don't.

	wp = FrontWindow();
	fileMenu = GetMHandle(mFile);

	if (!wp)
	{
		DisableItem(fileMenu, iClose);
		DisableItem(fileMenu, iPrint);
	}
	else
	{
		EnableItem(fileMenu, iClose);
		EnableItem(fileMenu, iPrint);
	}
}


/************************************************************
  DoMenuCommand - This routine handles our menu command
  dispatching.

*************************************************************/

void DoMenuCommand(long menuResult)
{
	short				menuID;
	short				menuItem;
	MenuHandle			targetMenu;
	short				itemHit;
	Str255				daName;
	StandardFileReply	reply7;
	SFReply				reply;
	WindowPtr			wp;
	OSErr				err = noErr;

// Get the selected menu's ID, item and MenuHandle.

	menuID = menuResult >>16;
	menuItem = menuResult & 0xFFFF;
	targetMenu = GetMHandle(menuID);

	switch (menuID)
	{
		case mApple:
			switch (menuItem)
			{
				case iAbout:		// Display About box.
					
					itemHit = Alert(r_aboutBox, nil);
					break;
				
				default:			// Handle DAs.
					
					GetItem(GetMHandle(mApple), menuItem, daName);
					OpenDeskAcc(daName);
					break;
			}
			break;
			
			case mFile:
				switch (menuItem)
				{
					case iOpen:		// Open a document.
					
						GetOpenName(&reply7, &reply);

						if ((gHaveNewSFCalls && reply7.sfGood) || (!gHaveNewSFCalls && reply.good))
						{
							err = CreatePictWindow((FSSpecPtr) &reply7.sfFile, &reply);
							AlertUser(err);
						}
						break;
					
					case iClose:	// Close a document.
					
						DoCloseWindow(FrontWindow());
						break;
						
					case iPageSetup:// Do a Page Setup.

						err = DoStyleDlog(ghPrint);
						if (!err) err = SavePHdl(ghPrint, gAppResRefNum);
						AlertUser(err);
						break;
												
					case iPrint:	// Print a document.
							
						if ((wp = FrontWindow()) != nil)
						{
							err = PrintOne(wp);
							AlertUser(err);
						}
						break;

					case iQuit:		// Quit the application.

						CleanUp();
						gQuitting = true;
						break;
				}
				break;
	}

	HiliteMenu(0);
}


/************************************************************
  CreatePictWindow - This routine creates a new window and
  loads the requested PICT image for it.

*************************************************************/

OSErr CreatePictWindow(FSSpecPtr opFSSpec, SFReply *reply)
{
	OSErr				err;
	WindowInfoHdl		windInfo;
	WindowPtr			wp;
	short				menuHt;
	Rect				windRect, scrnRect;
	StringPtr			namePtr;

// Find the largest rectangle that will our picture can use on the
// screen.

	menuHt = GetMBarHeight();
	
	scrnRect = qd.screenBits.bounds;	// Bounds of main screen
	scrnRect.top += menuHt +17;			// - window's title bar and menu bar.
	InsetRect(&scrnRect, 4, 4);			// - a 4 pixel border around window.


// Create a handle for our private window data, then load the requested
// PICT, (or get the spool information if we can't load it).  We specify
// that the picture can be as large as the scrnRect we just calculated.
// If the image is larger than that, it will need to be proportionally
// scaled into that area.

	windRect = scrnRect;
	windInfo = (WindowInfoHdl) NewHandle(sizeof(WindowInfoRec));

	if (!(err = MemError()))
	{
		BlockMove((Ptr) reply, &(*windInfo)->reply, sizeof(SFReply));
		BlockMove((Ptr) opFSSpec, &(*windInfo)->oFSSpec, sizeof(FSSpec));
		err = LoadAPICT(windInfo, &windRect);
	}

// Take the display rectangle that LoadAPict told us we need for this picture and
// center it within the main screen's bounds.  Create a color window there, select
// it, and store our private handle in the refCon field.

	if (!err)
	{	
		CenterRect(&windRect, &scrnRect, &windRect);
		namePtr = (gHaveNewSFCalls)? &opFSSpec->name : &reply->fName;

		wp = NewCWindow(nil, &windRect, namePtr, true, noGrowDocProc, (WindowPtr) -1,
						true, 0);
		
		SelectWindow(wp);
		SetWRefCon(wp, (long) windInfo);
	}
	
	return err;
}


/************************************************************
  PositionWindow - This routine positions a window on 
  the screen, centered horizontally and positioned
  vertPercent down the current screen.  If showIt is true,
  we make the window visible, otherwise we hide it.

*************************************************************/

void PositionWindow(WindowPtr windPtr, Boolean showIt, float vertPercent)
{
	Rect		devRect;
	short		amtFromTop, windWidth, deviceWidth;
	GDHandle	curGDH;

// Get the current GDevice and use its bounds to calculate where we
// should move the window to.

	if (windPtr != nil)
	{
		if (g8BQD)
		{
			curGDH = GetGDevice();
			devRect = (*curGDH)->gdRect;
		}
		else
			devRect = qd.screenBits.bounds;
		
		amtFromTop = (devRect.bottom - devRect.top +1) * vertPercent;
		windWidth = windPtr->portRect.right - windPtr->portRect.left +1;
		deviceWidth = devRect.right - devRect.left +1;


// Hide the window, move it, and (if we're supposed to) show it again.

		ShowHide(windPtr, false);
		MoveWindow(windPtr, devRect.left + (deviceWidth /2 - windWidth /2), devRect.top + amtFromTop, true);
		if (showIt) ShowHide(windPtr, true);
	}
}


/************************************************************
  DoCloseWindow - This routine disposes of the specified
  window and its window picture.

*************************************************************/

void DoCloseWindow(WindowPtr wp)
{
	WindowInfoHdl	windInfo;
	
	if (wp)
	{
		windInfo = (WindowInfoHdl) GetWRefCon(wp);

		if (windInfo != nil)
		{
			if ((*windInfo)->loaded)
				KillPicture((*windInfo)->windowPict);

			DisposHandle((Handle) windInfo);
		}
	
		DisposeWindow(wp);
	}
}


/************************************************************
  CleanUp - This routine disposes of any windows that are
  open, as well as our global print handle, when we quit.

*************************************************************/

void CleanUp()
{
	WindowPtr	wp;

// Close any open windows and dump our print handle.

	do
	{
		wp = FrontWindow();
		if (wp) DoCloseWindow(wp);
	}
	while (wp);
	
	if (ghPrint) DisposeHandle((Handle) ghPrint);
}


/************************************************************
  DoUpdate - This routine handles updating our windows.
  
  We can tell if the image needs to be spooled into the
  window (or if it's completely loaded) by looking at the
  WindowInfoHdl we declared and attached to each window.
  If we need to spool the pict in, do that, otherswise just
  draw it.  We scale the picture to the window's size so we
  can see the entire image, not only what would fit on the
  screen at the actual size.

*************************************************************/

void DoUpdate(WindowPtr wp)
{
	WindowInfoHdl	windInfo;
	OSErr			err = noErr;

// Begin updating.  If this window is one of ours, get our
// saved window data that's attached to it.

	SetPort(wp);
	BeginUpdate(wp);
	
	if (((WindowPeek) wp)->windowKind == userKind)
	{
		windInfo = (WindowInfoHdl) GetWRefCon(wp);


// If the picture is loaded, draw it.  Otherwise, spool it
// into the window.

		SetPort(wp);

		if ((*windInfo)->loaded)
			DrawPicture((*windInfo)->windowPict, &wp->portRect);
		else
			err = SpoolPictToWindow(wp);
	}


// Finish up.  If there were any errors, tell our users.

	EndUpdate(wp);

	if (err)
	{
		AlertUser(err);
		DoCloseWindow(wp);
	}
}


/************************************************************
  DoAnyUpdates - This routine updates any windows that need
  it.  This is a handy routine for those times when you need
  to service updates and you're far away from your event loop.
  For example, right after the user dismisses a print dialog
  that overwrote some of their windows.
  
  Note that this routine does NOT call WaitNextEvent because
  that gives the user a chance to switch out, unless you have
  a modal dialog up.  Immediately after a print dialog, a
  modal dialog is unlikely (usually that goes up a bit
  later).  This is bad because if the user switches out,
  they can go change printers or something.  Not good.  The
  solution is to walk our window list and see whose update
  regions are non-nil.  Then we just update those windows.

*************************************************************/

void DoAnyUpdates()
{
	WindowPtr	curWindow;
	
	curWindow = FrontWindow();
	
	while (curWindow)
	{
		if (!EmptyRgn(((WindowPeek) curWindow)->updateRgn))
			DoUpdate(curWindow);

		curWindow = (WindowPtr) ((WindowPeek) curWindow)->nextWindow;
	}
}


/************************************************************
  ShowStatusMsg - This routine displays a message in a
  status dialog.  It's used by the printing routines to
  inform the user about what's going on behind the scenes.
  
  If showCancelMsg is true, the standard "Press command-
  period to cancel" text is drawn at the bottom of the
  dialog.
  
  Note: docTitle is a Pascal-style string, msg1 and msg2
  are C strings.

*************************************************************/

void ShowStatusMsg(DialogPtr *statDlog, Str255 msg1, Str255 docTitle,
				   Str255 msg2, Boolean showCancelMsg)
{
	short			tempType;
	Handle			txtHdl;
	Rect			tempRect;
	Str255			contentStr, curTitle;
	Boolean			justOpened;
	GWorldPtr		origGW;
	GDHandle		origGDH;
	CGrafPtr		cwmPort;

// Make sure we're not pointing to an offscreen GWorld.  We'll
// restore the current GWorld/GDevice on exit.

	GetGWorld(&origGW, &origGDH);
	GetCWMgrPort(&cwmPort);
	SetGWorld((GWorldPtr) cwmPort, nil);


// If this is our first call (there's no dialog yet) load the
// status dialog and position it.

	justOpened = (*statDlog == nil);

	if (justOpened)
	{
		*statDlog = GetNewDialog(r_statDlog, nil, (WindowPtr) -1);
		if (*statDlog != nil) PositionWindow(*statDlog, true, n_dlogLevel);
	}

// If this isn't our first call, bring the status dialog to the front if
// it's not, and set its title (so the documents are named correctly when
// we print them.
	
	if (*statDlog != nil)
	{
		if (FrontWindow() != *statDlog)
			SelectWindow(*statDlog);

		SetPort(*statDlog);

		if (!(showCancelMsg))
			HideDItem(*statDlog, d_cancelItem);
		else
			ShowDItem(*statDlog, d_cancelItem);
		
		GetWTitle(*statDlog, curTitle);
		if (!EqualString(docTitle, curTitle, true, true))
			SetWTitle(*statDlog, docTitle);


// Set the status message for the dialog, draw the dialog if this is the first
// time through, restore the GWorld/GDevice we came in with, and exit.

		GetDItem(*statDlog, t_statusItem, &tempType, (Handle *) &txtHdl, &tempRect);
		
		p2cstr(docTitle);
		contentStr[0] = sprintf(&contentStr[1], "%s\"%s\"%s", msg1, docTitle, msg2);
		c2pstr(docTitle);
		SetIText(txtHdl, contentStr);

		if (justOpened) DrawDialog(*statDlog);
	}
	
	SetGWorld(origGW, origGDH);
}


/************************************************************
  HideStatusMsg - This routine disposes of the status
  dialog, and sets its reference to nil so that we know
  to create the dialog again the next time we enter
  ShowStatusMsg.

*************************************************************/

void HideStatusMsg(DialogPtr *statDlog)
{
	if (*statDlog != nil)
	{
		DisposeDialog(*statDlog);
		*statDlog = nil;
	}
}


/************************************************************
  GetGWPixMap - Due to a bug in GetGWorldPixMap under early
  versions of 32-bit QuickDraw, you should only call it
  when you're using the 7.0 version or later.  You could
  check for the specific version of 32-bit QuickDraw to
  avoid, but a good rule of thumb is to only make the call
  under system seven.  That's what goes on below.

*************************************************************/

PixMapHandle GetGWPixMap(GWorldPtr aGWorld)
{
	if (gSystemSeven)
		return GetGWorldPixMap(aGWorld);
	else
		return aGWorld->portPixMap;
}


/************************************************************
  AlertUser - This call simulates an alert, by displaying
  a dialog with an appropriate error message in it.  This
  routine is used by, oddly enough, the error handling
  routines.

*************************************************************/

void AlertUser(OSErr err)
{
	DialogPtr		errorDlog;
	StringHandle	errString;
	Str255			pStr;
	Handle			itmHandle;
	short			itemHit, itemKind, oldRes;
	Rect			itemRect;
	Boolean			errIsUnknown;
	GWorldPtr		origGW;
	GDHandle		origGDH;
	CGrafPtr		cwmPort;

// Some errors are OK and don't warrant a dialog.

	if ((err == noErr) || (err == iPrAbort)) return;


// Make sure we're not pointing to an offscreen GWorld.  We'll
// restore the current GWorld/GDevice on exit.
	
	if (g32BQD)
	{
		GetGWorld(&origGW, &origGDH);
		GetCWMgrPort(&cwmPort);
		SetGWorld((GWorldPtr) cwmPort, nil);
	}


// Switch to the application's resource file and load the error
// dialog.

	oldRes = CurResFile();
	UseResFile(gAppResRefNum);

	errorDlog = GetNewDialog(r_errorDialog, nil, (WindowPtr) -1);


// Load the 'STR ' resource that corresponds to the errorCode
// passed.  If no 'STR ' exists, load the special "unknown
// error" string that we created just for the occasion.

	if (errorDlog != nil)
	{
		errString = GetString(err);
		if (errIsUnknown = (errString == nil))
			errString = GetString(unknownErr);


// Set the 1st static text item to the error message we loaded.

		GetDItem(errorDlog, d_errMsg, &itemKind, &itmHandle, &itemRect);
		
		HLock(errString);
		SetIText(itmHandle, *errString);
		HUnlock(errString);
		ReleaseResource(errString);


// If we didn't have a specific error message for this situation,
// also display the error code, so that those in the know might
// have a clue what went wrong.

		if (errIsUnknown)
		{
			GetDItem(errorDlog, d_errNum, &itemKind, &itmHandle, &itemRect);
			pStr[0] = sprintf(&pStr[1], "%s%i%s", "(ID = ", err, ")");
			SetIText(itmHandle, pStr);
		}


// Position and draw the dialog.  Then, outline the default button and beep.

		PositionWindow(errorDlog, true, n_dlogLevel);
		GetDItem(errorDlog, d_OKButton, &itemKind, &itmHandle, &itemRect);
		DrawDialog(errorDlog);

		SetPort(errorDlog);
		PenSize(3,3);
		InsetRect(&itemRect, -4, -4);
		FrameRoundRect(&itemRect, 16, 16);
		PenSize(1,1);
		
		SysBeep(3);


// Keep looping until the user has had enough.  Once she clicks on
// the OK button, dispose of our dialog and switch back to the
// original GWorld/GDevice and resource file.

		do
		{
			ModalDialog(nil, &itemHit);
		}
		while (itemHit != d_OKButton);
		
		DisposeWindow(errorDlog);
	}

	if (g32BQD)
		SetGWorld(origGW, origGDH);

	UseResFile(oldRes);
}

