//--------------------------------------------------------------------------
//
//		ListProfiles.
//			by John Wang
//
//		Description:	Uses GetIndexedProfile to list profiles
//
//		Version:	1.0		11/24/92	Completed.
//		Version:	1.01	01/07/93	Added support for printer profiles.
//		Version:	1.02	01/13/93	Cleaned up for developer CD release.
//		Version:	1.03	03/11/93	Cleaned up develop column.
//
//--------------------------------------------------------------------------


//
//	#defines
//

#define	TRUE			1
#define	FALSE			0

//
//	#includes
//

#include <Memory.h>
#include <Fonts.h>
#include <Palettes.h>
#include <OSEvents.h>
#include <Menus.h>
#include <Sound.h>
#include <ToolUtils.h>
#include <Desk.h>
#include <Resources.h>
#include <Errors.h>
#include <Packages.h>
#include <GestaltEqu.h>
#include <Printing.h>

#include <CMApplication.h>
#include <CMComponent.h>

//--------------------------------------------------------------------------

waitClick()
{
	do {
	} while (!Button());
	do {
	} while (Button());
}

//--------------------------------------------------------------------------

void drawInfo(long loc, CMProfileHandle myProfile)
{
	long						offset;
	char						*myPtr;
	Str255						myStr;

	MoveTo(40, 80*(loc-1)+40);
	DrawString("\PProfile name: ");
	offset = (**myProfile).header.profileNameOffset;
	myPtr = (char *)*myProfile;
	DrawString(*(Str255 *)myPtr + offset + 2);

	MoveTo(40, 80*(loc-1)+50);
	DrawString("\PCMMType: ");
	BlockMove(&((**myProfile).header.CMMType), &myStr[2], 4);
	myStr[0] = 6; myStr[1] = myStr[6] = '\'';
	DrawString(myStr);

	MoveTo(40, 80*(loc-1)+60);
	DrawString("\PprofileVersion: ");
	NumToString((**myProfile).header.applProfileVersion, myStr);
	DrawString(myStr);

	MoveTo(40, 80*(loc-1)+70);
	DrawString("\PdataType: ");
	BlockMove(&((**myProfile).header.dataType), &myStr[2], 4);
	myStr[0] = 6; myStr[1] = myStr[6] = '\'';
	DrawString(myStr);

	MoveTo(40, 80*(loc-1)+80);
	DrawString("\PdeviceType: ");
	BlockMove(&((**myProfile).header.deviceType), &myStr[2], 4);
	myStr[0] = 6; myStr[1] = myStr[6] = '\'';
	DrawString(myStr);

	MoveTo(40, 80*(loc-1)+90);
	DrawString("\PdeviceManufacturer: ");
	BlockMove(&((**myProfile).header.deviceManufacturer), &myStr[2], 4);
	myStr[0] = 6; myStr[1] = myStr[6] = '\'';
	DrawString(myStr);
}

//--------------------------------------------------------------------------

void main()
{
	WindowPtr					mainWinPtr;
	long						CMversion;
	OSErr						err;
	Rect						windRect;

	CMProfileSearchRecordHandle	mySearchProfile;
	CMProfileHandle				myProfile;
	long						numProfiles;
	Str255						myStr;
	long						i;

	THPrint						prRecHdl;
	TRegisterProfileBlk			regProfileBlk;
	short						prError;
	Boolean						printerOpened;
	
	//	Initialize Managaer.
	MaxApplZone();
	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(nil);
	InitCursor();
	
	//	Use Gestalt to find out version of ColorSync available
	if (Gestalt(gestaltColorMatchingVersion, &CMversion))
		CMversion = 0;
	if (CMversion < 0x100)
		ExitToShell();

	//	Define output window.
	SetRect(&windRect, 10, 40, 510, 460);
	mainWinPtr = NewCWindow(nil, &windRect, "\pListProfiles", true, documentProc, (WindowPtr) -1, false, 0);
	SetPort(mainWinPtr);
	SetRect(&windRect, 0, 0, 500, 420);
	TextSize(9);
	
	//	Create search record that searches everything.
	mySearchProfile = (CMProfileSearchRecordHandle) NewHandle(sizeof(CMProfileSearchRecord));
	
	// You call this error handling?
	if(mySearchProfile == nil)
	{
		DebugStr("\PCan't make Search Profile.  Exiting to shell.");
		ExitToShell();
	}
	
	(**mySearchProfile).header.CMMType = 0;
	(**mySearchProfile).header.applProfileVersion = 0;
	(**mySearchProfile).header.dataType = 0;
	(**mySearchProfile).header.deviceType = 0;
	(**mySearchProfile).header.deviceManufacturer = 0;
	(**mySearchProfile).header.deviceModel = 0;
	(**mySearchProfile).header.deviceAttributes[0] = 0;
	(**mySearchProfile).header.deviceAttributes[1] = 0;
	(**mySearchProfile).header.profileNameOffset = 0;
	(**mySearchProfile).header.customDataOffset = 0;
	(**mySearchProfile).header.flags = 0;
	(**mySearchProfile).header.options = 0;
	(**mySearchProfile).header.white.X = 0;
	(**mySearchProfile).header.white.Y = 0;
	(**mySearchProfile).header.white.Z = 0;
	(**mySearchProfile).header.black.X = 0;
	(**mySearchProfile).header.black.Y = 0;
	(**mySearchProfile).header.black.Z = 0;
	(**mySearchProfile).fieldMask = 0;
	(**mySearchProfile).reserved[0] = 0;
	(**mySearchProfile).reserved[1] = 0;

	//	List system profiles

	EraseRect(&windRect);
	numProfiles = 0;
	
	//	Get number of profiles.
	if (err = GetIndexedProfile('sys ', 0, mySearchProfile, &myProfile, &numProfiles)) {
		DebugStr("\PGetIndexedProfile failed.  Exiting to shell.");
		ExitToShell();
	}
	MoveTo(20,20);
	DrawString("\PNumber of system profiles: ");
	NumToString(numProfiles, myStr);
	DrawString(myStr);

	//	List all profiles.
	for (i=1; i<=numProfiles; i++) {		//	List profiles in increasing order.
		if (err = GetIndexedProfile(0, 0, mySearchProfile, &myProfile, &i)) {
			DebugStr("\PGetIndexedProfile failed.  Exiting to shell.");
			ExitToShell();
		}
		drawInfo(i, myProfile);
	}

	// Wait until user clicks button.
	waitClick();
	
//	List printer profiles

	EraseRect(&windRect);
	numProfiles = 0;
	
	//	Register printer profile responder.
	PrOpen();
	if ((prError = PrError()) == noErr) {
		printerOpened = TRUE;
		prRecHdl = (THPrint)NewHandle(sizeof(TPrint));

		// You call this error handling?
		if(prRecHdl == nil)
		{
			DebugStr("\PCan't make prRecHdl.  Exiting to shell.");
			ExitToShell();
		}	

		PrintDefault(prRecHdl);

		regProfileBlk.iOpCode = registerProfileOp;
		regProfileBlk.iError = 0; 
		regProfileBlk.lReserved = 0;
		regProfileBlk.hPrint = prRecHdl;
		regProfileBlk.fRegisterIt = TRUE;  //	register the printer responder
		PrGeneral((Ptr)&regProfileBlk);
		prError = regProfileBlk.iError;
	}
	
	//	Get number of profiles.
	if (err = GetIndexedProfile('prtr', 0, mySearchProfile, &myProfile, &numProfiles)) {
		if (err == 0) {		//	Only fail if profile registered improperly.
			DebugStr("\PGetIndexedProfile failed.  Exiting to shell.");
			ExitToShell();
		} else {
			numProfiles = 0;
		}
	}
	MoveTo(20,20);
	if (prError != 0) {
		DrawString("\PPrinter does not respond to prGeneral call to register responder.");
	} else {
		DrawString("\PNumber of printer profiles: ");
		NumToString(numProfiles, myStr);
		DrawString(myStr);
	}
	
	//	List all profiles.
	for (i=1; i<=numProfiles; i++) {
		if (err = GetIndexedProfile(0, 0, mySearchProfile, &myProfile, &i)) {
			DebugStr("\PGetIndexedProfile failed.  Exiting to shell.");
			ExitToShell();
		}
		drawInfo(i, myProfile);
	}

	//	Unregister printer profile responder.
	if (printerOpened) {
		regProfileBlk.iOpCode = registerProfileOp ;
		regProfileBlk.iError = 0 ; 
		regProfileBlk.lReserved = 0 ;
		regProfileBlk.hPrint = prRecHdl ;
		regProfileBlk.fRegisterIt = FALSE ;  //	unregister the printer responder
		PrGeneral((Ptr)&regProfileBlk) ;
		prError = regProfileBlk.iError ;
		PrClose() ;
		printerOpened = FALSE;
	}

	// Wait until user clicks button.
	waitClick();
}

