/* ------------------------------------------------------------------------------

	FILENAME
		4-Up.r

	DESCRIPTION
		All resources required by the 4-Up extension are declared in this file.

	COPYRIGHT
		Copyright Apple Computer, Inc. 1991
		All rights reserved. 
	
	MODIFICATION HISTORY
		06/10/91			PrintShop			Initial Implementation


------------------------------------------------------------------------------- */


#include "Types.r"
#include "PrintingResTypes.r"


/*=================== 4-Up Code Segment and Jump Table Constants =================== */

#define	fourUpCodeSegmentID				0

#define	gxCountPagesOffset				4
#define	gxDespoolPageOffset				gxCountPagesOffset + 4


/*=================== Required Printing Extension Types and Resources =================== */



/*============== Printing Extension Resources ============== */


/*======= Printing Extension Version Resource ======= */

resource 'vers' (gxPrintingExtensionBaseID, purgeable) {
	0x01, 0x00, release, 0x00,
	verUS,
	"1.00",
	"1.00, Copyright \251 Apple Computer, Inc. 1989-1993"
};


/*======= Printing Extension Loading Priority ('load') Resource ======= */

resource gxExtensionLoadType (gxExtensionLoadID, purgeable, sysHeap)
{
	gxExtensionLoadAnywhere
};


/*======= Printing Extension Optimization ('eopt') Resource ======= */

resource gxExtensionOptimizationType (gxExtensionOptimizationID, purgeable, sysHeap)
{
	gxExecuteDuringImaging,
	gxDontNeedDeviceStatus,
	gxChangePageAtGXDespoolPage,
	gxDontChangePageAtGXImagePage,
	gxDontChangePageAtGXRenderPage,
	gxNotServerPresenceRequired,
	gxNotClientPresenceRequired
};


/*======= Scope ('scop') Resource ======= */

resource gxExtensionScopeType (gxDriverScopeID, purgeable, sysHeap)
{
	{
		'univ';		// fourUp has universal scope => compatible with all devices
	};
};


/*======= Message Override ('over') Resource ======= */

resource gxOverrideType (gxExtensionUniversalOverrideID, purgeable, sysHeap)
{
	{
		gxCountPages, fourUpCodeSegmentID, gxCountPagesOffset;
		gxDespoolPage, fourUpCodeSegmentID, gxDespoolPageOffset
	};
};


/*=================== Standard BNDL, ICN#, and FREF Resources =================== */


type '4-Up' as 'STR ';

resource '4-Up' (0, purgeable, sysHeap)
{
	"4-Up Printing Extension 1.0"
};


/* The 'BNDL', 'ICN#', and 'FREF' are needed so the extension's icon shows up. */

resource 'BNDL' (gxPrintingExtensionBaseID, purgeable, sysHeap)
{
	'4-Up', 0,
	{	'ICN#', {0, gxPrintingExtensionBaseID},
		'FREF', {0, gxPrintingExtensionBaseID}
	}
};

/*======= extension icon ======= */

resource 'ICN#' (gxPrintingExtensionBaseID, sysheap, purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"7FFF FFF0 8000 0008 8000 0008 8000 0008"
		$"81FE 7F88 8102 4088 817A 5888 8102 4088"
		$"A11A 5E88 D102 408A 9162 4F0D 1102 4089"
		$"117A 5E81 1102 4081 11FE 7F81 1000 0001"
		$"1000 0001 11FE 7F81 1102 4081 117A 5E81"
		$"1102 4089 9162 5E8D D102 408A A11A 4C88"
		$"8102 4088 817A 5E88 8102 4088 81FE 7F88"
		$"8000 0008 8000 0008 8000 0008 7FFF FFF0",
		/* [2] */
		$"7FFF FFF0 FFFF FFF8 FFFF FFF8 FFFF FFF8"
		$"FFFF FFF8 FFFF FFF8 FFFF FFF8 FFFF FFF8"
		$"FFFF FFF8 DFFF FFFA 9FFF FFFF 1FFF FFFF"
		$"1FFF FFFF 1FFF FFFF 1FFF FFFF 1FFF FFFF"
		$"1FFF FFFF 1FFF FFFF 1FFF FFFF 1FFF FFFF"
		$"1FFF FFFF 9FFF FFFF DFFF FFFA FFFF FFF8"
		$"FFFF FFF8 FFFF FFF8 FFFF FFF8 FFFF FFF8"
		$"FFFF FFF8 FFFF FFF8 FFFF FFF8 7FFF FFF0"
	}
};

resource 'icl4' (gxPrintingExtensionBaseID, sysheap, purgeable) {
	$"0FFF FFFF FFFF FFFF FFFF FFFF FFFF 0000"
	$"F000 0000 0000 0000 0000 0000 0000 F000"
	$"F000 0000 0000 0000 0000 0000 0000 F000"
	$"F000 0000 0000 0000 0000 0000 0000 F000"
	$"F000 000F FFFF FFF0 0FFF FFFF F000 F000"
	$"F000 000F 0000 00F0 0F00 0000 F000 F000"
	$"F000 000F 0FFF F0F0 0F0F F000 F000 F000"
	$"F000 000F 0000 00F0 0F00 0000 F000 F000"
	$"F0F0 000F 000F F0F0 0F0F FFF0 F000 F000"
	$"FF0F 000F 0000 00F0 0F00 0000 F000 F0F0"
	$"F00F 000F 0FF0 00F0 0F00 FFFF 0000 FF0F"
	$"000F 000F 0000 00F0 0F00 0000 F000 F00F"
	$"000F 000F 0FFF F0F0 0F0F FFF0 F000 000F"
	$"000F 000F 0000 00F0 0F00 0000 F000 000F"
	$"000F 000F FFFF FFF0 0FFF FFFF F000 000F"
	$"000F 0000 0000 0000 0000 0000 0000 000F"
	$"000F 0000 0000 0000 0000 0000 0000 000F"
	$"000F 000F FFFF FFF0 0FFF FFFF F000 000F"
	$"000F 000F 0000 00F0 0F00 0000 F000 000F"
	$"000F 000F 0FFF F0F0 0F0F FFF0 F000 000F"
	$"000F 000F 0000 00F0 0F00 0000 F000 F00F"
	$"F00F 000F 0FF0 00F0 0F0F FFF0 F000 FF0F"
	$"FF0F 000F 0000 00F0 0F00 0000 F000 F0F0"
	$"F0F0 000F 000F F0F0 0F00 FF00 F000 F000"
	$"F000 000F 0000 00F0 0F00 0000 F000 F000"
	$"F000 000F 0FFF F0F0 0F0F FFF0 F000 F000"
	$"F000 000F 0000 00F0 0F00 0000 F000 F000"
	$"F000 000F FFFF FFF0 0FFF FFFF F000 F000"
	$"F000 0000 0000 0000 0000 0000 0000 F000"
	$"F000 0000 0000 0000 0000 0000 0000 F000"
	$"F000 0000 0000 0000 0000 0000 0000 F000"
	$"0FFF FFFF FFFF FFFF FFFF FFFF FFFF"
};

resource 'icl8' (gxPrintingExtensionBaseID, sysheap, purgeable) {
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF 0000 0000"
	$"FF00 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 FF00 0000"
	$"FF00 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 FF00 0000"
	$"FF00 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 FF00 0000"
	$"FF00 0000 0000 00FF FFFF FFFF FFFF FF00"
	$"00FF FFFF FFFF FFFF FF00 0000 FF00 0000"
	$"FF00 0000 0000 00FF 0000 0000 0000 FF00"
	$"00FF 0000 0000 0000 FF00 0000 FF00 0000"
	$"FF00 0000 0000 00FF 00FF FFFF FF00 FF00"
	$"00FF 00FF FF00 0000 FF00 0000 FF00 0000"
	$"FF00 0000 0000 00FF 0000 0000 0000 FF00"
	$"00FF 0000 0000 0000 FF00 0000 FF00 0000"
	$"FF00 FF00 0000 00FF 0000 00FF FF00 FF00"
	$"00FF 00FF FFFF FF00 FF00 0000 FF00 0000"
	$"FFFF 00FF 0000 00FF 0000 0000 0000 FF00"
	$"00FF 0000 0000 0000 FF00 0000 FF00 FF00"
	$"FF00 00FF 0000 00FF 00FF FF00 0000 FF00"
	$"00FF 0000 FFFF FFFF 0000 0000 FFFF 00FF"
	$"0000 00FF 0000 00FF 0000 0000 0000 FF00"
	$"00FF 0000 0000 0000 FF00 0000 FF00 00FF"
	$"0000 00FF 0000 00FF 00FF FFFF FF00 FF00"
	$"00FF 00FF FFFF FF00 FF00 0000 0000 00FF"
	$"0000 00FF 0000 00FF 0000 0000 0000 FF00"
	$"00FF 0000 0000 0000 FF00 0000 0000 00FF"
	$"0000 00FF 0000 00FF FFFF FFFF FFFF FF00"
	$"00FF FFFF FFFF FFFF FF00 0000 0000 00FF"
	$"0000 00FF 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 00FF"
	$"0000 00FF 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 00FF"
	$"0000 00FF 0000 00FF FFFF FFFF FFFF FF00"
	$"00FF FFFF FFFF FFFF FF00 0000 0000 00FF"
	$"0000 00FF 0000 00FF 0000 0000 0000 FF00"
	$"00FF 0000 0000 0000 FF00 0000 0000 00FF"
	$"0000 00FF 0000 00FF 00FF FFFF FF00 FF00"
	$"00FF 00FF FFFF FF00 FF00 0000 0000 00FF"
	$"0000 00FF 0000 00FF 0000 0000 0000 FF00"
	$"00FF 0000 0000 0000 FF00 0000 FF00 00FF"
	$"FF00 00FF 0000 00FF 00FF FF00 0000 FF00"
	$"00FF 00FF FFFF FF00 FF00 0000 FFFF 00FF"
	$"FFFF 00FF 0000 00FF 0000 0000 0000 FF00"
	$"00FF 0000 0000 0000 FF00 0000 FF00 FF00"
	$"FF00 FF00 0000 00FF 0000 00FF FF00 FF00"
	$"00FF 0000 FFFF 0000 FF00 0000 FF00 0000"
	$"FF00 0000 0000 00FF 0000 0000 0000 FF00"
	$"00FF 0000 0000 0000 FF00 0000 FF00 0000"
	$"FF00 0000 0000 00FF 00FF FFFF FF00 FF00"
	$"00FF 00FF FFFF FF00 FF00 0000 FF00 0000"
	$"FF00 0000 0000 00FF 0000 0000 0000 FF00"
	$"00FF 0000 0000 0000 FF00 0000 FF00 0000"
	$"FF00 0000 0000 00FF FFFF FFFF FFFF FF00"
	$"00FF FFFF FFFF FFFF FF00 0000 FF00 0000"
	$"FF00 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 FF00 0000"
	$"FF00 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 FF00 0000"
	$"FF00 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 FF00 0000"
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF"
};

resource 'FREF' (gxPrintingExtensionBaseID, purgeable, sysHeap)
{
	'pext', 0, ""
};
