/* ------------------------------------------------------------------------------

	FILENAME
		Additions.r

	DESCRIPTION
		This file contains all of the resources used by the Additions printing extension.

	COPYRIGHT
		Copyright Apple Computer, Inc. 1991
		All rights reserved. 
	
	MODIFICATION HISTORY
		05/15/91			ALA				Initial Implementation


------------------------------------------------------------------------------- */


#include "PrintingResTypes.h"
#include "PrintingResTypes.r"


/*=================== Additions Code Segment and Jump Table Constants =================== */

#define	additionsCodeSegmentID					0

#define	additionsInitializeOffset				4
#define	additionsDespoolPageOffset			additionsInitializeOffset + 4
#define	additionsStartJobOffset					additionsDespoolPageOffset + 4
#define	additionsFinishJobOffset				additionsStartJobOffset + 4
#define	additionsRenderPageOffset				additionsFinishJobOffset + 4
#define	additionsPrintDlgOffset					additionsRenderPageOffset + 4
#define	additionsCleanupStartJobOffset	additionsPrintDlgOffset + 4
#define	additionsPanelFilterOffset			additionsCleanupStartJobOffset + 4
#define	additionsPanelEventOffset				additionsPanelFilterOffset + 4


/*=================== Required Printing Extension Types and Resources =================== */


/*======= Printing Extension Loading Priority ('load') Resource ======= */

resource extensionLoadType (extensionLoadID, purgeable, sysHeap)
{
	extensionLoadLast
};


/*======= Printing Extension Optimization ('eopt') Resource ======= */

resource extensionOptimizationType (extensionOptimizationID, purgeable, sysHeap)
{
	executeDuringImaging,
	dontNeedDeviceStatus,
	changePageAtDespoolPage,
	dontChangePageAtImagePage,
	changePageAtRenderPage
};


/*======= Scope ('scop') Resource ======= */

resource extensionScopeType (imagingSystemScopeID, purgeable, sysHeap)
{
	{
		'rast';		// Additions is compatible with all raster devices
		'post';		// Additions is compatible with all PostScript devices
	};
};


/*======= Message Override ('over') Resource ======= */

resource overrideType (extensionUniversalOverrideID, purgeable, sysHeap)
{
	{
		initialize,					additionsCodeSegmentID, additionsInitializeOffset;
		despoolPage,				additionsCodeSegmentID, additionsDespoolPageOffset;
		startJob,						additionsCodeSegmentID, additionsStartJobOffset;
		finishJob,					additionsCodeSegmentID, additionsFinishJobOffset;
		renderPage, 				additionsCodeSegmentID, additionsRenderPageOffset;
		jobPrintDialog, 		additionsCodeSegmentID, additionsPrintDlgOffset;
		cleanupStartJob, 		additionsCodeSegmentID, additionsCleanupStartJobOffset;
		filterPanelEvent, 	additionsCodeSegmentID, additionsPanelFilterOffset;
		handlePanelEvent, 	additionsCodeSegmentID, additionsPanelEventOffset
	};
};


/*=================== Standard BNDL, ICN#, and FREF Resources =================== */


type 'addi' as 'STR ';

resource 'addi' (0, purgeable, sysHeap) {
	"Additions Printing Extension 1.0d1"
};


/* The 'BNDL', 'ICN#', and 'FREF' are needed so the extension's icon shows up. */

resource 'BNDL' (printingExtensionBaseID, purgeable, sysHeap) {
	'addi', 0,
	{	'ICN#', {0, printingExtensionBaseID},
		'FREF', {0, printingExtensionBaseID}
	}
};

resource 'FREF' (printingExtensionBaseID, purgeable, sysHeap) {
	'pext', 0, ""
};

/*=================== extension icon =================== */

data 'ICN#' (printingExtensionBaseID, purgeable, sysHeap) {
	$"003F FC00 0040 0200 0020 0400 7FF0 0FFE"            /* .?...@... ...... */
	$"8000 0001 8000 0001 8003 FF81 8002 00C1"            /* .......... */
	$"87FF C0E1 8400 4021 85D7 4021 8501 4021"            /* ...@!@!.@! */
	$"857F F021 8540 1821 8440 1C21 8547 C421"            /* ..!@.!@.!G! */
	$"8440 04A1 8543 84A1 8540 04A1 854F E421"            /* @.C@.O.! */
	$"85C0 07E1 8440 0401 87C0 0401 8043 8401"            /* ..@....C. */
	$"8040 0401 8040 0401 807F FC01 8000 0001"            /* @..@........ */
	$"807F FE01 8080 0101 8040 0201 7FE0 07FE"            /* .....@...... */
	$"003F FC00 007F FE00 003F FC00 7FFF FFFE"            /* .?.......?...... */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FF80 01FF FFC0 03FF 7FE0 07FE"            /* .............. */
};

data 'icl8' (printingExtensionBaseID, purgeable, sysHeap) {
	$"0000 0000 0000 0000 0000 FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 00FF 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 FF00 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 FF00 F6F6 F6F6"            /* ................ */
	$"F6F6 F6F6 F6FF 0000 0000 0000 0000 0000"            /* ................ */
	$"00FF FFFF FFFF FFFF FFFF FFFF F6F6 F6F6"            /* ................ */
	$"F6F6 F6F6 FFFF FFFF FFFF FFFF FFFF FF00"            /* ................ */
	$"FF00 0000 0000 0000 0000 0000 F6F6 F6F6"            /* ................ */
	$"F6F6 F6F6 0000 0000 0000 0000 0000 00FF"            /* ................ */
	$"FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"            /* ................ */
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFF6 F6F6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 FFF5"            /* ................ */
	$"F5F5 F5F5 F5F5 F5F5 FFFF F6F6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF F5F5 F5F5 F5F5 FFFF FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF 00F6 00F6 00F6 00F6 00F6"            /* ................ */
	$"00FF F5F5 F5F5 F5F5 F5F5 FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF F6FF FFFF F6FF F6FF FFFF"            /* ................ */
	$"F6FF F5F5 F5F5 F5F5 F5F5 FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF 00FF 00F6 00F6 00F6 00FF"            /* ................ */
	$"00FF F5F5 F5F5 F5F5 F5F5 FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF F6FF F6FF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF F5F5 F5F5 F5F5 FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF 00FF 00FF 0000 0000 0000"            /* ................ */
	$"0000 00FF FFF5 F5F5 F5F5 FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF F600 F6FF 0000 0000 0000"            /* ................ */
	$"0000 00FF FFFF F5F5 F5F5 FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF 00FF 00FF 0000 00FF FFFF"            /* ................ */
	$"FFFF 0000 00FF F5F5 F5F5 FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF F600 F6FF 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF F5F5 FFF5 FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF 00FF 00FF 0000 0000 FFFF"            /* ................ */
	$"FF00 0000 00FF F5F5 FFF5 FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF F6FF F6FF 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF F5F5 FFF5 FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF 00FF 00FF 0000 FFFF FFFF"            /* ................ */
	$"FFFF FF00 00FF F5F5 F5F5 FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF F6FF FFFF 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF FFFF FFFF FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF 00F6 00FF 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF F6F6 F6F6 F6F6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF FFFF FFFF 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF F6F6 F6F6 F6F6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6F6 F6F6 F6FF 0000 0000 FFFF"            /* ................ */
	$"FF00 0000 00FF F6F6 F6F6 F6F6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6F6 F6F6 F6FF 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF F6F6 F6F6 F6F6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6F6 F6F6 F6FF 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF F6F6 F6F6 F6F6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6F6 F6F6 F6FF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF F6F6 F6F6 F6F6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6"            /* ................ */
	$"F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6F6 F6F6 F6FF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFF6 F6F6 F6F6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6F6 F6F6 FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF F6F6 F6F6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6F6 F6F6 F6FF 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 FFF6 F6F6 F6F6 F6F6 F6FF"            /* ................ */
	$"00FF FFFF FFFF FFFF FFFF FF00 0000 0000"            /* ................ */
	$"0000 0000 00FF FFFF FFFF FFFF FFFF FF00"            /* ................ */
};

data 'icl4' (printingExtensionBaseID, purgeable, sysHeap) {
	$"0000 0000 00FF FFFF FFFF FF00 0000 0000"            /* ................ */
	$"0000 0000 0F00 0000 0000 00F0 0000 0000"            /* ................ */
	$"0000 0000 00F0 C0C0 C0C0 CF00 0000 0000"            /* ........... */
	$"0FFF FFFF FFFF 0C0C 0C0C FFFF FFFF FFF0"            /* ................ */
	$"F000 0000 0000 C0C0 C0C0 0000 0000 000F"            /* ............ */
	$"F00C 0C0C 0C0C 0C0C 0C0C 0C0C 0C0C 0C0F"            /* ................ */
	$"F0C0 C0C0 C0C0 C0FF FFFF FFFF F0C0 C0CF"            /* ....... */
	$"F00C 0C0C 0C0C 0CFC 0C0C 0C0C FF0C 0C0F"            /* ................ */
	$"F0C0 CFFF FFFF FFFF FFC0 C0C0 FFF0 C0CF"            /* ......... */
	$"F00C 0FC0 00C0 00C0 0F0C 0C0C 0CFC 0C0F"            /* ............. */
	$"F0C0 CF0F FF0F CFFF CFC0 C0C0 C0F0 C0CF"            /* ...... */
	$"F00C 0FCF 00C0 00CF 0F0C 0C0C 0CFC 0C0F"            /* ............. */
	$"F0C0 CF0F CFFF FFFF FFFF C0C0 C0F0 C0CF"            /* ........ */
	$"F00C 0FCF 0F00 0000 000F FC0C 0CFC 0C0F"            /* ............... */
	$"F0C0 CF00 CF00 0000 000F FFC0 C0F0 C0CF"            /* ......... */
	$"F00C 0FCF 0F00 0FFF FF00 0F0C 0CFC 0C0F"            /* ............... */
	$"F0C0 CF00 CF00 0000 0000 0FC0 F0F0 C0CF"            /* .......... */
	$"F00C 0FCF 0F00 00FF F000 0F0C FCFC 0C0F"            /* ............... */
	$"F0C0 CF0F CF00 0000 0000 0FC0 F0F0 C0CF"            /* .......... */
	$"F00C 0FCF 0F00 FFFF FFF0 0F0C 0CFC 0C0F"            /* ............... */
	$"F0C0 CF0F FF00 0000 0000 0FFF FFF0 C0CF"            /* ............ */
	$"F00C 0FC0 0F00 0000 0000 0F0C 0C0C 0C0F"            /* ............... */
	$"F0C0 CFFF FF00 0000 0000 0FC0 C0C0 C0CF"            /* ......... */
	$"F00C 0C0C 0F00 00FF F000 0F0C 0C0C 0C0F"            /* ................ */
	$"F0C0 C0C0 CF00 0000 0000 0FC0 C0C0 C0CF"            /* ....... */
	$"F00C 0C0C 0F00 0000 0000 0F0C 0C0C 0C0F"            /* ................ */
	$"F0C0 C0C0 CFFF FFFF FFFF FFC0 C0C0 C0CF"            /* ....... */
	$"F00C 0C0C 0C0C 0C0C 0C0C 0C0C 0C0C 0C0F"            /* ................ */
	$"F0C0 C0C0 CFFF FFFF FFFF FFF0 C0C0 C0CF"            /* ........ */
	$"F00C 0C0C F000 0000 0000 000F 0C0C 0C0F"            /* ................ */
	$"F0C0 C0C0 CF00 0000 0000 00F0 C0C0 C0CF"            /* ........ */
	$"0FFF FFFF FFF0 0000 0000 0FFF FFFF FFF0"            /* ................ */
};

data 'ics#' (printingExtensionBaseID, purgeable, sysHeap) {
	$"07E0 FC3F 8001 81F1 8119 9F89 90A9 97C9"            /* ...?... */
	$"9479 9421 9DA1 8421 87E1 8001 87E1 FC3F"            /* y!!....? */
	$"07E0 FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FC3F"            /* ...............? */
};

data 'ics8' (printingExtensionBaseID, purgeable, sysHeap) {
	$"0000 0000 00FF FFFF FFFF FF00 0000 0000"            /* ................ */
	$"FFFF FFFF FFFF 0000 0000 FFFF FFFF FFFF"            /* ................ */
	$"FF00 0000 0000 F6F6 F6F6 F600 0000 00FF"            /* ................ */
	$"FF00 F6F6 F6F6 F6FF FFFF FFFF F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6F6 F6FF F6F6 F6FF FFF6 F6FF"            /* ................ */
	$"FF00 F6FF FFFF FFFF FFF6 F6F6 FFF6 F6FF"            /* ................ */
	$"FF00 F6FF F600 F600 FFF6 FFF6 FFF6 F6FF"            /* ................ */
	$"FF00 F6FF 00FF FFFF FFFF F6F6 FFF6 F6FF"            /* ................ */
	$"FF00 F6FF F6FF 0000 00FF FFFF FFF6 F6FF"            /* ................ */
	$"FF00 F6FF 00FF 00FF FF00 FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6FF FFFF 0000 0000 FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF 00FF FF00 FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF FFFF FFFF FFF6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6F6 F6F6 F6F6 F6F6 F6F6 F6FF"            /* ................ */
	$"FF00 F6F6 F6FF FFFF FFFF FFF6 F6F6 F6FF"            /* ................ */
	$"FFFF FFFF FFFF 0000 0000 FFFF FFFF FFFF"            /* ................ */
};

data 'ics4' (printingExtensionBaseID, purgeable, sysHeap) {
	$"0000 0FFF FFF0 0000 FFFF FF00 00FF FFFF"            /* ................ */
	$"F000 00C0 C0C0 000F F00C 0C0F FFFF 0C0F"            /* ............. */
	$"F0C0 C0CF C0CF F0CF F00F FFFF FC0C FC0F"            /* .......... */
	$"F0CF C00C F0F0 F0CF F00F 0FFF FF0C FC0F"            /* ............. */
	$"F0CF CF00 0FFF F0CF F00F 0F0F F0FC 0C0F"            /* ............. */
	$"F0CF FF00 00F0 C0CF F00C 0F0F F0FC 0C0F"            /* ............. */
	$"F0C0 CFFF FFF0 C0CF F00C 0C0C 0C0C 0C0F"            /* ............ */
	$"F0C0 CFFF FFF0 C0CF FFFF FF00 00FF FFFF"            /* ............ */
};



/*=================== panel icon =================== */

data 'ICN#' (printingExtensionBaseID + 1, sysHeap) {
	$"0000 0000 0000 0000 0007 FFE0 0004 0030"            /* ...............0 */
	$"0004 0028 0004 0024 7FFF E03E 4000 2002"            /* ...(...$...>@. . */
	$"5FAF A002 5000 A002 5000 A002 5000 A002"            /* _.P..P..P.. */
	$"40FF FC02 5080 0602 4080 0502 5083 F482"            /* @...P..@..P. */
	$"5080 07C2 4081 E04A 5080 004A 409F FE4A"            /* P.@.JP.J@.J */
	$"5080 0042 5080 007E 5080 0040 5E80 0040"            /* P.BP.~P.@^.@ */
	$"4080 0040 7F80 0040 0080 0040 0080 0040"            /* @.@..@..@..@ */
	$"0080 E040 0080 0040 0080 0040 00FF FFC0"            /* ..@..@..@... */
	$"0000 0000 0000 0000 0007 FFE0 0007 FFF0"            /* ................ */
	$"0007 FFF8 0007 FFFC 7FFF FFFE 7FFF FFFE"            /* ................ */
	$"7FFF FFFE 7FFF FFFE 7FFF FFFE 7FFF FFFE"            /* ................ */
	$"7FFF FFFE 7FFF FFFE 7FFF FFFE 7FFF FFFE"            /* ................ */
	$"7FFF FFFE 7FFF FFFE 7FFF FFFE 7FFF FFFE"            /* ................ */
	$"7FFF FFFE 7FFF FFFE 7FFF FFFE 7FFF FFC0"            /* ............... */
	$"7FFF FFC0 7FFF FFC0 7FFF FFC0 00FF FFC0"            /* ............ */
	$"00FF FFC0 00FF FFC0 00FF FFC0 00FF FFC0"            /* ............ */
};

data 'icl4' (printingExtensionBaseID + 1, sysHeap) {
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0FFF FFFF FFFF FFF0 0000"            /* ................ */
	$"0000 0000 0000 0F0C 0C0C 0C0C 0CFF 0000"            /* ................ */
	$"0000 0000 0000 0FC0 C0C0 C0C0 C0FC F000"            /* .......... */
	$"0000 0000 0000 0F0C 0C0C 0C0C 0CFC CF00"            /* ............... */
	$"0FFF FFFF FFFF FFFF FFF0 C0C0 C0FF FFF0"            /* ............. */
	$"0F00 0C00 0C00 0C00 0CFC 0C0C 0C0C 0CF0"            /* ................ */
	$"0F0F FFFF F0FC FFFF F0F0 C0C0 C0C0 C0F0"            /* ........... */
	$"0F0F 0C00 0C00 0C00 FCFC 0C0C 0C0C 0CF0"            /* ................ */
	$"0F0F 000C 000C 000C F0F0 C0C0 C0C0 C0F0"            /* ........... */
	$"0F0F 0C00 0C00 0C00 FCFC 0C0C 0C0C 0CF0"            /* ................ */
	$"0F0C 000C FFFF FFFF FFFF FFC0 C0C0 C0F0"            /* ............ */
	$"0F0F 0C00 F000 0000 0000 0FFC 0C0C 0CF0"            /* ................ */
	$"0F0C 000C F000 0000 0000 0F0F C0C0 C0F0"            /* ............. */
	$"0F0F 0C00 F000 00FF FFFF 0F00 FC0C 0CF0"            /* ................ */
	$"0F0F 000C F000 0000 0000 0FFF FFC0 C0F0"            /* .............. */
	$"0F00 0C00 F000 000F FFF0 0000 0F0C FCF0"            /* ................ */
	$"0F0F 000C F000 0000 0000 0000 0FC0 F0F0"            /* ............... */
	$"0F00 0C00 F00F FFFF FFFF FFF0 0F0C FCF0"            /* ................ */
	$"0F0F 000C F000 0000 0000 0000 0FC0 C0F0"            /* .............. */
	$"0F0F 0C00 F000 0000 0000 0000 0FFF FFF0"            /* ................ */
	$"0F0F 000C F000 0000 0000 0000 0F00 0000"            /* ................ */
	$"0F0F FFF0 F000 0000 0000 0000 0F00 0000"            /* ................ */
	$"0F0C 000C F000 0000 0000 0000 0F00 0000"            /* ................ */
	$"0FFF FFFF F000 0000 0000 0000 0F00 0000"            /* ................ */
	$"0000 0000 F000 0000 0000 0000 0F00 0000"            /* ................ */
	$"0000 0000 F000 0000 0000 0000 0F00 0000"            /* ................ */
	$"0000 0000 F000 0000 FFF0 0000 0F00 0000"            /* ................ */
	$"0000 0000 F000 0000 0000 0000 0F00 0000"            /* ................ */
	$"0000 0000 F000 0000 0000 0000 0F00 0000"            /* ................ */
	$"0000 0000 FFFF FFFF FFFF FFFF FF00 0000"            /* ................ */
};


data 'icl8' (printingExtensionBaseID + 1, sysHeap) {
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 00FF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF FF00 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 00FF F5F5"            /* ................ */
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FFFF 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 0000 0000 00FF F5F5"            /* ................ */
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FF2B FF00 0000"            /* ...........+.... */
	$"0000 0000 0000 0000 0000 0000 00FF F5F5"            /* ................ */
	$"F5F5 F5F5 F5F5 F5F5 F5F5 FF2B 2BFF 0000"            /* ...........++... */
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFF5 F5F5 F5F5 F5F5 FFFF FFFF FF00"            /* ................ */
	$"00FF F600 F600 F600 F600 F600 F600 F600"            /* ................ */
	$"F600 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 FF00"            /* ................ */
	$"00FF 00FF FFFF FFFF FFF6 FFF6 FFFF FFFF"            /* ................ */
	$"FFF6 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 FF00"            /* ................ */
	$"00FF F6FF F600 F600 F600 F600 F600 F600"            /* ................ */
	$"FF00 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 FF00"            /* ................ */
	$"00FF 00FF 00F6 00F6 00F6 00F6 00F6 00F6"            /* ................ */
	$"FFF6 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 FF00"            /* ................ */
	$"00FF F6FF F600 F600 F600 F600 F600 F600"            /* ................ */
	$"FF00 FFF5 F5F5 F5F5 F5F5 F5F5 F5F5 FF00"            /* ................ */
	$"00FF 00F6 00F6 00F6 FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF F5F5 F5F5 F5F5 F5F5 FF00"            /* ................ */
	$"00FF F6FF F600 F600 FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF FFF5 F5F5 F5F5 F5F5 FF00"            /* ................ */
	$"00FF 00F6 00F6 00F6 FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF F5FF F5F5 F5F5 F5F5 FF00"            /* ................ */
	$"00FF F6FF F600 F600 FF00 0000 0000 FFFF"            /* ................ */
	$"FFFF FFFF 00FF F5F5 FFF5 F5F5 F5F5 FF00"            /* ................ */
	$"00FF 00FF 00F6 00F6 FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 00FF FFFF FFFF F5F5 F5F5 FF00"            /* ................ */
	$"00FF F600 F600 F600 FF00 0000 0000 00FF"            /* ................ */
	$"FFFF FF00 0000 0000 00FF F5F5 FFF5 FF00"            /* ................ */
	$"00FF 00FF 00F6 00F6 FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 00FF F5F5 FFF5 FF00"            /* ................ */
	$"00FF F600 F600 F600 FF00 00FF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FF00 00FF F5F5 FFF5 FF00"            /* ................ */
	$"00FF 00FF 00F6 00F6 FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 00FF F5F5 F5F5 FF00"            /* ................ */
	$"00FF F6FF F600 F600 FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 00FF FFFF FFFF FF00"            /* ................ */
	$"00FF 00FF 00F6 00F6 FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 00FF 0000 0000 0000"            /* ................ */
	$"00FF F6FF FFFF FF00 FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 00FF 0000 0000 0000"            /* ................ */
	$"00FF 00F6 00F6 00F6 FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 00FF 0000 0000 0000"            /* ................ */
	$"00FF FFFF FFFF FFFF FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 00FF 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 00FF 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 00FF 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 FF00 0000 0000 0000"            /* ................ */
	$"FFFF FF00 0000 0000 00FF 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 00FF 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 FF00 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 00FF 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 0000 FFFF FFFF FFFF FFFF"            /* ................ */
	$"FFFF FFFF FFFF FFFF FFFF 0000 0000 0000"            /* ................ */
};

data 'ics#' (printingExtensionBaseID + 1, sysHeap) {
	$"0000 01FC 0106 7F82 4082 5E82 5286 57F6"            /* .......@^RW. */
	$"541A 5CCE 4408 7DE8 0408 04C8 0408 07F8"            /* T.\D.}........ */
	$"0000 01FC 0106 7F82 4082 5E82 5286 57F6"            /* .......@^RW. */
	$"541A 5CCE 4408 7DE8 0408 04C8 0408 07F8"            /* T.\D.}........ */
};

data 'ics4' (printingExtensionBaseID + 1, sysHeap) {
	$"0000 0000 0000 0000 0000 000F FFFF FF00"            /* ................ */
	$"0000 000F 0C0C 0FF0 0FFF FFFF F0C0 C0F0"            /* .............. */
	$"0FC0 00C0 FC0C 0CF0 0F0F FFF0 F0C0 C0F0"            /* ............ */
	$"0FCF 00F0 FC0C 0FF0 0F0F CFFF FFFF CFF0"            /* ............. */
	$"0FCF 0F00 000F F0F0 0F0F FF00 FF00 FFF0"            /* ............... */
	$"0FC0 0F00 0000 F000 0FFF FF0F FFF0 F000"            /* ............... */
	$"0000 0F00 0000 F000 0000 0F00 FF00 F000"            /* ................ */
	$"0000 0F00 0000 F000 0000 0FFF FFFF F000"            /* ................ */
};

data 'ics8' (printingExtensionBaseID + 1, sysHeap) {
	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
	$"0000 0000 0000 00FF FFFF FFFF FFFF 0000"            /* ................ */
	$"0000 0000 0000 00FF F5F5 F5F5 F5FF FF00"            /* ................ */
	$"00FF FFFF FFFF FFFF FFF5 F5F5 F5F5 FF00"            /* ................ */
	$"00FF F600 F600 F600 FFF5 F5F5 F5F5 FF00"            /* ................ */
	$"00FF 00FF FFFF FFF6 FFF5 F5F5 F5F5 FF00"            /* ................ */
	$"00FF F6FF F600 FF00 FFF5 F5F5 F5FF FF00"            /* ................ */
	$"00FF 00FF 00FF FFFF FFFF FFFF F5FF FF00"            /* ................ */
	$"00FF F6FF F6FF 0000 0000 00FF FF00 FF00"            /* ................ */
	$"00FF 00FF FFFF 0000 FFFF 0000 FFFF FF00"            /* ................ */
	$"00FF F600 F6FF 0000 0000 0000 FF00 0000"            /* ................ */
	$"00FF FFFF FFFF 00FF FFFF FF00 FF00 0000"            /* ................ */
	$"0000 0000 00FF 0000 0000 0000 FF00 0000"            /* ................ */
	$"0000 0000 00FF 0000 FFFF 0000 FF00 0000"            /* ................ */
	$"0000 0000 00FF 0000 0000 0000 FF00 0000"            /* ................ */
	$"0000 0000 00FF FFFF FFFF FFFF FF00 0000"            /* ................ */
};

/*=================== Print Dialog Panel Related Resources =================== */


/*======= Description of the panel resource added to Print Dialog by Additions extension ======= */

resource 'panl'  (printingExtensionBaseID, purgeable, sysHeap)
{
		"Additions", smRoman, printingExtensionBaseID + 1, printingExtensionBaseID
};


/*======= 'DITL' describing the items in the Addition's Print Dialog Panel ======= */

resource 'DITL' (printingExtensionBaseID, purgeable, sysHeap) {
	{	/* array DITLarray: 7 elements */
		/* [1] */
		{12, 23, 28, 203},
		StaticText {
			disabled,
			"Additions:"
		},

		/* [2] */
		{41, 33, 57, 136},
		Checkbox {
			enabled,
			"Page Border"
		},
				
		/* [3] */
		{70, 33, 86, 278},
		CheckBox {
			enabled,
			"Serialize all copies, starting with:"
		},

		/* [4] */
		{98, 33, 114, 133},
		CheckBox {
			enabled,
			"Cover page:"
		},
		
		/* [5] */
		{99, 138, 115, 153},	/* Represents selection of cover page first */
		RadioButton {
			enabled,
			""
		},
		
		/* [6] */
		{99, 203, 115, 218},	/* Represents selection of cover page last */
		RadioButton {
			enabled,
			""
		},
		
		/* [7] */
		{66, 288, 82, 338},	/* Starting copy number */
		EditText {
			enabled,
			""
		}
	}
};


/*======= 'xdtl' describing how to process the items in the  Filter's Print Dialog Panel ======= */

resource 'xdtl' (printingExtensionBaseID, purgeable, sysHeap)
{
	{	
		EditTextInteger	{ 'addi', printingTagID, 0, 7, 1, "", "" },		// Serial number (long)
		RadioButtons		{ 'addi', printingTagID, 4, {5,6} },					// Cover page radio buttons (short)
		CheckBox				{ 'addi', printingTagID, 5, 2 },							// Page Border
		CheckBox				{ 'addi', printingTagID, 6, 3 },							// Serialize Copies
		CheckBox				{ 'addi', printingTagID, 7, 4 }								// Cover Page
	};
};


/*======= Cover Page "on top" icon (displayed in the panel) ======= */

resource 'ICON' (printingExtensionBaseID, sysHeap) {	
		$"FFFF FE00 8000 0200 8000 0200 8000 0200"
		$"8000 03FF 8000 0201 8000 0201 8377 8201"
		$"8000 03B9 8010 0201 8000 02E9 80DE 0201"
		$"8074 03B9 8000 0201 8000 0301 8000 0201"
		$"8000 02E9 8000 0201 8000 0379 8000 0201"
		$"8000 03D9 8000 0201 86EF 4379 805C 0201"
		$"81B7 03D9 8000 0201 8000 03D9 FFFF FE01"
		$"0040 0001 0040 0801 0040 0001 007F FFFF"
};


/*======= Cover Page "on bottom" icon (displayed in the panel) ======= */

resource 'ICON' (printingExtensionBaseID + 1, sysHeap) {
		$"FFFF FE00 8000 0200 8000 0200 8000 0200"
		$"877B 73FF 8000 0201 9EDD D201 8000 0201"
		$"97AF 7201 8000 0201 9DD6 0201 8000 03C1"
		$"877D D201 8000 0201 9DDA F201 8000 0301"
		$"975F B201 8000 0201 9F6A F201 8000 0201"
		$"96FB B201 8000 0201 9B77 B201 8000 0201"
		$"8000 0201 8010 0201 8000 03A1 FFFF FE01"
		$"0040 DB81 0040 0001 0040 0001 007F FFFF"
};

