/*
	one rectangle ("GX").c
	
	This file contains the calls that an application needs to make for the "graphics shell" to work correctly.
	This "one rectangle" will draw 1 black rectangle ( {0, 0, ff(100), ff(100)} ) in the upper left corner of the window.
	
	NOTES:
	 This file requires the new "GX"-ified interface files and libraries. With the interface files, all types and function
	names are proceeded by a "gx" for types or "GX" for function names.
	
	 This file requires the following files to run correctly:
	    "graphics shell (GX).c", "graphics debug library.c", "transform library.c".
	    
	 This file prints the "best" in landscape mode.
	
	1992 - 1993  Apple Computer, Inc.
	All rights reserved.
*/

#include <events.h>
#include <windows.h>

#include "graphics libraries.h"
#include "graphics toolbox.h"
#include "graphics shell(GX).h"

/**  Set up the title and size of the window  **/
Str255 		gWindowTitle = "\p one rectangle (GX) ";
Rect 			gWindowQDRect  = {50, 50, 275, 275};

/** 
	If gDebugging = TRUE, graphics library errors and notices will be posted.  This functionality  will only work with 
	the "debugging" version of the Secret Graphics init.  If this version of the init is not installed, nothing bad will happen, 
	but these  functions will not work. The "debugging" version of the Secret  Graphics init is approximately 700K.
**/
Boolean		gDebugging = true;


/**  Set  "gGiveMeValidation" to TRUE, if you will receive run-time validation.  **/
Boolean		gGiveMeValidation = true;


/**	 
	gGraphicsHeapSize sets the size of the graphics heap created by calling the NewGraphicsClient routine
	in main () within graphics shell.c.  You can determine the amount of graphics heap required by using GraphicsBug.
	With  gGraphicsHeapSize set to 25k, I had 2 free blocks left in the graphics heap. Sounds good to me.
**/
long			gGraphicsHeapSize = 25;

gxShape 		gShape;


/*------ DoInitialization ---------------------------------------------------------------------------------*/

void DoInitialization(gWindow)
WindowPtr gWindow;
{
	gxRectangle 		rectData = {0, 0, ff(100), ff(100)};	

	gShape = GXNewRectangle(&rectData); 
}


/*------ DoDraw ---------------------------------------------------------------------------------------*/

void DoDraw(gWindow)
WindowPtr gWindow;
{
	GXDrawShape (gShape);
}


/*------ DoDispose -------------------------------------------------------------------------------------*/

void DoDispose(gWindow)
WindowPtr gWindow;
{
	/**  
		You should always dispose of your GX graphics objects before tossing your window. Why? It's generally good 
		form and this approach guarantees that everything is disposed. If you had not disposed of everything, the
		call to DisposeWindow should dispose of the objects. If you are running the debugging version of the 
		SecretGraphics init with notices set, you will receive a notice that you had not disposed of everything. You
		can turn notices on in this file by setting gDebugging = TRUE (above).
	**/
	GXDisposeShape(gShape);  
 	GXDisposeShape(gWindowBoundsShape);  
    	DisposeWindow(gWindow);
}
	


/*------ DoClick ---------------------------------------------------------------------------------------*/

void DoClick(gWindow)
WindowPtr gWindow;
{
}


/*------ DoIdle ----------------------------------------------------------------------------------------*/

void DoIdle(gWindow)
WindowPtr gWindow;
{
}
