/* graphics libraries:
  Perspective gxMapping generation routines
  by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
  Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.  */

#include "graphics libraries.h"
#include "camera library.h"


void InitCamera(camera *obscura)
{
  obscura->location.x = 0;
  obscura->location.y = 0;
  obscura->location.z = -GXIntToFixed(1440);  /* 20 gxInches backward */
  obscura->axis.x = 0;
  obscura->axis.y = 0;
  obscura->axis.z = fix1;       /* forward */
  obscura->zenith.x = 0;
  obscura->zenith.y = -fix1;      /* up */
  obscura->zenith.z = 0;
  obscura->observer.x = 0;
  obscura->observer.y = 0;
  obscura->observer.z = -GXIntToFixed(1440);  /* 20 gxInches backward */
  UpdateCamera(obscura);
}

void UpdateCamera(camera *obscura)
{
  unit3D temp[2];
  register unit3D *axis = temp;
  register unit3D *zenith = axis + 1;
  Unitize(&obscura->axis, axis);
  { register fixed dot = FracDot((unit3D *)&obscura->zenith, axis);
    zenith->x = obscura->zenith.x - GXFractMultiply(dot, axis->x);
    zenith->y = obscura->zenith.y - GXFractMultiply(dot, axis->y);
    zenith->z = obscura->zenith.z - GXFractMultiply(dot, axis->z);
    Unitize((point3D *)zenith, zenith);
  }
  { register unit3D *destPtr = (unit3D *)&obscura->orientation.map[0][0];
    FracCross(axis, zenith, destPtr++);
    *destPtr++ = *zenith;
    *destPtr++ = *axis;
  }
  { register fixed *destPtr = &obscura->orientation.map[0][0];
    register fixed x = obscura->observer.x;
    register fixed y = obscura->observer.y;
    register fixed z = obscura->observer.z;
    destPtr[0] = -GXFractMultiply(z, destPtr[0]) + GXFractMultiply(x, destPtr[6]);
    destPtr[1] = -GXFractMultiply(z, destPtr[1]) + GXFractMultiply(x, destPtr[7]);
    destPtr[2] = -GXFractMultiply(z, destPtr[2]) + GXFractMultiply(x, destPtr[8]);
    destPtr[3] = GXFractMultiply(z, destPtr[3]) + GXFractMultiply(y, destPtr[6]);
    destPtr[4] = GXFractMultiply(z, destPtr[4]) + GXFractMultiply(y, destPtr[7]);
    destPtr[5] = GXFractMultiply(z, destPtr[5]) + GXFractMultiply(y, destPtr[8]);
  }
}

void PatchToCameraMap(patch *quilt, camera *obscura, gxMapping *map)
{
  register fixed *destPtr = &map->map[0][0];
  register fixed *mapPtr = &obscura->orientation.map[0][0];
  register fixed *patchPtr;
  point3D diff;
  register fixed dot;

  diff.x = quilt->origin.x - obscura->location.x;
  diff.y = quilt->origin.y - obscura->location.y;
  diff.z = quilt->origin.z - obscura->location.z;
  dot = FracDot((unit3D *)&diff, (unit3D *)&obscura->orientation.map[2][0]);

  patchPtr = (fixed *)quilt;
  *destPtr++ = GXVectorMultiplyDivide(3, patchPtr, 1, mapPtr, 1, dot);
  *destPtr++ = GXVectorMultiplyDivide(3, patchPtr, 1, mapPtr+3, 1, dot);
  *destPtr++ = GXVectorMultiplyDivide(3, patchPtr, 1, mapPtr+6, 1, dot);
  patchPtr += 3;
  *destPtr++ = GXVectorMultiplyDivide(3, patchPtr, 1, mapPtr, 1, dot);
  *destPtr++ = GXVectorMultiplyDivide(3, patchPtr, 1, mapPtr+3, 1, dot);
  *destPtr++ = GXVectorMultiplyDivide(3, patchPtr, 1, mapPtr+6, 1, dot);
  patchPtr = (fixed *)&diff;
  *destPtr++ = GXVectorMultiplyDivide(3, patchPtr, 1, mapPtr, 1, dot);
  *destPtr++ = GXVectorMultiplyDivide(3, patchPtr, 1, mapPtr+3, 1, dot);
  *destPtr = frac1;
}

fixed Unitize(point3D *aPoint, unit3D *bPoint)
{
  register fixed r = GXMagnitude(GXMagnitude(aPoint->x, aPoint->y), aPoint->z);
  if (bPoint)
  { bPoint->x = GXFractDivide(aPoint->x, r);
    bPoint->y = GXFractDivide(aPoint->y, r);
    bPoint->z = GXFractDivide(aPoint->z, r);
  }
  return r;
}

fract FracDot(unit3D *aPoint, unit3D *bPoint)
{
  return GXFractMultiply(aPoint->x, bPoint->x) + GXFractMultiply(aPoint->y, bPoint->y) + GXFractMultiply(aPoint->z, bPoint->z);
}

void FracCross(unit3D *aPoint, unit3D *bPoint, unit3D *cPoint)
{
  unit3D temp;
  register fract *tempPtr = (fract *)&temp;
  *tempPtr++ = GXFractMultiply(aPoint->y, bPoint->z) - GXFractMultiply(aPoint->z, bPoint->y);
  *tempPtr++ = GXFractMultiply(aPoint->z, bPoint->x) - GXFractMultiply(aPoint->x, bPoint->z);
  *tempPtr++ = GXFractMultiply(aPoint->x, bPoint->y) - GXFractMultiply(aPoint->y, bPoint->x);
  *cPoint = temp;
}
