
/************************************************************

Created: Wednesday, August 14, 1991 at 12:09 PM
 Components.h
 C Interface to the Macintosh Libraries


 Copyright Apple Computer, Inc. 1990 - 1991
 All rights reserved

************************************************************/


#ifndef __COMPONENTS__
#define __COMPONENTS__

#ifndef __TYPES__
#include <Types.h>
#endif


#define gestaltComponentMgr 'cpnt'

#define gxKAnyComponentType     0
#define gxKAnyComponentSubType    0
#define gxKAnyComponentManufacturer 0
#define gxKAnyComponentFlagsMask    0

enum {
  gxCmpWantsRegisterMessage = 1L<<31
};

struct gxComponentDescription {
 OSType componentType;          /* A unique 4-byte code indentifying the command set */
 OSType componentSubType;       /* Particular flavor of this instance */
 OSType componentManufacturer;      /* Vendor indentification */
 unsigned long gxComponentFlags;      /* 8 each for gxComponent,Type,SubType,Manuf/revision */
 unsigned long componentFlagsMask;    /* Mask for specifying which flags to consider in search, zero during registration */
};

typedef struct gxComponentDescription gxComponentDescription;

struct gxResourceSpec {
 OSType resType;            /* 4-byte code  */
 short resId;             /*    */
};

typedef struct gxResourceSpec gxResourceSpec;

struct gxComponentResource {
 gxComponentDescription cd;       /* Registration parameters */
 gxResourceSpec component;        /* resource where gxComponent code is found */
 gxResourceSpec componentName;      /* name string resource */
 gxResourceSpec componentInfo;      /* info string resource */
 gxResourceSpec componentIcon;      /* icon resource */
};

typedef struct gxComponentResource gxComponentResource;
typedef gxComponentResource *gxComponentResourcePtr, **gxComponentResourceHandle;

/* Structure received by gxComponent:  */
struct gxComponentParameters {
 unsigned char flags;         /* call modifiers: sync/async, deferred, immed, etc */
 unsigned char paramSize;       /* size in bytes of actual parameters passed to this call */
 short what;              /* routine selector, negative for gxComponent management calls */
 long params[1];            /* actual parameters for the indicated routine */
};

typedef struct gxComponentParameters gxComponentParameters;


struct gxComponentRecord {
 long data[1];
};

typedef struct gxComponentRecord gxComponentRecord;
typedef gxComponentRecord *gxComponent;

struct gxComponentInstanceRecord {
 long data[1];
};

typedef struct gxComponentInstanceRecord gxComponentInstanceRecord;
typedef gxComponentInstanceRecord *gxComponentInstance;



typedef long gxComponentResult;

typedef pascal gxComponentResult (*gxComponentRoutine)
  (gxComponentParameters *tp, Handle componentStorage );

typedef pascal gxComponentResult (*gxComponentFunction)();





#define GXComponentCallNow( callNumber, paramSize ) \
  {0x2F3C,paramSize,callNumber,0x7000,0xA82A}

#ifdef __cplusplus
extern "C" {
#endif

/* 
 *******************************************************
 *                                                     *
 *              APPLICATION LEVEL CALLS                *
 *                                                     *
 *******************************************************
 * gxComponent Database Add, Delete, and Query Routines 
 *******************************************************
*/

pascal gxComponent GXRegisterComponent(gxComponentDescription *cd,gxComponentRoutine componentEntryPoint,
 short global,Handle componentName,Handle componentInfo,Handle componentIcon)
 = {0x7001,0xA82A}; 
pascal gxComponent GXRegisterComponentResource(gxComponentResourceHandle tr,short global)
 = {0x7012,0xA82A}; 
pascal OSErr GXUnregisterComponent(gxComponent aComponent)
 = {0x7002,0xA82A}; 

pascal gxComponent GXFindNextComponent(gxComponent aComponent,gxComponentDescription *looking)
 = {0x7004,0xA82A}; 
pascal long GXCountComponents(gxComponentDescription *looking)
 = {0x7003,0xA82A}; 

pascal OSErr GXGetComponentInfo(gxComponent aComponent,gxComponentDescription *cd,
 Handle componentName,Handle componentInfo,Handle componentIcon)
 = {0x7005,0xA82A}; 
pascal long GXGetComponentListModSeed(void)
 = {0x7006,0xA82A}; 


/* 
 *******************************************************
 * gxComponent Instance Allocation and dispatch routines 
 *******************************************************
*/

pascal gxComponentInstance GXOpenComponent(gxComponent aComponent)
 = {0x7007,0xA82A}; 
pascal OSErr GXCloseComponent(gxComponentInstance aComponentInstance)
 = {0x7008,0xA82A}; 

pascal OSErr GXGetComponentInstanceError(gxComponentInstance aComponentInstance)
 = {0x700A,0xA82A}; 

/*  direct calls to the Components  */
pascal long GXComponentFunctionImplemented(gxComponentInstance ci,short ftnNumber)
 = {0x2F3C,0x2,0xFFFD,0x7000,0xA82A}; 
pascal long GXGetComponentVersion(gxComponentInstance ci)
 = {0x2F3C,0x0,0xFFFC,0x7000,0xA82A}; 
#ifdef __cplusplus
}
#endif



/*****************************************************
*                                                    *
*               CALLS MADE BY Components             *
*                                                    *
******************************************************
******************************************************
* Required gxComponent routines
*******************************************************/


#define gxKComponentOpenSelect -1     /* gxComponentInstance for this open */
#define gxKComponentCloseSelect -2    /* gxComponentInstance for this close */
#define gxKComponentCanDoSelect -3    /* selector # being queried */
#define gxKComponentVersionSelect -4    /* no params */
#define gxKComponentRegisterSelect -5   /* no params */
#define gxKComponentTargetSelect -6   /* gxComponentInstance for top of call chain */

#ifdef __cplusplus
extern "C" {
#endif

/* 
 *******************************************************
 * gxComponent Management routines
 *******************************************************
*/

pascal void GXSetComponentInstanceError(gxComponentInstance aComponentInstance,
 OSErr theError)
 = {0x700B,0xA82A}; 

pascal long GXGetComponentRefcon(gxComponent aComponent)
 = {0x7010,0xA82A}; 
pascal void GXSetComponentRefcon(gxComponent aComponent,long theRefcon)
 = {0x7011,0xA82A}; 

pascal short GXOpenComponentResFile(gxComponent aComponent)
 = {0x7015,0xA82A}; 
pascal OSErr GXCloseComponentResFile(short refnum)
 = {0x7018,0xA82A}; 


/* 
 *******************************************************
 * gxComponent Instance Management routines
 *******************************************************
*/

pascal Handle GXGetComponentInstanceStorage(gxComponentInstance aComponentInstance)
 = {0x700C,0xA82A}; 
pascal void GXSetComponentInstanceStorage(gxComponentInstance aComponentInstance,
 Handle theStorage)
 = {0x700D,0xA82A}; 

pascal long GXGetComponentInstanceA5(gxComponentInstance aComponentInstance)
 = {0x700E,0xA82A}; 
pascal void GXSetComponentInstanceA5(gxComponentInstance aComponentInstance,
 long theA5)
 = {0x700F,0xA82A}; 

pascal long GXCountComponentInstances(gxComponent aComponent)
 = {0x7013,0xA82A}; 

/*  useful helper routines for convenient method dispatching  */
pascal long GXCallComponentFunction(gxComponentParameters *params,gxComponentFunction func)
 = {0x70FF,0xA82A}; 
pascal long GXCallComponentFunctionWithStorage(Handle storage,gxComponentParameters *params,
 gxComponentFunction func)
 = {0x70FF,0xA82A}; 
pascal long GXDelegateComponentCall(gxComponentParameters *originalParams,gxComponentInstance ci)
 = {0x7024,0xA82A}; 
#ifdef __cplusplus
}
#endif
enum {


/* Set Default gxComponent flags */
 gxDefaultComponentIdentical = 0,
 gxDefaultComponentAnyFlags = 1,
 gxDefaultComponentAnyManufacturer = 2,
 gxDefaultComponentAnySubType = 4

#define gxDefaultComponentAnyFlagsAnyManufacturer gxDefaultComponentAnyFlags+gxDefaultComponentAnyManufacturer
#define gxDefaultComponentAnyFlagsAnyManufacturerAnySubType gxDefaultComponentAnyFlags+gxDefaultComponentAnyManufacturer+gxDefaultComponentAnySubType
};

#ifdef __cplusplus
extern "C" {
#endif
pascal OSErr GXSetDefaultComponent(gxComponent aComponent,short flags)
 = {0x701E,0xA82A}; 
pascal gxComponentInstance GXOpenDefaultComponent( OSType componentType, OSType componentSubType )
 = {0x7021,0xA82A}; 
pascal gxComponent GXCaptureComponent(gxComponent capturedComponent,gxComponent capturingComponent)
 = {0x701C,0xA82A}; 
pascal OSErr GXUncaptureComponent(gxComponent aComponent)
 = {0x701D,0xA82A}; 
#ifdef __cplusplus
}
#endif

enum {          /* errors from component manager & components  */
  gxInvalidComponentID = -3000,
  gxValidInstancesExist = -3001,
  gxComponentNotCaptured = -3002,
  gxComponentDontRegister = -3003
};

#define gxBadComponentInstance  0x80008001
#define gxBadComponentSelector  0x80008002


#endif
