/* ------------------------------------------------------------------------------

  FILENAME
    PrintingDrivers.h

  DESCRIPTION
    This file defines data types and API functions for
    printer driver development.

  COPYRIGHT
    Copyright  Apple Computer, Inc. 1989, 1990, 1991, 1992, 1993
    All rights reserved. 
  
------------------------------------------------------------------------------- */

#ifndef __PRINTINGDRIVERS__
#define __PRINTINGDRIVERS__


#ifndef __PRINTINGMESSAGES__
  #include <PrintingMessages.h>
#endif


/* ------------------------------------------------------------------------------

            Printing Driver Constants and Types

-------------------------------------------------------------------------------- */


enum {
  gxInputTraysMenuItem = -1     // Menu item number for "Input Trays..."
};

enum {                  // bit fields in communicationsOptions field of IOPrefsRec
  gxUseCustomIO = 0x00000001      // driver uses a non-standard IO mechanism
};

typedef struct {            // buffering and IO preferences
  unsigned long communicationsOptions;
  unsigned long numBuffers;
  unsigned long bufferSize;
  unsigned long numReqBlocks;
  unsigned long openCloseTimeout;
  unsigned long readWriteTimeout;
} IOPrefsRec, *IOPrefsPtr, **IOPrefsHdl;


typedef struct {            // information about writing to a file
  Boolean   printToFile;      // true if output is to go to a file
  FSSpec    fSpec;          // if going to a file, the FSSpec for the file
  Boolean   includeFonts;     // true if fonts are to be included
  Str31     fileFormat;       // format to write file
} PrintDestinationRec, *PrintDestinationPtr, **PrintDestinationHdl;


// this structure is the content of each cell in the standard PACK LDEF

typedef struct {
  char  firstMarker, secondMarker;    // markers to indicate icon or non-icon version
                          // if these are  and , then the cell is an icon cell.
                          // Otherwise, it is assumed to be a standard text LDEF
                          // cell
  
  Handle  iconSuiteHandle;        // the icon suite to draw for this cell
  Handle  outputDriverName;       // handle to the output driver name (for serial)
  Handle  inputDriverName;        // handle to the input driver name (for serial)
  Str255  iconName;           // name to draw under the icon
} PortListRec, *PortListPtr;


/* ------------------------------------------------------------------------------

            Printing Driver Constants for resources in the desktop printer

-------------------------------------------------------------------------------- */

// ---------------------------------- 'comm' ----------------------------------

// The communications method and private data used to connect to the printer
#define gxDeviceCommunicationsType      'comm'
#define gxDeviceCommunicationsID      (0)

// ---------------------------------- 'prod' ----------------------------------

// for PostScript devices, the device and version names of the device.
// (0) product name is of type PString
// (1) version is of type PString
// (2) revision is of type PString
// (3) vm available is of type long

#define gxPostscriptProductInfoType 'prod'
#define gxPostscriptProductNameID   (0)
#define gxPostscriptVersionID     (1)
#define gxPostscriptRevisionID      (2)
#define gxPostscriptVMAvailableID   (3)


/* ------------------------------------------------------------------------------

            Printing Driver Constants for status alerts

-------------------------------------------------------------------------------- */

// the ditl id for the auto feed button in the manual feed alert

enum {                  
  gxAutoFeedButtonId  = 3
};


// status resource id for the alerts

enum {                  
  gxUnivAlertStatusResourceId = -28508
};


// status resource indices for the alerts

enum {
  gxUnivManualFeedIndex     = 2,
  gxUnivFailToPrintIndex  = 3,
  gxUnivPaperJamIndex     = 4,
  gxUnivOutOfPaperIndex     = 5,
  gxUnivNoPaperTrayIndex  = 6,
  gxUnivPrinterReadyIndex   = 7,
  gxUnivAlertBeforeIndex  = 9,
  gxUnivAlertAfterIndex     = 10
};

// allocation sizes for status buffers needed for automatic alerts


enum {                  
  gxDefaultStatusBufferSize   = 10,
  gxManualFeedStatusBufferSize = 34,
  gxOutOfPaperStatusBufferSize = 42
};


// structures for status buffers

typedef struct {              // structure passed in statusBuffer of gxStatusRecord for manual feed alert
  Boolean canAutoFeed;        // true if driver can switch to auto feed
  Str31   paperTypeName;        // name of paperType to feed manually
} gxManualFeedRecord;


typedef struct {              // structure passed in statusBuffer of gxStatusRecord for out of paper alert
  Str31   paperTypeName;        // name of printing document
} gxOutOfPaperRecord;


/* ------------------------------------------------------------------------------

                Old Application Support

-------------------------------------------------------------------------------- */


// the format of a 'cust' resource 

typedef struct {
  short horizontalResolution;
  short verticalResolution;
  short upDriverType;
  Point patternStretch;
  short translatorSettings;
} CustomizationRec, *CustomizationPtr, **CustomizationHdl;
  

// the format of a 'resl' resource

typedef struct {
  short   rangeType;        // always 1
  short   xMinimumResolution;
  short   xMaximumResolution;
  short   yMinimumResolution;
  short   yMaximumResolution;
  short   resolutionCount;
  Point   resolutions[1];
} ResolutionRec, *ResolutionPtr, **ResolutionHdl;


// constants for the "universal" print record...  


enum {                  // version number for "universal" print record
  gxPrintRecordVersion  = 8
};

enum {                  // for calculationMode field...
  gxEmulation       = 0,
  gxTheoretical       = 1
};

enum {                  // for feed field...
  gxAutoFeed          = 0,
  gxManualFeed        = 1
};

enum {                  // bit fields for options field...
  gxPreciseBitmap     = 0x0001, // tall adjusted (IW), precise gxBitmap (LW, SC)
  gxBiggerPages       = 0x0002, // no gaps (IW), larger print area (LW)
  gxGraphicSmoothing    = 0x0004, // graphic smoothing (LW)
  gxTextSmoothing     = 0x0008, // text smoothing (SC)
  gxFontSubstitution    = 0x0010, // gxFont substitution
  gxInvertPage        = 0x0020, // b/w invert image
  gxFlipPageHoriz     = 0x0040, // flip horizontal
  gxFlipPageVert      = 0x0080, // flip vertical
  gxColorMode       = 0x0100, // gxColor printing
  gxBidirectional     = 0x0200, // gxBidirectional printing
  gxUserFlag0       = 0x0400, // user flag 0
  gxUserFlag1       = 0x0800, // user flag 1
  gxUserFlag2       = 0x1000, // user flag 2
  gxReservedFlag0     = 0x2000, // reserved flag 0
  gxReservedFlag1     = 0x4000, // reserved flag 1
  gxReservedFlag2     = 0x8000    // reserved flag 2
};

enum {                  // for orientation field...
  gxPortraitOrientation   = 0,
  gxLandscapeOrientation    = 1,
  gxAltPortraitOrientation  = 2,
  gxAltLandscapeOrientation = 3
};

enum {                  // for qualityMode field...
  gxBestQuality       = 0,
  gxFasterQuality     = 1,
  gxDraftQuality      = 2
};

enum {                  // for gxFirstTray and remainingTray fields...
  gxFirstTray       = 0,
  gxSecondTray        = 1,
  gxThirdTray       = 2
};

enum {                  // for coverPage field...
  gxNoCoverPage       = 0,
  gxFirstPageCover      = 1,
  gxLastPageCover     = 2
};

enum {                  // for headMotion field...
  gxUnidirectionalMotion  = 0,
  gxBidirectionalMotion = 1
};

enum {                  // for saveFile field...
  gxNoFile          = 0,
  gxPostScriptFile      = 1
};


// the format of the "universal" print record

typedef struct {
  short       prVersion;        // print record version 7.5 == 0
  
                          // prInfo subrecord...
  
  short       appDev;         // device kind, always 0
  short       appVRes;          // application vertical resolution
  short       appHRes;          // application horizontal resolution
  Rect        appPage;          // page size, in application resolution
  
  Rect        appPaper;           // paper gxRectangle [offset from rPage]
  
                        // prStl subrecord...
                    
  short       devType;          // device type, always 0 (was wDev)
  short       pageV;          // page height in 120ths of an inch
  short       pageH;          // page width in 120ths of an inch
  char        calculationMode;    // page calculation mode
  char        feed;           // feed mode
  
                        // prInfoPT subrecord...
                    
  short       devKind;          // device kind, always 0
  short       devVRes;          // device vertical resolution
  short       devHRes;          // device horizontal resolution
  Rect        devPage;          // device page size
  
                        // prXInfo subrecord...
                    
  short       actualCopies;     // actual number of copies for this job
  short       options;          // options for this device
  short       reduction;        // reduce/enlarge factor
  
  char        orientation;      // orientation of paper ( 0=portrait, 1=landscape &c )
  
                        // clusters...
                    
  char        qualityMode;      // quality mode
  char        gxFirstTray;        // first feed tray
  char        remainingTray;      // remaining feed tray
  char        coverPage;        // cover page
  char        headMotion;       // head motion
  char        saveFile;       // save file
  
  char        userCluster1;   // three clusters left over...
  char        userCluster2;
  char        userCluster3;

                        // prJob subrecord...
                  
  short       firstPage;        // first page
  short       lastPage;       // last page
  short       copies;         // # of copies, always 1
  char        reserved1;        // always true, unused
  char        reserved2;        // always true, unused
  PrIdleProcPtr pIdleProc;        // idle proc
  Ptr       pFileName;        // spool file name pointer
  short       fileVol;          // spool file vRefNum
  char        fileVers;       // file version, must be 0
  char        reserved3;        // always 0
  
  short       printX[19];          // internal use
} UniversalPrintRecord, *UniversalPrintRecordPtr, **UniversalPrintRecordHdl;
    


/* ------------------------------------------------------------------------------

              Compatibility Printing Messages

-------------------------------------------------------------------------------- */


#ifdef __cplusplus
extern "C" {
#endif


OSErr Forward_PrOpenDoc (THPrint, TPPrPort *)
    = gxFORWARDGLUE;

OSErr Forward_PrCloseDoc (TPPrPort)
    = gxFORWARDGLUE;

OSErr Forward_PrOpenPage (TPPrPort, TPRect, Point resolution)
    = gxFORWARDGLUE;

OSErr Forward_PrClosePage (TPPrPort)
    = gxFORWARDGLUE;

OSErr Forward_PrintDefault (THPrint)
    = gxFORWARDGLUE;

OSErr Forward_PrStlDialog (THPrint, Boolean *)
    = gxFORWARDGLUE;

OSErr Forward_PrJobDialog (THPrint, Boolean *)
    = gxFORWARDGLUE;

OSErr Forward_PrStlInit (THPrint, TPPrDlg *)
    = gxFORWARDGLUE;

OSErr Forward_PrJobInit (THPrint, TPPrDlg *)
    = gxFORWARDGLUE;

OSErr Forward_PrDlgMain (THPrint, PDlgInitProcPtr, Boolean *)
    = gxFORWARDGLUE;

OSErr Forward_PrValidate (THPrint, Boolean *)
    = gxFORWARDGLUE;

OSErr Forward_PrJobMerge (THPrint, THPrint)
    = gxFORWARDGLUE;

OSErr Forward_PrGeneral (Ptr)
    = gxFORWARDGLUE;

OSErr Forward_ConvertPrintRecordTo (THPrint)
    = gxFORWARDGLUE;

OSErr Forward_ConvertPrintRecordFrom (THPrint)
    = gxFORWARDGLUE;

OSErr Forward_PrintRecordToJob (THPrint, gxJob)
    = gxFORWARDGLUE;


#ifdef __cplusplus
}
#endif



/* ------------------------------------------------------------------------------

            Raster Driver Contants and Types

-------------------------------------------------------------------------------- */



typedef long gxRasterPlaneOptions;

enum {                    // bit fields in gxRasterPlaneOptions
  gxDefaultOffscreen    = 0x00000000, // default value - bits are allocated for the client, halftoning takes place
  gxDontSetHalftone   = 0x00000001, // don't call GXSetViewPortHalftone
  gxDotTypeIsDitherLevel  = 0x00000002  // call GXSetViewPortDither using the gxDotType as the level
};

typedef struct {              // input structure for setting up the offscreen
  gxRasterPlaneOptions  planeOptions; // options for the offscreen package
  gxHalftone          planeHalftone;  // OPTIONAL: gxHalftone structure for this plane
  gxColorSpace        planeSpace;   // OPTIONAL: gxNoSpace will get the graphics default
  gxColorSet          planeSet;   // OPTIONAL: NIL gets the default
  gxColorProfile      planeProfile; // OPTIONAL: NIL gets no matching
} gxPlaneSetupRec;
  
typedef struct {
  short         width;        // Width in pixels
  short         minHeight;      // minimum height in pixels - actual height returned here
  short         maxHeight;      // maximum height in pixels
  Fixed         ramPercentage;    // maximum percentage of RAM to take
  long          ramSlop;        // amount of RAM to be sure to leave
  short         depth;        // Depths in bits of each plane
  gxMapping       vpMapping;      // gxMapping to assign to offscreen viewPorts
  gxMapping       vdMapping;      // gxMapping to assign to offscreen viewDevices
  short         planes;       // # of planes to allocate of depth bits each (can be more than 4)
  gxPlaneSetupRec   planeSetup[4];    // parameters for each plane, 4 is provided because it is most handy for writers of devices
} gxOffscreenSetupRec;



typedef struct {              // the format of one plane in the offscreen planar area
  gxViewPort      theViewPort;      // gxViewPort for the offscreen
  gxViewDevice    theDevice;        // gxViewDevice for the offscreen
  gxViewGroup   theViewGroup;     // the gxViewGroup that they share
  gxShape       theBitmap;        // the offscreen gxBitmap gxShape
  gxBitmap      theBits;          // the bits of the offscreen
} gxOffscreenPlaneRec;

typedef struct {              // an entire offscreen area
  short         numberOfPlanes; // number of planes we have
  Handle        offscreenStorage; // handle containing the bitmaps image data
  gxOffscreenPlaneRec thePlanes[1];   // planes to draw in
} OffscreenRec, *OffscreenPtr, **OffscreenHdl;



typedef long gxRasterRenderOptions;

enum {                    // bit fields in gxRasterRenderOptions
  gxDefaultRaster         = 0x00000000, // default raster options
  gxDontResolveTransferModes    = 0x00000001, // 0=Resolve, 1=Don't Resolve
  gxRenderInReverse       = 0x00000002, // traverse image in reverse order
  gxOnePlaneAtATime       = 0x00000004, // render each plane separately
  gxSendAllBands          = 0x00000008  // send even empty bands
};


typedef struct {              // structure of resource to control raster imaging
  gxRasterRenderOptions renderOptions;  // options for the raster imaging system
  Fixed           hImageRes;    // horizontal resolution to image at
  Fixed           vImageRes;    // vertical resolution to image at
  short           minBandSize;  // minimum band size to use (in pixels)
  short           maxBandSize;  // maximum band size to use (in pixels), 0 == entire page
  Fixed           ramPercentage;  // maximum percentage of RAM to take
  long            ramSlop;      // amount of RAM to be sure to leave
  short           depth;      // depth in pixels (PER PLANE!)
  short           numPlanes;    // number of planes to render
  gxPlaneSetupRec     planeSetup[1];  // one for each plane
} RasterPrefsRec, *RasterPrefsPtr, **RasterPrefsHdl;


typedef long gxRasterPackageOptions;

enum {                    // bit fields in gxRasterPackageOptions
  gxSendAllColors = 0x00000001,               // send even clean bands through
  gxInterlaceColor  = 0x00000002,               // ribbon contamination is a concern
  gxOverlayColor  = 0x00000004,               // gxColor printer without a ribbon problem
  gxUseColor      = (gxInterlaceColor|gxOverlayColor)   // this is a gxColor printer
};


typedef struct {              // resource to control the RasterDataIn message
  Ptr           bufferSize;     // buffer size for packaging (>= maximum head pass size)
  short           colorPasses;    // 1 (b/w) or 4 (CMYK) is typical
  short           headHeight;     // printhead height in pixels
  short           numberPasses;   // number of head passes it takes to == iHeadHeight
  short           passOffset;     // offset between passes, in pixels
  gxRasterPackageOptions  packageOptions; // packaging options
} RasterPackageRec, *RasterPackagePtr, **RasterPackageHdl;


typedef struct {                      // for RasterPackageBitmap message
  gxBitmap              *bitmapToPackage;   // gxBitmap containing the data to package
  unsigned short          startRaster;      // raster to begin the packaging from
  unsigned short          colorBand;        // for which gxColor pass this is a packaging request
  Boolean             isBandDirty;      // whether there are any dirty bits in this band
  Rect                dirtyRect;        // which bits are dirty
} gxRasterPackageBitmapRec;


typedef struct  {             // for outputing numbers
  short numberType;             // type of numberic output desired
  short minWidth;             // minimum output width of the number
  char  padChar;                // pad character for numbers shorter than the minWidth
  char  alignment;              // whee!
  Str31 startString;            // prefix string
  Str31 endString;              // postfix string
} StandardNumberRec, *StandardNumberPtr;



typedef struct {              // structure of resource to control other messages
  short         startPageStringID;  // 'wstr' to send to the device at start of page
  short         formFeedStringID;   // 'wstr' to send to the device to cause a form feed
  short         forwardMax;       // gxLine feed strings
  StandardNumberRec forwardLineFeed;
  short         reverseMax;
  StandardNumberRec reverseLineFeed;
} RasterPackageControlsRec, *RasterPackageControlsPtr, **RasterPackageControlsHdl;



typedef struct {              // raster imaging system imageData structure

  // setup values

  gxRasterRenderOptions     renderOptions;    // options for the raster imaging system
  Fixed               hImageRes;      // horizontal resolution to image at
  Fixed               vImageRes;      // vertical resolution to image at
  short               minBandSize;    // smallest band that makes sense for this device
  short               maxBandSize;    // biggest band that makes sense, or 0 for "full page"
  gxRectangle           pageSize;     // size of page for device
  
  // values used within the RasterDataIn message
  
  short               currentYPos;    // current position moving down the page
  RasterPackageRec        packagingInfo;    // raster packaging record
  
  // values used within the remaining messages
  
  Boolean             optionsValid;   // were options specified by the driver?
  RasterPackageControlsRec  packageControls;  // options for the packaging messages

  gxOffscreenSetupRec       theSetup;     // setup for the offscreen code, variable length componant
} RasterImageDataRec, *RasterImageDataPtr, **RasterImageDataHdl;



/* ------------------------------------------------------------------------------

                Raster Driver Imaging Messages

-------------------------------------------------------------------------------- */


#ifdef __cplusplus
extern "C" {
#endif


#define GXRASTERMSGGLUE(selector) SendMessageGlue(0x0001, selector)
#define gxFORWARDGLUE ForwardThisMessageGlue


OSErr Send_RasterDataIn (OffscreenHdl, gxRectangle *bandRectangle, gxRectangle *dirtyRectangle)
    = GXRASTERMSGGLUE(gxRasterDataIn);
OSErr Forward_RasterDataIn (OffscreenHdl, gxRectangle *bandRectangle, gxRectangle *dirtyRectangle)
    = gxFORWARDGLUE;
    
OSErr Send_RasterLineFeed (short *lineFeedSize, Ptr buffer, unsigned long *bufferPos, RasterImageDataHdl)
    = GXRASTERMSGGLUE(gxRasterLineFeed);
OSErr Forward_RasterLineFeed (short *lineFeedSize, Ptr buffer, unsigned long *bufferPos, RasterImageDataHdl)
    = gxFORWARDGLUE;
  
OSErr Send_RasterPackageBitmap (gxRasterPackageBitmapRec *whatToPackage, Ptr buffer,
                      unsigned long *bufferPos, RasterImageDataHdl)
    = GXRASTERMSGGLUE(gxRasterPackageBitmap);
OSErr Forward_RasterPackageBitmap (gxRasterPackageBitmapRec *whatToPackage, Ptr buffer,
                          unsigned long *bufferPos, RasterImageDataHdl)
    = gxFORWARDGLUE;


#ifdef __cplusplus
}
#endif



/* ------------------------------------------------------------------------------

            Vector Driver Contants and Types

-------------------------------------------------------------------------------- */



typedef long gxVectorRenderOptions;

enum {                // bit fields in gxVectorRenderOptions
  gxColorSort   = 0x00000001, // set for pen plotters. 
  gxATransferMode = 0x00000002, // set if transfer modes need to be resolved 
  gxNoOverlap   = 0x00000004, // set if non-overlapping output is desired
  gxAColorBitmap  = 0x00000008, // set if gxColor gxBitmap output is desired
  gxSortbyPenPos  = 0x00000010, // set if shapes are to be drawn in the order of the pen index
                      // in the pen table. NOTE: this is not the pen position in the carousel.
  gxPenLessPlotter  = 0x00000020, // indicates raster printer/plotter
  gxCutterPlotter = 0x00000040, // indicates cutter
  gxNoBackGround  = 0x00000080  // set if shapes that map to the background gxColor should not be sent to driver 
};

      
typedef long gxVectorShapeOptions;

enum {                    // bit fields in gxVectorShapeOptions
  gxUnidirectionalFill    = 0x00000001, // generate scanlines in one direction only.. useful for transparencies
  gxAlsoOutlineFilledShape  = 0x00000002  // turn on this bit to also outline solid filled shapes 
};


typedef struct {
  fixed   angle;        // angle to gxHalftone at. Must be 0, 90, 45 or 135
  long    penIndex;     // index of the pen to draw this component with
} gxVHalftoneCompRec;
  
  
typedef struct {
  gxColorSpace      halftoneSpace;
  gxVHalftoneCompRec  halftoneComps[4]; // info for each gxColor component
  long          penIndexForBW;  // pen index to draw one bit deep or black and white gxBitmap with
} gxVHalftoneRec;


typedef struct {
  gxVectorShapeOptions  shapeOptions;   // options to control gxShape handling
  long            maxPolyPoints;  // maximum number of gxPolygon points that device can support
  fixed           shapeError;     // defines allowed deviation from the original gxShape
  fixed           textSize;     // text above this size is filled; text below this size is outlined
  fixed           frameSize;        // frame's smaller than this -> gxShape stroked; frame's larger -> gxShape is filled
} gxVectorShapeDataRec;


typedef struct {                // vector imaging system imageData structure
  gxVectorRenderOptions renderOptions;    // options to control rendering: gxColor sort, clipping, etc.
  fixed           devRes;       // device resolution
  gxTransform         devTransform;   // gxMapping, clip and halftoning information for colored bitmaps
  gxColorSet          clrSet;       // entire set of colors; usually indexed gxColor space for pen plotters
  gxColor           bgColor;        // the background gxColor in the gxColor space specified by the clrSpace field
  gxVHalftoneRec      halftoneInfo;   // defines gxHalftone information for gxColor bitmaps
  PenTableHdl       hPenTable;      // complete list of pens along with their pen positions and thickness
  gxRectangle       pageRect;     // page dimensions
  gxVectorShapeDataRec  shapeData;      // information on how to render a gxShape
} VectorImageDataRec, *VectorImageDataPtr, **VectorImageDataHdl;



/* ------------------------------------------------------------------------------

                Vector Driver Imaging Messages

-------------------------------------------------------------------------------- */


#ifdef __cplusplus
extern "C" {
#endif


#define GXVECTORMSGGLUE(selector) SendMessageGlue(0x0001, selector)
#define gxFORWARDGLUE ForwardThisMessageGlue


OSErr Send_VectorPackageShape (gxShape, long penIndex)
  = GXVECTORMSGGLUE(gxVectorPackageData);
OSErr Forward_VectorPackageShape (gxShape, long penIndex)
  = gxFORWARDGLUE;
  
OSErr Send_VectorLoadPens (PenTableHdl, long *shapeCounts, gxBoolean *penTableChanged)
  = GXVECTORMSGGLUE(gxVectorLoadPens);
OSErr Forward_VectorLoadPens (PenTableHdl, long *shapeCounts, gxBoolean *penTableChanged)
  = gxFORWARDGLUE;

OSErr Send_VectorVectorizeShape (gxShape, long penIndex, gxVectorShapeDataRec *)
  = GXVECTORMSGGLUE(gxVectorVectorizeShape);
OSErr Forward_VectorVectorizeShape (gxShape, long penIndex, gxVectorShapeDataRec *)
  = gxFORWARDGLUE;


#ifdef __cplusplus
}
#endif



/* ------------------------------------------------------------------------------

              PostScript Driver Contants and Types

-------------------------------------------------------------------------------- */



enum { gxPostSynonym = (long)'post' };


typedef long gxPostScriptRenderOptions;

enum {                      // bit fields in gxPostScriptRenderOptions
  gxNeedsHexOption          = 0x00000001, // convert all binary data to hex
  gxNeedsCommentsOption     = 0x00000002, // issue PostScript comments
  gxBoundingBoxesOption     = 0x00000004, // calculate the values for %%BoundingBox: and %%PageBoundingBox: -- requires gxNeedsCommentsOption
  gxPortablePostScriptOption  = 0x00000008, // generate portable PostScript
  gxTextClipsToPathOption   = 0x00000010, // convert all clips that are composed of text to gxPath shapes
  gxFlattenClipPathOption   = 0x00000020, // convert all clips that are gxPath shapes to gxPolygons (helps better control gxPoint limit)
  gxUseCharpath1Option      = 0x00000040, // (ignored if text clips are converted to gxPaths)  When the clip is text, 
                              // do it one glyph at a time, redrawing the main gxShape each time
  gxUseLevel2ColorOption      = 0x00000080  // When printing to level-2 use level-2 device independent gxColor
};



typedef struct {

  gxFont            theFont;              // ---> gxFont reference.
  long            nGlyphs;              // ---> Number of glyphs in the gxFont.

  gxFontPlatforms     platform;           // <--- How printer gxFont is encoded.
  gxFontScript        script;             // <--- Script if platform != gxGlyphPlatform
  gxFontLanguage      language;           // <--- Language if platform != gxGlyphPlatform.
  
  long            vmUsage;              // <--- how much PostScript VM gxFont uses.

  /** size of this is longAlligned(nGlyphs) **/
  unsigned long   glyphBits[1];           // <--- Bit array of which system glyphs are in printer.

} gxPrinterGlyphsRec;

typedef struct {                // PostScript imaging system imageData structure
  short             languageLevel;  // PostScript language level
  gxColorSpace          devCSpace;    // the printer's gxColor space
  gxColorProfile        devCProfile;  // The printer's gxColor profile for matching
  gxPostScriptRenderOptions renderOptions;  // options for the imaging system
  short             pathLimit;    // maximum gxPath size
  short             gsaveLimit;   // maximum number of gsaves allowed
  short             opStackLimit; // operand stack limit
  gxStreamType          fontType;   // these are the gxFont types that the printer supports 
  long              printerVM;    // how much memory is in the printer
  long              reserved0;
} PostScriptImageDataRec, *PostScriptImageDataPtr, **PostScriptImageDataHdl;


typedef struct  {               // for GetProcSetList / DownLoadProcSetList messages
  Signature clientid;
  OSType    controlType;            // the driver will call FetchTaggedData on each of these resources
  short     controlid;
  OSType    dataType;
  long      reserved0;
} ProcSetListRec, *ProcSetListPtr, **ProcSetListHdl;




enum {              // possible results of querying printer (returned by PostScriptQueryPrinter message)
  gxPrinterOK     = 0,
  gxIntializePrinter  = 1, 
  gxFilePrinting    = 2,
  gxResetPrinter    = 128
};



/* ------------------------------------------------------------------------------

                PostScript Driver Imaging Messages

-------------------------------------------------------------------------------- */


#ifdef __cplusplus
extern "C" {
#endif


#define GXPOSTSCRIPTMSGGLUE(selector) SendMessageGlue(0x0001, selector)
#define gxFORWARDGLUE ForwardThisMessageGlue


// device control messages


OSErr Send_PostScriptQueryPrinter (long *)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptQueryPrinter);
OSErr Forward_PostScriptQueryPrinter (long *)
  = gxFORWARDGLUE;

OSErr Send_PostScriptInitializePrinter (void)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptInitializePrinter);
OSErr Forward_PostScriptInitializePrinter (void)
  = gxFORWARDGLUE;

OSErr Send_PostScriptResetPrinter (void)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptResetPrinter);
OSErr Forward_PostScriptResetPrinter (void)
  = gxFORWARDGLUE;

OSErr Send_PostScriptExitServer (void)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptExitServer);
OSErr Forward_PostScriptExitServer (void)
  = gxFORWARDGLUE;
  
  
// device communication messages


OSErr Send_PostScriptGetStatusText (Handle)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptGetStatusText);
OSErr Forward_PostScriptGetStatusText (Handle)
  = gxFORWARDGLUE;

OSErr Send_PostScriptGetPrinterText (Handle)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptGetPrinterText);
OSErr Forward_PostScriptGetPrinterText (Handle)
  = gxFORWARDGLUE;

OSErr Send_PostScriptScanStatusText (Handle)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptScanStatusText);
OSErr Forward_PostScriptScanStatusText (Handle)
  = gxFORWARDGLUE;

OSErr Send_PostScriptScanPrinterText (Handle)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptScanPrinterText);
OSErr Forward_PostScriptScanPrinterText (Handle)
  = gxFORWARDGLUE;


// proc set management messages


OSErr Send_PostScriptGetDocumentProcSetList (ProcSetListHdl, PostScriptImageDataHdl)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptGetDocumentProcSetList);
OSErr Forward_PostScriptGetDocumentProcSetList (ProcSetListHdl, PostScriptImageDataHdl)
  = gxFORWARDGLUE;

OSErr Send_PostScriptDownloadProcSetList (ProcSetListHdl, PostScriptImageDataHdl)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptDownloadProcSetList);
OSErr Forward_PostScriptDownloadProcSetList (ProcSetListHdl, PostScriptImageDataHdl)
  = gxFORWARDGLUE;
  
  
// gxFont management messages


OSErr Send_PostScriptGetPrinterGlyphsInformation (gxPrinterGlyphsRec *)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptGetPrinterGlyphsInformation);
OSErr Forward_PostScriptGetPrinterGlyphsInformation (gxPrinterGlyphsRec *)
  = gxFORWARDGLUE;

OSErr Send_PostScriptStreamFont (gxFont fontref, gxScalerStream* streamPtr)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptStreamFont);
OSErr Forward_PostScriptPostScriptStreamFont (gxFont fontref, gxScalerStream* streamPtr)
  = gxFORWARDGLUE;


// document structuring and formatting messages


OSErr Send_PostScriptDoDocumentHeader (PostScriptImageDataHdl)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptDoDocumentHeader);
OSErr Forward_PostScriptDoDocumentHeader (PostScriptImageDataHdl)
  = gxFORWARDGLUE;
  
OSErr Send_PostScriptDoDocumentSetup (PostScriptImageDataHdl)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptDoDocumentSetUp);
OSErr Forward_PostScriptDoDocumentSetup (PostScriptImageDataHdl)
  = gxFORWARDGLUE;

OSErr Send_PostScriptDoDocumentTrailer (PostScriptImageDataHdl)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptDoDocumentTrailer);
OSErr Forward_PostScriptDoDocumentTrailer (PostScriptImageDataHdl)
  = gxFORWARDGLUE;
  
  
// page structuring and formatting messages


OSErr Send_PostScriptDoPageSetup (gxFormat, long, PostScriptImageDataHdl)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptDoPageSetUp);
OSErr Forward_PostScriptDoPageSetup (gxFormat, long, PostScriptImageDataHdl)
  = gxFORWARDGLUE;

OSErr Send_PostScriptSelectPaperType (gxPaperType, long, PostScriptImageDataHdl)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptSelectPaperType);
OSErr Forward_PostScriptSelectPaperType (gxPaperType, long, PostScriptImageDataHdl)
  = gxFORWARDGLUE;

OSErr Send_PostScriptDoPageTrailer (PostScriptImageDataHdl)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptDoPageTrailer);
OSErr Forward_PostScriptDoPageTrailer (PostScriptImageDataHdl)
  = gxFORWARDGLUE;

OSErr Send_PostScriptEjectPage (gxPaperType, long pagenumber, long copiescount, short erasepage, PostScriptImageDataHdl)
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptEjectPage);
OSErr Forward_PostScriptEjectPage (gxPaperType, long pagenumber, long copiescount, short erasepage, PostScriptImageDataHdl)
  = gxFORWARDGLUE;


// gxShape imaging messages


OSErr Send_PostScriptProcessShape (gxShape page,long trcount, gxTransform trlist[])
  = GXPOSTSCRIPTMSGGLUE(gxPostscriptProcessShape);
OSErr Forward_PostScriptProcessShape (gxShape page,long trcount, gxTransform trlist[])
  = gxFORWARDGLUE;


#ifdef __cplusplus
}
#endif



/* ------------------------------------------------------------------------------

                      Driver API Functions

-------------------------------------------------------------------------------- */


#define GXDRIVERAPIGLUE(selector) {0x203C, 0x0002, selector, 0xABFE}


#ifdef __cplusplus
extern "C" {
#endif


OSErr GXAddPrinterViewDevice (gxPrinter, gxViewDevice)
  = GXDRIVERAPIGLUE(0);

OSErr GXGetAvailableJobFormatModes (JobFormatModeTableHdl *)
  = GXDRIVERAPIGLUE(1);

OSErr GXSetPreferredJobFormatMode (gxJobFormatMode, Boolean directOnly)
  = GXDRIVERAPIGLUE(2);

OSErr GXCoolAlert (short iconId,
            short txtSize,
            short defaultTitleNum,
            short cancelTitleNum,
            short textLength,
            Ptr pAlertMsg,
            StringPtr actionTitle,
            StringPtr title2,
            StringPtr title3,
            StringPtr msgFont,
            ModalFilterProcPtr filterProc,
            short *itemHit)
  = GXDRIVERAPIGLUE(3);

OSErr GXGetCoolAlert (short coolResId, ModalFilterProcPtr, short *itemHit) 
  = GXDRIVERAPIGLUE(4);


OSErr GXFetchDTPData (Str31 dtpName, ResType theType, short theID, Handle *theData)
  = GXDRIVERAPIGLUE(5);

OSErr GXWriteDTPData (Str31 dtpName, ResType theType, short theID, Handle theData)
  = GXDRIVERAPIGLUE(6);

OSErr GXHandleChooserMessage (gxJob *aJob, Str31 driverName, short message, short caller, 
                   StringPtr objName, StringPtr zoneName, ListHandle theList, long p2)
  = GXDRIVERAPIGLUE(7);


#ifdef __cplusplus
}
#endif


#endif __PRINTINGDRIVERS__
