/* ------------------------------------------------------------------------------
FILENAME
  PrintingErrors.h

DESCRIPTION
    This file defines all the public result codes for the Printing Manager.
  
COPYRIGHT
    Apple Computer, Inc. 1990-1991
   All rights reserved. 
  
-------------------------------------------------------------------------------- */

#ifndef __PRINTINGERRORS__
#define __PRINTINGERRORS__

#define gxPrintingResultBase    -510

enum {

//  RESULT CODES FOR I/O

    gxAioTimeout       = gxPrintingResultBase,        // -510 : Timeout condition occurred during op.
    gxAioBadRqstState    = (gxPrintingResultBase - 1),    // -511 : Async I/O request in invalid state for op.
    gxAioBadConn       = (gxPrintingResultBase - 2),    // -512 : Invalid AIO connection refnum
    gxAioInvalidXfer     = (gxPrintingResultBase - 3),    // -513 : Read DataXferStruc contained bad values
    gxAioNoRqstBlks    = (gxPrintingResultBase - 4),    // -514 : No available request blocks to process request
    gxAioNoDataXfer    = (gxPrintingResultBase - 5),    // -515 : DataXferStruc pointer not specified
    gxAioTooManyAutos    = (gxPrintingResultBase - 6),    // -516 : Auto status request already active
    gxAioNoAutoStat    = (gxPrintingResultBase - 7),    // -517 : Connection not configured for auto status
    gxAioBadRqstID     = (gxPrintingResultBase - 8),    // -518 : Invalid I/O request identifier
    gxAioCantKill      = (gxPrintingResultBase - 9),    // -519 : Comm. protocol doesn't support I/O term.
    gxAioAlreadyExists   = (gxPrintingResultBase - 10),   // -520 : Protocol spec. data already specified
    gxAioCantFind      = (gxPrintingResultBase - 11),   // -521 : Protocol spec. data does not exist
    gxAioDeviceDisconn   = (gxPrintingResultBase - 12),   // -522 : Machine disconnected from printer
    gxAioNotImplemented  = (gxPrintingResultBase - 13),   // -523 : Function not implemented
    gxAioOpenPending     = (gxPrintingResultBase - 14),   // -524 : Opening a connection for protocol, but another open pending
    gxAioNoProtocolData  = (gxPrintingResultBase - 15),   // -525 : No protocol specific data specified in request
    gxAioRqstKilled    = (gxPrintingResultBase - 16),   // -526 : I/O request was terminated
    gxBadBaudRate      = (gxPrintingResultBase - 17),   // -527 : Invalid baud rate specified
    gxBadParity      = (gxPrintingResultBase - 18),   // -528 : Invalid parity specified
    gxBadStopBits      = (gxPrintingResultBase - 19),   // -529 : Invalid stop bits specified
    gxBadDataBits      = (gxPrintingResultBase - 20),   // -530 : Invalid data bits specified
    gxBadPrinterName     = (gxPrintingResultBase - 21),   // -531 : Bad printer name specified
    gxAioBadMsgType    = (gxPrintingResultBase - 22),   // -532 : Bad masType field in xferInfo struc
    gxAioCantFindDevice  = (gxPrintingResultBase - 23),   // -533 : Cannot locate target device
    gxAioOutOfSeq      = (gxPrintingResultBase - 24),   // -534 : Non-atomic SCSI requests submitted out of sequence

  gxPrIOAbortErr     = (gxPrintingResultBase - 25),   // -535
    gxPrUserAbortErr     = (gxPrintingResultBase - 26),   // -536


//  RESULT CODES FOR THE DIALOGS

    gxCantAddPanelsNowErr    = (gxPrintingResultBase - 27), // -537 : panels can only be added during driver switch or before dialog is initiated
    gxBadxdtlKeyErr      = (gxPrintingResultBase - 28), // -538 : unknown key for xdtl - must be radiobutton, etc
    gxXdtlItemOutOfRangeErr  = (gxPrintingResultBase - 29), // -539 : referenced item does not belong to panel
    gxNoCoolActionButtonErr    = (gxPrintingResultBase - 30), // -540 : action button is nil
    gxCoolTitlesTooLongErr   = (gxPrintingResultBase - 31), // -541 : length of buttons exceeds alert maximum width
    gxUnknownCoolVersionErr    = (gxPrintingResultBase - 32), // -542 : bad version for cool alerts


// RESULT CODES FOR BUFFERING

    gxGbBufferTooSmallErr   = (gxPrintingResultBase - 33),  // -543


// RESULT CODES FOR VECTOR IMAGING

    gxInvalidPenTable    = (gxPrintingResultBase - 34),   // -544


// RESULT CODES FOR PRINT FILES

    gxIncompletePrintFileErr   = (gxPrintingResultBase - 35), // -545
    gxCrashedPrintFileErr    = (gxPrintingResultBase - 36), // -546
    gxInvalidPrintFileVersion  = (gxPrintingResultBase - 37), // -547


// MISCELLANEOUS RESULT CODES FROM THE API

  gxSegmentLoadFailedErr  = (gxPrintingResultBase - 38),  // -548 : keep in sync with Dispatcher.a in the core
  gxExtensionNotFoundErr  = (gxPrintingResultBase - 39),  // -549
    gxFlattenVersionTooNew    = (gxPrintingResultBase - 40),  // -550
    gxPaperTypeNotFound     = (gxPrintingResultBase - 41),  // -551
    gxNoSuchPTGroup     = (gxPrintingResultBase - 42),  // -552
  

// TO BE REMOVED...

    gxDuplicatePanelNameErr  = (gxPrintingResultBase - 43)  // -553 : panel name is not unique on add
  
};


#endif __PRINTINGERRORS__
