/* ------------------------------------------------------------------------------

  FILENAME
    PrintingMessages.h

  DESCRIPTION
    This file defines all the universal printing messages,
    and Printing Manager API functions callable only from
    within a message override.

  COPYRIGHT
    Copyright  Apple Computer, Inc. 1989-1993
    All rights reserved. 
  
------------------------------------------------------------------------------- */


#ifndef __PRINTINGMESSAGES__
#define __PRINTINGMESSAGES__


#ifndef __PRINTINGMANAGER__
  #include <PrintingManager.h>
#endif

#ifndef __PRINTINGRESTYPES__
  #include <PrintingResTypes.h>
#endif

#ifndef __MESSAGEMANAGER__
  #include <Messages.h>
#endif

#ifndef __MENUS__
  #include <Menus.h>
#endif

#ifndef __LISTS__
  #include <Lists.h>
#endif


/* ------------------------------------------------------------------------------

                  Constants and Types

-------------------------------------------------------------------------------- */



/*--------------------------*/
/* abstract data types...   */
/*--------------------------*/



typedef struct gxPrivateFileRecord *gxSpoolFile;



/*---------------------------------------*/
/* dialog panel constants and types...   */
/*---------------------------------------*/



typedef long gxPanelEvent;

enum  {                         // dialog panel events
  gxPanelNoEvt          = (gxPanelEvent) 0,
  gxPanelOpenEvt        = (gxPanelEvent) 1,     // gxInitialize and draw
  gxPanelCloseEvt         = (gxPanelEvent) 2,     // your panel is going away ( panel switch, confirm or cancel )
  gxPanelHitEvt         = (gxPanelEvent) 3,     // there's a hit in your panel
  gxPanelActivateEvt      = (gxPanelEvent) 4,     // the dialog window has just been activated
  gxPanelDeactivateEvt    = (gxPanelEvent) 5,     // the dialog window is about to be deactivated
  gxPanelIconFocusEvt     = (gxPanelEvent) 6,     // the focus changes from the panel to the icon list
  gxPanelPanelFocusEvt    = (gxPanelEvent) 7,     // the focus changes from the icon list to the panel
  gxPanelFilterEvt        = (gxPanelEvent) 8,     // this is called to filter every event
  gxPanelCancelEvt        = (gxPanelEvent) 9,     // the user has cancelled the dialog
  gxPanelConfirmEvt     = (gxPanelEvent) 10,    // the user has confirmed the dialog
  gxPanelDialogEvt        = (gxPanelEvent) 11,    // event to be handle by dialoghandler ( you get to see all events )
  gxPanelOtherEvt       = (gxPanelEvent) 12,    // osEvts, etc.
  gxPanelUserWillConfirmEvt = (gxPanelEvent) 13     // user has selected confirm, time to parse panel interdependencies
};


typedef long gxPanelResult;

enum {                    // possible panel responses to dialog handler calls
  gxPanelNoResult       = 0,
  gxPanelCancelConfirmation = 1   // only valid from gxPanelUserWillConfirmEvt - used to keep the dialog from going away
};


enum {                    // values for the evtAction field in gxPanelInfoRecord
  gxOtherAction         = 0,        // current item will not change
  gxClosePanelAction      = 1,        // panel will be closed
  gxCancelDialogAction    = 2,        // dialog will be cancelled
  gxConfirmDialogAction   = 3       // dialog will be confirmed
};


typedef struct {              // for FilterPanelEvent and HandlePanelEvent messages
  gxPanelEvent    panelEvt;         // why we're calling
  short       panelResId;         // 'panl' resource id of current panel
  DialogPtr     pDlg;             // pointer to dialog
  EventRecord   *theEvent;          // pointer to event
  short       itemHit;            // actual item number as dialog mgr thinks
  short       itemCount;          // number of items before your items
  short       evtAction;          // once this event is processed, the action that will result
                              // (only meaningful in filtering, used for parsing)
  short       errorStringId;        // STR id of string to put in error alert (0 means no string)
  gxFormat      theFormat;          // the current format (only meaningful in a format dialog)
  void        *refCon;            // refCon passed in gxPanelSetupRecord
} gxPanelInfoRecord;
          

typedef long gxPrintingPanelKind;

enum {
  gxApplicationPanel  = (gxPrintingPanelKind) 0,
  gxExtensionPanel    = (gxPrintingPanelKind) 1,
  gxDriverPanel     = (gxPrintingPanelKind) 2
};
  

// The PanelSetupInfo structure is passed to the dialog handler when the client calls
// to setup panels for the dialog


typedef struct {
  gxPrintingPanelKind     panelKind;
  short             panelResId;
  short             resourceRefNum;
  void              *refCon;
} gxPanelSetupRecord;


typedef long gxParsePageRangeResult;    // returned by ParsePageRange message

enum {
  gxRangeNotParsed    = (gxParsePageRangeResult) 0,   // default initial value
  gxRangeParsed     = (gxParsePageRangeResult) 1,
  gxRangeBadFromValue = (gxParsePageRangeResult) 2,
  gxRangeBadToValue = (gxParsePageRangeResult) 3
};



/*----------------------------------------*/
/* status-related constants and types...  */
/*----------------------------------------*/



// status type ids


enum {
  gxNonFatalError = 1,        // effects icon on spooling dialog
  gxFatalError = 2,         // sends up cool alert on spooling dialog
  gxPrinterReady = 3,         // signals PFE to leave alert mode
  gxUserAttention = 4,        // signals initiation of a modal alert
  gxUserAlert = 5,            // signals initiation of a moveable modal alert
  gxPageTransmission = 6,     // signals page sent to printer, increments page count in strings to user
  gxOpenConnectionStatus = 7,   // signals PFE to begin animation on printer icon
  gxInformationalStatus = 8,    // default status type, no side effects
  gxSpoolingPageStatus = 9,     // signals page spooled, increments page count in spooling dialog
  gxEndStatus = 10,         // signals end of spooling
  gxPercentageStatus = 11     // signals the PFE as to the amount of the job which is currently complete
};


typedef struct {              // for status messages
  unsigned short    statusType;     // one of the ids listed above ( gxNonFatalError, etc. )
  unsigned short    statusId;     // specific status ( out of paper, etc. ) - To be defined
  unsigned short    statusAlertId;    //  cool alert id (if any) for status
  Signature     statusOwner;    // creator type of status owner
  short         statResId;      // id for 'stat' resource
  short         statResIndex;   // index into 'stat' resource for this status
  short         dialogResult;   // id of button string selected on dismissal of cool alert
  unsigned short    bufferLen;      // number of bytes in status buffer - total record size must be <= 512
  char          statusBuffer[1];  // user response from alert
} gxStatusRecord;




typedef struct {              // for WriteStatusToDTPWindow message
  Boolean     useText;
  Handle      hPicture;
  Str255      theText;
} gxDisplayRecord;



/*-----------------------------------------------*/
/* paper gxMapping-related constants and types...  */
/*-----------------------------------------------*/


typedef long gxTrayMapping;

enum {
  gxDefaultTrayMapping    = (gxTrayMapping) 0,
  gxConfiguredTrayMapping   = (gxTrayMapping) 1
};



/* ------------------------------------------------------------------------------

        API Functions callable only from within message overrides

-------------------------------------------------------------------------------- */



#define GXPROTECTEDAPIGLUE(selector) {0x203C, 0x0001, selector, 0xABFE}

#define GXPRINTINGDISPATCH(segID, selector) {0x203C, 0x0001, 0, 0x223C, (segID & 0x0FFF), selector << 2, 0xABFE}


#ifdef __cplusplus
extern "C" {
#endif


#ifdef applec

  #pragma parameter __D0 GXPrintingDispatch(__D1)
  OSErr GXPrintingDispatch (long selector, ...)
      = GXPROTECTEDAPIGLUE(0);

#else

  OSErr GXPrintingDispatch (long selector, ...)
      = {0x221F, 0x203C, 0x0001, 0x0000, 0xABFE, 0x598F};

#endif applec

/*
  How to use the GXPRINTINGDISPATCH macro...
  
  If your driver or extension is large, you may want to segment it
  across smaller boundaries than is permitted by the messaging system.
  Without using the Printing Manager's segmentation manager directly,
  the smallest segment you can create consists of the code to override
  a single message.  If you are overriding workhorse messages such as
  RenderPage, you may want to divide up the work among many functions
  distributed across several segments.  Here's how...
  
  The Printing Manager segment scheme involves the construction of a
  single 32-bit dispatch selector, which contains all the information
  necessary for the dispatcher to find a single routine.  It contains the
  segment's resource ID, and the offset within the segment which contains
  the start of the routine.  The GXPRINTINGDISPATCH macro will construct the
  dispatch selector for you, as well as the code to do the dispatch.
  
  Usually, it is convenient to start your segment with a long aligned jump table,
  beginning after the 4 byte header required by the Printing Manager.  The
  macro assumes this is the case and takes a 1-based routine selector from
  which it conmstructs the offset.
  
  For example, if your code is in resource 'pdvr' (print driver), ID=2
  at offset=12 (third routine in segment), you would declare your
  routine as follows:
  
  OSErr MyRenderingRoutine (long param1, Ptr param2)
    = GXPRINTINGDISPATCH(2, 3);
    
  Remember, ALL segment dispatches must return OSErr.  If your routine
  does not generate errors, you must still declare it to return OSErr
  and have the routine itself return noErr.
  
  An alternative way to call across segments is to call the GXPrintingDispatch
  function directly.  You must construct the 32-bit selector yourself and pass
  it as the first parameter.  This is usually not preferable since you don't get
  type-checking unless you declare a prototype as shown above, and your code
  isn't as easy to read.
  
  So given the above prototype, there are two ways to call the function:
  
    anErr = MyRenderingRoutine(p1, p2);     // free type checking!
    
  or:
  
    #define kMyRenderRoutineSelector 0x0002000C
    anErr = GXPrintingDispatch(kMyRenderRoutineSelector, p1, p2);   // no type-checking!
  
  
  Both have the same effect.
*/


gxJob GXGetJob (void)
    = GXPROTECTEDAPIGLUE(1);

short GXGetMyResFile (void)
  = GXPROTECTEDAPIGLUE(2);
  
Boolean GXSpoolingAborted (void)
  = GXPROTECTEDAPIGLUE(3);
  
OSErr GXJobIdle (void)
  = GXPROTECTEDAPIGLUE(4);

OSErr GXReportStatus (short statusID, unsigned short statusIndex)
  = GXPROTECTEDAPIGLUE(5);

OSErr GXAlertTheUser (gxStatusRecord *)
  = GXPROTECTEDAPIGLUE(6);

OSErr GXSetupDialogPanel (gxPanelSetupRecord *)
  = GXPROTECTEDAPIGLUE(7);

OSErr GXCountTrays (gxTrayIndex *numTrays)
  = GXPROTECTEDAPIGLUE(8);
  
OSErr GXGetTrayName (gxTrayIndex trayNumber, Str31 trayName)
  = GXPROTECTEDAPIGLUE(9);

OSErr GXSetTrayPaperType (gxTrayIndex whichTray, gxPaperType)
  = GXPROTECTEDAPIGLUE(10);

OSErr GXGetTrayPaperType (gxTrayIndex whichTray, gxPaperType)
  = GXPROTECTEDAPIGLUE(11);

OSErr GXGetTrayMapping (gxTrayMapping *trayMapping)
  = GXPROTECTEDAPIGLUE(12);

void GXCleanupStartJob (void)
  = GXPROTECTEDAPIGLUE(13);

void GXCleanupStartPage (void)
  = GXPROTECTEDAPIGLUE(14);

void GXCleanupOpenConnection (void)
  = GXPROTECTEDAPIGLUE(15);

void GXCleanupStartSendPage (void)
  = GXPROTECTEDAPIGLUE(16);



/* ------------------------------------------------------------------------------

          Constants and types for Universal Printing Messages

-------------------------------------------------------------------------------- */



enum {                    // options for CreateSpoolFile message
  gxNoCreateOptions = 0x00000000, // just create the file
  gxInhibitAlias    = 0x00000001, // do not create an alias in the PMD folder
  gxInhibitUniqueName = 0x00000002  // do not append to the filename to make it unique
};


enum {                    // options for CloseSpoolFile message
  gxNoCloseOptions  = 0x00000000,   // just close the file
  gxDeleteOnClose = 0x00000001,   // delete the file rather than closing it
  gxUpdateJobData = 0x00000002,   // write current job information into file prior to closing
  gxMakeRemoteFile  = 0x00000004    // mark job as a remote file
};


enum {                    // options for CreateImageFile message
  gxNoImageFile   = 0x00000000,       // don't create image file
  gxMakeImageFile   = 0x00000001,       // create an image file
  gxEachPlane   = 0x00000002,       // only save up planes before rewinding
  gxEachPage      = 0x00000004,       // save up entire pages before rewinding
  gxEntireFile    = gxEachPlane + gxEachPage  // save up the entire file before rewinding
};


enum {                    // options for BufferData message
  gxNoBufferOptions = 0x00000000, 
  gxMakeBufferHex   = 0x00000001, 
  gxDontSplitBuffer = 0x00000002
};


typedef struct {              // for DumpBuffer and FreeBuffer messages
  long      size;             //  size of buffer in bytes
  long      userData;         //  client assigned id for the buffer
  char      data[1];            // array of size bytes
} gxPrintingBuffer;


typedef struct {              // for RenderPage message
  long    docPageNum;           // number of page being printed
  long    copyNum;              // copy number being printed
  Boolean formatChanged;          // true if format changed from last page
  Boolean pageChanged;          // true if page contents changed from last page
  long    internalUse;          // private
} gxPageInfoRecord;



/* ------------------------------------------------------------------------------

                Universal Printing Messages

-------------------------------------------------------------------------------- */


#define GXUNIVSENDGLUE(selector) SendMessageGlue(0x0000, selector)
#define gxFORWARDGLUE ForwardThisMessageGlue


#define Send_FetchTaggedDriverData(gxTag, id, pHandle) Send_FetchTaggedData(gxTag, id, pHandle, 'drvr')
#define Forward_FetchTaggedDriverData(gxTag, id, pHandle) Forward_FetchTaggedData(gxTag, id, pHandle, 'drvr')


OSErr Send_JobIdle (void)
    = GXUNIVSENDGLUE(gxJobIdle);
OSErr Forward_JobIdle (void)
    = gxFORWARDGLUE;

OSErr Send_JobStatus (gxStatusRecord *)
    = GXUNIVSENDGLUE(gxJobStatus);
OSErr Forward_JobStatus (gxStatusRecord *)
    = gxFORWARDGLUE;

OSErr Send_PrintingEvent (EventRecord *, Boolean filterEvent)
    = GXUNIVSENDGLUE(gxPrintingEvent);
OSErr Forward_PrintingEvent (EventRecord *, Boolean filterEvent)
    = gxFORWARDGLUE;


OSErr Send_JobDefaultFormatDialog (gxDialogResult *)
    = GXUNIVSENDGLUE(gxJobDefaultFormatDialog);
OSErr Forward_JobDefaultFormatDialog (gxDialogResult *)
    = gxFORWARDGLUE;
  
OSErr Send_FormatDialog (gxFormat, StringPtr title, gxDialogResult *)
    = GXUNIVSENDGLUE(gxFormatDialog);
OSErr Forward_FormatDialog (gxFormat, StringPtr title, gxDialogResult *)
    = gxFORWARDGLUE;
  
OSErr Send_JobPrintDialog (gxDialogResult *)
    = GXUNIVSENDGLUE(gxJobPrintDialog);
OSErr Forward_JobPrintDialog (gxDialogResult *)
    = gxFORWARDGLUE;

OSErr Send_FilterPanelEvent (gxPanelInfoRecord *, Boolean *)
  = GXUNIVSENDGLUE(gxFilterPanelEvent);

OSErr Send_HandlePanelEvent (gxPanelInfoRecord *, gxPanelResult *)
  = GXUNIVSENDGLUE(gxHandlePanelEvent);

OSErr Send_ParsePageRange (StringPtr fromString, StringPtr toString, gxParsePageRangeResult *result)
  = GXUNIVSENDGLUE(gxParsePageRange);
OSErr Forward_ParsePageRange (StringPtr fromString, StringPtr toString, gxParsePageRangeResult *result)
  = gxFORWARDGLUE;
  
  
OSErr Send_DefaultJob (void)
    = GXUNIVSENDGLUE(gxDefaultJob);
OSErr Forward_DefaultJob (void)
    = gxFORWARDGLUE;
  
OSErr Send_DefaultFormat (gxFormat)
    = GXUNIVSENDGLUE(gxDefaultFormat);
OSErr Forward_DefaultFormat (gxFormat)
    = gxFORWARDGLUE;

OSErr Send_DefaultPaperType (gxPaperType)
    = GXUNIVSENDGLUE(gxDefaultPaperType);
OSErr Forward_DefaultPaperType (gxPaperType)
    = gxFORWARDGLUE;
  
OSErr Send_DefaultPrinter (gxPrinter)
    = GXUNIVSENDGLUE(gxDefaultPrinter);
OSErr Forward_DefaultPrinter (gxPrinter)
    = gxFORWARDGLUE;
  

OSErr Send_CreateSpoolFile (FSSpecPtr, long createOptions, gxSpoolFile *)
    = GXUNIVSENDGLUE(gxCreateSpoolFile);
OSErr Forward_CreateSpoolFile (FSSpecPtr, long createOptions, gxSpoolFile *)
    = gxFORWARDGLUE;
  
OSErr Send_SpoolPage (gxSpoolFile, gxFormat, gxShape)
    = GXUNIVSENDGLUE(gxSpoolPage);
OSErr Forward_SpoolPage (gxSpoolFile, gxFormat, gxShape)
    = gxFORWARDGLUE;

OSErr Send_SpoolData (gxSpoolFile, Ptr data, long *length)
    = GXUNIVSENDGLUE(gxSpoolData);
OSErr Forward_SpoolData (gxSpoolFile, Ptr data, long *length)
    = gxFORWARDGLUE;

OSErr Send_SpoolResource (gxSpoolFile, Handle, ResType, short id)
    = GXUNIVSENDGLUE(gxSpoolResource);
OSErr Forward_SpoolResource (gxSpoolFile, Handle, ResType, short id)
    = gxFORWARDGLUE;

OSErr Send_CompleteSpoolFile (gxSpoolFile)
    = GXUNIVSENDGLUE(gxCompleteSpoolFile);
OSErr Forward_CompleteSpoolFile (gxSpoolFile)
    = gxFORWARDGLUE;


OSErr Send_CountPages (gxSpoolFile, long *numPages)
    = GXUNIVSENDGLUE(gxCountPages);
OSErr Forward_CountPages (gxSpoolFile, long *numPages)
    = gxFORWARDGLUE;

OSErr Send_DespoolPage (gxSpoolFile, long, gxFormat, gxShape *,  Boolean *formatChanged)
    = GXUNIVSENDGLUE(gxDespoolPage);
OSErr Forward_DespoolPage (gxSpoolFile, long, gxFormat, gxShape *,  Boolean *formatChanged)
    = gxFORWARDGLUE;

OSErr Send_DespoolData (gxSpoolFile, Ptr data, long *length)
    = GXUNIVSENDGLUE(gxDespoolData);
OSErr Forward_DespoolData (gxSpoolFile, Ptr data, long *length)
    = gxFORWARDGLUE;

OSErr Send_DespoolResource (gxSpoolFile, ResType, short id, Handle *)
    = GXUNIVSENDGLUE(gxDespoolResource);
OSErr Forward_DespoolResource (gxSpoolFile, ResType, short id, Handle *)
    = gxFORWARDGLUE;

OSErr Send_CloseSpoolFile (gxSpoolFile, long closeOptions)
    = GXUNIVSENDGLUE(gxCloseSpoolFile);
OSErr Forward_CloseSpoolFile (gxSpoolFile, long closeOptions)
    = gxFORWARDGLUE;
  
  
OSErr Send_StartJob (StringPtr docName, long pageCount)
    = GXUNIVSENDGLUE(gxStartJob);
OSErr Forward_StartJob (StringPtr docName, long pageCount)
    = gxFORWARDGLUE;
  
OSErr Send_FinishJob (void)
    = GXUNIVSENDGLUE(gxFinishJob);
OSErr Forward_FinishJob (void)
    = gxFORWARDGLUE;
  
OSErr Send_StartPage (gxFormat, long numViewPorts, gxViewPort *viewPortList)
  = GXUNIVSENDGLUE(gxStartPage);
OSErr Forward_StartPage (gxFormat, long numViewPorts, gxViewPort *viewPortList)
  = gxFORWARDGLUE;

OSErr Send_FinishPage (void)
  = GXUNIVSENDGLUE(gxFinishPage);
OSErr Forward_FinishPage (void)
  = gxFORWARDGLUE;
  
OSErr Send_PrintPage (gxFormat, gxShape)
  = GXUNIVSENDGLUE(gxPrintPage);
OSErr Forward_PrintPage (gxFormat, gxShape)
  = gxFORWARDGLUE;
  

OSErr Send_SetupImageData (void *imageData)
    = GXUNIVSENDGLUE(gxSetupImageData);
OSErr Forward_SetupImageData (void *imageData)
    = gxFORWARDGLUE;

OSErr Send_ImageJob(gxSpoolFile, long *closeOptions)
    = GXUNIVSENDGLUE(gxImageJob);
OSErr Forward_ImageJob (gxSpoolFile, long *closeOptions)
    = gxFORWARDGLUE;

OSErr Send_ImageDocument (gxSpoolFile, void *imageData)
    = GXUNIVSENDGLUE(gxImageDocument);
OSErr Forward_ImageDocument (gxSpoolFile, void *imageData)
    = gxFORWARDGLUE;

OSErr Send_ImagePage (gxSpoolFile, long pageNumber, gxFormat, void *imageData)
    = GXUNIVSENDGLUE(gxImagePage);
OSErr Forward_ImagePage (gxSpoolFile, long pageNumber, gxFormat, void *imageData)
    = gxFORWARDGLUE;

OSErr Send_RenderPage (gxFormat, gxShape, gxPageInfoRecord *, void *imageData)
    = GXUNIVSENDGLUE(gxRenderPage);
OSErr Forward_RenderPage (gxFormat, gxShape, gxPageInfoRecord *, void *imageData)
    = gxFORWARDGLUE;


OSErr Send_CreateImageFile (FSSpecPtr, long imageFileOptions, long *)
    = GXUNIVSENDGLUE(gxCreateImageFile);
OSErr Forward_CreateImageFile (FSSpecPtr, long imageFileOptions, long *)
    = gxFORWARDGLUE;


OSErr Send_OpenConnection (void)
    = GXUNIVSENDGLUE(gxOpenConnection);
OSErr Forward_OpenConnection (void)
    = gxFORWARDGLUE;

OSErr Send_CloseConnection (void)
    = GXUNIVSENDGLUE(gxCloseConnection);
OSErr Forward_CloseConnection (void)
    = gxFORWARDGLUE;

OSErr Send_StartSendPage (gxFormat)
    = GXUNIVSENDGLUE(gxStartSendPage);
OSErr Forward_StartSendPage (gxFormat)
    = gxFORWARDGLUE;

OSErr Send_FinishSendPage (void)
    = GXUNIVSENDGLUE(gxFinishSendPage);
OSErr Forward_FinishSendPage (void)
    = gxFORWARDGLUE;


OSErr Send_WriteData (Ptr data, long length)
    = GXUNIVSENDGLUE(gxWriteData);
OSErr Forward_WriteData (Ptr data, long length)
    = gxFORWARDGLUE;

OSErr Send_BufferData (Ptr data, long length, long bufferOptions)
    = GXUNIVSENDGLUE(gxBufferData);
OSErr Forward_BufferData (Ptr data, long length, long bufferOptions)
    = gxFORWARDGLUE;

OSErr Send_DumpBuffer (gxPrintingBuffer *)
    = GXUNIVSENDGLUE(gxDumpBuffer);
OSErr Forward_DumpBuffer (gxPrintingBuffer *)
    = gxFORWARDGLUE;

OSErr Send_FreeBuffer (gxPrintingBuffer *)
    = GXUNIVSENDGLUE(gxFreeBuffer);
OSErr Forward_FreeBuffer (gxPrintingBuffer *)
    = gxFORWARDGLUE;


OSErr Send_CheckStatus (Ptr data, long length, short statusType, Signature owner)
    = GXUNIVSENDGLUE(gxCheckStatus);
OSErr Forward_CheckStatus (Ptr data, long length, short statusType, Signature owner)
    = gxFORWARDGLUE;

OSErr Send_GetDeviceStatus (Ptr cmdData, long cmdSize, Ptr responseData, long *responseSize, Str255 termination)
    = GXUNIVSENDGLUE(gxGetDeviceStatus);
OSErr Forward_GetDeviceStatus (Ptr cmdData, long cmdSize, Ptr responseData, long *responseSize, Str255 termination)
    = gxFORWARDGLUE;


OSErr Send_FetchTaggedData (ResType, short id, Handle *, Signature owner)
  = GXUNIVSENDGLUE(gxFetchTaggedData);
OSErr Forward_FetchTaggedData (ResType, short id, Handle *, Signature owner)
  = gxFORWARDGLUE;


OSErr Send_GetDTPMenuList (MenuHandle)
  = GXUNIVSENDGLUE(gxGetDTPMenuList);
OSErr Forward_GetDTPMenuList (MenuHandle)
  = gxFORWARDGLUE;

OSErr Send_DTPMenuSelect (short id, Boolean *done)
  = GXUNIVSENDGLUE(gxDtpMenuSelect);
OSErr Forward_DTPMenuSelect (short id, Boolean *done)
  = gxFORWARDGLUE;

OSErr Send_DTPMenuEventFilter (short id, Boolean *done, Boolean *handledEvent, EventRecord *)
  = GXUNIVSENDGLUE(gxDtpMenuEventFilter);
OSErr Forward_DTPMenuEventFilter (short id, Boolean *done, Boolean *handledEvent, EventRecord *)
  = gxFORWARDGLUE;
  

OSErr Send_JobFormatModeQuery (gxQueryType, void *srcData, void *dstData)
  = GXUNIVSENDGLUE(gxJobFormatModeQuery);
OSErr Forward_JobFormatModeQuery (gxQueryType, void *srcData, void *dstData)
  = gxFORWARDGLUE;


OSErr Send_WriteStatusToDTPWindow (gxStatusRecord *, gxDisplayRecord *)
  = GXUNIVSENDGLUE(gxWriteStatusToDTPWindow);
OSErr Forward_WriteStatusToDTPWindow (gxStatusRecord *, gxDisplayRecord *)
  = gxFORWARDGLUE;

OSErr Send_InitializeStatusAlert (gxStatusRecord *, DialogPtr *)
  = GXUNIVSENDGLUE(gxInitializeStatusAlert);
OSErr Forward_InitializeStatusAlert (gxStatusRecord *, DialogPtr *)
  = gxFORWARDGLUE;

OSErr Send_HandleAlertStatus (gxStatusRecord *)
  = GXUNIVSENDGLUE(gxHandleAlertStatus);
OSErr Forward_HandleAlertStatus (gxStatusRecord *)
  = gxFORWARDGLUE;
  
OSErr Send_HandleAlertEvent (gxStatusRecord *, DialogPtr, EventRecord *, short *response)
  = GXUNIVSENDGLUE(gxHandleAlertEvent);
OSErr Forward_HandleAlertEvent (gxStatusRecord *, DialogPtr, EventRecord *, short *response)
  = gxFORWARDGLUE;
    

void Send_CleanupStartJob (void)
  = GXUNIVSENDGLUE(gxCleanupStartJob);
void Forward_CleanupStartJob (void)
  = gxFORWARDGLUE;

void Send_CleanupStartPage (void)
  = GXUNIVSENDGLUE(gxCleanupStartPage);
void Forward_CleanupStartPage (void)
  = gxFORWARDGLUE;

void Send_CleanupOpenConnection (void)
  = GXUNIVSENDGLUE(gxCleanupOpenConnection);
void Forward_CleanupOpenConnection (void)
  = gxFORWARDGLUE;

void Send_CleanupStartSendPage (void)
  = GXUNIVSENDGLUE(gxCleanupStartSendPage);
void Forward_CleanupStartSendPage (void)
  = gxFORWARDGLUE;


OSErr Send_DefaultDesktopPrinter (Str31)
  = GXUNIVSENDGLUE(gxDefaultDesktopPrinter);
OSErr Forward_DefaultDesktopPrinter (Str31)
  = gxFORWARDGLUE;
  
OSErr Send_CaptureOutputDevice (Boolean capture)
  = GXUNIVSENDGLUE(gxCaptureOutputDevice);
OSErr Forward_CaptureOutputDevice (Boolean capture)
  = gxFORWARDGLUE;


OSErr Send_OpenConnectionRetry (ResType, void *, Boolean *, OSErr)
  = GXUNIVSENDGLUE(gxOpenConnectionRetry);
OSErr Forward_OpenConnectionRetry (ResType, void *, Boolean *, OSErr)
  = gxFORWARDGLUE;
  
OSErr Send_ExamineSpoolFile (gxSpoolFile)
  = GXUNIVSENDGLUE(gxExamineSpoolFile);
OSErr Forward_ExamineSpoolFile (gxSpoolFile)
  = gxFORWARDGLUE;

OSErr Send_FinishSendPlane (void)
  = GXUNIVSENDGLUE(gxFinishSendPlane);
OSErr Forward_FinishSendPlane (void)
  = gxFORWARDGLUE;

OSErr Send_DoesPaperFit( gxPaperType paper, Boolean *fits )
  = GXUNIVSENDGLUE(gxDoesPaperFit);
OSErr Forward_DoesPaperFit( gxPaperType paper, Boolean *fits )
  = gxFORWARDGLUE;
  
OSErr Send_ChooserMessage (short message, short caller, StringPtr objName,
                  StringPtr zoneName, ListHandle theList, long p2)
  = GXUNIVSENDGLUE(gxChooserMessage);
OSErr Forward_ChooserMessage (short message, short caller, StringPtr objName,
                    StringPtr zoneName, ListHandle theList, long p2)
  = gxFORWARDGLUE;
  

#ifdef __cplusplus
}
#endif


#endif __PRINTINGMESSAGES__
