/* graphics:  
  gxFont library interfaces
  by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Mike Reed, Oliver Steele, David Van Brink, Chris Yerga
  Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.
*/

#ifndef fontLibraryIncludes
#define fontLibraryIncludes

#ifndef gxFontTypesIncludes
  #include "font types.h"
#endif

#ifndef gxGraphicsTypesIncludes
  #include "graphics types.h"
#endif

#ifndef gxFontRoutinesIncludes
  #include "font routines.h"
#endif


/*  example gxFont descriptor tags
*/
#define weightVariationTag  0x77676874  /* 'wght' */
#define widthVariationTag   0x77647468  /* 'wdth' */
#define slantVariationTag   0x736c6e74  /* 'slnt' */
#define opticalSizeTag    0x6f70737a  /* 'opsz' */

/* weights for gxStyle matching */
/* these will have to be tweaked*/
/* this info could be in the gxFont???*/
#define prefwghtweighting 0x00010000
#define prefwdthweighting 0x00020000
#define prefslntweighting 0x00010000/*this is naturally weighted to last place*/
#define prefcontweighting 0x00040000

typedef enum commonFontEnums {
  firstCommonFont,
  chicagoFont = firstCommonFont,
  courierFont,
  genevaFont,
  helveticaFont,
  monacoFont,
  newyorkFont,
  symbolFont,
  timesFont,
  lastCommonFont = timesFont
} commonFontEnums;


enum{noMatching = 0,
  useStyleMatching,
  useVariationsMatching,
  useTextFaceMatching= 4
  };

typedef long commonFontEnum;

gxFont GetCommonFont(commonFontEnum);
void SetShapeCommonFont(gxShape, commonFontEnum);
void SetStyleCommonFont(gxStyle, commonFontEnum);

gxFont FindCNameFont(gxFontName meaning, const char name[]);
gxFont FindPNameFont(gxFontName meaning, const unsigned char name[]);

long FindFontCName(gxFont fontID, gxFontName meaning, char name[]);
long FindFontPName(gxFont fontID, gxFontName meaning, unsigned char name[]);

long FindStyleFontCName(gxStyle s, gxFontName meaning, char name[]);
long FindStyleFontPName(gxStyle s, gxFontName meaning, unsigned char name[]);
void SetStylePNamedFont(gxStyle s, const unsigned char name[]);
void SetStyleCNamedFont(gxStyle s, const char* name);

long CountFontFamilies(void);
gxFont FindFontFamily(long index, gxFontPlatform platform, gxFontScript script, gxFontLanguage language,
  long nameLength, const unsigned char *name);
long CountFontStyles(gxFont family);
gxFont FindFontStyle(gxFont family, long index, gxFontPlatform platform, gxFontScript script, gxFontLanguage language,
  long nameLength, const unsigned char *name);

void SetMatchingStyle(gxFont targetFamily, gxStyle theStyle, long matchInfo);
gxStyle ReturnMatchingStyle(gxFont targetFamily, gxStyle theStyle, long matchInfo);

#endif
