/* graphics:
  debugging routines
  by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
  Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.  */

#pragma once
#ifndef gxGraphicsDebuggingIncludes
  #define gxGraphicsDebuggingIncludes
  #ifndef gxGraphicsTypesIncludes
    #include "graphics types.h"
  #endif

  typedef enum {

  /* These levels tell how to validate routines.  Choose one. */
    gxNoValidation        = 0x00,   /* no validation */
    gxPublicValidation        = 0x01,   /* check parameters to public routines */
    gxInternalValidation      = 0x02,   /* check parameters to internal routines */

  /* These levels tell how to validate types.  Choose one. */
    gxTypeValidation        = 0x00,   /* check types of objects */
    gxStructureValidation     = 0x10,   /* check fields of private structures */
    gxAllObjectValidation     = 0x20,   /* check every object over every call */

  /* These levels tell how to validate memory manager blocks.  Choose any combination. */
    gxNoMemoryManagerValidation = 0x0000,
    gxApBlockValidation     = 0x0100, /* check the relevant block structures after each memory mgr. call */
    gxFontBlockValidation     = 0x0200, /* check the system gxHeap as well */
    gxApHeapValidation      = 0x0400, /* check the memory managers gxHeap after every mem. call */
    gxFontHeapValidation      = 0x0800, /* check the system gxHeap as well */
    gxCheckApHeapValidation   = 0x1000, /* check the memory managers gxHeap if checking routine parameters */
    gxCheckFontHeapValidation   = 0x2000    /* check the system gxHeap as well */

  } gxValidationLevels;

  typedef long gxValidationLevel;

  typedef enum {
    gxNoDrawError,

    /* gxShape type errors */
    gxShape_emptyType,
    gxShape_inverse_fullType,
    gxRectangle_zero_width,
    gxRectangle_zero_height,
    gxPolygon_empty,
    gxPath_empty,
    gxBitmap_zero_width,
    gxBitmap_zero_height,
    gxText_empty,
    gxGlyph_empty,
    gxLayout_empty,
    gxPicture_empty,

    /* general gxShape errors */
    gxShape_no_fill,
    gxShape_no_enclosed_area,
    gxShape_no_enclosed_pixels,
    gxShape_very_small,
    gxShape_very_large,
    gxShape_contours_cancel,

    /* gxStyle errors */
    gxPen_too_small,
    gxText_size_too_small,
    gxDash_empty,
    gxStart_cap_empty,
    gxPattern_empty,
    gxTextFace_empty,
    gxShape_primitive_empty,
    gxShape_primitive_very_small,

    /* gxInk errors */
    gxTransfer_equals_noMode,
    gxTransfer_matrix_ignores_source,
    gxTransfer_matrix_ignores_device,
    gxTransfer_source_reject,
    gxTransfer_mode_ineffective,
    gxColorSet_no_entries,
    gxBitmap_colorSet_one_entry,

    /* gxTransform errors */
    gxTransform_scale_too_small,
    gxTransform_map_too_large,
    gxTransform_move_too_large,
    gxTransform_scale_too_large,
    gxTransform_rotate_too_large,
    gxTransform_perspective_too_large,
    gxTransform_skew_too_large,
    gxTransform_clip_no_intersection,
    gxTransform_clip_empty,
    gxTransform_no_viewPorts,

    /* gxViewPort errors */
    gxViewPort_disposed,
    gxViewPort_clip_empty,
    gxViewPort_clip_no_intersection,
    gxViewPort_scale_too_small,
    gxViewPort_map_too_large,
    gxViewPort_move_too_large,
    gxViewPort_scale_too_large,
    gxViewPort_rotate_too_large,
    gxViewPort_perspective_too_large,
    gxViewPort_skew_too_large,
    gxViewPort_viewGroup_offscreen,

    /* gxViewDevice errors */
    gxViewDevice_clip_no_intersection,
    gxViewDevice_scale_too_small,
    gxViewDevice_map_too_large,
    gxViewDevice_move_too_large,
    gxViewDevice_scale_too_large,
    gxViewDevice_rotate_too_large,
    gxViewDevice_perspective_too_large,
    gxViewDevice_skew_too_large
  } gxDrawErrors;

  typedef long gxDrawError;

  typedef gxBoolean (*gxUserDebugFunction)(const char *str, long message, long reference);

  #ifdef appleInternal
    #define GXInlineCode(x)
  #endif
  #ifndef GXInlineCode
    #define GXInlineCode(x) = {0x303C, x, 0xA832}
  #endif

  #ifdef __cplusplus
  extern "C" {
  #endif

  gxDrawError GXGetShapeDrawError(gxShape source) GXInlineCode(0x1173);

  void GXValidateAll(void) GXInlineCode(0x117E);
  void GXValidateColorSet(gxColorSet) GXInlineCode(0x1174);
  void GXValidateColorProfile(gxColorProfile) GXInlineCode(0x1175);
  void GXValidateGraphicsClient(gxGraphicsClient) GXInlineCode(0x3006);
  void GXValidateInk(gxInk) GXInlineCode(0x1176);
  void GXValidateShape(gxShape) GXInlineCode(0x1177);
  void GXValidateStyle(gxStyle) GXInlineCode(0x1178);
  void GXValidateTag(gxTag) GXInlineCode(0x1179);
  void GXValidateTransform(gxTransform) GXInlineCode(0x117A);
  void GXValidateViewDevice(gxViewDevice) GXInlineCode(0x117B);
  void GXValidateViewPort(gxViewPort) GXInlineCode(0x117C);
  void GXValidateViewGroup(gxViewGroup) GXInlineCode(0x117D);

  gxValidationLevel GXGetValidation(void) GXInlineCode(0x2011);
  void GXSetValidation(gxValidationLevel) GXInlineCode(0x2012);

  long GXGetValidationError(char *procedureName, void **argument, long *argumentNumber) GXInlineCode(0x1244);
  gxBoolean GXGetGraphicsBugParametersPointer(struct graphicsBugParameters **blockPointer) GXInlineCode(0x125C);
  gxUserDebugFunction GXGetUserGraphicsDebug(long *reference) GXInlineCode(0x125D);
  void GXSetUserGraphicsDebug(gxUserDebugFunction userFunction, long reference) GXInlineCode(0x125E);

  #ifdef __cplusplus
  }
  #endif

  #undef GXInlineCode

#endif
