/* graphics:
  public interface definition
  by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Josh Horwich, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
  Copyright 1987 - 1993 Apple Computer, Inc.  All rights reserved.

  Thanks to Bruce Leak for taking many mushy ideas and making them consistent and logical.
  Thanks to Mike Reed for the Mathematica, Athens, conics, cubics and the tireless (but well-voiced) concerns.
  Thanks to Bill Atkinson, Ernie Beernink for the inspiration and the well-trodden gxPath.
  Thanks to Jerome Coonen for the permission to do this in the first place.
  Thanks to Jim Batson and Gifford Calenda for not getting in the way.
  Thanks to Sampo Kaasila for the vector proofs, rotated parabolas and the gxFont scaler.
  Thanks to Scott Knaster, Will Stein, Jim Friedlander, Gene Pope, Laurie Girand and a host of others for
    their unflagging support, without which this project would have been abandoned long ago.
  Thanks to Andrew Singer and Michael Kahl for the speedy development environment.

  The sorting routine is originally from Knuth, coded in assembly by B. Atkinson and then in C by Galyn Susman.
  The matrix routines and gxCurve walking routines were originally coded by Sampo Kaasila.
  The gxPolygon and gxPath blitting edge walk was developed originally by Art Cabral.
  The gxHalftone matrix was developed by Daniel I. Lipton.
  The gxColor matching algorithms were developed by Robin Myers.
  The code was developed in C (not C++) and assembly in THINK C 1.0, 2.0, 3.0, 4.0, and 5.0, and MPW 3.0, 3.1 and 3.2.
*/

#pragma once

#ifndef gxGraphicsRoutinesIncludes
#define gxGraphicsRoutinesIncludes

#ifndef gxGraphicsTypesIncludes
  #include "graphics types.h"
#endif
#ifndef gxGraphicsErrorsIncludes
  #include "graphics errors.h"
#endif
#ifndef gxFontTypesIncludes
  #include "font types.h"
#endif
#ifndef memoryTypesIncludes
  #include "memory types.h"
#endif


#ifdef appleInternal
  #define GXInlineCode(x)
#endif
#ifndef GXInlineCode
  #define GXInlineCode(x) = {0x303C, x, 0xA832}
#endif

#ifdef __cplusplus
extern "C" {
#endif

long GXCountGraphicsFunctions(const unsigned char function[]) GXInlineCode(0x300F);
void GXInsertGraphicsFunction(const unsigned char function[], long index, void *address, long identifier, long reference,
  gxViewDevice device) GXInlineCode(0x3010);
void *GXGetGraphicsFunction(const unsigned char function[], long index, long *identifier, long *reference,
  gxViewDevice *device) GXInlineCode(0x3011);
void GXSetGraphicsFunction(const unsigned char function[], long index, void *address, long identifier, long reference,
  gxViewDevice device) GXInlineCode(0x3012);
void GXRemoveGraphicsFunction(const unsigned char function[], long index) GXInlineCode(0x3013);

void *GXGetGraphicsFunctionAddress(const unsigned char function[]) GXInlineCode(0x3017);

long GXGetGraphicsFunctionReference(void) GXInlineCode(0x3014);
gxBoolean GXGetGraphicsFunctionDone(long *result) GXInlineCode(0x3015);
void GXSetGraphicsFunctionDone(gxBoolean isDone, long result) GXInlineCode(0x3016);

gxGraphicsClient GXNewGraphicsClient(void *memoryStart, long memoryLength, long separateStack) GXInlineCode(0x3000);
gxGraphicsClient GXGetGraphicsClient(void) GXInlineCode(0x3001);
void GXSetGraphicsClient(gxGraphicsClient client) GXInlineCode(0x3002);
void GXDisposeGraphicsClient(gxGraphicsClient client) GXInlineCode(0x3003);
gxHeap *GXGetGraphicsClientHeap(gxGraphicsClient client) GXInlineCode(0x3004);

long GXGetGraphicsClients(long index, long count, gxGraphicsClient clients[]) GXInlineCode(0x3005); /*returns the count */

void GXEnterGraphics(void) GXInlineCode(0x2000);
void GXExitGraphics(void) GXInlineCode(0x1000);

gxGraphicsError GXGetGraphicsError(gxGraphicsError *stickyError) GXInlineCode(0x2001);
gxGraphicsNotice GXGetGraphicsNotice(gxGraphicsNotice *stickyNotice) GXInlineCode(0x2002);
gxGraphicsWarning GXGetGraphicsWarning(gxGraphicsWarning *stickyWarning) GXInlineCode(0x2003);
void GXPostGraphicsError(gxGraphicsError error) GXInlineCode(0x2004);
void GXPostGraphicsNotice(gxGraphicsNotice notice) GXInlineCode(0x2005);
void GXPostGraphicsWarning(gxGraphicsWarning warning) GXInlineCode(0x2006);
gxUserErrorFunction GXGetUserGraphicsError(long *reference) GXInlineCode(0x2007);
gxUserNoticeFunction GXGetUserGraphicsNotice(long *reference) GXInlineCode(0x2008);
gxUserWarningFunction GXGetUserGraphicsWarning(long *reference) GXInlineCode(0x2009);
void GXSetUserGraphicsError(gxUserErrorFunction userFunction, long reference) GXInlineCode(0x200A);
void GXSetUserGraphicsNotice(gxUserNoticeFunction userFunction, long reference) GXInlineCode(0x200B);
void GXSetUserGraphicsWarning(gxUserWarningFunction userFunction, long reference) GXInlineCode(0x200C);

void GXIgnoreGraphicsNotice(gxGraphicsNotice notice) GXInlineCode(0x200D);
void GXIgnoreGraphicsWarning(gxGraphicsWarning warning) GXInlineCode(0x200E);
void GXPopGraphicsNotice(void) GXInlineCode(0x200F);
void GXPopGraphicsWarning(void) GXInlineCode(0x2010);

gxShape GXNewShapeVector(gxShapeType aType, const fixed vector[]) GXInlineCode(0x1001);
void GXSetShapeVector(gxShape target, const fixed vector[]) GXInlineCode(0x1002);

gxShape GXNewBitmap(const gxBitmap *data, const gxPoint *position) GXInlineCode(0x1003);
gxShape GXNewCurve(const gxCurve *data) GXInlineCode(0x1004);
gxShape GXNewGlyphs(long charCount, const unsigned char text[], const gxPoint positions[], const long advance[], 
  const gxPoint tangents[], const short styleRuns[], const gxStyle glyphStyles[]) GXInlineCode(0x1005);
gxShape GXNewLine(const gxLine *data) GXInlineCode(0x1006);
gxShape GXNewPaths(const gxPaths *data) GXInlineCode(0x1007);
gxShape GXNewPicture(long count, const gxShape shapes[], const gxStyle styles[], const gxInk inks[], const gxTransform transforms[]) GXInlineCode(0x1008);
gxShape GXNewPoint(const gxPoint *data) GXInlineCode(0x1009);
gxShape GXNewPolygons(const gxPolygons *data) GXInlineCode(0x100A);
gxShape GXNewRectangle(const gxRectangle *data) GXInlineCode(0x100B);
gxShape GXNewText(long charCount, const unsigned char text[], const gxPoint *position) GXInlineCode(0x100C);

gxBitmap *GXGetBitmap(gxShape source, gxBitmap *data, gxPoint *position) GXInlineCode(0x100D);
gxCurve *GXGetCurve(gxShape source, gxCurve *data) GXInlineCode(0x100E);
long GXGetGlyphs(gxShape source, long *charCount, unsigned char text[], gxPoint positions[], long advance[], gxPoint tangents[],
  long *runCount, short styleRuns[], gxStyle glyphStyles[]) GXInlineCode(0x100F);         /* returns byte length of glyphs */
gxLine *GXGetLine(gxShape source, gxLine *data) GXInlineCode(0x1010);
long GXGetPaths(gxShape source, gxPaths *data) GXInlineCode(0x1011);                  /* returns byte length */
long GXGetPicture(gxShape source, gxShape shapes[], gxStyle styles[], gxInk inks[], gxTransform transforms[]) GXInlineCode(0x1012); /* returns count */
gxPoint *GXGetPoint(gxShape source, gxPoint *data) GXInlineCode(0x1013);
long GXGetPolygons(gxShape source, gxPolygons *data) GXInlineCode(0x1014);              /* returns byte length */
gxRectangle *GXGetRectangle(gxShape source, gxRectangle *data) GXInlineCode(0x1015);
long GXGetText(gxShape source, long *charCount, unsigned char text[], gxPoint *position) GXInlineCode(0x1016);    /* returns byte length */

void GXSetBitmap(gxShape target, const gxBitmap *data, const gxPoint *position) GXInlineCode(0x1017);
void GXSetCurve(gxShape target, const gxCurve *data) GXInlineCode(0x1018);
void GXSetGlyphs(gxShape target, long charCount, const unsigned char text[], const gxPoint positions[], const long advance[], 
  const gxPoint tangents[], const short styleRuns[], const gxStyle glyphStyles[]) GXInlineCode(0x1019);
void GXSetLine(gxShape target, const gxLine *data) GXInlineCode(0x101A);
void GXSetPaths(gxShape target, const gxPaths *data) GXInlineCode(0x101B);
void GXSetPicture(gxShape target, long count, const gxShape shapes[], const gxStyle styles[], const gxInk inks[], const gxTransform transforms[]) GXInlineCode(0x101C);
void GXSetPoint(gxShape target, const gxPoint *data) GXInlineCode(0x101D);
void GXSetPolygons(gxShape target, const gxPolygons *data) GXInlineCode(0x101E);
void GXSetRectangle(gxShape target, const gxRectangle *data) GXInlineCode(0x101F);
void GXSetText(gxShape target, long charCount, const unsigned char text[], const gxPoint *position) GXInlineCode(0x1020);

void GXDrawBitmap(const gxBitmap *data, const gxPoint *position) GXInlineCode(0x1021);
void GXDrawCurve(const gxCurve *data) GXInlineCode(0x1022);
void GXDrawGlyphs(long charCount, const unsigned char text[], const gxPoint positions[], const long advance[], 
  const gxPoint tangents[], const short styleRuns[], const gxStyle glyphStyles[]) GXInlineCode(0x1023);
void GXDrawLine(const gxLine *data) GXInlineCode(0x1024);
void GXDrawPaths(const gxPaths *data, gxShapeFill fill) GXInlineCode(0x1025);
void GXDrawPicture(long count, const gxShape shapes[], const gxStyle styles[], const gxInk inks[], const gxTransform transforms[]) GXInlineCode(0x1026);
void GXDrawPoint(const gxPoint *data) GXInlineCode(0x1027);
void GXDrawPolygons(const gxPolygons *data, gxShapeFill fill) GXInlineCode(0x1028);
void GXDrawRectangle(const gxRectangle *data, gxShapeFill fill) GXInlineCode(0x1029);
void GXDrawText(long charCount, const unsigned char text[], const gxPoint *position) GXInlineCode(0x102A);

gxColorProfile GXNewColorProfile(const gxProfileRecord *profile, const gxProfileResponse *responses) GXInlineCode(0x102B);
gxColorSet GXNewColorSet(gxColorSpace space, long count, const gxSetColor colors[]) GXInlineCode(0x102C);
gxInk GXNewInk(void) GXInlineCode(0x102D);
gxShape GXNewShape(gxShapeType aType) GXInlineCode(0x102E);
gxStyle GXNewStyle(void) GXInlineCode(0x102F);
gxTag GXNewTag(long tagType, long length, const void *data) GXInlineCode(0x1030);
gxTransform GXNewTransform(void) GXInlineCode(0x1031);
gxViewDevice GXNewViewDevice(gxViewGroup group, gxShape bitmapShape) GXInlineCode(0x1062);
gxViewGroup GXNewViewGroup(void) GXInlineCode(0x1060);
gxViewPort GXNewViewPort(gxViewGroup group) GXInlineCode(0x1063);

void GXDisposeColorProfile(gxColorProfile target) GXInlineCode(0x1032);
void GXDisposeColorSet(gxColorSet target) GXInlineCode(0x1033);
void GXDisposeInk(gxInk target) GXInlineCode(0x1034);
void GXDisposeShape(gxShape target) GXInlineCode(0x1035);
void GXDisposeStyle(gxStyle target) GXInlineCode(0x1036);
void GXDisposeTag(gxTag target) GXInlineCode(0x1037);
void GXDisposeTransform(gxTransform target) GXInlineCode(0x1038);
void GXDisposeViewDevice(gxViewDevice target) GXInlineCode(0x1064);
void GXDisposeViewGroup(gxViewGroup target) GXInlineCode(0x1061);
void GXDisposeViewPort(gxViewPort target) GXInlineCode(0x1065);

gxColorProfile GXCloneColorProfile(gxColorProfile source) GXInlineCode(0x1039);
gxColorSet GXCloneColorSet(gxColorSet source) GXInlineCode(0x103A);
gxInk GXCloneInk(gxInk source) GXInlineCode(0x103B);
gxShape GXCloneShape(gxShape source) GXInlineCode(0x103C);
gxStyle GXCloneStyle(gxStyle source) GXInlineCode(0x103D);
gxTag GXCloneTag(gxTag source) GXInlineCode(0x103E);
gxTransform GXCloneTransform(gxTransform source) GXInlineCode(0x103F);

gxColorProfile GXCopyToColorProfile(gxColorProfile target, gxColorProfile source) GXInlineCode(0x1040);
gxColorSet GXCopyToColorSet(gxColorSet target, gxColorSet source) GXInlineCode(0x1041);
gxInk GXCopyToInk(gxInk target, gxInk source) GXInlineCode(0x1042);
gxShape GXCopyToShape(gxShape target, gxShape source) GXInlineCode(0x1043);
gxStyle GXCopyToStyle(gxStyle target, gxStyle source) GXInlineCode(0x1044);
gxTag GXCopyToTag(gxTag target, gxTag source) GXInlineCode(0x1045);
gxTransform GXCopyToTransform(gxTransform target, gxTransform source) GXInlineCode(0x1046);
gxViewDevice GXCopyToViewDevice(gxViewDevice target, gxViewDevice source) GXInlineCode(0x1066);
gxViewPort GXCopyToViewPort(gxViewPort target, gxViewPort source) GXInlineCode(0x1067);

gxBoolean GXEqualColorProfile(gxColorProfile one, gxColorProfile two) GXInlineCode(0x1047);
gxBoolean GXEqualColorSet(gxColorSet one, gxColorSet two) GXInlineCode(0x1048);
gxBoolean GXEqualInk(gxInk one, gxInk two) GXInlineCode(0x1049);
gxBoolean GXEqualShape(gxShape one, gxShape two) GXInlineCode(0x104A);
gxBoolean GXEqualStyle(gxStyle one, gxStyle two) GXInlineCode(0x104B);
gxBoolean GXEqualTag(gxTag one, gxTag two) GXInlineCode(0x104C);
gxBoolean GXEqualTransform(gxTransform one, gxTransform two) GXInlineCode(0x104D);
gxBoolean GXEqualViewDevice(gxViewDevice one, gxViewDevice two) GXInlineCode(0x1068);
gxBoolean GXEqualViewPort(gxViewPort one, gxViewPort two) GXInlineCode(0x1069);

void GXResetInk(gxInk target) GXInlineCode(0x104E);
void GXResetShape(gxShape target) GXInlineCode(0x104F);
void GXResetStyle(gxStyle target) GXInlineCode(0x1050);
void GXResetTransform(gxTransform target) GXInlineCode(0x1051);

void GXLoadColorProfile(gxColorProfile target) GXInlineCode(0x1052);
void GXLoadColorSet(gxColorSet target) GXInlineCode(0x1053);
void GXLoadInk(gxInk target) GXInlineCode(0x1054);
void GXLoadShape(gxShape target) GXInlineCode(0x1055);
void GXLoadStyle(gxStyle target) GXInlineCode(0x1056);
void GXLoadTag(gxTag target) GXInlineCode(0x1057);
void GXLoadTransform(gxTransform target) GXInlineCode(0x1058);

void GXUnloadColorProfile(gxColorProfile target) GXInlineCode(0x1059);
void GXUnloadColorSet(gxColorSet target) GXInlineCode(0x105A);
void GXUnloadInk(gxInk target) GXInlineCode(0x105B);
void GXUnloadShape(gxShape target) GXInlineCode(0x105C);
void GXUnloadStyle(gxStyle target) GXInlineCode(0x105D);
void GXUnloadTag(gxTag target) GXInlineCode(0x105E);
void GXUnloadTransform(gxTransform target) GXInlineCode(0x105F);

void GXCacheShape(gxShape source) GXInlineCode(0x106B);
gxShape GXCopyDeepToShape(gxShape target, gxShape source) GXInlineCode(0x106A);
void GXDrawShape(gxShape source) GXInlineCode(0x106C);
void GXDisposeShapeCache(gxShape target) GXInlineCode(0x106D);

gxColorProfile GXGetDefaultColorProfile(void) GXInlineCode(0x121B);
gxShape GXGetDefaultShape(gxShapeType aType) GXInlineCode(0x106E);
gxColorSet GXGetDefaultColorSet( long pixelDepth ) GXInlineCode(0x1256);


void GXSetDefaultColorProfile(gxColorProfile target) GXInlineCode(0x121C);
void GXSetDefaultShape(gxShape target) GXInlineCode(0x106F);
void GXSetDefaultColorSet( gxColorSet target, long pixelDepth ) GXInlineCode(0x1257);

long GXGetTag(gxTag source, long *tagType, void *data) GXInlineCode(0x1070);
void GXSetTag(gxTag target, long tagType, long length, const void *data) GXInlineCode(0x1071);

gxRectangle *GXGetShapeBounds(gxShape source, long index, gxRectangle *bounds) GXInlineCode(0x1072);
gxShapeFill GXGetShapeFill(gxShape source) GXInlineCode(0x1073);
gxInk GXGetShapeInk(gxShape source) GXInlineCode(0x1074);
long GXGetShapePixel(gxShape source, long x, long y, gxColor *data, long *index) GXInlineCode(0x1075);
gxStyle GXGetShapeStyle(gxShape source) GXInlineCode(0x1076);
gxTransform GXGetShapeTransform(gxShape source) GXInlineCode(0x1077);
gxShapeType GXGetShapeType(gxShape source) GXInlineCode(0x1078);
gxRectangle *GXGetShapeTypographicBounds(gxShape source, gxRectangle *bounds) GXInlineCode(0x124E);
gxShape GXGetBitmapParts(gxShape source, const gxLongRectangle *bounds) GXInlineCode(0x1079);
void GXGetStyleFontMetrics(gxStyle sourceStyle, gxPoint* before, gxPoint* after, gxPoint* caretAngle, gxPoint* caretOffset) GXInlineCode(0x1258);
void GXGetShapeFontMetrics(gxShape source, gxPoint* before, gxPoint* after, gxPoint* caretAngle, gxPoint* caretOffset) GXInlineCode(0x1259);

void GXSetShapeBounds(gxShape target, const gxRectangle *newBounds) GXInlineCode(0x107A);
void GXSetShapeFill(gxShape target, gxShapeFill newFill) GXInlineCode(0x107B);
void GXSetShapeInk(gxShape target, gxInk newInk) GXInlineCode(0x107C);
void GXSetShapePixel(gxShape target, long x, long y, const gxColor *newColor, long newIndex) GXInlineCode(0x107D);
void GXSetShapeStyle(gxShape target, gxStyle newStyle) GXInlineCode(0x107E);
void GXSetShapeTransform(gxShape target, gxTransform newTransform) GXInlineCode(0x107F);
void GXSetShapeType(gxShape target, gxShapeType newType) GXInlineCode(0x1080);
void GXSetBitmapParts(gxShape target, const gxLongRectangle *bounds, gxShape bitmapShape) GXInlineCode(0x1081);

void GXSetShapeGeometry(gxShape target, gxShape geometry) GXInlineCode(0x1082);

fixed GXGetShapeCurveError(gxShape source) GXInlineCode(0x1083);
gxDashRecord *GXGetShapeDash(gxShape source, gxDashRecord *dash) GXInlineCode(0x1084);
gxCapRecord *GXGetShapeCap(gxShape source, gxCapRecord *cap) GXInlineCode(0x1085);
long GXGetShapeFace(gxShape source, gxTextFace *face) GXInlineCode(0x1086);     /* returns the number of layers */
gxFont GXGetShapeFont(gxShape source) GXInlineCode(0x1087);
gxJoinRecord *GXGetShapeJoin(gxShape source, gxJoinRecord *join) GXInlineCode(0x1088);
fract GXGetShapeJustification(gxShape source) GXInlineCode(0x1089);
gxPatternRecord *GXGetShapePattern(gxShape source, gxPatternRecord *pattern) GXInlineCode(0x108A);
fixed GXGetShapePen(gxShape source) GXInlineCode(0x108B);
gxFontPlatform GXGetShapeEncoding(gxShape source, gxFontScript *script, gxFontLanguage *language) GXInlineCode(0x108C);
fixed GXGetShapeTextSize(gxShape source) GXInlineCode(0x108E);
long GXGetShapeFontVariations(gxShape source, gxFontVariation variations[]) GXInlineCode(0x108F);

fixed GXGetStyleCurveError(gxStyle source) GXInlineCode(0x1090);
gxDashRecord *GXGetStyleDash(gxStyle source, gxDashRecord *dash) GXInlineCode(0x1091);
gxCapRecord *GXGetStyleCap(gxStyle source, gxCapRecord *cap) GXInlineCode(0x1092);
long GXGetStyleFace(gxStyle source, gxTextFace *face) GXInlineCode(0x1093);     /* returns the number of layers */
gxFont GXGetStyleFont(gxStyle source) GXInlineCode(0x1094);
gxJoinRecord *GXGetStyleJoin(gxStyle source, gxJoinRecord *join) GXInlineCode(0x1095);
fract GXGetStyleJustification(gxStyle source) GXInlineCode(0x1096);
gxPatternRecord *GXGetStylePattern(gxStyle source, gxPatternRecord *pattern) GXInlineCode(0x1097);
fixed GXGetStylePen(gxStyle source) GXInlineCode(0x1098);
gxFontPlatform GXGetStyleEncoding(gxStyle source, gxFontScript *script, gxFontLanguage *language) GXInlineCode(0x1099);
fixed GXGetStyleTextSize(gxStyle source) GXInlineCode(0x109B);
long GXGetStyleFontVariations(gxStyle source, gxFontVariation variations[]) GXInlineCode(0x109C);

void GXSetShapeCurveError(gxShape target, fixed error) GXInlineCode(0x109D);
void GXSetShapeDash(gxShape target, const gxDashRecord *dash) GXInlineCode(0x109E);
void GXSetShapeCap(gxShape target, const gxCapRecord *cap) GXInlineCode(0x109F);
void GXSetShapeFace(gxShape target, const gxTextFace *face) GXInlineCode(0x10A0);
void GXSetShapeFont(gxShape target, gxFont aFont) GXInlineCode(0x10A1);
void GXSetShapeJoin(gxShape target, const gxJoinRecord *join) GXInlineCode(0x10A2);
void GXSetShapeJustification(gxShape target, fract justify) GXInlineCode(0x10A3);
void GXSetShapePattern(gxShape target, const gxPatternRecord *pattern) GXInlineCode(0x10A4);
void GXSetShapePen(gxShape target, fixed pen) GXInlineCode(0x10A5);
void GXSetShapeEncoding(gxShape target, gxFontPlatform platform, gxFontScript script, gxFontLanguage language) GXInlineCode(0x10A6);
void GXSetShapeTextSize(gxShape target, fixed size) GXInlineCode(0x10A8);
void GXSetShapeFontVariations(gxShape target, long count, const gxFontVariation variations[]) GXInlineCode(0x10A9);

void GXSetStyleCurveError(gxStyle target, fixed error) GXInlineCode(0x10AA);
void GXSetStyleDash(gxStyle target, const gxDashRecord *dash) GXInlineCode(0x10AB);
void GXSetStyleCap(gxStyle target, const gxCapRecord *cap) GXInlineCode(0x10AC);
void GXSetStyleFace(gxStyle target, const gxTextFace *face) GXInlineCode(0x10AD);
void GXSetStyleFont(gxStyle target, gxFont aFont) GXInlineCode(0x10AE);
void GXSetStyleJoin(gxStyle target, const gxJoinRecord *join) GXInlineCode(0x10AF);
void GXSetStyleJustification(gxStyle target, fract justify) GXInlineCode(0x10B0);
void GXSetStylePattern(gxStyle target, const gxPatternRecord *pattern) GXInlineCode(0x10B1);
void GXSetStylePen(gxStyle target, fixed pen) GXInlineCode(0x10B2);
void GXSetStyleEncoding(gxStyle target, gxFontPlatform platform, gxFontScript script, gxFontLanguage language) GXInlineCode(0x10B3);
void GXSetStyleTextSize(gxStyle target, fixed size) GXInlineCode(0x10B5);
void GXSetStyleFontVariations(gxStyle target, long count, const gxFontVariation variations[]) GXInlineCode(0x10B6);

gxColor *GXGetShapeColor(gxShape source, gxColor *data) GXInlineCode(0x10B7);
gxTransferMode *GXGetShapeTransfer(gxShape source, gxTransferMode *data) GXInlineCode(0x10B8);

gxColor *GXGetInkColor(gxInk source, gxColor *data) GXInlineCode(0x10B9);
gxTransferMode *GXGetInkTransfer(gxInk source, gxTransferMode *data) GXInlineCode(0x10BA);

void GXSetShapeColor(gxShape target, const gxColor *data) GXInlineCode(0x10BB);
void GXSetShapeTransfer(gxShape target, const gxTransferMode *data) GXInlineCode(0x10BC);

void GXSetInkColor(gxInk target, const gxColor *data) GXInlineCode(0x10BD);
void GXSetInkTransfer(gxInk target, const gxTransferMode *data) GXInlineCode(0x10BE);

gxShape GXGetShapeClip(gxShape source) GXInlineCode(0x10BF);
gxMapping *GXGetShapeMapping(gxShape source, gxMapping *map) GXInlineCode(0x10C0);
gxShapePart GXGetShapeHitTest(gxShape source, fixed *tolerance) GXInlineCode(0x10C1);
long GXGetShapeViewPorts(gxShape source, gxViewPort list[]) GXInlineCode(0x10C2);

gxShape GXGetTransformClip(gxTransform source) GXInlineCode(0x10C3);
gxMapping *GXGetTransformMapping(gxTransform source, gxMapping *map) GXInlineCode(0x10C4);
gxShapePart GXGetTransformHitTest(gxTransform source, fixed *tolerance) GXInlineCode(0x10C5);
long GXGetTransformViewPorts(gxTransform source, gxViewPort list[]) GXInlineCode(0x10C6);

void GXSetShapeClip(gxShape target, gxShape clip) GXInlineCode(0x10C7);
void GXSetShapeMapping(gxShape target, const gxMapping *map) GXInlineCode(0x10C8);
void GXSetShapeHitTest(gxShape target, gxShapePart mask, fixed tolerance) GXInlineCode(0x10C9);
void GXSetShapeViewPorts(gxShape target, long count, const gxViewPort list[]) GXInlineCode(0x10CA);

void GXSetTransformClip(gxTransform target, gxShape clip) GXInlineCode(0x10CB);
void GXSetTransformMapping(gxTransform target, const gxMapping *map) GXInlineCode(0x10CC);
void GXSetTransformHitTest(gxTransform target, gxShapePart mask, fixed tolerance) GXInlineCode(0x10CD);
void GXSetTransformViewPorts(gxTransform target, long count, const gxViewPort list[]) GXInlineCode(0x10CE);

long GXGetColorSet(gxColorSet source, gxColorSpace *space, gxSetColor colors[]) GXInlineCode(0x10CF);
long GXGetColorProfile(gxColorProfile source, gxProfileRecord *profile, gxProfileResponse *responses) GXInlineCode(0x10D0);

void GXSetColorSet(gxColorSet target, gxColorSpace space, long count, const gxSetColor colors[]) GXInlineCode(0x10D1);
void GXSetColorProfile(gxColorProfile target, const gxProfileRecord *profile, const gxProfileResponse *responses) GXInlineCode(0x10D2);

gxShape GXGetViewDeviceBitmap(gxViewDevice source) GXInlineCode(0x10D3);
gxShape GXGetViewDeviceClip(gxViewDevice source) GXInlineCode(0x10D4);
gxMapping *GXGetViewDeviceMapping(gxViewDevice source, gxMapping *map) GXInlineCode(0x10D5);
gxViewGroup GXGetViewDeviceViewGroup(gxViewDevice source) GXInlineCode(0x10D6);

void GXSetViewDeviceBitmap(gxViewDevice target, gxShape bitmapShape) GXInlineCode(0x10D7);
void GXSetViewDeviceClip(gxViewDevice target, gxShape clip) GXInlineCode(0x10D8);
void GXSetViewDeviceMapping(gxViewDevice target, const gxMapping *map) GXInlineCode(0x10D9);
void GXSetViewDeviceViewGroup(gxViewDevice target, gxViewGroup group) GXInlineCode(0x10DA);

long GXGetViewPortChildren(gxViewPort source, gxViewPort list[]) GXInlineCode(0x10DB);
gxShape GXGetViewPortClip(gxViewPort source) GXInlineCode(0x10DC);
long GXGetViewPortDither(gxViewPort source) GXInlineCode(0x10DD);
gxBoolean GXGetViewPortHalftone(gxViewPort source, gxHalftone *data) GXInlineCode(0x10DE);
gxMapping *GXGetViewPortMapping(gxViewPort source, gxMapping *map) GXInlineCode(0x10DF);
gxViewPort GXGetViewPortParent(gxViewPort source) GXInlineCode(0x10E0);
gxViewGroup GXGetViewPortViewGroup(gxViewPort source) GXInlineCode(0x10E1);

void GXSetViewPortChildren(gxViewPort target, long count, const gxViewPort list[]) GXInlineCode(0x10E2);
void GXSetViewPortClip(gxViewPort target, gxShape clip) GXInlineCode(0x10E3);
void GXSetViewPortDither(gxViewPort target, long level) GXInlineCode(0x10E4);
void GXSetViewPortHalftone(gxViewPort target, const gxHalftone *data) GXInlineCode(0x10E5);
void GXSetViewPortMapping(gxViewPort target, const gxMapping *map) GXInlineCode(0x10E6);
void GXSetViewPortParent(gxViewPort target, gxViewPort parent) GXInlineCode(0x10E7);
void GXSetViewPortViewGroup(gxViewPort target, gxViewGroup group) GXInlineCode(0x10E8);

long GXGetColorProfileTags(gxColorProfile source, long tagType, long index, long count, gxTag items[]) GXInlineCode(0x10E9);
long GXGetColorSetTags(gxColorSet source, long tagType, long index, long count, gxTag items[]) GXInlineCode(0x10EA);
long GXGetInkTags(gxInk source, long tagType, long index, long count, gxTag items[]) GXInlineCode(0x10EB);
long GXGetShapeTags(gxShape source, long tagType, long index, long count, gxTag items[]) GXInlineCode(0x10EC);
long GXGetStyleTags(gxStyle source, long tagType, long index, long count, gxTag items[]) GXInlineCode(0x10ED);
long GXGetTransformTags(gxTransform source, long tagType, long index, long count, gxTag items[]) GXInlineCode(0x10EE);
long GXGetViewDeviceTags(gxViewDevice source, long tagType, long index, long count, gxTag items[]) GXInlineCode(0x10EF);
long GXGetViewPortTags(gxViewPort source, long tagType, long index, long count, gxTag items[]) GXInlineCode(0x10F0);

void GXSetColorProfileTags(gxColorProfile target, long tagType, long index, long oldCount, long newCount, const gxTag items[]) GXInlineCode(0x10F1);
void GXSetColorSetTags(gxColorSet target, long tagType, long index, long oldCount, long newCount, const gxTag items[]) GXInlineCode(0x10F2);
void GXSetInkTags(gxInk target, long tagType, long index, long oldCount, long newCount, const gxTag items[]) GXInlineCode(0x10F3);
void GXSetShapeTags(gxShape target, long tagType, long index, long oldCount, long newCount, const gxTag items[]) GXInlineCode(0x10F4);
void GXSetStyleTags(gxStyle target, long tagType, long index, long oldCount, long newCount, const gxTag items[]) GXInlineCode(0x10F5);
void GXSetTransformTags(gxTransform target, long tagType, long index, long oldCount, long newCount, const gxTag items[]) GXInlineCode(0x10F6);
void GXSetViewDeviceTags(gxViewDevice target, long tagType, long index, long oldCount, long newCount, const gxTag items[]) GXInlineCode(0x10F7);
void GXSetViewPortTags(gxViewPort target, long tagType, long index, long oldCount, long newCount, const gxTag items[]) GXInlineCode(0x10F8);

gxProfileAttribute GXGetColorProfileAttributes(gxColorProfile source) GXInlineCode(0x1229);
gxInkAttribute GXGetInkAttributes(gxInk source) GXInlineCode(0x10F9);
gxShapeAttribute GXGetShapeAttributes(gxShape source) GXInlineCode(0x10FA);
gxInkAttribute GXGetShapeInkAttributes(gxShape source) GXInlineCode(0x10FB);
gxStyleAttribute GXGetShapeStyleAttributes(gxShape source) GXInlineCode(0x10FC);
gxStyleAttribute GXGetStyleAttributes(gxStyle source) GXInlineCode(0x10FD);
gxTextAttribute GXGetShapeTextAttributes(gxShape source) GXInlineCode(0x122B);
gxTextAttribute GXGetStyleTextAttributes(gxStyle source) GXInlineCode(0x122C);
gxDeviceAttribute GXGetViewDeviceAttributes(gxViewDevice source) GXInlineCode(0x10FF);
gxPortAttribute GXGetViewPortAttributes(gxViewPort source) GXInlineCode(0x1100);

void GXSetColorProfileAttributes(gxColorProfile target, gxProfileAttribute attributes) GXInlineCode(0x122a);
void GXSetInkAttributes(gxInk target, gxInkAttribute attributes) GXInlineCode(0x1101);
void GXSetShapeAttributes(gxShape target, gxShapeAttribute attributes) GXInlineCode(0x1102);
void GXSetShapeInkAttributes(gxShape target, gxInkAttribute attributes) GXInlineCode(0x1103);
void GXSetShapeStyleAttributes(gxShape target, gxStyleAttribute attributes) GXInlineCode(0x1104);
void GXSetStyleAttributes(gxStyle target, gxStyleAttribute attributes) GXInlineCode(0x1105);
void GXSetShapeTextAttributes(gxShape target, gxTextAttribute attributes) GXInlineCode(0x122E);
void GXSetStyleTextAttributes(gxStyle target, gxTextAttribute attributes) GXInlineCode(0x122D);
void GXSetViewDeviceAttributes(gxViewDevice target, gxDeviceAttribute attributes) GXInlineCode(0x1107);
void GXSetViewPortAttributes(gxViewPort target, gxPortAttribute attributes) GXInlineCode(0x1108);

long GXGetColorProfileOwners(gxColorProfile source) GXInlineCode(0x1109);
long GXGetColorSetOwners(gxColorSet source) GXInlineCode(0x110A);
long GXGetInkOwners(gxInk source) GXInlineCode(0x110B);
long GXGetShapeOwners(gxShape source) GXInlineCode(0x110C);
long GXGetStyleOwners(gxStyle source) GXInlineCode(0x110D);
long GXGetTagOwners(gxTag source) GXInlineCode(0x110E);
long GXGetTransformOwners(gxTransform source) GXInlineCode(0x110F);

void GXLockShape(gxShape target) GXInlineCode(0x1110);
void GXLockTag(gxTag target) GXInlineCode(0x1111);
void GXUnlockShape(gxShape target) GXInlineCode(0x1112);
void GXUnlockTag(gxTag target) GXInlineCode(0x1113);
void *GXGetShapeStructure(gxShape source, long *length) GXInlineCode(0x1114);
void *GXGetTagStructure(gxTag source, long *length) GXInlineCode(0x1115);

fixed GXGetColorDistance(const gxColor *target, const gxColor *source) GXInlineCode(0x1116);
gxPoint *GXShapeLengthToPoint(gxShape target, long index, fixed length, gxPoint *location, gxPoint *tangent) GXInlineCode(0x1117);

gxWide *GXGetShapeArea(gxShape source, long index, gxWide *area) GXInlineCode(0x1118);
long GXGetShapeCacheSize(gxShape source) GXInlineCode(0x1119);
gxPoint *GXGetShapeCenter(gxShape source, long index, gxPoint *center) GXInlineCode(0x111A);
gxContourDirection GXGetShapeDirection(gxShape source, long contour) GXInlineCode(0x111B);
long GXGetShapeIndex(gxShape source, long contour, long vector) GXInlineCode(0x111C);
gxWide *GXGetShapeLength(gxShape source, long index, gxWide *length) GXInlineCode(0x111D);
long GXGetShapeSize(gxShape source) GXInlineCode(0x111E);

long GXCountShapeContours(gxShape source) GXInlineCode(0x111F);
long GXCountShapePoints(gxShape source, long contour) GXInlineCode(0x1120);

long GXGetShapeDashPositions(gxShape source, gxMapping dashMappings[]) GXInlineCode(0x1121);    /* returns the number of positions */
long GXGetShapeDeviceArea(gxShape source, gxViewPort port, gxViewDevice device) GXInlineCode(0x1122);
gxBoolean GXGetShapeDeviceBounds(gxShape source, gxViewPort port, gxViewDevice device, gxRectangle *bounds) GXInlineCode(0x1123);
gxColorSet GXGetShapeDeviceColors(gxShape source, gxViewPort port, gxViewDevice device, long *width) GXInlineCode(0x1124);
gxBoolean GXGetShapeGlobalBounds(gxShape source, gxViewPort port, gxViewGroup group, gxRectangle *bounds) GXInlineCode(0x1125);
long GXGetShapeGlobalViewDevices(gxShape source, gxViewPort port, gxViewDevice list[]) GXInlineCode(0x1126);
long GXGetShapeGlobalViewPorts(gxShape source, gxViewPort list[]) GXInlineCode(0x1127);
gxRectangle *GXGetShapeLocalBounds(gxShape source, gxRectangle *bounds) GXInlineCode(0x1128);
long GXGetShapePatternPositions(gxShape source, gxPoint positions[]) GXInlineCode(0x1129);    /* returns the number of positions */
void GXGetShapeLocalFontMetrics(gxShape sourceShape, gxPoint* before, gxPoint* after, gxPoint* caretAngle, gxPoint* caretOffset) GXInlineCode(0x125A);
void GXGetShapeDeviceFontMetrics(gxShape sourceShape, gxViewPort port, gxViewDevice device, gxPoint* before, gxPoint* after, gxPoint* caretAngle, gxPoint* caretOffset) GXInlineCode(0x125B);

long GXGetViewGroupViewDevices(gxViewGroup source, gxViewDevice list[]) GXInlineCode(0x112A);
long GXGetViewGroupViewPorts(gxViewGroup source, gxViewPort list[]) GXInlineCode(0x112B);

gxMapping *GXGetViewPortGlobalMapping(gxViewPort source, gxMapping *map) GXInlineCode(0x112C);
long GXGetViewPortViewDevices(gxViewPort source, gxViewDevice list[]) GXInlineCode(0x112D);

gxShape GXHitTestPicture(gxShape target, const gxPoint *test, gxHitTestInfo *result, long level, long depth) GXInlineCode(0x112F);

gxBoolean GXIntersectRectangle(gxRectangle *target, const gxRectangle *source, const gxRectangle *operand) GXInlineCode(0x1130);
gxRectangle *GXUnionRectangle(gxRectangle *target, const gxRectangle *source, const gxRectangle *operand) GXInlineCode(0x1131);

gxBoolean GXTouchesRectanglePoint(const gxRectangle *target, const gxPoint *test) GXInlineCode(0x1132);
gxBoolean GXTouchesShape(gxShape target, gxShape test) GXInlineCode(0x1133);
gxBoolean GXTouchesBoundsShape(const gxRectangle *target, gxShape test) GXInlineCode(0x1134);

gxBoolean GXContainsRectangle(const gxRectangle *container, const gxRectangle *test) GXInlineCode(0x1135);
gxBoolean GXContainsShape(gxShape container, gxShape test) GXInlineCode(0x1136);
gxBoolean GXContainsBoundsShape(const gxRectangle *container, gxShape test, long index) GXInlineCode(0x1137);

gxColor *GXConvertColor(gxColor *target, gxColorSpace space, gxColorSet aSet, gxColorProfile profile) GXInlineCode(0x1138);
gxColor *GXCombineColor(gxColor *target, gxInk operand) GXInlineCode(0x1139);

gxBoolean GXCheckColor(const gxColor *source, gxColorSpace space, gxColorSet aSet, gxColorProfile profile) GXInlineCode(0x120B);
gxShape GXCheckBitmapColor(gxShape source, gxRectangle *area, gxColorSpace space, gxColorSet aSet, gxColorProfile profile) GXInlineCode(0x120C);

fixed GXGetHalftoneDeviceAngle(gxViewDevice source, const gxHalftone *data) GXInlineCode(0x120D);

long GXGetColorSetParts(gxColorSet source, long index, long count, gxColorSpace *space, gxSetColor data[]) GXInlineCode(0x113A);
long GXGetGlyphParts(gxShape source, long index, long charCount, long *byteLength, unsigned char text[], gxPoint positions[],
  long advanceBits[], gxPoint tangents[], long *runCount, short styleRuns[], gxStyle styles[]) GXInlineCode(0x113B);  /* returns the glyph count */
long GXGetPathParts(gxShape source, long index, long count, gxPaths *data) GXInlineCode(0x113C);
long GXGetPictureParts(gxShape source, long index, long count, gxShape shapes[], gxStyle styles[], gxInk inks[], gxTransform transforms[]) GXInlineCode(0x113D);
long GXGetPolygonParts(gxShape source, long index, long count, gxPolygons *data) GXInlineCode(0x113E);
gxShape GXGetShapeParts(gxShape source, long index, long count, gxShape destination) GXInlineCode(0x113F);
long GXGetTextParts(gxShape source, long index, long charCount, unsigned char text[]) GXInlineCode(0x1140);

void GXSetColorSetParts(gxColorSet target, long index, long oldCount, long newCount, const gxSetColor data[]) GXInlineCode(0x1141);
void GXSetGlyphParts(gxShape source, long index, long oldCharCount, long newCharCount, const unsigned char text[], 
  const gxPoint positions[], const long advanceBits[], const gxPoint tangents[], const short styleRuns[], const gxStyle styles[]) GXInlineCode(0x1142);
void GXSetPathParts(gxShape target, long index, long count, const gxPaths *data, gxEditShapeFlag flags) GXInlineCode(0x1143);
void GXSetPictureParts(gxShape target, long index, long oldCount, long newCount, const gxShape shapes[],
  const gxStyle styles[], const gxInk inks[], const gxTransform transforms[]) GXInlineCode(0x1144);
void GXSetPolygonParts(gxShape target, long index, long count, const gxPolygons *data, gxEditShapeFlag flags) GXInlineCode(0x1145);
void GXSetShapeParts(gxShape target, long index, long count, gxShape insert, gxEditShapeFlag flags) GXInlineCode(0x1146);
void GXSetTextParts(gxShape target, long index, long oldCharCount, long newCharCount, const unsigned char text[]) GXInlineCode(0x1147);

long GXGetShapePoints(gxShape source, long index, long count, gxPoint data[]) GXInlineCode(0x1148);
void GXSetShapePoints(gxShape target, long index, long count, const gxPoint data[]) GXInlineCode(0x1149);

long GXGetGlyphPositions(gxShape source, long index, long charCount, long advance[], gxPoint positions[]) GXInlineCode(0x114A);
long GXGetGlyphTangents(gxShape source, long index, long charCount, gxPoint tangents[]) GXInlineCode(0x114B);
void GXSetGlyphPositions(gxShape target, long index, long charCount, const long advance[], const gxPoint positions[]) GXInlineCode(0x114C);
void GXSetGlyphTangents(gxShape target, long index, long charCount, const gxPoint tangents[]) GXInlineCode(0x114D);

long GXGetGlyphMetrics(gxShape source, gxPoint advances[], gxRectangle boundingBoxes[], gxPoint sideBearings[]) GXInlineCode(0x1150);

void GXDifferenceShape(gxShape target, gxShape operand) GXInlineCode(0x1151);
void GXExcludeShape(gxShape target, gxShape operand) GXInlineCode(0x1152);
void GXIntersectShape(gxShape target, gxShape operand) GXInlineCode(0x1153);
void GXMapShape(gxShape target, const gxMapping *map) GXInlineCode(0x1154);
void GXMoveShape(gxShape target, fixed deltaX, fixed deltaY) GXInlineCode(0x1155);
void GXMoveShapeTo(gxShape target, fixed x, fixed y) GXInlineCode(0x1156);
void GXReverseDifferenceShape(gxShape target, gxShape operand) GXInlineCode(0x1157);
void GXRotateShape(gxShape target, fixed degrees, fixed xOffset, fixed yOffset) GXInlineCode(0x1158);
void GXScaleShape(gxShape target, fixed hScale, fixed vScale, fixed xOffset, fixed yOffset) GXInlineCode(0x1159);
void GXSkewShape(gxShape target, fixed xSkew, fixed ySkew, fixed xOffset, fixed yOffset) GXInlineCode(0x115A);
void GXUnionShape(gxShape target, gxShape operand) GXInlineCode(0x115B);

void GXDifferenceTransform(gxTransform target, gxShape operand) GXInlineCode(0x115C);
void GXExcludeTransform(gxTransform target, gxShape operand) GXInlineCode(0x115D);
void GXIntersectTransform(gxTransform target, gxShape operand) GXInlineCode(0x115E);
void GXMapTransform(gxTransform target, const gxMapping *map) GXInlineCode(0x115F);
void GXMoveTransform(gxTransform target, fixed deltaX, fixed deltaY) GXInlineCode(0x1160);
void GXMoveTransformTo(gxTransform target, fixed x, fixed y) GXInlineCode(0x1161);
void GXReverseDifferenceTransform(gxTransform target, gxShape operand) GXInlineCode(0x1162);
void GXRotateTransform(gxTransform target, fixed degrees, fixed xOffset, fixed yOffset) GXInlineCode(0x1163);
void GXScaleTransform(gxTransform target, fixed hScale, fixed vScale, fixed xOffset, fixed yOffset) GXInlineCode(0x1164);
void GXSkewTransform(gxTransform target, fixed xSkew, fixed ySkew, fixed xOffset, fixed yOffset) GXInlineCode(0x1165);
void GXUnionTransform(gxTransform target, gxShape operand) GXInlineCode(0x1166);

void GXBreakShape(gxShape target, long index) GXInlineCode(0x1167);
void GXChangedShape(gxShape target) GXInlineCode(0x1168);
gxShapePart GXHitTestShape(gxShape target, const gxPoint *test, gxHitTestInfo *result) GXInlineCode(0x1169);
gxShape GXHitTestDevice(gxShape target, gxViewPort port, gxViewDevice device, const gxPoint *test, const gxPoint *tolerance) GXInlineCode(0x116A);
void GXInsetShape(gxShape target, fixed inset) GXInlineCode(0x116B);
void GXInvertShape(gxShape target) GXInlineCode(0x116C);
void GXPrimitiveShape(gxShape target) GXInlineCode(0x116D);
void GXReduceShape(gxShape target, long contour) GXInlineCode(0x116E);
void GXReverseShape(gxShape target, long contour) GXInlineCode(0x116F);
void GXSimplifyShape(gxShape target) GXInlineCode(0x1170);

void GXFlattenShape(gxShape source, gxFlattenFlag flags, struct gxSpoolBlock *block) GXInlineCode(0x1171);
gxShape GXUnflattenShape(struct gxSpoolBlock *block, long count, const gxViewPort portList[]) GXInlineCode(0x1172);

#ifdef __cplusplus
}
#endif

#undef GXInlineCode



#endif
