/* graphics:
  graphics stream types
  by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
  Copyright 1987 - 1993 Apple Computer, Inc.  All rights reserved.
*/

#ifndef gxGraphicsStreamTypesIncludes
  #define gxGraphicsStreamTypesIncludes
  
  #ifndef gxFontTypesIncludes
    #include "font types.h"
  #endif

  #ifndef gxGraphicsTypesIncludes
    #include "graphics types.h"
  #endif

#define gxLayoutStyleRuns

  typedef enum {
    gxNewObjectOpcode     = 0x00,   /* create new object */
    gxSetDataOpcode       = 0x40,   /* add reference to current object */
    gxSetDefaultOpcode      = 0x80,   /* replace current default with this object */
    gxReservedOpcode      = 0xC0,   /* (may be used in future expansion) */
    gxNextOpcode        = 0xFF    /* used by currentOperand field to say opcode is coming */
  } gxGraphicsOperationOpcode;
  
/* new object types (new object opcode) */
  typedef enum {  
    gxHeaderTypeOpcode      = 0x00, /* byte following new object uses bottom 6 bits for type */
      /* gxShape types use values 1 (gxEmptyType) through 13 (gxPictureType) */
    gxStyleTypeOpcode     = 0x28,
    gxInkTypeOpcode,
    gxTransformTypeOpcode,    
    gxColorProfileTypeOpcode,
    gxColorSetTypeOpcode,
    gxTagTypeOpcode,
    gxBitImageOpcode,
    gxFontNameTypeOpcode,
    gxTrailerTypeOpcode     = 0x3F
  } gxGraphicsNewOpcode;
  
/* fields of objects (set data opcodes) */

  typedef enum {
    gxShapeAttributesOpcode,
    gxShapeTagOpcode,
    gxShapeFillOpcode
  } gxShapeDataOpcode;
  
  typedef enum {
    gxOmitPathPositionXMask       = 0xC0,
    gxOmitPathPositionYMask       = 0x30,
    gxOmitPathDeltaXMask        = 0x0C,
    gxOmitPathDeltaYMask        = 0x03
  } gxOmitPathMask;
  
  typedef enum {
    gxOmitPathPositionXShift      = 6,
    gxOmitPathPositionYShift      = 4,
    gxOmitPathDeltaXShift         = 2,
    gxOmitPathDeltaYShift         = 0
  } gxOmitPathShift;
  
  typedef enum {
    gxOmitBitmapImageMask       = 0xC0,
    gxOmitBitmapWidthMask       = 0x30,
    gxOmitBitmapHeightMask      = 0x0C,
    gxOmitBitmapRowBytesMask    = 0x03
  } gxOmitBitmapMask1;
  
  typedef enum {
    gxOmitBitmapImageShift      = 6,
    gxOmitBitmapWidthShift      = 4,
    gxOmitBitmapHeightShift       = 2,
    gxOmitBitmapRowBytesShift     = 0
  } gxOmitBitmapShift1;
  
  typedef enum {
    gxOmitBitmapPixelSizeMask       = 0xC0,
    gxOmitBitmapSpaceMask       = 0x30,
    gxOmitBitmapSetMask         = 0x0C,
    gxOmitBitmapProfileMask       = 0x03
  } gxOmitBitmapMask2;
  
  typedef enum {
    gxOmitBitmapPixelSizeShift      = 6,
    gxOmitBitmapSpaceShift      = 4,
    gxOmitBitmapSetShift        = 2,
    gxOmitBitmapProfileShift      = 0
  } gxOmitBitmapShift2;
  
  typedef enum {
    gxOmitBitmapPositionXMask       = 0xC0,
    gxOmitBitmapPositionYMask       = 0x30
  } gxOmitBitmapMask3;
  
  typedef enum {
    gxOmitBitmapPositionXShift      = 6,
    gxOmitBitmapPositionYShift      = 4
  } gxOmitBitmapShift3;
  
  typedef enum {
    gxOmitBitImageRowBytesMask    = 0xC0,
    gxOmitBitImageHeightMask      = 0x30,
    gxOmitBitImageDataMask      = 0x08
  } gxOmitBitImageMask;
  
  typedef enum {
    gxOmitBitImageRowBytesShift     = 6,
    gxOmitBitImageHeightShift       = 4,
    gxOmitBitImageDataShift     = 3
  } gxOmitBitImageShift;
  
  typedef enum {
    gxCopyBitImageBytesOpcode     = 0x00,
    gxRepeatBitImageBytesOpcode   = 0x40,
    gxLookupBitImageBytesOpcode   = 0x80,
    gxRepeatBitImageScanOpcode    = 0xC0
  } gxBitImageCompression;
  
  typedef enum {
    gxOmitTextCharactersMask      = 0xC0,
    gxOmitTextPositionXMask       = 0x30,
    gxOmitTextPositionYMask       = 0x0C,
    gxOmitTextDataMask        = 0x02
  } gxOmitTextMask;
  
  typedef enum {
    gxOmitTextCharactersShift       = 6,
    gxOmitTextPositionXShift      = 4,
    gxOmitTextPositionYShift      = 2,
    gxOmitTextDataShift         = 1
  } gxOmitTextShift;
  
  typedef enum {
    gxOmitGlyphCharactersMask     = 0xC0,
    gxOmitGlyphLengthMask       = 0x30,
    gxOmitGlyphRunNumberMask    = 0x0C,
    gxOmitGlyphOnePositionMask    = 0x02,
    gxOmitGlyphDataMask         = 0x01
  } gxOmitGlyphMask1;
  
  typedef enum {
    gxOmitGlyphCharactersShift    = 6,
    gxOmitGlyphLengthShift      = 4,
    gxOmitGlyphRunNumberShift     = 2,
    gxOmitGlyphOnePositionShift     = 1,
    gxOmitGlyphDataShift        = 0
  } gxOmitGlyphShift1;
  
  typedef enum {
    gxOmitGlyphPositionsMask      = 0xC0,
    gxOmitGlyphAdvancesMask       = 0x20,
    gxOmitGlyphTangentsMask       = 0x18,
    gxOmitGlyphRunsMask         = 0x04,
    gxOmitGlyphStylesMask       = 0x03
  } gxOmitGlyphMask2;
  
  typedef enum {
    gxOmitGlyphPositionsShift       = 6,
    gxOmitGlyphAdvancesShift      = 5,
    gxOmitGlyphTangentsShift      = 3,
    gxOmitGlyphRunsShift        = 2,
    gxOmitGlyphStylesShift      = 0
  } gxOmitGlyphShift2;
  
  typedef enum {
    gxOmitLayoutLengthMask      = 0xC0,
    gxOmitLayoutPositionXMask       = 0x30,
    gxOmitLayoutPositionYMask       = 0x0C,
    gxOmitLayoutDataMask      = 0x02
  } gxOmitLayoutMask1;
  
  typedef enum {
    gxOmitLayoutLengthShift       = 6,
    gxOmitLayoutPositionXShift      = 4,
    gxOmitLayoutPositionYShift      = 2,
    gxOmitLayoutDataShift         = 1
  } gxOmitLayoutShift1;
  
  typedef enum {
    gxOmitLayoutWidthMask       = 0xC0,
    gxOmitLayoutFlushMask       = 0x30,
    gxOmitLayoutJustMask      = 0x0C,
    gxOmitLayoutOptionsMask       = 0x03
  } gxOmitLayoutMask2;
  
  typedef enum {
    gxOmitLayoutWidthShift      = 6,
    gxOmitLayoutFlushShift      = 4,
    gxOmitLayoutJustShift         = 2,
    gxOmitLayoutOptionsShift      = 0
  } gxOmitLayoutShift2;
  
  typedef enum {
    gxOmitLayoutStyleRunNumberMask  = 0xC0,
    gxOmitLayoutLevelRunNumberMask  = 0x30,
    gxOmitLayoutHasBaselineMask     = 0x08,
    gxOmitLayoutStyleRunsMask     = 0x04,
    gxOmitLayoutStylesMask      = 0x03
  } gxOmitLayoutMask3;
  
  typedef enum {
    gxOmitLayoutStyleRunNumberShift   = 6,
    gxOmitLayoutLevelRunNumberShift   = 4,
    gxOmitLayoutHasBaselineShift    = 3,
    gxOmitLayoutStyleRunsShift    = 2,
    gxOmitLayoutStylesShift       = 0
  } gxOmitLayoutShift3;
  
  typedef enum {
    gxOmitLayoutLevelRunsMask     = 0x80,
    gxOmitLayoutLevelsMask      = 0x40
  } gxOmitLayoutMask4;
  
  typedef enum {
    gxOmitLayoutLevelRunsShift    = 7,
    gxOmitLayoutLevelsShift       = 6
  } gxOmitLayoutShift4;
  
  typedef enum {
    gxInkAttributesOpcode,
    gxInkTagOpcode,
    gxInkColorOpcode,
    gxInkTransferModeOpcode
  } gxInkDataOpcode;

  typedef enum {
    gxOmitColorsSpaceMask       = 0xC0,
    gxOmitColorsProfileMask       = 0x30,
    gxOmitColorsComponentsMask    = 0x0F,
    gxOmitColorsIndexMask       = 0x0C,
    gxOmitColorsIndexSetMask      = 0x03
  } gxOmitColorsMask;
  
  typedef enum {
    gxOmitColorsSpaceShift      = 6,
    gxOmitColorsProfileShift      = 4,
    gxOmitColorsComponentsShift     = 0,
    gxOmitColorsIndexShift      = 2,
    gxOmitColorsIndexSetShift       = 0
  } gxOmitColorsShift;
  
  typedef enum {
    gxOmitTransferSpaceMask       = 0xC0,
    gxOmitTransferSetMask     = 0x30,
    gxOmitTransferProfileMask       = 0x0C
  } gxOmitTransferMask1;
  
  typedef enum {
    gxOmitTransferSpaceShift      = 6,
    gxOmitTransferSetShift      = 4,
    gxOmitTransferProfileShift      = 2
  } gxOmitTransferShift1;
  
  typedef enum {
    gxOmitTransferSourceMatrixMask  = 0xC0,
    gxOmitTransferDeviceMatrixMask  = 0x30,
    gxOmitTransferResultMatrixMask    = 0x0C,
    gxOmitTransferFlagsMask       = 0x03
  } gxOmitTransferMask2;

  typedef enum {
    gxOmitTransferSourceMatrixShift = 6,
    gxOmitTransferDeviceMatrixShift   = 4,
    gxOmitTransferResultMatrixShift   = 2,
    gxOmitTransferFlagsShift      = 0
  } gxOmitTransferShift2; 

  typedef enum {
    gxOmitTransferComponentModeMask       = 0x80,
    gxOmitTransferComponentFlagsMask      = 0x40,
    gxOmitTransferComponentSourceMinimumMask  = 0x30,
    gxOmitTransferComponentSourceMaximumMask  = 0x0C,
    gxOmitTransferComponentDeviceMinimumMask  = 0x03
  } gxOmitTransferComponentMask1;

  typedef enum {
    gxOmitTransferComponentModeShift      = 7,
    gxOmitTransferComponentFlagsShift       = 6,
    gxOmitTransferComponentSourceMinimumShift   = 4,
    gxOmitTransferComponentSourceMaximumShift   = 2,
    gxOmitTransferComponentDeviceMinimumShift   = 0
  } gxOmitTransferComponentShift1;

  typedef enum {
    gxOmitTransferComponentDeviceMaximumMask  = 0xC0,
    gxOmitTransferComponentClampMinimumMask = 0x30,
    gxOmitTransferComponentClampMaximumMask = 0x0C,
    gxOmitTransferComponentOperandMask    = 0x03
  } gxOmitTransferComponentMask2;

  typedef enum {
    gxOmitTransferComponentDeviceMaximumShift   = 6,
    gxOmitTransferComponentClampMinimumShift  = 4,
    gxOmitTransferComponentClampMaximumShift  = 2,
    gxOmitTransferComponentOperandShift     = 0
  } gxOmitTransferComponentShift2;
  
  typedef enum {
    gxStyleAttributesOpcode,
    gxStyleTagOpcode,
    gxStyleCurveErrorOpcode,
    gxStylePenOpcode,
    gxStyleJoinOpcode,
    gxStyleDashOpcode,
    gxStyleCapsOpcode,
    gxStylePatternOpcode,
    gxStyleTextAttributesOpcode,
    gxStyleTextSizeOpcode,
    gxStyleFontOpcode,
    gxStyleTextFaceOpcode,
    gxStylePlatformOpcode,
    gxStyleFontVariationsOpcode ,
  #ifdef gxLayoutStyleRuns 
    gxStyleRunControlsOpcode,
    gxStyleRunPriorityJustOverrideOpcode,
    gxStyleRunGlyphJustOverridesOpcode,
    gxStyleRunGlyphSubstitutionsOpcode,
    gxStyleRunFeaturesOpcode,
    gxStyleRunKerningAdjustmentsOpcode,
  #else
    gxStyleLayoutInfoOpcode,
  #endif
    gxStyleJustificationOpcode
  } gxStyleDataOpcode;

  typedef enum {
    gxOmitDashAttributesMask          = 0xC0,
    gxOmitDashShapeMask           = 0x30,
    gxOmitDashAdvanceMask         = 0x0C,
    gxOmitDashPhaseMask           = 0x03
  } gxOmitDashMask1;

  typedef enum {
    gxOmitDashAttributesShift           = 6,
    gxOmitDashShapeShift            = 4,
    gxOmitDashAdvanceShift          = 2,
    gxOmitDashPhaseShift            = 0
  } gxOmitDashShift1;
  
  typedef enum {
    gxOmitDashScaleMask             = 0xC0
  } gxOmitDashMask2;

  typedef enum {
    gxOmitDashScaleShift            = 6
  } gxOmitDashShift2;
  
  typedef enum {
    gxOmitPatternAttributesMask         = 0xC0,
    gxOmitPatternShapeMask          = 0x30,
    gxOmitPatternUXMask           = 0x0C,
    gxOmitPatternUYMask           = 0x03
  } gxOmitPatternMask1;

  typedef enum {
    gxOmitPatternAttributesShift        = 6,
    gxOmitPatternShapeShift         = 4,
    gxOmitPatternUXShift            = 2,
    gxOmitPatternUYShift            = 0
  } gxOmitPatternShift1;
  
  typedef enum {
    gxOmitPatternVXMask             = 0xC0,
    gxOmitPatternVYMask             = 0x30
  } gxOmitPatternMask2;

  typedef enum {
    gxOmitPatternVXShift            = 6,
    gxOmitPatternVYShift            = 4
  } gxOmitPatternShift2;
  
  typedef enum {
    gxOmitJoinAttributesMask          = 0xC0,
    gxOmitJoinShapeMask           = 0x30,
    gxOmitJoinMiterMask           = 0x0C
  } gxOmitJoinMask;

  typedef enum {
    gxOmitJoinAttributesShift           = 6,
    gxOmitJoinShapeShift            = 4,
    gxOmitJoinMiterShift            = 2
  } gxOmitJoinShift;
  
  typedef enum {
    gxOmitCapAttributesMask           = 0xC0,
    gxOmitCapStartShapeMask         = 0x30,
    gxOmitCapEndShapeMask         = 0x0C
  } gxOmitCapMask;

  typedef enum {
    gxOmitCapAttributesShift          = 6,
    gxOmitCapStartShapeShift          = 4,
    gxOmitCapEndShapeShift          = 2
  } gxOmitCapShift;
  
  typedef enum {
    gxOmitFaceLayersMask          = 0xC0,
    gxOmitFaceMappingMask         = 0x30
  } gxOmitFaceMask;

  typedef enum {
    gxOmitFaceLayersShift           = 6,
    gxOmitFaceMappingShift          = 4
  } gxOmitFaceShift;
  
  typedef enum {
    gxOmitFaceLayerFillMask           = 0xC0,
    gxOmitFaceLayerFlagsMask          = 0x30,
    gxOmitFaceLayerStyleMask          = 0x0C,
    gxOmitFaceLayerTransformMask        = 0x03
  } gxOmitFaceLayerMask1;

  typedef enum {
    gxOmitFaceLayerFillShift          = 6,
    gxOmitFaceLayerFlagsShift         = 4,
    gxOmitFaceLayerStyleShift         = 2,
    gxOmitFaceLayerTransformShift       = 0
  } gxOmitFaceLayerShift1;
  
  typedef enum {
    gxOmitFaceLayerBoldXMask          = 0xC0,
    gxOmitFaceLayerBoldYMask          = 0x30
  } gxOmitFaceLayerMask2;

  typedef enum {
    gxOmitFaceLayerBoldXShift           = 6,
    gxOmitFaceLayerBoldYShift         = 4
  } gxOmitFaceLayerShift2;
  
  typedef enum {
    gxColorSetReservedOpcode,
    gxColorSetTagOpcode
  } gxColorSetDataOpcode;

  typedef enum {
    gxColorProfileAttributesOpcode,
    gxColorProfileTagOpcode
  } gxProfileDataOpcode;
  
  typedef enum {
    gxTransformReservedOpcode,
    gxTransformTagOpcode,
    gxTransformClipOpcode,
    gxTransformMappingOpcode,
    gxTransformPartMaskOpcode,
    gxTransformToleranceOpcode
  } gxTransformDataOpcode;
  
  typedef enum {
    gxTypeOpcode,
    gxSizeOpcode
  } gxGenericDataOpcode;        /* used by currentOperand when currentOperation is gxNextOpcode */
  
/*  format of top byte:
  xx yyyyyy   xx == 0x00, 0x40, 0x80, 0xC0: defines graphics operation (see gxGraphicsOperationOpcode)
        yyyyyy == size of operation in bytes
        if (yyyyyy == 0), byte size follows. If byte following == 0, word size follows; if == 0, long follows
        word and long, if present, are specified in high-endian order (first byte is largest magnitude)
        
  format of byte following size specifiers, if any:
  xx yyyyyy   xx == 0x00, 0x40, 0x80, 0xC0: defines compression level (0 == none, 0xC0 == most)
            exact method of compression is defined by type of data
        yyyyyy == data type selector (0 to 63): see gxGraphicsNewOpcode, __DataOpcode
*/

  #define gxOpcodeShift     6
  #define gxObjectSizeMask    0x3F
  #define gxCompressionShift    6
  #define gxObjectTypeMask    0x3F

  #define gxBitImageOpcodeMask  0xC0
  #define gxBitImageCountMask 0x3f
  #define gxBitImageOpcodeShift 6
  
  typedef enum {
    gxNoCompression,
    gxWordCompression,
    gxByteCompression,
    gxOmitCompression
  } gxTwoBitCompressionValues;
  
  #define gxCompressionMask   0x03
  
  /*  the following structures define how primitives without a public geometry
    are stored (their format mirrors that of the New call to create them)   */

  typedef struct {
    unsigned char   name;   /* gxFontName */
    unsigned char   platform;   /* gxFontPlatform */
    unsigned char   script;   /* gxFontScript */
    unsigned char   language;   /* gxFontLanguage */
    short     length;   /* byte length */
  /*  unsigned char     gxFontName[length] */
  } gxFlatFontName;

  #define gxFlatFontListItemTag 0x666C7374  /* 'flst' */

  typedef struct {
    gxFont      fontID;     /*** if we get rid of this, remove #include "font types.h", above */
    unsigned char name;     /* gxFontName */
    unsigned char platform;     /* gxFontPlatform */
    unsigned char script;     /* gxFontScript */
    unsigned char language;     /* gxFontLanguage */
    short   length;     /* byte length */
    short   glyphCount;
  /*  unsigned char   gxFontName[gxAnyNumber];        */  
  /*  long      glyphBits[glyphBitsSize + 31 >> 5];     */
  } gxFlatFontListItem;
  
  typedef struct {
    long        count;
    gxFlatFontListItem  items[gxAnyNumber];
  } gxFlatFontList;

  typedef struct {
    fixed       version;
    unsigned char   flatFlags;
  } gxFlattenHeader;

  typedef enum {
    gxOmitPictureShapeMask        = 0xC0,
    gxOmitOverrideStyleMask     = 0x30,
    gxOmitOverrideInkMask       = 0x0C,
    gxOmitOverrideTransformMask   = 0x03
  } gxOmitPictureParametersMask;

  typedef enum {
    gxOmitPictureShapeShift       = 0x6,
    gxOmitOverrideStyleShift      = 0x4,
    gxOmitOverrideInkShift        = 0x2,
    gxOmitOverrideTransformShift    = 0x0
  } gxOmitPictureParametersShift;

#endif
