/* graphics:
  public object and constant definitions
  by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
  Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.  */

#ifndef gxGraphicsTypesIncludes
  #define gxGraphicsTypesIncludes
  /* everything outdented from here to conserve space */

  #ifndef gxMathTypesIncludes
    #include "math types.h"
  #endif

  #ifndef gxAnyNumber
    #define gxAnyNumber 1
  #endif

  #ifndef nil
    #define nil 0L
  #endif
  
/* graphics objects */

  typedef struct gxPrivateShapeRecord *gxShape;
  typedef struct gxPrivateStyleRecord *gxStyle;
  typedef struct gxPrivateInkRecord *gxInk;
  typedef struct gxPrivateTransformRecord *gxTransform;
  typedef struct gxPrivateTagRecord *gxTag;
  typedef struct gxPrivateColorSetRecord *gxColorSet;
  typedef struct gxPrivateProfileRecord *gxColorProfile;
  typedef struct gxPrivateGraphicsClientRecord *gxGraphicsClient;
  typedef struct gxPrivateViewGroupRecord *gxViewGroup;
  typedef struct gxPrivateViewPortRecord *gxViewPort;
  typedef struct gxPrivateViewDeviceRecord *gxViewDevice;

  typedef long gxColorSpace;

/* gxShape enumerations */

  typedef enum {
    gxEmptyType = 1,
    gxPointType,
    gxLineType,
    gxCurveType,
    gxRectangleType,
    gxPolygonType,
    gxPathType,
    gxBitmapType,
    gxTextType,
    gxGlyphType,
    gxLayoutType,
    gxFullType,
    gxPictureType
  } gxShapeTypes;

  typedef long gxShapeType;

  typedef enum {
    gxNoFill,
    gxOpenFrameFill,
    gxFrameFill = gxOpenFrameFill,
    gxClosedFrameFill,
    gxHollowFill = gxClosedFrameFill,
    gxEvenOddFill,
    gxSolidFill = gxEvenOddFill,
    gxWindingFill,
    gxInverseEvenOddFill,
    gxInverseSolidFill = gxInverseEvenOddFill,
    gxInverseFill = gxInverseEvenOddFill,
    gxInverseWindingFill
  } gxShapeFills;

  typedef long gxShapeFill;

  typedef enum {
    gxNoAttributes,
    gxDirectShape   = 0x0001,
    gxRemoteShape   = 0x0002,
    gxCachedShape   = 0x0004,
    gxLockedShape   = 0x0008,
    gxGroupShape    = 0x0010,
    gxMapTransformShape = 0x0020,
    gxUniqueItemsShape  = 0x0040,
    gxIgnorePlatformShape = 0x0080,
    gxNoMetricsGridShape  = 0x0100,
    gxDiskShape     = 0x0200,
    gxMemoryShape   = 0x0400
  } gxShapeAttributes;

  typedef long gxShapeAttribute;

/* gxShape editing enumerations */

  typedef enum {
    gxBreakNeitherEdit      = 0,
    gxBreakLeftEdit       = 0x0001,
    gxBreakRightEdit        = 0x0002,
    gxRemoveDuplicatePointsEdit = 0x0004    /* if the new first (or last) gxPoint exactly matches the gxPoint before it in */
                      /* the same contour, then remove it) */
  } gxEditShapeFlags;

  typedef long gxEditShapeFlag;

  #define gxSelectToEnd -1
  #define gxSetToNil  (void *)(-1)

  typedef enum {
    gxCounterclockwiseDirection,
    gxClockwiseDirection
  } gxContourDirections;
  
  typedef long gxContourDirection;

/* gxShape structures */

/* The type 'gxPoint' is defined in "math types.h" */
/*
  typedef struct {
    fixed   x;
    fixed   y;
  } gxPoint;
*/

  typedef struct {
    gxPoint   first;
    gxPoint   last;
  } gxLine;

  typedef struct {
    gxPoint   first;
    gxPoint   control;
    gxPoint   last;
  } gxCurve;

  typedef struct {
    fixed   left;
    fixed   top;
    fixed   right;
    fixed   bottom;
  } gxRectangle;

  typedef struct {
    long    vectors;
    gxPoint   vector[gxAnyNumber];
  } gxPolygon;

  typedef struct {
    long    contours;
    gxPolygon contour[gxAnyNumber];
  } gxPolygons;

  typedef struct {
    long    vectors;
    long    controlBits[gxAnyNumber];
    gxPoint   vector[gxAnyNumber];
  } gxPath;

  typedef struct {
    long    contours;
    gxPath    contour[gxAnyNumber];
  } gxPaths;

  typedef struct {
    char      *image;   /* pointer to pixels */
    long      width;    /* width in pixels */
    long      height;   /* height in pixels */
    long      rowBytes; /* width in bytes */
    long      pixelSize;    /* physical bits per pixel */
    gxColorSpace  space;
    gxColorSet    set;
    gxColorProfile  profile;
  } gxBitmap;
  
  typedef struct {
    long    left;
    long    top;
    long    right;
    long    bottom;
  } gxLongRectangle;

/* gxStyle enumerations */

  typedef enum {
    gxCenterFrameStyle  = 0,
    gxSourceGridStyle = 0x0001,
    gxDeviceGridStyle = 0x0002,
    gxInsideFrameStyle  = 0x0004,
    gxOutsideFrameStyle = 0x0008,
    gxAutoInsetStyle    = 0x0010
  } gxStyleAttributes;

  typedef long gxStyleAttribute;

  typedef enum {
    gxBendDash      = 0x0001,
    gxBreakDash     = 0x0002,
    gxClipDash      = 0x0004,
    gxLevelDash     = 0x0008,
    gxAutoAdvanceDash = 0x0010
  } gxDashAttributes;
  
  typedef long gxDashAttribute;
  
  typedef enum {
    gxPortAlignPattern  = 0x0001,
    gxPortMapPattern    = 0x0002
  } gxPatternAttributes;
  
  typedef long gxPatternAttribute;

  typedef enum {
    gxSharpJoin     = 0x0000,
    gxCurveJoin     = 0x0001,
    gxLevelJoin     = 0x0002
  } gxJoinAttributes;
  
  typedef long gxJoinAttribute;

  typedef enum {
    gxLevelStartCap   = 0x0001,
    gxLevelEndCap   = 0x0002
  } gxCapAttributes;
  
  typedef long gxCapAttribute;

  typedef enum {
    gxAutoAdvanceText = 0x0001,
    gxNoContourGridText = 0x0002,
    gxNoMetricsGridText = 0x0004,
    gxAnchorPointsText  = 0x0008,
    gxVerticalText    = 0x0010
  } gxTextAttributes;
  
  typedef long gxTextAttribute;   

  #define gxLeftJustify 0
  #define gxCenterJustify (frac1/2)
  #define gxRightJustify frac1
  #define gxFillJustify -1

  typedef enum {
    gxUnderlineAdvanceLayer = 1,      /* a gxLine is drawn through the advances */
    gxSkipWhiteSpaceLayer = 2,        /* except characters describing white space */
    gxUnderlineIntervalLayer = 4,     /* (+ gxStringLayer) a gxLine is drawn through the gaps between advances */
    gxUnderlineContinuationLayer = 8,   /* (+ gxStringLayer) join this underline with another face */
    gxWhiteLayer = 16,          /* the layer draws to white instead of black */
    gxClipLayer = 32,           /* the characters define a clip */
    gxStringLayer = 64          /* all characters in run are combined */
  } gxLayerFlags;

  typedef long gxLayerFlag;

/* gxStyle structures */

  typedef struct {
    gxShapeFill   outlineFill;      /* outline framed or filled */
    gxLayerFlag   flags;        /* various additional effects */
    gxStyle     outlineStyle;     /* outline */
    gxTransform   outlineTransform;   /* italic, condense, extend */
    gxPoint     boldOutset;     /* bold */
  } gxFaceLayer;

  typedef struct {
    long      faceLayers;     /* layer to implement shadow */
    gxMapping   advanceMapping;   /* algorithmic change to advance width */
    gxFaceLayer   faceLayer[gxAnyNumber]; /* zero or more face layers describing the face */
  } gxTextFace;

  typedef struct {
    gxJoinAttribute   attributes;
    gxShape     join;
    fixed       miter;
  } gxJoinRecord;

  typedef struct {
    gxDashAttribute   attributes;
    gxShape     dash; /* similar to pattern, except rotated to gxLine slope */
    fixed       advance;  /* specifies repeating frequency of dash */
    fract       phase;  /* specifies offset into the gxPath to start dashing */
    fixed       scale;  /* specifies height of dash to be mapped to penWidth */
  } gxDashRecord;

  typedef struct {
    gxPatternAttribute  attributes;
    gxShape     pattern;
    gxPoint       u;
    gxPoint       v;
  } gxPatternRecord;

  typedef struct {
    gxCapAttribute    attributes;
    gxShape     startCap;
    gxShape     endCap;
  } gxCapRecord;

/* gxInk enumerations */

  typedef enum {
    gxPortAlignDitherInk  = 0x0001,
    gxForceDitherInk    = 0x0002,
    gxSuppressDitherInk = 0x0004,
    gxSuppressHalftoneInk = 0x0008
  } gxInkAttributes;

  typedef long gxInkAttribute;

  typedef enum {
    gxNoMode = 0,
    gxCopyMode,
    gxAddMode,
    gxBlendMode,
    gxMigrateMode,
    gxMinimumMode,
    gxMaximumMode,
    gxHighlightMode,
    gxAndMode,
    gxOrMode,
    gxXorMode,
    gxRampAndMode,
    gxRampOrMode,
    gxRampXorMode,
    gxOverMode,       /* Alpha channel modes          */
    gxAtopMode,         /* Note: In England = Beta channel modes    */
    gxExcludeMode,
    gxFadeMode
  } gxComponentModes;

  typedef unsigned char gxComponentMode;

  typedef enum {
    gxRejectSourceTransfer    = 0x0001, /* at least one component must be out of range      */
    gxRejectDeviceTransfer    = 0x0002, /* at least one component must be out of range      */
    gxSingleComponentTransfer   = 0x0004    /* duplicate gxTransferComponent[0] for all components in transfer */
  } gxTransferFlags;
  
  typedef long gxTransferFlag;
  
  typedef enum {
    gxOverResultComponent = 0x01,     /* & result gxColor with 0xFFFF before clamping     */
    gxReverseComponent    = 0x02      /* reverse source and device before moding      */
  } gxComponentFlags;

  typedef unsigned char gxComponentFlag;

/* gxInk structures */

  typedef struct {
    gxComponentMode   mode;       /* how the component is operated upon       */
    gxComponentFlag   flags;        /* flags for each component           */
    gxColorValue      sourceMinimum;
    gxColorValue      sourceMaximum;    /* input filter range               */
    gxColorValue      deviceMinimum;
    gxColorValue      deviceMaximum;    /* output filter range              */
    gxColorValue      clampMinimum;
    gxColorValue      clampMaximum;   /* output clamping range              */
    gxColorValue      operand;        /* ratio for blend, step for migrate, gxColor for highlight */
  } gxTransferComponent;

  typedef struct {
    gxColorSpace    space;        /* the gxColor-space the transfer mode is to operate in */
    gxColorSet      set;
    gxColorProfile    profile;
    fixed       sourceMatrix[5][4];
    fixed       deviceMatrix[5][4];
    fixed       resultMatrix[5][4];
    gxTransferFlag    flags;
    gxTransferComponent component[4];     /* how each component is operated upon        */
  } gxTransferMode;

/* gxColor space enumerations */

  typedef enum {
    gxNoColorPacking    = 0x0000,   /* 16 bits per channel */
    gxAlphaSpace    = 0x0080,   /* space includes alpha channel */
    gxWord5ColorPacking   = 0x0500,   /* 5 bits per channel, right-justified */
    gxLong8ColorPacking = 0x0800,   /* 8 bits per channel, right-justified */
    gxLong10ColorPacking  = 0x0a00,   /* 10 bits per channel, right-justified */
    gxAlphaFirstPacking = 0x1000      /* alpha channel is the first field in the packed space */
  } gxColorPackingTypes;
  
  typedef enum {
    gxNoSpace = 0,
    gxRgbSpace,
    gxCmykSpace,
    gxHsvSpace,
    gxHlsSpace,
    gxCieSpace,
    gxXyzSpace,
    gxLuvSpace,
    gxLabSpace,
    gxYiqSpace,
    gxNtscSpace   = gxYiqSpace,
    gxPalSpace    = gxYiqSpace,
    gxGraySpace ,
    gxIndexedSpace,
    gxRgbaSpace   = gxRgbSpace + gxAlphaSpace,
    gxGrayaSpace  = gxGraySpace + gxAlphaSpace,
    gxRgb16Space  = gxWord5ColorPacking + gxRgbSpace,
    gxRgb32Space  = gxLong8ColorPacking + gxRgbSpace,
    gxArgb32Space = gxLong8ColorPacking + gxAlphaFirstPacking + gxRgbaSpace,
    gxCmyk32Space = gxLong8ColorPacking + gxCmykSpace,
    gxHsv32Space  = gxLong10ColorPacking + gxHsvSpace,
    gxHls32Space  = gxLong10ColorPacking + gxHlsSpace,
    gxCie32Space  = gxLong10ColorPacking + gxCieSpace,
    gxXyz32Space  = gxLong10ColorPacking + gxXyzSpace,
    gxLuv32Space  = gxLong10ColorPacking + gxLuvSpace,
    gxLab32Space  = gxLong10ColorPacking + gxLabSpace,
    gxYiq32Space  = gxLong10ColorPacking + gxYiqSpace,
    gxNtsc32Space   = gxYiq32Space,
    gxPal32Space  = gxYiq32Space
  } gxColorSpaces;

/* gxColor space structures */

  typedef struct {
     gxColorValue red;
     gxColorValue green;
     gxColorValue blue;
  } gxRgbColor;

  typedef struct {
     gxColorValue red;
     gxColorValue green;
     gxColorValue blue;
     gxColorValue alpha;
  } gxRgbaColor;

  typedef struct {
    gxColorValue hue;
    gxColorValue saturation;
    gxColorValue value;
  } gxHsvColor;

  typedef struct {
    gxColorValue hue;
    gxColorValue lightness;
    gxColorValue saturation;
  } gxHlsColor;

  typedef struct {
    gxColorValue cyan;
    gxColorValue magenta;
    gxColorValue yellow;
    gxColorValue black;
  } gxCmykColor;

  typedef struct {
    gxColorValue x;
    gxColorValue y;
    gxColorValue z;
  } gxCieColor;

  typedef struct {
    gxColorValue y;
    gxColorValue i;
    gxColorValue q;
  } gxYiqColor;

  typedef struct {
    gxColorValue gray;
    gxColorValue alpha;
  } gxGrayaColor;

  typedef long gxColorIndex;

  typedef struct {
    gxColorIndex  index;
    gxColorSet  set;
  } gxIndexedColor;

  typedef struct {
    gxColorSpace space;
    gxColorProfile profile;
    union {
      gxCmykColor cmyk;
      gxRgbColor    rgb;
      gxRgbaColor   rgba;
      gxHsvColor    hsv;
      gxHlsColor    hls;
      gxCieColor    cie;
      gxYiqColor    yiq;
      gxColorValue    gray;
      gxGrayaColor  graya;
      unsigned short  pixel16;
      unsigned long pixel32;
      gxIndexedColor  indexed;
      gxColorValue    component[4];
    } element;
  } gxColor;

/* gxColorSet structures */

  typedef union {
    gxCmykColor cmyk;
    gxRgbColor    rgb;
    gxRgbaColor   rgba;
    gxHsvColor    hsv;
    gxHlsColor    hls;
    gxCieColor    cie;
    gxYiqColor    yiq;
    gxColorValue    gray;
    gxGrayaColor  graya;
    unsigned short  pixel16;
    unsigned long pixel32;
    gxColorValue    component[4];
  } gxSetColor;

/* gxColorProfile enumerations */

  typedef enum {
    gxPerceptualProfile = 0,    /* only one of these three can be used at once */
    gxColorimetricProfile = 1,
    gxSaturationProfile = 2,
    
    gxSuppressMatchProfile = 4    /* added to perceptual / colorimetric / saturation to suppress matching */
  } gxProfileAttributes;

  typedef long gxProfileAttribute;

/* gxColorProfile structures */
  
  typedef enum {
    gxGrayResponse,
    gxRedResponse,
    gxGreenResponse,
    gxBlueResponse,
    gxCyanResponse,
    gxMagentaResponse,
    gxYellowResponse,
    gxUnderColorRemovalResponse,
    gxBlackGenerationResponse,
    gxOnePlusLastResponse
  } gxResponseColors;
  
  typedef long gxResponseColor;

  typedef struct {
    short   colorCount[gxOnePlusLastResponse];
    gxColorValue    data[gxAnyNumber];
  } gxProfileResponse;

  typedef struct {
    gxCieColor    red;
    gxCieColor    green;
    gxCieColor    blue;
    gxCieColor    cyan;
    gxCieColor    magenta;
    gxCieColor    yellow;
    gxCieColor    black;
    gxCieColor    white;
  } gxProfileRecord;

/* gxTransform enumerations */

  typedef enum {          /* parts of a gxShape considered in hit testing: */
    gxNoPart      = 0,      /* (in order of evaluation) */
    gxBoundsPart    = 0x0001,
    gxGeometryPart    = 0x0002,
    gxPenPart     = 0x0004,
    gxCornerPointPart = 0x0008,
    gxControlPointPart  = 0x0010,
    gxEdgePart      = 0x0020,
    gxJoinPart      = 0x0040,
    gxStartCapPart    = 0x0080,
    gxEndCapPart    = 0x0100,
    gxDashPart      = 0x0200,
    gxPatternPart   = 0x0400,
    gxGlyphBoundsPart = gxJoinPart,
    gxGlyphFirstPart    = gxStartCapPart,
    gxGlyphLastPart   = gxEndCapPart,
    gxSideBearingPart = gxDashPart,
    gxAnyPart     = gxBoundsPart | gxGeometryPart | gxPenPart | gxCornerPointPart | gxControlPointPart |
              gxEdgePart | gxJoinPart | gxStartCapPart | gxEndCapPart | gxDashPart | gxPatternPart
  } gxShapeParts;

  typedef long gxShapePart;

/* gxTransform structures */

  typedef struct {
    gxShapePart   what;       /* which part of gxShape */
    long      index;        /* control gxPoint index */
    fixed     distance;       /* how far from gxPoint or outside of area click was */

    /* these fields are only set by GXHitTestPicture */
    gxShape   which;
    gxShape   containerPicture;   /* picture which contains gxShape hit */
    long      containerIndex;     /* the index within that picture  */
    long      totalIndex;     /* the total index within the root picture */
  } gxHitTestInfo;

/* gxViewPort enumerations */

  typedef enum {
    gxGrayPort      = 0x0001,
    gxAlwaysGridPort    = 0x0002
  } gxPortAttributes;

  typedef long gxPortAttribute;

/* gxViewDevice enumerations */

  typedef enum {
    gxDirectDevice    = 0x01,     /* for the device gxBitmap baseAddr pointer */
    gxRemoteDevice    = 0x02,
    gxInactiveDevice    = 0x04
  } gxDeviceAttributes;

  typedef long gxDeviceAttribute;

  typedef enum {
    gxRoundDot = 1,
    gxSpiralDot,
    gxSquareDot,
    gxLineDot,
    gxEllipticDot,
    gxTriangleDot,
    gxHilbertDot,
    gxOrderedDitherDot
  } gxDotTypes;

  typedef long gxDotType;

/* gxViewPort structures */

  typedef enum {
    gxNoTint,
    gxLuminanceTint,      /* use the luminance of the gxColor */
    gxAverageTint,      /* add all the components and divide by the number of components */
    gxMixtureTint,      /* find the closest gxColor on the axis between the foreground and background */
    gxComponent1Tint,   /* use the value of the first component of the gxColor */
    gxComponent2Tint,   /* ... etc. */
    gxComponent3Tint,
    gxComponent4Tint
  } gxTintTypes;

  typedef long gxTintType;

  typedef struct {
    fixed     angle;
    fixed     frequency;
    gxDotType   method;
    gxTintType    tinting;
    gxColor     dotColor;
    gxColor     backgroundColor;
    gxColorSpace  tintSpace;
  } gxHalftone;

/* gxViewGroup enumerations */

  #define gxAllViewDevices    ((gxViewGroup) 0)
  #define gxScreenViewDevices ((gxViewGroup) 1)

/* graphics stream constants and structures */

  typedef enum {
    gxOpenReadSpool = 1,
    gxOpenWriteSpool,
    gxReadSpool,
    gxWriteSpool,
    gxCloseSpool
  } gxSpoolCommands;
  
  typedef long gxSpoolCommand;
  
  typedef unsigned char gxGraphicsOpcode;

  typedef struct gxSpoolBlock {
  /* these fields are read only */
    long      (*spoolProcedure)(gxSpoolCommand command, struct gxSpoolBlock *block);
    void      *buffer;      /* source/destination pointer to data */
    long      bufferSize;   /* how many bytes for the system to read (flatten) / write (unflatten) */
  /* these fields are written to (but are not read from) */
    long      count;      /* how many bytes for the caller to read (unflatten) /write (flatten) */
    long      operationSize;    /* operation size (including operand byte) */
    long      operationOffset;  /* the data offset, if any, within the current operation */
    gxGraphicsOpcode  lastTypeOpcode; /* type of last created object */
    gxGraphicsOpcode  currentOperation; /* operation emitted by flatten, or intrepreted by last unflatten */
    gxGraphicsOpcode  currentOperand; /* e.g., gxTransformTypeOpcode, gxInkTagOpcode */
    unsigned char compressed;   /* one of: gxTwoBitCompressionValues */
  } gxSpoolBlock;

  typedef enum {
    gxFontListFlatten   = 0x01,   /* if set, generate a gxTag containing list of each gxFont referenced */
    gxFontGlyphsFlatten = 0x02    /* if set, generate a gxTag containing the list of glyphs referenced inside the gxFont */
  } gxFlattenFlags;
  
  typedef long gxFlattenFlag;

/* gxGraphicsClient constants */

  #define gxGraphicsSystemClient    (gxGraphicsClient)(-1)
  #define gxGraphicsCodeClient      (gxGraphicsClient)(-2)

/* graphics patching constants */

  #define gxOriginalGraphicsFunction    -1
  #define gxOriginalGraphicsIdentifier    'grfx'

#endif /* of graphic types includes */
