/*
  layout edit library.h
  
  Simple layout editing based on the TextEdit model.
  
  Copyright 1992 Apple Computer, Inc. All rights reserved.
*/

#ifndef layoutEditLibraryIncludes
#define layoutEditLibraryIncludes

#include <Types.h>
#include <Memory.h>

#ifndef selectionLibraryIncludes
#include "selection library.h"
#endif

#ifndef gxLayoutTypesIncludes
#include "layout types.h"
#endif


typedef Handle LayoutEditHandle;

LayoutEditHandle NewLayoutEditHandle(
  long textRunCount,
  const short textRunLengths[],
  const void *text[],
  long styleRunCount,
  const short styleRunLengths[],
  const gxStyle styles[],
  long levelRunCount,
  const short levelRunLengths[],
  const short levels[],
  gxLayoutOptions *layoutOptions,
  gxPoint *position,
  gxStyle defaultStyle);

LayoutEditHandle LayoutEditHandleFromLayout(gxShape layout);

long GetLayoutEditHandle(
  LayoutEditHandle handle,
  void *text,
  long *styleRunCount,
  short styleRunLengths[],
  gxStyle styles[],
  long *levelRunCount,
  short levelRunLengths[],
  short levels[],
  gxLayoutOptions *layoutOptions,
  gxPoint *position);

void SetLayoutEditHandle(
  LayoutEditHandle handle,
  long textRunCount,
  const short textRunLengths[],
  const void *text[],
  long styleRunCount,
  const short styleRunLengths[],
  const gxStyle styles[],
  long levelRunCount,
  const short levelRunLengths[],
  const short levels[],
  const gxLayoutOptions *layoutOptions,
  const gxPoint *position);

void SetLayoutEditHandleParts(
  LayoutEditHandle handle,
  gxByteOffset oldStartOffset,
  gxByteOffset oldEndOffset,
  long newTextRunCount,
  const short newTextRunLengths[],
  const void *newText[],
  long newStyleRunCount,
  const short newStyleRunLengths[],
  const gxStyle newStyles[],
  long newLevelRunCount,
  const short newLevelRunLengths[],
  const short newLevels[]);

void SetLayoutEditHandleShapeParts(
  LayoutEditHandle handle,
  gxByteOffset startOffset,
  gxByteOffset endOffset,
  gxShape insert);

long GetLayoutEditHandleParts(
  LayoutEditHandle handle,
  gxByteOffset startOffset,
  gxByteOffset endOffset,
  void *text,
  long *styleRunCount,
  short styleRunLengths[],
  gxStyle styles[],
  long *levelRunCount,
  short levelRunLengths[],
  short levels[]);

gxShape GetLayoutEditHandleShapeParts(
  LayoutEditHandle handle,
  gxByteOffset startOffset,
  gxByteOffset endOffset,
  gxShape dest);

void LayoutEditIdle(LayoutEditHandle handle);

void LayoutEditClick(LayoutEditHandle handle, gxPoint hitDown);

void LayoutEditActivate(LayoutEditHandle handle);

void LayoutEditDeactivate(LayoutEditHandle handle);

void LayoutEditKey(LayoutEditHandle handle, char key);

void LayoutEditUpdate(LayoutEditHandle handle);

SelectionHandle LayoutEditGetSelection(LayoutEditHandle handle);

void LayoutEditSetSelection(LayoutEditHandle handle, SelectionHandle selection);

void LayoutEditSetStyle(LayoutEditHandle handle, gxStyle newStyle);
void LayoutEditIncrementLevel(LayoutEditHandle handle);
void LayoutEditDecrementLevel(LayoutEditHandle handle);
void LayoutEditSetLevel(LayoutEditHandle handle, long level);

void LayoutEditCut(LayoutEditHandle handle);
void LayoutEditCopy(LayoutEditHandle handle);
void LayoutEditPaste(LayoutEditHandle handle);
void LayoutEditClear(LayoutEditHandle handle);

void LayoutEditFromScrap(LayoutEditHandle handle);
void LayoutEditToScrap(LayoutEditHandle handle);

void DisposeLayoutEditHandle(LayoutEditHandle handle);

#endif
