/* graphics:
  math routines
  by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
  Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.  */

#ifndef gxMathRoutinesIncludes
  #define gxMathRoutinesIncludes

  #ifndef gxMathTypesIncludes
    #include "math types.h"
  #endif

  #ifdef appleInternal
    #define GXInlineCode(x)
  #endif
  #ifndef GXInlineCode
    #define GXInlineCode(x) = {0x303C, x, 0xA832}
  #endif

  #ifdef __cplusplus
  extern "C" {
  #endif

  gxMapping *GXCopyToMapping(gxMapping *target, const gxMapping *source) GXInlineCode(0x2013);
  gxMapping *GXInvertMapping(gxMapping *target, const gxMapping *source) GXInlineCode(0x2014);
  gxMapping *GXMapMapping(gxMapping *target, const gxMapping *source) GXInlineCode(0x2015);
  gxMapping *GXMoveMapping(gxMapping *target, fixed hOffset, fixed vOffset) GXInlineCode(0x2016);
  gxMapping *GXMoveMappingTo(gxMapping *target, fixed hPosition, fixed vPosition) GXInlineCode(0x2017);
  gxMapping *GXNormalizeMapping(gxMapping *target) GXInlineCode(0x2018);
  gxMapping *GXRotateMapping(gxMapping *target, fixed angle, fixed xCenter, fixed yCenter) GXInlineCode(0x2019);
  gxMapping *GXScaleMapping(gxMapping *target, fixed hFactor, fixed vFactor, fixed xCenter, fixed yCenter) GXInlineCode(0x201A);
  gxMapping *GXResetMapping(gxMapping *target) GXInlineCode(0x201B);
  gxMapping *GXSkewMapping(gxMapping *target, fixed skewX, fixed skewY, fixed xCenter, fixed yCenter) GXInlineCode(0x201C);

  void GXMapPoints(const gxMapping *source, long count, gxPoint vector[]) GXInlineCode(0x201D);

  short GXFirstBit(unsigned long) GXInlineCode(0x201E);
  short GXWideScale(const gxWide *source) GXInlineCode(0x201F);

  short GXLinearRoot(fixed first, fixed last, fract t[]) GXInlineCode(0x2020);
  short GXQuadraticRoot(fixed first, fixed control, fixed last, fract t[]) GXInlineCode(0x2021);

  gxPoint *GXPolarToPoint(const gxPolar *, gxPoint *) GXInlineCode(0x2022);
  gxPolar *GXPointToPolar(const gxPoint *, gxPolar *) GXInlineCode(0x2023);

  fract GXFractCubeRoot(fract source) GXInlineCode(0x2024);
  fract GXFractDivide(fract dividend, fract divisor) GXInlineCode(0x2025);
  fract GXFractMultiply(fract multiplicand, fract multiplier) GXInlineCode(0x2026);
  fract GXFractSineCosine(fixed degrees, fract *cosine) GXInlineCode(0x2027);
  fract GXFractSquareRoot(fract source) GXInlineCode(0x2028);

  fixed GXFixedDivide(fixed dividend, fixed divisor) GXInlineCode(0x2029);
  fixed GXFixedMultiply(fixed multiplicand, fixed multiplier) GXInlineCode(0x202A);

  /* This next call is (source * multiplier / divisor) -- it avoids underflow, overflow by using wides */
  long GXMultiplyDivide(long source, long multiplier, long divisor) GXInlineCode(0x202B);

  unsigned long GXMagnitude(long deltaX, long deltaY) GXInlineCode(0x202C);
  long GXVectorMultiplyDivide(long count, const long *vector1, long step1, const long *vector2, long step2, long divisor) GXInlineCode(0x202D);

  gxWide *GXWideAdd(gxWide *target, const gxWide *source) GXInlineCode(0x202E);
  short GXWideCompare(const gxWide *target, const gxWide *source) GXInlineCode(0x202F);
  gxWide *GXWideNegate(gxWide *target) GXInlineCode(0x2030);
  gxWide *GXWideShift(gxWide *target, long shift) GXInlineCode(0x2031);
  unsigned long GXWideSquareRoot(const gxWide *source) GXInlineCode(0x2032);
  gxWide *GXWideSubtract(gxWide *target, const gxWide *source) GXInlineCode(0x2033);

  gxWide *GXWideMultiply(long multiplicand, long multiplier, gxWide *target) GXInlineCode(0x2034);
  long GXWideDivide(const gxWide *dividend, long divisor, long *remainder) GXInlineCode(0x2035);    /* returns the quotient */
  gxWide *GXVectorMultiply(long count, const long *vector1, long step1, const long *vector2, long step2, gxWide *dot) GXInlineCode(0x2036);
  gxWide *GXWideWideDivide(gxWide *dividend, long divisor, long *remainder);            /* quotient replaces dividend */

  unsigned long GXRandomBits(long count, long focus) GXInlineCode(0x2037);
  void GXSetRandomSeed(const gxWide *seed) GXInlineCode(0x2038);
  gxWide *GXGetRandomSeed(gxWide *seed) GXInlineCode(0x2039);

  #ifdef __cplusplus
  }
  #endif

  #undef GXInlineCode

  #define GXFixedRound(a)   ((short)((fixed)(a) + fix1/2 >> 16))
  #define GXFixedSquareRoot(a)  ((fixed)GXFractSquareRoot(a) + 64 >> 7)
  #define GXFixedTruncate(a)    ((short)((fixed)(a) >> 16))

  #define GXFixedToFract(a)   ((fract)(a) << 14)
  #define GXFractToFixed(a)   ((fixed)(a) + 8192L >> 14)

  #define GXFixedToInt(a)   ((short)((fixed)(a) + fix1/2 >> 16))
  #define GXIntToFixed(a)   ((fixed)(a) << 16)

  #define GXFixedToFloat(a)   ((float)(a) / fix1)
  #define GXFloatToFixed(a)     ((fixed)((float)(a) * fix1))

  #define GXFractToFloat(a)   ((float)(a) / frac1)
  #define GXFloatToFract(a)   ((fract)((float)(a) * frac1))

  #define GXColorToFract(a)   (((fract) (a) << 14) + ((fract)(a) + 2 >> 2))
  #define GXFractToColor(a)   ((gxColorValue) ((a) - ((a) >> 16) + 8191 >> 14))

  #define ff(a)       GXIntToFixed(a)
  #define fl(a)       GXFloatToFixed(a)

#endif
