/* graphics:  
  gxFont file structures
  
  by  Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good,
    Barton House, Robert Johnson, Eric Mader, Keith McGreggor,
    Dave Opstad, Mike Reed, Oliver Steele, David Van Brink, Chris Yerga
    
  Copyright 1987 - 1993 Apple Computer, Inc.  All rights reserved.
*/

#ifndef gxSfntTypesIncludes
#define gxSfntTypesIncludes

#ifndef gxMathTypesIncludes
  #include "math types.h"
#endif

#ifndef gxAnyNumber
  #define gxAnyNumber 1
#endif

/* ---------------- */

#ifndef gxSharedFontTypeDefines
  #define gxSharedFontTypeDefines
  
  typedef long gxFontTableTag;
  typedef long gxFontVariationTag;
  typedef long gxFontFormatTag;
  typedef long gxFontStorageTag;

  typedef struct {
    gxFontVariationTag  name;
    fixed       value;
  } gxFontVariation;
  typedef gxFontVariation gxFontDescriptor;

  typedef struct {
    unsigned short  featureType;
    unsigned short  setting;
  } gxFontRunFeature;

  typedef struct {
    unsigned short  setting;
    unsigned short  nameID;
  } gxFontFeatureSetting;
#endif


/* ---------------- */

typedef struct {
  gxFontTableTag    tableTag;
  unsigned long   checkSum;
  unsigned long   offset;
  unsigned long   length;
} gxSfntDirectoryEntry;

/*
 *  The search fields limits numOffsets to 4096.
 */

typedef struct {
  gxFontFormatTag     format;
  unsigned short      numOffsets;     /* number of tables */
  unsigned short      searchRange;      /* (max2 <= numOffsets)*16 */
  unsigned short      entrySelector;      /* log2(max2 <= numOffsets) */
  unsigned short      rangeShift;     /* numOffsets*16-searchRange*/
  gxSfntDirectoryEntry    table[gxAnyNumber];   /* table[numOffsets] */
} gxSfntDirectory;

#define gxSizeof_sfntDirectory      12

/*
 *    Cmap - character id to glyph id gxMapping
 */

#define gxCmapTableTag    0x636d6170  /* 'cmap' */

typedef struct {
  unsigned short      format;
  unsigned short      length;
  unsigned short      languageID; /* base-1 */
} gxSfntCMapSubHeader;

#define gxSizeof_sfntCMapSubHeader    6

typedef struct {
  unsigned short      platformID; /* base-0 */
  unsigned short      scriptID;   /* base-0 */
  unsigned long     offset;
} gxSfntCMapEncoding;

#define gxSizeof_sfntCMapEncoding 8

typedef struct {
  unsigned short    version;
  unsigned short    numTables;
  gxSfntCMapEncoding  encoding[gxAnyNumber];
} gxSfntCMapHeader;

#define gxSizeof_sfntCMapHeader     4

/*
 *    Name table
 */

#define gxNameTableTag      0x6e616d65    /* 'name' */

typedef struct {
  unsigned short      platformID;   /* base-0 */
  unsigned short      scriptID;     /* base-0 */
  unsigned short      languageID;   /* base-0 */
  unsigned short      nameID;     /* base-0 */
  unsigned short      length;
  unsigned short      offset;
} gxSfntNameRecord;

#define gxSizeof_sfntNameRecord     12

typedef struct {
  unsigned short      format;
  unsigned short      count;
  unsigned short      stringOffset;
  gxSfntNameRecord    record[gxAnyNumber];
} gxSfntNameHeader;

#define gxSizeof_sfntNameHeader   6


/*
 *    Fvar table - gxFont variations
 */

#define gxVariationTableTag     0x66766172    /* 'fvar' */

/*  These define each gxFont variation */

typedef struct {
  gxFontVariationTag    axisTag;
  fixed         minValue;
  fixed         defaultValue;
  fixed         maxValue;
  short       flags;
  short       nameID;
} gxSfntVariationAxis;

#define gxSizeof_sfntVariationAxis    20

/*  These are named locations in gxStyle-space for the user */

typedef struct {
  short       nameID;
  short       flags;
  fixed         coord[gxAnyNumber];   /* [axisCount] */
  /* room to grow since the header carries a tupleSize field */
} gxSfntInstance;

#define gxSizeof_sfntInstance     4

typedef struct {
  fixed         version;        /* 1.0 fixed */
  unsigned short      offsetToData;     /* to first axis = 16*/
  unsigned short      countSizePairs;     /* axis+inst = 2 */
  unsigned short      axisCount, axisSize;
  unsigned short      instanceCount, instanceSize;
  /*  other <count,size> pairs */
  gxSfntVariationAxis   axis[gxAnyNumber];    /* [axisCount] */
  gxSfntInstance      instance[gxAnyNumber];  /* [instanceCount] */
  /*  other arrays of data */
} gxSfntVariationHeader;

#define gxSizeof_sfntVariationHeader    16

/*
 *    Fdsc table - gxFont descriptor
 */

#define gxDescriptorTableTag    0x66647363    /* 'fdsc' */

typedef struct {
  fixed         version;    /* 1.0 fixed */
  long          descriptorCount;
  gxFontDescriptor      descriptor[gxAnyNumber];
} gxSfntDescriptorHeader;

#define gxSizeof_sfntDescriptorHeader   8

/*
 * Feat Table - layout feature table
 */

#define gxFeatureTableTag     0x66656174  /* 'feat' */

typedef struct {
  unsigned short    runCount;
  unsigned short    nameID;
  unsigned short    flags;
  unsigned short    groupID;
  long        offsetToRuns;
} gxSfntFeatureSet;

typedef struct {
  unsigned short    featureType;
  unsigned short    settingCount;
  long        offsetToSettings;
  unsigned short    featureFlags;
  unsigned short    nameID;
} gxSfntFeatureName;

typedef struct {
  long        version;      /* 1.0 */
  unsigned short    featureNameCount;
  unsigned short    featureSetCount;
  long        offsetToFeatureSets;
  gxSfntFeatureName names[gxAnyNumber];
  gxSfntFeatureSet    sets[gxAnyNumber];
  gxFontFeatureSetting  settings[gxAnyNumber];
  gxFontRunFeature    runs[gxAnyNumber];
} gxSfntFeatureHeader;

/*
 *    OS/2 Table
 */

#define gxOs2TableTag         0x4f532f32    /* 'OS/2' */

/* need to put os/2 structure definition here */

#endif
