/*****************************************

  synonym tags:

  These are the data structures for Graphics 
  synonyms stored in tags.

  Copyright  1991, 1992 Apple Computer Inc.  All rights reserved

******************************************/

#ifndef gxSynonymTags
#define gxSynonymTags


  #ifndef gxMathTypesIncludes
    #include "math types.h"
  #endif

  #ifndef gxAnyNumber
    #define gxAnyNumber 1
  #endif

  #ifndef gxGraphicsTypesIncludes
    #include "graphics types.h"
  #endif

  #ifndef gxFontTypesIncludes
    #include "font types.h"
  #endif


  #define gxPostScriptTag   0x706f7374  /* 'post' */

  #define gxPostControlTag 0x70736374 /* 'psct' */

  typedef enum {
    gxNoSave = 1          /* don't do save-restore around PS data */
  } gxPsStateFlags;

  typedef struct {
    long    flags;        /* PostScript state flags */
    long    numFonts;       /* number of fonts in the list */
    short name;       /* gxFontName */
    short platform;       /* gxFontPlatform */
    short script;       /* gxFontScript */
    short language;       /* gxFontLanguage */
    char    fontList[gxAnyNumber];  /* packed list of pascal strings, containing screen gxFont names */
  } gxPostControl;


  #define gxDashSynonymTag 0x73647368 /* 'sdsh' */

  typedef struct {
    long    size;         /* number of elements in array */
    fixed   dashLength[gxAnyNumber];  /* Array of dash lengths */
  } gxDashSynonym;


  #define gxLineCapSynonymTag 0x6c636170  /* 'lcap' */

  typedef enum {
    gxButtCap,
    gxRoundCap,
    gxSquareCap,
    gxTriangleCap
  } gxLineCaps;

  typedef long  gxLineCapSynonym;             /* gxLine cap type */


  #define gxCubicSynonymTag 0x63756278  /* 'cubx' */
  
  typedef enum {
  
    gxIgnoreFlag =  0x0000,   /* Ignore this word, get next one */
    gxLineToFlag =  0x0001,   /* Draw a gxLine to gxPoint following this flag */
    gxCurveToFlag =   0x0002,   /* Draw a gxCurve through the 3 points following this flag */
    gxMoveToFlag =  0x0003,   /* Start a new contour at the gxPoint following this flag */
    gxClosePathFlag = 0x0004    /* Close the contour */
    
  } gxCubicSynonym;

  #define gxCubicInstructionMask 0x000F   /* low four bits are gxPoint instructions */
  
  typedef short gxCubicSynonymFlags;  /* Low four bits are instruction (moveto, lineto, curveto, closepath) */


  #define gxPatternSynonymTag 0x7074726E  /* 'ptrn' */

  typedef enum {
    gxHatch,
    gxCrossHatch
  } gxPatterns;

  typedef struct {
    long  patternType;      /* one of the gxPatterns: gxHatch or gxCrossHatch */
    fixed   angle;        /* angle at which pattern is drawn */
    fixed   spacing;        /* distance between two parallel pattern lines */
    fixed   thickness;        /* thickness of the pattern */
    gxPoint   anchorPoint;      /* gxPoint with with respect to which pattern position is calculated */
  } gxPatternSynonym;


#endif
