/*
 *	File:		FontHIMenuData.h
 *
 *	Contains:	internal routines and data structs for FontHI Font and Style menus
 *
 *	Written by:	Dennis Buchheim
 *
 *	Copyright:	 1992, 1993 by Apple Computer, Inc., all rights reserved.
 *
 *	Change History (most recent first):
 *
 */

#pragma once

#include <Types.h>
#include <QuickDraw.h>
#include <Menus.h>

#include "font types.h"

#include "FontHIMenus.h"


#define FHI_MenuMagic			0x13579bdf


typedef struct {
	Str255			name;
	short			nameIndex;
	short			styleMenuItem;
	font			fontID;
	Style			textface;
	Boolean			selected;
} FHI_StyleRec, *FHI_StylePtr, **FHI_StyleHan;

typedef struct {
	Str255			name;
	short			nameIndex;
	short			familyMenuItem;
	short			numStyles;
	short			defaultStyleIndex;
	FHI_StyleHan	styleH;
} FHI_FamilyRec;

typedef struct {
	unsigned long	magicNum;
	short			numSelectedFonts;
	short			selFamilyMenuItem;
	short			selStyleMenuItem;
	short			totNumActualStyles;
	short			numFamilies;
	FHI_FamilyRec	families[];
} FHI_MenuDataRec, *FHI_MenuDataPtr, **FHI_MenuDataHan;

/*----------*/

FHI_MenuDataHan	FHIInitMenuData		(void);
void			FHIDisposeMenuData	(FHI_MenuDataHan fontMenuDataH);

short		FHIFamilyMenu			(FHI_MenuDataHan fontMenuDataH, MenuHandle familyMenu);
short		FHIFamilyMenuSetItem	(FHI_MenuDataHan fontMenuDataH, MenuHandle familyMenu,
									 MenuHandle styleMenu, short newFamilyItem,
									 short *numFontMappings, FHI_FontMapPtr *fontMappings);
short		FHIStyleMenu			(FHI_MenuDataHan fontMenuDataH, MenuHandle styleMenu);
short		FHIStyleMenuSetItem		(FHI_MenuDataHan fontMenuDataH,
									 MenuHandle styleMenu, short newStyleItem,
									 short *numFontMappings, FHI_FontMapPtr *fontMappings);

short		FHIGetMarkedFamilyItem	(FHI_MenuDataHan fontMenuDataH);
short		FHIGetMarkedStyleItem	(FHI_MenuDataHan fontMenuDataH);

short		FHICountSelectedFonts	(FHI_MenuDataHan fontMenuDataH);
short		FHIGetSelectedFontIDs	(FHI_MenuDataHan fontMenuDataH,
									 short *numSelectedFonts, FHI_FontIDPtr *fontIDs);
									 
void		FHIGetFamilyName		(FHI_MenuDataHan fontMenuDataH, font fontID, Style textface,
									 Str255 familyName);
void		FHIGetStyleName			(FHI_MenuDataHan fontMenuDataH, font fontID, Style textface,
									 Str255 styleName);

void		FHIMarkFonts			(FHI_MenuDataHan fontMenuDataH,
									 MenuHandle familyMenu, MenuHandle styleMenu,
									 short numFontsToSelect, FHI_FontIDPtr fontIDs);
