/*
	File:		FontHIMenus.c

 	Contains:	public interface to routines for FontHI Font and Style menus
 	
	Written by:	Dennis Buchheim

	Copyright:	 1993 by Apple Computer, Inc., All rights reserved.

	Change History (most recent first):


*/

#include <Errors.h>

#include "FontHIValidation.h"
#include "FontHIMenuData.h"
#include "FontHIMenus.h"


void * FontHIInitMenuData	()
{
	FHI_Errors		err = FontHI_NoErr;
	void 			*fontMenuDataH = NULL;
	
	fontMenuDataH = FHIInitMenuData ();
	err = FHIValidateMenuData (fontMenuDataH);
	
	if (err)
		FHIPostError (err);

	return (fontMenuDataH);
}

void FontHIDisposeMenuData	(void *fontMenuDataH)
{
	FHI_Errors		err = FontHI_NoErr;

	err = FHIValidateMenuData (fontMenuDataH);
	
	if (!err)
		FHIDisposeMenuData (fontMenuDataH);
	else
		FHIPostError (err);
}

short FontHIFamilyMenu	(void *fontMenuDataH, MenuHandle familyMenu)
{
	FHI_Errors		validationErr = FontHI_NoErr;
	short			memErr = noErr;
	
	validationErr = FHIValidateMenuData (fontMenuDataH);
	
	if (!validationErr)
		memErr = FHIFamilyMenu (fontMenuDataH, familyMenu);
	else
		FHIPostError (validationErr);
	
	return (memErr);
}

short FontHIFamilyMenuSetItem	(void *fontMenuDataH, MenuHandle familyMenu,
								 MenuHandle styleMenu, short newFamilyItem,
								 short *numFontMappings, FHI_FontMapPtr *fontMappings)
{
	FHI_Errors		validationErr = FontHI_NoErr;
	short			memErr = noErr;
	
	validationErr = FHIValidateMenuData (fontMenuDataH);
	
	if (!validationErr)
		memErr = FHIFamilyMenuSetItem (fontMenuDataH, familyMenu, styleMenu,
										newFamilyItem, numFontMappings, fontMappings);
	else
		FHIPostError (validationErr);
	
	return (memErr);
}
									 
short FontHIStyleMenu	(void *fontMenuDataH, MenuHandle styleMenu)
{
	FHI_Errors		validationErr = FontHI_NoErr;
	short			memErr = noErr;
	
	validationErr = FHIValidateMenuData (fontMenuDataH);
	
	if (!validationErr)
		memErr = FHIStyleMenu (fontMenuDataH, styleMenu);
	else
		FHIPostError (validationErr);
	
	return (memErr);
}

short FontHIStyleMenuSetItem	(void *fontMenuDataH,
								 MenuHandle styleMenu, short newStyleItem,
								 short *numFontMappings, FHI_FontMapPtr *fontMappings)
{
	FHI_Errors		validationErr = FontHI_NoErr;
	short			memErr = noErr;
	
	validationErr = FHIValidateMenuData (fontMenuDataH);
	
	if (!validationErr)
		memErr = FHIStyleMenuSetItem (fontMenuDataH, styleMenu, newStyleItem,
									  numFontMappings, fontMappings);
	else
		FHIPostError (validationErr);
	
	return (memErr);
}

short FontHIGetMarkedFamilyItem	(void *fontMenuDataH)
{
	FHI_Errors		err = FontHI_NoErr;
	short			itemNum = 0;
	
	err = FHIValidateMenuData (fontMenuDataH);
	
	if (!err)
		itemNum = FHIGetMarkedFamilyItem (fontMenuDataH);
	else
		FHIPostError (err);
	
	return (itemNum);
}

short FontHIGetMarkedStyleItem	(void *fontMenuDataH)
{
	FHI_Errors		err = FontHI_NoErr;
	short			itemNum = 0;
	
	err = FHIValidateMenuData (fontMenuDataH);
	
	if (!err)
		itemNum = FHIGetMarkedStyleItem (fontMenuDataH);
	else
		FHIPostError (err);
	
	return (itemNum);
}

short FontHICountSelectedFonts	(void *fontMenuDataH)
{
	FHI_Errors		err = FontHI_NoErr;
	short			numFonts = 0;
	
	err = FHIValidateMenuData (fontMenuDataH);
	
	if (!err)
		numFonts = FHICountSelectedFonts (fontMenuDataH);
	else
		FHIPostError (err);
	
	return (numFonts);
}

short FontHIGetSelectedFontIDs	(void *fontMenuDataH,
								 short *numSelectedFonts, FHI_FontIDPtr *fontIDs)
{
	FHI_Errors		validationErr = FontHI_NoErr;
	short			memErr = noErr;
	
	validationErr = FHIValidateMenuData (fontMenuDataH);
	
	if (!validationErr)
		memErr = FHIGetSelectedFontIDs (fontMenuDataH, numSelectedFonts, fontIDs);
	else
		FHIPostError (validationErr);
	
	return (memErr);
}
									 
void FontHIGetFamilyName	(void *fontMenuDataH,
							 font fontID, Style textface, Str255 familyName)
{
	FHI_Errors		err = FontHI_NoErr;

	err = FHIValidateMenuData (fontMenuDataH);
	
	if (!err)
		FHIGetFamilyName (fontMenuDataH, fontID, textface, familyName);
	else
		FHIPostError (err);
}

void FontHIGetStyleName		(void *fontMenuDataH,
							 font fontID, Style textface, Str255 styleName)
{
	FHI_Errors		err = FontHI_NoErr;

	err = FHIValidateMenuData (fontMenuDataH);
	
	if (!err)
		FHIGetStyleName (fontMenuDataH, fontID, textface, styleName);
	else
		FHIPostError (err);
}

void FontHIMarkFonts	(void *fontMenuDataH,
						 MenuHandle familyMenu, MenuHandle styleMenu,
						 short numFontsToSelect, FHI_FontIDPtr fontIDs)
{
	FHI_Errors		err = FontHI_NoErr;

	err = FHIValidateMenuData (fontMenuDataH);
	
	if (!err)
		FHIMarkFonts (fontMenuDataH, familyMenu, styleMenu,
					  numFontsToSelect, fontIDs);
	else
		FHIPostError (err);
}
