/*
	File:		FontHIPalette.c

 	Contains:	public interface to routines for FontHI font palette

	Written by:	Dennis Buchheim

	Copyright:	 1993 by Apple Computer, Inc., All rights reserved.

	Change History (most recent first):


*/

#include <Errors.h>

#include "FontHIValidation.h"
#include "FontHIFontUtils.h"
#include "FontHIPaletteData.h"
#include "FontHIPaletteWind.h"
#include "FontHIPalette.h"


void * FontHIOpenPalette	(void *fontMenuDataH)
{
	FHI_Errors		err = FontHI_NoErr;
	void 			*fontPalDataH = NULL;
	
	err = FHIValidateMenuData (fontMenuDataH);
	
	if (!err)
	{
		fontPalDataH = FHIOpenPalette (fontMenuDataH);
		err = FHIValidatePaletteData (fontPalDataH);
	}
	
	if (err)
		FHIPostError (err);

	return (fontPalDataH);
}

void FontHIShowHidePalette	(void *fontPalDataH,
							 Boolean show)
{
	FHI_Errors		err = FontHI_NoErr;

	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		FHIShowHidePalette (fontPalDataH, show);
	else
		FHIPostError (err);
}
										 
void FontHIDisposePalette	(void *fontPalDataH)
{
	FHI_Errors		err = FontHI_NoErr;

	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		FHIDisposePalette (fontPalDataH);
	else
		FHIPostError (err);
}

FHI_Changes FontHIDoPaletteControl	(void 			*fontPalDataH,
									 ControlHandle	whichControl,
				 					 short	 		whichPart,
							 		 Point			where)
{
	FHI_Errors		err = FontHI_NoErr;
	FHI_Changes		change = FHI_NoChange;
	
	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		change = FHIDoPaletteControl (fontPalDataH, whichControl, whichPart, where);
	else
		FHIPostError (err);
	
	return (change);
}
							 			 
FHI_Changes FontHIDoPaletteClick	(void 			*fontPalDataH,
									 Point			where,
									 short			modifiers,
									 fontRunFeature	*newFeature,
									 Boolean		*newFeatExcl)
{
	FHI_Errors		err = FontHI_NoErr;
	FHI_Changes		change = FHI_NoChange;
	
	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		change = FHIDoPaletteClick (fontPalDataH, where, modifiers,
									newFeature, newFeatExcl);
	else
		FHIPostError (err);
	
	return (change);
}

void FontHIUpdatePalette	(void *fontPalDataH)
{
	FHI_Errors		err = FontHI_NoErr;

	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		FHIUpdatePalette (fontPalDataH);
	else
		FHIPostError (err);
}

void FontHIActivatePalette	(void 		*fontPalDataH,
							 Boolean	activate)
{
	FHI_Errors		err = FontHI_NoErr;

	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		FHIActivatePalette (fontPalDataH, activate);
	else
		FHIPostError (err);
}

short FontHIPaletteFontChange	(void *fontPalDataH,
								 void *fontMenuDataH,
								 short numMappings,
								 FHI_FontMapPtr fontMappings)
{
	FHI_Errors		validationErr = FontHI_NoErr;
	short			memErr = noErr;
	
	validationErr = FHIValidateMenuData (fontMenuDataH);
	
	if (!validationErr)
	{
		validationErr = FHIValidatePaletteData (fontPalDataH);
		
		if (!validationErr)
			memErr = FHIPaletteFontChange (fontPalDataH, fontMenuDataH,
											numMappings, fontMappings);
	}
	
	if (validationErr)
		FHIPostError (validationErr);
	
	if (validationErr = FHIValidatePaletteData (fontPalDataH));
		FHIPostError (validationErr);
	
	return (memErr);
}

WindowPtr FontHIGetPalettePort	(void *fontPalDataH)
{
	FHI_Errors		err = FontHI_NoErr;
	WindowPtr		palettePort = NULL;
	
	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		palettePort = FHIGetPalettePort (fontPalDataH);
	else
		FHIPostError (err);
	
	return (palettePort);
}

Boolean FontHIWindowIsPalette	(void *fontPalDataH,
								 WindowPtr theWindow)
{
	FHI_Errors		err = FontHI_NoErr;
	Boolean			windowIsPalette = false;
	
	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		windowIsPalette = FHIWindowIsPalette (fontPalDataH, theWindow);
	else
		FHIPostError (err);
	
	return (windowIsPalette);
}

Boolean FontHIPaletteVisible	(void *fontPalDataH)
{
	FHI_Errors		err = FontHI_NoErr;
	Boolean			paletteVisible = false;
	
	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		paletteVisible = FHIPaletteVisible (fontPalDataH);
	else
		FHIPostError (err);
	
	return (paletteVisible);
}

font FontHIGetPaletteFontID		(void *fontPalDataH)
{
	FHI_Errors		err = FontHI_NoErr;
	font			theFontID = 0;
	
	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		theFontID = FHIGetPaletteFontID (fontPalDataH);
	else
		FHIPostError (err);
	
	return (theFontID);
}

short FontHIGetSelectedInstance	(void *fontPalDataH)
{
	FHI_Errors		err = FontHI_NoErr;
	short			instanceIndex = 0;
	
	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		instanceIndex = FHIGetSelectedInstance (fontPalDataH);
	else
		FHIPostError (err);
	
	return (instanceIndex);
}

short FontHIGetSelectedFeatures	(void *fontPalDataH,
								 short *numFeatures, fontRunFeature **features)
{
	FHI_Errors		validationErr = FontHI_NoErr;
	short			memErr = noErr;
	
	validationErr = FHIValidatePaletteData (fontPalDataH);
	
	if (!validationErr)
		memErr = FHIGetSelectedFeatures (fontPalDataH, numFeatures, features);
	else
		FHIPostError (validationErr);
		
	return (memErr);
}

Fixed FontHIGetTrackingValue	(void *fontPalDataH)
{
	FHI_Errors		err = FontHI_NoErr;
	Fixed			trackVal = 0;
	
	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		trackVal = FHIGetTrackingValue (fontPalDataH);
	else
		FHIPostError (err);
	
	return (trackVal);
}

short FontHIVariationToIndex	(font theFontID,
								 short numCmpVariations, fontVariation *cmpVariations)
{
	return (FHIVariationToIndex (theFontID, numCmpVariations, cmpVariations));
}

void FontHISetSelectedInstances	(void *fontPalDataH,
								 font fontID, Style textface,
								 short numIndices, short *indices,
								 Boolean clearAll)
{
	FHI_Errors		err = FontHI_NoErr;
	
	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		FHISetSelectedInstances (fontPalDataH, fontID, textface,
								 numIndices, indices, clearAll);
	else
		FHIPostError (err);
}

void FontHISetAllSelectedFeatures	(void *fontPalDataH,
									 font fontID, Style textface,
									 short numFeatureGroups,
									 short **featureGroupSizes,
									 fontRunFeature ***featureGroups)
{
	FHI_Errors		err = FontHI_NoErr;
	
	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		FHISetAllSelectedFeatures (fontPalDataH,
									fontID, textface,
									numFeatureGroups, featureGroupSizes,
									featureGroups);
	else
		FHIPostError (err);
}

void FontHISetTrackingValue		(void *fontPalDataH,
								 font fontID, Style textface, float newValue)
{
	FHI_Errors		err = FontHI_NoErr;
	
	err = FHIValidatePaletteData (fontPalDataH);
	
	if (!err)
		FHISetTrackingValue (fontPalDataH, fontID, textface, newValue);
	else
		FHIPostError (err);
}
