/*
	File:		FontHIPaletteData.h

 	Contains:	internal routines and data structs for FontHI font palette data handling

	Written by:	Dennis Buchheim

	Copyright:	 1992, 1993 by Apple Computer, Inc., All rights reserved.

	Change History (most recent first):

*/

#pragma once

#include <Types.h>
#include <Windows.h>
#include <Menus.h>
#include <Controls.h>

#include <FixMath.h>

#include "font types.h"

#include "FontHIMenuData.h"


#define FHI_PaletteMagic		0x01107084

/*----------*/

typedef struct {
	Str255				name;
	short				menuItem;
	Boolean				selected;
	short				indexInFont;
} FHI_InstItemInfoRec;

typedef struct {
	short				numSelInstances;
	short				numInstances;
	FHI_InstItemInfoRec	instances[];
} FHI_InstMenuInfoRec, *FHI_InstMenuInfoPtr, **FHI_InstMenuInfoHan;

typedef struct {
	fontFeature			featureType;
	fontFeatureFlag		featureFlag;
	fontFeatureSetting	featureSetting;
	short				menuItem;
	Boolean				selected;
} FHI_FeatItemInfoRec;

typedef struct {
	short				numFeatGroups;	/* flags multiple groups of features in selected run */
	short				numDefaultFeatures;
	fontRunFeature		**defaultFeatures;
	long				numOfFeatures;
	FHI_FeatItemInfoRec	features[];
} FHI_FeatMenuInfoRec, *FHI_FeatMenuInfoPtr, **FHI_FeatMenuInfoHan;

typedef struct {
	short				numTracks;
	short				multiplier;
	float				minVal;
	float				maxVal;
	float				curVal;
} FHI_TrackSliderRec, *FHI_TrackSliderPtr, **FHI_TrackSliderHan;

typedef struct {
	Str255				familyName;
	Str255				styleName;
	font				fontID;
	Style				textface;
	FHI_InstMenuInfoHan	instanceH;
	FHI_FeatMenuInfoHan	featH;
	FHI_TrackSliderHan	trackingH;
} FHI_FontDescRec, *FHI_FontDescPtr, **FHI_FontDescHan;

typedef struct {
	
	unsigned long	magicNum;
	
	Boolean			paletteVisible;
	
	WindowPtr		palettePort;
	Handle			paletteWictH;
	Handle			paletteWitlH;
	
	MenuHandle		selFontPopup;
	MenuHandle		selStylePopup;
	MenuHandle		styleVarPopup;
	MenuHandle		featuresPopup;
	ControlHandle	trackingSliderH;
	
	short			selFontChoice;
	short			selStyleChoice;
	
	short			curFontIndex;
	short			numFontsInPalette;
	FHI_FontDescHan	fontDescH;
	
} FHI_PalDataRec, *FHI_PalDataPtr, **FHI_PalDataHan;

/*----------*/

#define FP_FONT_DESC_P(fpdH,i)	((FHI_FontDescPtr) (*(**fpdH).fontDescH + i))
#define FP_INSTANCE_P(fpdH,i)	((FHI_InstMenuInfoPtr) (*(*(**fpdH).fontDescH + i)->instanceH))
#define FP_FEAT_P(fpdH,i)		((FHI_FeatMenuInfoPtr) (*(*(**fpdH).fontDescH + i)->featH))
#define FP_TRACKING_P(fpdH,i)	((FHI_TrackSliderPtr) (*(*(**fpdH).fontDescH + i)->trackingH))

/*----------*/

/*	routines called in FontHIPalette.c
 */
WindowPtr	FHIGetPalettePort			(FHI_PalDataHan fontPalDataH);
Boolean		FHIWindowIsPalette			(FHI_PalDataHan fontPalDataH, WindowPtr theWindow);
Boolean		FHIPaletteVisible			(FHI_PalDataHan fontPalDataH);

font		FHIGetPaletteFontID			(FHI_PalDataHan fontPalDataH);
short		FHIGetSelectedInstance		(FHI_PalDataHan fontPalDataH);
short		FHIGetSelectedFeatures		(FHI_PalDataHan fontPalDataH,
								 		 short *numFeatures, fontRunFeature **features);
Fixed		FHIGetTrackingValue			(FHI_PalDataHan fontPalDataH);

void		FHISetSelectedInstances		(FHI_PalDataHan fontPalDataH,
										 font fontID, Style textface,
										 short numIndices, short *indices,
										 Boolean clearAll);
void		FHISetAllSelectedFeatures	(FHI_PalDataHan fontPalDataH,
										 font fontID, Style textface,
										 short numFeatureGroups, short **featureGroupSizes,
										 fontRunFeature ***featureGroups);
void		FHISetTrackingValue			(FHI_PalDataHan fontPalDataH,
										 font fontID, Style textface, float newValue);


/*	routines called only in FontHIPaletteWind.c
 */
FHI_PalDataHan	FHIInitPaletteData (void);
void			FHIDisposePaletteData (FHI_PalDataHan fontPalDataH);

short FHIGetFontIDIndex			(FHI_PalDataHan fontPalDataH, font fontID, Style textface);
short FHIGetFontNameIndex		(FHI_PalDataHan fontPalDataH, Str255 familyName, Str255 styleName);
short FHIGetFirstMarkedInstance	(FHI_PalDataHan fontPalDataH);

short FHIUpdatePaletteFontInfo	(FHI_PalDataHan fontPalDataH, FHI_MenuDataHan fontMenuDataH);
