/*
	File:		FontHIScrolling.c
	
	Contains:	internal scrolling utility routines for tracking slider
				in Font HI font palette

	Copyright:	Original AppMaker source
				 1988-91, Bowers Development Corp.

				Modifications by Dennis Buchheim
				 1993, Apple Computer, Inc.

*/

#include "FontHIScrolling.h"


/*----------*/
static void DoScrollPart	(ControlHandle	whichScroll,
							 short			partCode)
{
	short			pageSize;
	short			delta;
	short			oldValue;

	pageSize = GetCRefCon (whichScroll);
	switch (partCode) {
	case inUpButton:
			delta = -1;
		break;
	case inDownButton:
			delta = 1;
		break;
	case inPageUp:
			delta = -pageSize;
		break;
	case inPageDown:
			delta = pageSize;
		break;
	default:
			delta = 0;
		break;
	} /*switch*/
	if (delta != 0) {
		oldValue = GetCtlValue (whichScroll);
		SetCtlValue (whichScroll, oldValue + delta);
	}
} /*FontHIDoScrollPart*/
	
/*----------*/
static pascal void ActionGlue	(ControlHandle	whichScroll,
								 short			partCode)
{
	short				oldValue;
	short				newValue;
	FHI_ScrollProcPtr	actionProc;

	oldValue = GetCtlValue (whichScroll);
	DoScrollPart (whichScroll, partCode);
	newValue = GetCtlValue (whichScroll);
	if (newValue != oldValue) {
		actionProc = (FHI_ScrollProcPtr) (**whichScroll).contrlAction;
		(*actionProc) (newValue, oldValue);
	}
} /*ActionGlue*/
 
/*----------*/
void FHITrackScroll	(ControlHandle		whichScroll,
					 short				partCode,
					 Point				where,
					 FHI_ScrollProcPtr	actionProc)
{
	short			oldValue;
	short			newValue;

	if (partCode == inThumb) {
		oldValue = GetCtlValue (whichScroll);
		partCode = TrackControl (whichScroll, where, NULL);
		if (actionProc != NULL) {
			newValue = GetCtlValue (whichScroll);
			(*actionProc) (newValue, oldValue);
		}
	} else {
		if (actionProc == NULL) {
			partCode = TrackControl (whichScroll, where, NULL);
			DoScrollPart (whichScroll, partCode);
		} else {
			(**whichScroll).contrlAction = (ProcPtr) actionProc;
			partCode = TrackControl (whichScroll, where, (ProcPtr) &ActionGlue);
		}
	}
} /*FHITrackScroll*/
