/*
	File:		FontHIWindowAids.c

	Contains:	internal window utility routines supporting FontHI font palette

	Copyright:	Original AppMaker source
				 1988-91, Bowers Development Corp.

				Modifications by Dennis Buchheim
				 1992, 1993, Apple Computer, Inc.

*/

#include <TextEdit.h>
#include <Memory.h>
#include <Fonts.h>
#include <GestaltEqu.h>
#include <OSUtils.h>
#include <Resources.h>
#include <ToolUtils.h>
#include <Menus.h>

#include "FontHIWindowAids.h"


#define topLeft(r)	(((Point *) &(r))[0])


typedef struct {
		Handle			reserved;
		Rect			displayRect;
		Byte			itemKind;
		Byte			dataLen;
} ItemTemplate, *ItemTPtr;

#define ItemTBaseSize	sizeof (ItemTemplate)

typedef struct {
		short			maxItem;
		ItemTemplate	items;	/* repeated */
} WitlTemplate, *WitlTPtr, **WitlTHndl;

typedef struct {
		short			header;
		short			offset;
} ictbEntry, *ictbEntryPtr;

typedef struct {
		ictbEntry		item [5000];
} WictTemplate, *WictTPtr, **WictTHndl;

typedef struct {
		short			fontNum;
		Style			typeStyle;
		SignedByte		just;		/* AppMaker addition */
		short			typeSize;
		RGBColor		fgColor;
		RGBColor		bgColor;
		short			xferMode;
} StyleData, *StyleDataPtr;

/* set background color */
#define doBgColor	0x2000

/* set transfer mode */
#define doXferMode	0x4000

/* set font by name */
#define doByName	0x8000


static short textJust;		/* set by SetWFont */


/*----------*/
void	FHIGetWRect	(Handle			theWitlH,
					 short			itemNr,
					 Rect			*itemRect)
{
	WitlTHndl		witlHandle;
	ItemTPtr		wItemPtr;
	short			I;
	short			dataSize;

	SetRect (itemRect, 0, 0, 16, 16);
	witlHandle = (WitlTHndl) theWitlH;
	if ((witlHandle != NULL)
	&&  (itemNr <= (**witlHandle).maxItem + 1) ) {
		wItemPtr = &(**witlHandle).items;
		for (I = 1; I <= itemNr - 1; I++) {
			dataSize = wItemPtr->dataLen + (wItemPtr->dataLen & 1);
			wItemPtr = (ItemTPtr) (((Ptr)wItemPtr) + (ItemTBaseSize + dataSize));
		} /*for*/
		*itemRect = wItemPtr->displayRect;
	}
} /* FHIGetWRect */
	
/*----------*/
void	FHISetWFont	(Handle			theWictH,
					 short			itemNr)
{
	WictTHndl		wictHandle;
	Ptr				wictPtr;
	ictbEntryPtr	itemPtr;
	short			header;
	short			offset;
	StyleDataPtr	dataPtr;
	StyleData		data;
	StringPtr		fontPtr;
	Str255			fontName;

	wictHandle = (WictTHndl) theWictH;
	if (wictHandle != nil) {
		wictPtr = (Ptr) &(**wictHandle);
		itemPtr = &(**wictHandle).item [itemNr - 1];
		header = itemPtr->header;
		offset = itemPtr->offset;
		dataPtr = (StyleDataPtr) (wictPtr + offset);
		data = *dataPtr;
		if ((header & doFont) == 0) {
			data.fontNum = 0;
		} else {
			if ((header & doByName) != 0) {
				fontPtr = (StringPtr) (wictPtr + data.fontNum);
				BlockMove (fontPtr, &fontName, 256);
				GetFNum (fontName, &data.fontNum);
			}
		}
		if ((header & doFace) == 0) {
			data.typeStyle = 0;
		}
		if ((header & doSize) == 0) {
			data.typeSize = 0;
		}
		if ((header & doXferMode) == 0) {
			data.xferMode = srcCopy;
		}
		TextFont (data.fontNum);
		TextFace (data.typeStyle);
		TextMode (data.xferMode);
		TextSize (data.typeSize);
		textJust = data.just;
	}
} /* FHISetWFont */

/*----------*/
WindowPtr FHIGetWindow	(short			windowID)
{
	long			tempLong;		/*for Gestalt*/
	SysEnvRec		environs;
	Boolean			hasColorQD;

	if (Gestalt (gestaltVersion, &tempLong) == noErr)
		hasColorQD = (tempLong != gestaltOriginalQD);
	else
	{
		(void) SysEnvirons (curSysEnvVers, &environs);
		hasColorQD = environs.hasColorQD;
	}
	
	if (tempLong != gestaltOriginalQD) {
		return (GetNewCWindow (windowID, NULL, (WindowPtr) -1L));
	} else {
		return (GetNewWindow (windowID, NULL, (WindowPtr) -1L));
	}
} /* FHIGetWindow */
	
/*----------*/
void FHITextIDBox	(short			textID,
					 Rect			bounds)
{
	Handle			text;

	text = GetResource ('TEXT', textID);
	if (text != NULL) {
		LoadResource (text);
		HLock (text);
		TextBox (&(**text), GetHandleSize (text), &bounds, textJust);
		HUnlock (text);
	}
} /*FHITextIDBox*/

/*----------*/
void FHIDrawPictID	(short			pictID,
					 Rect			bounds)
{
	PicHandle		pict;

#undef GetPicture
#undef DrawPicture

	pict = GetPicture (pictID);
	if (pict != NULL) {
		LoadResource ((Handle) (pict));
		DrawPicture (pict, &bounds);
	}
} /*FHIDrawPictID*/

/*----------*/
void FHIDrawGrayLine	(Rect			bounds)
{
	PenState		savePen;

	GetPenState (&savePen);
	PenNormal ();
	PenPat ((ConstPatternParam) &qd.gray);
	MoveTo (bounds.left, bounds.top);
	LineTo (bounds.right - 1, bounds.bottom - 1);
	SetPenState (&savePen);
} /*FHIDrawGrayLine*/

/*----------*/
static void	DrawTriangle (const Rect *bounds, Boolean pointDown)
{
	PolyHandle		arrowTriangle;

	arrowTriangle = OpenPoly ();

	if (pointDown)
	{
		MoveTo (bounds->right - 15, bounds->top + 5);
		Line (5, 5);
		Line (5, -5);
		Line (-10, 0);
	}
	else
	{
		MoveTo (bounds->right - 10, bounds->top + 2);
		Line (5, 5);
		Line (-5, 5);
		Line (0, -10);
	}

	ClosePoly ();
	FillPoly (arrowTriangle, (ConstPatternParam) &qd.black);
	KillPoly (arrowTriangle);
} /*DrawTriangle*/

/*----------*/
void FHIUpdatePopup	(Rect			bounds,
					 short			menuID,
					 short			choice)
{
	MenuHandle		menu;
	Str255			menuItem;
	Rect			textRect;

	menu = (MenuHandle) (GetResource ('MENU', menuID));
	if (menu != NULL) {
		GetItem (menu, choice, menuItem);
		textRect = bounds;
		textRect.left = textRect.left + 12;
		textRect.right = textRect.right + 20;
		TextBox (&menuItem [1], menuItem [0], &textRect, teJustLeft);
	}

	DrawTriangle (&bounds, true);

	bounds.right  = bounds.right  - 1;
	bounds.bottom = bounds.bottom - 1;
	FrameRect (&bounds);
	MoveTo (bounds.right, bounds.top + 2);
	LineTo (bounds.right, bounds.bottom);
	LineTo (bounds.left + 2, bounds.bottom);
	
} /*FHIUpdatePopup*/

/*----------*/
void FHITrackPopup	(Rect			bounds,
					 short			menuID,
					 short			*choice)
{
	MenuHandle		menu;
	Point			corner;
	long			result;
	Rect			triangleRect;

	menu = (MenuHandle) (GetResource ('MENU', menuID));
	if (menu != NULL) {
		InsertMenu (menu, -1);
		triangleRect.top = bounds.top + 5;
		triangleRect.left = bounds.right - 15;
		triangleRect.bottom = triangleRect.top + 5;
		triangleRect.right = triangleRect.left + 10;
		EraseRect (&triangleRect);
		corner = topLeft (bounds);
		LocalToGlobal (&corner);
		CheckItem (menu, *choice, true);
		result = PopUpMenuSelect (menu, corner.v, corner.h + 1, *choice);
		CheckItem (menu, *choice, false);
		DeleteMenu (menuID);
		InvalRect (&triangleRect);			/*because we always erase triangleRect*/
		if ((HiWord (result) != 0) && (LoWord (result) != *choice)) {
			*choice = LoWord (result);
			InsetRect (&bounds, 1, 1);
			InvalRect (&bounds);
		}
	}
} /*FHITrackPopup*/

/*----------*/
/*
 *	does not check the selected item; for multiple selections, dashes must be used
 */
void FHITrackMultSelPopup	(Rect			bounds,
							 short			menuID,
							 short			*choice)
{
	MenuHandle		menu;
	Point			corner;
	long			result;
	Rect			triangleRect;

	menu = (MenuHandle) (GetResource ('MENU', menuID));
	
	if (menu != NULL) {
		InsertMenu (menu, -1);
		
		triangleRect.top = bounds.top + 5;
		triangleRect.left = bounds.right - 15;
		triangleRect.bottom = triangleRect.top + 5;
		triangleRect.right = triangleRect.left + 10;
		EraseRect (&triangleRect);
		
		corner = topLeft (bounds);
		LocalToGlobal (&corner);
		
		result = PopUpMenuSelect (menu, corner.v, corner.h + 1, *choice);
		
		DeleteMenu (menuID);
		
		InvalRect (&triangleRect);			/*because we always erase triangleRect*/
		
		if ((HiWord (result) != 0) && (LoWord (result) != *choice)) {
			*choice = LoWord (result);

			InsetRect (&bounds, 1, 1);
			InvalRect (&bounds);
		}
	}
} /*FHITrackMultSelPopup*/

/*----------*/
void FHIUpdatePopRight	(Rect			bounds,
						 short			menuID,
						 short			choice)
{
	MenuHandle		menu;
	Str255			menuItem;
	Rect			textRect;

	menu = (MenuHandle) (GetResource ('MENU', menuID));
	if (menu != NULL) {
		GetItem (menu, choice, menuItem);
		textRect = bounds;
		textRect.left = textRect.left + 12;
		textRect.right = textRect.right + 20;
		TextBox (&menuItem [1], menuItem [0], &textRect, teJustLeft);
	}

	DrawTriangle (&bounds, false);

	bounds.right  = bounds.right  - 1;
	bounds.bottom = bounds.bottom - 1;
	FrameRect (&bounds);
	MoveTo (bounds.right, bounds.top + 2);
	LineTo (bounds.right, bounds.bottom);
	LineTo (bounds.left + 2, bounds.bottom);
	
} /*FHIUpdatePopRight*/

/*----------*/
/*
 *	does not check the selected item; for multiple selections, dashes must be used
 *	creates pop-right menu appearance
 */
void FHITrackMultSelPopRight	(Rect			bounds,
								 short			menuID,
								 short			*choice)
{
	MenuHandle		menu;
	Point			corner;
	long			result;
	Rect			triangleRect;

	*choice = 0;
		
	menu = (MenuHandle) (GetResource ('MENU', menuID));
	
	if (menu != NULL) {
		InsertMenu (menu, -1);
		
		triangleRect.top = bounds.top + 2;
		triangleRect.left = bounds.right - 10;
		triangleRect.bottom = triangleRect.top + 10;
		triangleRect.right = triangleRect.left + 5;
		EraseRect (&triangleRect);
		
		corner.h = bounds.right;
		corner.v = bounds.top;
		LocalToGlobal (&corner);
		
		bounds.left += 1;
		bounds.right -= 2;
		bounds.top += 1;
		bounds.bottom -= 2;
		InvertRect (&bounds);
			
		result = PopUpMenuSelect (menu, corner.v + 1, corner.h, 1);
			
		InvertRect (&bounds);
		bounds.left -= 1;
		bounds.right += 2;
		bounds.top -= 1;
		bounds.bottom += 2;
		
		DeleteMenu (menuID);
		
		InvalRect (&triangleRect);			/*because we always erase triangleRect*/
		
		if ((HiWord (result) != 0) && (LoWord (result) != *choice))
			*choice = LoWord (result);
	}
} /*FHITrackMultSelPopRight*/

/*----------*/
void	FHIEnableControl	(ControlHandle	theControl,
							 Boolean		active)
{
	if (theControl != NULL) {
		if (active) {
			HiliteControl (theControl, 0);
		} else {
			HiliteControl (theControl, 255);
		}
	}
} /*FHIEnableControl*/
