/*
	File:			GXEdit.h
	
	Contains:
	
	Written by:	Barton R. House
	
	Copyright:	 1993 by Apple Computer, Inc., All rights reserved.
	
*/

#ifndef _H_GXEdit_
#define	_H_GXEdit_

#ifndef __PRINTINGMANAGER__
	#include "PrintingManager.h"
#endif

#ifndef graphicsTypesIncludes
	#include "graphics types.h"
#endif

#ifndef fontTypesIncludes
	#include "font types.h"
#endif

#ifndef layoutTypesIncludes
	#include "layout types.h"
#endif

typedef enum {
	gx_edit_no_error = 0,
	gx_edit_internal_fatal_error,
	gx_edit_bad_magic
} GXEditError;

typedef void * GXEditDoc;
typedef void (* GXEditUserErrorProc) (GXEditError, void * reference);

extern void GXEditNew(GXEditDoc * doc, rectangle * viewRect, viewPort docViewPort, fixed leftMargin, fixed rightMargin);
extern void GXEditDispose(GXEditDoc doc);

extern void GXEditClick(GXEditDoc doc, point where, Boolean extend);
extern void GXEditScroll(GXEditDoc doc, short dv);
extern void GXEditSize(GXEditDoc doc, rectangle * viewRect);
extern void GXEditKey(GXEditDoc doc, char key);
extern void GXEditClear(GXEditDoc doc);
extern void GXEditInsert(GXEditDoc doc, void * text, short numText, short textSize);

extern long GXEditGetTop(GXEditDoc doc);
extern long GXEditGetMaxTop(GXEditDoc doc);
extern long GXEditHeight(GXEditDoc doc);

extern void GXEditSetTextFonts(GXEditDoc doc, short numFonts, font * srcFonts, font * dstFonts);
extern void GXEditGetTextFonts(GXEditDoc doc, short * numFonts, font * fonts);

extern void GXEditSetTextSize(GXEditDoc doc, short size);
extern short GXEditGetTextSize(GXEditDoc doc);

extern void GXEditSetFeatures(GXEditDoc doc, font fontId, short numFeatureGroups,
									short * srcFeatureGroupSizes,
									fontRunFeature ** srcFeatureGroups,
									short * dstFeatureGroupSizes,
									fontRunFeature ** dstFeatureGroups);
extern void GXEditGetFeatures(GXEditDoc doc, font fontId, short * numFeatureGroupsPtr,
									short *** featureGroupSizesPtr,
									fontRunFeature **** featureGroupsPtr);

extern void GXEditGetVariations(GXEditDoc doc, font fontId, short * numInstancesPtr,
									short *** instanceSizesPtr, fontVariation **** instancesPtr);
extern void GXEditSetVariations(GXEditDoc doc, font fontId, short numVariations,
									fontVariation * variations);

extern void GXEditSave(GXEditDoc doc, short fRefNum);
extern void GXEditOpen(GXEditDoc * doc, Rect * viewRect, viewPort docViewPort, short fRefNum,
									fixed leftMargin, fixed rightMargin);

extern void GXEditPrint(GXEditDoc doc, Job printJob);

extern void GXEditCut(GXEditDoc doc);
extern void GXEditCopy(GXEditDoc doc);
extern void GXEditPaste(GXEditDoc doc);

extern long GXEditGetScrapLength(GXEditDoc doc);
extern long GXEditGetTextLength(GXEditDoc doc);
extern long GXEditGetSelectionLength(GXEditDoc doc);
extern void GXEditSetSelection(GXEditDoc doc, long selStart, long selEnd);

extern void GXEditGetRunControls(GXEditDoc doc, RunControls * runControls, RunControls * mask);
extern void GXEditSetRunControls(GXEditDoc doc, RunControls * runControls, RunControls * mask);

extern void GXEditGetLayoutOptions(GXEditDoc doc, LayoutOptions * options, LayoutOptions * mask);
extern void GXEditSetLayoutOptions(GXEditDoc doc, LayoutOptions * options, LayoutOptions * mask);

extern Boolean GXEditIdle(GXEditDoc doc);
extern void GXEditUpdate(GXEditDoc doc, rectangle * updateRect);

extern void GXEditGetError(GXEditDoc doc, GXEditError * error, GXEditError * stickyError);
extern void GXEditGetUserError(GXEditDoc doc, GXEditUserErrorProc * userProc, void ** reference);
extern void GXEditSetUserError(GXEditDoc doc, GXEditUserErrorProc userProc, void * reference);

extern void GXEditFlushCaches(GXEditDoc doc);

#endif


