/*
	File:			GXEditDoc.h
	
	Contains:
	
	Written by:	Barton R. House
	
	Copyright:	 1993 by Apple Computer, Inc., All rights reserved.
	
*/

#ifndef _H_Doc
#define	_H_Doc

#ifndef __PRINTINGMANAGER__
	#include <PrintingManager.h>
#endif

#ifndef graphicsTypesIncludes
	#include "graphics types.h"
#endif

#ifndef fontTypesIncludes
	#include "font types.h"
#endif

#ifndef layoutTypesIncludes
	#include "layout types.h"
#endif

#define kDefaultSize			24
#define kDefaultFace			0		/* no defined constant for plain text */

#define kDefaultHeight			18

#define kDefaultFont			0
#define kDefaultFontName		"\pMonaco"

#define kDefaultStyle			0

#define kGXEditFileVersion		0x00010000
#define	kGXEditMagic			0xabcd1234

#define kUndefinedStyle			-1
#define	kUndefinedSize			-1
#define	kUndefinedFont			-1

#define kPrintTopMargin			(72 * 1.0)
#define kPrintLeftMargin		(72 * 1.0)
#define kPrintRightMargin		(72 * 1.0)
#define kPrintBottomMargin		(72 * 1.0)

typedef struct {

	style				textStyle;
	font				textFont;
	short				textSize;
	
} StyleRec, * StylePtr, ** StyleHan;

typedef struct {

	short				styleIndex;		/* the style for this run */
		
	short				paraOffset;		/* offset in the para of first glyph */
	
	short				numText;		/* the amount of text that make up the run */
	void				** text;		/* the text data */
	short				textSize;		/* the size of a single element text */
				
} NewRunRec, * NewRunPtr, ** NewRunHan;

typedef struct {

	void				* layout;		/* the layout for the line */
	point				layoutPos;
	Boolean				dirty;			/* this line is dirty -- the layout needs to be recalculated */
	
	short				ascent;
	short				descent;

	short				start;			/* where this line starts in the paragraph */

	short				height;			/* height of this line */

	short				paraOffset;		/* the paragraph offset of the first glyph */
	short				numText;		/* number of glyphs in this line */

	Boolean				reflow;			/* the line has to be reflowed */
										    
	Boolean				drawn;			/* this line has been drawn and is upto date */
	Boolean				selected;		/* the line selection range is drawn and up to date */
	short				startOffset;	/* the start offset of the selection */
	short				endOffset;		/* the end offset of the selection */
	
} LineRec, * LinePtr, ** LineHan;

typedef struct {

	short				start;			/* where this paragraph starts in the document */

	short				width;			/* paragraph width */
	short				height;			/* paragraph height */
	
	long				docOffset;		/* the document offset of the first glyph */
	short				numText;		/* number of glyphs in this paragraph */
	
	short				numLines;
	LineHan				lines;			/* lines that make up the paragraph */
	
	short				numRuns;		/* the runs that make up the paragraph */
	NewRunHan			runs;
	
	Boolean				drawn;			/* this paragraph has been drawn and parts are uptodate */
	
	Boolean				reflow;			/* this paragraph needs to be reflowed */
	
	LayoutOptions		layoutOptions;	/* the layout options for this paragraph */
	
} ParaRec, * ParaPtr, ** ParaHan;

typedef struct {

	long				startDocOffset;
	long				endDocOffset;
	
	Boolean				endOfLine;			/* the caret selection is at the end of a line */
	
	short				startParaIndex;
	short				startParaOffset;
	short				endParaIndex;
	short				endParaOffset;
		
} SelRec, * SelPtr, ** SelHan;

typedef struct {

	long				numText;
	short				numRuns;
	NewRunHan			runs;
	
} ScrapRec, * ScrapPtr, ** ScrapHan;

typedef struct {
	
	Boolean				start;
	
	short				numStyles;
	short				** styles;

	short				numFonts;
	font				** fonts;
	
	short				size;
	
} AttrRec, * AttrPtr, ** AttrHan;

typedef struct {
	
	long				magic;					/* magic number for validity */
	
	long				length;					/* length of overall document */

	short				lineWidth;				/* width of a line in the document  */
	LayoutOptions		currentLayoutOptions;	/* the default layout options for new paragraphs */
	
	short				numStyles;				/* the styles that appear in the document */
	StyleHan			styles;
			
	long				numText;				/* number of glyphs in the document */

	short				numParagraphs;
	ParaHan				paragraphs;				/* paragraphs that make up the document */
		
	SelRec				selection;				/* the current selection */
	
	Boolean				active;					/* is the document active */
	
	Rect				viewRect;				/* the drawing area */
	short				leftMargin;				/* the left margin */
	short				rightMargin;			/* the right margin */
	
	long				top;					/* the top position in the current view area */
	long				maxTop;					/* maximum top position */
	
	short				numCurrentStyles;		/* the styles, size and font of the current selection */
	short				** currentStyles;
	
	short				currentSize;
	
	short				numCurrentFonts;
	font				** currentFonts;
	
	short				emptyStyle;				/* the style used when the selection is empty */
	
	viewPort			docViewPort;			/* view port for document */
	viewPort			offscreenPort;			/* offscreen view port */
	viewDevice			offscreenDevice;
	viewGroup			offscreenGroup;
	Ptr					offscreenBytes;
	shape				offscreenBitmap;
	
	long				pageTop;				/* the top page margin */
	long				pageBottom;				/* the bottom page margin */
	long				pageLeft;				/* the left page margin position */
	long				pageRight;				/* the right page margin position */
	
	ScrapRec			scrap;					/* the current document scrap */
	
	Boolean				drawn;					/* this document has been drawn and parts are up to date */
	short				bottomClear;			/* from this point to the bottom is clear */

	long				currentTime;			/* the time at the last idle call */
	
	GXEditUserErrorProc	errorProc;				/* user error posting routine */
	long				* errorProcRef;
	
	GXEditError			error;
	GXEditError			stickyError;
	
} DocRec, * DocPtr, ** DocHan;

extern void 	NewDoc(DocHan * doc, Rect * viewRect, viewPort docViewPort, short leftMargin, short rightMargin);
extern void 	DisposeDoc(DocHan doc);

extern void 	DocSetSelection(DocPtr dp, long selStart, long selEnd);
extern long		DocHitTest(DocPtr dp, Point pt);

extern void		DocClick(DocPtr dp, Point where, Boolean extend);
extern void		DocScroll(DocPtr dp, short dv);
extern void		DocSize(DocPtr dp, Rect * viewRect);
extern long		DocSetTop(DocPtr dp, long top);
extern long		DocGetTop(DocPtr dp);
extern long		DocGetMaxTop(DocPtr dp);
extern long		DocGetMaxTop(DocPtr dp);
extern void		DocKey(DocPtr dp, char key);
extern void 	DocClear(DocPtr dp);
extern void 	DocInsert(DocPtr dp, void * text, short numText, short textSize);

extern long		DocHeight(DocPtr dp);

extern void		DocSetTextFonts(DocPtr dp, short numFonts, font * srcFonts, font * dstFonts);
extern void		DocGetTextFonts(DocPtr dp, short * numFonts, font * fonts);

extern void		DocSetTextSize(DocPtr dp, short size);
extern short	DocGetTextSize(DocPtr dp);

extern void		DocSetFeatures(DocPtr dp, font fontId, short numFeatureGroups,
									short * srcFeatureGroupSizes,
									fontRunFeature ** srcFeatureGroups,
									short * dstFeatureGroupSizes,
									fontRunFeature ** dstFeatureGroups);
extern void		DocGetFeatures(DocPtr dp, font fontId, short * numFeatureGroupsPtr,
									short *** featureGroupSizesPtr,
									fontRunFeature **** featureGroupsPtr);

extern void		DocGetVariations(DocPtr dp, font fontId, short * numInstancesPtr,
										short *** instanceSizesPtr, fontVariation **** instancesPtr);
extern void		DocSetVariations(DocPtr dp, font fontId, short numVariations, fontVariation * variations);

extern void		DocSave(DocPtr dp, short fRefNum);
extern void		DocOpen(DocHan * doc, Rect * viewRect, viewPort docViewPort, short fRefNum,
																	short leftMargin, short rightMargin);

extern void		DocPrint(DocPtr dp, Job printJob);

extern void		DocCut(DocPtr dp);
extern void		DocCopy(DocPtr dp);
extern void		DocPaste(DocPtr dp);

extern long		DocGetScrapLen(DocPtr dp);
extern long		DocGetTextLen(DocPtr dp);
extern long		DocGetSelectionLen(DocPtr dp);

extern void		DocGetRunControls(DocPtr dp, RunControls * runControls, RunControls *mask);
extern void		DocSetRunControls(DocPtr dp, RunControls * runControls, RunControls *mask);

extern void		DocGetLayoutOptions(DocPtr dp, LayoutOptions * options, LayoutOptions *mask);
extern void		DocSetLayoutOptions(DocPtr dp, LayoutOptions * options, LayoutOptions *mask);

extern void		DocGetJustification(DocPtr dp, Boolean *setting);
extern void		DocSetJustification(DocPtr dp, Boolean setting);

extern Boolean	DocIdle(DocPtr dp);
extern void		DocUpdate(DocPtr dp, long start, long end);

extern void		DocSetMaxTop(DocPtr dp);

#endif


