/*
	File:			GXEditError.c
	
	Contains:
	
	Written by:		Barton R. House
	
	Copyright:		 1993 by Apple Computer, Inc., All rights reserved.
	
*/

#include <Traps.h>

#include "GXEdit.h"
#include "GXEditDoc.h"
#include "GXEditError.h"

void gxEditPostFatalError(DocPtr dp, GXEditError error)
{
	long trapAddr;
	long unimplementedAddr;
	
	/* check to see if DebugStr() trap is installed */
	
	trapAddr = GetOSTrapAddress(_DebugStr);
	unimplementedAddr = GetOSTrapAddress(_Unimplemented);

	if(trapAddr != unimplementedAddr)
		DebugStr("\pGXEdit fatal error");
	
	if(dp != nil) {
	
		if(dp->errorProc != nil)
			(*dp->errorProc)(error, dp->errorProcRef);
		
		if(dp->stickyError == gx_edit_no_error)
			dp->stickyError = error;
			
		dp->error = error;
		
	}
	
}

void gxEditSetUserError(DocPtr dp, GXEditUserErrorProc errorProc, void * reference)
{
	dp->errorProc = errorProc;
	dp->errorProcRef = reference;
}

void gxEditGetUserError(DocPtr dp, GXEditUserErrorProc * errorProc, void ** reference)
{
	if(errorProc != nil)
		*errorProc = dp->errorProc;
		
	if(reference != nil)
		*reference = dp->errorProcRef;
}

void gxEditGetError(DocPtr dp, GXEditError * error, GXEditError * stickyError)
{
	
	if(error != nil)
		*error = dp->error;
	
	if(stickyError != nil)
		*stickyError = dp->stickyError;
		
	dp->stickyError = gx_edit_no_error;
	
}
