/*
	File:				GXEditNewRun.c
	
	Contains:
	
	Written by:		Barton R. House
	
	Copyright:		 1993 by Apple Computer, Inc., All rights reserved.
	
*/

#include "GXEdit.h"
#include "GXEditDebug.h"
#include "GXEditDoc.h"
#include "GXEditNewRun.h"
#include "GXEditStyle.h"
#include "GXEditError.h"

#include "graphics routines.h"
#include "graphics libraries.h"
#include "font routines.h"
#include "math routines.h"
#include "layout routines.h"


static void CalcNewRunStarts(NewRunPtr rp, short runOffset);

void InsertNewRunText(DocPtr dp, NewRunPtr rp, short runOffset, void * text, short numText)
{
	
	if(runOffset < 0 || runOffset > rp->numText)
		gxEditPostFatalError(dp, gx_edit_internal_fatal_error);
	
	rp->numText += numText;
	
	/* resize the data structures */
	
	SetHandleSize((Handle) rp->text, rp->numText * rp->textSize);
	
	/* shift the text data */
	
	BlockMove((Ptr) ((long) *rp->text + (runOffset * rp->textSize)),
			  (Ptr) ((long) *rp->text + ((runOffset + numText) * rp->textSize)),
					(rp->numText - runOffset - numText) * rp->textSize);
	
	/* copy the new text data */
	
	BlockMove((Ptr) text, (Ptr) ((long) *rp->text + (runOffset * rp->textSize)), numText * rp->textSize);
			
}

void SplitNewRun(DocPtr dp, NewRunPtr rp, short runOffset, NewRunPtr newRun)
{
	long			size;
		
	if(runOffset < 0 || runOffset > rp->numText)
		gxEditPostFatalError(dp, gx_edit_internal_fatal_error);
	
	/* initialize the new run */
	
	newRun->styleIndex = rp->styleIndex;
	newRun->paraOffset = rp->paraOffset + runOffset;
	newRun->numText = rp->numText - runOffset;
	newRun->textSize = rp->textSize;
	
	/* adjust the old line */
	
	rp->numText = runOffset;
	
	/* create and copy over the text */
	
	size = newRun->numText * rp->textSize;
	newRun->text = (void **) NewHandle(size);
	
	BlockMove((Ptr) ((long) *rp->text + (runOffset * newRun->textSize)), (Ptr) *newRun->text, size);
				
	/* adjust the size of the text */
	
	size = rp->numText * rp->textSize;
	SetHandleSize((Handle) rp->text, size);
	
}

/* join the two runs, freeing up the right run */

void JoinNewRuns(DocPtr dp, NewRunPtr left, NewRunPtr right)
{
	short			runOffset;
	
	if(left->styleIndex != right->styleIndex)
		gxEditPostFatalError(dp, gx_edit_internal_fatal_error);
		
	if(left->textSize != right->textSize)
		gxEditPostFatalError(dp, gx_edit_internal_fatal_error);
		
	runOffset = left->numText;
	
	left->numText += right->numText;
	
	/* grow the data structures */
	
	SetHandleSize((Handle) left->text, left->numText * left->textSize);
	
	/* copy over the glyph information */
	
	BlockMove((Ptr) *right->text, (Ptr) ((long) *left->text + (runOffset * right->textSize)),
										right->numText * right->textSize);
	
	/* dispose of the right side data structures */
	
	DisposeNewRun(right);
	
}

void NewNewRun(DocPtr dp, NewRunPtr rp, short styleIndex, short textSize)
{
	#pragma unused(dp)
	rp->styleIndex = styleIndex;
		
	rp->paraOffset = 0;
	rp->numText = 0;
	rp->text = (void **) NewHandle(0);
	rp->textSize = textSize;
			
}

void NewRunClear(DocPtr dp, NewRunPtr rp, short runOffset, short numText)
{
	long			size;
	
	if(numText == 0)
		return;
		
	if(runOffset < 0 || runOffset > rp->numText)
		gxEditPostFatalError(dp, gx_edit_internal_fatal_error);
		
	if(runOffset + numText > rp->numText)
		gxEditPostFatalError(dp, gx_edit_internal_fatal_error);
		
	size = (rp->numText - runOffset - numText) * rp->textSize;
	BlockMove((Ptr)((long) *rp->text + ((runOffset + numText) * rp->textSize)),
			  (Ptr)((long) *rp->text + (runOffset * rp->textSize)), size);

	rp->numText -= numText;
	
	size = rp->numText * rp->textSize;
	SetHandleSize((Handle) rp->text, size);
	
}

void DisposeNewRun(NewRunPtr rp)
{
	DisposeHandle((Handle) rp->text);
}

void SetNewRunTextFont(DocPtr dp, NewRunPtr rp, font fontId)
{
	void		* runFont;
	
	runFont = (void *) GetDocStyleTextFont(dp, rp->styleIndex);
	
	if(runFont == fontId) {
		gxEditPostFatalError(dp, gx_edit_internal_fatal_error);
		return;
	}
	
	rp->styleIndex = SetDocStyleTextFont(dp, rp->styleIndex, fontId);

}

void SetNewRunTextSize(DocPtr dp, NewRunPtr rp, short textSize)
{
	short		runSize;
	
	runSize = GetDocStyleTextSize(dp, rp->styleIndex);
	
	if(runSize == textSize) {
		gxEditPostFatalError(dp, gx_edit_internal_fatal_error);
		return;
	}
		
	rp->styleIndex = SetDocStyleTextSize(dp, rp->styleIndex, textSize);
	
}
