/*
	File:				GXEditScrap.c
	
	Contains:
	
	Written by:		Barton R. House
	
	Copyright:		 1993 by Apple Computer, Inc., All rights reserved.
	
*/

#include <Scrap.h>
#include <Errors.h>

#include "PrintingManager.h"
#include "graphics routines.h"
#include "graphics libraries.h"
#include "font routines.h"
#include "math routines.h"
#include "layout routines.h"

#include "GXEdit.h"
#include "GXEditDebug.h"
#include "GXEditDoc.h"
#include "GXEditScrap.h"
#include "GXEditError.h"

#define	kGXEditScrapType		'gxdt'
#define kGXEditScrapMagic		0x3abf435d
#define kGXEditScrapVersion		0x0

typedef struct {
	spoolBlock		sb;			/* this must be first */
	short			fRefNum;
	long			start;
	long			numBytes;
	DocPtr			dp;
} MySpoolBlock;

static long		MySpoolProcedure(spoolCommand command, spoolBlock * sb);

static Handle	ScrapToHandle(DocPtr dp);
static void 	HandleToScrap(DocPtr dp, Handle scrap);

static	void 	WriteLong(Handle scrap, long *pos, long value);
static	void 	WriteNewRun(Handle scrap, long *pos, NewRunPtr rp);
static	void 	WriteStyle(Handle scrap, long *pos, short styleIndex);
static	void	WriteData(Handle scrap, long * pos, void * buf, long size);

static	void 	ReadLong(Handle scrap, long *pos, long *value);
static	void 	ReadNewRun(Handle scrap, long *pos, NewRunPtr rp);
static	void 	ReadStyle(Handle scrap, long *pos, short *styleIndex);
static	void	ReadData(Handle scrap, long * pos, void * buf, long size);

void gxEditScrapToDesk(DocPtr dp)
{
	Handle	scrap;
	
	scrap = ScrapToHandle(dp);
	
	ZeroScrap();
	
	HLock((Handle) scrap);
	
	PutScrap(GetHandleSize(scrap), kGXEditScrapType, *scrap);
	
	HUnlock((Handle) scrap);
	
	DisposeHandle(scrap);
	
}

void gxEditScrapFromDesk(DocPtr dp)
{
	Handle	scrap;
	long	offset;
	long	length;
	
	scrap = NewHandle(0);
	
	if(GXEditDebug)
		RemoveBlock((long) scrap);
		
	length = GetScrap(scrap, kGXEditScrapType, &offset);
	
	if(length != noTypeErr) {
	
		if(GXEditDebug)
			AddBlock((long) scrap, length, true);
				
		HandleToScrap(dp, scrap);
		
	} else {
	
		if(GXEditDebug)
			AddBlock((long) scrap, 0, true);

		/* empty the current scrap */
		
		dp->scrap.numRuns = 0;
		SetHandleSize((Handle) dp->scrap.runs, 0);
		
	}
	
	
	DisposeHandle(scrap);
}

static Handle ScrapToHandle(DocPtr dp)
{
	long		pos;
	long		numRuns;
	NewRunPtr	rp;
	short		i;
	Handle		scrap;
	
	pos = 0;
	scrap = NewHandle(0);
	
	numRuns = dp->scrap.numRuns;
	
	WriteLong(scrap, &pos, kGXEditScrapMagic);
	WriteLong(scrap, &pos, kGXEditScrapVersion);
	WriteLong(scrap, &pos, numRuns);
	
	HLock((Handle) dp->scrap.runs);

	rp = * dp->scrap.runs;
	for(i=0; i<numRuns; i++, rp++) {
	
		WriteNewRun(scrap, &pos, rp);
		WriteStyle(scrap, &pos, rp->styleIndex);
		
	}
	
	HUnlock((Handle) dp->scrap.runs);
	
	return(scrap);
	
}

static void WriteLong(Handle scrap, long * pos, long value)
{
	if(*pos + sizeof(long) > GetHandleSize(scrap) )
		SetHandleSize((Handle) scrap, *pos + sizeof(long));
	
	*((long *) (((Ptr) *scrap) + *pos)) = value;
	
	*pos += sizeof(long);
}

static void WriteData(Handle scrap, long * pos, void * buf, long size)
{
	if(*pos + size > GetHandleSize(scrap) )
		SetHandleSize((Handle) scrap, *pos + size);
	
	BlockMove((Ptr) buf, ((Ptr) *scrap) + *pos, size);
	
	*pos += size;
}

static void WriteNewRun(Handle scrap, long * pos, NewRunPtr rp)
{
	long		size;
	
	if(*pos + sizeof(NewRunRec) > GetHandleSize(scrap) )
		SetHandleSize((Handle) scrap, *pos + sizeof(NewRunRec));
	
	*((NewRunPtr) (((Ptr) *scrap) + *pos)) = *rp;
	
	*pos += sizeof(NewRunRec);

	/* write out the text information */

	HLock((Handle) rp->text);
	
	size = rp->textSize * rp->numText;
	WriteData(scrap, pos, (void *) *rp->text, size);
	
	HUnlock((Handle) rp->text);
	
}

static void WriteStyle(Handle scrap, long * pos, short styleIndex)
{
	
	WriteLong(scrap, pos, (long) styleIndex);
	
}

static void HandleToScrap(DocPtr dp, Handle scrap)
{
	long		pos;
	long		magic;
	long		version;
	long		numRuns;
	NewRunPtr	rp;
	short		i;
	
	pos = 0;
	
	ReadLong(scrap, &pos, &magic);
	ReadLong(scrap, &pos, &version);
	ReadLong(scrap, &pos, &numRuns);
	
	if(magic != kGXEditScrapMagic || version != kGXEditScrapVersion)
		gxEditPostFatalError(dp, gx_edit_internal_fatal_error);
	
	if(numRuns < 0)
		gxEditPostFatalError(dp, gx_edit_internal_fatal_error);
		
	/* setup the scrap runs in the document */
		
	dp->scrap.numRuns = numRuns;
	SetHandleSize((Handle) dp->scrap.runs, sizeof(NewRunRec) * numRuns);
	
	HLock((Handle) dp->scrap.runs);

	rp = * dp->scrap.runs;
	for(i=0; i<numRuns; i++, rp++) {
	
		ReadNewRun(scrap, &pos, rp);
		ReadStyle(scrap, &pos, &rp->styleIndex);
		
	}
	
	HUnlock((Handle) dp->scrap.runs);
	
}

static void ReadLong(Handle scrap, long * pos, long * value)
{
	if(*pos + sizeof(long) > GetHandleSize(scrap) )
		gxEditPostFatalError(nil, gx_edit_internal_fatal_error);
		
	*value = *((long *) (((Ptr) *scrap) + *pos));
	
	*pos += sizeof(long);
}

static void ReadData(Handle scrap, long * pos, void * buf, long size)
{
	if(*pos + size > GetHandleSize(scrap) )
		gxEditPostFatalError(nil, gx_edit_internal_fatal_error);
	
	BlockMove(((Ptr) *scrap) + *pos, (Ptr) buf, size);
		
	*pos += size;
}


static void ReadNewRun(Handle scrap, long * pos, NewRunPtr rp)
{
	long	size;
	
	if(*pos + sizeof(NewRunRec) > GetHandleSize(scrap) )
		gxEditPostFatalError(nil, gx_edit_internal_fatal_error);
		
	*rp = *((NewRunPtr) (((Ptr) *scrap) + *pos));
	
	*pos += sizeof(NewRunRec);
	
	/* read in the text */

	size = rp->textSize * rp->numText;
	rp->text = (void **) NewHandle(size);
	
	ReadData(scrap, pos, *rp->text, size);
	
}

static void ReadStyle(Handle scrap, long * pos, short * styleIndex)
{
	long		value;
	
	ReadLong(scrap, pos, &value);
	*styleIndex = value;
	
}


#if 0
static void ReadStyle(DocPtr dp, Handle scrap, long * pos, short * styleIndex)
{
	MySpoolBlock		msb;
	long				count;
	shape				emptyShape;
	OSErr				err;
	long				endOfShape;
	
	/* read the buffer size needed for this shape */
	
	ReadLong(scrap, pos, &bufSize);
	
	
	/* read in the GX style information */
	
	msb.sb.buffer = NewPtrbufSize);
	msb.sb.bufferSize = bufSize;
	msb.sb.spoolProcedure = MySpoolProcedure;
	
	GetFPos(fRefNum, &msb.start);
	
	msb.numBytes = endOfShape - msb.start;
	
	msb.fRefNum = fRefNum;
	msb.dp = dp;
	
	emptyShape = UnflattenShape((spoolBlock *) &msb, 1, &dp->docViewPort );
	
	sp->textStyle = GetShapeStyle(emptyShape);		/* this does not increment the owner count */
	sp->textStyle = CopyToStyle(nil, sp->textStyle);
	
	DisposeShape(emptyShape);
	
	/* check that the correct number of bytes was read */
	
	if(msb.numBytes != 0)
		gxEditPostFatalError(dp, gx_edit_internal_fatal_error);
	
	DisposePtr(msb.sb.buffer);
	
	if((err = SetFPos(fRefNum, fsFromStart, endOfShape)) != noErr)
		gxEditPostFatalError(dp, gx_edit_internal_fatal_error);
	
}

#endif


