/*
	File:				GXEditUtils.c
	
	Contains:
	
	Written by:		Barton R. House
	
	Copyright:		 1993 by Apple Computer, Inc., All rights reserved.
	
*/

#include <Memory.h>

#include "GXEdit.h"
#include "GXEditDoc.h"
#define _GXEditUtilsPrivate_
#include "GXEditUtils.h"

#include "graphics routines.h"
#include "math routines.h"

Boolean gxEditSetBitMaskOp(char * src, char * dst, char * mask, long size)
{
	Boolean		modified;
	char		val;
	
	modified = false;
	
	while(size--) {
	
		val = (*dst & ~(*mask)) | (*src & *mask);
		
		if(val != *dst) {
		
			modified = true;
			*dst = val;
			
		}
		
		src++;
		dst++;
		mask++;
		
	}
	
	return(modified);
	
}

void gxEditGetBitMaskOp(char * src, char * dst, char * mask, long size, Boolean firstTime)
{
	if(firstTime) {
	
		while(size--) {
			*dst++ = *src++;
			*mask++ = 0xff;
		}
		
	} else {
	
		while(size--) {
		
			if(*src != *dst)
				*mask &= ~(*src ^ *dst);
			src++;
			dst++;
			mask++;
			
		}
	}
}

void gxEditDisposeOffscreen(DocPtr dp)
{

	if(dp->offscreenBytes != nil)
		DisposePtr(dp->offscreenBytes);
		
	if(dp->offscreenBitmap != nil)
		DisposeShape(dp->offscreenBitmap);
		
	if(dp->offscreenPort != nil)
		DisposeViewPort(dp->offscreenPort);
		
	if(dp->offscreenDevice != nil)
		DisposeViewDevice(dp->offscreenDevice);
		
	if(dp->offscreenGroup != nil)
		DisposeViewGroup(dp->offscreenGroup);
		
}


void gxEditBuildOffscreen(DocPtr dp)
{
	bitmap			bitmapRec;
	long			height;
	long			width;
	long			rowBytes;
	point			bitmapPos;
	mapping			deviceMatrix;
			
	height = dp->viewRect.bottom - dp->viewRect.top;
	width = dp->viewRect.right - dp->viewRect.left;
	rowBytes = (width + 7) / 8;
	rowBytes = (rowBytes + 0x3L) & ~0x3L;

	if(dp->offscreenBytes != nil)
		DisposePtr(dp->offscreenBytes);
	
	dp->offscreenBytes = NewPtr(rowBytes * height);
	
	bitmapRec.image = dp->offscreenBytes;
	bitmapRec.width = width;
	bitmapRec.height = height;
	bitmapRec.rowBytes = rowBytes;
	bitmapRec.pixelSize = 1;
	bitmapRec.space = indexedSpace;
	bitmapRec.set = nil;
	bitmapRec.profile = nil;
	
	bitmapPos.x = ff(dp->viewRect.left);
	bitmapPos.y = ff(dp->viewRect.top);
	
	if(dp->offscreenBitmap == nil)
		dp->offscreenBitmap = NewBitmap(&bitmapRec, &bitmapPos);
	else
		SetBitmap(dp->offscreenBitmap, &bitmapRec, &bitmapPos);
		
	if(dp->offscreenGroup == nil)
		dp->offscreenGroup = NewViewGroup();
		
	if(dp->offscreenDevice == nil)
		dp->offscreenDevice = NewViewDevice(dp->offscreenGroup, dp->offscreenBitmap);
	else
		SetViewDeviceBitmap(dp->offscreenDevice, dp->offscreenBitmap);
		
	if(dp->offscreenPort == nil)
		dp->offscreenPort = NewViewPort(dp->offscreenGroup);
	
	ResetMapping(&deviceMatrix);
	MoveMappingTo(&deviceMatrix, ff(-dp->viewRect.left), ff(-dp->viewRect.top));
	SetViewDeviceMapping(dp->offscreenDevice, &deviceMatrix);
		
}

void gxEditFlushCaches(DocPtr dp)
{
	ParaRec* para = *dp->paragraphs;
	short paraCount = dp->numParagraphs;

	while (paraCount--)
	{	LineRec* line = *para->lines;
		short lineCount = para->numLines;
		
		while (lineCount--)
		{	line->dirty = true;
			++line;
		}
		++para;
	}
}


