/* graphics:	
	font library routines
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Mike Reed, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.
*/

	#include <Memory.h>
	#include <QuickDraw.h>

#include "font library.h"
#include "graphics libraries.h"
#include "font routines.h"
	#include "graphics toolbox.h"

#pragma options(signed_pstrs)	/* for Think C 5.0 */
#ifdef MacintoshIncludes
	#define GetFontName gGetFontName
	#define NewString gNewString
#endif

typedef struct {
	font		fontID;
	char*	fullName;
} fontNameRecord;

static fontNameRecord commonFonts[] = {
	{ nil, "Chicago" },
	{ nil, "Courier" },
	{ nil, "Geneva" },
	{ nil, "Helvetica" },
	{ nil, "Monaco" },
	{ nil, "New York" },
	{ nil, "Symbol" },
	{ nil, "Times Roman" }
};


/***********************************
 *	shape and style library routines to handle  fonts *
 ***********************************/

font GetCommonFont(commonFontEnum fontIndex)
{
	if (fontIndex >= firstCommonFont && fontIndex <= lastCommonFont)
	{	if (commonFonts[fontIndex].fontID == nil)
			commonFonts[fontIndex].fontID = FindCNameFont(fullFontName, commonFonts[fontIndex].fullName);
		return commonFonts[fontIndex].fontID;
	}
	return GetDefaultFont();
}

void SetShapeCommonFont(shape s, commonFontEnum fontIndex)
{
	SetShapeFont(s, GetCommonFont(fontIndex));
}

void SetStyleCommonFont(style s, commonFontEnum fontIndex)
{
	SetStyleFont(s, GetCommonFont(fontIndex));
}

static long CLength(const char *name)
{
	const char *nameEnd = name;
	
	while (*nameEnd++);
	return nameEnd - name - 1;
}

font FindCNameFont(fontName meaning, const char* name)
{
	font fontID = nil;
	FindFonts(0, meaning, macintoshPlatform, romanScript, englishLanguage, CLength(name), (unsigned char *) name, 1, 1, &fontID);
	return fontID;
}

font FindPNameFont(fontName meaning, const unsigned char name[])
{
	font fontID = nil;
	FindFonts(0, meaning, macintoshPlatform, romanScript, englishLanguage, name[0], &name[1], 1, 1, &fontID);
	return fontID;
}

/************ font names *************/

long FindFontCName(font fontID, fontName meaning, char name[])
{
	long length = FindFontName(fontID, meaning, macintoshPlatform, romanScript, englishLanguage, (unsigned char*)name, nil);

	if (name)
		name[length] = 0;
	return length;
}

long FindFontPName(font fontID, fontName meaning, unsigned char name[])
{
	long length = FindFontName(fontID, meaning, macintoshPlatform, romanScript, englishLanguage, &name[1], nil);

	if (name)
		name[0] = length;
	return length;
}
 
long FindStyleFontCName(style s, fontName meaning, char name[])
 {
 	font fontID = GetStyleFont(s);
 	return(FindFontCName(fontID, meaning, name));
 }

long FindStyleFontPName(style s, fontName meaning, unsigned char name[])
 {
 	font fontID = GetStyleFont(s);
 	return(FindFontPName(fontID, meaning, name));
 }

void SetStylePNamedFont(style s, const unsigned char name[])
{
	font fontID = FindPNameFont(fullFontName, name);
	if (fontID != GetStyleFont(s))
		SetStyleFont(s, fontID);
}

void SetStyleCNamedFont(style s, const char* name)
{
	font fontID = FindCNameFont(fullFontName, name);
	if (fontID != GetStyleFont(s))
		SetStyleFont(s, fontID);
}

/************************
 *	Routines for using FindFonts
 ************************/
 
long CountFontFamilies(void)
{
	return FindFonts(nil, familyFontName, 0, 0, 0, 0, nil, 1, selectToEnd, nil);
}

font FindFontFamily(long index, fontPlatform platform, fontScript script, fontLanguage language,
	long nameLength, const unsigned char *name)
{
	font fontID = nil;
	FindFonts(nil, familyFontName, platform, script, language, nameLength, name, index, 1, &fontID);
	return fontID;
}

long CountFontStyles(font family)
{
	return FindFonts(family, 0, 0, 0, 0, 0, nil, 1, selectToEnd, nil);
}

static font GetFontStyle(font family, long index)
{
	font fontID = nil;
	FindFonts(family, 0, 0, 0, 0, 0, nil, index, 1, &fontID);
	return fontID;
}

font FindFontStyle(font family, long index, fontPlatform platform, fontScript script, fontLanguage language,
	long nameLength, const unsigned char *name)
{
	font fontID = nil;

	if (index)
		FindFonts(family, 0, 0, 0, 0, 0, nil, index, 1, &fontID);
	else
		FindFonts(family, styleFontName, platform, script, language, nameLength, name, index, 1, &fontID);
	return fontID;
}

/****************************
		Style matching routines
 ****************************/

typedef struct {
	fixed		minValue;
	fixed		defaultValue;
	fixed		maxValue;
} descriptorRange;

typedef struct {
	descriptorRange	weight;
	descriptorRange	width;
	descriptorRange	slant;
} fontMatchDescriptor;

typedef struct {
	fixed		weight;
	fixed		width;
	fixed		slant;
} fontStyleCoord;

/*********************************************/
/*				PrepareStyleForMatching					*/
/*	this sets the styles font variations to that of the current font	*/
/*	we'll use this as a master and keep it to do our matching off of	*/
/*	if the style already has variations then we'll check to see if	*/
/*	the font has axes, if not we'll nuke the variations and set new	*/
/*	ones based upon the 'fdsc' information(FindFontDescriptor should*/
/*	look at the fond header for old type fonts		???			*/
/*********************************************/
static void PrepareStyleForMatching(style aStyle)
{
	fontStyleCoord coord;
	font fontID = GetStyleFont(aStyle);
	
	coord.weight = 1;
	coord.width = 1;
	coord.slant = 0;

	FindFontDescriptor(fontID, weightVariationTag, &coord.weight);	
	FindFontDescriptor(fontID, widthVariationTag, &coord.width);	
	FindFontDescriptor(fontID, slantVariationTag, &coord.slant);	

	//if (!( GetStyleFontVariations(aStyle, nil)))
	{	fontVariation* variations = (fontVariation*)NewPtr(3 * sizeof(fontVariation));
		{	variations[0].name = weightVariationTag;	/*will actually check to see if current variations are set*/
			variations[0].value = coord.weight;	/*then we'll append any needed ones*/
			variations[1].name = widthVariationTag;	/*need a new routine that will set and create if needed*/
			variations[1].value = coord.width;
			variations[2].name = slantVariationTag;
			variations[2].value = coord.slant;
		}
		SetStyleFontVariations(aStyle, 3, variations);
		DisposPtr((Ptr)variations);
	}
}

/*
 *	This builds a complete description of a font as it is being used in a style.
 *	Use this to compare against the styles in another font family, to find the closest match.
 */
static void BuildFontStyleCoord(style aStyle, fontStyleCoord* coord)
{
	font fontID = GetStyleFont(aStyle);
	long i, count;
	fixed value;
	fontTableTag aTag;

	coord->weight = 1;
	coord->width = 1;
	coord->slant = 0;

	FindFontDescriptor(fontID, weightVariationTag, &coord->weight);	
	FindFontDescriptor(fontID, widthVariationTag, &coord->width);	
	FindFontDescriptor(fontID, slantVariationTag, &coord->slant);	
	
	if (count = GetStyleFontVariations(aStyle, nil))
	{	fontVariation* variations = (fontVariation*)NewPtr(count * sizeof(fontVariation));
		GetStyleFontVariations(aStyle, variations);
		for (i = 0; i < count; i++)
		{	aTag = variations[i].name;
			value = variations[i].value;
			if (aTag == weightVariationTag)
				coord->weight = value;
			else if (aTag == widthVariationTag)
				coord->width = value;
			else if (aTag == slantVariationTag)
				coord->slant = value;
		}
		DisposPtr((Ptr)variations);
	}
}

/*
 *	This builds a complete description of a font, giving its ranges for the various styles
 *		weight, width, slant.
 *	If the font does not support a range for a given style, then min = max = default.
 */
static void BuildFontMatchDescriptor(font fontID, fontMatchDescriptor* descriptor)
{

	descriptor->weight.minValue = descriptor->weight.defaultValue = descriptor->weight.maxValue = 0;
	descriptor->width.minValue = descriptor->width.defaultValue = descriptor->width.maxValue = 0;
	descriptor->slant.minValue = descriptor->slant.defaultValue = descriptor->slant.maxValue = 0;

	if (FindFontDescriptor(fontID, weightVariationTag, &descriptor->weight.defaultValue))
		descriptor->weight.minValue = descriptor->weight.maxValue = descriptor->weight.defaultValue;
	if (FindFontDescriptor(fontID, widthVariationTag, &descriptor->width.defaultValue))
		descriptor->width.minValue = descriptor->width.maxValue = descriptor->width.defaultValue;
	if (FindFontDescriptor(fontID, slantVariationTag, &descriptor->slant.defaultValue))
		descriptor->slant.minValue = descriptor->slant.maxValue = descriptor->slant.defaultValue;

	FindFontVariation(fontID, weightVariationTag, &descriptor->weight.minValue, &descriptor->weight.defaultValue, &descriptor->weight.maxValue, nil);
	FindFontVariation(fontID, widthVariationTag, &descriptor->width.minValue, &descriptor->width.defaultValue, &descriptor->width.maxValue, nil);
	FindFontVariation(fontID, slantVariationTag, &descriptor->slant.minValue, &descriptor->slant.defaultValue, &descriptor->slant.maxValue, nil);
}

/*
 *	This needs to scale each difference by some amount that balances the various styles
 *		weight, width, slant
 *	This could be a global table, or possible font defined?
 */
static fixed ComputeDescriptorMetric(fontStyleCoord* coord, fontMatchDescriptor* descriptor, fontStyleCoord* setting, fontStyleCoord* remaining)
{
	fixed distance = 0;
	fontStyleCoord remainBuffer;

	if (remaining == nil)
		remaining = &remainBuffer;

	remaining->weight = remaining->width = remaining->slant = 0;
																/*setting must fall within the range of descriptor min and max*/
	setting->weight = (coord->weight < descriptor->weight.maxValue) ? coord->weight: descriptor->weight.maxValue;/* min of the maximums*/
	setting->weight = (setting->weight > descriptor->weight.minValue) ? setting->weight: descriptor->weight.minValue;/* max of the minimums*/
	remaining->weight = coord->weight - setting->weight;	/* if positive then we need to add more bolding with a textface, if negative we need to take some away*/
	distance += (remaining->weight > 0) ? FixedMultiply(remaining->weight , prefwghtweighting): -FixedMultiply(remaining->weight , prefwghtweighting);/*absolute value for distance metric */
	
	setting->width = (coord->width < descriptor->width.maxValue) ? coord->width: descriptor->width.maxValue;/* min of the maximums*/
	setting->width = (setting->width > descriptor->width.minValue) ? setting->width: descriptor->width.minValue;/* max of the minimums*/
	remaining->width = coord->width - setting->width;	/* if positive then we need to extend via textface, if negative we need to condense*/
	distance += (remaining->width > 0) ? FixedMultiply(remaining->width , prefwdthweighting): -FixedMultiply(remaining->width , prefwdthweighting);/*absolute value for distance metric  */

	setting->slant = (coord->slant < descriptor->slant.maxValue) ? coord->slant: descriptor->slant.maxValue;/* min of the maximums*/
	setting->slant = (setting->slant > descriptor->slant.minValue) ? setting->slant: descriptor->slant.minValue;/* max of the minimums*/
	remaining->slant = coord->slant - setting->slant;	/* if positive then we need to skew clockwise via textface, if negative we need to skew counter clockwise*/
	/*actually, if there is any slant at all in the coord->slant then we don't want to make an italic font more italic*/
	distance += (remaining->slant > 0) ? FixedMultiply(remaining->slant, prefslntweighting): -FixedMultiply(remaining->slant, prefslntweighting);/*absolute value for distance metric  this is naturally weighted to be last pref*/
	/* could use something like this and then weight it??? distance += FixedDivide(remaining->slant, ff(90) ); */
	if(setting->slant)
		remaining->slant = 0;

	return distance;
}

static void AppendVariation(fontVariation** variation, fontTableTag aTag, fixed value)
{
	fontVariation* var;
	long size = GetHandleSize((Handle)variation);
	
	SetHandleSize((Handle)variation, size + sizeof(fontVariation));
	var = (fontVariation*)((char*)*variation + size);
	var->name = aTag;
	var->value = value;
}

static void CreateCoordVariations(fontStyleCoord* setting, fontMatchDescriptor* descriptor, fontVariation* variation)
{
	long count = 0;

	if (setting->weight != descriptor->weight.defaultValue)
	{	variation[count].name = weightVariationTag;
		variation[count].value = setting->weight;
		count++;
	}
	if (setting->width != descriptor->width.defaultValue) count++;
	{	variation[count].name = widthVariationTag;
		variation[count].value = setting->width;
		count++;
	}
	if (setting->slant != descriptor->slant.defaultValue) count++;
	{	variation[count].name = slantVariationTag;
		variation[count].value = setting->slant;
		count++;
	}
}

/*	currently CreateCoordTextFace makes up the difference between the variation axes and the actual settings of the old style*/
static textFace* CreateCoordTextFace(fontStyleCoord* remainingCoord, fontMatchDescriptor* descriptor, fontStyleCoord* setting)
{
	#pragma unused(setting)
	
	textFace* newFace;
	fixed newBoldAddition = 0;
	
	newFace = (textFace*)NewPtr(sizeof(textFace) +  (1 - anyNumber) * sizeof(faceLayer));
	newFace->faceLayers = 1;
	ResetMapping(&newFace->advanceMapping);

	newFace->faceLayer[0].outlineStyle = nil;
	newFace->faceLayer[0].outlineFill = solidFill;
	newFace->faceLayer[0].flags = 0;
	/*X=default length, Y = desired percentage bold, x = setting weight, y =new percentage bold to apply*/
	/*	y = XY/x	*/		/*newBoldAddition = FixedDivide( FixedMultiply(remainingCoord->weight, descriptor->weight.defaultValue), setting->weight);*/
	 if ( (remainingCoord->weight) && (descriptor->weight.defaultValue == fix1) ) /*if there is bold left and the font is not natuarally bold*/
		/*newBoldAddition = FixedDivide( FixedMultiply(remainingCoord->weight, descriptor->weight.defaultValue), setting->weight);*/
	 	newBoldAddition = fix1/12;	/*this is just a temporary hack until textfaces are nailed down????*/
	newBoldAddition *= (remainingCoord->weight > 0) ? 1: -1;	/*increase or decrease weight*/
	newFace->faceLayer[0].boldOutset.x = newBoldAddition;	/*if negative this makes lighter???*/
	newFace->faceLayer[0].boldOutset.y = 0;

	if (remainingCoord->width || remainingCoord->slant)
	{	transform curTransform = newFace->faceLayer[0].outlineTransform = NewTransform();
		if (remainingCoord->width)
		{	fixed newWidthFactor = fix1+ remainingCoord->width;
			ScaleTransform(curTransform, newWidthFactor, fix1, 0, 0);
		}
		if (remainingCoord->slant)
			SkewTransform(curTransform, -fix1/4, 0, 0, 0);/*should do a slope from angle????*/
	}
	else
		newFace->faceLayer[0].outlineTransform = nil;
	return newFace;
}

static font FindMatchingFont(style currentStyle, font targetFamily, fontVariation* newVariations, textFace** newFace)
{
	fixed bestMetric = ff(32767);
	font bestFont = targetFamily;
	fontStyleCoord currentCoord, bestRemaining, bestSetting;
	fontMatchDescriptor bestDesc;
	long styleIndex, styleCount;

	BuildFontStyleCoord(currentStyle, &currentCoord);	/*maybe take this out later??? use preparestyleformatching*/

	styleCount = CountFontStyles(targetFamily);
	for (styleIndex = 1; styleIndex <= styleCount; styleIndex++)
	{	fixed metric;
		font targetFont = GetFontStyle(targetFamily, styleIndex);
		fontStyleCoord remainingCoord, targetSetting;
		fontMatchDescriptor targetDescriptor;

		BuildFontMatchDescriptor(targetFont, &targetDescriptor);
		if(!GetFont(targetFont, nil, nil))
			DebugStr((const unsigned char *)"\pError in FindMatchingFont with targetFont");
		metric = ComputeDescriptorMetric(&currentCoord, &targetDescriptor, &targetSetting, &remainingCoord);
		if (metric < bestMetric)
		{	bestMetric = metric;
			bestFont = targetFont;
			bestDesc = targetDescriptor;
			bestSetting = targetSetting;
			bestRemaining = remainingCoord;
		}
	}
	if (newVariations)	/*new way we won't have to do this?????*/
		CreateCoordVariations(&bestSetting, &bestDesc, newVariations);
	if (newFace)
		*newFace = bestMetric ? CreateCoordTextFace(&bestRemaining, &bestDesc, &bestSetting) : nil;

	return bestFont;
}

static void DisposeFaceParts(textFace *newFace)
{
	long activeLayer = newFace->faceLayers - 1;

	do {
		if (newFace->faceLayer[activeLayer].outlineStyle)
			DisposeStyle(newFace->faceLayer[activeLayer].outlineStyle);
		if (newFace->faceLayer[activeLayer].outlineTransform)
			DisposeTransform(newFace->faceLayer[activeLayer].outlineTransform);
	} while (--activeLayer >= 0);
	DisposPtr((Ptr) newFace);
}


void SetMatchingStyle(font targetFamily, style theStyle, long matchInfo)
{
	font toFont;
	textFace *newFace = nil;
	fontVariation newVariations[3];/*will only match on wght, wdth, slnt*/
	font fromFont = GetStyleFont(theStyle);
	boolean toggle = false;
	
	if ( !( GetStyleFontVariations(theStyle, nil)) )
		PrepareStyleForMatching(theStyle);
	if(matchInfo & useTextFaceMatching)
	{
		if(matchInfo & useVariationsMatching)
			toFont = FindMatchingFont(theStyle, targetFamily, newVariations, &newFace);
		else
			toFont = FindMatchingFont(theStyle, targetFamily, nil, &newFace);
	}
	if(!(matchInfo & useTextFaceMatching))
	{
		if(matchInfo & useVariationsMatching)
			toFont = FindMatchingFont(theStyle, targetFamily, newVariations, nil);
		else
			toFont = FindMatchingFont(theStyle, targetFamily, nil, nil);
	}

	
	SetStyleFont(theStyle, toFont);
	SetStyleFace(theStyle, nil);	/*remove any textfaces set  will have to do this selectively in the future???*/
	
	if(matchInfo & useVariationsMatching)
	{	short varCount;
		if(varCount = CountFontVariations(toFont))
			SetStyleFontVariations(theStyle, varCount, newVariations);
	}
	if(matchInfo & useTextFaceMatching)
	{
		if(newFace)
			SetStyleFace(theStyle, newFace);
	}	
	if (newFace)
		DisposeFaceParts(newFace);
}

style ReturnMatchingStyle(font targetFamily, style theStyle, long matchInfo)
{	style aStyle = CopyToStyle(nil, theStyle);
	SetMatchingStyle(targetFamily, aStyle, matchInfo);
	return(aStyle);
}
