/* graphics libraries:
	general library routines	
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

	#include <Memory.h>
#include "graphics libraries.h"


#ifdef MacintoshIncludes
	#define GetPicture gGetPicture
#endif


/*************/
/* General routines */
/*************/

void DisposeShapeAt(shape *oldShape)
{
	NilParamReturn(oldShape);
	if (*oldShape) {
		DisposeShape(*oldShape);
		*oldShape = nil;
	}
}

void DisposeStyleAt(style	*oldStyle)
{
	NilParamReturn(oldStyle);
	if (*oldStyle) {
		DisposeStyle(*oldStyle);
		*oldStyle = nil;
	}
}

void DisposeInkAt(ink *oldInk)
{
	NilParamReturn(oldInk);
	if (*oldInk) {
		DisposeInk(*oldInk);
		*oldInk = nil;
	}
}

void DisposeTransformAt(transform *oldTransform)
{
	NilParamReturn(oldTransform);
	if (*oldTransform) {
		DisposeTransform(*oldTransform);
		*oldTransform = nil;
	}
}


/****************/
/* operations on shapes */
/****************/
static void GetSpaceMapping(viewPort port, viewDevice device, mapping *map)
{
	if (port)
		GetViewPortMapping(port, map);
	else
		ResetMapping(map);

	if( device ) {
		mapping temp;
		MapMapping(map, GetViewDeviceMapping(device, &temp));
	}
}

void MapShapeToSpace(shape source, viewPort port, viewDevice device)
{
	mapping map;

	GetSpaceMapping(port, device, &map);
	MapShape(source, &map);
}

void MapShapeFromSpace(shape source, viewPort port, viewDevice device)
{
	mapping map;

	GetSpaceMapping(port, device, &map);
	InvertMapping(&map, &map);
	MapShape(source, &map);
}

void MapPointToSpace(point *pt, viewPort port, viewDevice device)
{
	mapping map;

	GetSpaceMapping(port, device, &map);
	MapPoints(&map, 1, pt);
}

void MapPointFromSpace(point *pt, viewPort port, viewDevice device)
{
	mapping map;

	GetSpaceMapping(port, device, &map);
	InvertMapping(&map, &map);
	MapPoints(&map, 1, pt);
}
