/* graphics libraries:
	graphics state routines	
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

	#include <Memory.h>
#include "graphics libraries.h"
#include "graphics state library.h"

graphicsState lastGraphicsState = nil;

graphicsState NewGraphicsState(void)
{
	graphicsState state = (graphicsState) NewPtr(sizeof(graphicsStateRecord));
	short i;
	
	IfErrorReturnNil(state == nil, out_of_memory);
	for (i = emptyType; i <= pictureType; i++)
		state->defaultShapes[i - 1] = GetDefaultShape(i);
	return lastGraphicsState = state;
}

void DisposeGraphicsState(graphicsState state)
{
	NilParamReturn(state);
	if (lastGraphicsState == state)
		lastGraphicsState = nil;
	DisposPtr((Ptr) state);
}

void GetGraphicsState(graphicsState state)
{
	short i;
	
	NilParamReturn(state);
	for (i = emptyType; i <= pictureType; i++) {
		shape def = GetDefaultShape(i);
		if (state->defaultShapes[i - 1] != def) {
			DisposeShape(state->defaultShapes[i - 1]);
			state->defaultShapes[i - 1] = def;
		}
	}
}

void SetGraphicsState(graphicsState state)
{
	short i;
	
	NilParamReturn(state);
	for (i = emptyType; i <= pictureType; i++)
		SetDefaultShape(state->defaultShapes[i - 1]);
	lastGraphicsState = state;
}

graphicsState SwapGraphicsState(graphicsState state)
{
	graphicsState lastState = lastGraphicsState;
	
	if (lastGraphicsState)
		SetGraphicsState(lastGraphicsState);
	if (state)
		SetGraphicsState(state);
	lastGraphicsState = state;
	return lastState;
}

style SeparateShapeStyle(shape source)
{
	style old = CloneStyle(GetShapeStyle(source)), xnew = CopyToStyle(nil, old);
	SetShapeStyle(source, xnew);
	DisposeStyle(xnew);
	return old;
}

void ReuniteShapeStyle(shape target, style separate)
{
	SetShapeStyle(target, separate);
	DisposeStyle(separate);
	return;
}

ink SeparateShapeInk(shape source)
{
	ink old = CloneInk(GetShapeInk(source)), xnew = CopyToInk(nil, old);
	SetShapeInk(source, xnew);
	DisposeInk(xnew);
	return old;
}

void ReuniteShapeInk(shape target, ink separate)
{
	SetShapeInk(target, separate);
	DisposeInk(separate);
	return;
}
