/* graphics libraries:
	oval routines
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/


#include "graphics libraries.h"

#define FRACROOT2less1	444758426

/*
  * ovalPath must be 18 fixed numbers. Set the path
  * to the oval described in theOval.
  */
static void OvalPath(fixed ovalPath[], const rectangle *r)
{
	register fixed *pointWalker;
	register fixed cx, cy, h, v;

	cx = (r->left + r->right)/2;
	cy = (r->top + r->bottom)/2;
	h = FractMultiply(FRACROOT2less1, (r->right - r->left)/2);
	v = FractMultiply(FRACROOT2less1, (r->bottom - r->top)/2);
	pointWalker = ovalPath;						/* start at the beginning		*/
	*pointWalker++ = 8;							/* eight points				*/
	*pointWalker++ = 0xFF000000;				/* all points off curve		*/
/*
  *	Start on the left edge, go up and around
  */
	*pointWalker++ = r->left;		*pointWalker++ = cy - v;
	*pointWalker++ = cx - h;		*pointWalker++ = r->top;
	*pointWalker++ = cx + h;		*pointWalker++ = r->top;
	*pointWalker++ = r->right;	*pointWalker++ = cy - v;

	*pointWalker++ = r->right;	*pointWalker++ = cy + v;
	*pointWalker++ = cx + h;		*pointWalker++ = r->bottom;
	*pointWalker++ = cx - h;		*pointWalker++ = r->bottom;
	*pointWalker++ = r->left;		*pointWalker++ = cy + v;
}


shape NewOval(const rectangle *ovalData)
{
	fixed ovalPath[18]; 							 /* 1 vector count, 1 control bit long, 8 points */

	NilParamReturnNil(ovalData);
	OvalPath(ovalPath, ovalData);
	return NewPath((path *) ovalPath);
}

void SetOval(shape s, const rectangle *ovalData)
{
	fixed ovalPath[18];

	NilShapeReturn(s);
	NilParamReturn(ovalData);

	OvalPath(ovalPath, ovalData);
	SetPath(s, 0, (path *)ovalPath);
}


void DrawOval(const rectangle *ovalData, shapeFill geom)
{
	register shape ovalShape;
	
	NilParamReturn(ovalData);
	ovalShape = NewOval(ovalData);
	SetShapeFill(ovalShape, geom);
	DrawShape(ovalShape);
	DisposeShape(ovalShape);
}
