/* graphics libraries:	
	quickdraw conversion library
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/


	#include <Errors.h>
	#include <GestaltEqu.h>
	#include <QuickDraw.h>
	#include <Memory.h>
	#include <Resources.h>

#include "graphics libraries.h"
#include "offscreen library.h"
#include "qd library.h"


rectangle *ShortRectToFixed(const Rect *shortRect, rectangle *fixedRect)
{
	register fixed *coord;
	
	NilParamReturnNil(shortRect);
	NilParamReturnNil(fixedRect);
	coord = (fixed *) fixedRect;
	*coord++ = IntToFixed(shortRect->left);
	*coord++ = IntToFixed(shortRect->top);
	*coord++ = IntToFixed(shortRect->right);
	*coord    = IntToFixed(shortRect->bottom);
	return fixedRect;
}


Rect *FixedRectToShort(const rectangle *fixedRect, Rect *shortRect)
{
	register short *coord;
	
	NilParamReturnNil(shortRect);
	NilParamReturnNil(fixedRect);
	coord = (short *) shortRect;
	*coord++ = FixedRound(fixedRect->top);
	*coord++ = FixedRound(fixedRect->left);
	*coord++ = FixedRound(fixedRect->bottom);
	*coord     = FixedRound(fixedRect->right);
	return shortRect;
}

point *ShortPointToFixed(const Point *shortPoint, point *fixedPoint)
{
	NilParamReturnNil(shortPoint);
	NilParamReturnNil(fixedPoint);
	fixedPoint->x = IntToFixed(shortPoint->h);
	fixedPoint->y = IntToFixed(shortPoint->v);
	return fixedPoint;
}


Point *FixedPointToShort(const point *fixedPoint, Point *shortPoint)
{	
	NilParamReturnNil(shortPoint);
	NilParamReturnNil(fixedPoint);
	shortPoint->h = FixedRound(fixedPoint->x);
	shortPoint->v = FixedRound(fixedPoint->y);
	return shortPoint;
}

static colorSet CTabPtrToColorSet(const CTabPtr qdColors)
{
	colorSet gnuSet;

	NilParamReturnNil(qdColors);
	{	setColor *tempColors, *colorPtr;
		unsigned short *qdSpec;
		short count, size;

		size = qdColors->ctSize + 1;
		tempColors = (setColor *) NewPtr(size * sizeof(setColor));
		NilParamReturnNil(tempColors);
		count = size;
		colorPtr = tempColors;
		qdSpec = (unsigned short *) &qdColors->ctTable;
		while (--count >= 0)
		{
			++qdSpec;
			colorPtr->rgb.red = *qdSpec++;
			colorPtr->rgb.green = *qdSpec++;
			colorPtr->rgb.blue = *qdSpec++;
			++colorPtr;
		}
		gnuSet = NewColorSet(rgbSpace, size, tempColors);
		DisposPtr((char *) tempColors);
	}
	return gnuSet;
}


colorSet CTableToColorSet(const CTabHandle qdColors)
{	char flags;
	colorSet gnuSet;

	NilParamReturnNil(qdColors);
	flags = HGetState((char**)qdColors);
	HLock((char**)qdColors);
	gnuSet = CTabPtrToColorSet(*qdColors);
	HSetState((char**)qdColors,flags);
	return gnuSet;
}


CTabHandle ColorSetToCTable(const colorSet set)
{
	long i,n,numbertocopy;
	CTabHandle mycolortable;
	setColor * mycolorlist;
	colorSpace setSpace;
	
	NilColorSetReturnNil(set);
	n = GetColorSet(set, nil, nil);
	if (n < 4) numbertocopy = 2;
	else if (n < 16) numbertocopy = 4;
	else if (n < 256) numbertocopy = 16;
	else numbertocopy = 256;
	mycolorlist = (setColor *)NewPtr(sizeof(setColor)*numbertocopy);
	NilParamReturnNil(mycolorlist);
	mycolortable = (CTabHandle)NewHandle(sizeof(ColorTable)+numbertocopy*sizeof(ColorSpec));
	NilParamReturnNil(mycolortable);
	GetColorSet(set, &setSpace, mycolorlist );
	(*mycolortable)->ctSeed = 0;
	(*mycolortable)->ctFlags = 0;
	(*mycolortable)->ctSize = numbertocopy-1;
	for (i = 0; i < numbertocopy; i++) 
	{	color tempcolor;
	
		tempcolor.element.rgba = mycolorlist[i].rgba;
		tempcolor.space = setSpace;
		tempcolor.profile = nil;
		if (setSpace != rgbSpace) ConvertColor(&tempcolor,rgbSpace, nil, nil);
		(*mycolortable)->ctTable[i].value = 0;
		(*mycolortable)->ctTable[i].rgb.red = tempcolor.element.rgb.red;
		(*mycolortable)->ctTable[i].rgb.green = tempcolor.element.rgb.green;
		(*mycolortable)->ctTable[i].rgb.blue = tempcolor.element.rgb.blue;
	}
	return mycolortable;
}


/*** WARNING - this proc. should only be used with a rowBytes that is a multiple of four */
/*** We should rev this to no longer have this restriction (by copying the bitmap, for example) */
bitmap *ConvertFromQDBitmap(const BitMap *qdBits, bitmap *newBits)
{
	NilParamReturnNil(qdBits);
	NilParamReturnNil(newBits);
	newBits->image = qdBits->baseAddr;
	newBits->rowBytes = qdBits->rowBytes & 0x3fffffff;
	newBits->width = qdBits->bounds.right - qdBits->bounds.left;
	newBits->height = qdBits->bounds.bottom - qdBits->bounds.top;
	newBits->pixelSize = 1;
	newBits->space = indexedSpace;
	newBits->set = nil;
	newBits->profile = nil;
	return newBits;
}

BitMap *ConvertToQDBitmap(const bitmap *newBits, BitMap *qdBits)
{
	NilParamReturnNil(qdBits);
	NilParamReturnNil(newBits);
	qdBits->baseAddr = newBits->image;
	qdBits->rowBytes = newBits->rowBytes;
	SetRect(&qdBits->bounds, 0, 0, newBits->width, newBits->height);
	return qdBits;
}

shape BitMapToShape(const BitMap *srcBits)
{
	bitmap newBits;
	shape bitmapShape;

	NilParamReturnNil(srcBits);
	bitmapShape = NewBitmap(ConvertFromQDBitmap(srcBits, &newBits), nil);
	return bitmapShape;
}

shape PixMapToShape(const PixMapHandle pmHandle)
{
	shape bitsShape;		/* the result shape */
	PixMap pixmaprecord;	/* the header is read into here if we're using partial resources */
	PixMap *pixmap;		/* points either into the resource or to the above record */
	bitmap srcBits;
	bitmap dstBits;
	register short srcRowBytes;
	char savedHandleFlags;		/* so we can restore the handle state after we lock it */
	boolean partialReads = false;	/* are we using partial resources to read a bit at a time? */
	
	NilParamReturnNil(pmHandle);
	if (*pmHandle == nil) {
		long systemVersion;	/* filled out by Gestalt */
		OSErr err;
		
		/* The resource isn't loaded.  Try to read it in all at once, since that's quite a bit faster,
		  * but if that isn't possible because there isn't enough memory, use the partial resource
		  * manager to read it in a bit at a time.
		  */
		LoadResource((Handle) pmHandle);
		err = ResError();
		if (err == memFullErr && Gestalt(gestaltSystemVersion, &systemVersion) == noErr && systemVersion >= 1792) {
			partialReads = true;
			ReadPartialResource((Handle) pmHandle, 0, (Ptr) (pixmap = &pixmaprecord), sizeof(pixmaprecord));
			if (err = ResError()) goto returnResError;
		} else if (err) {
		returnResError:
				PostGraphicsError(err);
				return nil;
		} else {
			(*pmHandle)->pmTable = nil;
			(*pmHandle)->baseAddr = nil;
		}
	}

	
	savedHandleFlags = HGetState((Handle) pmHandle);
	HLock((Handle) pmHandle);
	if (partialReads == false)
		pixmap = *pmHandle;
	dstBits.space = indexedSpace;
	dstBits.set = nil;
	dstBits.profile = nil;
	dstBits.pixelSize = pixmap->pixelSize;
	dstBits.width = pixmap->bounds.right - pixmap->bounds.left;
	dstBits.height = pixmap->bounds.bottom - pixmap->bounds.top;
	srcRowBytes = pixmap->rowBytes & 0x3FFF;
	dstBits.rowBytes = srcRowBytes;
	if (dstBits.pixelSize <= 8) {
		CTabHandle ctHandle;
		
		if (partialReads) {
			long resourceOffset = sizeof(PixMap) + srcRowBytes * dstBits.height;
			ColorTable cTable;
			CTabPtr ctPtr;
			long size;
			
			ReadPartialResource((Handle) pmHandle, resourceOffset, (Ptr) &cTable, sizeof(cTable));
 			size = sizeof(ColorTable) - sizeof(CSpecArray) + cTable.ctSize * sizeof(CSpecArray);
 			ctPtr = (CTabPtr) NewPtr(size);
			IfDebug(ctPtr == nil, "\pcouldn't allocate CTabPtr");
			ReadPartialResource((Handle) pmHandle, resourceOffset, (Ptr) ctPtr, size);
			dstBits.set = CTabPtrToColorSet(ctPtr);
			DisposPtr((Ptr) ctPtr);
		} else if ((ctHandle = pixmap->pmTable) != nil)
			dstBits.set = CTableToColorSet(ctHandle);
		else
			dstBits.set = CTabPtrToColorSet((CTabPtr) ((char *) pixmap + sizeof(PixMap) + srcRowBytes * dstBits.height));
		dstBits.space = indexedSpace;
	} else if (dstBits.pixelSize == 16)
		dstBits.space = rgb16Space;
	else
		dstBits.space = rgb32Space;
	{
		long resourceOffset;
		shape pixShape;
		short scans;
		longRectangle bounds;
		
		/*	Decide how many scan lines to copy at once.  If we use smaller chunks than the whole image,
			we never have to load the whole resource into memory, and the graphics system can offload
			pieces of it as they're written.  Otherwise, the whole resource and the whole destination shape
			must be in memory at once.  Enough lines to come in just under 32K seems like a reasonable number.
		*/
		dstBits.image = nil;
		srcBits = dstBits;
		if (srcRowBytes & 3) {	/* make it a long multiple */
			dstBits.rowBytes = srcRowBytes + 3 & ~3;
			scans = 1;
		} else if (partialReads)
			scans = 1;
		else {
			scans = 32768 / srcRowBytes;
			if (scans == 0)
				scans = 1;
		}
		if (scans > dstBits.height)
			scans = dstBits.height;
		srcBits.height = scans;
		if (partialReads) {
			resourceOffset = sizeof(PixMap);
			srcBits.image = NewPtr(srcBits.rowBytes);
			IfDebug(srcBits.image == nil, "\pcouldn't allocate rowBytes");
		} else if ((srcBits.image = pixmap->baseAddr) == nil)
			srcBits.image = (char *) pixmap + sizeof(PixMap);
		bitsShape = NewBitmap(&dstBits, nil);
		pixShape = NewBitmap(&dstBits, nil);
		bounds.left = bounds.top = 0;
		bounds.right = srcBits.width;
		bounds.bottom = scans;
		while (bounds.top < dstBits.height) {
			if (partialReads) {
				ReadPartialResource((Handle) pmHandle, resourceOffset, srcBits.image, srcBits.rowBytes);
				resourceOffset += srcBits.rowBytes;
			}
			SetBitmap(pixShape, &srcBits, nil);
			SetBitmapParts(bitsShape, &bounds, pixShape);
			bounds.top += scans;
			bounds.bottom += scans;
			if (bounds.bottom > dstBits.height)
				bounds.bottom = dstBits.height;
			if (partialReads == false)
				srcBits.image += scans * srcBits.rowBytes;
		}
		if (partialReads)
			DisposPtr(srcBits.image);
		DisposeShape(pixShape);
	}
	HSetState((Handle) pmHandle, savedHandleFlags);
	if (dstBits.set)
		DisposeColorSet(dstBits.set);
	return bitsShape;
}


shape GetPixMapShape(short resourceID)
{
	PixMapHandle pmHandle;
	
	SetResLoad(false);
	pmHandle = (PixMapHandle) GetResource('pxmp', resourceID);
	SetResLoad(true);
	if (pmHandle) {
		shape dest;
		
		dest = PixMapToShape(pmHandle);
		ReleaseResource((char**)pmHandle);
		return dest;
	} else
		return nil;
}

#define kColorSpecSize 8

shape CICNToMask(CIconHandle iconH)
{
	CIconPtr iconP;
	
	NilParamReturnNil(iconH);
	iconP = *iconH;
	iconP->iconMask.baseAddr = (Ptr) &iconP->iconMaskData;
	return BitMapToShape(&iconP->iconMask);
}

shape CICNToShape(CIconHandle iconH)
{
	NilParamReturnNil(iconH);
	{	shape result = nil;
		CIconPtr iconP = *iconH;
		long height = iconP->iconPMap.bounds.bottom - iconP->iconPMap.bounds.top;
		ColorTable* clut = 	(ColorTable*) (((char*)&iconP->iconMaskData) 
												+ iconP->iconBMap.rowBytes*height
												+ iconP->iconMask.rowBytes*height);
		long clutSize = 8+((clut->ctSize+1)*kColorSpecSize);
	
		PtrToHand((Ptr) clut, (Handle *) &iconP->iconPMap.pmTable, clutSize);
		iconP->iconPMap.baseAddr = (((char*)clut) + clutSize);
		result = PixMapToShape((PixMapHandle) iconH);
		DisposHandle((Handle) iconP->iconPMap.pmTable);
		return result;
	}
}
	
shape GetCICNMask(long resourceID)
{
	shape result = nil;
	CIconHandle iconH = (CIconHandle) GetResource('cicn', resourceID);
	
	if (iconH) {
		result = CICNToMask(iconH);
		ReleaseResource((Handle)iconH);
	}
	return result;
}
	
shape GetCICNShape(long resourceID)
{
	shape result = nil;
	CIconHandle iconH = (CIconHandle) GetResource('cicn', resourceID);
	
	if (iconH) {
		result = CICNToShape(iconH);
		ReleaseResource((Handle)iconH);
	}		
	return result;
}
