/* graphics libraries
	rounded corner rectangle routines
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

/************
	Includes
*************/
#include "graphics libraries.h"

/*********
	Code
**********/
static void RoundRectPath(fixed *contourData, register const rectangle *r, const point *ovalSize)
{
	NilParamReturn(contourData);
	NilParamReturn(r);
	NilParamReturn(ovalSize);
	{	register fixed *contourWalker = contourData;
		register fixed offX = ovalSize->x/2;
		register fixed offY = ovalSize->y/2;
		register fixed maxX = (r->right - r->left) / 2;
		register fixed maxY = (r->bottom - r->top) / 2;
		
		if (offX > maxX)
			offX = maxX;
		if (offY >  maxY)
			offY = maxY;
		*contourWalker++ = 12;				/* twelve points, all in all			*/
		*contourWalker++ = 0x49200000;		/* one on, one off, one on... times 4		*/
	
		*contourWalker++ = r->left;			/* 1st pt (top left corner)			*/
		*contourWalker++ = r->top + offY;
		*contourWalker++ = r->left;			/* 2nd pt							*/
		*contourWalker++ = r->top;
		*contourWalker++ = r->left + offX;		/* 3rd pt							*/
		*contourWalker++ = r->top;
	
		*contourWalker++ = r->right - offX;		/* 4th pt (top right corner)			*/
		*contourWalker++ = r->top;
		*contourWalker++ = r->right;			/* 5th pt							*/
		*contourWalker++ = r->top;
		*contourWalker++ = r->right;			/* 6th pt							*/
		*contourWalker++ = r->top + offY;
	
		*contourWalker++ = r->right;			/* 7th pt (bottom right corner)			*/
		*contourWalker++ = r->bottom - offY;
		*contourWalker++ = r->right;			/* 8th pt							*/
		*contourWalker++ = r->bottom;
		*contourWalker++ = r->right - offX;		/* 9th pt							*/
		*contourWalker++ = r->bottom;
	
		*contourWalker++ = r->left + offX;		/* 10th pt (bottom left corner)			*/
		*contourWalker++ = r->bottom;
		*contourWalker++ = r->left;			/* 11th pt						*/
		*contourWalker++ = r->bottom;
		*contourWalker++ = r->left;			/* 12th pt						*/
		*contourWalker++ = r->bottom - offY;
	}
}

shape NewRoundRect(const rectangle *rectData, const point *ovalSize)
{
	long newContour[26];

	NilParamReturnNil(rectData);
	NilParamReturnNil(ovalSize);

	RoundRectPath(newContour, rectData, ovalSize);
	return NewPath((path *) newContour);
}

void SetRoundRect(shape aShape, const rectangle *rectData, const point *ovalSize)
{
	long newContour[26];

	NilShapeReturn(aShape);
	NilParamReturn(rectData);
	NilParamReturn(ovalSize);

	RoundRectPath(newContour, rectData, ovalSize);
	SetPath(aShape, 0, (path *) newContour);
}

void DrawRoundRect(const rectangle *bounds, const point *ovalSize, shapeFill fill)
{	
	NilParamReturn(bounds);
	NilParamReturn(ovalSize);
	{	register shape newShape = NewRoundRect(bounds, ovalSize);
	
		SetShapeFill(newShape, fill);
		DrawShape(newShape);
		DisposeShape(newShape);
	}
}
