/* graphics libraries:
	shape library
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

	#include <Memory.h>
#include "graphics libraries.h"

shape NewPolygon(const polygon *polyData)
{
	shape sh;

	NilParamReturnNil(polyData);
	sh = NewShape(polygonType);
	SetPolygon(sh, 0, polyData);
	return sh;
}

shape NewPath(const path *pathData)
{
	shape sh;

	NilParamReturnNil(pathData);
	sh = NewShape(pathType);
	SetPath(sh, 0, pathData);
	return sh;
}

path *GetPath(shape source, long contour, register path *pathData)
{
	NilShapeReturnNil(source);
	NilParamReturnNil(pathData);
	{	register paths *tempData;
		long index = GetShapeIndex(source, contour, 1);
		long points = CountShapePoints(source, contour);
		long size = GetPathParts(source, index, points, nil);
		
		tempData = (paths *) NewPtr(size);
		NilParamReturnNil(tempData);
		GetPathParts(source, index, points, tempData);
		BlockMove(&tempData->contour[0], pathData, size - sizeof(long));
		DisposPtr((void *) tempData);
		return pathData;
	}
}

polygon *GetPolygon(shape source, long contour, register polygon *polygonData)
{
	NilShapeReturnNil(source);
	NilParamReturnNil(polygonData);
	{	register polygons *tempData;
		long index = GetShapeIndex(source, contour, 1);
		long points = CountShapePoints(source, contour);
		long size = GetPolygonParts(source, index, points, nil);
	
		tempData = (polygons *) NewPtr(size);
		NilParamReturnNil(tempData);
		GetPolygonParts(source, index, points, tempData);
		BlockMove(&tempData->contour[0], polygonData, size - sizeof(long));
		DisposPtr((void *) tempData);
		return polygonData;
	}
}

void SetPolygon(shape source, long contour, register const polygon *polygonData)
{
	NilShapeReturn(source);
	NilParamReturn(polygonData);
	{	long size = (long) &((polygons *) 0)->contour[0].vector[polygonData->vectors];
		register polygons *tempData;
		
		tempData = (polygons *) NewPtr(size);
		NilParamReturn(tempData);
		BlockMove(polygonData, &tempData->contour[0], size - sizeof(long));
		tempData->contours = 1;
		if( contour )
			SetPolygonParts(source, GetShapeIndex(source, contour, 1), CountShapePoints(source, contour), tempData, 0);
		else
			SetPolygons(source, tempData);
		DisposPtr((void *) tempData);
	}
}

void SetPath(shape source, long contour, register const path *pathData)
{
	NilShapeReturn(source);
	NilParamReturn(pathData);
	{	long size = ((long) &((paths *) 0)->contour[0].vector[pathData->vectors]) + pathData->vectors / 32 * sizeof(long);
		register paths *tempData;
	
		tempData = (paths *) NewPtr(size);
		NilParamReturn(tempData);
		BlockMove(pathData, &tempData->contour[0], size - sizeof(long));
		tempData->contours = 1;
		if( contour )
			SetPathParts(source, GetShapeIndex(source, contour, 1), CountShapePoints(source, contour), tempData, 0);
		else
			SetPaths(source, tempData);
		DisposPtr((void *) tempData);
	}
}

void DrawPolygon(const polygon *polyData, shapeFill fill)
{
	shape sh;

	NilParamReturn(polyData);
	sh = NewPolygon(polyData);
	SetShapeFill(sh, fill);
	DrawShape(sh);
	DisposeShape(sh);
}


void DrawPath(const path *pathData, shapeFill fill)
{
	shape sh;

	NilParamReturn(pathData);
	sh = NewPath(pathData);
	SetShapeFill(sh, fill);
	DrawShape(sh);
	DisposeShape(sh);
}

void SetShapeIndexPoint(shape source, long index, const point *data)
{
	SetShapePoints(source, index, 1, data);
}

point *GetShapeIndexPoint(shape source, long index, point *data)
{
	GetShapePoints(source, index, 1, data);
	return data;
}

void SetShapeIndexControl(shape source, long index, long control)
{
	NilShapeReturn(source);
	if( GetShapeType(source) == pathType ) 
	{	long pathData[6];
	
		GetPathParts(source, index, 1, (paths *)pathData);
		if( control )
			pathData[2] = 0x80000000;
		else
			pathData[2] = 0x00000000;
		SetPathParts(source, index, 1, (paths *)pathData, 0);
	} else
		PostGraphicsWarning(graphic_type_does_not_contain_control_bits);
}

long GetShapeIndexControl(shape source, long index, long *control)
{
	NilShapeReturnNil(source);
	IfErrorReturnNil(index < 1 || index > CountShapePoints(source, 0), index_out_of_range);
	{	shapeType theType = GetShapeType(source);
		long result = 0;
	
		switch( theType )
		{
			case pathType:
			{
				long pathData[6];
				GetPathParts(source, index, 1, (paths *)pathData);
				if( pathData[2] )
					result = true;
				break;
			}
			case glyphType:
			{
				long bits;
				GetGlyphPositions(source, index, 1, &bits, nil);
				if( bits )
					result = true;
				break;
			}
			case curveType:
				if( index == 2 )
					result = 1;
		}
	
		if( control )
			*control = result;
		return result;
	}
}

void InsertShape(shape source, long index, shape toAdd)
{
	SetShapeParts(source, index, 0, toAdd, breakLeftEdit + breakRightEdit + removeDuplicatePointsEdit);
}

shape ExtractShape(shape source, long firstPoint, long numPoints)
{
	shape newShape;

	newShape = GetShapeParts(source, firstPoint, numPoints, nil);
	if( newShape )
		SetShapeParts(source, firstPoint, numPoints, nil, removeDuplicatePointsEdit);
	return newShape;
}

void AddToShape(shape dest, shape add)
{
	SetShapeParts(dest, 0, 0, add, breakLeftEdit);
}

void ExtendShape(shape dest, shape add)
{
	SetShapeParts(dest, 0, 0, add, 0);
}

#ifdef debugging
shape NewShape2(shapeType typeID, long arg1, long arg2)
{
	IfWarningReturnNil(typeID != pointType, point_expected);
	return NewShapeMany(typeID, arg1, arg2);
}

shape NewShape4(shapeType typeID, long arg1, long arg2, long arg3, long arg4)
{
	IfWarningReturnNil(typeID != lineType && typeID != rectangleType, line_or_rectangle_expected);
	return NewShapeMany(typeID, arg1, arg2, arg3, arg4);
}

shape NewShape6(shapeType typeID, long arg1, long arg2, long arg3, long arg4, long arg5, long arg6)
{
	IfWarningReturnNil(typeID != curveType, curve_expected);
	return NewShapeMany(typeID, arg1, arg2, arg3, arg4, arg5, arg6);
}

void SetShape2(register shape dest, long arg1, long arg2)
{
	IfWarningReturn(GetShapeType(dest) != pointType, point_expected);
	SetShapeMany(dest, arg1, arg2);
}

void SetShape4(register shape dest, long arg1, long arg2, long arg3, long arg4)
{
	register shapeType typeID;

	NilShapeReturn(dest);
	typeID = GetShapeType(dest);
	IfWarningReturn(typeID != lineType && typeID != rectangleType, line_or_rectangle_expected);
	SetShapeMany(dest, arg1, arg2, arg3, arg4);
}

void SetShape6(register shape dest, long arg1, long arg2, long arg3, long arg4, long arg5, long arg6)
{
	register shapeType typeID;

	NilShapeReturn(dest);
	typeID = GetShapeType(dest);
	IfWarningReturn(typeID != curveType, curve_expected);
	SetShapeMany(dest, arg1, arg2, arg3, arg4, arg5, arg6);
}
#endif

shape NewShapeMany(shapeType type, fixed arg1, ...)
{
	return NewShapeVector(type, &arg1);
}

void SetShapeMany(shape sourceShape, fixed arg1, ...)
{
	NilShapeReturn(sourceShape);
	SetShapeVector(sourceShape, &arg1);
}

void CenterShape(shape target, rectangle *container)
{
	rectangle shapeBounds;
	
	NilShapeReturn(target);
	NilParamReturn(container);
	GetShapeBounds(target, 0, &shapeBounds);
	MoveShape(target,	((container->left + container->right) - (shapeBounds.left + shapeBounds.right))/2,
					((container->top + container->bottom) - (shapeBounds.top + shapeBounds.bottom))/2);
}

void MoveShapeCenterTo(shape target, fixed x, fixed y)
{
	point centerPt;

	NilShapeReturn(target);
	GetShapeCenter(target, 0, &centerPt);
	MoveShape(target, x - centerPt.x, y - centerPt.y);
}

void RotateShapeAboutCenter(shape target, fixed degrees)
{
	point centerPt;

	NilShapeReturn(target);
	GetShapeCenter(target, 0, &centerPt);
	RotateShape(target, degrees, centerPt.x, centerPt.y);
}

void SkewShapeAboutCenter(shape target, fixed xSkew, fixed ySkew)
{
	point centerPt;

	NilShapeReturn(target);
	GetShapeCenter(target, 0, &centerPt);
	SkewShape(target, xSkew, ySkew, centerPt.x, centerPt.y);
}

void ScaleShapeAboutCenter(shape target, fixed hScale, fixed vScale)
{
	point centerPt;

	NilShapeReturn(target);
	GetShapeCenter(target, 0, &centerPt);
	ScaleShape(target, hScale, vScale, centerPt.x, centerPt.y);
}

void SetShapeOpenPath(shape target)
{
	long contours, i, index;

	NilShapeReturn(target);
	IgnoreGraphicsNotice(shape_type_already_set);
	SetShapeType(target, pathType);
	PopGraphicsNotice();
	IgnoreGraphicsNotice(parameters_have_no_effect);
	SetShapeFill(target, openFrameFill);
	PopGraphicsNotice();

	contours = CountShapeContours(target);
	index = 0;
	for (i = 1; i <= contours; i++) {
		long points = CountShapePoints(target,i);
		if (points) {
			SetShapeIndexControl(target,index += 1,false);
			SetShapeIndexControl(target,index += points-1,false);
		}
	}
}

/*******************/
/* drawing operations          */
/*******************/
static void CommonPaint(shape s, commonColor c)
{
	NilShapeReturn(s);
	IgnoreGraphicsNotice(color_already_set);
	SetShapeCommonColor(s,c);
	PopGraphicsNotice();
	DrawShape(s);
	DisposeShape(s);
}


void PaintRectangle(const rectangle *r, commonColor c)
{
	CommonPaint(NewRectangle(r),c);
}

void PaintRectangle2(const point *pt1, const point *pt2, commonColor c)
{
	CommonPaint(NewShape4(rectangleType, pt1->x, pt1->y, pt2->x, pt2->y),c);
}

void PaintRectangle4(fixed left, fixed top, fixed right, fixed bottom, commonColor c)
{
	CommonPaint(NewShape4(rectangleType, left, top, right, bottom),c);
}

/*******************/
/* operations on geometries */
/*******************/

void GetPathsIndexPointControl(const paths *pathData, long index, point **pt, long **controlPtr, long *controlMask)
{
	NilParamReturn(pathData);
	{	register long *pathWalker = (long *) pathData;
		register short contours = *pathWalker++;
		register short vectors;
	
		while (contours && index > (vectors = *pathWalker++)) {
			pathWalker += (vectors + 31 >> 5) + (vectors << 1);
			contours--;
			index -= vectors;
			}
		if (contours) {
			index--;
			if (controlPtr)
				*controlPtr = &pathWalker[index >> 5];
			 if (controlMask)
			 	*controlMask = 0x80000000 >> (index & 31);
			pathWalker += vectors + 31 >> 5;
			if (pt)
				*pt = (point *) (pathWalker + (index << 1));
		} else
			PostGraphicsWarning(index_out_of_range_in_contour);
	}
}

/**************************/
/* Get/SetBitmapParts utility routines */
/**************************/

shape GetBitmapPartsFromFixedBounds(shape source, const rectangle *bounds)
{
	longRectangle pixelBounds;
	pixelBounds.top = FixedRound(bounds->top);			/* use = ((bounds->top + 0x00007fff) >> 16) if you want to include */
	pixelBounds.left = FixedRound(bounds->left);			/* pixels whose centers touch the bounding rectangle on the top */
	pixelBounds.bottom = FixedRound(bounds->bottom);		/* (the same applies for left, as well) */
	pixelBounds.right = FixedRound(bounds->right);
	return (GetBitmapParts(source, &pixelBounds));
}

void SetBitmapPartsFromFixedBounds(shape target, const rectangle *bounds, shape bitmapShape)
{
	longRectangle pixelBounds;
	pixelBounds.top = FixedRound(bounds->top);			/* use = ((bounds->top + 0x00007fff) >> 16) if you want to include */
	pixelBounds.left = FixedRound(bounds->left);			/* pixels whose centers touch the bounding rectangle on the top */
	pixelBounds.bottom = FixedRound(bounds->bottom);
	pixelBounds.right = FixedRound(bounds->right);
	SetBitmapParts(target, &pixelBounds, bitmapShape);
}
