/* graphics libraries:
	Spline generation routines
	by Cary Clark, Georgiann Delaney, Michael Fairman, Pablo Fernicola, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1993 Apple Computer, Inc.  All rights reserved.	*/

#include <Memory.h>
#include "graphics libraries.h"

enum { onCurvePoint, offCurvePoint };

typedef struct
{	long count;
	unsigned long mask;
	long control;
	long *bits;
	point *points;
} splineBuild;

static void BeginSplineBuild(splineBuild *build, long *bits, point *points)
{
	build->count = 0;
	build->mask = 0;
	build->control = 0;
	build->bits = bits;
	build->points = points;
}

static void EndSplineBuild(splineBuild *build)
{	
	if (build->mask) *build->bits = build->control;
}

static void AddPoint(splineBuild *build, point *add, boolean offCurve)
{
	*build->points++ = *add;
	if (build->mask == 0) build->mask = (-1UL >> 1) + 1;
	if (offCurve) build->control |= build->mask;
	if ((build->mask >>= 1) == 0)
	{	*build->bits++ = build->control;
		build->control = 0;
	}
	++build->count;
}

static void AddTangent(splineBuild *build, point *base, point *tangent, fixed factor)
{
	point control;
	control.x = base->x + FractMultiply(tangent->x, factor);
	control.y = base->y + FractMultiply(tangent->y, factor);
	AddPoint(build, &control, offCurvePoint);
}

shape MirrorSpline(long count, point *points, fract slack, boolean closed)
{
	shape spline;
	point *this, *next;
	fixed prevLen, nextLen;
	point prevDir, nextDir;
	splineBuild build;
	long actIndex, estCount = 3 * count, estIndex = 2 + (estCount + 31 >> 5);
	long *storage = (long *)NewPtr(estIndex * sizeof(long) + estCount * sizeof(point));

	BeginSplineBuild(&build, storage + 2, (point *)(storage + estIndex));

	this = next = points;
	if (closed) this += count - 1;
	nextDir.x = next->x - this->x;
	nextDir.y = next->y - this->y;
	if (nextLen = Magnitude(nextDir.x, nextDir.y))
	{	nextDir.x = FractDivide(nextDir.x, nextLen);
		nextDir.y = FractDivide(nextDir.y, nextLen);
	}

	while (count--)
	{	this = next;
		if (count) ++next;
		else if (closed) next = points;
		if (prevLen = nextLen) prevDir = nextDir;
		nextDir.x = next->x - this->x;
		nextDir.y = next->y - this->y;
		if (nextLen = Magnitude(nextDir.x, nextDir.y))
		{	nextDir.x = FractDivide(nextDir.x, nextLen);
			nextDir.y = FractDivide(nextDir.y, nextLen);
			if (prevLen && slack)
			{	point tangent;
				tangent.x = (prevDir.x >> 1) + (nextDir.x >> 1) >> 1;
				tangent.y = (prevDir.y >> 1) + (nextDir.y >> 1) >> 1;
				AddTangent(&build, this, &tangent, -FractMultiply(slack, prevLen));
				if (nextLen != prevLen) AddPoint(&build, this, onCurvePoint);
				AddTangent(&build, this, &tangent, FractMultiply(slack, nextLen));
			}
			else goto addThis;
		}
		else if (!closed && count == 0)
	addThis:	AddPoint(&build, this, onCurvePoint);
	}

	EndSplineBuild(&build);

	actIndex = 2 + (build.count + 31 >> 5);
	if (actIndex < estIndex)
		BlockMove(storage + estIndex, storage + actIndex, build.count * sizeof(point));
	storage[0] = 1;
	storage[1] = build.count;
	spline = NewPaths((paths *)storage);
	if (closed) SetShapeFill(spline, closedFrameFill);
	else SetShapeFill(spline, openFrameFill);

	DisposePtr((Ptr)storage);
	return spline;
}
