/* graphics libraries:
	text library
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/


	#include <Memory.h>

#include "font routines.h"
#include "graphics libraries.h"
#include "layout routines.h"

#ifdef MacintoshIncludes
	#define DrawText gDrawText
	#define DrawChar gDrawChar
#endif


shape NewChar(char data, const point *position)
{
	return NewText(1, (unsigned char *)&data, position);
}


void SetChar(shape target, char data, const point *position)
{
	SetText(target, 1, (unsigned char *)&data, position);
}


void DrawChar(char data, const point *position)
{
	DrawText(1, (unsigned char *)&data, position);
}


shape NewCString(const char *cString, const point *position)
{
	register long len = 0;
	register const char *find0 = cString;

	while (*find0++ != 0)
		++len;
	return NewText(len, (unsigned char *)cString, position);
}


void SetCString(shape source, const char *cString, const point *position)
{
	register long len = 0;
	register const char *find0 = cString;

	while (*find0++ != 0)
		++len;
	SetText(source, len, (unsigned char *)cString, position);
}


fixed FixTextWidth(register const unsigned char *ch, register short len)
{
	point pt;
	shape textShape = NewText(len, ch, nil);
	fixed width = GetShapeAdvance(textShape, &pt)->x;

	DisposeShape(textShape);
	return width;
}


fixed FixCStringWidth(register const char *ch)
{
	register long len = 0;
	register const char *find0 = ch;

	while (*find0++ != 0)
		++len;
	return FixTextWidth((unsigned char *) ch, len);
}


shape NewPString(register const char *pString, register const point *position)
{
	return NewText(*pString, (unsigned char *)pString + 1, position);
}


void SetPString(register shape source, register const char *pString, register const point *position)
{
	SetText(source, *pString, (unsigned char *) pString + 1, position);
}


fixed FixPStringWidth(register const char *ch)
{
	return FixTextWidth((unsigned char *) ch + 1, *ch);
}


fixed FixCharWidth(char ch)
{
	return FixTextWidth((unsigned char *) &ch, 1);
}


void DrawCString(register const char *ch, register const point *position)
{
	register long len = 0;
	register const char *find0 = ch;

	while (*find0++ != 0)
		++len;
	DrawText(len, (unsigned char *)ch, position);
}


void DrawPString(register const char *ch, register const point *position)
{
	DrawText(*ch, (unsigned char *)ch + 1, position);
}


void SetGlyphText(register shape source, register unsigned const char *text, long length)
{
	SetGlyphs(source, length, text, nil, nil, nil, nil, nil);
}


void SetGlyphAdvance(register shape source, register const long advanceBits[])
{
	SetGlyphs(source, 0, nil, nil, advanceBits, nil, nil, nil);
}


void SetGlyphStyles(register shape source, register const short styleRuns[], register const style glyphStyles[])
{
	SetGlyphs(source, 0, nil, nil, nil, nil, styleRuns, glyphStyles);
}


long GetGlyphText(register shape source, register unsigned char *text)
{
	return GetGlyphs(source, nil, text, nil, nil, nil, nil, nil, nil);
}


long GetGlyphAdvance(register shape source, register long advanceBits[])
{
	long length;

	GetGlyphs(source, &length, nil, nil, advanceBits, nil, nil, nil, nil);
	return length;
}


long GetGlyphStyles(register shape source, register short styleRuns[], register style glyphStyles[])
{
	long runs;

	GetGlyphs(source, nil, nil, nil, nil, nil, &runs, styleRuns, glyphStyles);
	return runs;
}

#define kBoldnessFactor 18

static textFace *SetCommonFace(register commonFace face)
{
	register textFace *newFace;
	register long faceLayers = 1;
	register faceLayer *activeLayer;
	transform italicTransform = nil;
	style nullStyle = nil;
	boolean doOutline = (face & outline) != 0;
	boolean doBold = (face & bold) != 0;
	boolean doShadow = (face & shadow) != 0;
	boolean doShadowOutline = (face & shadow+outline) != 0;
	fixed standardBolding = fix1/kBoldnessFactor;
	fixed shadowBolding = ff(3)/(kBoldnessFactor*2);
	fixed shadowTranslate = 2*standardBolding / 3;
	
	if (face == 0)
		return nil;
	
	{
		register long shadowOutlineLayers = doShadowOutline ? 2 : 0;
		register long underlineLayers = (face & underline) ? doShadowOutline + 2 : 0;

		faceLayers = shadowOutlineLayers + underlineLayers + (shadowOutlineLayers == 0);
	}
	if (face & italic) {
		italicTransform = NewTransform();
		
		SkewTransform(italicTransform, -fix1/4, 0, 0, 0);
	}
	if (face & underline) {
		nullStyle = NewStyle();
		SetStyleTextSize(nullStyle, fix1);
		SetStyleFont(nullStyle, 0);
	}
	
	newFace = (textFace *) NewPtr(sizeof(textFace) +  (faceLayers - anyNumber) * sizeof(faceLayer));
	newFace->faceLayers = faceLayers;
	ResetMapping(&newFace->advanceMapping);
	if ( ((face & extend) != 0) ^ ((face & condense) != 0) )
		if (face & extend)
			ScaleMapping(&newFace->advanceMapping, ff(100)/85, fix1, 0, 0);
		else
			ScaleMapping(&newFace->advanceMapping, ff(85)/100, fix1, 0, 0);
	
	activeLayer = &newFace->faceLayer[0];
	
	if (doShadowOutline) {
		if (face & underline) {
			/*** doesn't discriminate between outline and shadow ***/
			activeLayer->outlineFill = noFill;
			activeLayer->flags = underlineAdvanceLayer;
			activeLayer->outlineStyle = CopyToStyle(nil, nullStyle);
			activeLayer->outlineTransform = italicTransform ? CopyToTransform(nil, italicTransform) : NewTransform();
			MoveTransform(activeLayer->outlineTransform, 0, ff(2)/kBoldnessFactor);
			activeLayer->boldOutset.x = activeLayer->boldOutset.y = 0;
			++activeLayer;
			
			activeLayer->outlineFill = noFill;
			activeLayer->flags = underlineAdvanceLayer;
			activeLayer->outlineStyle = CopyToStyle(nil, nullStyle);
			SetStylePen(activeLayer->outlineStyle, fix1/8);
			activeLayer->outlineTransform = italicTransform ? CopyToTransform(nil, italicTransform) : NewTransform();
			MoveTransform(activeLayer->outlineTransform, 0, ff(2+doShadow)/kBoldnessFactor);
			activeLayer->boldOutset.x = activeLayer->boldOutset.y = 0;
			++activeLayer;

			activeLayer->outlineFill = noFill;
			activeLayer->flags = underlineAdvanceLayer + whiteLayer;
			activeLayer->outlineStyle = CopyToStyle(nil, nullStyle);
			SetStylePen(activeLayer->outlineStyle, fix1/16);
			activeLayer->outlineTransform = italicTransform ? CopyToTransform(nil, italicTransform) : NewTransform();
			MoveTransform(activeLayer->outlineTransform, 0, ff(2+doShadow)/kBoldnessFactor);
			activeLayer->boldOutset.x = activeLayer->boldOutset.y = 0;
			++activeLayer;
		}
		
		activeLayer->outlineFill = windingFill;
		activeLayer->flags = 0;
		activeLayer->outlineStyle = nil;
		activeLayer->boldOutset.x = (doShadow ? shadowBolding : standardBolding) + standardBolding*doBold;
		activeLayer->boldOutset.y = doShadow ? shadowBolding : standardBolding;
		if (doShadow) {
			activeLayer->outlineTransform = italicTransform ? CopyToTransform(nil, italicTransform) : NewTransform();
			MoveTransform(activeLayer->outlineTransform, shadowTranslate, shadowTranslate);
		} else
			activeLayer->outlineTransform = italicTransform ? CloneTransform(italicTransform) : nil;
		++activeLayer;
		
		activeLayer->outlineFill = windingFill;
		activeLayer->flags = whiteLayer;
		activeLayer->outlineStyle = nil;
		activeLayer->outlineTransform = italicTransform ? CloneTransform(italicTransform) : nil;
		activeLayer->boldOutset.x = doBold*standardBolding;
		activeLayer->boldOutset.y = 0;	
	} else {
		if (face & underline) {
			activeLayer->outlineFill = noFill;
			activeLayer->flags = underlineAdvanceLayer;
			activeLayer->outlineStyle = CopyToStyle(nil, nullStyle);
			activeLayer->outlineTransform = italicTransform ? CopyToTransform(nil, italicTransform) : NewTransform();
			MoveTransform(activeLayer->outlineTransform, 0, ff(doBold+1)/kBoldnessFactor + shadowTranslate);
			activeLayer->boldOutset.x = activeLayer->boldOutset.y = 0;
			++activeLayer;
			
			activeLayer->outlineFill = noFill;
			activeLayer->flags = underlineAdvanceLayer;
			activeLayer->outlineStyle = CopyToStyle(nil, nullStyle);
			SetStylePen(activeLayer->outlineStyle, fix1/12);	
			activeLayer->outlineTransform = italicTransform ? CopyToTransform(nil, italicTransform) : NewTransform();
			MoveTransform(activeLayer->outlineTransform, 0, ff(doBold+1)/kBoldnessFactor + shadowTranslate);
			activeLayer->boldOutset.x = activeLayer->boldOutset.y = 0;
			++activeLayer;
		} 
	
		activeLayer->outlineFill = windingFill;
		activeLayer->flags = 0;
		activeLayer->outlineStyle = nil;
		activeLayer->outlineTransform = italicTransform ? CloneTransform(italicTransform) : nil;
		activeLayer->boldOutset.x = doBold * standardBolding;
		activeLayer->boldOutset.y = doBold * standardBolding;
	}
	if (activeLayer != &newFace->faceLayer[faceLayers-1])
		DebugStr("\pMiscalculated size of textFace structure");
	
	if (italicTransform)
		DisposeTransform(italicTransform);
	if (nullStyle)
		DisposeStyle(nullStyle);
		
	return newFace;
}


static void DisposeFaceParts(textFace *newFace)
{
	long activeLayer = newFace->faceLayers - 1;

	do {
		if (newFace->faceLayer[activeLayer].outlineStyle)
			DisposeStyle(newFace->faceLayer[activeLayer].outlineStyle);
		if (newFace->faceLayer[activeLayer].outlineTransform)
			DisposeTransform(newFace->faceLayer[activeLayer].outlineTransform);
	} while (--activeLayer >= 0);
	DisposPtr((Ptr) newFace);
}


void SetStyleCommonFace(style source, commonFace face)
{
	textFace *newFace = SetCommonFace(face);

	SetStyleFace(source, newFace);
	if (newFace)
		DisposeFaceParts(newFace);
}


void SetShapeCommonFace(shape source, commonFace face)
{
	textFace *newFace = SetCommonFace(face);

	SetShapeFace(source, newFace);
	if (newFace)
		DisposeFaceParts(newFace);
}


commonFace GetStyleCommonFace(style source)
{
	long layers = GetStyleFace(source, nil);

	if (layers == -1)
		return 0;
	{	textFace *realFacePtr = (textFace *) NewPtr(sizeof(textFace) +  (layers - 1) * sizeof(faceLayer));
		commonFace face = 0;
		long activeLayer = 0;
		style curStyle;
		transform curTransform;

		GetStyleFace(source, realFacePtr);
		if (realFacePtr->faceLayers == 2) {
			face |= shadow;
			activeLayer++;
		}
		if (realFacePtr->faceLayer[activeLayer].outlineFill == closedFrameFill)
			face |= outline;
		if (realFacePtr->faceLayer[activeLayer].flags & underlineAdvanceLayer)
			face |= underline;
		if (curStyle = realFacePtr->faceLayer[activeLayer].outlineStyle)
		if (realFacePtr->faceLayer[activeLayer].boldOutset.x | realFacePtr->faceLayer[activeLayer].boldOutset.y)
			face |= bold;
		if (curTransform = realFacePtr->faceLayer[activeLayer].outlineTransform)
		{	mapping map;

			GetTransformMapping(curTransform, &map);
			if (map.map[0][0] >= IntToFixed(100) / 85)
				face |= extend;
			if (map.map[0][0] <= IntToFixed(85) / 100)
				face |= condense;
			if (map.map[1][0])
				face |= italic;
		}
		{	faceLayer *layer = realFacePtr->faceLayer;

			while (layers--) {
				DisposeStyleAt(&layer->outlineStyle);
				DisposeTransformAt(&layer->outlineTransform);
				layer++;
			}
		}
		return face;
	}
}


commonFace GetShapeCommonFace(shape source)
{
	return GetStyleCommonFace(GetShapeStyle(source));
}

#define offsetField(type, field)		((long) &((type *) 0)->field)

point *GetShapeAdvance(shape source, point *advancePt)
{
	shapeType type = GetShapeType(source);

	switch (type) {
		case emptyType:
		case fullType:
			PostGraphicsWarning(graphic_type_does_not_contain_points);
		break;
		case pointType:
		case lineType:
		case curveType:
		case rectangleType:
		case polygonType:
		case pathType:
		case pictureType:
			{	rectangle bounds;

				GetShapeBounds(source, 0, &bounds);
				advancePt->x = bounds.right - bounds.left;
				advancePt->y = bounds.bottom - bounds.top;
			}
		break;
		case textType:
		case glyphType:
		case layoutType:
			{	long characters;
				point *positions;

				if (type == textType)
					GetText(source, &characters, nil, nil);
				else if (type == glyphType)
					GetGlyphs(source, &characters, nil, nil, nil, nil, nil, nil, nil);
				else characters = GetLayoutGlyphs(source, nil, nil, nil, nil, nil, nil, nil);
				positions = (void *) NewPtr((characters + 1) * sizeof(point));
				GetGlyphMetrics(source, positions, nil, nil);
				advancePt->x = positions[characters].x - positions[0].x;
				advancePt->y = positions[characters].y - positions[0].y;
				DisposPtr((void *) positions);
			}
		break;
	}
	return advancePt;
}