/* graphics libraries:
	operations on transforms	
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

	#include <Memory.h>
#include "graphics libraries.h"
#include "memory routines.h"

viewPort GetTransformViewPort(transform xform)
{
	short portCount;

	if (xform == nil)
		xform = GetShapeTransform(GetDefaultShape(emptyType));
	portCount = GetTransformViewPorts(xform, nil);
	if (portCount == 0)
		return nil;
	else if (portCount <= 10) {	/* use the stack if there's a reasonable number of ports */
		viewPort ports[10];

		GetTransformViewPorts(xform, ports);
		return ports[0];
	} else {				/* else get memory from the heap */
		viewPort *ports;
		viewPort port;

		ports = (viewPort *) NewPtr(portCount * sizeof(viewPort));
		GetTransformViewPorts(xform, ports);
		port = ports[0];
		DisposPtr((Ptr) ports);
		return port;
	}
}

void SetTransformViewPort(transform xForm, viewPort port)
{
	SetTransformViewPorts(xForm, 1, &port);
}


void AddToTransformViewPort(transform xform, viewPort newPort)
{
	viewPort *ports, *portPtr;
	short portCount, count;

	if (xform == nil)
		xform = GetShapeTransform(GetDefaultShape(emptyType));
	count = portCount = GetTransformViewPorts(xform, nil) + 1;
	portPtr = ports = (viewPort *) NewPtr(portCount * sizeof(viewPort));
	GetTransformViewPorts(xform, ports);
	while (--count > 0)					/* do nothing if port order is already in transform */
		if (*portPtr++ == newPort) 
			goto disposeTemp;
	*portPtr = newPort;
	SetTransformViewPorts(xform, portCount, ports);
disposeTemp:
	DisposPtr((void *) ports);
}

void SetShapeViewPort(shape sh, viewPort port)
{
	SetShapeViewPorts(sh, 1, &port);
}

viewPort GetShapeViewPort(shape sh)
{
	return GetTransformViewPort(GetShapeTransform(sh));
}

void SetDeepShapeViewPorts(shape sh, long portCount, const viewPort portList[])
{
	if (GetShapeType(sh) == pictureType) {
		register short count = GetPicture(sh, nil, nil, nil, nil);
		shape itemShape;
		transform itemTransform;

		while( count ) {
			GetPictureParts(sh, count, 1, &itemShape, nil, nil, &itemTransform);
			if (itemTransform)
				SetTransformViewPorts(itemTransform, portCount, portList);
			SetDeepShapeViewPorts(itemShape, portCount, portList);
			--count;
		}
	} 
	SetShapeViewPorts(sh, portCount, portList);
}


void SetDeepShapeTransform(shape sh, transform xform)
{
	if (GetShapeType(sh) == pictureType) {
		register short count = GetPicture(sh, nil, nil, nil, nil);
		shape itemShape;
		transform itemTransform;
		
		while (count) {
			GetPictureParts(sh, count, 1, &itemShape, nil, nil, &itemTransform);
			if (itemTransform) {
				style itemStyle;
				ink itemInk;
				
				GetPictureParts(sh, count, 1, &itemShape, &itemStyle, &itemInk, &itemTransform);
				SetPictureParts(sh, count, 1, 1, &itemShape, &itemStyle, &itemInk, &itemTransform);
			}
			SetDeepShapeTransform(itemShape, xform);
			count--;
		}
	}
	SetShapeTransform(sh, xform);
}

void SetDeepShapeViewPort(shape sh, viewPort newPort)
{
	SetDeepShapeViewPorts(sh, 1, &newPort);
}

void PreMapTransform(transform source, mapping *map)
{
	mapping copy, xform;

	CopyToMapping(&copy, map);
	GetTransformMapping(source, &xform);
	MapMapping(&copy, &xform);
	SetTransformMapping(source, &copy);
}

/*	Set each of the default shapes' transforms to have only the viewPort in their viewPort list.
	Default shapes that shared a transform with each other will still share it; those with unique
	transform will still have unique ones.  Default shapes that shared a transform with a non-
	default shape will do so no longer:  only the default shapes have their transform changed.	*/
void SetDefaultViewPort(viewPort port)
{
	/*	xforms contains the transform for each shape, or nil if the shape shares a transform
		with another default shape.  owners contains the number of default shapes that own
		the transform.  offset contains the index of the shape's transform in the xforms array,
		if the shape share's a transform with an earlier shape.  copied is true for transforms
		that were created in this routine, and hence need to be disposed here.
		
		An index of zero means the default transform.  owners and offsets could be folded into the
		same array, as owners is only used where xforms is non-nil, and offsets where it is nil.
	*/
	transform xforms[pictureType + 1];
	short owners[pictureType + 1];
	short offsets[pictureType + 1];
	boolean copied[pictureType + 1];
	short i, j;

/* the following lines are desirable when debugging with memory flipping enabled */
#ifdef debugging	
	FillBytes(xforms, sizeof(xforms), 0);
	FillBytes(owners, sizeof(owners), 0);
	FillBytes(offsets, sizeof(offsets), 0);
	FillBytes(copied, sizeof(copied), 0);
#endif
	/* collect the transforms into the arrays */
	for (i = emptyType; i <= pictureType; i++) {
		xforms[i] = GetShapeTransform(GetDefaultShape(i));
		owners[i] = 1;
		copied[i] = false;
			
		for (j = emptyType; j < i; j++)
			if (xforms[i] == xforms[j]) {
				xforms[i] = nil;
				offsets[i] = j;
				owners[j]++;
				break;
			}
	}
	
	/* make copies of the transforms, where necessary */
	for (i = emptyType; i <= pictureType; i++) {
		transform xform = xforms[i];
		viewPort oldPort;
		
		if (xform == nil) continue;
		
		/* maybe the transform already has the right port */
		if (GetTransformViewPorts(xform, nil) == 1 && (GetTransformViewPorts(xform, &oldPort), oldPort) == port)
			continue;
		if (GetTransformOwners(xform) != owners[i]) {
			/* some of the owners aren't default shapes */
			xforms[i] = xform = CopyToTransform(nil, xform);
			copied[i] = true;
		}
		SetTransformViewPorts(xform, 1, &port);
	}
	
	IgnoreGraphicsNotice(default_transform_already_set);
	PopGraphicsNotice();
	IgnoreGraphicsNotice(transform_already_set);
	for (i = emptyType; i <= pictureType; i++) {
		transform xform = xforms[i];
		
		SetShapeTransform(GetDefaultShape(i), xform ? xform : xforms[offsets[i]]);
		if (copied[i])
			DisposeTransform(xform);
	}
	PopGraphicsNotice();
#undef defaultType
}

viewGroup CopyViewGroup(viewGroup group)
{
	/* create a new viewGroup */
	viewGroup newGroup = NewViewGroup();

	/* copy the viewDevice list */
	{	long deviceCount = GetViewGroupViewDevices(group, nil), count = deviceCount;
		viewDevice *deviceList = (void *)NewPtr(deviceCount * sizeof(viewDevice)), *list = deviceList;
		GetViewGroupViewDevices(group, deviceList);
		while (count--)
		{	SetViewDeviceViewGroup(*list = CopyToViewDevice(nil, *list), newGroup);
			list++;
		}
		DisposePtr((void *)deviceList);
	}

	/* copy the viewPort list */
	{	long portCount = GetViewGroupViewPorts(group, nil), count = portCount;
		viewPort *oldPortList = (void *)NewPtr(portCount * sizeof(viewPort)), *oldList = oldPortList;
		viewPort *newPortList = (void *)NewPtr(portCount * sizeof(viewPort)), *newList = newPortList;
		GetViewGroupViewPorts(group, oldPortList);
		while (count--)
			SetViewPortViewGroup(*newList++ = CopyToViewPort(nil, *oldList++), newGroup);

	/* copy the viewPort hierarchy */
		oldList = oldPortList; newList = newPortList;
		for (count = portCount; count--; newList++)
		{	viewPort parent = GetViewPortParent(*oldList++);
			if (parent)
			{	viewPort *optr = oldPortList, *nptr = newPortList;
				long pcnt = portCount;
				boolean match = false;
				while (pcnt-- && !(match = *optr == parent)) { optr++; nptr++; }
				if (match) SetViewPortParent(*newList, *nptr);
			}
		}
		DisposePtr((void *)newPortList);
		DisposePtr((void *)oldPortList);
	}
	return newGroup;
}

/* the newGroup should have at least as many viewPorts in it as the oldGroup */
/* ideally, newGroup and oldGroup should be copies of each other */
transform ChangeTransformViewGroup(transform xform, viewGroup oldGroup, viewGroup newGroup)
{
	long oldPortCount = GetViewGroupViewPorts(oldGroup, nil);
	long newPortCount = GetViewGroupViewPorts(newGroup, nil);
	long xformPortCount = GetTransformViewPorts(xform, nil);
	viewPort *oldPortList = (void *)NewPtr(oldPortCount * sizeof(viewPort));
	viewPort *newPortList = (void *)NewPtr(newPortCount * sizeof(viewPort));
	viewPort *xformPortList = (void *)NewPtr(xformPortCount * sizeof(viewPort));
	GetViewGroupViewPorts(oldGroup, oldPortList);
	GetViewGroupViewPorts(newGroup, newPortList);
	GetTransformViewPorts(xform, xformPortList);
	{	long xcnt = xformPortCount;
		viewPort *xptr = xformPortList;
		while (xcnt--)
		{	viewPort order = *xptr, *optr = oldPortList, *nptr = newPortList;
			long pcnt = oldPortCount;
			boolean match = false;
			while (pcnt-- && !(match = *optr == order)) { optr++; nptr++; }
			if (match) *xptr = *nptr;
			xptr++;
		}
	}
	SetTransformViewPorts(xform, xformPortCount, xformPortList);
	DisposePtr((void *)xformPortList);
	DisposePtr((void *)newPortList);
	DisposePtr((void *)oldPortList);
	return xform;
}

shape ChangeShapeViewGroup(shape source, viewGroup oldGroup, viewGroup newGroup)
{
	transform xform = GetShapeTransform(source);
	if (GetTransformOwners(xform) > 1)			/* is it shared? */
	{	xform = CopyToTransform(nil, xform);
		SetShapeTransform(source, xform);
		DisposeTransform(xform);
	}
	ChangeTransformViewGroup(xform, oldGroup, newGroup);
	return source;
}

transform SeparateShapeTransform(shape source)
{
	transform old = CloneTransform(GetShapeTransform(source)), xnew = CopyToTransform(nil, old);
	SetShapeTransform(source, xnew);
	DisposeTransform(xnew);
	return old;
}

void ReuniteShapeTransform(shape target, transform separate)
{
	SetShapeTransform(target, separate);
	DisposeTransform(separate);
	return;
}
