/* ------------------------------------------------------------------------------

	FILENAME
		PrintingDrivers.h

	DESCRIPTION
		This file defines data types and API functions for
		printer driver development.

	COPYRIGHT
		Copyright  Apple Computer, Inc. 1989, 1990, 1991, 1992, 1993
		All rights reserved. 
	
------------------------------------------------------------------------------- */

#ifndef __PRINTINGDRIVERS__
#define __PRINTINGDRIVERS__


#ifndef __PRINTINGMESSAGES__
	#include <PrintingMessages.h>
#endif


/* ------------------------------------------------------------------------------

						Printing Driver Constants and Types

-------------------------------------------------------------------------------- */


enum {
	inputTraysMenuItem = -1			// Menu item number for "Input Trays..."
};

enum {									// bit fields in communicationsOptions field of IOPrefsRec
	useCustomIO	= 0x00000001			// driver uses a non-standard IO mechanism
};

typedef struct {						// buffering and IO preferences
	unsigned long	communicationsOptions;
	unsigned long	numBuffers;
	unsigned long	bufferSize;
	unsigned long	numReqBlocks;
	unsigned long	openCloseTimeout;
	unsigned long	readWriteTimeout;
} IOPrefsRec, *IOPrefsPtr, **IOPrefsHdl;


typedef struct {						// information about writing to a file
	Boolean		printToFile;			// true if output is to go to a file
	FSSpec		fSpec;					// if going to a file, the FSSpec for the file
	Boolean		includeFonts;			// true if fonts are to be included
	Str31			fileFormat;				// format to write file
} PrintDestinationRec, *PrintDestinationPtr, **PrintDestinationHdl;


// this structure is the content of each cell in the standard PACK LDEF

typedef struct {
	char	firstMarker, secondMarker;		// markers to indicate icon or non-icon version
													// if these are  and , then the cell is an icon cell.
													// Otherwise, it is assumed to be a standard text LDEF
													// cell
	
	Handle	iconSuiteHandle;				// the icon suite to draw for this cell
	Handle	outputDriverName;				// handle to the output driver name (for serial)
	Handle	inputDriverName;				// handle to the input driver name (for serial)
	Str255	iconName;						// name to draw under the icon
} PortListRec, *PortListPtr;


/* ------------------------------------------------------------------------------

						Printing Driver Constants for resources in the desktop printer

-------------------------------------------------------------------------------- */

// ---------------------------------- 'comm' ----------------------------------

// The communications method and private data used to connect to the printer
#define deviceCommunicationsType			'comm'
#define deviceCommunicationsID			(0)

// ---------------------------------- 'prod' ----------------------------------

// for PostScript devices, the device and version names of the device.
// (0) product name is of type PString
// (1) version is of type PString
// (2) revision is of type PString
// (3) vm available is of type long

#define postscriptProductInfoType	'prod'
#define postscriptProductNameID		(0)
#define postscriptVersionID			(1)
#define postscriptRevisionID			(2)
#define postscriptVMAvailableID		(3)


/* ------------------------------------------------------------------------------

						Printing Driver Constants for status alerts

-------------------------------------------------------------------------------- */

// the ditl id for the auto feed button in the manual feed alert

enum {									
	autoFeedButtonId	= 3
};


// status resource id for the alerts

enum {									
	univAlertStatusResourceId	= -28508
};


// status resource indices for the alerts

enum {
	univManualFeedIndex 		= 2,
	univFailToPrintIndex 	= 3,
	univPaperJamIndex 		= 4,
	univOutOfPaperIndex 		= 5,
	univNoPaperTrayIndex 	= 6,
	univPrinterReadyIndex 	= 7,
	univAlertBeforeIndex 	= 9,
	univAlertAfterIndex 		= 10
};

// allocation sizes for status buffers needed for automatic alerts


enum {									
	defaultStatusBufferSize		= 10,
	manualFeedStatusBufferSize = 34,
	outOfPaperStatusBufferSize = 42
};


// structures for status buffers

typedef struct {							// structure passed in statusBuffer of StatusRecord for manual feed alert
	Boolean	canAutoFeed;				// true if driver can switch to auto feed
	Str31		paperTypeName;				// name of paperType to feed manually
} ManualFeedRecord;


typedef struct {							// structure passed in statusBuffer of StatusRecord for out of paper alert
	Str31		paperTypeName;				// name of printing document
} OutOfPaperRecord;


/* ------------------------------------------------------------------------------

								Old Application Support

-------------------------------------------------------------------------------- */


// the format of a 'cust' resource 

typedef struct {
	short horizontalResolution;
	short	verticalResolution;
	short upDriverType;
	Point patternStretch;
	short	translatorSettings;
} CustomizationRec, *CustomizationPtr, **CustomizationHdl;
	

// the format of a 'resl' resource

typedef struct {
	short		rangeType;				// always 1
	short		xMinimumResolution;
	short		xMaximumResolution;
	short		yMinimumResolution;
	short		yMaximumResolution;
	short		resolutionCount;
	Point		resolutions[1];
} ResolutionRec, *ResolutionPtr, **ResolutionHdl;


// constants for the "universal" print record...	


enum {									// version number for "universal" print record
	printRecordVersion	= 8
};

enum {									// for calculationMode field...
	emulation				= 0,
	theoretical				= 1
};

enum {									// for feed field...
	autoFeed					= 0,
	manualFeed				= 1
};

enum {									// bit fields for options field...
	preciseBitmap			= 0x0001,	// tall adjusted (IW), precise bitmap (LW, SC)
	biggerPages				= 0x0002,	// no gaps (IW), larger print area (LW)
	graphicSmoothing		= 0x0004,	// graphic smoothing (LW)
	textSmoothing			= 0x0008,	// text smoothing (SC)
	fontSubstitution		= 0x0010,	// font substitution
	invertPage				= 0x0020,	// b/w invert image
	flipPageHoriz			= 0x0040,	// flip horizontal
	flipPageVert			= 0x0080,	// flip vertical
	colorMode				= 0x0100,	// color printing
	bidirectional			= 0x0200,	// bidirectional printing
	userFlag0				= 0x0400,	// user flag 0
	userFlag1				= 0x0800,	// user flag 1
	userFlag2				= 0x1000,	// user flag 2
	reservedFlag0			= 0x2000,	// reserved flag 0
	reservedFlag1			= 0x4000,	// reserved flag 1
	reservedFlag2			= 0x8000		// reserved flag 2
};

enum {									// for orientation field...
	portraitOrientation		= 0,
	landscapeOrientation		= 1,
	altPortraitOrientation	= 2,
	altLandscapeOrientation	= 3
};

enum {									// for qualityMode field...
	bestQuality				= 0,
	fasterQuality			= 1,
	draftQuality			= 2
};

enum {									// for firstTray and remainingTray fields...
	firstTray				= 0,
	secondTray				= 1,
	thirdTray				= 2
};

enum {									// for coverPage field...
	noCoverPage				= 0,
	firstPageCover			= 1,
	lastPageCover			= 2
};

enum {									// for headMotion field...
	unidirectionalMotion	= 0,
	bidirectionalMotion	= 1
};

enum {									// for saveFile field...
	noFile					= 0,
	postScriptFile			= 1
};


// the format of the "universal" print record

typedef struct {
	short				prVersion;      	// print record version 7.5 == 0
	
													// prInfo subrecord...
	
	short				appDev;					// device kind, always 0
	short				appVRes;					// application vertical resolution
	short				appHRes;					// application horizontal resolution
	Rect				appPage;					// page size, in application resolution
	
	Rect 				appPaper;       	 	// paper rectangle [offset from rPage]
	
												// prStl subrecord...
										
	short				devType;					// device type, always 0 (was wDev)
	short				pageV;					// page height in 120ths of an inch
	short				pageH;					// page width in 120ths of an inch
	char				calculationMode;		// page calculation mode
	char				feed;						// feed mode
	
												// prInfoPT subrecord...
										
	short				devKind;					// device kind, always 0
	short				devVRes;					// device vertical resolution
	short				devHRes;					// device horizontal resolution
	Rect				devPage;					// device page size
	
												// prXInfo subrecord...
										
	short				actualCopies;			// actual number of copies for this job
	short				options;					// options for this device
	short				reduction;				// reduce/enlarge factor
	
	char				orientation;			// orientation of paper ( 0=portrait, 1=landscape &c )
	
												// clusters...
										
	char				qualityMode;			// quality mode
	char				firstTray;				// first feed tray
	char				remainingTray;			// remaining feed tray
	char				coverPage;				// cover page
	char				headMotion;				// head motion
	char				saveFile;				// save file
	
	char				userCluster1;		// three clusters left over...
	char				userCluster2;
	char				userCluster3;

												// prJob subrecord...
									
	short				firstPage;				// first page
	short				lastPage;				// last page
	short				copies;					// # of copies, always 1
	char				reserved1;				// always true, unused
	char				reserved2;				// always true, unused
	PrIdleProcPtr	pIdleProc;				// idle proc
	Ptr				pFileName;				// spool file name pointer
	short				fileVol;					// spool file vRefNum
	char				fileVers;				// file version, must be 0
	char				reserved3;				// always 0
	
	short 			printX[19];          // internal use
} UniversalPrintRecord, *UniversalPrintRecordPtr, **UniversalPrintRecordHdl;
		


/* ------------------------------------------------------------------------------

							Compatibility Printing Messages

-------------------------------------------------------------------------------- */


#ifdef __cplusplus
extern "C" {
#endif


OSErr Forward_PrOpenDoc (THPrint, TPPrPort *)
    = FORWARDGLUE;

OSErr Forward_PrCloseDoc (TPPrPort)
    = FORWARDGLUE;

OSErr Forward_PrOpenPage (TPPrPort, TPRect, Point resolution)
    = FORWARDGLUE;

OSErr Forward_PrClosePage (TPPrPort)
    = FORWARDGLUE;

OSErr Forward_PrintDefault (THPrint)
    = FORWARDGLUE;

OSErr Forward_PrStlDialog (THPrint, Boolean *)
    = FORWARDGLUE;

OSErr Forward_PrJobDialog (THPrint, Boolean *)
    = FORWARDGLUE;

OSErr Forward_PrStlInit (THPrint, TPPrDlg *)
    = FORWARDGLUE;

OSErr Forward_PrJobInit (THPrint, TPPrDlg *)
    = FORWARDGLUE;

OSErr Forward_PrDlgMain (THPrint, PDlgInitProcPtr, Boolean *)
    = FORWARDGLUE;

OSErr Forward_PrValidate (THPrint, Boolean *)
    = FORWARDGLUE;

OSErr Forward_PrJobMerge (THPrint, THPrint)
    = FORWARDGLUE;

OSErr Forward_PrGeneral (Ptr)
    = FORWARDGLUE;

OSErr Forward_ConvertPrintRecordTo (THPrint)
    = FORWARDGLUE;

OSErr Forward_ConvertPrintRecordFrom (THPrint)
    = FORWARDGLUE;

OSErr Forward_PrintRecordToJob (THPrint, Job)
    = FORWARDGLUE;


#ifdef __cplusplus
}
#endif



/* ------------------------------------------------------------------------------

						Raster Driver Contants and Types

-------------------------------------------------------------------------------- */



typedef long RasterPlaneOptions;

enum {										// bit fields in RasterPlaneOptions
	defaultOffscreen 		= 0x00000000,	// default value - bits are allocated for the client, halftoning takes place
	dontSetHalftone		= 0x00000001,	// don't call SetViewPortHalftone
	dotTypeIsDitherLevel	= 0x00000002	// call SetViewPortDither using the dotType as the level
};

typedef struct {							// input structure for setting up the offscreen
	RasterPlaneOptions	planeOptions;	// options for the offscreen package
	halftone					planeHalftone;	// OPTIONAL: halftone structure for this plane
	colorSpace				planeSpace;		// OPTIONAL: noSpace will get the graphics default
	colorSet					planeSet;		// OPTIONAL: NIL gets the default
	colorProfile			planeProfile;	// OPTIONAL: NIL gets no matching
} PlaneSetupRec;
	
typedef struct {
	short 				width; 				// Width in pixels
	short 				minHeight; 			// minimum height in pixels - actual height returned here
	short					maxHeight;			// maximum height in pixels
	Fixed					ramPercentage;		// maximum percentage of RAM to take
	long					ramSlop;				// amount of RAM to be sure to leave
	short 				depth; 				// Depths in bits of each plane
	mapping				vpMapping;			// mapping to assign to offscreen viewPorts
	mapping				vdMapping;			// mapping to assign to offscreen viewDevices
	short 				planes;				// # of planes to allocate of depth bits each (can be more than 4)
	PlaneSetupRec		planeSetup[4];		// parameters for each plane, 4 is provided because it is most handy for writers of devices
} OffscreenSetupRec;



typedef struct {							// the format of one plane in the offscreen planar area
	viewPort			theViewPort;			// viewPort for the offscreen
	viewDevice		theDevice;				// viewDevice for the offscreen
	viewGroup		theViewGroup;			// the viewGroup that they share
	shape				theBitmap;				// the offscreen bitmap shape
	bitmap			theBits;					// the bits of the offscreen
} OffscreenPlaneRec;

typedef struct {							// an entire offscreen area
	short					numberOfPlanes;	// number of planes we have
	Handle				offscreenStorage;	// handle containing the bitmaps image data
	OffscreenPlaneRec	thePlanes[1];		// planes to draw in
} OffscreenRec, *OffscreenPtr, **OffscreenHdl;



typedef long RasterRenderOptions;

enum {										// bit fields in RasterRenderOptions
	defaultRaster					= 0x00000000,	// default raster options
	dontResolveTransferModes  	= 0x00000001,	// 0=Resolve, 1=Don't Resolve
	renderInReverse		 		= 0x00000002,	// traverse image in reverse order
	onePlaneAtATime				= 0x00000004,	// render each plane separately
	sendAllBands					= 0x00000008	// send even empty bands
};


typedef struct {							// structure of resource to control raster imaging
	RasterRenderOptions	renderOptions;	// options for the raster imaging system
	Fixed						hImageRes;		// horizontal resolution to image at
	Fixed						vImageRes;		// vertical resolution to image at
	short						minBandSize;	// minimum band size to use (in pixels)
	short						maxBandSize;	// maximum band size to use (in pixels), 0 == entire page
	Fixed						ramPercentage;	// maximum percentage of RAM to take
	long						ramSlop;			// amount of RAM to be sure to leave
	short						depth;			// depth in pixels (PER PLANE!)
	short						numPlanes;		// number of planes to render
	PlaneSetupRec			planeSetup[1];	// one for each plane
} RasterPrefsRec, *RasterPrefsPtr, **RasterPrefsHdl;


typedef long RasterPackageOptions;

enum {										// bit fields in RasterPackageOptions
	sendAllColors	= 0x00000001,								// send even clean bands through
	interlaceColor	= 0x00000002,								// ribbon contamination is a concern
	overlayColor	= 0x00000004,								// color printer without a ribbon problem
	useColor			=	(interlaceColor|overlayColor)		// this is a color printer
};


typedef struct {							// resource to control the RasterDataIn message
	Ptr						bufferSize;			// buffer size for packaging (>= maximum head pass size)
	short						colorPasses;		// 1 (b/w) or 4 (CMYK) is typical
	short 					headHeight;			// printhead height in pixels
	short 					numberPasses;		// number of head passes it takes to == iHeadHeight
	short						passOffset;			// offset between passes, in pixels
	RasterPackageOptions	packageOptions;	// packaging options
} RasterPackageRec, *RasterPackagePtr, **RasterPackageHdl;


typedef struct {											// for RasterPackageBitmap message
	bitmap							*bitmapToPackage;		// bitmap containing the data to package
	unsigned short					startRaster;			// raster to begin the packaging from
	unsigned short					colorBand;				// for which color pass this is a packaging request
	Boolean							isBandDirty;			// whether there are any dirty bits in this band
	Rect								dirtyRect;				// which bits are dirty
} RasterPackageBitmapRec;


typedef struct	{							// for outputing numbers
	short	numberType;							// type of numberic output desired
	short minWidth;							// minimum output width of the number
	char	padChar;								// pad character for numbers shorter than the minWidth
	char	alignment;							// whee!
	Str31	startString;						// prefix string
	Str31	endString;							// postfix string
} StandardNumberRec, *StandardNumberPtr;



typedef struct {							// structure of resource to control other messages
	short					startPageStringID;	// 'wstr' to send to the device at start of page
	short					formFeedStringID;		// 'wstr' to send to the device to cause a form feed
	short					forwardMax;				// line feed strings
	StandardNumberRec	forwardLineFeed;
	short					reverseMax;
	StandardNumberRec	reverseLineFeed;
} RasterPackageControlsRec, *RasterPackageControlsPtr, **RasterPackageControlsHdl;



typedef struct {							// raster imaging system imageData structure

	// setup values

	RasterRenderOptions			renderOptions;		// options for the raster imaging system
	Fixed								hImageRes;			// horizontal resolution to image at
	Fixed								vImageRes;			// vertical resolution to image at
	short								minBandSize;		// smallest band that makes sense for this device
	short								maxBandSize;		// biggest band that makes sense, or 0 for "full page"
	rectangle						pageSize;			// size of page for device
	
	// values used within the RasterDataIn message
	
	short								currentYPos;		// current position moving down the page
	RasterPackageRec				packagingInfo;		// raster packaging record
	
	// values used within the remaining messages
	
	Boolean							optionsValid;		// were options specified by the driver?
	RasterPackageControlsRec	packageControls;	// options for the packaging messages

	OffscreenSetupRec				theSetup;			// setup for the offscreen code, variable length componant
} RasterImageDataRec, *RasterImageDataPtr, **RasterImageDataHdl;



/* ------------------------------------------------------------------------------

								Raster Driver Imaging Messages

-------------------------------------------------------------------------------- */


#ifdef __cplusplus
extern "C" {
#endif


#define RASTERMSGGLUE(selector) SendMessageGlue(0x0001, selector)
#define FORWARDGLUE ForwardThisMessageGlue


OSErr Send_RasterDataIn (OffscreenHdl, rectangle *bandRectangle, rectangle *dirtyRectangle)
    = RASTERMSGGLUE(rasterDataIn);
OSErr Forward_RasterDataIn (OffscreenHdl, rectangle *bandRectangle, rectangle *dirtyRectangle)
    = FORWARDGLUE;
	 	
OSErr Send_RasterLineFeed (short *lineFeedSize, Ptr buffer, unsigned long *bufferPos, RasterImageDataHdl)
    = RASTERMSGGLUE(rasterLineFeed);
OSErr Forward_RasterLineFeed (short *lineFeedSize, Ptr buffer, unsigned long *bufferPos, RasterImageDataHdl)
    = FORWARDGLUE;
	
OSErr Send_RasterPackageBitmap (RasterPackageBitmapRec *whatToPackage, Ptr buffer,
										  unsigned long *bufferPos, RasterImageDataHdl)
    = RASTERMSGGLUE(rasterPackageBitmap);
OSErr Forward_RasterPackageBitmap (RasterPackageBitmapRec *whatToPackage, Ptr buffer,
										  	  unsigned long *bufferPos, RasterImageDataHdl)
    = FORWARDGLUE;


#ifdef __cplusplus
}
#endif



/* ------------------------------------------------------------------------------

						Vector Driver Contants and Types

-------------------------------------------------------------------------------- */



typedef long VectorRenderOptions;

enum {								// bit fields in VectorRenderOptions
	colorSort		= 0x00000001,	// set for pen plotters. 
	aTransferMode	= 0x00000002,	// set if transfer modes need to be resolved 
	noOverlap		= 0x00000004,	// set if non-overlapping output is desired
	aColorBitmap	= 0x00000008,	// set if color bitmap output is desired
	sortbyPenPos	= 0x00000010,	// set if shapes are to be drawn in the order of the pen index
 											// in the pen table. NOTE: this is not the pen position in the carousel.
	penLessPlotter	= 0x00000020,	// indicates raster printer/plotter
	cutterPlotter	= 0x00000040,	// indicates cutter
	noBackGround	= 0x00000080	// set if shapes that map to the background color should not be sent to driver 
};

			
typedef long VectorShapeOptions;

enum {										// bit fields in VectorShapeOptions
	unidirectionalFill		= 0x00000001,	// generate scanlines in one direction only.. useful for transparencies
	alsoOutlineFilledShape	= 0x00000002	// turn on this bit to also outline solid filled shapes 
};


typedef struct {
	fixed		angle;				// angle to halftone at. Must be 0, 90, 45 or 135
	long		penIndex;			// index of the pen to draw this component with
} VHalftoneCompRec;
  
  
typedef struct {
	colorSpace			halftoneSpace;
	VHalftoneCompRec	halftoneComps[4]; // info for each color component
	long					penIndexForBW; 	// pen index to draw one bit deep or black and white bitmap with
} VHalftoneRec;


typedef struct {
	VectorShapeOptions	shapeOptions;  	// options to control shape handling
	long						maxPolyPoints; 	// maximum number of polygon points that device can support
	fixed						shapeError;    	// defines allowed deviation from the original shape
	fixed						textSize;	   	// text above this size is filled; text below this size is outlined
	fixed						frameSize;    		// frame's smaller than this -> shape stroked; frame's larger -> shape is filled
} VectorShapeDataRec;


typedef struct {								// vector imaging system imageData structure
	VectorRenderOptions	renderOptions;		// options to control rendering: color sort, clipping, etc.
	fixed						devRes;				// device resolution
	transform    			devTransform;		// mapping, clip and halftoning information for colored bitmaps
	colorSet					clrSet;				// entire set of colors; usually indexed color space for pen plotters
	color				   	bgColor;				// the background color in the color space specified by the clrSpace field
	VHalftoneRec			halftoneInfo;		// defines halftone information for color bitmaps
	PenTableHdl				hPenTable;			// complete list of pens along with their pen positions and thickness
	rectangle				pageRect;			// page dimensions
	VectorShapeDataRec	shapeData;			// information on how to render a shape
} VectorImageDataRec, *VectorImageDataPtr, **VectorImageDataHdl;



/* ------------------------------------------------------------------------------

								Vector Driver Imaging Messages

-------------------------------------------------------------------------------- */


#ifdef __cplusplus
extern "C" {
#endif


#define VECTORMSGGLUE(selector) SendMessageGlue(0x0001, selector)
#define FORWARDGLUE ForwardThisMessageGlue


OSErr Send_VectorPackageShape (shape, long penIndex)
	= VECTORMSGGLUE(vectorPackageData);
OSErr Forward_VectorPackageShape (shape, long penIndex)
	= FORWARDGLUE;
	
OSErr Send_VectorLoadPens (PenTableHdl, long *shapeCounts, boolean *penTableChanged)
	= VECTORMSGGLUE(vectorLoadPens);
OSErr Forward_VectorLoadPens (PenTableHdl, long *shapeCounts, boolean *penTableChanged)
	= FORWARDGLUE;

OSErr Send_VectorVectorizeShape (shape, long penIndex, VectorShapeDataRec *)
	= VECTORMSGGLUE(vectorVectorizeShape);
OSErr Forward_VectorVectorizeShape (shape, long penIndex, VectorShapeDataRec *)
	= FORWARDGLUE;


#ifdef __cplusplus
}
#endif



/* ------------------------------------------------------------------------------

							PostScript Driver Contants and Types

-------------------------------------------------------------------------------- */



enum { postSynonym = (long)'post' };


typedef long PostScriptRenderOptions;

enum { 											// bit fields in PostScriptRenderOptions
	needsHexOption					= 0x00000001,	// convert all binary data to hex
	needsCommentsOption			= 0x00000002,	// issue PostScript comments
	boundingBoxesOption			= 0x00000004,	// calculate the values for %%BoundingBox: and %%PageBoundingBox: -- requires needsCommentsOption
	portablePostScriptOption	= 0x00000008,	// generate portable PostScript
	textClipsToPathOption		= 0x00000010,	// convert all clips that are composed of text to path shapes
	flattenClipPathOption		= 0x00000020,	// convert all clips that are path shapes to polygons (helps better control point limit)
	useCharpath1Option			= 0x00000040,	// (ignored if text clips are converted to paths)  When the clip is text, 
															// do it one glyph at a time, redrawing the main shape each time
	useLevel2ColorOption			= 0x00000080	// When printing to level-2 use level-2 device independent color
};



typedef struct {

	font						theFont;							// ---> font reference.
	long						nGlyphs;							// ---> Number of glyphs in the font.

	fontPlatforms			platform;						// <--- How printer font is encoded.
	fontScript				script;							// <--- Script if platform != glyphPlatform
	fontLanguage			language;						// <--- Language if platform != glyphPlatform.
	
	long						vmUsage;							// <--- how much PostScript VM font uses.

	/** size of this is longAlligned(nGlyphs) **/
	unsigned long		glyphBits[1];						// <--- Bit array of which system glyphs are in printer.

} PrinterGlyphsRec;

typedef struct {								// PostScript imaging system imageData structure
	short							languageLevel;	// PostScript language level
	colorSpace					devCSpace;		// the printer's color space
	colorProfile				devCProfile;	// The printer's color profile for matching
	PostScriptRenderOptions	renderOptions;	// options for the imaging system
	short							pathLimit;		// maximum path size
	short							gsaveLimit;		// maximum number of gsaves allowed
	short							opStackLimit;	// operand stack limit
	streamType					fontType;		// these are the font types that the printer supports 
	long							printerVM;		// how much memory is in the printer
	long							reserved0;
} PostScriptImageDataRec, *PostScriptImageDataPtr, **PostScriptImageDataHdl;


typedef struct	{								// for GetProcSetList / DownLoadProcSetList messages
	Signature	clientid;
	OSType		controlType;						// the driver will call FetchTaggedData on each of these resources
	short			controlid;
	OSType		dataType;
	long			reserved0;
} ProcSetListRec, *ProcSetListPtr, **ProcSetListHdl;




enum { 							// possible results of querying printer (returned by PostScriptQueryPrinter message)
	printerOK			= 0,
	intializePrinter	= 1, 
	filePrinting		= 2,
	resetPrinter 		= 128
};



/* ------------------------------------------------------------------------------

								PostScript Driver Imaging Messages

-------------------------------------------------------------------------------- */


#ifdef __cplusplus
extern "C" {
#endif


#define POSTSCRIPTMSGGLUE(selector) SendMessageGlue(0x0001, selector)
#define FORWARDGLUE ForwardThisMessageGlue


// device control messages


OSErr Send_PostScriptQueryPrinter (long *)
	= POSTSCRIPTMSGGLUE(postscriptQueryPrinter);
OSErr Forward_PostScriptQueryPrinter (long *)
	= FORWARDGLUE;

OSErr Send_PostScriptInitializePrinter (void)
	= POSTSCRIPTMSGGLUE(postscriptInitializePrinter);
OSErr Forward_PostScriptInitializePrinter (void)
	= FORWARDGLUE;

OSErr Send_PostScriptResetPrinter (void)
	= POSTSCRIPTMSGGLUE(postscriptResetPrinter);
OSErr Forward_PostScriptResetPrinter (void)
	= FORWARDGLUE;

OSErr Send_PostScriptExitServer (void)
	= POSTSCRIPTMSGGLUE(postscriptExitServer);
OSErr Forward_PostScriptExitServer (void)
	= FORWARDGLUE;
	
	
// device communication messages


OSErr Send_PostScriptGetStatusText (Handle)
	= POSTSCRIPTMSGGLUE(postscriptGetStatusText);
OSErr Forward_PostScriptGetStatusText (Handle)
	= FORWARDGLUE;

OSErr Send_PostScriptGetPrinterText (Handle)
	= POSTSCRIPTMSGGLUE(postscriptGetPrinterText);
OSErr Forward_PostScriptGetPrinterText (Handle)
	= FORWARDGLUE;

OSErr Send_PostScriptScanStatusText (Handle)
	= POSTSCRIPTMSGGLUE(postscriptScanStatusText);
OSErr Forward_PostScriptScanStatusText (Handle)
	= FORWARDGLUE;

OSErr Send_PostScriptScanPrinterText (Handle)
	= POSTSCRIPTMSGGLUE(postscriptScanPrinterText);
OSErr Forward_PostScriptScanPrinterText (Handle)
	= FORWARDGLUE;


// proc set management messages


OSErr Send_PostScriptGetDocumentProcSetList (ProcSetListHdl, PostScriptImageDataHdl)
	= POSTSCRIPTMSGGLUE(postscriptGetDocumentProcSetList);
OSErr Forward_PostScriptGetDocumentProcSetList (ProcSetListHdl, PostScriptImageDataHdl)
	= FORWARDGLUE;

OSErr Send_PostScriptDownloadProcSetList (ProcSetListHdl, PostScriptImageDataHdl)
	= POSTSCRIPTMSGGLUE(postscriptDownloadProcSetList);
OSErr Forward_PostScriptDownloadProcSetList (ProcSetListHdl, PostScriptImageDataHdl)
	= FORWARDGLUE;
	
	
// font management messages


OSErr Send_PostScriptGetPrinterGlyphsInformation (PrinterGlyphsRec *)
	= POSTSCRIPTMSGGLUE(postscriptGetPrinterGlyphsInformation);
OSErr Forward_PostScriptGetPrinterGlyphsInformation (PrinterGlyphsRec *)
	= FORWARDGLUE;

OSErr Send_PostScriptStreamFont (font fontref, scalerStream* streamPtr)
	= POSTSCRIPTMSGGLUE(postscriptStreamFont);
OSErr Forward_PostScriptPostScriptStreamFont (font fontref, scalerStream* streamPtr)
	= FORWARDGLUE;


// document structuring and formatting messages


OSErr Send_PostScriptDoDocumentHeader (PostScriptImageDataHdl)
	= POSTSCRIPTMSGGLUE(postscriptDoDocumentHeader);
OSErr Forward_PostScriptDoDocumentHeader (PostScriptImageDataHdl)
	= FORWARDGLUE;
	
OSErr Send_PostScriptDoDocumentSetup (PostScriptImageDataHdl)
	= POSTSCRIPTMSGGLUE(postscriptDoDocumentSetUp);
OSErr Forward_PostScriptDoDocumentSetup (PostScriptImageDataHdl)
	= FORWARDGLUE;

OSErr Send_PostScriptDoDocumentTrailer (PostScriptImageDataHdl)
	= POSTSCRIPTMSGGLUE(postscriptDoDocumentTrailer);
OSErr Forward_PostScriptDoDocumentTrailer (PostScriptImageDataHdl)
	= FORWARDGLUE;
	
	
// page structuring and formatting messages


OSErr Send_PostScriptDoPageSetup (Format, long, PostScriptImageDataHdl)
	= POSTSCRIPTMSGGLUE(postscriptDoPageSetUp);
OSErr Forward_PostScriptDoPageSetup (Format, long, PostScriptImageDataHdl)
	= FORWARDGLUE;

OSErr Send_PostScriptSelectPaperType (PaperType, long, PostScriptImageDataHdl)
	= POSTSCRIPTMSGGLUE(postscriptSelectPaperType);
OSErr Forward_PostScriptSelectPaperType (PaperType, long, PostScriptImageDataHdl)
	= FORWARDGLUE;

OSErr Send_PostScriptDoPageTrailer (PostScriptImageDataHdl)
	= POSTSCRIPTMSGGLUE(postscriptDoPageTrailer);
OSErr Forward_PostScriptDoPageTrailer (PostScriptImageDataHdl)
	= FORWARDGLUE;

OSErr Send_PostScriptEjectPage (PaperType, long pagenumber, long copiescount, short erasepage, PostScriptImageDataHdl)
	= POSTSCRIPTMSGGLUE(postscriptEjectPage);
OSErr Forward_PostScriptEjectPage (PaperType, long pagenumber, long copiescount, short erasepage, PostScriptImageDataHdl)
	= FORWARDGLUE;


// shape imaging messages


OSErr Send_PostScriptProcessShape (shape page,long trcount, transform trlist[])
	= POSTSCRIPTMSGGLUE(postscriptProcessShape);
OSErr Forward_PostScriptProcessShape (shape page,long trcount, transform trlist[])
	= FORWARDGLUE;


#ifdef __cplusplus
}
#endif



/* ------------------------------------------------------------------------------

											Driver API Functions

-------------------------------------------------------------------------------- */


#define DRIVERAPIGLUE(selector) {0x203C, 0x0002, selector, 0xABFE}


#ifdef __cplusplus
extern "C" {
#endif


OSErr AddPrinterViewDevice (Printer, viewDevice)
	= DRIVERAPIGLUE(0);

OSErr GetAvailableJobFormatModes (JobFormatModeTableHdl *)
	= DRIVERAPIGLUE(1);

OSErr SetPreferredJobFormatMode (JobFormatMode, Boolean directOnly)
	= DRIVERAPIGLUE(2);

OSErr CoolAlert (short iconId,
					  short txtSize,
					  short defaultTitleNum,
					  short cancelTitleNum,
					  short textLength,
					  Ptr pAlertMsg,
					  StringPtr actionTitle,
					  StringPtr title2,
					  StringPtr title3,
					  StringPtr msgFont,
					  ModalFilterProcPtr filterProc,
					  short *itemHit)
	= DRIVERAPIGLUE(3);

OSErr GetCoolAlert (short coolResId, ModalFilterProcPtr, short *itemHit) 
	= DRIVERAPIGLUE(4);


OSErr FetchDTPData (Str31 dtpName, ResType theType, short theID, Handle *theData)
	= DRIVERAPIGLUE(5);

OSErr WriteDTPData (Str31 dtpName, ResType theType, short theID, Handle theData)
	= DRIVERAPIGLUE(6);

OSErr HandleChooserMessage (Job *aJob, Str31 driverName, short message, short caller, 
									 StringPtr objName, StringPtr zoneName, ListHandle theList, long p2)
	= DRIVERAPIGLUE(7);


#ifdef __cplusplus
}
#endif


#endif __PRINTINGDRIVERS__