/* ------------------------------------------------------------------------------
FILENAME
	PrintingErrors.h

DESCRIPTION
		This file defines all the public result codes for the Printing Manager.
	
COPYRIGHT
	  Apple Computer, Inc. 1990-1991
	 All rights reserved. 
	
-------------------------------------------------------------------------------- */

#ifndef __PRINTINGERRORS__
#define __PRINTINGERRORS__

#define printingResultBase		-510

enum {

//  RESULT CODES FOR I/O

    aioTimeout			 = printingResultBase,				// -510 : Timeout condition occurred during op.
    aioBadRqstState		 = (printingResultBase - 1),		// -511 : Async I/O request in invalid state for op.
    aioBadConn			 = (printingResultBase - 2),		// -512 : Invalid AIO connection refnum
    aioInvalidXfer		 = (printingResultBase - 3),		// -513 : Read DataXferStruc contained bad values
    aioNoRqstBlks		 = (printingResultBase - 4),		// -514 : No available request blocks to process request
    aioNoDataXfer		 = (printingResultBase - 5),		// -515 : DataXferStruc pointer not specified
    aioTooManyAutos		 = (printingResultBase - 6),		// -516 : Auto status request already active
    aioNoAutoStat		 = (printingResultBase - 7),		// -517 : Connection not configured for auto status
    aioBadRqstID		 = (printingResultBase - 8),		// -518 : Invalid I/O request identifier
    aioCantKill			 = (printingResultBase - 9),		// -519 : Comm. protocol doesn't support I/O term.
    aioAlreadyExists	 = (printingResultBase - 10),		// -520 : Protocol spec. data already specified
    aioCantFind			 = (printingResultBase - 11),		// -521 : Protocol spec. data does not exist
    aioDeviceDisconn	 = (printingResultBase - 12),		// -522 : Machine disconnected from printer
    aioNotImplemented	 = (printingResultBase - 13),		// -523 : Function not implemented
    aioOpenPending		 = (printingResultBase - 14),		// -524 : Opening a connection for protocol, but another open pending
    aioNoProtocolData	 = (printingResultBase - 15),		// -525 : No protocol specific data specified in request
    aioRqstKilled		 = (printingResultBase - 16),		// -526 : I/O request was terminated
    badBaudRate			 = (printingResultBase - 17),		// -527 : Invalid baud rate specified
    badParity			 = (printingResultBase - 18),		// -528 : Invalid parity specified
    badStopBits			 = (printingResultBase - 19),		// -529 : Invalid stop bits specified
    badDataBits			 = (printingResultBase - 20),		// -530 : Invalid data bits specified
    badPrinterName		 = (printingResultBase - 21),		// -531 : Bad printer name specified
    aioBadMsgType		 = (printingResultBase - 22),		// -532 : Bad masType field in xferInfo struc
    aioCantFindDevice	 = (printingResultBase - 23),		// -533 : Cannot locate target device
    aioOutOfSeq			 = (printingResultBase - 24),		// -534 : Non-atomic SCSI requests submitted out of sequence

	prIOAbortErr		 = (printingResultBase - 25),		// -535
    prUserAbortErr		 = (printingResultBase - 26),		// -536


//  RESULT CODES FOR THE DIALOGS

    cantAddPanelsNowErr		 = (printingResultBase - 27),	// -537 : panels can only be added during driver switch or before dialog is initiated
    badxdtlKeyErr			 = (printingResultBase - 28),	// -538 : unknown key for xdtl - must be radiobutton, etc
    xdtlItemOutOfRangeErr	 = (printingResultBase - 29),	// -539 : referenced item does not belong to panel
    noCoolActionButtonErr 	 = (printingResultBase - 30),	// -540 : action button is nil
    coolTitlesTooLongErr	 = (printingResultBase - 31),	// -541 : length of buttons exceeds alert maximum width
    unknownCoolVersionErr 	 = (printingResultBase - 32),	// -542 : bad version for cool alerts


// RESULT CODES FOR BUFFERING

    gbBufferTooSmallErr		= (printingResultBase - 33),	// -543


// RESULT CODES FOR VECTOR IMAGING

    invalidPenTable		 = (printingResultBase - 34),		// -544


// RESULT CODES FOR PRINT FILES

    incompletePrintFileErr	 = (printingResultBase - 35),	// -545
    crashedPrintFileErr		 = (printingResultBase - 36),	// -546
    invalidPrintFileVersion	 = (printingResultBase - 37),	// -547


// MISCELLANEOUS RESULT CODES FROM THE API

	segmentLoadFailedErr	= (printingResultBase - 38),	// -548 : keep in sync with Dispatcher.a in the core
	extensionNotFoundErr	= (printingResultBase - 39),	// -549
    flattenVersionTooNew  	= (printingResultBase - 40),	// -550
    paperTypeNotFound 		= (printingResultBase - 41),	// -551
    noSuchPTGroup			= (printingResultBase - 42),	// -552
	

// TO BE REMOVED...

    duplicatePanelNameErr	 = (printingResultBase - 43)	// -553 : panel name is not unique on add
	
};


#endif __PRINTINGERRORS__
