/* ------------------------------------------------------------------------------

FILENAME
	PrintingResTypes.h

DESCRIPTION
	This module contains resource constants which can be
	included by the C compiler or the Rez compiler.  These constants
	are used by creators of Printer Drivers or Printing Extensions.
		
COPYRIGHT
	 Copyright Apple Computer, Inc. 1988-1993
	 All rights reserved. 
	
-------------------------------------------------------------------------------- */


#ifndef __PRINTINGRESTYPES__
#define __PRINTINGRESTYPES__


// -------------------------------------------------------------------------------
// Basic client types
// -------------------------------------------------------------------------------


#define printingManagerType		'pmgr'
#define printerDriverType		'pdvr'
#define printingExtensionType	'pext'
#define	anyPrinterType			'univ'
#define	unknownPrinterType		'none'
#define	quickdrawPrinterType	'qdrw'
#define	rasterPrinterType		'rast'
#define	postscriptPrinterType	'post'
#define	vectorPrinterType		'vect'


// -------------------------------------------------------------------------------
// Printing collection tags have the following ID
// -------------------------------------------------------------------------------


#define printingTagID (-28672)


// -------------------------------------------------------------------------------
// Resource types and IDs used by both extension and driver writers
// -------------------------------------------------------------------------------


// Resources in a printer driver or extension must be based off of this ID

#define printingDriverBaseID (-27648)
#define printingExtensionBaseID (-27136)

// override resources tell the system what messages a driver or extension
// is overriding.  A driver may have a series of these resources.

#define	overrideType			'over'
	

// -------------------------------------------------------------------------------
// Message ID definitions by both extension and driver writers
// -------------------------------------------------------------------------------


// identifiers for universal message overrides...


#define initialize				0
#define shutDown				1

#define jobIdle					2
#define jobStatus				3
#define printingEvent			4

#define jobDefaultFormatDialog	5
#define formatDialog			6
#define jobPrintDialog			7
#define filterPanelEvent		8
#define handlePanelEvent		9
#define parsePageRange			10

#define defaultJob				11
#define defaultFormat			12
#define defaultPaperType		13
#define defaultPrinter			14

#define createSpoolFile			15
#define spoolPage				16
#define spoolData				17
#define spoolResource			18
#define completeSpoolFile		19

#define countPages				20
#define despoolPage				21
#define despoolData				22
#define despoolResource			23
#define closeSpoolFile			24

#define startJob				25
#define finishJob				26
#define startPage				27
#define finishPage				28
#define printPage				29

#define setupImageData			30
#define imageJob				31
#define imageDocument			32
#define imagePage				33
#define renderPage				34
#define createImageFile			35

#define openConnection			36
#define closeConnection			37
#define startSendPage			38
#define finishSendPage			39

#define writeData				40
#define bufferData				41
#define dumpBuffer				42
#define freeBuffer				43

#define checkStatus				44
#define getDeviceStatus			45

#define fetchTaggedData			46

#define getDTPMenuList			47
#define dtpMenuSelect			48
#define dtpMenuEventFilter		49

#define jobFormatModeQuery		50

#define writeStatusToDTPWindow	51
#define initializeStatusAlert	52
#define handleAlertStatus		53
#define handleAlertEvent		54

#define cleanupStartJob			55
#define cleanupStartPage		56
#define cleanupOpenConnection	57
#define cleanupStartSendPage	58

#define defaultDesktopPrinter	59
#define captureOutputDevice		60

#define openConnectionRetry		61
#define examineSpoolFile		62

#define finishSendPlane			63
#define doesPaperFit			64
#define chooserMessage			65


// identifiers for Quickdraw message overrides...


#define	prOpenDoc 	 				0
#define	prCloseDoc 	 				1
#define	prOpenPage 	 				2
#define	prClosePage  				3
#define	printDefault 				4
#define	prStlDialog  				5
#define	prJobDialog  				6
#define	prStlInit 					7
#define	prJobInit 	 				8
#define	prDlgMain 	 				9
#define	prValidate 	 				10
#define	prJobMerge 	 				11
#define	prGeneral 	 				12
#define	convertPrintRecordTo 		13
#define	convertPrintRecordFrom 		14
#define	printRecordToJob 	 		15


// identifiers for raster imaging message overrides...


#define rasterDataIn			0
#define rasterLineFeed			1
#define rasterPackageBitmap		2


// identifiers for PostScript imaging message overrides...


#define 	postscriptQueryPrinter					0
#define 	postscriptInitializePrinter				1
#define 	postscriptResetPrinter					2
#define 	postscriptExitServer					3
#define 	postscriptGetStatusText					4
#define 	postscriptGetPrinterText				5
#define		postscriptScanStatusText				6
#define		postscriptScanPrinterText				7
#define		postscriptGetDocumentProcSetList		8
#define 	postscriptDownloadProcSetList			9
#define 	postscriptGetPrinterGlyphsInformation	10
#define		postscriptStreamFont					11
#define 	postscriptDoDocumentHeader				12
#define 	postscriptDoDocumentSetUp				13
#define 	postscriptDoDocumentTrailer				14
#define 	postscriptDoPageSetUp					15
#define 	postscriptSelectPaperType				16
#define 	postscriptDoPageTrailer					17
#define		postscriptEjectPage						18
#define 	postscriptProcessShape					19


// identifiers for Vector imaging message overrides...


#define vectorPackageData		0
#define vectorLoadPens			1
#define vectorVectorizeShape	2


// -------------------------------------------------------------------------------
// Resource types and IDs used by extension writers
// -------------------------------------------------------------------------------


#define	extensionUniversalOverrideID		printingExtensionBaseID

#define extensionImagingOverrideSelectorID	printingExtensionBaseID

#define	extensionScopeType			'scop'
	#define imagingSystemScopeID		printingExtensionBaseID
	#define driverScopeID				printingExtensionBaseID + 1
	#define exceptionScopeID			printingExtensionBaseID + 2

#define	extensionLoadType			'load'
	#define extensionLoadID				printingExtensionBaseID
	
	#define extensionLoadFirst			0x00000100
	#define extensionLoadAnywhere		0x7FFFFFFF
	#define extensionLoadLast			0xFFFFFF00

#define	extensionOptimizationType	'eopt'
	#define extensionOptimizationID		printingExtensionBaseID


// -------------------------------------------------------------------------------
// Resource types and IDs used by driver writers
// -------------------------------------------------------------------------------


#define	driverUniversalOverrideID		(printingDriverBaseID)
#define	driverImagingOverrideID			(printingDriverBaseID + 1)
#define	driverCompatibilityOverrideID	(printingDriverBaseID + 2)


#define driverFileFormatType			'pfil'
#define driverFileFormatID				(printingDriverBaseID)


// IMAGING RESOURCES

// The imaging system resource specifies which imaging system a printer
// driver wishes to use.

#define	imagingSystemSelectorType		'isys'
#define imagingSystemSelectorID			(printingDriverBaseID)


// Resource for type for color matching

#define colorMatchingDataType			'cmat'
#define colorMatchingDataID				(printingDriverBaseID)


// Resource type and id for the tray count

#define trayCountDataType				'tray'
#define trayCountDataID					(printingDriverBaseID)


// Resource type for the tray names

#define trayNameDataType				'tryn'


// IO Resources

// Resource type and ID for default IO and buffering resources

#define universalIOPrefsType			'iobm'
#define universalIOPrefsID				(printingDriverBaseID)


// Resource types and IDs for default implementation of CaptureOutputDevice.
// The default implementation of CaptureOutputDevice only handles PAP devices

#define captureType							'cpts'
#define captureStringID						(printingDriverBaseID)
#define releaseStringID						(printingDriverBaseID + 1)
#define uncapturedAppleTalkType				(printingDriverBaseID + 2)
#define capturedAppleTalkType				(printingDriverBaseID + 3)


// Resource type and ID for raster driver rendering preferences

#define rasterPrefsType					'rdip'
#define rasterPrefsID					(printingDriverBaseID)


// Resource type for specifiying a colorSet

#define colorSetResType	'cset'


// Resource type and ID for raster driver packaging preferences

#define rasterPackType					'rpck'
#define rasterPackID					(printingDriverBaseID)


// Resource type and ID for raster driver packaging options

#define rasterNumNone		0	// number isn't output at all
#define rasterNumDirect		1	// lowest minWidth bytes as data
#define rasterNumToASCII	2	// minWidth ASCII characters

#define rasterPackOptionsType			'ropt'
#define rasterPackOptionsID				(printingDriverBaseID)


// Resource type for the PostScript imaging system procedure set control resource

#define postscriptProcSetControlType	'prec'


// Resource type for the PostScript imaging system printer font resource

#define postscriptPrinterFontType 		'pfnt'


// Resource type and id for the PostScript imaging system imaging preferences

#define postscriptPrefsType				'pdip'
#define postscriptPrefsID				(printingDriverBaseID)

// Resource type and id for the PostScript imaging system default scanning code

#define postscriptScanningType			'scan'
#define	postscriptScanningID			(printingDriverBaseID)


// Old Application Support Resources

#define	custType 		'cust'
#define	custID 			-8192

#define	reslType 		'resl'
#define	reslID 	 		-8192
#define discreteResolution 0

#define	stlDialogResID 		-8192
#define	jobDialogResID 		-8191


// -------------------------------------------------------------------------------
// Resource types and IDs used in papertype files
// -------------------------------------------------------------------------------


// Resource type and ID for driver papertypes placed in individual files

#define	signatureType					'sig '
#define	papertypeSignatureID			0


// file type for driver papertypes placed in individual files

#define	drvrPaperType					'drpt'


#endif
