/* ------------------------------------------------------------------------------
FILENAME
	PrintingResTypes.r

DESCRIPTION
	This file contains the resource descriptions used by writers ofapplications, 
	Printer Drivers and Printing Extensions.
		
COPYRIGHT
	 Copyright Apple Computer, Inc. 1988-1993
	 All rights reserved. 
	
-------------------------------------------------------------------------------- */
#include "PrintingResTypes.h"

// -------------------------------------------------------------------------------
// Resource definitions used by applications, extension and driver writers
// -------------------------------------------------------------------------------

// 'cltn' - definition of a collection resource

type 'cltn' {
	longint = $$CountOf(ItemArray);
	array ItemArray
		{
		longint	tag;
		longint	id;
			boolean		itemUnlocked			=	false,	// defined attributes bits...
						itemLocked				=	true;
			boolean		itemNonPersistent		=	false,
						itemPersistent			=	true;
			unsigned bitstring[14] = 0;						// reserved attributes bits...
			unsigned bitstring[16] userBits;				// user attributes bits...
		wstring;
		align word;
	};
};



// -------------------------------------------------------------------------------
// Resource definitions used by both extension and driver writers
// -------------------------------------------------------------------------------

type overrideType
	{
	integer = $$CountOf(OverrideArray);
	array OverrideArray
		{
		integer	enabledMessageID;
		unsigned bitstring[4] = 0;
		unsigned bitstring[12] dispatchResID;
		integer	codeOffset;
		longint = -1;
	};
};


// -------------------------------------------------------------------------------
// Resource definitions REQUIRED by extension writers
// -------------------------------------------------------------------------------

type extensionScopeType {
	
	integer = $$CountOf(ScopeArray);
	
	array ScopeArray {
		longint	scopeOSType;
	};
};

type extensionLoadType {

	longint	loadPriority;
};

type extensionOptimizationType {

	boolean		executeDuringImaging		=	true,
				dontExecuteDuringImaging	=	false;
	boolean		needDeviceStatus			=	true,
				dontNeedDeviceStatus		=	false;
	boolean		changePageAtDespoolPage		=	true,
				dontChangePageAtDespoolPage	=	false;
	boolean		changePageAtImagePage		=	true,
				dontChangePageAtImagePage	=	false;
	boolean		changePageAtRenderPage		=	true,
				dontChangePageAtRenderPage	=	false;
	unsigned bitstring[27] = 0;							// reserved flags
};


// -------------------------------------------------------------------------------
// Resource definitions REQUIRED by driver writers
// -------------------------------------------------------------------------------

// imaging system selection
type imagingSystemSelectorType
	{
	longint;		// type of imaging system to select
};


#define	defaultOptionsTranslation		0x0000
#define optimizedTranslation		 	0x0001
#define replaceLineWidthTranslation	 	0x0002
#define simpleScalingTranslation	 	0x0004
#define simpleGeometryTranslation	 	0x0008	/* implies simple scaling */
#define simpleLinesTranslation			0x000C	/* implies simple geometry & scaling */
#define layoutTextTranslation		 	0x0010	/* turn on line layout (normally off) */

type custType
	{
	integer; 										// horizontalResolution;
	integer; 										// verticalResolution;
	integer 	defaultUpDriver 	= 0,
				laserWriter 		= 0,
				laserWriterSC 		= 1;			// upDriverType

	point;											// pattern stretch factor
	integer;										// translator settings
	};

type reslType
	{
	integer	rangeType = 1; 								// constant;
	integer;											//	xMinimumResolution
	integer;											//	xMaximumResolution
	
	integer;											//	yMinimumResolution
	integer;											//	yMaximumResolution
	
	integer = $$CountOf(ResolutionArray);
	array		ResolutionArray
		{
		integer;										// xResolution
		integer;										// yResolution
		};
	};



type 'PREC' (3)
	{
	integer;								/* count: number of paper types */
	point;								/* a paper type's bottom/left coordinate */
	point;								/* ...expressed in 120th of an inch */
	point;								/* There are always six of them */
	point;
	point;
	point;
	pstring;								/* The paper type name */
	pstring;								/* The paper type name */
	pstring;								/* The paper type name */
	pstring;								/* The paper type name */
	pstring;								/* The paper type name */
	pstring;								/* The paper type name */
	};

type 'PREC' (5)
	{
	integer;								/* count: number of paper types */
	point;								/* a paper type's bottom/left coordinate */
											/* ...expressed in 120th of an inch */
											/* There are always six of them */
	integer emulation = 0, theoretical = 1;	/* how the page size should be calculated */
	point;									
	integer emulation = 0, theoretical = 1;
	point;									
	integer emulation = 0, theoretical = 1;
	point;
	integer emulation = 0, theoretical = 1;
	point;
	integer emulation = 0, theoretical = 1;
	point;
	integer emulation = 0, theoretical = 1;	/* how the page size should be calculated */
	pstring;								/* The paper type name */
	pstring;								/* The paper type name */
	pstring;								/* The paper type name */
	pstring;								/* The paper type name */
	pstring;								/* The paper type name */
	pstring;								/* The paper type name */
	};


// IO RELATED RESOURCES

// the "look"er resource defines the list of things to look for - and thus the types
// of communications this device will support.


#define isAppleTalk		1			/* looker type is AppleTalk */
#define iconCells		2			/* looker wants large cells w/ icons in them */
#define isPrinterShare	4			/* looker is for a PrinterShare connection */

type 'look' (-4096) {
	integer;						// looker to select by default
	integer = $$CountOf(LookerList);
	array LookerList {
	pstring[33];					// name of looker - displayed to user in list
	align word;
	integer;						// id of 'comm' resource for this looker;
	longint noFlags = 0;			// flags for this looker
	pstring[33];					// NBP type, or default item name for non-AppleTalk
	};
};


// -------------------------------------------------------------------------------
// Resource definitions optional for driver writers
// -------------------------------------------------------------------------------
/* this resource should be included in specific drivers that wish to supply driver */
/* specific output file formats for the print to file option.  The pstrings will */
/* be added to the standard file dialog popup menu when the user confirms the  */
/* print dialog after selecting 'Print to a file'.  The job contains a destination */
/* record which is updated from this dialog.  The destination record specifies */
/* whether output is to go to the printer or to disk.  If it is to go to disk */
/* the format string in the record will designate the format of the file to create. */
/* the format string will either be 'Print File' specifying a spool file or */
/* a string the driver has provided through this resource.  */

type 'pfil' {
	integer = $$Countof(fileFormatArray);
	array fileFormatArray {
		pstring;								/* what to show in menu - formatname */
		};
	};

/* this resource should be included in specific drivers that wish to suppress quality */
/* or paper feed controls.  This would be used if a driver does not support more */
/* than one type of paper feed or more than one type of quality.  This resource */
/* should also be supplied by any driver which wants to force the quality setting */
/* on a driver switch ( setRoughQuality, etc ).  If this setting is other than */
/* setCurrentQuality, the last field should be set to indicate whether the user */
/* should be alerted before a quality setting is changed.  The system looks */
/* for a 'prop' resource ( print options ) with an id of 0 whenever there is a driver switch.  */

type 'prop' {
	byte		disableQuality, enableQuality;
	byte		disablePaperFeed, enablePaperFeed;
	byte	 	setRoughQuality, setFinalQuality, setCurrentQuality;
	byte		dontAlertBeforeResettingQuality, alertBeforeResettingQuality;
	};

// IO RELATED RESOURCES

// Connection to the printer control resources 													*/
type 'comm'
{
	switch 
		{
		case Serial:
			key unsigned longint='SPTL';																/* Communications type identifier */
		
			integer						baud300 = 380, baud600 = 189, baud1200 = 94, baud1800 = 62, /* Output baud rate */
											baud2400 = 46, baud3600 = 30, baud4800 = 22, baud7200 = 14, 
											baud9600 = 10, baud19200 = 4, baud57600 = 0;
			integer						noParity = 0, oddParity = 4096, evenParity = 12288;			/* Output parity */
			integer						oneStop = 16384, oneFiveStop = -32768, twoStop = -16384;		/* Output stop bits */
			integer						data5 = 0, data6 = 2048, data7 = 1024, data8 = 3072;			/* Output data bits */
			unsigned hex longint;																		/* Output handshaking high word */
			unsigned hex longint;																		/* Output handshaking low word */
			integer						baud300 = 380, baud600 = 189, baud1200 = 94, baud1800 = 62, /* Input baud rate */
											baud2400 = 46, baud3600 = 30, baud4800 = 22, baud7200 = 14, 
											baud9600 = 10, baud19200 = 4, baud57600 = 0;
			integer						noParity = 0, oddParity = 4096, evenParity = 12288;			/* Input parity */
			integer						oneStop = 16384, oneFiveStop = -32768, twoStop = -16384;		/* Input stop bits */
			integer						data5 = 0, data6 = 2048, data7 = 1024, data8 = 3072;			/* Input data bits */
			unsigned hex longint;																		/* Input handshaking high word */
			unsigned hex longint;																		/* Input handshaking low word */
			integer;																							/* Serial input buffer size */
			pstring[63];																					/* Input port name */
			pstring[63];																					/* Output port name */
			
		case PAP:
			key unsigned longint='PPTL';		/* Communications type identifier */
			integer;									/* Flow quantum */
			string[99];								/* Compacted AppleTalk name of printer */
			fill byte;
			longint;									/* Location of open status buffer pointer */
			longint;									/* Location if eof read byte pointer */
			longint;									/* Location if eof write byte pointer */
			longint;									/* Most recent network address of the printer */
			
		case SCSI:
			key unsigned longint='sPTL';		/* Communications type identifier */
			longint;									/* Pointer to C routine that can release SCSI device; can be nil */
			integer;									/* SCSI I/O attributes applicable to data transfers*/
			longint;									/* Location to store status byte from data transfer operation; can be nil */
			integer;									/* SCSI bus number where device is located (for HIOOpenConn call). 0 = motherboard */
			integer;									/* SCSI device number of device (for HIOOpenConn call). */
			longint;									/* 0 => ignored; > 0 => break data transfer into chunks of this size */
														/* (at SCSI TIB level) */
			longint;									/* Pointer to C routine that can acquire SCSI device; can be nil */
			integer;									/* deviceType to look for */
			integer;									/* minimum amount of data in the response */
			integer;									/* offset from start to look in the response data */
			pstring;									/* string to look for in the response data */
			
		case PrinterShare:
			key unsigned longint='ptsr';		/* Communications type identifier */
			string[99];								/* Compacted AppleTalk name of server */
			fill byte;
			longint;									/* Most recent network address of the printer */

		case NotConnected:
			key unsigned longint='nops';		/* Communications type identifier */
		};
};


// This resource controls the behavior of the standard buffering and IO
// within the Printing Manager.  If the specific driver does not include
// such a resource, the system defaults to 2 buffers of 1K each,
// and timeout values of 10 seconds each.  The specific driver can prevent
// the system from doing buffering or IO by specifying the correct
// values within this resource
type universalIOPrefsType
{
	longint standardIO = 0x00000000, customIO = 0x00000001;
	
	longint;		// number of buffers to allocate, 0 = none
	longint;		// size of each buffer
	longint;		// number of IO requests that can be pending at any one time
	longint;		// open/close timeout in ticks
	longint;		// read/write timeout in ticks
};


// A driver may supply 3 capture strings which are used to capture PAP devices.
//	 captureStringID				- string used for capture/uncapture (no length byte)
//	 uncapturedAppleTalkType	- AppleTalk type used for uncaptured devices (w/ length byte)
//	 capturedAppleTalkType		- AppleTalk type used for captured devices (w/ length byte)
// The default implementation performs string substitution on the capture string
// prior to sending it to the device, and the following strings are defined:
//  PRINTERNAME	- name of the printer 
//	 PRINTERTYPE	- type of the printer 
//	 NAMELEN	- name length byte
//	 TYPELEN - type length byte

type captureType {
		string;												/* String				*/
};



// RASTER RELATED RESOURCES

// This resource specifies the imaging preferences for raster drivers.
// Many common drivers can simply specify their imaging preferences via this resource.
#define noColorPacking			0x0000
#define word5ColorPacking		0x0500
#define long8ColorPacking		0x0800
#define long10ColorPacking		0x0a00
#define alphaFirstPacking		0x1000

#define noSpace					0

#define rgbSpace				1
#define cmykSpace				2
#define hsvSpace				3
#define hlsSpace				4

#define cieSpace				5
#define xyzSpace				6
#define luvSpace				7
#define labSpace				8

#define yiqSpace				9
#define ntscSpace				yiqSpace
#define palSpace				yiqSpace

#define graySpace				10
#define indexedSpace			11
#define alphaSpace				0x0080

#define rgbaSpace				rgbSpace + alphaSpace
#define grayaSpace				graySpace + alphaSpace

#define rgb16Space				word5ColorPacking + rgbSpace
#define rgb32Space				long8ColorPacking + rgbSpace
#define argb32Space				long8ColorPacking + alphaFirstPacking + rgbaSpace
#define cmyk32Space				long8ColorPacking + cmykSpace
#define hsv32Space				long10ColorPacking + hsvSpace
#define hls32Space				long10ColorPacking + hlsSpace
#define cie32Space				long10ColorPacking + cieSpace
#define xyz32Space				long10ColorPacking + xyzSpace
#define luv32Space				long10ColorPacking + luvSpace
#define lab32Space				long10ColorPacking + labSpace
#define yiq32Space				long10ColorPacking + yiqSpace
#define ntsc32Space				yiq32Space
#define pal32Space				yiq32Space
#define graya32Space			grayaSpace

type rasterPrefsType
{
	longint 	defaultRaster = 0,								// default options
				dontResolveTransferModes   	= 0x01,				// 0=Resolve, 1=Don't Resolve
				renderInReverse		      	= 0x02,				// traverse image in reverse order
				onePlaneAtATime				= 0x04,				// render each plane separately
				sendAllBands				= 0x08;				// send even white bands

															// both of these are fixed point numbers
	longint;												// horizontal resolution to image at
	longint;												// vertical resolution to image at
	
	integer;												// min band size
	integer; 												// max band size
	
	longint;												// RAM percentage
	longint;												// RAM slop
	
	integer;												// depth of imaging, in pixels per plane
	integer = $$CountOf(PlaneArray);
	array PlaneArray 
		{
		// plane flags
		longint defaultOffscreen = 0,
				dontSetHalftone = 1,
				dotTypeIsDitherLevel = 2;
					
		// HALFTONE STRUCTURE
		hex longint;									// fixed point angle
		hex longint;									// fixed point frequency
		
		longint roundDot=1, spiralDot, 	// type of dither
			squareDot, lineDot, ellipticDot, 
			triangleDot, hilbertDot, orderedDitherDot;
		longint 											// tintType
			luminanceTint=1, averageTint, mixtureTint, 
			component1Tint, component2Tint, component3Tint, component4Tint;
		
		longint;											// dot color
		longint noProfile = 0;											
		hex integer;
		hex integer;
		hex integer;
		hex integer;
		
		longint;											// background color
		longint noProfile = 0;											
		hex integer;
		hex integer;
		hex integer;
		hex integer;

		longint;											// tintSpace
		
		// OTHER PLANE FLAGS
		longint;											// plane colorSpace, can be noSpace
		longint noSet = 0;									// plane colorSet
		longint noProfile = 0;								// plane colorProfile
		};
};


// The raster packaging resource controls how your driver uses the default
// RasterDataIn message.  If you implement this message yourself, you should
// not have this resource.  If you use the default RasterDataIn message,
// you MUST have a resource of this type
#define sendAllColors		0x00000001
#define interlaceColor		0x00000002
#define overlayColor		0x00000004

type rasterPackType
{
	longint;			// buffer size for packaging (>= maximum head pass size)
	integer;			// iColorPasses :1 or 4 is typical
	integer;			// iHeadHeight  :Printhead height in pixels
	integer;			// iNumberPasses :number of head passes it takes to == iHeadHeight
	integer;			// iPassOffset : offset between passes, in pixels
	hex longint;	// flags
}; 


// This resource controls the remaining 2 raster packaging messages: RasterLineFeed,
// and RasterPackageBitmap.  If you allow the generic driver to implement these messages for
// you then you MUST have a resource of this type. 

type rasterPackOptionsType
{
	integer;		// start page wstring ID
	integer;		// form feed wstring ID
	
	// forward line feed
	integer;		// max value
	integer;		// numberType
	integer;		// minWidth
	char;			// pad char
	fill byte;
	pstring[31];	// prefix string		
	pstring[31];	// postfix string
	
	// reverse line feed
	integer;		// max value
	integer;		// numberType
	integer;		// minWidth
	char;			// pad char
	fill byte;
	pstring[31];	// prefix string
	pstring[31];	// postfix string
	
};


// For specifying a colorSet.

type colorSetResType {
	longint;							// colorSpace for the colorSet
	longint = $$CountOf(colorSet);
	array colorSet 						// members of the colorSet
		{
		hex integer;	
		hex integer;
		hex integer;
		hex integer;
		};
};



// POSTSCRIPT IMAGING RESOURCES

// resource definition for color matchin data
#define colortriplet { unsigned integer; unsigned integer; unsigned integer; }
#define	colorheptet  { unsigned integer red; 		\
						unsigned integer green;		\
						unsigned integer blue;		\
						unsigned integer cyan;		\
						unsigned integer magenta;	\
						unsigned integer yellow;	\
						unsigned integer gray;	}

type colorMatchingDataType {

	wide array red		colortriplet;
	wide array green 	colortriplet;
	wide array blue 	colortriplet;
	wide array cyan 	colortriplet;
	wide array magenta 	colortriplet;
	wide array yellow 	colortriplet;
	wide array black 	colortriplet;
	wide array white 	colortriplet;	
	
redcount:		unsigned integer;
greencount:		unsigned integer;
bluecount:		unsigned integer;
cyancount:		unsigned integer;
magentacount:	unsigned integer;
yellowcount:	unsigned integer;
graycount:		unsigned integer;
	
	wide array [ $$Word( redcount ) ] 		{ unsigned integer; };
	wide array [ $$Word( greencount ) ] 	{ unsigned integer; };
	wide array [ $$Word( bluecount ) ] 		{ unsigned integer; };
	wide array [ $$Word( cyancount ) ] 		{ unsigned integer; };
	wide array [ $$Word( magentacount ) ]	{ unsigned integer; };
	wide array [ $$Word( yellowcount ) ] 	{ unsigned integer; };
	wide array [ $$Word( graycount ) ] 		{ unsigned integer; };
};



// resource for PostScript procedure set control resource

// these types are for the second integer in the array

#define donothing			0
#define dumpwidestring		1
#define dumpstringlist		2
#define converttohex		0x0100

type postscriptProcSetControlType	{

		pstring;		 								// procset name
		align word;
		hex longint;								// procset version -- type( Fixed )
		integer;										// revision
		longint;										// vm usage
		
		integer = $$Countof(IDArray);				// Number of resources that make up the data set.

			wide array IDArray {
				
				integer;								// ID of the resource containing the data.
				integer;								// the flags are defined above

			};
	};


type postscriptPrinterFontType {

	longint		ROMFont = 0;				// memory usage
	
	switch		{
		case	AdobeCharacterSet:			// a font with the adobe character set
			key hex longint = 0x00000000;
		
		case	AppleCharacterSet:
			key hex longint = 0x00010000;	// a font with some of the glyph in the apple std set
			glyphbits:
				longint		glyphcount;		// the max number of the glyph in the array
				hex string [ ( ( $$Long( glyphbits ) + 7 ) / 8 ) ];
				align long;
	
		case	EquivalentFont:				// a font with all equivalent glyphs on the mac
			key hex longint = 0x00020000;
	
		case	EncodedFont:				// a font that must be used as encoded
			key hex longint = 0x00030000;
				longint;					// platform
				longint;					// script
				longint;					// language
	};
};

// the following defines are used for the flags fields in the PostScriptPrefs data structure

#define needsHex				1
#define needsComments			2
#define boundingBoxesOption		4
#define portablePostScript		8
#define textClipsToPath			16
#define flattenClipPath			32
#define usercharpath1			64
#define useLevel2Color			128

// the following define is for the fontType field in the PostScriptPrefs data structure
// it should parallel the enumeration for "streamType" inside the file "scaler types.h"

#define truetypeStream	0x0001
#define type1Stream		0x0002
#define type3Stream		0x0004
#define type42Stream	0x0008
#define type42GXStream	0x0010
#define portableStream	0x0020

type postscriptPrefsType	{

		integer			languageLevel;
		longint			deviceCSpace;						// the color space for the device
		longint			renderOptions;

		integer			pathLimit;
		integer			gsaveLimit;
		integer			opStackLimit;
		longint			FontTypes;

		longint			printerVM;
};


// the following is the definition for the default scanning resource for postscript imaging system

#define StringScan	switch	{					\
			case SimpleScan:					\
				key integer = 0;				\
						wstring;				\
						align word;				\
			case UserNameScan:					\
				key integer = 1;				\
			case DocumentNameScan:				\
				key integer = 2;				\
			case PrinterNameScan:				\
				key integer = 3;				\
			case NilPtrScan:					\
				key integer = 4;				\
				integer	length;					\
		}
		
#define	OffsetScan	switch	{					\
			case SimpleOffset:					\
				key integer = 0;				\
			case SameAsPreviousOffset:			\
				key integer = 1;				\
			case ReturnedOffset:				\
				key integer = 2;				\
			case SimpleRepeat:					\
				key integer = 16;				\
			case SampleAsPreviousRepeat:		\
				key integer = 17;				\
			case ReturnedRepeat:				\
				key integer = 18;				\
		}
		
#define	ActionScan	switch	{					\
			case NoAction:						\
				key integer = 0;				\
			case SimpleAction:					\
				key integer = 1;				\
				integer	normal = 0,				\
						nonFatalError,			\
						fatalError;				\
				integer	alertID;				\
		}

type postscriptScanningType		{

	longint ownerCount;
	
	array	{ 
				StringScan;
				StringScan;
				OffsetScan;
				ActionScan;
			};
};

// PAPER TYPE ('ptyp') DEFINITION AND CONSTANTS	

// Version of the 'ptyp' resource definition

#define		paperTypeVersion	0x00010000	// Version 1.0


// Paper Type Creators (OSType's)

#define		sysPaperType	'sypt'	// System paper type creator
#define		userPaperType	'uspt'	// User paper type creators
									// Driver creator types = driver file's creator value


// Miscellaneous types used by the 'ptyp' definition

#define		fixed			hex longint
#define		rectangle		fixed; /* left */ fixed; /* top */ fixed; /* right */ fixed; /* bottom */


// 'ptyp' - definition of a paper type resource

type 'ptyp' {
	pstring;								// paper type name
	align word;
	rectangle;								// page rectangle
	rectangle;								// paper rectangle
	longint		unknownBase		=	0,		// base paper type from which this paper type is
				usLetterBase	=	1,		// derived.  This is not a complete set.
				usLegalBase		=	2,
				a4LetterBase	=	3,
				b5LetterBase	=	4,
				tabloidBase		=	5;
	literal longint;						// creator
	byte			pica	=	0,			// Unit of measure
					mm		=	1,
					inch	=	2;
	unsigned bitstring[2]	newStylePaperType		=	1,	// Flags
							oldStylePaperType		=	2,
							oldAndNewStylePaperType	=	3;
	boolean		notFixedPrintArea		=	false,
				fixedPrintArea			=	true;
	boolean		notDefaultPaperType		=	false,
				isDefaultPaperType		=	true;
	boolean		notDefaultBasePT		=	false,
				defaultBasePT			=	true;
	unsigned bitstring[19] = 0;						// Reserved flags
	
	longint = $$CountOf(ItemArray);					// embedded collection
	array ItemArray
		{
		longint	tag;
		longint	id;
			boolean		itemUnlocked			=	false,	// defined attributes bits...
						itemLocked				=	true;
			boolean		itemNonPersistent		=	false,
						itemPersistent			=	true;
			unsigned bitstring[14] = 0;						// reserved attributes bits...
			unsigned bitstring[16] userBits;				// user attributes bits...
		wstring;
		align word;
	};
};

// STATUS RELATED RESOURCES

type 'stat' {
	longint		statusOwner;
	array statarray {
		INTEGER		nonFatalError = 1, 
					fatalError = 2, 
					printerReady = 3, 
					userAttention = 4, 
					userAlert = 5, 
					pageTransmission = 6, 
					openConnectionStatus = 7,
					informationalStatus = 8,
					spoolingPageStatus = 9,
					endStatus = 10,
					percentageStatus = 11;
					
		INTEGER		statusId;
		INTEGER		statusAlertId;
		pstring; // statusMessage;
		align word;
		};
	};
	

// DIALOG RELATED RESOURCES

#define		xdtlRadioButtons		0
#define		xdtlCheckBox			1
#define		xdtlEditTextInteger		2
#define		xdtlEditTextReal		3
#define		xdtlEditTextString		4
#define		xdtlPopUp				5

type 'panl' {
	pstring[31];			// the panel name
	integer	Script;			// script id
	integer;				// the icon id
	integer;				// the ditl id
};

type 'xdtl' {
	integer = $$CountOf(xdtlarray) -1;
	wide array xdtlarray {
		switch {
			case RadioButtons:
				key		integer = xdtlRadioButtons;
				literal	longint;		// 4 byte id for storage in job or format
						longint;		// numerical id for storage in job or format
						integer;		// offset in bytes into tag item
						integer = $$CountOf(RadioButtonsArray) - 1;
						wide array RadioButtonsArray
						{
							byte;		// array of corresponding items
						};
			case CheckBox:
				key		integer = xdtlCheckBox;
				literal	longint;		// 4 byte id for storage in job or format
						longint;		// numerical id for storage in job or format
						integer;		// offset in bytes into tag item
						byte;			// corresponding ditl item
						fill byte;
									
			case EditTextInteger:
				key		integer = xdtlEditTextInteger;
				literal	longint;			// 4 byte id for storage in job or format
							longint;		// numerical id for storage in job or format
							integer;		// offset in bytes into tag item
							byte;			// corresponding ditl item
							byte;			// 0 = dont select, 1 = select
							pstring[15];	// low bound - nil means 'I don't care'
							pstring[15];	// high bound - nil means 'I don't care'

			case EditTextReal:
				key		integer = xdtlEditTextReal;
				literal	longint;			// 4 byte id for storage in job or format
							longint;		// numerical id for storage in job or format
							integer;		// offset in bytes into tag item
							byte;			// corresponding ditl item
							byte;			// 0 = dont select, 1 = select
							pstring[15];	// low bound - nil means 'I don't care'
							pstring[15];	// high bound - nil means 'I don't care'

			case EditTextString:
				key		integer = xdtlEditTextString;
				literal	longint;		// 4 byte id for storage in job or format
						longint;		// numerical id for storage in job or format
						integer;		// offset in bytes into tag item
						byte;			// corresponding ditl item
						byte;			// 0 = dont select, 1 = select

			case PopUp:
				key		integer = xdtlPopUp;
				literal	longint;		// 4 byte id for storage in job or format
						longint;		// numerical id for storage in job or format
						integer;		// offset in bytes into tag item
						byte;			// corresponding ditl item 
						fill byte;
			};
			align word;
		};
	};
	
// COOL ALERT RELATED RESOURCES

type 'cool' {
	integer		coolAlert = 1, coolStatus = 2;									// cool alert version
	integer 	noIcon = -1, stopIcon = 0, noteIcon = 1, cautionIcon = 2;		// icon id
	integer 	defaultSystemSize = 0;											// text size
	byte		noDefaultTitle, defaultAction, defaultTitle2, defaultTitle3;	// default button
	byte		noCancelTitle, cancelAction, cancelTitle2, cancelTitle3;		// cancel button
	wstring;																	// text string
	pstring;																	// action button label
	pstring;																	// button label 2
	pstring;																	// button label 3
	pstring;																	// font name
	pstring		AlertTitle = "Alert";											// alert title
	};


// DESKTOP CONFIGURATION FILE RELATED RESOURCES

// driver resource to specify the tray name

type trayNameDataType {					// tray name data type ('tryn')
	pstring[31];
};

// driver resource to specify the tray count

type trayCountDataType {
	longint;							// count of trays
};

type 'dctl'{
	integer DTILsize;								/* Maximum item count for DITL */
	
	integer = $$CountOf(dctlarray) - 1;		/* Array size */
	wide array dctlarray {
		switch {

		case Button:
			key integer 	= 1;
				integer itemID;				/* Item ID that this is the button of */
				integer							/* Button kind */
					cancel	= 0;	
		case Cluster:
			key integer 	= 2;
			integer							/* What kind of cluster this is? */
				feed		= 0,
				quality		= 1,
				coverPage	= 2,
				firstPage	= 3,
				restPage	= 4,
				headMotion	= 5,
				createFile	= 6,
				user0		= 7,
				user1		= 8,
				user2		= 9;
				
			integer = $$CountOf(ClusterArray) - 1;
				wide array ClusterArray
					{
					integer item;			/* The item ID this corresponds to */
					};

		case Copies:
			key integer = 3;
				integer itemID;				/* ID of the item */
				
		case DialogBtn:
			key integer = 4;
				integer itemID;				/* Item ID of the cascade button */
				integer DLOGID;				/* ID of the dialog */
				integer dctlID;				/* ID of the dctl for the dialog */

		case Frill:
			key integer = 5;		
				integer itemID;				/* Item ID of this frill */
				integer						/* What kind of frill might this be? */
					line 		= 0,
					version		= 1,
					default		= 2,
					printerName	= 3;

		case Moof:
			key integer = 6;
				integer itemID;				/* ID of the Moof */
				
		case OKButton:
			key integer = 7;
				integer itemID;				/* Item ID of the OK button */
				integer	strID1;				/* ID of the "Print" string */
				integer	strID2;				/* ID of the "Save" string */

		case Orientation:
			key integer = 8;	
				integer portraitID;			/* ID of portrait orientation */
				integer landscapeID;		/* ID of the landscape orientation */
				integer altPortaitID;		/* ID of flipped portrait */
				integer altLandscapeID;		/* ID of the flipped landscape */

		case PageRange:
			key integer = 9;
				integer allID;				/* ID of the "all" button */
				integer rangeID;			/* ID of the range button */
				integer fromID;				/* ID of the from edit text */
				integer toID;				/* ID of the to edit text */
				
		case PaperSizes:
			key integer = 10;
			integer 		popupID;		/* Item of the popup */
			integer		popupRadio;			/* Item of the popup radio button */
			fill long;
			fill word;
						
			integer = $$CountOf(ClusterArray) - 1;
				wide array ClusterArray
					{
					integer item;			/* The item ID this corresponds to */
					};
					
		case Scale:
			key integer = 11;
					integer editID;			/* Item ID of the edit text */
					integer arrowID;		/* Item ID of the arrow useritem */
					integer stabResource;	/* resource ID for the 'stab' resource */
		case Toggle:
			key integer = 12;
			integer itemID;					/* Item ID that this coresponds to */
			integer							/* Value to OR into flags to set it */
				bPreciseBitmap 		= $0001,
				bBiggerPages		= $0002,
				bGraphicSmoothing	= $0004,
				bTextSmoothing		= $0008,
				bFontSubstitution	= $0010,
				bInvert				= $0020,
				bFlipHoriz			= $0040,
				bFlipVert			= $0080,
				bColorMode			= $0100,
				bBidirectional		= $0200,
				bUser0				= $0400,
				bUser1				= $0800,
				bUser2				= $1000,
				bReserved0			= $2000,
				bReserved1			= $4000,
				bReserved2			= $8000;
		};
	align word;								/* Each item is word aligned */
	};
};


type 'stab' {					/* definition for values in reduction table -	*/
								/*  NOTE - VALUES EXPECTED IN ASCENDING ORDER  	*/
	integer = $$Countof(valarray);
	array valarray {
		integer;
		};
	};



/********
	This is the resource type expected by RDResPrintf.
		It is an array of pascal strings.
		
**********/
type 'rdpf'
	{
	
		integer = $$Countof(StringArray);					//the number of script elements in resource.
		array		StringArray {
			pstring;
		};
		
	};

/**********
	This resource type is for RDResBDump.  The resource may be 
	up to 65k bytes and will be stored in a really big pascal string.
	the first two bytes will be the length.
***********/
type 'rdws'
	{
		wstring;
	};

