/* graphics libraries:
	general library interfaces
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

#pragma once
#ifndef graphicsLibrariesIncludes
	#define graphicsLibrariesIncludes

	#ifndef graphicsRoutinesIncludes
		#include "graphics routines.h"
	#endif
	#ifndef mathRoutinesIncludes
		#include "math routines.h"
	#endif

	#ifndef qdTextIncludes
		#define DrawChar gDrawChar
	#endif

	/*	We use if( a ) b; else instead of { if (a ) b; } because the first one can be used in an if-else
		statement and the second one cant (due to the ; at the end of the macro instance. */

	#define NilShapeReturn(a)			if ((a) == (shape) nil) {PostGraphicsError(shape_is_nil); return; } else
	#define NilShapeReturnNil(a)		if ((a) == (shape) nil) {PostGraphicsError(shape_is_nil); return 0L; } else
	#define NilStyleReturn(a)			if ((a) == (style) nil) {PostGraphicsError(style_is_nil); return; } else
	#define NilStyleReturnNil(a)		if ((a) == (style) nil) {PostGraphicsError(style_is_nil); return 0L; } else
	#define NilInkReturn(a)			if ((a) == (ink) nil) {PostGraphicsError(ink_is_nil); return; } else
	#define NilInkReturnNil(a)			if ((a) == (ink) nil) {PostGraphicsError(ink_is_nil); return 0L; } else
	#define NilTransformReturn(a)		if ((a) == (transform) nil) {PostGraphicsError(transform_is_nil); return; } else
	#define NilTransformReturnNil(a)	if ((a) == (transform) nil) {PostGraphicsError(transform_is_nil); return 0L; } else
	#define NilColorSetReturn(a)		if ((a) == (colorSet) nil) {PostGraphicsError(colorSet_is_nil); return; } else
	#define NilColorSetReturnNil(a)		if ((a) == (colorSet) nil) {PostGraphicsError(colorSet_is_nil); return 0L; } else
	#define NilColorProfileReturn(a)	if ((a) == (colorProfile) nil) {PostGraphicsError(colorProfile_is_nil); return; } else
	#define NilColorProfileReturnNil(a)	if ((a) == (colorProfile) nil) {PostGraphicsError(colorProfile_is_nil); return 0L; } else
	#define NilTagReturn(a)			if ((a) == (tag) nil) {PostGraphicsError(tag_is_nil); return; } else
	#define NilTagReturnNil(a)		if ((a) == (tag) nil) {PostGraphicsError(tag_is_nil); return 0L; } else

	#ifdef debugging
		/* just in case the application (or the graphics system) redefined these to something weird... */
		#undef DebugStr
		#undef IfDebug

		#define IfDebug(a, b)				if (a) DebugStr(b); else
		#define IfDebugReturn(a, b)		if (a) { DebugStr(b); return; } else
		#define IfDebugReturnNil(a, b)		if (a) { DebugStr(b); return 0L; } else
		#define NilParamReturn(a)		if ((a) == 0L) {PostGraphicsError(parameter_is_nil); return; } else	
		#define NilParamReturnNil(a)		if ((a) == 0L) {PostGraphicsError(parameter_is_nil); return 0L; } else	
		#define IfErrorReturn(a,b)  		if (a) { PostGraphicsError(b); return; } else
		#define IfErrorReturnNil(a,b)  		if (a) { PostGraphicsError(b); return 0L; } else
		#define IfNotice(a, b)			if (a) PostGraphicsNotice(b); else
		#define IfWarning(a,b) 			if (a) PostGraphicsWarning(b); else
		#define IfWarningReturn(a,b) 		if (a) { PostGraphicsWarning(b); return; } else
		#define IfWarningReturnNil(a,b) 	if (a) { PostGraphicsWarning(b); return 0L; } else
	#else
		#define IfDebug(a, b)
		#define IfDebugReturn(a, b)
		#define IfDebugReturnNil(a, b)
		#define NilParamReturn(a)
		#define NilParamReturnNil(a)
		#define IfErrorReturn(a,b)
		#define IfErrorReturnNil(a,b)
		#define IfNotice(a, b)
		#define IfWarning(a,b)
		#define IfWarningReturn(a,b)		if (a) return; else
		#define IfWarningReturnNil(a,b)	if (a) return 0L; else
	#endif


	typedef enum {
			/* color names */
		gxWhite = 1,		gxBlack,				gxGray,				gxGrey = gxGray,
	#ifndef __QUICKDRAW__
		white = 1,			black,				gray,				grey = gray,
	#endif
			/* primaries */
		red,				green,				blue,
			/* secondaries */
		cyan,			magenta,				yellow,
			/* tertiaries */
		orange,			chartreuse,			aqua,				teal = aqua,
		slate,			purple,				violet = purple,			maroon,
			/* others */
		brown,			pink,					turquoise,
		cadmium_lemon,	cadmium_light_yellow,	aureoline_yellow,		naples_deep_yellow,
		cadmium_yellow,	cadmium_deep_yellow,	cadmium_orange,		cadmium_light_red,
		cadmium_deep_red,	geranium_lake,			alizarin_crimson,		rose_madder,
		madder_deep_lake,	brown_madder,			permanent_red_violet,	cobalt_deep_violet,
		ultramarine_violet,	ultramarine_blue,		cobalt_blue,			royal_blue,
		cerulean_blue,		manganese_blue,		indigo,				turquoise_blue,
		emerald_green,		permanent_green,		viridian_light,			cobalt_green,
		cinnabar_green,	sap_green,			chromium_oxide_green,	terre_verte,
		yellow_ochre,		mars_yellow,			raw_sienna,			mars_orange,
		gold_ochre,		brown_ochre,			deep_ochre,			burnt_umber,
		burnt_sienna,		flesh,				flesh_ochre,			english_red,
		venetian_red,		indian_red,			raw_umber,			greenish_umber,
		van_dyck_brown,	sepia,				warm_grey,			cold_grey,
		ivory_black,		lamp_black,			titanium_white,		zinc_white,
		pale_gold,			gold,					old_gold,				pink_gold,
		white_gold,		yellow_gold,			green_gold,			platinum,
		silver,			antique_silver,			chrome,				steel,
		copper,			antique_copper,		oxidized_copper,		bronze,
		brass,			iron,					rusted_iron,			lead,
		fluorescent_pink,	fluorescent_green,		fluorescent_blue,
		incadescent_high,	incadescent_low,
		moonlight,			sodium,				daylight,				dawn,
		afternoon,			dusk,
		mauve,
		apple_green,		apple_yellow,			apple_orange,			apple_red,
		apple_purple,		apple_blue,

		/* color modifiers */
		light = 512,
		dark = 1024,
		warm = 2048,
		reddish = warm,
		cool = 4096,
		bluish = cool,
		grayish = 8192,
		whitish = light,
		blackish = dark,
		greenish = 16384
	} commonColors;


	typedef long commonColor;

	extern colorSet commonColorSet;
	extern setColor commonColorList[];
	extern short commonColorCount;

/* some very useful constructive definitions for creating colors */
/* see 'color library.c' for an example of their use... */

	#define makeRGBSHORT(a,b,c) ((c >> 14) + ((b >> 13) + (a >> 13) << 3) << 3)
	#define makeCMYKSHORT(a,b,c,d) ((d >> 14) + ((c >> 14) + ((b >> 14) + (a >> 14) << 2) << 2) << 2)
	#define makeRGB16(a,b,c) ((c >> 11) + ((b >> 11) + (a >> 11) << 5) << 5)
	#define makeCMYK16(a,b,c,d) ((d >> 12) + ((c >> 12) + ((b >> 12) + (a >> 12) << 4) << 4) << 4)
	#define xRGB256(a,b,c) {rgbSpace,nil,{(a << 8)+a,(b << 8)+b,(c << 8)+c, 0 }}

	#define xRGB(a,b,c)	{rgbSpace,nil,{a,b,c,0}}
	#define xCMYK(a,b,c,d)	{cmykSpace,nil,{a,b,c,d}}
	#define xHSV(a,b,c) {hsvSpace,nil,{a,b,c,0}}
	#define xCIE(a,b,c) {cieSpace,nil,{a,b,c,0}}
	#define xYIQ(a,b,c) {yiqSpace,nil,{a,b,c,0}}
	#define xXYZ(a,b,c) {xyzSpace,nil,{a,b,c,0}}
	#define xYUV(a,b,c) {yuvSpace,nil,{a,b,c,0}}
	#define xLUV(a,b,c) {luvSpace,nil,{a,b,c,0}}
	#define xLAB(a,b,c) {labSpace,nil,{a,b,c,0}}
	#define xHLS(a,b,c) {hlsSpace,nil,{a,b,c,0}}
	#define xACC(a,b,c) {accSpace,nil,{a,b,c,0}}
	#define xNTSC(a,b,c) {ntscSpace,nil,{a,b,c,0}}
	#define xPAL(a,b,c) {palSpace,nil,{a,b,c,0}}
	#define xSECAM(a,b,c) {secamSpace,nil,{a,b,c,0}}
	#define xDIGITALVIDEO(a,b,c) {digitalVideoSpace,nil,{a,b,c,0}}
	#define xGRAY(a) {graySpace,nil,{a,0,0,0}}
	#define xRGB16(a,b,c) {rgb16Space,nil,{makeRGB16(a,b,c),0,0,0}}
	#define xCMYK16(a,b,c,d) {cmyk16Space,nil,{makeCMYK16(a,b,c,d),0,0,0}}
	#define xRGB32(a,b,c) {rgb32Space,nil,{(a >> 8),((c >> 8) + (b >> 8) << 8),0,0}}
	#define xCMYK32(a,b,c,d) {cmyk32Space,nil,{((b >> 8) + (a >> 8) << 8),((d >> 8) + (c >> 8) << 8),0,0}}

	#define sRGB(r,g,b) {r,g,b,0}
	#define sCMYK(c,m,y,k) {c,m,y,k}
	#define sRGB256(a,b,c) {(a << 8)+a,(b << 8)+b,(c << 8)+c, 0 }

#ifndef _Quickdraw_
#ifndef __QUICKDRAW__
/*
  * The following items are duplicated in LSC QuickDraw.h, so we might
  * skip them here.
  */
	typedef enum {
		bold = 1,
		italic = 2,
		underline = 4,
		outline = 8,
		shadow = 0x10,
		condense = 0x20,
		extend = 0x40
	} commonFaces;

#endif
#endif /* ifndef _Quickdraw_ */

	enum {
		plain,
		lighten = 0x80
		};

	typedef char commonFace;


	typedef enum {						/* These modes simulate QuickDraw's transfer modes		*/
		commonAddOverMode = 2000,
		commonSubtractOverMode,
		commonSubtractPinMode,
		commonTransparentMode,
		commonInMode,
		commonOutMode
	} commonTransferModes;

	typedef short commonTransferMode;
	
	typedef struct {
		point a;
		point b;
		point c;
		point d;
	} cubic;

	typedef struct {
		point a;
		point b;
		point c;
		fixed lambda;
	} conic;

	/* shape library */
	shape NewPath(const path *);
	shape NewPolygon(const polygon *);
	polygon *GetPolygon(shape, long contour, polygon *);
	path *GetPath(shape, long contour, path *);
	void SetPath(shape, long contour, const path *);
	void SetPolygon(shape, long contour, const polygon *);
	void DrawPolygon(const polygon *, shapeFill);
	void DrawPath(const path *, shapeFill);

	void SetShapeIndexPoint(shape, long index, const point *);
	void SetShapeIndexControl(shape, long index, long control);
	point *GetShapeIndexPoint(shape, long index, point *);
	long GetShapeIndexControl(shape, long index, long *control);

	void InsertShape(shape, long index, shape toAdd);
	shape ExtractShape(shape source, long firstPoint, long numPoints);
	void AddToShape(shape dest, shape add);
	void ExtendShape(shape dest, shape add);

	#ifdef debugging
		shape NewShape2(shapeType, fixed, fixed);
		shape NewShape4(shapeType, fixed, fixed, fixed, fixed);
		shape NewShape6(shapeType, fixed, fixed, fixed, fixed, fixed, fixed);

		void SetShape2(shape, fixed, fixed);
		void SetShape4(shape, fixed, fixed, fixed, fixed);
		void SetShape6(shape, fixed, fixed, fixed, fixed, fixed, fixed);
	#else
		#define NewShape2(type,p1,p2)				NewShapeMany(type, (fixed)p1, (fixed)p2)
		#define NewShape4(type,p1,p2, p3, p4)			NewShapeMany(type, (fixed)p1, (fixed)p2, (fixed)p3, (fixed)p4)
		#define NewShape6(type,p1,p2, p3, p4, p5, p6)	NewShapeMany(type, (fixed)p1, (fixed)p2, (fixed)p3, (fixed)p4, (fixed)p5, (fixed)p6)

		#define SetShape2(source,p1,p2)				SetShapeMany(source, (fixed)p1, (fixed)p2)
		#define SetShape4(source,p1,p2, p3, p4)			SetShapeMany(source, (fixed)p1, (fixed)p2, (fixed)p3, (fixed)p4)
		#define SetShape6(source,p1,p2, p3, p4, p5, p6)	SetShapeMany(source, (fixed)p1, (fixed)p2, (fixed)p3, (fixed)p4, (fixed)p5, (fixed)p6)
	#endif
	
	shape NewShapeMany(shapeType type, fixed firstArg, ...);
	void SetShapeMany(shape target, fixed firstArg, ...);
	
	shape GetBitmapPartsFromFixedBounds(shape source, const rectangle *bounds);
	void SetBitmapPartsFromFixedBounds(shape target, const rectangle *bounds, shape bitmapShape);

	/* arc library, oval library, roundrect library, cubic library, and conic library */
	shape NewArc(const rectangle *, fixed startAng, fixed sweep, boolean wedge);
	shape NewOval(const rectangle *);
	shape NewRoundRect(const rectangle *, const point *ovalSize);
	shape NewCubic(const cubic*);
	shape NewConic(const conic*);

	void DrawArc(const rectangle *, fixed startAng, fixed sweep, boolean wedge);
	void DrawOval(const rectangle *, shapeFill);
	void DrawRoundRect(const rectangle *, const point *ovalSize, shapeFill);
	void DrawCubic(const cubic *, shapeFill);
	void DrawConic(const conic *, shapeFill);

	void SetArc(shape, const rectangle *, fixed startAng, fixed sweep, boolean wedge);
	void SetOval(shape, const rectangle *);
	void SetRoundRect(shape, const rectangle *, const point *ovalSize);
	void SetCubic(shape, const cubic *);
	void SetConic(shape, const conic *);
	
	/* graphics debug library */
	void SetGraphicsLibraryErrors(void);	/* set the error and warning routines */
	void SetGraphicsLibraryNotices(void);	/* set the notice routine */
	char *GraphicsErrorMessage(graphicsError);
	char *GraphicsWarningMessage(graphicsWarning);
	char *GraphicsNoticeMessage(graphicsNotice);
	void DisplayGraphicsErrorMessage(graphicsError, long);
	void DisplayGraphicsWarningMessage(graphicsWarning, long);
	void DisplayGraphicsNoticeMessage(graphicsNotice, long);
	
	/* shape library */
	void CenterShape(shape, rectangle *);
	void MoveShapeCenterTo(shape, fixed x, fixed y);
	void RotateShapeAboutCenter(shape, fixed degrees);
	void SkewShapeAboutCenter(shape, fixed xSkew, fixed ySkew);
	void ScaleShapeAboutCenter(shape, fixed hScale, fixed vScale);
	
	/* graphics library */
	void MapShapeToSpace(shape, viewPort, viewDevice);
	void MapShapeFromSpace(shape, viewPort, viewDevice);
	void MapPointToSpace(point *, viewPort, viewDevice);
	void MapPointFromSpace(point *, viewPort, viewDevice);
	
	/* transform library */
	viewPort GetTransformViewPort(transform);
	void SetTransformViewPort(transform, viewPort);
	void AddToTransformViewPort(transform target, viewPort);
	void SetShapeViewPort(shape, viewPort);
	viewPort GetShapeViewPort(shape);
	void SetDeepShapeViewPort(shape, viewPort);
	void SetDeepShapeViewPorts(shape, long count, const viewPort portList[]);
	void SetDeepShapeTransform(shape, transform);
	void SetDefaultViewPort(viewPort);
	viewGroup CopyViewGroup(viewGroup group);
	transform ChangeTransformViewGroup(transform xform, viewGroup oldGroup, viewGroup newGroup);
	shape ChangeShapeViewGroup(shape source, viewGroup oldGroup, viewGroup newGroup);
	transform SeparateShapeTransform(shape source);
	void ReuniteShapeTransform(shape target, transform separate);
	
	/* graphics state library */
	style SeparateShapeStyle(shape source);
	void ReuniteShapeStyle(shape target, style separate);
	ink SeparateShapeInk(shape source);
	void ReuniteShapeInk(shape target, ink separate);
	
	/* shape library */
	void GetPathsIndexPointControl(const paths *source, long index, point **pt, long **controlPtr, long *controlMask);
	void SetShapeOpenPath(shape target);
	void PreMapTransform(transform source, mapping *map);
	
	/* graphics library */
	#define CopyShape(s) CopyToShape(nil,s)
	#define CopyStyle(s) CopyToStyle(nil,s)
	#define CopyInk(s) CopyToInk(nil,s)
	#define CopyTransform(s) CopyToTransform(nil,s)
	void DisposeTransformAt(transform *target);
	void DisposeShapeAt(shape *target);
	void DisposeStyleAt(style *target);
	void DisposeInkAt(ink *target);
	void DisposeTagAt(tag *target);

	/* color matching library */
	colorProfile CreateQMSColorProfile(void);
	colorProfile CreateCanonColorProfile(void);
	
	/* color library */
	void InitCommonColors(void);
	void DisposeCommonColors(void);
	void SetShapeRGB(shape target, colorValue red, colorValue green, colorValue blue);
	void SetInkRGB(ink target, colorValue red, colorValue green, colorValue blue);
	void SetShapeHSV(shape target, colorValue hue, colorValue saturation, colorValue value);
	void SetInkHSV(ink target, colorValue hue, colorValue saturation, colorValue value);
	void SetShapeGray(shape target, colorValue gray);
	void SetInkGray(ink target, colorValue gray);
	
	/* transferMode library, color library, text library */
	void SetShapeCommonTransfer(shape target, commonTransferMode mode);
	void SetShapeCommonColor(shape target, commonColor color);
	void SetShapeCommonFace(shape target, commonFace face);

	void SetInkCommonTransfer(ink target, commonTransferMode mode);
	void SetInkCommonColor(ink target, commonColor);
	void SetStyleCommonFace(style target, commonFace);

	commonTransferMode GetInkCommonTransfer(ink source);
	commonColor GetInkCommonColor(ink source);
	commonFace GetStyleCommonFace(style source);

	commonTransferMode GetShapeCommonTransfer(shape source);
	commonColor GetShapeCommonColor(shape source);
	commonFace GetShapeCommonFace(shape source);

	color *SetCommonColor(color *target, commonColor);
	commonColor GetCommonColor(const color *source);

	/* color library */
	colorSpace GetShapeColorSpace(shape target);
	colorProfile GetShapeColorProfile(shape source);
	colorSet GetShapeColorSet(shape source);
	colorSpace GetInkColorSpace(ink target);
	colorProfile GetInkColorProfile(ink source);
	colorSet GetInkColorSet(ink source);
	
	void SetShapeColorSpace(shape target, colorSpace space);
	void SetShapeColorProfile(shape target, colorProfile profile);
	void SetShapeColorSet(shape target, colorSet set);
	void SetInkColorSpace(ink target, colorSpace space);
	void SetInkColorProfile(ink target, colorProfile profile);
	void SetInkColorSet(ink target, colorSet set);
	
	colorSet GetViewDeviceColorSet(viewDevice source);
	void SetViewDeviceColorSet(viewDevice target, colorSet set);
	colorProfile GetViewDeviceColorProfile(viewDevice source);
	void SetViewDeviceColorProfile(viewDevice target, colorProfile profile);

	long GetColorSpaceComponents(colorSpace space);

	/* transferMode library */
	void InitTransferMode(transferMode *);
	transferMode *SetCommonTransfer(transferMode *transferModeRecord, commonTransferMode mode, unsigned short opValue, const color *opColor);
	color *TransmogrifyColor(color *dstColor, const color *srcColor, const transferMode *);
	void SetInkFastXorTransfer(ink inky, viewDevice destDevice, color *background, color *result);
	void SetShapeFastXorTransfer(shape source, color *background, color *result);
	
	shape NewCString(const char *cString, const point *position);
	shape NewPString(const char *pascalString, const point *position);
	shape NewChar(const char theChar, const point *position);
	void SetCString(shape target, const char *cString, const point *position);
	void SetPString(shape target, const char *pascalString, const point *position);
	void SetChar(shape target, char theChar, const point *position);
	void DrawCString(const char *cString, const point *position);
	void DrawPString(const char *pascalString, const point *position);
	#ifndef qdTextIncludes
		void DrawChar(const char theChar, const point *position);
	#endif

	fixed FixTextWidth(const unsigned char *, short length);
	fixed FixCStringWidth(const char *);
	fixed FixPStringWidth(const char *);
	fixed FixCharWidth(char);
	point *GetShapeAdvance(shape target, point *advance);

	void SetGlyphText(shape, const unsigned char *text, long length);
	void SetGlyphAdvance(shape, const long advanceBits[]);
	void SetGlyphStyles(shape, const short styleRuns[], const style glyphStyles[]);

	long GetGlyphText(shape, unsigned char *text);
	long GetGlyphAdvance(shape, long advanceBits[]);
	long GetGlyphStyles(shape, short styleRuns[], style glyphStyles[]);

	/* mapping library */
	void PolyToPolyMap(const polygon *source, const polygon *dest, mapping *);
	
	/* shape library */
	void PaintRectangle(const rectangle *, commonColor);
	void PaintRectangle2(const point *, const point *, commonColor);
	void PaintRectangle4(long left, long top, long right, long bottom, commonColor);

	/* picture library */
	void AddToPicture(shape picture, shape newShape, style newStyle, ink newInk, transform newTransform);
	void InsertPictureItem(shape picture, long index, shape newShape, style newStyle, ink newInk, transform newTransform);
	shape GetPictureItem(shape picture, long index, shape *destShape, style *destStyle, ink *destInk, transform *destTransform);
	void SetPictureItem(shape picture, long index, shape newShape, style newStyle, ink newInk, transform newTransform);

	/* user library */
	void AddShapeUser(shape source, const void *data, long length, long type);
	long GetShapeUser(shape source, void *data, long *length, long requestedType, long *foundType, long index);
	void RemoveShapeUser(shape source, long type, long index);

	/* ramp library */
	shape NewRamp(const color *firstColor, const color *lastColor, long steps, const rectangle *bounds);
	void DrawRamp(const color *firstColor, const color *lastColor, long steps, const rectangle *bounds);
	shape NewCommonRamp(commonColor firstColor, commonColor lastColor, long steps, const rectangle *bounds);
	void DrawCommonRamp(commonColor firstColor, commonColor lastColor, long steps, const rectangle *bounds);

	/* spline library */
	#define circleSlack ((fract)0x3504f334)	/* if the points form a square, this slop will approximate a circle */
	enum { openSpline, closedSpline };		/* passed as the closed parameter */
	shape MirrorSpline(long count, point *points, fract slack, boolean closed);

	/* framing libarary */
	void OutsetShape(shape source, fixed distance);
	void FrameShape(shape source, fixed distance);

	#ifdef MacintoshIncludes
		#ifndef qdTextIncludes
			#undef DrawChar
		#endif
	#endif
#endif
