/* graphics:
	public interface definition
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Josh Horwich, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1993 Apple Computer, Inc.  All rights reserved.

	Thanks to Bruce Leak for taking many mushy ideas and making them consistent and logical.
	Thanks to Mike Reed for the Mathematica, Athens, conics, cubics and the tireless (but well-voiced) concerns.
	Thanks to Bill Atkinson, Ernie Beernink for the inspiration and the well-trodden path.
	Thanks to Jerome Coonen for the permission to do this in the first place.
	Thanks to Jim Batson and Gifford Calenda for not getting in the way.
	Thanks to Sampo Kaasila for the vector proofs, rotated parabolas and the font scaler.
	Thanks to Scott Knaster, Will Stein, Jim Friedlander, Gene Pope, Laurie Girand and a host of others for
		their unflagging support, without which this project would have been abandoned long ago.
	Thanks to Andrew Singer and Michael Kahl for the speedy development environment.

	The sorting routine is originally from Knuth, coded in assembly by B. Atkinson and then in C by Galyn Susman.
	The matrix routines and curve walking routines were originally coded by Sampo Kaasila.
	The polygon and path blitting edge walk was developed originally by Art Cabral.
	The halftone matrix was developed by Daniel I. Lipton.
	The color matching algorithms were developed by Robin Myers.
	The code was developed in C (not C++) and assembly in THINK C 1.0, 2.0, 3.0, 4.0, and 5.0, and MPW 3.0, 3.1 and 3.2.
*/

#pragma once

#ifndef graphicsRoutinesIncludes
#define graphicsRoutinesIncludes

#ifndef graphicsTypesIncludes
	#include "graphics types.h"
#endif
#ifndef graphicsErrorsIncludes
	#include "graphics errors.h"
#endif
#ifndef fontTypesIncludes
	#include "font types.h"
#endif
#ifndef memoryTypesIncludes
	#include "memory types.h"
#endif

#define DrawPicture gDrawPicture
#define DrawText gDrawText
#define GetPicture gGetPicture

#ifdef appleInternal
	#define InlineCode(x)
#endif
#ifndef InlineCode
	#define InlineCode(x)	= {0x303C, x, 0xA832}
#endif

#ifdef __cplusplus
extern "C" {
#endif

long CountGraphicsFunctions(const unsigned char function[]) InlineCode(0x300F);
void InsertGraphicsFunction(const unsigned char function[], long index, void *address, long identifier, long reference,
	viewDevice device) InlineCode(0x3010);
void *GetGraphicsFunction(const unsigned char function[], long index, long *identifier, long *reference,
	viewDevice *device) InlineCode(0x3011);
void SetGraphicsFunction(const unsigned char function[], long index, void *address, long identifier, long reference,
	viewDevice device) InlineCode(0x3012);
void RemoveGraphicsFunction(const unsigned char function[], long index) InlineCode(0x3013);

void *GetGraphicsFunctionAddress(const unsigned char function[]) InlineCode(0x3017);

long GetGraphicsFunctionReference(void) InlineCode(0x3014);
boolean GetGraphicsFunctionDone(long *result) InlineCode(0x3015);
void SetGraphicsFunctionDone(boolean isDone, long result) InlineCode(0x3016);

graphicsClient NewGraphicsClient(void *memoryStart, long memoryLength, long separateStack) InlineCode(0x3000);
graphicsClient GetGraphicsClient(void) InlineCode(0x3001);
void SetGraphicsClient(graphicsClient client) InlineCode(0x3002);
void DisposeGraphicsClient(graphicsClient client) InlineCode(0x3003);
heap *GetGraphicsClientHeap(graphicsClient client) InlineCode(0x3004);

long GetGraphicsClients(long index, long count, graphicsClient clients[]) InlineCode(0x3005);	/*returns the count */

void EnterGraphics(void) InlineCode(0x2000);
void ExitGraphics(void) InlineCode(0x1000);

graphicsError GetGraphicsError(graphicsError *stickyError) InlineCode(0x2001);
graphicsNotice GetGraphicsNotice(graphicsNotice *stickyNotice) InlineCode(0x2002);
graphicsWarning GetGraphicsWarning(graphicsWarning *stickyWarning) InlineCode(0x2003);
void PostGraphicsError(graphicsError error) InlineCode(0x2004);
void PostGraphicsNotice(graphicsNotice notice) InlineCode(0x2005);
void PostGraphicsWarning(graphicsWarning warning) InlineCode(0x2006);
userErrorFunction GetUserGraphicsError(long *reference) InlineCode(0x2007);
userNoticeFunction GetUserGraphicsNotice(long *reference) InlineCode(0x2008);
userWarningFunction GetUserGraphicsWarning(long *reference) InlineCode(0x2009);
void SetUserGraphicsError(userErrorFunction userFunction, long reference) InlineCode(0x200A);
void SetUserGraphicsNotice(userNoticeFunction userFunction, long reference) InlineCode(0x200B);
void SetUserGraphicsWarning(userWarningFunction userFunction, long reference) InlineCode(0x200C);

void IgnoreGraphicsNotice(graphicsNotice notice) InlineCode(0x200D);
void IgnoreGraphicsWarning(graphicsWarning warning) InlineCode(0x200E);
void PopGraphicsNotice(void) InlineCode(0x200F);
void PopGraphicsWarning(void) InlineCode(0x2010);

shape NewShapeVector(shapeType aType, const fixed vector[]) InlineCode(0x1001);
void SetShapeVector(shape target, const fixed vector[]) InlineCode(0x1002);

shape NewBitmap(const bitmap *data, const point *position) InlineCode(0x1003);
shape NewCurve(const curve *data) InlineCode(0x1004);
shape NewGlyphs(long charCount, const unsigned char text[], const point positions[], const long advance[], 
	const point tangents[], const short styleRuns[], const style glyphStyles[]) InlineCode(0x1005);
shape NewLine(const line *data) InlineCode(0x1006);
shape NewPaths(const paths *data) InlineCode(0x1007);
shape NewPicture(long count, const shape shapes[], const style styles[], const ink inks[], const transform transforms[]) InlineCode(0x1008);
shape NewPoint(const point *data) InlineCode(0x1009);
shape NewPolygons(const polygons *data) InlineCode(0x100A);
shape NewRectangle(const rectangle *data) InlineCode(0x100B);
shape NewText(long charCount, const unsigned char text[], const point *position) InlineCode(0x100C);

bitmap *GetBitmap(shape source, bitmap *data, point *position) InlineCode(0x100D);
curve *GetCurve(shape source, curve *data) InlineCode(0x100E);
long GetGlyphs(shape source, long *charCount, unsigned char text[], point positions[], long advance[], point tangents[],
	long *runCount, short styleRuns[], style glyphStyles[]) InlineCode(0x100F);					/* returns byte length of glyphs */
line *GetLine(shape source, line *data) InlineCode(0x1010);
long GetPaths(shape source, paths *data) InlineCode(0x1011);									/* returns byte length */
long GetPicture(shape source, shape shapes[], style styles[], ink inks[], transform transforms[]) InlineCode(0x1012);	/* returns count */
point *GetPoint(shape source, point *data) InlineCode(0x1013);
long GetPolygons(shape source, polygons *data) InlineCode(0x1014);							/* returns byte length */
rectangle *GetRectangle(shape source, rectangle *data) InlineCode(0x1015);
long GetText(shape source, long *charCount, unsigned char text[], point *position) InlineCode(0x1016);		/* returns byte length */

void SetBitmap(shape target, const bitmap *data, const point *position) InlineCode(0x1017);
void SetCurve(shape target, const curve *data) InlineCode(0x1018);
void SetGlyphs(shape target, long charCount, const unsigned char text[], const point positions[], const long advance[], 
	const point tangents[], const short styleRuns[], const style glyphStyles[]) InlineCode(0x1019);
void SetLine(shape target, const line *data) InlineCode(0x101A);
void SetPaths(shape target, const paths *data) InlineCode(0x101B);
void SetPicture(shape target, long count, const shape shapes[], const style styles[], const ink inks[], const transform transforms[]) InlineCode(0x101C);
void SetPoint(shape target, const point *data) InlineCode(0x101D);
void SetPolygons(shape target, const polygons *data) InlineCode(0x101E);
void SetRectangle(shape target, const rectangle *data) InlineCode(0x101F);
void SetText(shape target, long charCount, const unsigned char text[], const point *position) InlineCode(0x1020);

void DrawBitmap(const bitmap *data, const point *position) InlineCode(0x1021);
void DrawCurve(const curve *data) InlineCode(0x1022);
void DrawGlyphs(long charCount, const unsigned char text[], const point positions[], const long advance[], 
	const point tangents[], const short styleRuns[], const style glyphStyles[]) InlineCode(0x1023);
void DrawLine(const line *data) InlineCode(0x1024);
void DrawPaths(const paths *data, shapeFill fill) InlineCode(0x1025);
void DrawPicture(long count, const shape shapes[], const style styles[], const ink inks[], const transform transforms[]) InlineCode(0x1026);
void DrawPoint(const point *data) InlineCode(0x1027);
void DrawPolygons(const polygons *data, shapeFill fill) InlineCode(0x1028);
void DrawRectangle(const rectangle *data, shapeFill fill) InlineCode(0x1029);
void DrawText(long charCount, const unsigned char text[], const point *position) InlineCode(0x102A);

colorProfile NewColorProfile(const profileRecord *profile, const profileResponse *responses) InlineCode(0x102B);
colorSet NewColorSet(colorSpace space, long count, const setColor colors[]) InlineCode(0x102C);
ink NewInk(void) InlineCode(0x102D);
shape NewShape(shapeType aType) InlineCode(0x102E);
style NewStyle(void) InlineCode(0x102F);
tag NewTag(long tagType, long length, const void *data) InlineCode(0x1030);
transform NewTransform(void) InlineCode(0x1031);
viewDevice NewViewDevice(viewGroup group, shape bitmapShape) InlineCode(0x1062);
viewGroup NewViewGroup(void) InlineCode(0x1060);
viewPort NewViewPort(viewGroup group) InlineCode(0x1063);

void DisposeColorProfile(colorProfile target) InlineCode(0x1032);
void DisposeColorSet(colorSet target) InlineCode(0x1033);
void DisposeInk(ink target) InlineCode(0x1034);
void DisposeShape(shape target) InlineCode(0x1035);
void DisposeStyle(style target) InlineCode(0x1036);
void DisposeTag(tag target) InlineCode(0x1037);
void DisposeTransform(transform target) InlineCode(0x1038);
void DisposeViewDevice(viewDevice target) InlineCode(0x1064);
void DisposeViewGroup(viewGroup target) InlineCode(0x1061);
void DisposeViewPort(viewPort target) InlineCode(0x1065);

colorProfile CloneColorProfile(colorProfile source) InlineCode(0x1039);
colorSet CloneColorSet(colorSet source) InlineCode(0x103A);
ink CloneInk(ink source) InlineCode(0x103B);
shape CloneShape(shape source) InlineCode(0x103C);
style CloneStyle(style source) InlineCode(0x103D);
tag CloneTag(tag source) InlineCode(0x103E);
transform CloneTransform(transform source) InlineCode(0x103F);

colorProfile CopyToColorProfile(colorProfile target, colorProfile source) InlineCode(0x1040);
colorSet CopyToColorSet(colorSet target, colorSet source) InlineCode(0x1041);
ink CopyToInk(ink target, ink source) InlineCode(0x1042);
shape CopyToShape(shape target, shape source) InlineCode(0x1043);
style CopyToStyle(style target, style source) InlineCode(0x1044);
tag CopyToTag(tag target, tag source) InlineCode(0x1045);
transform CopyToTransform(transform target, transform source) InlineCode(0x1046);
viewDevice CopyToViewDevice(viewDevice target, viewDevice source) InlineCode(0x1066);
viewPort CopyToViewPort(viewPort target, viewPort source) InlineCode(0x1067);

boolean EqualColorProfile(colorProfile one, colorProfile two) InlineCode(0x1047);
boolean EqualColorSet(colorSet one, colorSet two) InlineCode(0x1048);
boolean EqualInk(ink one, ink two) InlineCode(0x1049);
boolean EqualShape(shape one, shape two) InlineCode(0x104A);
boolean EqualStyle(style one, style two) InlineCode(0x104B);
boolean EqualTag(tag one, tag two) InlineCode(0x104C);
boolean EqualTransform(transform one, transform two) InlineCode(0x104D);
boolean EqualViewDevice(viewDevice one, viewDevice two) InlineCode(0x1068);
boolean EqualViewPort(viewPort one, viewPort two) InlineCode(0x1069);

void ResetInk(ink target) InlineCode(0x104E);
void ResetShape(shape target) InlineCode(0x104F);
void ResetStyle(style target) InlineCode(0x1050);
void ResetTransform(transform target) InlineCode(0x1051);

void LoadColorProfile(colorProfile target) InlineCode(0x1052);
void LoadColorSet(colorSet target) InlineCode(0x1053);
void LoadInk(ink target) InlineCode(0x1054);
void LoadShape(shape target) InlineCode(0x1055);
void LoadStyle(style target) InlineCode(0x1056);
void LoadTag(tag target) InlineCode(0x1057);
void LoadTransform(transform target) InlineCode(0x1058);

void UnloadColorProfile(colorProfile target) InlineCode(0x1059);
void UnloadColorSet(colorSet target) InlineCode(0x105A);
void UnloadInk(ink target) InlineCode(0x105B);
void UnloadShape(shape target) InlineCode(0x105C);
void UnloadStyle(style target) InlineCode(0x105D);
void UnloadTag(tag target) InlineCode(0x105E);
void UnloadTransform(transform target) InlineCode(0x105F);

void CacheShape(shape source) InlineCode(0x106B);
shape CopyDeepToShape(shape target, shape source) InlineCode(0x106A);
void DrawShape(shape source) InlineCode(0x106C);
void DisposeShapeCache(shape target) InlineCode(0x106D);

colorProfile GetDefaultColorProfile(void) InlineCode(0x121B);
shape GetDefaultShape(shapeType aType) InlineCode(0x106E);
colorSet GetDefaultColorSet( long pixelDepth ) InlineCode(0x1256);


void SetDefaultColorProfile(colorProfile target) InlineCode(0x121C);
void SetDefaultShape(shape target) InlineCode(0x106F);
void SetDefaultColorSet( colorSet target, long pixelDepth ) InlineCode(0x1257);

long GetTag(tag source, long *tagType, void *data) InlineCode(0x1070);
void SetTag(tag target, long tagType, long length, const void *data) InlineCode(0x1071);

rectangle *GetShapeBounds(shape source, long index, rectangle *bounds) InlineCode(0x1072);
shapeFill GetShapeFill(shape source) InlineCode(0x1073);
ink GetShapeInk(shape source) InlineCode(0x1074);
long GetShapePixel(shape source, long x, long y, color *data, long *index) InlineCode(0x1075);
style GetShapeStyle(shape source) InlineCode(0x1076);
transform GetShapeTransform(shape source) InlineCode(0x1077);
shapeType GetShapeType(shape source) InlineCode(0x1078);
rectangle *GetShapeTypographicBounds(shape source, rectangle *bounds) InlineCode(0x124E);
shape GetBitmapParts(shape source, const longRectangle *bounds) InlineCode(0x1079);
void GetStyleFontMetrics(style sourceStyle, point* before, point* after, point* caretAngle, point* caretOffset) InlineCode(0x1258);
void GetShapeFontMetrics(shape source, point* before, point* after, point* caretAngle, point* caretOffset) InlineCode(0x1259);

void SetShapeBounds(shape target, const rectangle *newBounds) InlineCode(0x107A);
void SetShapeFill(shape target, shapeFill newFill) InlineCode(0x107B);
void SetShapeInk(shape target, ink newInk) InlineCode(0x107C);
void SetShapePixel(shape target, long x, long y, const color *newColor, long newIndex) InlineCode(0x107D);
void SetShapeStyle(shape target, style newStyle) InlineCode(0x107E);
void SetShapeTransform(shape target, transform newTransform) InlineCode(0x107F);
void SetShapeType(shape target, shapeType newType) InlineCode(0x1080);
void SetBitmapParts(shape target, const longRectangle *bounds, shape bitmapShape) InlineCode(0x1081);

void SetShapeGeometry(shape target, shape geometry) InlineCode(0x1082);

fixed GetShapeCurveError(shape source) InlineCode(0x1083);
dashRecord *GetShapeDash(shape source, dashRecord *dash) InlineCode(0x1084);
capRecord *GetShapeCap(shape source, capRecord *cap) InlineCode(0x1085);
long GetShapeFace(shape source, textFace *face) InlineCode(0x1086);			/* returns the number of layers */
font GetShapeFont(shape source) InlineCode(0x1087);
joinRecord *GetShapeJoin(shape source, joinRecord *join) InlineCode(0x1088);
fract GetShapeJustification(shape source) InlineCode(0x1089);
patternRecord *GetShapePattern(shape source, patternRecord *pattern) InlineCode(0x108A);
fixed GetShapePen(shape source) InlineCode(0x108B);
fontPlatform GetShapeEncoding(shape source, fontScript *script, fontLanguage *language) InlineCode(0x108C);
fixed GetShapeTextSize(shape source) InlineCode(0x108E);
long GetShapeFontVariations(shape source, fontVariation variations[]) InlineCode(0x108F);

fixed GetStyleCurveError(style source) InlineCode(0x1090);
dashRecord *GetStyleDash(style source, dashRecord *dash) InlineCode(0x1091);
capRecord *GetStyleCap(style source, capRecord *cap) InlineCode(0x1092);
long GetStyleFace(style source, textFace *face) InlineCode(0x1093);			/* returns the number of layers */
font GetStyleFont(style source) InlineCode(0x1094);
joinRecord *GetStyleJoin(style source, joinRecord *join) InlineCode(0x1095);
fract GetStyleJustification(style source) InlineCode(0x1096);
patternRecord *GetStylePattern(style source, patternRecord *pattern) InlineCode(0x1097);
fixed GetStylePen(style source) InlineCode(0x1098);
fontPlatform GetStyleEncoding(style source, fontScript *script, fontLanguage *language) InlineCode(0x1099);
fixed GetStyleTextSize(style source) InlineCode(0x109B);
long GetStyleFontVariations(style source, fontVariation variations[]) InlineCode(0x109C);

void SetShapeCurveError(shape target, fixed error) InlineCode(0x109D);
void SetShapeDash(shape target, const dashRecord *dash) InlineCode(0x109E);
void SetShapeCap(shape target, const capRecord *cap) InlineCode(0x109F);
void SetShapeFace(shape target, const textFace *face) InlineCode(0x10A0);
void SetShapeFont(shape target, font aFont) InlineCode(0x10A1);
void SetShapeJoin(shape target, const joinRecord *join) InlineCode(0x10A2);
void SetShapeJustification(shape target, fract justify) InlineCode(0x10A3);
void SetShapePattern(shape target, const patternRecord *pattern) InlineCode(0x10A4);
void SetShapePen(shape target, fixed pen) InlineCode(0x10A5);
void SetShapeEncoding(shape target, fontPlatform platform, fontScript script, fontLanguage language) InlineCode(0x10A6);
void SetShapeTextSize(shape target, fixed size) InlineCode(0x10A8);
void SetShapeFontVariations(shape target, long count, const fontVariation variations[]) InlineCode(0x10A9);

void SetStyleCurveError(style target, fixed error) InlineCode(0x10AA);
void SetStyleDash(style target, const dashRecord *dash) InlineCode(0x10AB);
void SetStyleCap(style target, const capRecord *cap) InlineCode(0x10AC);
void SetStyleFace(style target, const textFace *face) InlineCode(0x10AD);
void SetStyleFont(style target, font aFont) InlineCode(0x10AE);
void SetStyleJoin(style target, const joinRecord *join) InlineCode(0x10AF);
void SetStyleJustification(style target, fract justify) InlineCode(0x10B0);
void SetStylePattern(style target, const patternRecord *pattern) InlineCode(0x10B1);
void SetStylePen(style target, fixed pen) InlineCode(0x10B2);
void SetStyleEncoding(style target, fontPlatform platform, fontScript script, fontLanguage language) InlineCode(0x10B3);
void SetStyleTextSize(style target, fixed size) InlineCode(0x10B5);
void SetStyleFontVariations(style target, long count, const fontVariation variations[]) InlineCode(0x10B6);

color *GetShapeColor(shape source, color *data) InlineCode(0x10B7);
transferMode *GetShapeTransfer(shape source, transferMode *data) InlineCode(0x10B8);

color *GetInkColor(ink source, color *data) InlineCode(0x10B9);
transferMode *GetInkTransfer(ink source, transferMode *data) InlineCode(0x10BA);

void SetShapeColor(shape target, const color *data) InlineCode(0x10BB);
void SetShapeTransfer(shape target, const transferMode *data) InlineCode(0x10BC);

void SetInkColor(ink target, const color *data) InlineCode(0x10BD);
void SetInkTransfer(ink target, const transferMode *data) InlineCode(0x10BE);

shape GetShapeClip(shape source) InlineCode(0x10BF);
mapping *GetShapeMapping(shape source, mapping *map) InlineCode(0x10C0);
shapePart GetShapeHitTest(shape source, fixed *tolerance) InlineCode(0x10C1);
long GetShapeViewPorts(shape source, viewPort list[]) InlineCode(0x10C2);

shape GetTransformClip(transform source) InlineCode(0x10C3);
mapping *GetTransformMapping(transform source, mapping *map) InlineCode(0x10C4);
shapePart GetTransformHitTest(transform source, fixed *tolerance) InlineCode(0x10C5);
long GetTransformViewPorts(transform source, viewPort list[]) InlineCode(0x10C6);

void SetShapeClip(shape target, shape clip) InlineCode(0x10C7);
void SetShapeMapping(shape target, const mapping *map) InlineCode(0x10C8);
void SetShapeHitTest(shape target, shapePart mask, fixed tolerance) InlineCode(0x10C9);
void SetShapeViewPorts(shape target, long count, const viewPort list[]) InlineCode(0x10CA);

void SetTransformClip(transform target, shape clip) InlineCode(0x10CB);
void SetTransformMapping(transform target, const mapping *map) InlineCode(0x10CC);
void SetTransformHitTest(transform target, shapePart mask, fixed tolerance) InlineCode(0x10CD);
void SetTransformViewPorts(transform target, long count, const viewPort list[]) InlineCode(0x10CE);

long GetColorSet(colorSet source, colorSpace *space, setColor colors[]) InlineCode(0x10CF);
long GetColorProfile(colorProfile source, profileRecord *profile, profileResponse *responses) InlineCode(0x10D0);

void SetColorSet(colorSet target, colorSpace space, long count, const setColor colors[]) InlineCode(0x10D1);
void SetColorProfile(colorProfile target, const profileRecord *profile, const profileResponse *responses) InlineCode(0x10D2);

shape GetViewDeviceBitmap(viewDevice source) InlineCode(0x10D3);
shape GetViewDeviceClip(viewDevice source) InlineCode(0x10D4);
mapping *GetViewDeviceMapping(viewDevice source, mapping *map) InlineCode(0x10D5);
viewGroup GetViewDeviceViewGroup(viewDevice source) InlineCode(0x10D6);

void SetViewDeviceBitmap(viewDevice target, shape bitmapShape) InlineCode(0x10D7);
void SetViewDeviceClip(viewDevice target, shape clip) InlineCode(0x10D8);
void SetViewDeviceMapping(viewDevice target, const mapping *map) InlineCode(0x10D9);
void SetViewDeviceViewGroup(viewDevice target, viewGroup group) InlineCode(0x10DA);

long GetViewPortChildren(viewPort source, viewPort list[]) InlineCode(0x10DB);
shape GetViewPortClip(viewPort source) InlineCode(0x10DC);
long GetViewPortDither(viewPort source) InlineCode(0x10DD);
boolean GetViewPortHalftone(viewPort source, halftone *data) InlineCode(0x10DE);
mapping *GetViewPortMapping(viewPort source, mapping *map) InlineCode(0x10DF);
viewPort GetViewPortParent(viewPort source) InlineCode(0x10E0);
viewGroup GetViewPortViewGroup(viewPort source) InlineCode(0x10E1);

void SetViewPortChildren(viewPort target, long count, const viewPort list[]) InlineCode(0x10E2);
void SetViewPortClip(viewPort target, shape clip) InlineCode(0x10E3);
void SetViewPortDither(viewPort target, long level) InlineCode(0x10E4);
void SetViewPortHalftone(viewPort target, const halftone *data) InlineCode(0x10E5);
void SetViewPortMapping(viewPort target, const mapping *map) InlineCode(0x10E6);
void SetViewPortParent(viewPort target, viewPort parent) InlineCode(0x10E7);
void SetViewPortViewGroup(viewPort target, viewGroup group) InlineCode(0x10E8);

long GetColorProfileTags(colorProfile source, long tagType, long index, long count, tag items[]) InlineCode(0x10E9);
long GetColorSetTags(colorSet source, long tagType, long index, long count, tag items[]) InlineCode(0x10EA);
long GetInkTags(ink source, long tagType, long index, long count, tag items[]) InlineCode(0x10EB);
long GetShapeTags(shape source, long tagType, long index, long count, tag items[]) InlineCode(0x10EC);
long GetStyleTags(style source, long tagType, long index, long count, tag items[]) InlineCode(0x10ED);
long GetTransformTags(transform source, long tagType, long index, long count, tag items[]) InlineCode(0x10EE);
long GetViewDeviceTags(viewDevice source, long tagType, long index, long count, tag items[]) InlineCode(0x10EF);
long GetViewPortTags(viewPort source, long tagType, long index, long count, tag items[]) InlineCode(0x10F0);

void SetColorProfileTags(colorProfile target, long tagType, long index, long oldCount, long newCount, const tag items[]) InlineCode(0x10F1);
void SetColorSetTags(colorSet target, long tagType, long index, long oldCount, long newCount, const tag items[]) InlineCode(0x10F2);
void SetInkTags(ink target, long tagType, long index, long oldCount, long newCount, const tag items[]) InlineCode(0x10F3);
void SetShapeTags(shape target, long tagType, long index, long oldCount, long newCount, const tag items[]) InlineCode(0x10F4);
void SetStyleTags(style target, long tagType, long index, long oldCount, long newCount, const tag items[]) InlineCode(0x10F5);
void SetTransformTags(transform target, long tagType, long index, long oldCount, long newCount, const tag items[]) InlineCode(0x10F6);
void SetViewDeviceTags(viewDevice target, long tagType, long index, long oldCount, long newCount, const tag items[]) InlineCode(0x10F7);
void SetViewPortTags(viewPort target, long tagType, long index, long oldCount, long newCount, const tag items[]) InlineCode(0x10F8);

profileAttribute GetColorProfileAttributes(colorProfile source) InlineCode(0x1229);
inkAttribute GetInkAttributes(ink source) InlineCode(0x10F9);
shapeAttribute GetShapeAttributes(shape source) InlineCode(0x10FA);
inkAttribute GetShapeInkAttributes(shape source) InlineCode(0x10FB);
styleAttribute GetShapeStyleAttributes(shape source) InlineCode(0x10FC);
styleAttribute GetStyleAttributes(style source) InlineCode(0x10FD);
textAttribute GetShapeTextAttributes(shape source) InlineCode(0x122B);
textAttribute GetStyleTextAttributes(style source) InlineCode(0x122C);
deviceAttribute GetViewDeviceAttributes(viewDevice source) InlineCode(0x10FF);
portAttribute GetViewPortAttributes(viewPort source) InlineCode(0x1100);

void SetColorProfileAttributes(colorProfile target, profileAttribute attributes) InlineCode(0x122a);
void SetInkAttributes(ink target, inkAttribute attributes) InlineCode(0x1101);
void SetShapeAttributes(shape target, shapeAttribute attributes) InlineCode(0x1102);
void SetShapeInkAttributes(shape target, inkAttribute attributes) InlineCode(0x1103);
void SetShapeStyleAttributes(shape target, styleAttribute attributes) InlineCode(0x1104);
void SetStyleAttributes(style target, styleAttribute attributes) InlineCode(0x1105);
void SetShapeTextAttributes(shape target, textAttribute attributes) InlineCode(0x122E);
void SetStyleTextAttributes(style target, textAttribute attributes) InlineCode(0x122D);
void SetViewDeviceAttributes(viewDevice target, deviceAttribute attributes) InlineCode(0x1107);
void SetViewPortAttributes(viewPort target, portAttribute attributes) InlineCode(0x1108);

long GetColorProfileOwners(colorProfile source) InlineCode(0x1109);
long GetColorSetOwners(colorSet source) InlineCode(0x110A);
long GetInkOwners(ink source) InlineCode(0x110B);
long GetShapeOwners(shape source) InlineCode(0x110C);
long GetStyleOwners(style source) InlineCode(0x110D);
long GetTagOwners(tag source) InlineCode(0x110E);
long GetTransformOwners(transform source) InlineCode(0x110F);

void LockShape(shape target) InlineCode(0x1110);
void LockTag(tag target) InlineCode(0x1111);
void UnlockShape(shape target) InlineCode(0x1112);
void UnlockTag(tag target) InlineCode(0x1113);
void *GetShapeStructure(shape source, long *length) InlineCode(0x1114);
void *GetTagStructure(tag source, long *length) InlineCode(0x1115);

fixed GetColorDistance(const color *target, const color *source) InlineCode(0x1116);
point *ShapeLengthToPoint(shape target, long index, fixed length, point *location, point *tangent) InlineCode(0x1117);

wide *GetShapeArea(shape source, long index, wide *area) InlineCode(0x1118);
long GetShapeCacheSize(shape source) InlineCode(0x1119);
point *GetShapeCenter(shape source, long index, point *center) InlineCode(0x111A);
contourDirection GetShapeDirection(shape source, long contour) InlineCode(0x111B);
long GetShapeIndex(shape source, long contour, long vector) InlineCode(0x111C);
wide *GetShapeLength(shape source, long index, wide *length) InlineCode(0x111D);
long GetShapeSize(shape source) InlineCode(0x111E);

long CountShapeContours(shape source) InlineCode(0x111F);
long CountShapePoints(shape source, long contour) InlineCode(0x1120);

long GetShapeDashPositions(shape source, mapping dashMappings[]) InlineCode(0x1121);		/* returns the number of positions */
long GetShapeDeviceArea(shape source, viewPort port, viewDevice device) InlineCode(0x1122);
boolean GetShapeDeviceBounds(shape source, viewPort port, viewDevice device, rectangle *bounds) InlineCode(0x1123);
colorSet GetShapeDeviceColors(shape source, viewPort port, viewDevice device, long *width) InlineCode(0x1124);
boolean GetShapeGlobalBounds(shape source, viewPort port, viewGroup group, rectangle *bounds) InlineCode(0x1125);
long GetShapeGlobalViewDevices(shape source, viewPort port, viewDevice list[]) InlineCode(0x1126);
long GetShapeGlobalViewPorts(shape source, viewPort list[]) InlineCode(0x1127);
rectangle *GetShapeLocalBounds(shape source, rectangle *bounds) InlineCode(0x1128);
long GetShapePatternPositions(shape source, point positions[]) InlineCode(0x1129);		/* returns the number of positions */
void GetShapeLocalFontMetrics(shape sourceShape, point* before, point* after, point* caretAngle, point* caretOffset) InlineCode(0x125A);
void GetShapeDeviceFontMetrics(shape sourceShape, viewPort port, viewDevice device, point* before, point* after, point* caretAngle, point* caretOffset) InlineCode(0x125B);

long GetViewGroupViewDevices(viewGroup source, viewDevice list[]) InlineCode(0x112A);
long GetViewGroupViewPorts(viewGroup source, viewPort list[]) InlineCode(0x112B);

mapping *GetViewPortGlobalMapping(viewPort source, mapping *map) InlineCode(0x112C);
long GetViewPortViewDevices(viewPort source, viewDevice list[]) InlineCode(0x112D);

shape HitTestPicture(shape target, const point *test, hitTestInfo *result, long level, long depth) InlineCode(0x112F);

boolean IntersectRectangle(rectangle *target, const rectangle *source, const rectangle *operand) InlineCode(0x1130);
rectangle *UnionRectangle(rectangle *target, const rectangle *source, const rectangle *operand) InlineCode(0x1131);

boolean TouchesRectanglePoint(const rectangle *target, const point *test) InlineCode(0x1132);
boolean TouchesShape(shape target, shape test) InlineCode(0x1133);
boolean TouchesBoundsShape(const rectangle *target, shape test) InlineCode(0x1134);

boolean ContainsRectangle(const rectangle *container, const rectangle *test) InlineCode(0x1135);
boolean ContainsShape(shape container, shape test) InlineCode(0x1136);
boolean ContainsBoundsShape(const rectangle *container, shape test, long index) InlineCode(0x1137);

color *ConvertColor(color *target, colorSpace space, colorSet aSet, colorProfile profile) InlineCode(0x1138);
color *CombineColor(color *target, ink operand) InlineCode(0x1139);

boolean CheckColor(const color *source, colorSpace space, colorSet aSet, colorProfile profile) InlineCode(0x120B);
shape CheckBitmapColor(shape source, rectangle *area, colorSpace space, colorSet aSet, colorProfile profile) InlineCode(0x120C);

fixed GetHalftoneDeviceAngle(viewDevice source, const halftone *data) InlineCode(0x120D);

long GetColorSetParts(colorSet source, long index, long count, colorSpace *space, setColor data[]) InlineCode(0x113A);
long GetGlyphParts(shape source, long index, long charCount, long *byteLength, unsigned char text[], point positions[],
	long advanceBits[], point tangents[], long *runCount, short styleRuns[], style styles[]) InlineCode(0x113B);	/* returns the glyph count */
long GetPathParts(shape source, long index, long count, paths *data) InlineCode(0x113C);
long GetPictureParts(shape source, long index, long count, shape shapes[], style styles[], ink inks[], transform transforms[]) InlineCode(0x113D);
long GetPolygonParts(shape source, long index, long count, polygons *data) InlineCode(0x113E);
shape GetShapeParts(shape source, long index, long count, shape destination) InlineCode(0x113F);
long GetTextParts(shape source, long index, long charCount, unsigned char text[]) InlineCode(0x1140);

void SetColorSetParts(colorSet target, long index, long oldCount, long newCount, const setColor data[]) InlineCode(0x1141);
void SetGlyphParts(shape source, long index, long oldCharCount, long newCharCount, const unsigned char text[], 
	const point positions[], const long advanceBits[], const point tangents[], const short styleRuns[], const style styles[]) InlineCode(0x1142);
void SetPathParts(shape target, long index, long count, const paths *data, editShapeFlag flags) InlineCode(0x1143);
void SetPictureParts(shape target, long index, long oldCount, long newCount, const shape shapes[],
	const style styles[], const ink inks[], const transform transforms[]) InlineCode(0x1144);
void SetPolygonParts(shape target, long index, long count, const polygons *data, editShapeFlag flags) InlineCode(0x1145);
void SetShapeParts(shape target, long index, long count, shape insert, editShapeFlag flags) InlineCode(0x1146);
void SetTextParts(shape target, long index, long oldCharCount, long newCharCount, const unsigned char text[]) InlineCode(0x1147);

long GetShapePoints(shape source, long index, long count, point data[]) InlineCode(0x1148);
void SetShapePoints(shape target, long index, long count, const point data[]) InlineCode(0x1149);

long GetGlyphPositions(shape source, long index, long charCount, long advance[], point positions[]) InlineCode(0x114A);
long GetGlyphTangents(shape source, long index, long charCount, point tangents[]) InlineCode(0x114B);
void SetGlyphPositions(shape target, long index, long charCount, const long advance[], const point positions[]) InlineCode(0x114C);
void SetGlyphTangents(shape target, long index, long charCount, const point tangents[]) InlineCode(0x114D);

long GetGlyphMetrics(shape source, point advances[], rectangle boundingBoxes[], point sideBearings[]) InlineCode(0x1150);

void DifferenceShape(shape target, shape operand) InlineCode(0x1151);
void ExcludeShape(shape target, shape operand) InlineCode(0x1152);
void IntersectShape(shape target, shape operand) InlineCode(0x1153);
void MapShape(shape target, const mapping *map) InlineCode(0x1154);
void MoveShape(shape target, fixed deltaX, fixed deltaY) InlineCode(0x1155);
void MoveShapeTo(shape target, fixed x, fixed y) InlineCode(0x1156);
void ReverseDifferenceShape(shape target, shape operand) InlineCode(0x1157);
void RotateShape(shape target, fixed degrees, fixed xOffset, fixed yOffset) InlineCode(0x1158);
void ScaleShape(shape target, fixed hScale, fixed vScale, fixed xOffset, fixed yOffset) InlineCode(0x1159);
void SkewShape(shape target, fixed xSkew, fixed ySkew, fixed xOffset, fixed yOffset) InlineCode(0x115A);
void UnionShape(shape target, shape operand) InlineCode(0x115B);

void DifferenceTransform(transform target, shape operand) InlineCode(0x115C);
void ExcludeTransform(transform target, shape operand) InlineCode(0x115D);
void IntersectTransform(transform target, shape operand) InlineCode(0x115E);
void MapTransform(transform target, const mapping *map) InlineCode(0x115F);
void MoveTransform(transform target, fixed deltaX, fixed deltaY) InlineCode(0x1160);
void MoveTransformTo(transform target, fixed x, fixed y) InlineCode(0x1161);
void ReverseDifferenceTransform(transform target, shape operand) InlineCode(0x1162);
void RotateTransform(transform target, fixed degrees, fixed xOffset, fixed yOffset) InlineCode(0x1163);
void ScaleTransform(transform target, fixed hScale, fixed vScale, fixed xOffset, fixed yOffset) InlineCode(0x1164);
void SkewTransform(transform target, fixed xSkew, fixed ySkew, fixed xOffset, fixed yOffset) InlineCode(0x1165);
void UnionTransform(transform target, shape operand) InlineCode(0x1166);

void BreakShape(shape target, long index) InlineCode(0x1167);
void ChangedShape(shape target) InlineCode(0x1168);
shapePart HitTestShape(shape target, const point *test, hitTestInfo *result) InlineCode(0x1169);
shape HitTestDevice(shape target, viewPort port, viewDevice device, const point *test, const point *tolerance) InlineCode(0x116A);
void InsetShape(shape target, fixed inset) InlineCode(0x116B);
void InvertShape(shape target) InlineCode(0x116C);
void PrimitiveShape(shape target) InlineCode(0x116D);
void ReduceShape(shape target, long contour) InlineCode(0x116E);
void ReverseShape(shape target, long contour) InlineCode(0x116F);
void SimplifyShape(shape target) InlineCode(0x1170);

void FlattenShape(shape source, flattenFlag flags, struct spoolBlock *block) InlineCode(0x1171);
shape UnflattenShape(struct spoolBlock *block, long count, const viewPort portList[]) InlineCode(0x1172);

#ifdef __cplusplus
}
#endif

#undef InlineCode


#ifdef MacintoshIncludes
	#undef DrawPicture
	#undef DrawText
	#undef GetPicture
#endif

#endif
