/* graphics:
	graphics stream types
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1993 Apple Computer, Inc.  All rights reserved.
*/

#ifndef graphicsStreamTypesIncludes
	#define graphicsStreamTypesIncludes
	
	#ifndef fontTypesIncludes
		#include "font types.h"
	#endif

	#ifndef graphicsTypesIncludes
		#include "graphics types.h"
	#endif

#define layoutStyleRuns

	typedef enum {
		newObjectOpcode			= 0x00,		/* create new object */
		setDataOpcode				= 0x40,		/* add reference to current object */
		setDefaultOpcode			= 0x80,		/* replace current default with this object */
		reservedOpcode			= 0xC0,		/* (may be used in future expansion) */
		nextOpcode				= 0xFF		/* used by currentOperand field to say opcode is coming */
	} graphicsOperationOpcode;
	
/* new object types (new object opcode) */
	typedef enum {	
		headerTypeOpcode			= 0x00,	/* byte following new object uses bottom 6 bits for type */
			/* shape types use values 1 (emptyType) through 13 (pictureType) */
		styleTypeOpcode			= 0x28,
		inkTypeOpcode,
		transformTypeOpcode,		
		colorProfileTypeOpcode,
		colorSetTypeOpcode,
		tagTypeOpcode,
		bitImageOpcode,
		fontNameTypeOpcode,
		trailerTypeOpcode			= 0x3F
	} graphicsNewOpcode;
	
/* fields of objects (set data opcodes) */

	typedef enum {
		shapeAttributesOpcode,
		shapeTagOpcode,
		shapeFillOpcode
	} shapeDataOpcode;
	
	typedef enum {
		omitPathPositionXMask 			= 0xC0,
		omitPathPositionYMask 			= 0x30,
		omitPathDeltaXMask 				= 0x0C,
		omitPathDeltaYMask 				= 0x03
	} omitPathMask;
	
	typedef enum {
		omitPathPositionXShift 			= 6,
		omitPathPositionYShift 			= 4,
		omitPathDeltaXShift 				= 2,
		omitPathDeltaYShift 				= 0
	} omitPathShift;
	
	typedef enum {
		omitBitmapImageMask 			= 0xC0,
		omitBitmapWidthMask 			= 0x30,
		omitBitmapHeightMask 			= 0x0C,
		omitBitmapRowBytesMask 		= 0x03
	} omitBitmapMask1;
	
	typedef enum {
		omitBitmapImageShift 			= 6,
		omitBitmapWidthShift 			= 4,
		omitBitmapHeightShift 			= 2,
		omitBitmapRowBytesShift 		= 0
	} omitBitmapShift1;
	
	typedef enum {
		omitBitmapPixelSizeMask 			= 0xC0,
		omitBitmapSpaceMask 			= 0x30,
		omitBitmapSetMask 				= 0x0C,
		omitBitmapProfileMask 			= 0x03
	} omitBitmapMask2;
	
	typedef enum {
		omitBitmapPixelSizeShift 			= 6,
		omitBitmapSpaceShift 			= 4,
		omitBitmapSetShift 				= 2,
		omitBitmapProfileShift 			= 0
	} omitBitmapShift2;
	
	typedef enum {
		omitBitmapPositionXMask 			= 0xC0,
		omitBitmapPositionYMask 			= 0x30
	} omitBitmapMask3;
	
	typedef enum {
		omitBitmapPositionXShift 			= 6,
		omitBitmapPositionYShift 			= 4
	} omitBitmapShift3;
	
	typedef enum {
		omitBitImageRowBytesMask 		= 0xC0,
		omitBitImageHeightMask 			= 0x30,
		omitBitImageDataMask			= 0x08
	} omitBitImageMask;
	
	typedef enum {
		omitBitImageRowBytesShift 		= 6,
		omitBitImageHeightShift 			= 4,
		omitBitImageDataShift			= 3
	} omitBitImageShift;
	
	typedef enum {
		copyBitImageBytesOpcode			= 0x00,
		repeatBitImageBytesOpcode		= 0x40,
		lookupBitImageBytesOpcode		= 0x80,
		repeatBitImageScanOpcode		= 0xC0
	} bitImageCompression;
	
	typedef enum {
		omitTextCharactersMask 			= 0xC0,
		omitTextPositionXMask 			= 0x30,
		omitTextPositionYMask 			= 0x0C,
		omitTextDataMask 				= 0x02
	} omitTextMask;
	
	typedef enum {
		omitTextCharactersShift 			= 6,
		omitTextPositionXShift 			= 4,
		omitTextPositionYShift 			= 2,
		omitTextDataShift 				= 1
	} omitTextShift;
	
	typedef enum {
		omitGlyphCharactersMask 		= 0xC0,
		omitGlyphLengthMask 			= 0x30,
		omitGlyphRunNumberMask 		= 0x0C,
		omitGlyphOnePositionMask 		= 0x02,
		omitGlyphDataMask 				= 0x01
	} omitGlyphMask1;
	
	typedef enum {
		omitGlyphCharactersShift 		= 6,
		omitGlyphLengthShift 			= 4,
		omitGlyphRunNumberShift 		= 2,
		omitGlyphOnePositionShift 		= 1,
		omitGlyphDataShift 				= 0
	} omitGlyphShift1;
	
	typedef enum {
		omitGlyphPositionsMask 			= 0xC0,
		omitGlyphAdvancesMask 			= 0x20,
		omitGlyphTangentsMask 			= 0x18,
		omitGlyphRunsMask 				= 0x04,
		omitGlyphStylesMask 			= 0x03
	} omitGlyphMask2;
	
	typedef enum {
		omitGlyphPositionsShift 			= 6,
		omitGlyphAdvancesShift 			= 5,
		omitGlyphTangentsShift 			= 3,
		omitGlyphRunsShift 				= 2,
		omitGlyphStylesShift 			= 0
	} omitGlyphShift2;
	
	typedef enum {
		omitLayoutLengthMask 			= 0xC0,
		omitLayoutPositionXMask 			= 0x30,
		omitLayoutPositionYMask 			= 0x0C,
		omitLayoutDataMask 			= 0x02
	} omitLayoutMask1;
	
	typedef enum {
		omitLayoutLengthShift 			= 6,
		omitLayoutPositionXShift 			= 4,
		omitLayoutPositionYShift 			= 2,
		omitLayoutDataShift 				= 1
	} omitLayoutShift1;
	
	typedef enum {
		omitLayoutWidthMask 			= 0xC0,
		omitLayoutFlushMask 			= 0x30,
		omitLayoutJustMask 			= 0x0C,
		omitLayoutOptionsMask 			= 0x03
	} omitLayoutMask2;
	
	typedef enum {
		omitLayoutWidthShift 			= 6,
		omitLayoutFlushShift 			= 4,
		omitLayoutJustShift 				= 2,
		omitLayoutOptionsShift 			= 0
	} omitLayoutShift2;
	
	typedef enum {
		omitLayoutStyleRunNumberMask 	= 0xC0,
		omitLayoutLevelRunNumberMask 	= 0x30,
		omitLayoutHasBaselineMask 		= 0x08,
		omitLayoutStyleRunsMask 		= 0x04,
		omitLayoutStylesMask 			= 0x03
	} omitLayoutMask3;
	
	typedef enum {
		omitLayoutStyleRunNumberShift 	= 6,
		omitLayoutLevelRunNumberShift 	= 4,
		omitLayoutHasBaselineShift 		= 3,
		omitLayoutStyleRunsShift 		= 2,
		omitLayoutStylesShift 			= 0
	} omitLayoutShift3;
	
	typedef enum {
		omitLayoutLevelRunsMask 		= 0x80,
		omitLayoutLevelsMask 			= 0x40
	} omitLayoutMask4;
	
	typedef enum {
		omitLayoutLevelRunsShift 		= 7,
		omitLayoutLevelsShift 			= 6
	} omitLayoutShift4;
	
	typedef enum {
		inkAttributesOpcode,
		inkTagOpcode,
		inkColorOpcode,
		inkTransferModeOpcode
	} inkDataOpcode;

	typedef enum {
		omitColorsSpaceMask 			= 0xC0,
		omitColorsProfileMask 			= 0x30,
		omitColorsComponentsMask 		= 0x0F,
		omitColorsIndexMask 			= 0x0C,
		omitColorsIndexSetMask 			= 0x03
	} omitColorsMask;
	
	typedef enum {
		omitColorsSpaceShift 			= 6,
		omitColorsProfileShift 			= 4,
		omitColorsComponentsShift 		= 0,
		omitColorsIndexShift 			= 2,
		omitColorsIndexSetShift 			= 0
	} omitColorsShift;
	
	typedef enum {
		omitTransferSpaceMask 			= 0xC0,
		omitTransferSetMask			= 0x30,
		omitTransferProfileMask 			= 0x0C
	} omitTransferMask1;
	
	typedef enum {
		omitTransferSpaceShift 			= 6,
		omitTransferSetShift			= 4,
		omitTransferProfileShift 			= 2
	} omitTransferShift1;
	
	typedef enum {
		omitTransferSourceMatrixMask	= 0xC0,
		omitTransferDeviceMatrixMask	= 0x30,
		omitTransferResultMatrixMask		= 0x0C,
		omitTransferFlagsMask 			= 0x03
	} omitTransferMask2;

	typedef enum {
		omitTransferSourceMatrixShift	= 6,
		omitTransferDeviceMatrixShift		= 4,
		omitTransferResultMatrixShift		= 2,
		omitTransferFlagsShift 			= 0
	} omitTransferShift2; 

	typedef enum {
		omitTransferComponentModeMask 			= 0x80,
		omitTransferComponentFlagsMask 			= 0x40,
		omitTransferComponentSourceMinimumMask 	= 0x30,
		omitTransferComponentSourceMaximumMask 	= 0x0C,
		omitTransferComponentDeviceMinimumMask 	= 0x03
	} omitTransferComponentMask1;

	typedef enum {
		omitTransferComponentModeShift 			= 7,
		omitTransferComponentFlagsShift 			= 6,
		omitTransferComponentSourceMinimumShift 	= 4,
		omitTransferComponentSourceMaximumShift 	= 2,
		omitTransferComponentDeviceMinimumShift 	= 0
	} omitTransferComponentShift1;

	typedef enum {
		omitTransferComponentDeviceMaximumMask 	= 0xC0,
		omitTransferComponentClampMinimumMask	= 0x30,
		omitTransferComponentClampMaximumMask	= 0x0C,
		omitTransferComponentOperandMask 		= 0x03
	} omitTransferComponentMask2;

	typedef enum {
		omitTransferComponentDeviceMaximumShift 	= 6,
		omitTransferComponentClampMinimumShift	= 4,
		omitTransferComponentClampMaximumShift	= 2,
		omitTransferComponentOperandShift 		= 0
	} omitTransferComponentShift2;
	
	typedef enum {
		styleAttributesOpcode,
		styleTagOpcode,
		styleCurveErrorOpcode,
		stylePenOpcode,
		styleJoinOpcode,
		styleDashOpcode,
		styleCapsOpcode,
		stylePatternOpcode,
		styleTextAttributesOpcode,
		styleTextSizeOpcode,
		styleFontOpcode,
		styleTextFaceOpcode,
		stylePlatformOpcode,
		styleFontVariationsOpcode	,
	#ifdef layoutStyleRuns 
		styleRunControlsOpcode,
		styleRunPriorityJustOverrideOpcode,
		styleRunGlyphJustOverridesOpcode,
		styleRunGlyphSubstitutionsOpcode,
		styleRunFeaturesOpcode,
		styleRunKerningAdjustmentsOpcode,
	#else
		styleLayoutInfoOpcode,
	#endif
		styleJustificationOpcode
	} styleDataOpcode;

	typedef enum {
		omitDashAttributesMask 					= 0xC0,
		omitDashShapeMask						= 0x30,
		omitDashAdvanceMask					= 0x0C,
		omitDashPhaseMask						= 0x03
	} omitDashMask1;

	typedef enum {
		omitDashAttributesShift 					= 6,
		omitDashShapeShift						= 4,
		omitDashAdvanceShift					= 2,
		omitDashPhaseShift 						= 0
	} omitDashShift1;
	
	typedef enum {
		omitDashScaleMask 						= 0xC0
	} omitDashMask2;

	typedef enum {
		omitDashScaleShift 						= 6
	} omitDashShift2;
	
	typedef enum {
		omitPatternAttributesMask 				= 0xC0,
		omitPatternShapeMask					= 0x30,
		omitPatternUXMask						= 0x0C,
		omitPatternUYMask						= 0x03
	} omitPatternMask1;

	typedef enum {
		omitPatternAttributesShift 				= 6,
		omitPatternShapeShift					= 4,
		omitPatternUXShift						= 2,
		omitPatternUYShift						= 0
	} omitPatternShift1;
	
	typedef enum {
		omitPatternVXMask 						= 0xC0,
		omitPatternVYMask 						= 0x30
	} omitPatternMask2;

	typedef enum {
		omitPatternVXShift						= 6,
		omitPatternVYShift						= 4
	} omitPatternShift2;
	
	typedef enum {
		omitJoinAttributesMask 					= 0xC0,
		omitJoinShapeMask						= 0x30,
		omitJoinMiterMask						= 0x0C
	} omitJoinMask;

	typedef enum {
		omitJoinAttributesShift 					= 6,
		omitJoinShapeShift						= 4,
		omitJoinMiterShift						= 2
	} omitJoinShift;
	
	typedef enum {
		omitCapAttributesMask 					= 0xC0,
		omitCapStartShapeMask					= 0x30,
		omitCapEndShapeMask					= 0x0C
	} omitCapMask;

	typedef enum {
		omitCapAttributesShift 					= 6,
		omitCapStartShapeShift					= 4,
		omitCapEndShapeShift					= 2
	} omitCapShift;
	
	typedef enum {
		omitFaceLayersMask 					= 0xC0,
		omitFaceMappingMask					= 0x30
	} omitFaceMask;

	typedef enum {
		omitFaceLayersShift 					= 6,
		omitFaceMappingShift					= 4
	} omitFaceShift;
	
	typedef enum {
		omitFaceLayerFillMask 					= 0xC0,
		omitFaceLayerFlagsMask					= 0x30,
		omitFaceLayerStyleMask					= 0x0C,
		omitFaceLayerTransformMask				= 0x03
	} omitFaceLayerMask1;

	typedef enum {
		omitFaceLayerFillShift 					= 6,
		omitFaceLayerFlagsShift					= 4,
		omitFaceLayerStyleShift					= 2,
		omitFaceLayerTransformShift				= 0
	} omitFaceLayerShift1;
	
	typedef enum {
		omitFaceLayerBoldXMask 					= 0xC0,
		omitFaceLayerBoldYMask					= 0x30
	} omitFaceLayerMask2;

	typedef enum {
		omitFaceLayerBoldXShift 					= 6,
		omitFaceLayerBoldYShift					= 4
	} omitFaceLayerShift2;
	
	typedef enum {
		colorSetReservedOpcode,
		colorSetTagOpcode
	} colorSetDataOpcode;

	typedef enum {
		colorProfileAttributesOpcode,
		colorProfileTagOpcode
	} profileDataOpcode;
	
	typedef enum {
		transformReservedOpcode,
		transformTagOpcode,
		transformClipOpcode,
		transformMappingOpcode,
		transformPartMaskOpcode,
		transformToleranceOpcode
	} transformDataOpcode;
	
	typedef enum {
		typeOpcode,
		sizeOpcode
	} genericDataOpcode;				/* used by currentOperand when currentOperation is nextOpcode */
	
/* 	format of top byte:
	xx yyyyyy		xx == 0x00, 0x40, 0x80, 0xC0: defines graphics operation (see graphicsOperationOpcode)
				yyyyyy == size of operation in bytes
				if (yyyyyy == 0), byte size follows. If byte following == 0, word size follows; if == 0, long follows
				word and long, if present, are specified in high-endian order (first byte is largest magnitude)
				
	format of byte following size specifiers, if any:
	xx yyyyyy		xx == 0x00, 0x40, 0x80, 0xC0: defines compression level (0 == none, 0xC0 == most)
						exact method of compression is defined by type of data
				yyyyyy == data type selector (0 to 63): see graphicsNewOpcode, __DataOpcode
*/

	#define opcodeShift			6
	#define objectSizeMask		0x3F
	#define compressionShift		6
	#define objectTypeMask		0x3F

	#define bitImageOpcodeMask	0xC0
	#define bitImageCountMask	0x3f
	#define bitImageOpcodeShift	6
	
	typedef enum {
		noCompression,
		wordCompression,
		byteCompression,
		omitCompression
	} twoBitCompressionValues;
	
	#define compressionMask		0x03
	
	/*	the following structures define how primitives without a public geometry
		are stored (their format mirrors that of the New call to create them) 	*/

	typedef struct {
		unsigned char		name;		/* fontName */
		unsigned char		platform;		/* fontPlatform */
		unsigned char		script;		/* fontScript */
		unsigned char		language;		/* fontLanguage */
		short			length;		/* byte length */
	/*	unsigned char 		fontName[length] */
	} flatFontName;

	#define flatFontListItemTag	0x666C7374	/* 'flst' */

	typedef struct {
		font			fontID;			/*** if we get rid of this, remove #include "font types.h", above */
		unsigned char	name;			/* fontName */
		unsigned char	platform;			/* fontPlatform */
		unsigned char	script;			/* fontScript */
		unsigned char	language;			/* fontLanguage */
		short		length;			/* byte length */
		short		glyphCount;
	/*	unsigned char 	fontName[anyNumber];				*/	
	/*	long 			glyphBits[glyphBitsSize + 31 >> 5]; 		*/
	} flatFontListItem;
	
	typedef struct {
		long				count;
		flatFontListItem	items[anyNumber];
	} flatFontList;

	typedef struct {
		fixed				version;
		unsigned char		flatFlags;
	} flattenHeader;

	typedef enum {
		omitPictureShapeMask	   		= 0xC0,
		omitOverrideStyleMask			= 0x30,
		omitOverrideInkMask 			= 0x0C,
		omitOverrideTransformMask	 	= 0x03
	} omitPictureParametersMask;

	typedef enum {
		omitPictureShapeShift	   		= 0x6,
		omitOverrideStyleShift			= 0x4,
		omitOverrideInkShift				= 0x2,
		omitOverrideTransformShift	 	= 0x0
	} omitPictureParametersShift;

#endif
