/* graphics:
	public object and constant definitions
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

#ifndef graphicsTypesIncludes
	#define graphicsTypesIncludes
	/* everything outdented from here to conserve space */

	#ifndef mathTypesIncludes
		#include "math types.h"
	#endif

	#ifndef anyNumber
		#define anyNumber 1
	#endif

	#ifndef nil
		#define nil 0L
	#endif
	
/* graphics objects */

	typedef struct privateShapeRecord *shape;
	typedef struct privateStyleRecord *style;
	typedef struct privateInkRecord *ink;
	typedef struct privateTransformRecord *transform;
	typedef struct privateTagRecord *tag;
	typedef struct privateColorSetRecord *colorSet;
	typedef struct privateProfileRecord *colorProfile;
	typedef struct privateGraphicsClientRecord *graphicsClient;
	typedef struct privateViewGroupRecord *viewGroup;
	typedef struct privateViewPortRecord *viewPort;
	typedef struct privateViewDeviceRecord *viewDevice;

	typedef long colorSpace;

/* shape enumerations */

	typedef enum {
		emptyType = 1,
		pointType,
		lineType,
		curveType,
		rectangleType,
		polygonType,
		pathType,
		bitmapType,
		textType,
		glyphType,
		layoutType,
		fullType,
		pictureType
	} shapeTypes;

	typedef long shapeType;

	typedef enum {
		noFill,
		openFrameFill,
		frameFill = openFrameFill,
		closedFrameFill,
		hollowFill = closedFrameFill,
		evenOddFill,
		solidFill = evenOddFill,
		windingFill,
		inverseEvenOddFill,
		inverseSolidFill = inverseEvenOddFill,
		inverseFill = inverseEvenOddFill,
		inverseWindingFill
	} shapeFills;

	typedef long shapeFill;

	typedef enum {
		noAttributes,
		directShape		= 0x0001,
		remoteShape		= 0x0002,
		cachedShape		= 0x0004,
		lockedShape		= 0x0008,
		groupShape		= 0x0010,
		mapTransformShape	= 0x0020,
		uniqueItemsShape	= 0x0040,
		ignorePlatformShape	= 0x0080,
		noMetricsGridShape	= 0x0100,
		diskShape			= 0x0200,
		memoryShape		= 0x0400
	} shapeAttributes;

	typedef long shapeAttribute;

/* shape editing enumerations */

	typedef enum {
		breakNeitherEdit			= 0,
		breakLeftEdit				= 0x0001,
		breakRightEdit				= 0x0002,
		removeDuplicatePointsEdit	= 0x0004		/* if the new first (or last) point exactly matches the point before it in */
											/* the same contour, then remove it) */
	} editShapeFlags;

	typedef long editShapeFlag;

	#define selectToEnd	-1
	#define setToNil	(void *)(-1)

	typedef enum {
		counterclockwiseDirection,
		clockwiseDirection
	} contourDirections;
	
	typedef long contourDirection;

/* shape structures */

/* The type 'point' is defined in "math types.h" */
/*
	typedef struct {
		fixed 	x;
		fixed 	y;
	} point;
*/

	typedef struct {
		point		first;
		point		last;
	} line;

	typedef struct {
		point		first;
		point		control;
		point		last;
	} curve;

	typedef struct {
		fixed		left;
		fixed		top;
		fixed		right;
		fixed		bottom;
	} rectangle;

	typedef struct {
		long		vectors;
		point		vector[anyNumber];
	} polygon;

	typedef struct {
		long		contours;
		polygon	contour[anyNumber];
	} polygons;

	typedef struct {
		long		vectors;
		long		controlBits[anyNumber];
		point		vector[anyNumber];
	} path;

	typedef struct {
		long		contours;
		path		contour[anyNumber];
	} paths;

	typedef struct {
		char			*image;		/* pointer to pixels */
		long			width;		/* width in pixels */
		long			height;		/* height in pixels */
		long			rowBytes;	/* width in bytes */
		long			pixelSize;		/* physical bits per pixel */
		colorSpace	space;
		colorSet		set;
		colorProfile	profile;
	} bitmap;
	
	typedef struct {
		long		left;
		long		top;
		long		right;
		long		bottom;
	} longRectangle;

/* style enumerations */

	typedef enum {
		centerFrameStyle	= 0,
		sourceGridStyle	= 0x0001,
		deviceGridStyle	= 0x0002,
		insideFrameStyle	= 0x0004,
		outsideFrameStyle	= 0x0008,
		autoInsetStyle		= 0x0010
	} styleAttributes;

	typedef long styleAttribute;

	typedef enum {
		bendDash			= 0x0001,
		breakDash			= 0x0002,
		clipDash			= 0x0004,
		levelDash			= 0x0008,
		autoAdvanceDash	= 0x0010
	} dashAttributes;
	
	typedef long dashAttribute;
	
	typedef enum {
		portAlignPattern 	= 0x0001,
		portMapPattern		= 0x0002
	} patternAttributes;
	
	typedef long patternAttribute;

	typedef enum {
		sharpJoin			= 0x0000,
		curveJoin			= 0x0001,
		levelJoin			= 0x0002
	} joinAttributes;
	
	typedef long joinAttribute;

	typedef enum {
		levelStartCap		= 0x0001,
		levelEndCap		= 0x0002
	} capAttributes;
	
	typedef long capAttribute;

	typedef enum {
		autoAdvanceText	= 0x0001,
		noContourGridText	= 0x0002,
		noMetricsGridText	= 0x0004,
		anchorPointsText 	= 0x0008,
		verticalText		= 0x0010
	} textAttributes;
	
	typedef long textAttribute;		

	#define leftJustify 0
	#define centerJustify (frac1/2)
	#define rightJustify frac1
	#define fillJustify -1

	typedef enum {
		underlineAdvanceLayer = 1,			/* a line is drawn through the advances */
		skipWhiteSpaceLayer = 2,				/* except characters describing white space */
		underlineIntervalLayer = 4,			/* (+ stringLayer) a line is drawn through the gaps between advances */
		underlineContinuationLayer = 8,		/* (+ stringLayer) join this underline with another face */
		whiteLayer = 16,					/* the layer draws to white instead of black */
		clipLayer = 32,						/* the characters define a clip */
		stringLayer = 64					/* all characters in run are combined */
	} layerFlags;

	typedef long layerFlag;

/* style structures */

	typedef struct {
		shapeFill		outlineFill;			/* outline framed or filled */
		layerFlag		flags;				/* various additional effects */
		style			outlineStyle;			/* outline */
		transform		outlineTransform;		/* italic, condense, extend */
		point			boldOutset;			/* bold */
	} faceLayer;

	typedef struct {
		long			faceLayers;			/* layer to implement shadow */
		mapping		advanceMapping;		/* algorithmic change to advance width */
		faceLayer		faceLayer[anyNumber];	/* zero or more face layers describing the face */
	} textFace;

	typedef struct {
		joinAttribute		attributes;
		shape			join;
		fixed				miter;
	} joinRecord;

	typedef struct {
		dashAttribute		attributes;
		shape			dash;	/* similar to pattern, except rotated to line slope */
		fixed				advance;	/* specifies repeating frequency of dash */
		fract				phase;	/* specifies offset into the path to start dashing */
		fixed				scale;	/* specifies height of dash to be mapped to penWidth */
	} dashRecord;

	typedef struct {
		patternAttribute	attributes;
		shape			pattern;
		point				u;
		point				v;
	} patternRecord;

	typedef struct {
		capAttribute		attributes;
		shape			startCap;
		shape			endCap;
	} capRecord;

/* ink enumerations */

	typedef enum {
		portAlignDitherInk	= 0x0001,
		forceDitherInk		= 0x0002,
		suppressDitherInk	= 0x0004,
		suppressHalftoneInk	= 0x0008
	} inkAttributes;

	typedef long inkAttribute;

	typedef enum {
		noMode = 0,
		copyMode,
		addMode,
		blendMode,
		migrateMode,
		minimumMode,
		maximumMode,
		highlightMode,
		andMode,
		orMode,
		xorMode,
		rampAndMode,
		rampOrMode,
		rampXorMode,
		overMode,				/* Alpha channel modes					*/
		atopMode,					/* Note: In England = Beta channel modes		*/
		excludeMode,
		fadeMode
	} componentModes;

	typedef unsigned char componentMode;

	typedef enum {
		rejectSourceTransfer		= 0x0001,	/* at least one component must be out of range			*/
		rejectDeviceTransfer		= 0x0002,	/* at least one component must be out of range			*/
		singleComponentTransfer		= 0x0004		/* duplicate transferComponent[0] for all components in transfer */
	} transferFlags;
	
	typedef long transferFlag;
	
	typedef enum {
		overResultComponent	= 0x01,			/* & result color with 0xFFFF before clamping			*/
		reverseComponent		= 0x02			/* reverse source and device before moding			*/
	} componentFlags;

	typedef unsigned char componentFlag;

/* ink structures */

	typedef struct {
		componentMode		mode;				/* how the component is operated upon				*/
		componentFlag		flags;				/* flags for each component						*/
		colorValue			sourceMinimum;
		colorValue			sourceMaximum;		/* input filter range								*/
		colorValue			deviceMinimum;
		colorValue			deviceMaximum;		/* output filter range							*/
		colorValue			clampMinimum;
		colorValue			clampMaximum;		/* output clamping range							*/
		colorValue			operand;				/* ratio for blend, step for migrate, color for highlight	*/
	} transferComponent;

	typedef struct {
		colorSpace		space;				/* the color-space the transfer mode is to operate in	*/
		colorSet			set;
		colorProfile		profile;
		fixed				sourceMatrix[5][4];
		fixed				deviceMatrix[5][4];
		fixed				resultMatrix[5][4];
		transferFlag		flags;
		transferComponent	component[4];			/* how each component is operated upon				*/
	} transferMode;

/* color space enumerations */

	typedef enum {
		noColorPacking 		= 0x0000,		/* 16 bits per channel */
		alphaSpace		= 0x0080,		/* space includes alpha channel */
		word5ColorPacking 	= 0x0500,		/* 5 bits per channel, right-justified */
		long8ColorPacking	= 0x0800,		/* 8 bits per channel, right-justified */
		long10ColorPacking	= 0x0a00,		/* 10 bits per channel, right-justified */
		alphaFirstPacking	= 0x1000			/* alpha channel is the first field in the packed space */
	} colorPackingTypes;
	
	typedef enum {
		noSpace = 0,
		rgbSpace,
		cmykSpace,
		hsvSpace,
		hlsSpace,
		cieSpace,
		xyzSpace,
		luvSpace,
		labSpace,
		yiqSpace,
		ntscSpace 	= yiqSpace,
		palSpace 		= yiqSpace,
		graySpace	,
		indexedSpace,
		rgbaSpace		= rgbSpace + alphaSpace,
		grayaSpace	= graySpace + alphaSpace,
		rgb16Space	= word5ColorPacking + rgbSpace,
		rgb32Space	= long8ColorPacking + rgbSpace,
		argb32Space	= long8ColorPacking + alphaFirstPacking + rgbaSpace,
		cmyk32Space	= long8ColorPacking + cmykSpace,
		hsv32Space	= long10ColorPacking + hsvSpace,
		hls32Space	= long10ColorPacking + hlsSpace,
		cie32Space	= long10ColorPacking + cieSpace,
		xyz32Space	= long10ColorPacking + xyzSpace,
		luv32Space	= long10ColorPacking + luvSpace,
		lab32Space	= long10ColorPacking + labSpace,
		yiq32Space	= long10ColorPacking + yiqSpace,
		ntsc32Space 	= yiq32Space,
		pal32Space 	= yiq32Space
	} colorSpaces;

/* color space structures */

	typedef struct {
		 colorValue red;
		 colorValue green;
		 colorValue blue;
	} rgbColor;

	typedef struct {
		 colorValue red;
		 colorValue green;
		 colorValue blue;
		 colorValue alpha;
	} rgbaColor;

	typedef struct {
		colorValue hue;
		colorValue saturation;
		colorValue value;
	} hsvColor;

	typedef struct {
		colorValue hue;
		colorValue lightness;
		colorValue saturation;
	} hlsColor;

	typedef struct {
		colorValue cyan;
		colorValue magenta;
		colorValue yellow;
		colorValue black;
	} cmykColor;

	typedef struct {
		colorValue x;
		colorValue y;
		colorValue z;
	} cieColor;

	typedef struct {
		colorValue y;
		colorValue i;
		colorValue q;
	} yiqColor;

	typedef struct {
		colorValue gray;
		colorValue alpha;
	} grayaColor;

	typedef long colorIndex;

	typedef struct {
		colorIndex	index;
		colorSet	set;
	} indexedColor;

	typedef struct {
		colorSpace space;
		colorProfile profile;
		union {
			cmykColor	cmyk;
			rgbColor		rgb;
			rgbaColor		rgba;
			hsvColor		hsv;
			hlsColor		hls;
			cieColor		cie;
			yiqColor		yiq;
			colorValue		gray;
			grayaColor	graya;
			unsigned short	pixel16;
			unsigned long	pixel32;
			indexedColor	indexed;
			colorValue		component[4];
		} element;
	} color;

/* colorSet structures */

	typedef union {
		cmykColor	cmyk;
		rgbColor		rgb;
		rgbaColor		rgba;
		hsvColor		hsv;
		hlsColor		hls;
		cieColor		cie;
		yiqColor		yiq;
		colorValue		gray;
		grayaColor	graya;
		unsigned short	pixel16;
		unsigned long	pixel32;
		colorValue		component[4];
	} setColor;

/* colorProfile enumerations */

	typedef enum {
		perceptualProfile = 0,		/* only one of these three can be used at once */
		colorimetricProfile = 1,
		saturationProfile = 2,
		
		suppressMatchProfile = 4		/* added to perceptual / colorimetric / saturation to suppress matching */
	} profileAttributes;

	typedef long profileAttribute;

/* colorProfile structures */
	
	typedef enum {
		grayResponse,
		redResponse,
		greenResponse,
		blueResponse,
		cyanResponse,
		magentaResponse,
		yellowResponse,
		underColorRemovalResponse,
		blackGenerationResponse,
		onePlusLastResponse
	} responseColors;
	
	typedef long responseColor;

	typedef struct {
		short		colorCount[onePlusLastResponse];
		colorValue		data[anyNumber];
	} profileResponse;

	typedef struct {
		cieColor		red;
		cieColor		green;
		cieColor		blue;
		cieColor		cyan;
		cieColor		magenta;
		cieColor		yellow;
		cieColor		black;
		cieColor		white;
	} profileRecord;

/* transform enumerations */

	typedef enum {					/* parts of a shape considered in hit testing: */
		noPart			= 0,			/* (in order of evaluation) */
		boundsPart		= 0x0001,
		geometryPart		= 0x0002,
		penPart			= 0x0004,
		cornerPointPart	= 0x0008,
		controlPointPart	= 0x0010,
		edgePart			= 0x0020,
		joinPart			= 0x0040,
		startCapPart 		= 0x0080,
		endCapPart 		= 0x0100,
		dashPart			= 0x0200,
		patternPart		= 0x0400,
		glyphBoundsPart	= joinPart,
		glyphFirstPart		= startCapPart,
		glyphLastPart		= endCapPart,
		sideBearingPart	= dashPart,
		anyPart			= boundsPart | geometryPart | penPart | cornerPointPart | controlPointPart |
							edgePart | joinPart | startCapPart | endCapPart | dashPart | patternPart
	} shapeParts;

	typedef long shapePart;

/* transform structures */

	typedef struct {
		shapePart		what;				/* which part of shape */
		long			index;				/* control point index */
		fixed			distance;				/* how far from point or outside of area click was */

		/* these fields are only set by HitTestPicture */
		shape		which;
		shape		containerPicture;		/* picture which contains shape hit */
		long			containerIndex;			/* the index within that picture  */
		long			totalIndex;			/* the total index within the root picture */
	} hitTestInfo;

/* viewPort enumerations */

	typedef enum {
		grayPort		 	= 0x0001,
		alwaysGridPort		= 0x0002
	} portAttributes;

	typedef long portAttribute;

/* viewDevice enumerations */

	typedef enum {
		directDevice		= 0x01,			/* for the device bitmap baseAddr pointer */
		remoteDevice 		= 0x02,
		inactiveDevice		= 0x04
	} deviceAttributes;

	typedef long deviceAttribute;

	typedef enum {
		roundDot = 1,
		spiralDot,
		squareDot,
		lineDot,
		ellipticDot,
		triangleDot,
		hilbertDot,
		orderedDitherDot
	} dotTypes;

	typedef long dotType;

/* viewPort structures */

	typedef enum {
		noTint,
		luminanceTint,			/* use the luminance of the color */
		averageTint,			/* add all the components and divide by the number of components */
		mixtureTint,			/* find the closest color on the axis between the foreground and background */
		component1Tint,		/* use the value of the first component of the color */
		component2Tint,		/* ... etc. */
		component3Tint,
		component4Tint
	} tintTypes;

	typedef long tintType;

	typedef struct {
		fixed			angle;
		fixed			frequency;
		dotType		method;
		tintType		tinting;
		color			dotColor;
		color			backgroundColor;
		colorSpace	tintSpace;
	} halftone;

/* viewGroup enumerations */

	#define allViewDevices		((viewGroup) 0)
	#define screenViewDevices	((viewGroup) 1)

/* graphics stream constants and structures */

	typedef enum {
		openReadSpool	= 1,
		openWriteSpool,
		readSpool,
		writeSpool,
		closeSpool
	} spoolCommands;
	
	typedef long spoolCommand;
	
	typedef unsigned char graphicsOpcode;

	typedef struct spoolBlock {
	/* these fields are read only */
		long			(*spoolProcedure)(spoolCommand command, struct spoolBlock *block);
		void			*buffer;			/* source/destination pointer to data */
		long			bufferSize;		/* how many bytes for the system to read (flatten) / write (unflatten) */
	/* these fields are written to (but are not read from) */
		long			count;			/* how many bytes for the caller to read (unflatten) /write (flatten) */
		long			operationSize;		/* operation size (including operand byte) */
		long			operationOffset;	/* the data offset, if any, within the current operation */
		graphicsOpcode	lastTypeOpcode;	/* type of last created object */
		graphicsOpcode	currentOperation;	/* operation emitted by flatten, or intrepreted by last unflatten */
		graphicsOpcode	currentOperand;	/* e.g., transformTypeOpcode, inkTagOpcode */
		unsigned char	compressed;		/* one of: twoBitCompressionValues */
	} spoolBlock;

	typedef enum {
		fontListFlatten		= 0x01,		/* if set, generate a tag containing list of each font referenced */
		fontGlyphsFlatten	= 0x02		/* if set, generate a tag containing the list of glyphs referenced inside the font */
	} flattenFlags;
	
	typedef long flattenFlag;

/* graphicsClient constants */

	#define	graphicsSystemClient		(graphicsClient)(-1)
	#define	graphicsCodeClient			(graphicsClient)(-2)

/* graphics patching constants */

	#define	originalGraphicsFunction		-1
	#define	originalGraphicsIdentifier		'grfx'

#endif /* of graphic types includes */
