/*
	layout edit library.h
	
	Simple layout editing based on the TextEdit model.
	
	Copyright 1992 Apple Computer, Inc. All rights reserved.
*/

#ifndef layoutEditLibraryIncludes
#define layoutEditLibraryIncludes

#include <Types.h>
#include <Memory.h>

#ifndef selectionLibraryIncludes
#include "selection library.h"
#endif

#ifndef layoutTypesIncludes
#include "layout types.h"
#endif


typedef Handle LayoutEditHandle;

LayoutEditHandle NewLayoutEditHandle(
	long textRunCount,
	const short textRunLengths[],
	const void *text[],
	long styleRunCount,
	const short styleRunLengths[],
	const style styles[],
	long levelRunCount,
	const short levelRunLengths[],
	const short levels[],
	LayoutOptions *layoutOptions,
	point *position,
	style defaultStyle);

LayoutEditHandle LayoutEditHandleFromLayout(shape layout);

long GetLayoutEditHandle(
	LayoutEditHandle handle,
	void *text,
	long *styleRunCount,
	short styleRunLengths[],
	style styles[],
	long *levelRunCount,
	short levelRunLengths[],
	short levels[],
	LayoutOptions *layoutOptions,
	point *position);

void SetLayoutEditHandle(
	LayoutEditHandle handle,
	long textRunCount,
	const short textRunLengths[],
	const void *text[],
	long styleRunCount,
	const short styleRunLengths[],
	const style styles[],
	long levelRunCount,
	const short levelRunLengths[],
	const short levels[],
	const LayoutOptions *layoutOptions,
	const point *position);

void SetLayoutEditHandleParts(
	LayoutEditHandle handle,
	ByteOffset oldStartOffset,
	ByteOffset oldEndOffset,
	long newTextRunCount,
	const short newTextRunLengths[],
	const void *newText[],
	long newStyleRunCount,
	const short newStyleRunLengths[],
	const style newStyles[],
	long newLevelRunCount,
	const short newLevelRunLengths[],
	const short newLevels[]);

void SetLayoutEditHandleShapeParts(
	LayoutEditHandle handle,
	ByteOffset startOffset,
	ByteOffset endOffset,
	shape insert);

long GetLayoutEditHandleParts(
	LayoutEditHandle handle,
	ByteOffset startOffset,
	ByteOffset endOffset,
	void *text,
	long *styleRunCount,
	short styleRunLengths[],
	style styles[],
	long *levelRunCount,
	short levelRunLengths[],
	short levels[]);

shape GetLayoutEditHandleShapeParts(
	LayoutEditHandle handle,
	ByteOffset startOffset,
	ByteOffset endOffset,
	shape dest);

void LayoutEditIdle(LayoutEditHandle handle);

void LayoutEditClick(LayoutEditHandle handle, point hitDown);

void LayoutEditActivate(LayoutEditHandle handle);

void LayoutEditDeactivate(LayoutEditHandle handle);

void LayoutEditKey(LayoutEditHandle handle, char key);

void LayoutEditUpdate(LayoutEditHandle handle);

SelectionHandle LayoutEditGetSelection(LayoutEditHandle handle);

void LayoutEditSetSelection(LayoutEditHandle handle, SelectionHandle selection);

void LayoutEditSetStyle(LayoutEditHandle handle, style newStyle);
void LayoutEditIncrementLevel(LayoutEditHandle handle);
void LayoutEditDecrementLevel(LayoutEditHandle handle);
void LayoutEditSetLevel(LayoutEditHandle handle, long level);

void LayoutEditCut(LayoutEditHandle handle);
void LayoutEditCopy(LayoutEditHandle handle);
void LayoutEditPaste(LayoutEditHandle handle);
void LayoutEditClear(LayoutEditHandle handle);

void LayoutEditFromScrap(LayoutEditHandle handle);
void LayoutEditToScrap(LayoutEditHandle handle);

void DisposeLayoutEditHandle(LayoutEditHandle handle);

#endif
