/* layout feature constants.h -- Names for layout feature types and selectors. */

/* Copyright 1992, 1993 Apple Computer, Inc.  All rights reserved. */

#ifndef layoutFeatureConstantsIncludes
#define layoutFeatureConstantsIncludes

/* The following values can be used to set RunFeature values. Note that unless the
    feature is defaulted differently in different fonts, the zero value for the
    selectors represents the default value. */

enum {

	typographicFeaturesType = 0,
	
		enableEffectsOnSelector									= 0,
		enableEffectsOffSelector								= 1,
	
  ligatureType = 1,
	
		ligatureRequiredOnSelector							= 0,
		ligatureRequiredOffSelector							= 1,
		ligatureCommonOnSelector								= 2,
		ligatureCommonOffSelector								= 3,
		ligatureRareOnSelector									= 4,
		ligatureRareOffSelector									= 5,
		ligatureLogotypeOnSelector							= 6,
		ligatureLogotypeOffSelector							= 7,
		ligatureRebusOnSelector									= 8,
		ligatureRebusOffSelector								= 9,
	
  cursiveConnectionType = 2,
	
		cursiveConnectionNoneSelector						= 0,
		cursiveConnectionPartialSelector				= 1,
		cursiveConnectionFullSelector						= 2,
	
  letterNumberCaseType = 3,
	
		letterNumberCaseUpperLowerSelector			= 0,
		letterNumberCaseAllUpperSelector				= 1,
		letterNumberCaseAllLowerSelector				= 2,
		letterNumberCaseSmallCapsSelector				= 3,
		letterNumberCaseUpperThenLowerSelector	= 4,
		letterNumberCaseUpperThenSmCapsSelector	= 5,
	
  verticalSubstitutionType = 4,
	
		verticalSubstitutionOnSelector					= 0,
		verticalSubstitutionOffSelector					= 1,
	
  rearrangementType = 5,
	
		rearrangementOnSelector									= 0,
		rearrangementOffSelector								= 1,
	
  numberStyleType = 6,
	
		numberStyleForColumnsSelector						= 0,
		numberStyleProportionalSelector					= 1,
	
  alternateDesignsType = 7,
	
		alternateDesignsEngravedOnSelector			= 0,
		alternateDesignsEngravedOffSelector			= 1,
		alternateDesignsCursiveOnSelector				= 2,
		alternateDesignsCursiveOffSelector			= 3,
		alternateDesignsSlopedOnSelector				= 4,
		alternateDesignsSlopedOffSelector				= 5,
		alternateDesignsChanceryOnSelector			= 6,
		alternateDesignsChanceryOffSelector			= 7,
		alternateDesignsBookTextOnSelector			= 8,
		alternateDesignsBookTextOffSelector			= 9,
		alternateDesignsDisplayTextOnSelector		= 10,
		alternateDesignsDisplayTextOffSelector	= 11,
		alternateDesignsIlluminatedOnSelector		= 12,
		alternateDesignsIlluminatedOffSelector	= 13,
		alternateDesignsTitlingOnSelector				= 14,
		alternateDesignsTitlingOffSelector			= 15,
	
  smartSwashType = 8,
	
		smartSwashWordInitialsOnSelector				= 0,
		smartSwashWordInitialsOffSelector				= 1,
		smartSwashWordFinalsOnSelector					= 2,
		smartSwashWordFinalsOffSelector					= 3,
		smartSwashLineInitialsOnSelector				= 4,
		smartSwashLineInitialsOffSelector				= 5,
		smartSwashLineFinalsOnSelector					= 6,
		smartSwashLineFinalsOffSelector					= 7,
		smartSwashArchaicNonFinalsOnSelector		= 8,
		smartSwashArchaicNonFinalsOffSelector		= 9,
	
  diacriticsType = 9,
	
		diacriticsShowSelector									= 0,
		diacriticsHideSelector									= 1,
		diacriticsSerializeSelector							= 2,
	
  verticalPositionType = 10,
	
		verticalPositionNormalSelector					= 0,
		verticalPositionSuperiorsSelector				= 1,
		verticalPositionInferiorsSelector				= 2,
		verticalPositionOrdinalsSelector				= 3,
	
  fractionsType = 11,
	
		fractionsDontFormSelector								= 0,
		fractionsFormVerticalSelector						= 1,
		fractionsFormDiagonalSelector						= 2,
	
	diphthongLigaturesType = 12,
	
		diphthongLigaturesOnSelector						= 0,
		diphthongLigaturesOffSelector						= 1,
	
	overlappingGlyphsType = 13,
	
		overlappingGlyphsPreventOnSelector			= 0,
		overlappingGlyphsPreventOffSelector			= 1,
	
	typographicExtrasType = 14,
	
		extrasHyphensToEmDashOnSelector					= 0,
		extrasHyphensToEmDashOffSelector				= 1,
		extrasHyphenToEnDashOnSelector					= 2,
		extrasHyphenToEnDashOffSelector					= 3,
		extrasForceUnslashedZeroOnSelector			= 4,
		extrasForceUnslashedZeroOffSelector			= 5,
		extrasMakeInterrobangOnSelector					= 6,
		extrasMakeInterrobangOffSelector				= 7,
	
	mathExtrasType = 15,
	
		mathHyphenToMinusOnSelector							= 0,
		mathHyphenToMinusOffSelector						= 1,
		mathStarToMultiplyOnSelector						= 2,
		mathStarToMultiplyOffSelector						= 3,
		mathSlashToDivideOnSelector							= 4,
		mathSlashToDivideOffSelector						= 5,
		mathInequalityLigaturesOnSelector				= 6,
		mathInequalityLigaturesOffSelector			= 7,
		mathExponentiationOnSelector						= 8,
		mathExponentiationOffSelector						= 9,
	
  ornamentSetsType = 16,
	
		ornamentSetsNoneSelector								= 0,
		ornamentSetsDingbatsSelector						= 1,
		ornamentSetsPiCharactersSelector				= 2,
		ornamentSetsFleuronsSelector						= 3,
		ornamentSetsDecorativeBordersSelector		= 4,
		ornamentSetsIntlSymbolsSelector					= 5,
		ornamentSetsMathSymbolsSelector					= 6,
	
	glyphAlternativesType = 17,
	
		/* The selectors for this type are simply numbers starting with zero for
			 the different sets of swash glyphs in a font. For example, a font with
			 twenty ampersands could place them in twenty selectors under this feature
			 type. Use of the alternateDesignsType, however, is preferred (when feasible). */

	lastFeatureType = -1
	
  };

#endif layoutFeatureConstantsIncludes
