/* layout library.h -- Line Layout support routines. */

/* Copyright 1990, 1991, 1992 Apple Computer, Inc.  All rights reserved. */

#ifndef layoutLibraryIncludes
#define layoutLibraryIncludes

#ifndef graphicsRoutinesIncludes
#include "graphics routines.h"
#endif

#ifndef layoutTypesIncludes
#include "layout types.h"
#endif

typedef struct {
	PriorityJustificationOverride	*priorityJustOverride;
	GlyphJustificationOverride		*glyphJustOverrides;
	long													glyphJustOverridesCount;
	GlyphSubstitution							*glyphSubstitutions;
	long													glyphSubstitutionsCount;
	KerningAdjustment							*kerningAdjustments;
	long													kerningAdjustmentsCount;
} StyleRunOverrides;

typedef struct {
	short	nLayouts;
	fixed	totalHeight;
	shape	layouts[];
} ParagraphRecord, **ParagraphRecordHandle;

typedef struct {
	RunFeatureType	featureType;
	unsigned short	nSelectors;
	unsigned short	selectorTableOffset;
	short						nameIndex;
} RunFeatureTypeName;

typedef struct {
	RunFeatureSelector	featureSelector;
	short								nameIndex;
} RunFeatureSelectorName;

void InitializeRunControls (RunControls *runControls);

void InitializeLayoutOptions (LayoutOptions *layoutOptions);

void InitializeStyleRunOverrides (StyleRunOverrides *overrides);

void SetDefaultPriorityJustOverride (PriorityJustificationOverride *override);

void SetLayoutStyle (
	style							s,
	char							*fontName,
	fixed							textSize,
	textAttribute			attr,
	RunControls				*runControls,
	RunFeature				runFeatures[],
	long							runFeaturesCount,
	StyleRunOverrides	*overrides);

style NewLayoutStyle (
	char							*fontName,
	fixed							textSize,
	textAttribute			attr,
	RunControls				*runControls,
	RunFeature				runFeatures[],
	long							runFeaturesCount,
	StyleRunOverrides	*overrides);

shape NewSingleLayout (
	char							*text,
	char							*fontName,
	fixed							textSize,
	LayoutOptions			*options,
	point							*position,
	textAttribute			attr,
	RunControls				*runControls,
	RunFeature				runFeatures[],
	long							runFeaturesCount,
	StyleRunOverrides	*overrides);

/* The following functions provide a limited paragraph-creation function. They make calls
		to GetWidthArray and do simple line-breaking using the obtained widths.
		
		!!! IMPORTANT NOTE FOLLOWS !!!
		
		These functions need to be able to deal with text in multiple scripts, which might
		contain zero bytes. Therefore calls that determine hard stops (such as NewParagraph)
		do NOT stop at zero bytes; this means that just passing in a single C string is NOT
		sufficient! Callers will need to add a carriage return themselves to the end of the
		text source. */

ParagraphRecordHandle NewParagraph(
	char		*text,
	style		baseStyle,
	fixed		width,
	long		justified,			/* really a boolean; this way for compiler difference reasons */
	fixed		lineHeight,			/* if zero, we will deduce */
	point		*firstOrigin);	/* origin of first line in paragraph */

ParagraphRecordHandle NewStyledParagraph(
	long								textRunCount,
	const void					*text[],
	const short					textRunLengths[],
	long								styleRunCount,
	const style					styles[],
	const short					styleRunLengths[],
	long								levelRunCount,
	const short					levels[],
	const short					levelRunLengths[],
	long								totalByteCount,
	const LayoutOptions	*layoutOptions,
	fixed								lineHeight,
	const point					*firstOrigin);

void DisposeParagraph(ParagraphRecordHandle paraRec);

/* GetLayoutBounds can be used to return the bounds of a layout. */

shape GetLayoutBounds(shape layout);

#endif
