/* layout routines.h -- Line Layout calls

	by Eric Mader and Dave Opstad,
	with special thanks to the following folks:
		 Mark Davis, for starting it all off;
		 Russ Sonnenschein, for many good ideas;
		 Cary Clark, Dave Good, Rob Johnson, Keith McGreggor, and Chris Yerga for the graphics system;
		 Richard Becker, Lynn Bekkala and Mike Reed for fonts and the font system;
		 Robert Dierkes and the SQE team for testing and good ideas;
		 Parry Kejriwal and Richard Klein, for font tools;
		 Diane Patterson, for great patience in writing documentation;
		 Hugo Ayala, Tom Dowdy, Dan Lipton, Sean Parent and the rest of the PrintShop;
		 Mitch Allen for managing to manage us; and
		 many other good folk at Apple!
*/

/* Copyright 1998-1992 Apple Computer, Inc.  All rights reserved. */

#ifndef layoutRoutinesIncludes
#define layoutRoutinesIncludes

#ifndef graphicsTypesIncludes
	#include "graphics types.h"
#endif

#ifndef layoutTypesIncludes
	#include "layout types.h"
#endif layoutTypesIncludes

#ifdef appleInternal
	#define InlineCode(x)
#endif
#ifndef InlineCode
	#define InlineCode(x)	= {0x303C, x, 0xA832}
#endif

#ifdef __cplusplus
extern "C" {
#endif

shape NewLayout(long textRunCount, const short textRunLengths[], const void *text[],
	long styleRunCount, const short styleRunLengths[], const style styles[], long levelRunCount,
	const short levelRunLengths[], const short levels[], const LayoutOptions *layoutOptions,
	const point *position) InlineCode(0x117F);

long GetLayout(shape layout, void *text, long *styleRunCount, short styleRunLengths[],
	style styles[], long *levelRunCount, short levelRunLengths[], short levels[],
	LayoutOptions *layoutOptions, point *position) InlineCode(0x1180);

void SetLayout(shape layout, long textRunCount, const short textRunLengths[], const void *text[],
	long styleRunCount, const short styleRunLengths[], const style styles[], long levelRunCount,
	const short levelRunLengths[], const short levels[], const LayoutOptions *layoutOptions,
	const point *position) InlineCode(0x1181);

void DrawLayout(long textRunCount, const short textRunLengths[], const void *text[],
	long styleRunCount, const short styleRunLengths[], const style styles[], long levelRunCount,
	const short levelRunLengths[], const short levels[], const LayoutOptions *layoutOptions,
	const point *position) InlineCode(0x1182);

void SetLayoutParts(shape layout, ByteOffset oldStartOffset, ByteOffset oldEndOffset,
	long newTextRunCount, const short newTextRunLengths[], const void *newText[],
	long newStyleRunCount, const short newStyleRunLengths[], const style newStyles[],
	long newLevelRunCount, const short newLevelRunLengths[], const short newLevels[]) InlineCode(0x1237);

void SetLayoutShapeParts(shape layout, ByteOffset startOffset, ByteOffset endOffset,
	shape insert) InlineCode(0x1242);

long GetLayoutParts(shape layout, ByteOffset startOffset, ByteOffset endOffset, void *text,
	long *styleRunCount, short styleRunLengths[], style styles[], long *levelRunCount,
	short levelRunLengths[], short levels[]) InlineCode(0x1238);

shape GetLayoutShapeParts(shape layout, ByteOffset startOffset, ByteOffset endOffset,
	shape dest) InlineCode(0x1243);

long GetLayoutGlyphs(shape layout, Glyphcode *glyphs, point positions[], long advance[],
	point tangents[], long *runCount, short styleRuns[], style glyphStyles[]) InlineCode(0x1185);

ByteOffset HitTestLayout(shape layout, const point *hitDown, HighlightType highlightType,
	LayoutHitInfo *hitInfo, shape hitTrackingArea) InlineCode(0x1187);

shape GetLayoutHighlight(shape layout, ByteOffset startOffset, ByteOffset endOffset,
	HighlightType highlightType, shape highlight) InlineCode(0x1188);

shape GetLayoutVisualHighlight(shape layout, ByteOffset startOffset, long startLeadingEdge,
	ByteOffset endOffset, long endLeadingEdge, HighlightType highlightType, shape highlight) InlineCode(0x1208);

shape GetLayoutCaret(shape layout, ByteOffset offset, HighlightType highlightType,
	CaretType caretType, shape caret) InlineCode(0x1189);

ByteOffset GetLayoutBreakOffset(shape layout, ByteOffset startOffset, fixed lineWidth,
	long hyphenationCount, const ByteOffset hyphenationPoints[], boolean *startIsStaked,
	ByteOffset *priorStake, ByteOffset *nextStake) InlineCode(0x1239);

fixed GetLayoutRangeWidth(shape layout, ByteOffset startOffset, ByteOffset endOffset,
	shape supplementaryText) InlineCode(0x123A);

shape NewLayoutFromRange(shape layout, ByteOffset startOffset, ByteOffset endOffset,
	const LayoutOptions *layoutOptions, shape supplementaryText) InlineCode(0x123B);

shape GetCaretAngleArea(shape layout, const point *hitPoint, HighlightType highlightType,
	shape caretArea, short *returnedRise, short *returnedRun) InlineCode(0x118A);

void GetStyleBaselineDeltas(style baseStyle, BaselineType baseType,
	BaselineDeltas returnedDeltas) InlineCode(0x118B);

ByteOffset GetRightVisualOffset(shape layout, ByteOffset currentOffset) InlineCode(0x118D);
ByteOffset GetLeftVisualOffset(shape layout, ByteOffset currentOffset) InlineCode(0x118E);

void GetCompoundCharacterLimits(shape layout, ByteOffset trial, ByteOffset *minOffset,
	ByteOffset *maxOffset, boolean *onBoundary) InlineCode(0x118F);

void GetOffsetGlyphs(shape layout, ByteOffset trial, long leadingEdge,
	LayoutOffsetState *offsetState, unsigned short *firstGlyph, unsigned short *secondGlyph) InlineCode(0x1209);

void GetGlyphOffsets(shape layout, ByteOffset trial, long onLeftTop, ByteOffset *offset,
	boolean *leadingEdge, boolean *wasRealCharacter) InlineCode(0x120A);

void GetLayoutSpan(shape layout, fixed *lineAscent, fixed *lineDescent) InlineCode(0x1190);
void SetLayoutSpan(shape layout, fixed lineAscent, fixed lineDescent) InlineCode(0x1191);

void SetStyleRunControls(style target, const RunControls *runControls) InlineCode(0x1192);
void SetStyleRunPriorityJustOverride(style target,
	const PriorityJustificationOverride *priorityJustificationOverride) InlineCode(0x1193);
void SetStyleRunGlyphJustOverrides(style target, long count,
	const GlyphJustificationOverride glyphJustificationOverrides[]) InlineCode(0x1194);
void SetStyleRunGlyphSubstitutions(style target, long count,
	const GlyphSubstitution glyphSubstitutions[]) InlineCode(0x1195);
void SetStyleRunFeatures(style target, long count, const RunFeature runFeatures[]) InlineCode(0x1196);
void SetStyleRunKerningAdjustments(style target, long count,
	const KerningAdjustment kerningAdjustments[]) InlineCode(0x1197);

void SetShapeRunControls(shape target, const RunControls *runControls) InlineCode(0x1198);
void SetShapeRunPriorityJustOverride(shape target,
	const PriorityJustificationOverride *priorityJustificationOverride) InlineCode(0x1199);
void SetShapeRunGlyphJustOverrides(shape target, long count,
	const GlyphJustificationOverride glyphJustificationOverrides[]) InlineCode(0x119A);
void SetShapeRunGlyphSubstitutions(shape target, long count,
	const GlyphSubstitution glyphSubstitutions[]) InlineCode(0x119B);
void SetShapeRunFeatures(shape target, long count, const RunFeature runFeatures[]) InlineCode(0x119C);
void SetShapeRunKerningAdjustments(shape target, long count,
	const KerningAdjustment kerningAdjustments[]) InlineCode(0x119D);

long GetStyleRunControls(style source, RunControls *runControls) InlineCode(0x119E);
long GetStyleRunPriorityJustOverride(style source,
	PriorityJustificationOverride *priorityJustificationOverride) InlineCode(0x119F);
long GetStyleRunGlyphJustOverrides(style source,
	GlyphJustificationOverride glyphJustificationOverrides[]) InlineCode(0x11A0);
long GetStyleRunGlyphSubstitutions(style source, GlyphSubstitution glyphSubstitutions[]) InlineCode(0x11A1);
long GetStyleRunFeatures(style source, RunFeature runFeatures[]) InlineCode(0x11A2);
long GetStyleRunKerningAdjustments(style source, KerningAdjustment kerningAdjustments[]) InlineCode(0x11A3);

long GetShapeRunControls(shape source, RunControls *runControls) InlineCode(0x11A4);
long GetShapeRunPriorityJustOverride(shape source,
	PriorityJustificationOverride *priorityJustificationOverride) InlineCode(0x11A5);
long GetShapeRunGlyphJustOverrides(shape source,
	GlyphJustificationOverride glyphJustificationOverrides[]) InlineCode(0x11A6);
long GetShapeRunGlyphSubstitutions(shape source, GlyphSubstitution glyphSubstitutions[]) InlineCode(0x11A7);
long GetShapeRunFeatures(shape source, RunFeature runFeatures[]) InlineCode(0x11A8);
long GetShapeRunKerningAdjustments(shape source, KerningAdjustment kerningAdjustments[]) InlineCode(0x11A9);

#ifdef __cplusplus
}
#endif

#undef InlineCode

#endif layoutRoutinesIncludes
