/* graphics:
	math routines
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

#ifndef mathRoutinesIncludes
	#define mathRoutinesIncludes

	#ifndef mathTypesIncludes
		#include "math types.h"
	#endif

	#ifdef appleInternal
		#define InlineCode(x)
	#endif
	#ifndef InlineCode
		#define InlineCode(x)	= {0x303C, x, 0xA832}
	#endif

	#ifdef __cplusplus
	extern "C" {
	#endif

	mapping *CopyToMapping(mapping *target, const mapping *source) InlineCode(0x2013);
	mapping *InvertMapping(mapping *target, const mapping *source) InlineCode(0x2014);
	mapping *MapMapping(mapping *target, const mapping *source) InlineCode(0x2015);
	mapping *MoveMapping(mapping *target, fixed hOffset, fixed vOffset) InlineCode(0x2016);
	mapping *MoveMappingTo(mapping *target, fixed hPosition, fixed vPosition) InlineCode(0x2017);
	mapping *NormalizeMapping(mapping *target) InlineCode(0x2018);
	mapping *RotateMapping(mapping *target, fixed angle, fixed xCenter, fixed yCenter) InlineCode(0x2019);
	mapping *ScaleMapping(mapping *target, fixed hFactor, fixed vFactor, fixed xCenter, fixed yCenter) InlineCode(0x201A);
	mapping *ResetMapping(mapping *target) InlineCode(0x201B);
	mapping *SkewMapping(mapping *target, fixed skewX, fixed skewY, fixed xCenter, fixed yCenter) InlineCode(0x201C);

	void MapPoints(const mapping *source, long count, point vector[]) InlineCode(0x201D);

	short FirstBit(unsigned long) InlineCode(0x201E);
	short WideScale(const wide *source) InlineCode(0x201F);

	short LinearRoot(fixed first, fixed last, fract t[]) InlineCode(0x2020);
	short QuadraticRoot(fixed first, fixed control, fixed last, fract t[]) InlineCode(0x2021);

	point *PolarToPoint(const polar *, point *) InlineCode(0x2022);
	polar *PointToPolar(const point *, polar *) InlineCode(0x2023);

	fract FractCubeRoot(fract source) InlineCode(0x2024);
	fract FractDivide(fract dividend, fract divisor) InlineCode(0x2025);
	fract FractMultiply(fract multiplicand, fract multiplier) InlineCode(0x2026);
	fract FractSineCosine(fixed degrees, fract *cosine) InlineCode(0x2027);
	fract FractSquareRoot(fract source) InlineCode(0x2028);

	fixed FixedDivide(fixed dividend, fixed divisor) InlineCode(0x2029);
	fixed FixedMultiply(fixed multiplicand, fixed multiplier) InlineCode(0x202A);

	/* This next call is (source * multiplier / divisor) -- it avoids underflow, overflow by using wides */
	long MultiplyDivide(long source, long multiplier, long divisor) InlineCode(0x202B);

	unsigned long Magnitude(long deltaX, long deltaY) InlineCode(0x202C);
	long VectorMultiplyDivide(long count, const long *vector1, long step1, const long *vector2, long step2, long divisor) InlineCode(0x202D);

	wide *WideAdd(wide *target, const wide *source) InlineCode(0x202E);
	short WideCompare(const wide *target, const wide *source) InlineCode(0x202F);
	wide *WideNegate(wide *target) InlineCode(0x2030);
	wide *WideShift(wide *target, long shift) InlineCode(0x2031);
	unsigned long WideSquareRoot(const wide *source) InlineCode(0x2032);
	wide *WideSubtract(wide *target, const wide *source) InlineCode(0x2033);

	wide *WideMultiply(long multiplicand, long multiplier, wide *target) InlineCode(0x2034);
	long WideDivide(const wide *dividend, long divisor, long *remainder) InlineCode(0x2035);		/* returns the quotient */
	wide *VectorMultiply(long count, const long *vector1, long step1, const long *vector2, long step2, wide *dot) InlineCode(0x2036);
	wide *WideWideDivide(wide *dividend, long divisor, long *remainder);						/* quotient replaces dividend */

	unsigned long RandomBits(long count, long focus) InlineCode(0x2037);
	void SetRandomSeed(const wide *seed) InlineCode(0x2038);
	wide *GetRandomSeed(wide *seed) InlineCode(0x2039);

	#ifdef __cplusplus
	}
	#endif

	#undef InlineCode

	#define FixedRound(a)		((short)((fixed)(a) + fix1/2 >> 16))
	#define FixedSquareRoot(a)	((fixed)FractSquareRoot(a) + 64 >> 7)
	#define FixedTruncate(a)		((short)((fixed)(a) >> 16))

	#define FixedToFract(a)		((fract)(a) << 14)
	#define FractToFixed(a)		((fixed)(a) + 8192L >> 14)

	#define FixedToInt(a)		((short)((fixed)(a) + fix1/2 >> 16))
	#define IntToFixed(a)		((fixed)(a) << 16)

	#define FixedToFloat(a)		((float)(a) / fix1)
	#define FloatToFixed(a) 		((fixed)((float)(a) * fix1))

	#define FractToFloat(a)		((float)(a) / frac1)
	#define FloatToFract(a)		((fract)((float)(a) * frac1))

	#define ColorToFract(a)		(((fract) (a) << 14) + ((fract)(a) + 2 >> 2))
	#define FractToColor(a)		((colorValue) ((a) - ((a) >> 16) + 8191 >> 14))

	#define ff(a)				IntToFixed(a)
	#define fl(a)				FloatToFixed(a)

#endif
