/* memory manager:
	public interface definition
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

#ifndef memoryRoutinesIncludes
#define memoryRoutinesIncludes

	#ifndef memoryTypesIncludes
		#include "memory types.h"
	#endif
	#ifndef mathTypesIncludes
		#include "math types.h"
	#endif

	#ifdef appleInternal
		#if defined(insideLoader) || defined(insideDispatcher)
			#define backpatchMemoryManager
		#else
			#define InlineCode(x)
		#endif
	#endif
	#ifndef InlineCode
		#ifdef backpatchMemoryManager
			#define InlineCode(x)	= {0x303C, 0x0800 + (x), 0xA832}
		#else
			#define InlineCode(x)	= {0x7000 + (x), 0xA832}
		#endif
	#endif

	#ifdef __cplusplus
	extern "C" {
	#endif

	heap *GetCurrentHeap(void) InlineCode(0);
	void SetCurrentHeap(const heap *newHeap) InlineCode(1);

	long GetMinimumHeapSize(void) InlineCode(2);
	long GetHeaps(heap *parent, long index, long count, heap *heaps[]) InlineCode(3);

	heap *NewHeap(void *memory, long size) InlineCode(4);
	void GrowHeap(void *newMemory, long size) InlineCode(5);
	boolean ShrinkHeap(void *oldMemory, long size) InlineCode(6);
	long GetHeapSize(const heap *source) InlineCode(7);
	void *CompactHeap(heap *target, void *address) InlineCode(8);
	long GetHeapBlocks(heap *source, long types, long index, long count, void *blocks[]) InlineCode(9);
	void DisposeHeap(heap *target) InlineCode(10);

	void *NewDirectBlock(long size) InlineCode(11);
	long GetDirectBlockSize(const void *block) InlineCode(12);
	void *SetDirectBlockSize(void *block, long newSize) InlineCode(13);
	long DisposeDirectBlock(void *block) InlineCode(14);

	void *NewWeakBlock(long size) InlineCode(15);
	void *NewFailBlock(long size) InlineCode(16);
	void *SetFailBlockSize(void *block, long newSize) InlineCode(60);

	void *NewIndirectBlock(long size) InlineCode(17);
	long GetIndirectBlockSize(const void *block) InlineCode(18);
	void *SetIndirectBlockSize(void *block, long newSize) InlineCode(19);
	void *RecoverIndirectBlock(const void *block) InlineCode(20);
	long DisposeIndirectBlock(void *block) InlineCode(21);
	void ExchangeIndirectBlocks(void *block1, void *block2) InlineCode(22);
	boolean IsMasterPointer(const void *source) InlineCode(23);

	long GetBlockBusy(const void *block) InlineCode(24);
	long GetBlockFlags(const void *block) InlineCode(25);
	void *GetBlockParent(const void *block) InlineCode(26);
	long GetBlockType(const void *block) InlineCode(27);

#ifdef appleInternal
	void xSetBlockBusy(void *block, long busyCount) InlineCode(28);
#else
	void SetBlockBusy(void *block, long busyCount) InlineCode(28);
#endif
	void SetBlockFlags(void *block, long newFlags) InlineCode(29);
	void SetBlockParent(void *block, void *parent) InlineCode(30);
	void SetBlockType(void *block, long newTypeID) InlineCode(31);

	long GetBlockDelta(const void *block) InlineCode(32);
	long GetBlockPhysicalSize(const void *block) InlineCode(33);

	void IncrementBlockBusy(void *block) InlineCode(34);
	void DecrementBlockBusy(void *block) InlineCode(35);

	void *NewTemporaryBlock(long size) InlineCode(36);
	void DisposeTemporaryBlock(void *block) InlineCode(37);

	long GetBlockBytes(const void *block, long index, long count, void *destination) InlineCode(39);
	void *SetBlockBytes(void *block, long index, long oldCount, long newCount, const void *source)  InlineCode(40);

	heap *FindHeap(const void *address) InlineCode(41);
	void *FindBlock(const void *address) InlineCode(42);

	void MoveBytes(const void *source, void *destination, long length) InlineCode(43);
	void CopyBytes(const void *source, void *destination, long length) InlineCode(44);
	void FillBytes(void *destination, long length, long pattern) InlineCode(45);
	boolean EqualBytes(const void *address1, const void *address2, long length) InlineCode(46);

	void MoveBits(const void *src, long srcInset, void *dst, long dstInset, long length) InlineCode(47);
	void FillBits(void *dst, long dstInset, long length, long pattern) InlineCode(48);

	void InitializeMemoryManager(void) InlineCode(58);

#ifdef appleInternal
	heap **GetFirstHeapPtr(void) InlineCode(59);

	/* IMPORTANT: dont add any functions after this except for GetMemoryTrapAddress */
	#ifdef insideStartup
		#undef InlineCode
		#define InlineCode(x)	= {0x7000 + (x), 0xA832}
	#endif
#endif

	void *GetMemoryTrapAddress(long selector) InlineCode(57);

	#ifdef __cplusplus
	}
	#endif

	#undef  InlineCode

#endif
