/* graphics:	
	font file structures
	
	by	Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good,
		Barton House, Robert Johnson, Eric Mader, Keith McGreggor,
		Dave Opstad, Mike Reed, Oliver Steele, David Van Brink, Chris Yerga
		
	Copyright 1987 - 1993 Apple Computer, Inc.  All rights reserved.
*/

#ifndef sfntTypesIncludes
#define sfntTypesIncludes

#ifndef mathTypesIncludes
	#include "math types.h"
#endif

#ifndef anyNumber
	#define anyNumber 1
#endif

/* ---------------- */

#ifndef sharedFontTypeDefines
	#define sharedFontTypeDefines
	
	typedef long fontTableTag;
	typedef long fontVariationTag;
	typedef long fontFormatTag;
	typedef long fontStorageTag;

	typedef struct {
		fontVariationTag	name;
		fixed				value;
	} fontVariation;
	typedef fontVariation fontDescriptor;

	typedef struct {
		unsigned short	featureType;
		unsigned short	setting;
	} fontRunFeature;

	typedef struct {
		unsigned short	setting;
		unsigned short	nameID;
	} fontFeatureSetting;
#endif


/* ---------------- */

typedef struct {
	fontTableTag		tableTag;
	unsigned long		checkSum;
	unsigned long		offset;
	unsigned long		length;
} sfntDirectoryEntry;

/*
 *	The search fields limits numOffsets to 4096.
 */

typedef struct {
	fontFormatTag			format;
	unsigned short			numOffsets;			/* number of tables */
	unsigned short			searchRange;			/* (max2 <= numOffsets)*16 */
	unsigned short			entrySelector;			/* log2(max2 <= numOffsets) */
	unsigned short			rangeShift;			/* numOffsets*16-searchRange*/
	sfntDirectoryEntry		table[anyNumber];		/* table[numOffsets] */
} sfntDirectory;

#define sizeof_sfntDirectory			12

/*
 *		Cmap - character id to glyph id mapping
 */

#define	cmapTableTag		0x636d6170	/* 'cmap' */

typedef struct {
	unsigned short			format;
	unsigned short			length;
	unsigned short			languageID;	/* base-1 */
} sfntCMapSubHeader;

#define sizeof_sfntCMapSubHeader		6

typedef struct {
	unsigned short			platformID;	/* base-0 */
	unsigned short			scriptID;		/* base-0 */
	unsigned long			offset;
} sfntCMapEncoding;

#define sizeof_sfntCMapEncoding	8

typedef struct {
	unsigned short		version;
	unsigned short		numTables;
	sfntCMapEncoding 	encoding[anyNumber];
} sfntCMapHeader;

#define sizeof_sfntCMapHeader			4

/*
 * 		Name table
 */

#define	nameTableTag			0x6e616d65		/* 'name' */

typedef struct {
	unsigned short 			platformID;		/* base-0 */
	unsigned short 			scriptID;			/* base-0 */
	unsigned short 			languageID;		/* base-0 */
	unsigned short 			nameID;			/* base-0 */
	unsigned short 			length;
	unsigned short 			offset;
} sfntNameRecord;

#define sizeof_sfntNameRecord			12

typedef struct {
	unsigned short 			format;
	unsigned short 			count;
	unsigned short 			stringOffset;
	sfntNameRecord		record[anyNumber];
} sfntNameHeader;

#define sizeof_sfntNameHeader		6


/*
 * 		Fvar table - font variations
 */

#define variationTableTag			0x66766172		/* 'fvar' */

/*	These define each font variation */

typedef struct {
	fontVariationTag		axisTag;
	fixed					minValue;
	fixed					defaultValue;
	fixed					maxValue;
	short				flags;
	short				nameID;
} sfntVariationAxis;

#define sizeof_sfntVariationAxis		20

/*	These are named locations in style-space for the user */

typedef struct {
	short				nameID;
	short				flags;
	fixed					coord[anyNumber];		/* [axisCount] */
	/* room to grow since the header carries a tupleSize field */
} sfntInstance;

#define sizeof_sfntInstance			4

typedef struct {
	fixed					version;				/* 1.0 fixed */
	unsigned short			offsetToData;			/* to first axis = 16*/
	unsigned short			countSizePairs;			/* axis+inst = 2 */
	unsigned short			axisCount, axisSize;
	unsigned short			instanceCount, instanceSize;
	/*	other <count,size> pairs */
	sfntVariationAxis		axis[anyNumber];		/* [axisCount] */
	sfntInstance			instance[anyNumber];	/* [instanceCount] */
	/*	other arrays of data */
} sfntVariationHeader;

#define sizeof_sfntVariationHeader		16

/*
 * 		Fdsc table - font descriptor
 */

#define descriptorTableTag		0x66647363		/* 'fdsc' */

typedef struct {
	fixed					version;		/* 1.0 fixed */
	long					descriptorCount;
	fontDescriptor			descriptor[anyNumber];
} sfntDescriptorHeader;

#define sizeof_sfntDescriptorHeader		8

/*
 * Feat Table - layout feature table
 */

#define	featureTableTag			0x66656174	/* 'feat' */

typedef struct {
	unsigned short		runCount;
	unsigned short		nameID;
	unsigned short		flags;
	unsigned short		groupID;
	long				offsetToRuns;
} sfntFeatureSet;

typedef struct {
	unsigned short		featureType;
	unsigned short		settingCount;
	long				offsetToSettings;
	unsigned short		featureFlags;
	unsigned short		nameID;
} sfntFeatureName;

typedef struct {
	long				version;			/* 1.0 */
	unsigned short		featureNameCount;
	unsigned short		featureSetCount;
	long				offsetToFeatureSets;
	sfntFeatureName	names[anyNumber];
	sfntFeatureSet		sets[anyNumber];
	fontFeatureSetting	settings[anyNumber];
	fontRunFeature		runs[anyNumber];
} sfntFeatureHeader;

/*
 * 		OS/2 Table
 */

#define os2TableTag					0x4f532f32		/* 'OS/2' */

/* need to put os/2 structure definition here */

#endif
