/*****************************************

	synonym tags:

	These are the data structures for Graphics 
	synonyms stored in tags.

	Copyright  1991, 1992 Apple Computer Inc.  All rights reserved

******************************************/

#ifndef synonymTags
#define synonymTags


	#ifndef mathTypesIncludes
		#include "math types.h"
	#endif

	#ifndef anyNumber
		#define anyNumber 1
	#endif

	#ifndef graphicsTypesIncludes
		#include "graphics types.h"
	#endif

	#ifndef fontTypesIncludes
		#include "font types.h"
	#endif


	#define postScriptTag		0x706f7374	/* 'post' */

	#define postControlTag 0x70736374	/* 'psct' */

	typedef enum {
		noSave = 1					/* don't do save-restore around PS data */
	} psStateFlags;

	typedef struct {
		long		flags;				/* PostScript state flags */
		long		numFonts;				/* number of fonts in the list */
		short	name;				/* fontName */
		short	platform;				/* fontPlatform */
		short	script;				/* fontScript */
		short	language;				/* fontLanguage */
		char		fontList[anyNumber];	/* packed list of pascal strings, containing screen font names */
	} postControl;


	#define dashSynonymTag 0x73647368	/* 'sdsh' */

	typedef struct {
		long		size;					/* number of elements in array */
		fixed		dashLength[anyNumber];	/* Array of dash lengths */
	} dashSynonym;


	#define lineCapSynonymTag 0x6c636170	/* 'lcap' */

	typedef enum {
		buttCap,
		roundCap,
		squareCap,
		triangleCap
	} lineCaps;

	typedef long	lineCapSynonym;							/* line cap type */


	#define cubicSynonymTag 0x63756278	/* 'cubx' */
	
	typedef enum {
	
		IgnoreFlag =	0x0000,		/* Ignore this word, get next one */
		LineToFlag = 	0x0001,		/* Draw a line to point following this flag */
		CurveToFlag = 	0x0002,		/* Draw a curve through the 3 points following this flag */
		MoveToFlag = 	0x0003,		/* Start a new contour at the point following this flag */
		ClosePathFlag =	0x0004		/* Close the contour */
		
	} cubicSynonym;

	#define cubicInstructionMask 0x000F		/* low four bits are point instructions */
	
	typedef short cubicSynonymFlags;	/* Low four bits are instruction (moveto, lineto, curveto, closepath) */


	#define patternSynonymTag 0x7074726E	/* 'ptrn' */

	typedef enum {
		hatch,
		crossHatch
	} patterns;

	typedef struct {
		long	patternType;			/* one of the patterns: hatch or crossHatch */
		fixed		angle;				/* angle at which pattern is drawn */
		fixed		spacing;				/* distance between two parallel pattern lines */
		fixed		thickness;				/* thickness of the pattern */
		point		anchorPoint;			/* point with with respect to which pattern position is calculated */
	} patternSynonym;


#endif
