/* ------------------------------------------------------------------------------

	FILENAME
		JobFormatModeLibrary.c
		
	DESCRIPTION
		Handy job format mode functions.
		
	COPYRIGHT
		  Apple Computer, Inc. 1992
		 All rights reserved. 
	
-------------------------------------------------------------------------------- */


#include <PrintingManager.h>



/* Wrappers for JobFormatModeQuery() */


/****************************************************************************************

							GetJobFormatLineConstraint
							
	function :
					Returns position constraint table for direct mode line drawing
	parameters :
					the job and optional handle to hold the position constraint table
					
****************************************************************************************/
PositionConstraintTableHdl GetJobFormatLineConstraint(Job theJob, PositionConstraintTableHdl hPositionConstraintTable) {
	
	JobFormatModeQuery(theJob, getJobFormatLineConstraintQuery, NULL, &hPositionConstraintTable);
	
	return hPositionConstraintTable;
}


/****************************************************************************************

							GetJobFormatFonts
							
	function :
					Returns font table for direct mode
	parameters :
					the job and optional handle to hold the font table
					
****************************************************************************************/
FontTableHdl GetJobFormatFonts(Job theJob, FontTableHdl hFontTable) {
	
	JobFormatModeQuery(theJob, getJobFormatFontsQuery, NULL, &hFontTable);
	
	return hFontTable;
}


/****************************************************************************************

							GetJobFormatFontCommonStyles
							
	function :
					Returns style name table for direct mode
	parameters :
					the job and optional handle to hold the style name table
					
****************************************************************************************/
StyleNameTableHdl GetJobFormatFontCommonStyles(Job theJob, font theFont, StyleNameTableHdl hStyleTable) {
	
	JobFormatModeQuery(theJob, getJobFormatFontCommonStylesQuery, theFont, &hStyleTable);
	
	return hStyleTable;
}


/****************************************************************************************

							GetJobFormatFontConstraint
							
	function :
					Returns position constraint table for direct mode fonts
	parameters :
					the job and optional handle to hold the position constraint table
					
****************************************************************************************/
PositionConstraintTableHdl GetJobFormatFontConstraint(Job theJob, font theFont, PositionConstraintTableHdl hPositionConstraintTable) {
	
	JobFormatModeQuery(theJob, getJobFormatFontConstraintQuery, theFont, &hPositionConstraintTable);
	
	return hPositionConstraintTable;
}


/****************************************************************************************

							SetStyleJobFormatCommonStyle
							
	function :
					Returns the style with the additional font style information set
	parameters :
					the job and a style with the font and size values set
					
****************************************************************************************/
style SetStyleJobFormatCommonStyle(Job theJob, Str255 theStyleName, style theStyle) {
	
	JobFormatModeQuery(theJob, setStyleJobFormatCommonStyleQuery, theStyleName, theStyle);
	
	return theStyle;
}
