/* graphics libraries:
	QuickDraw bitblt routines
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

#include <Memory.h>
#include <ToolUtils.h>
#include "qd globals.h"
#include "qd emulation.h"
#include "qd utilities.h"
#include "qd library.h"
#include "graphics libraries.h"

static BitMap iconBits = {0, 4, 0, 0, 32, 32};
static BitMap sicnBits = {0, 2, 0, 0, 16, 16};
#define sicnLen (16*16/sizeof(char))

static viewDevice offDevice;
static viewPort offPort;
static viewGroup offGroup;
static transform offXform;

/* don't use me for color yet!  I don't even check to see if I should post a warning. */
/* doesn't deal with the distinction between srcBits.bounds and srcRect yet */
/* needs to ignore ports clipRgn */
pascal void StdBits(const BitMap *srcBits, const Rect *srcRect, const Rect *dstRect, short mode, RgnHandle maskRgn)
{
	shape qdBitmapShape;
	fixed scaleX = FixRatio(dstRect->right - dstRect->left, srcRect->right - srcRect->left);
	fixed scaleY = FixRatio(dstRect->bottom - dstRect->top, srcRect->bottom - srcRect->top);

	qdBitmapShape = BitMapToShape(srcBits);
	SetShapeAttributes(qdBitmapShape, GetShapeAttributes(qdBitmapShape) & ~mapTransformShape);	/* */
	IgnoreGraphicsNotice(transform_mapping_unaffected);
	if (scaleX != fix1 || scaleY != fix1)
		ScaleShape(qdBitmapShape, scaleX, scaleY, 0, 0);
	MoveShapeTo(qdBitmapShape, IntToFixed(dstRect->left), IntToFixed(dstRect->top));
	PopGraphicsNotice();
	SetShapeCommonTransfer(qdBitmapShape, ConvertFromQDMode(mode));
	if (maskRgn)
		SetShapeClip(qdBitmapShape, maskRgn);
	DrawQDShape(qdBitmapShape, paint, false, false);
	DisposeShape(qdBitmapShape);
}

pascal void CopyBits(const BitMap *srcBits, const BitMap *dstBits, const Rect *srcRect, const Rect *dstRect, short mode, RgnHandle maskRgn)
{
	bitmap dstBitmap;
	shape qdBitmapShape, dstShape;

	ConvertFromQDBitmap(dstBits, &dstBitmap);
	dstShape = NewBitmap(&dstBitmap, nil);
	if (!offGroup) {
		offGroup = NewViewGroup();
		offDevice = NewViewDevice(offGroup, dstShape);
		DisposeShape(dstShape);
		offPort = NewViewPort(offGroup);
		offXform = NewTransform();
		SetTransformViewPort(offXform, offPort);
	}

	SetViewPortClip(offPort, maskRgn);
	qdBitmapShape = BitMapToShape(srcBits);
	SetShapeTransform(qdBitmapShape, offXform);
	ScaleShape(qdBitmapShape,
		FixRatio(dstRect->right - dstRect->left, srcRect->right - srcRect->left),
		FixRatio(dstRect->bottom - dstRect->top, srcRect->bottom - srcRect->top), 0, 0);
	MoveShapeTo(qdBitmapShape, IntToFixed(dstRect->left), IntToFixed(dstRect->top));
	SetShapeCommonTransfer(qdBitmapShape, mode);
	if (maskRgn)
		SetShapeClip(qdBitmapShape, maskRgn);

	DrawQDShape(qdBitmapShape, fill, false, false);
	DisposeShape(qdBitmapShape);
}

/* use stdBits */
pascal void ScrollRect(const Rect *theRect, short dh, short dv, RgnHandle updateRgn)
{
	Rect srcRect, dstRect;
	#pragma unused (updateRgn)

	srcRect = dstRect = *theRect;
	OffsetRect(&dstRect, dh, dv);
	SectRect(&dstRect, theRect, &dstRect);
	PostGraphicsError(functionality_unimplemented);
	/*** somehow get the bitmap for the current port */
}

/* needs to lock the handle */
/* needs to use stdBits */
pascal void PlotIcon(const Rect *theRect, Handle theIcon)
{
	char flags;

	flags = HGetState(theIcon);
	HLock(theIcon);
	iconBits.baseAddr = *theIcon;
	StdBits(&iconBits, &iconBits.bounds, theRect, copyMode, nil);
	HSetState(theIcon, flags);
}

/*** should use stdBits */
pascal void PlotSicn(const Rect *theRect, Handle theSicn, short index)
{
	char flags;

	flags = HGetState(theSicn);
	HLock(theSicn);
	sicnBits.baseAddr = *theSicn + index*sicnLen;
	StdBits(&sicnBits, &sicnBits.bounds, theRect, copyMode, nil);
	HSetState(theSicn, flags);
}
