/* graphics libraries:
	QuickDraw specific initialization routines
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

#include "qd globals.h"
#include "qd emulation.h"
#include "graphics libraries.h"

emulationPortPtr gEmulationPortPtr;
const Pattern
	whitePat	= {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},
	ltGrayPat	= {0x77, 0xdd, 0x77, 0xdd, 0x77, 0xdd, 0x77, 0xdd},
	grayPat	= {0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa, 0x55, 0xaa},
	dkGrayPat	= {0x22, 0x88, 0x822, 0x88, 0x22, 0x88, 0x22, 0x88},
	blackPat	= {0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff};

static emulationGrafPort emulationPortRecord;
const Point oneOne = {1, 1};
shape openPicture = nil;
shape openRgn = nil;
shape openPoly = nil;
boolean lastRgnOpClosed = true;
shape picturePlayBack = nil;
style qdEraseStyle, qdPatternStyle, qdFillStyle, qdTextStyle;
ink qdPenInk, qdFillInk, qdEraseInk, qdInvertInk, qdTextInk;
shape qdLineShape, qdTextShape, qdRectShape, qdRRectShape, qdOvalShape, qdArcShape;
shape qdFrameRectShape, qdPaintRectShape, qdEraseRectShape, qdInvertRectShape, qdFillRectShape;

pascal void (*stdTextProc)(short count, const StringPtr textAddr, Point numer, Point denom);
pascal void (*stdLineProc)(Point);
pascal void (*stdRectProc)(GrafVerb, const Rect *);
pascal void (*stdRRectProc)(GrafVerb, const Rect *, short ovalWidth, short ovalHeight);
pascal void (*stdOvalProc)(GrafVerb, const Rect *);
pascal void (*stdArcProc)(GrafVerb, const Rect *, short startAngle, short arcAngle);
pascal void (*stdPolyProc)(GrafVerb, PolyHandle);
pascal void (*stdRgnProc)(GrafVerb, RgnHandle);
pascal void (*stdBitsProc)(const BitMap *, const Rect *srcRect, const Rect *dstRect, short mode, RgnHandle maskRgn);
pascal void (*stdCommentProc)(short kind, short dataSize, Handle dataHandle);
pascal short (*stdTxMeasProc)(short count, const StringPtr textAddr, Point *numer, Point *denom, FontInfo *info);


void InitGraf(void)
{
#undef thePort
#undef InitGraf
	gEmulationPortPtr = &emulationPortRecord;
	InitGraf(&qd.thePort);
	InitCommonColors();

	qdEraseStyle = NewStyle();
		SetStyleAttributes(qdEraseStyle, deviceGridStyle | insideFrameStyle);
	qdPatternStyle = NewStyle();
		SetStyleAttributes(qdPatternStyle, deviceGridStyle | insideFrameStyle);
	qdFillStyle = NewStyle();
		SetStyleAttributes(qdFillStyle, deviceGridStyle);
	qdTextStyle = NewStyle();
		SetStyleAttributes(qdTextStyle, deviceGridStyle);
	qdPenInk = NewInk();
	qdFillInk = CopyToInk(nil, qdPenInk);
	qdEraseInk = NewInk();
		SetInkCommonColor(qdEraseInk, gxWhite);
	qdInvertInk = NewInk();
		SetInkCommonTransfer(qdInvertInk, highlightMode);
	qdTextInk = NewInk();

	IgnoreGraphicsWarning(new_shape_contains_invalid_data);

	qdLineShape = NewShape(lineType);
		/*SetShapeStyle(qdLineShape, qdPatternStyle);*/
	qdTextShape = NewShape(textType);
		SetShapeStyle(qdTextShape, qdTextStyle);
		DisposeStyle(qdTextStyle);
		TextFont(1); /* applFont */
	qdRectShape = NewShape(rectangleType);
	qdRRectShape = NewShape(pathType);
	qdOvalShape = NewShape(pathType);
	qdArcShape = NewShape(curveType);

	qdFrameRectShape = NewShape(rectangleType);
		SetShapeFill(qdFrameRectShape, closedFrameFill);
		SetShapeStyle(qdFrameRectShape, qdPatternStyle);
		SetShapeInk(qdFrameRectShape, qdPenInk);
	qdPaintRectShape = NewShape(rectangleType);
		SetShapeStyle(qdPaintRectShape, qdPatternStyle);
		SetShapeInk(qdPaintRectShape, qdPenInk);
	qdEraseRectShape = NewShape(rectangleType);
		SetShapeStyle(qdEraseRectShape, qdEraseStyle);
		SetShapeInk(qdEraseRectShape, qdEraseInk);
	qdInvertRectShape = NewShape(rectangleType);
		SetShapeStyle(qdInvertRectShape, qdFillStyle);
		SetShapeInk(qdInvertRectShape, qdInvertInk);
	qdFillRectShape = NewShape(rectangleType);
		SetShapeStyle(qdFillRectShape, qdFillStyle);
		SetShapeInk(qdFillRectShape, qdPenInk);

	PopGraphicsWarning();

	stdTextProc = StdText;
	stdLineProc = StdLine;
	stdRectProc = StdRect;
	stdRRectProc = StdRRect;
	stdOvalProc = StdOval;
	stdArcProc = StdArc;
	stdPolyProc = StdPoly;
	stdRgnProc = StdRgn;
	stdBitsProc = StdBits;
	stdCommentProc = StdComment;
	stdTxMeasProc = StdTxMeas;

	PenNormal();
}


void ExitGraf(void)
{
	DisposeStyle(qdEraseStyle);
	DisposeStyle(qdPatternStyle);
	DisposeStyle(qdFillStyle);
	DisposeInk(qdPenInk);
	DisposeInk(qdEraseInk);
	DisposeInk(qdInvertInk);
	DisposeInk(qdTextInk);
	DisposeInk(qdFillInk);
	DisposeShape(qdLineShape);
	DisposeShape(qdTextShape);
	DisposeShape(qdRectShape);
	DisposeShape(qdRRectShape);
	DisposeShape(qdOvalShape);
	DisposeShape(qdArcShape);
	DisposeShape(qdFrameRectShape);
	DisposeShape(qdPaintRectShape);
	DisposeShape(qdEraseRectShape);
	DisposeShape(qdInvertRectShape);
	DisposeShape(qdFillRectShape);
}


/* most of this can be done in InitGraf instead */
void QDSetPort(const void *port)
{
	transform xform;
	
#undef SetPort
#undef GrafPtr
	SetPort((GrafPtr) port);
	xform = GetShapeTransform(GetDefaultShape(lineType));

	IgnoreGraphicsNotice(transform_already_set);
	SetShapeTransform(qdLineShape, xform);
	SetShapeTransform(qdTextShape, xform);
	SetShapeTransform(qdRectShape, xform);
	SetShapeTransform(qdRRectShape, xform);
	SetShapeTransform(qdOvalShape, xform);
	SetShapeTransform(qdArcShape, xform);
	SetShapeTransform(qdFrameRectShape, xform);
	SetShapeTransform(qdPaintRectShape, xform);
	SetShapeTransform(qdEraseRectShape, xform);
	SetShapeTransform(qdInvertRectShape, xform);
	SetShapeTransform(qdFillRectShape, xform);
	PopGraphicsNotice();
}
