/* graphics libraries:
	QuickDraw pen routines
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

#include <Memory.h>
#include "qd globals.h"
#include "qd emulation.h"
#include "qd utilities.h"
#include "graphics libraries.h"


pascal void HidePen(void)
{
	--thePort->visible;
}


pascal void ShowPen(void)
{
	thePort->visible++;
}


pascal void GetPenState(PenState *pnState)
{
	NilParamReturn(pnState);
	pnState->pnLoc.h = FixedToInt(thePort->penPosition.x);
	pnState->pnLoc.v = FixedToInt(thePort->penPosition.y);
	GetPen(&pnState->pnSize);
	pnState->pnMode = GetInkCommonTransfer(GetShapeInk(GetDefaultShape(lineType)));
	BlockMove((char *)&thePort->fgPat, (char *)&pnState->pnPat, sizeof(Pattern));
}


pascal void SetPenState(const PenState *pnState)
{
	NilParamReturn(pnState);
	MoveTo(pnState->pnLoc.h, pnState->pnLoc.v);
	PenSize(pnState->pnSize.h, pnState->pnSize.v);
	PenMode(pnState->pnMode);
	PenPat(pnState->pnPat);
}


pascal void GetPen(Point *pt)
{
	NilParamReturn(pt);
	pt->v = pt->h = FixedToInt(GetStylePen(GetShapeStyle(GetDefaultShape(lineType))));
}


pascal void PenSize(short width, short height)
{
	fixed size = (IntToFixed(height)+IntToFixed(width)) >> 1;
	/*if (height <= 1 && width <= 1) size = 0;*/

	IgnoreGraphicsNotice(pen_size_already_set);
	SetStylePen(GetShapeStyle(GetDefaultShape(lineType)), size);
	SetStylePen(qdPatternStyle, size);
	PopGraphicsNotice();
}


pascal void PenMode(short mode)
{
	SetInkCommonTransfer(GetShapeInk(GetDefaultShape(lineType)), ConvertFromQDMode(mode));
}


pascal void PenPat(const Pattern pat)
{
	BlockMove((Ptr)&pat, (Ptr)&thePort->fgPat, sizeof(Pattern));
	SetStyleQDPat(GetShapeStyle(GetDefaultShape(lineType)), GetShapeInk(GetDefaultShape(lineType)), pat);
	SetStyleQDPat(qdPatternStyle, qdPenInk, pat);
}


pascal void PenNormal(void)
{
	PenSize(1, 1);
	PenMode(srcCopy);
	BackPat(whitePat);
	PenPat(blackPat);
}


pascal void BackPat(const Pattern pat)
{
	BlockMove((Ptr)&pat, (Ptr)&thePort->bgPat, sizeof(Pattern));
	SetStyleQDPat(qdEraseStyle, qdEraseInk, pat);
}


pascal void GetBackPat(Pattern pat)
{
	BlockMove((Ptr)&thePort->bgPat,(Ptr)&pat,sizeof(Pattern));
}
