/* graphics libraries:
	QuickDraw text routines
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

#include <Memory.h>
#include "qd globals.h"
#include "qd emulation.h"
#include "qd library.h"
#include "graphics libraries.h"


/*
  * Add a comment to the open picture by adding a tag to an
  * empyType shape. The item 'type' is the commend 'kind'.
  */
pascal void StdComment(short kind, short dataSize, Handle dataHandle)
{
	NilParamReturn(dataHandle);

	IfErrorReturn(!openPicture && !picturePlayBack, no_active_picture);
	if (openPicture)
	{
		shape emptyShape;
		tag tempItem;

		emptyShape = NewShape(emptyType);
		tempItem = NewTag(kind, dataSize, *dataHandle);
		SetShapeTags(emptyShape, 0, 0, 0, 1, &tempItem);
		DisposeTag(tempItem);
		SetPictureParts(openPicture, 0, 0, 1, &emptyShape, nil, nil, nil);
		DisposeShape(emptyShape);
	}
}


pascal void PicComment(short kind, short dataSize, Handle dataHandle)
{
	(*stdCommentProc)(kind, dataSize, dataHandle);
}


pascal void StdGetPic(void *dataPtr, short byteCount)
{
	long commentSize;

	NilParamReturn(dataPtr);
	IfErrorReturn(picturePlayBack == nil, no_picture_drawn);
	IfErrorReturn(byteCount <= 0, count_is_less_than_one);
	GetShapeUser(picturePlayBack, nil,&commentSize,0,nil,1);	/* get 1st user obj of any type, length only	*/

	if (commentSize > 0)
	{	char *comment = (char *) NewPtr(commentSize);

		if (!comment) PostGraphicsError(out_of_memory);
		GetShapeUser(picturePlayBack, comment,nil,0,nil,1);	/* get 1st user obj, data only				*/
		if (byteCount > commentSize)
			PostGraphicsWarning(request_exceeds_available_data);
		else if (byteCount < commentSize) {
			PostGraphicsWarning(extra_data_unread);
			commentSize = byteCount;
		}
		BlockMove(comment, dataPtr, commentSize);
		DisposPtr(comment);
	}
}


pascal void StdPutPic(const void *dataPtr, short byteCount)
{
	IfErrorReturn(openPicture == nil, no_open_picture);
	StdComment(0, byteCount, (Handle) &dataPtr);
}


pascal PicHandle OpenPicture(const Rect *picFrame)
{
	tag tempItem;
	
	IfErrorReturnNil(picFrame == nil, parameter_is_nil);
	IfErrorReturnNil(openPicture, picture_already_open);
	HidePen();
	openPicture = NewShape(pictureType);
	SetShapeAttributes(openPicture, GetShapeAttributes(openPicture) | uniqueItemsShape);

	tempItem = NewTag('rect', sizeof(Rect), (char *)picFrame);
	SetShapeTags(openPicture, 0, 0, 0, 1, &tempItem);
	DisposeTag(tempItem);

	return openPicture;
}


pascal void ClosePicture(void)
{
	IfErrorReturn(openPicture == nil, no_open_picture);
	ShowPen();
	openPicture = nil;
}


pascal void DrawPicture(PicHandle myPicture, const Rect *dstRect)
{
	rectangle srcRectangle;
	mapping savemap;

	NilShapeReturn(myPicture);
	NilParamReturn(dstRect);

	if (thePort->visible<0)
		return;
	GetShapeMapping(myPicture, &savemap);

	IgnoreGraphicsWarning(no_variable_length_user_data_saved);
	if (GetShapeUser(myPicture, nil,nil,'rect',nil,1))
	{	Rect srcRect;

		GetShapeUser(myPicture, (char *) &srcRect,nil,'rect',nil,1);
		ShortRectToFixed(&srcRect, &srcRectangle);
	} else
		GetShapeBounds(myPicture, 0, &srcRectangle);
	PopGraphicsWarning();

	IgnoreGraphicsNotice(transform_mapping_unaffected);
	ScaleShape(myPicture,
		FixedDivide(IntToFixed(dstRect->right - dstRect->left), srcRectangle.right - srcRectangle.left),
		FixedDivide(IntToFixed(dstRect->bottom - dstRect->top), srcRectangle.bottom - srcRectangle.top), 0, 0);
	MoveShapeTo(myPicture, IntToFixed(dstRect->left), IntToFixed(dstRect->top));
	PopGraphicsNotice();
	DrawShape(myPicture);
	SetShapeMapping(myPicture, &savemap);
}


pascal void KillPicture(PicHandle myPicture)
{
	NilParamReturn(myPicture);
	DisposeShape(myPicture);
}

pascal void GetPictureBounds(PicHandle myPicture, Rect *bounds)
{
	NilShapeReturn(myPicture);
	NilParamReturn(bounds);
	if (GetShapeUser(myPicture, nil,nil,'rect',nil,1))
		GetShapeUser(myPicture, (char *) bounds,nil,'rect',nil,1);
	else {
		rectangle fixedBounds;

		GetShapeBounds(myPicture, 0, &fixedBounds);
		FixedRectToShort(&fixedBounds, bounds);
	}
}
