/* graphics libraries:
	QuickDraw polygon routines
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

#include "qd globals.h"
#include "qd emulation.h"
#include "qd utilities.h"
#include "qd library.h"
#include "graphics libraries.h"


pascal void StdPoly(GrafVerb verb, PolyHandle polygonShape)
{
	NilShapeReturn(polygonShape);

	if (verb == frame && openPoly)
		ExtendShape(openPoly, polygonShape);
	DrawQDShape(polygonShape, verb, true, true);
}


pascal void FramePoly(PolyHandle polygonShape)
{
	(*stdPolyProc)(frame, polygonShape);
}


pascal void FastFramePoly(PolyHandle polygonShape)
{
	NilShapeReturn(polygonShape);
	SetShapeStyle(polygonShape, GetShapeStyle(GetDefaultShape(rectangleType)));
	SetShapeFill(polygonShape, closedFrameFill);
	DrawShape(polygonShape);
	SetShapeFill(polygonShape, evenOddFill);
}



pascal void PaintPoly(PolyHandle polygonShape)
{
	(*stdPolyProc)(paint, polygonShape);
}


pascal void FastPaintPoly(PolyHandle polygonShape)
{
	NilShapeReturn(polygonShape);
	SetShapeStyle(polygonShape, GetShapeStyle(GetDefaultShape(rectangleType)));
	DrawShape(polygonShape);
}


pascal void ErasePoly(PolyHandle polygonShape)
{
	(*stdPolyProc)(erase, polygonShape);
}


pascal void FastErasePoly(PolyHandle polygonShape)
{
	NilShapeReturn(polygonShape);
	SetShapeStyle(polygonShape, qdEraseStyle);
	DrawShape(polygonShape);
}


pascal void InvertPoly(PolyHandle polygonShape)
{
	(*stdPolyProc)(invert, polygonShape);
}


pascal void FastInvertPoly(PolyHandle polygonShape)
{
	NilShapeReturn(polygonShape);
	SetShapeInk(polygonShape, qdInvertInk);
	DrawShape(polygonShape);
}


pascal void FillPoly(PolyHandle polygonShape, const Pattern pat)
{
	SaveQDFillStyleInk
	SetStyleQDPat(qdFillStyle, qdFillInk, pat);
	(*stdPolyProc)(fill, polygonShape);
	RestoreQDFillStyleInk
}


pascal PolyHandle OpenPoly(void)
{
	IfErrorReturnNil(openPoly, poly_already_open);
	HidePen();
	openPoly = NewShape(emptyType);
	SetShapeFill(openPoly, noFill);
	return openPoly;
}


pascal void ClosePoly(void)
{
	IfErrorReturn(openPoly == nil, no_open_poly);
	SetShapeFill(openPoly,evenOddFill);
	ShowPen();
	openPoly = nil;
}

pascal void KillPoly(PolyHandle poly)
{
	NilShapeReturn(poly);
	DisposeShape(poly);
}

pascal void OffsetPoly(PolyHandle poly, short dh, short dv)
{
	NilShapeReturn(poly);
	MoveShape(poly, IntToFixed(dh), IntToFixed(dv));
}

pascal void GetPolyBounds(PolyHandle poly, Rect *bounds)
{
	shapeType polyType;

	NilShapeReturn(poly);
	NilParamReturn(bounds);
	polyType = GetShapeType(poly);
	if (polyType == emptyType)
		SetRect(bounds, 0, 0, 0, 0);
	else if (polyType == fullType)
		SetRect(bounds, -32768, -32768, 32767, 32767);
	else {
		rectangle fixedBounds;

		GetShapeBounds(poly, 0, &fixedBounds);
		FixedRectToShort(&fixedBounds, bounds);
	}
}
