/* graphics libraries:
	QuickDraw text routines
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

#include <Fonts.h>
#include <Memory.h>
#include <QuickDraw.h>
#include <Resources.h>
#include <ToolUtils.h>

#include "qd globals.h"
#include "qd emulation.h"
#include "qd utilities.h"
#include "font routines.h"
#include "graphics libraries.h"
#include "font library.h"


#ifdef MacintoshIncludes
	#define NewString gNewString
#endif


pascal void StdText(short count, const  StringPtr textAddr, Point numer, Point denom)
{
	NilParamReturn(textAddr);
	if (count > 0)
	{	point advancePt;
		boolean scaled;
		transform oldXform;

		SetText(qdTextShape, count, textAddr, &thePort->penPosition);
		scaled = (*(long *) &numer != *(long *) &oneOne || *(long *) &denom != *(long *) &oneOne);
		if (scaled) {
			IgnoreGraphicsNotice(transform_mapping_unaffected);
			oldXform = SeparateShapeTransform(qdTextShape);
			ScaleShape(qdTextShape, FixRatio(numer.h, denom.h), FixRatio(numer.v, denom.v), 0, 0);
			PopGraphicsNotice();
		}
		if (thePort->visible >= 0) {
			IgnoreGraphicsWarning(font_substitution_took_place);
			DrawShape(qdTextShape);
			PopGraphicsWarning();
		}
		thePort->penPosition.x += GetShapeAdvance(qdTextShape, &advancePt)->x;

		if( openPicture )
			SetPictureParts(openPicture, 0, 0, 1, &qdTextShape, nil, nil, nil);

		if (scaled)
			ReuniteShapeTransform(qdTextShape, oldXform);
	} else
		PostGraphicsNotice(zero_length_string_passed);
}


pascal void DrawText(const StringPtr data, short first, short len)
{
	(*stdTextProc)(len, data + first, oneOne, oneOne);
}


pascal void DrawString(const StringPtr str)
{
	DrawText(&str[1], 0, str[0]);
 }


pascal void DrawChar(unsigned char ch)
{
	unsigned char c = ch;	/* since C passes chars as shorts */
	DrawText(&c, 0, 1);
}


pascal void FastDrawText(const StringPtr data, short first, short len)
{
	NilParamReturn(data);
	if (len > 0)
	{	point pt;

		SetText(qdTextShape, len, data + first, &thePort->penPosition);
		DrawShape(qdTextShape);
		thePort->penPosition.x += GetShapeAdvance(qdTextShape, &pt)->x;
	}
}


pascal void FastDrawCString(const StringPtr str)
{
	register short len = 0;
	register unsigned char *p = str;
	
	NilParamReturn(str);
	while (*p++)
		++len;
	FastDrawText(str, 0, len);
}


pascal void FastDrawString(const StringPtr str)
{
	FastDrawText(str, 1, str[0]);
 }


pascal void FastDrawChar(char ch)
{
	unsigned char c = ch;	/* since C passes chars as shorts */
	FastDrawText(&c, 0, 1);
}


pascal void MeasureText(short c, const StringPtr text, short *charLocs)
{
	register short count = c;

	NilParamReturn(text);
	NilParamReturn(charLocs);
	if (count > 0)
	{	register short *widths = charLocs;
		shape myText = NewText(count, text, nil);
		point *fixWidths;

		MoveShape(myText, fix1/2, 0);
		SetShapeType(myText, glyphType);
		fixWidths = (point *) NewPtr(c + 1 << 3);
		GetGlyphMetrics(myText, fixWidths, nil, nil);
		for (count = c; count >= 0; --count) {
			*widths++ = FixedRound(fixWidths->x);
			fixWidths++;
		}
		DisposPtr((Ptr) fixWidths);
	} else
		PostGraphicsNotice(zero_length_string_passed);
}


pascal void GetFontInfo(FontInfo *info)
{
	point ascent, descent;

	NilParamReturn(info);
	GetShapeFontMetrics(qdTextShape, &ascent, &descent, nil, nil);
	info->ascent = FixedRound(ascent.y);
	info->descent = FixedRound(descent.y);
	info->widMax = GetShapeTextSize(qdTextShape) << 1;		/* don't really know this */
	info->leading = 0;
}


pascal short StdTxMeas(short count, const StringPtr textAddr, Point *numer, Point *denom, FontInfo *info)
{
	NilParamReturnNil(textAddr);
	NilParamReturnNil(numer);
	NilParamReturnNil(denom);
	NilParamReturnNil(info);
	*numer = oneOne;
	*denom = oneOne;
	GetFontInfo(info);
	return FixedRound(FixTextWidth(textAddr, count));
}


pascal short TextWidth(const StringPtr text, short first, short count)
{
	if (stdTxMeasProc)
	{	Point numer, denom;
		FontInfo info;

		numer = oneOne;
		denom = oneOne;
		return (*stdTxMeasProc)(count, text + first, &numer, &denom, &info);
	} else
	 	return FixedRound(FixTextWidth(text + first, count));
}


pascal short StringWidth(const StringPtr str)
{
	NilParamReturnNil(str);
	return TextWidth(&str[1], 0, str[0]);
}


pascal short CharWidth(unsigned char ch)
{
	unsigned char c = ch;	/* since C passes chars as shorts */
	return TextWidth(&c, 0, 1);
}


pascal void TextSize(short size)
{
	SetShapeTextSize(qdTextShape, IntToFixed(size));
}


pascal void TextFont(short txFont)
{
	font fontID = 0;
	
	thePort->textFont = txFont;
	if (txFont == applFont)
		txFont = *(short *) 0x984; /* apFontID */
	{	Handle fond = GetResource('FOND', txFont);
	
		if (!ResError()) {
			ResType resType;
			Str255 name;
			
			GetResInfo(fond, &txFont, &resType, name);
			fontID = FindFontFamily(1, macintoshPlatform, romanScript, englishLanguage, name[0], &name[1]);
		}
	}
	if (fontID)
		SetShapeFont(qdTextShape, fontID);
}


pascal void TextFace(short faceSet)
{
	IgnoreGraphicsNotice(attributes_already_set);
	SetShapeCommonFace(qdTextShape, faceSet);
	PopGraphicsNotice();
}


pascal void TextMode(short mode)
{
	SetShapeCommonTransfer(qdTextShape, ConvertFromQDMode(mode));
}

pascal short GetTextSize(void)
{
	return GetShapeTextSize(qdTextShape);
}

pascal short GetTextFont(void)
{
	return thePort->textFont;
}

pascal short GetTextFace(void)
{
	return GetShapeCommonFace(qdTextShape);
}

pascal short GetTextMode(void)
{
	return GetShapeCommonTransfer(qdTextShape);
}

pascal void TextNamedFont(const Str255 txName)
{
	font fontID;
	
	if (fontID = FindFontFamily(0, macintoshPlatform, romanScript, englishLanguage, txName[0], &txName[1]))
		SetShapeFont(qdTextShape, fontID);
}

pascal void GetTextNamedFont(Str255 txName)
{
	txName[0] = FindFontName(GetShapeFont(qdTextShape), familyFontName, macintoshPlatform, romanScript,
		englishLanguage, &txName[1], nil);
}

pascal void TextPat(const Pattern pat)
{
	SetStyleQDPat(qdTextStyle, qdTextInk, pat);
}

pascal void GetTextPat(Pattern pat)
{
	BlockMove(&thePort->txPat, &pat, sizeof(Pattern));
}

pascal void SpaceExtra(fixed extra)
{
	thePort->spaceExtra = extra;
}


pascal void CharExtra(fixed extra)
{
	thePort->charExtra = extra;
}


pascal void GetTextState(textState *state)
{
	register style txStyle = GetShapeStyle(qdTextShape);

	NilParamReturn(state);
	state->txMode = GetInkCommonTransfer(GetShapeInk(qdTextShape));
	state->txFace = GetStyleCommonFace(txStyle);
	{	Str255 txName;
		ResType res;
		GetTextNamedFont(txName);
		GetResInfo(GetNamedResource('FOND', txName), &state->txFont, &res, txName);
	}
	state->txSize = FixedRound(GetStyleTextSize(txStyle));
	BlockMove(&thePort->txPat, &state->txPat, sizeof(Pattern));
	state->spaceExtra = thePort->spaceExtra;
	state->charExtra = thePort->charExtra;
}


pascal void SetTextState(const textState *state)
{
	NilParamReturn(state);
	SetShapeCommonTransfer(qdTextShape, state->txMode);
	SetShapeCommonFace(qdTextShape, state->txFace);
	TextFont(state->txFont);
	SetShapeTextSize(qdTextShape, IntToFixed(state->txSize));
	TextPat(state->txPat);
	SpaceExtra(state->spaceExtra);
	CharExtra(state->charExtra);
}
