/* graphics libraries:
	QuickDraw std proc shared utilities
	by Cary Clark, Georgiann Delaney, Michael Fairman, Dave Good, Robert Johnson, Keith McGreggor, Oliver Steele, David Van Brink, Chris Yerga
	Copyright 1987 - 1991 Apple Computer, Inc.  All rights reserved.	*/

#include "qd globals.h"
#include "qd emulation.h"
#include "qd utilities.h"
#include "graphics libraries.h"


void SetStyleQDPat(style obj, ink colors, const Pattern pat)
{
#ifdef useBitmapPatterns
	static bitmap patBitmap = {nil, 8, 8, 1, 1, indexedSpace, nil, nil};
	static patternRecord patRec = {nil, ff(8), 0, 0,ff(8)};

	patBitmap.bits = (char *) pat;
	patRec.pattern = NewBitmap(&patBitmap, nil);
	patRec.attributes = portMapPattern | portAlignPattern;
	SetStylePattern(obj, &patRec);
	PopGraphicsNotice();
	DisposeShape(patRec.pattern);
#else
	short count;
	unsigned long *patternPtr = (unsigned long *)pat;
	unsigned long value, mask;
	short sum = 0;
	color c;
	#pragma unused (obj)
	
	for (count = 2; count--; ) {
		value = *patternPtr++;
		mask = 0x01;
		do
			if (value & mask)
				sum++;
		while (mask <<= 1);
	}
	c.space = graySpace;
	c.profile = nil;
	c.element.gray = MultiplyDivide(65535, 2 * 32 - sum, 2 * 32);
	SetInkColor(colors, &c);
#endif
}

componentMode ConvertFromQDMode(short mode)
{
	static componentMode bwModes[] = {
		/* copy or xor bic */	copyMode/**/, copyMode/**/, copyMode/**/, copyMode/**/,
		/* not ^^^^ */			copyMode/**/, copyMode/**/, copyMode/**/, copyMode/**/
	};
	static componentMode cModes[] = {
		copyMode/**/, copyMode/**/, commonAddOverMode, commonSubtractPinMode,
		copyMode/**/, copyMode/**/, commonSubtractOverMode, copyMode/**/
	};

	if (mode>=srcCopy && mode<=notSrcBic) return bwModes[mode-srcCopy];
	if (mode>=patCopy && mode<=notPatBic) return bwModes[mode-patCopy];
	if (mode>=blend && mode<=adMin) return cModes[mode-blend];
	PostGraphicsError(parameter_out_of_range);
	return copyMode;
}


void DrawQDShape(shape newShape, GrafVerb verb, boolean permanantShape, boolean addToRgn)
{
	boolean toRgn = addToRgn && openRgn && (verb == frame);
	shape copyShape = newShape;
	boolean madeCopy = false;
	boolean visible = thePort->visible >= 0;
	static style *verbStyles[5] = {&qdPatternStyle, &qdPatternStyle, &qdEraseStyle, &qdPatternStyle, &qdFillStyle};
	static ink *verbInks[5] = {&qdPenInk, &qdPenInk, &qdEraseInk, &qdInvertInk, &qdFillInk};
	style verbStyle = *verbStyles[verb];
	ink verbInk = *verbInks[verb];

	if (!visible && !toRgn && !openPicture) return;
	if (GetShapeType(newShape) != bitmapType)
		SetShapeInk(newShape, verbInk);
	else
		verbInk = GetShapeInk(newShape);
	if (GetShapeType(newShape) != bitmapType)
		SetShapeStyle(newShape, verbStyle);
	else
		verbStyle = GetShapeStyle(newShape);

	if (verb == frame)
		SetShapeFill(newShape, closedFrameFill);

	if ((toRgn || openPicture) && permanantShape) {
		copyShape = CopyToShape(nil, newShape);
		madeCopy = true;
	}

	if (visible)
		DrawShape(newShape);

	if( openPicture )
		SetPictureParts(openPicture, 0, 0, 1, &newShape, nil, nil, nil);

	if (toRgn) {
		SetShapeFill(copyShape, evenOddFill);
		AddToShape(openRgn, copyShape);
		lastRgnOpClosed = true;
	}

	if (madeCopy)
		DisposeShape(copyShape);
	if (permanantShape && verb == frame)
		SetShapeFill(newShape, evenOddFill);
}
