/*	File: kaboom.r

	Resources for a simple printing extension.
	
	Dave Hersey
	Apple Developer Technical Support

	4/26/93	- dmh - Modified to work around b1 shutdown bug.
	2/01/93 - dmh - created.
*/

#include "SysTypes.r"
#include "Types.r"
#include "PrintingResTypes.r"


#define r_str		200


/*=================== Required Printing Extension Types and Resources =================== */


/*======= Scope ('scop') Resource ======= */

/*	This resource tells the printing system which types
	of drivers we work with.  (We work with all of them.) */

resource extensionScopeType (imagingSystemScopeID, sysheap, purgeable)
{
	{
		'univ'		
	};
};


/*======= Message Override ('over') Resource ======= */

/*	This resource tells the printing system which messages we
	want to override and where the override code is located. */

resource overrideType (extensionUniversalOverrideID, sysheap, purgeable)
{
	{
		// message to override	segmentID		offset into JumpTable
		startJob,					0, 					 4,
		finishJob,					0, 					 8
	};
};


/*======= Extension Optimization ('eopt') Resource ======= */

/*	This resource tells the printing system at which times we'll
	be doing things so that it can optimize for us.		*/

resource extensionOptimizationType (extensionOptimizationID, sysheap, purgeable)
{
	dontExecuteDuringImaging,
	dontNeedDeviceStatus,
	dontChangePageAtDespoolPage,
	dontChangePageAtImagePage,
	dontChangePageAtRenderPage
};


/*======= Extension Load ('load') Resource ======= */

/*	This resource tells the printing system where we want
	to be loaded in the handler chain. */

resource extensionLoadType (extensionLoadID, sysheap, purgeable)
{
	extensionLoadAnywhere
};


/*=================== Standard 'vers', bundle and icon resources follow =================== */

resource 'vers' (1, sysheap, purgeable) {
	0x1,
	0x0,
	final,
	0x0,
	verUS,
	"1.0",
	"1.0,  Apple Computer, Inc.  1983-1992"
};


resource 'vers' (2, sysheap, purgeable) {
	0x1,
	0x0,
	final,
	0x0,
	verUS,
	"1.0",
	"kaboom v1.0"
};


resource 'BNDL' (128, sysheap, purgeable) {
	'Kabm',
	0,
	{	/* array TypeArray: 2 elements */
		/* [1] */
		'FREF',
		{	/* array IDArray: 1 elements */
			/* [1] */
			0, 128
		},
		/* [2] */
		'ICN#',
		{	/* array IDArray: 1 elements */
			/* [1] */
			0, 128
		}
	}
};

resource 'STR#' (r_str, sysheap, purgeable)
{
	{
		"This is string #1",
		"This is string #2",
		"This is string #3",
		"This is string #4"
	}
};

Type 'Kabm' as 'STR ';

resource 'Kabm' (0, "Owner resource", sysheap, purgeable) {
	"This is just a simple printing extension."
};


data 'FREF' (128, sysheap, purgeable) {
	$"7065 7874 0000 7F"                                  /* pext... */
};


resource 'ICN#' (128, sysheap, purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"FFFF FFFC FFFF FFFC FFFF FFFC FFFF FFFC"
		$"FFFF FFFC FFFF FFFC FFFF FFFC FFFF FFFC"
		$"FFFF FFFC DFEF FFFD 9F0F FFFF 1E47 00FF"
		$"1E00 003F 1800 001F 1900 000F 1E00 0007"
		$"1E00 0007 1F00 0007 1FE0 0007 1FF0 000F"
		$"1FF8 001F 9FF8 001F DFF8 813D FFFD FFBC"
		$"FFFD FFBC FFFD FFBC FFFF FFFC FFFF FFFC"
		$"FFFF FFFC FFFF FFFC FFFF FFFC 7FFF FFFC",
		/* [2] */
		$"FFFF FFFC FFFF FFFC FFFF FFFC FFFF FFFC"
		$"FFFF FFFC FFFF FFFC FFFF FFFC FFFF FFFC"
		$"FFFF FFFC DFFF FFFD 9FFF FFFF 1FFF FFFF"
		$"1FFF FFFF 1FFF FFFF 1FFF FFFF 1FFF FFFF"
		$"1FFF FFFF 1FFF FFFF 1FFF FFFF 1FFF FFFF"
		$"1FFF FFFF 9FFF FFFF DFFF FFFD FFFF FFFC"
		$"FFFF FFFC FFFF FFFC FFFF FFFC FFFF FFFC"
		$"FFFF FFFC FFFF FFFC FFFF FFFC 7FFF FFFC"
	}
};


resource 'icl8' (128, sysheap, purgeable) {
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF 0000"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3"
	$"E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF 0000"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3"
	$"E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF 0000"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3"
	$"E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF 0000"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3"
	$"E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF 0000"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3"
	$"E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF 0000"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3"
	$"E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF 0000"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 FFE3 E3E3"
	$"E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF 0000"
	$"FFE3 FFE3 E3E3 E3E3 E3E3 E3FF FFE3 E3E3"
	$"E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF 0000"
	$"FFFF 00FF E3E3 E3E3 FFFF FF08 FFE3 E3E3"
	$"E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF 00FF"
	$"FF00 00FF E3E3 E3FF 0808 0808 FFFF E3E3"
	$"FFFF FFFF FFFF FFFF E3E3 E3E3 E3FF FFFF"
	$"0000 00FF E3E3 FF08 08FF 0808 08FF FFFF"
	$"0808 0808 0808 0808 FFFF E3E3 E3E3 E3FF"
	$"0000 00FF FFFF FF08 0808 0808 0808 0808"
	$"0808 0808 0808 0808 0808 FFFF E3E3 E3FF"
	$"0000 00FF FF08 0808 0808 0808 0808 0808"
	$"0808 0808 0808 0808 0808 08FF FFE3 E3FF"
	$"0000 00FF FF08 08FF 0808 0808 0808 0808"
	$"0808 0808 0808 0808 0808 0808 FFE3 E3FF"
	$"0000 00FF FFFF FF08 0808 0808 0808 0808"
	$"0808 0808 0808 0808 0808 0808 08FF FFFF"
	$"0000 00FF E3E3 FF08 0808 0808 0808 0808"
	$"0808 0808 0808 0808 0808 0808 08FF E3FF"
	$"0000 00FF E3E3 E3FF 0808 0808 0808 0808"
	$"0808 0808 0808 0808 0808 0808 08FF E3FF"
	$"0000 00FF E3E3 E3E3 FFFF FF08 0808 0808"
	$"0808 0808 0808 0808 0808 0808 08FF E3FF"
	$"0000 00FF E3E3 E3E3 E3E3 E3FF 0808 0808"
	$"0808 0808 0808 0808 0808 0808 FFE3 E3FF"
	$"0000 00FF E3E3 E3E3 E3E3 E3E3 FF08 0808"
	$"0808 0808 0808 0808 0808 08FF E3E3 E3FF"
	$"FF00 00FF E3E3 E3E3 E3E3 E3E3 FF08 0808"
	$"0808 0808 0808 0808 0808 08FF E3FF FFFF"
	$"FFFF 00FF E3E3 E3E3 E3E3 E3E3 FF08 0808"
	$"FF08 0808 0808 08FF 0808 FFE3 E3FF 00FF"
	$"FFE3 FFE3 E3E3 E3E3 E3E3 E3E3 E3FF 08FF"
	$"E3FF FFFF FFFF FFE3 FF08 FFE3 E3FF 0000"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF 08FF"
	$"E3E3 E3E3 E3E3 E3E3 FF08 FFE3 E3FF 0000"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF 08FF"
	$"E3E3 E3E3 E3E3 E3E3 FF08 FFE3 E3FF 0000"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF FFE3"
	$"E3E3 E3E3 E3E3 E3E3 E3FF FFE3 E3FF 0000"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF E3E3"
	$"E3E3 E3E3 E3E3 E3E3 E3E3 FFE3 E3FF 0000"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3"
	$"E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF 0000"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3"
	$"E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF 0000"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3"
	$"E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 E3FF 0000"
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
};


resource 'ics8' (128, sysheap, purgeable) {
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 FF00"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 FF00"
	$"FFE3 E3E3 E3E3 FFE3 E3E3 E3E3 E3E3 FF00"
	$"FFFF E3E3 FFFF FFE3 E3E3 E3E3 E3E3 FFFF"
	$"FFFF E3FF FF08 FFFF FFFF FFFF FFE3 FFFF"
	$"00FF FFFF 0808 0808 0808 0808 08FF FFFF"
	$"00FF FFFF 0808 0808 0808 0808 0808 FFFF"
	$"00FF E3FF 0808 0808 0808 0808 0808 FFFF"
	$"00FF E3E3 FFFF 0808 0808 0808 0808 FFFF"
	$"FFFF E3E3 E3E3 FF08 0808 0808 08FF FFFF"
	$"FFFF E3E3 E3E3 FFFF FFFF FFFF FFFF FFFF"
	$"FFE3 E3E3 E3E3 FFFF E3E3 E3E3 FFFF FF00"
	$"FFE3 E3E3 E3E3 FFFF E3E3 E3E3 FFFF FF00"
	$"FFE3 E3E3 E3E3 E3E3 E3E3 E3E3 E3E3 FF00"
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF"
};


resource 'ics#' (128, sysheap, purgeable) {
	{	/* array: 2 elements */
		/* [1] */
		$"FFFE FFFE FFFE FFFE FFFF FBFF 7007 7003"
		$"7003 7C03 FE07 FFFF FFFE FFFE FFFE FFFE",
		/* [2] */
		$"FFFE FFFE FFFE FFFE FFFF FFFF 7FFF 7FFF"
		$"7FFF 7FFF FFFF FFFF FFFE FFFE FFFE FFFE"
	}
};


resource 'icl4' (128, sysheap, purgeable) {
	$"FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00"
	$"F888 8888 8888 8888 8888 8888 8888 8F00"
	$"F888 8888 8888 8888 8888 8888 8888 8F00"
	$"F888 8888 8888 8888 8888 8888 8888 8F00"
	$"F888 8888 8888 8888 8888 8888 8888 8F00"
	$"F888 8888 8888 8888 8888 8888 8888 8F00"
	$"F888 8888 8888 8888 8888 8888 8888 8F00"
	$"F888 8888 8888 F888 8888 8888 8888 8F00"
	$"F8F8 8888 888F F888 8888 8888 8888 8F00"
	$"FF0F 8888 FFFC F888 8888 8888 8888 8F0F"
	$"F00F 888F CCCC FF88 FFFF FFFF 8888 8FFF"
	$"000F 88FC CFCC CFFF CCCC CCCC FF88 888F"
	$"000F FFFC CCCC CCCC CCCC CCCC CCFF 888F"
	$"000F FCCC CCCC CCCC CCCC CCCC CCCF F88F"
	$"000F FCCF CCCC CCCC CCCC CCCC CCCC F88F"
	$"000F FFFC CCCC CCCC CCCC CCCC CCCC CFFF"
	$"000F 88FC CCCC CCCC CCCC CCCC CCCC CF8F"
	$"000F 888F CCCC CCCC CCCC CCCC CCCC CF8F"
	$"000F 8888 FFFC CCCC CCCC CCCC CCCC CF8F"
	$"000F 8888 888F CCCC CCCC CCCC CCCC F88F"
	$"000F 8888 8888 FCCC CCCC CCCC CCCF 888F"
	$"F00F 8888 8888 FCCC CCCC CCCC CCCF 8FFF"
	$"FF0F 8888 8888 FCCC FCCC CCCF CCF8 8F0F"
	$"F8F8 8888 8888 8FCF 8FFF FFF8 FCF8 8F00"
	$"F888 8888 8888 8FCF 8888 8888 FCF8 8F00"
	$"F888 8888 8888 8FCF 8888 8888 FCF8 8F00"
	$"F888 8888 8888 8FF8 8888 8888 8FF8 8F00"
	$"F888 8888 8888 8F88 8888 8888 88F8 8F00"
	$"F888 8888 8888 8888 8888 8888 8888 8F00"
	$"F888 8888 8888 8888 8888 8888 8888 8F00"
	$"F888 8888 8888 8888 8888 8888 8888 8F00"
	$"0FFF FFFF FFFF FFFF FFFF FFFF FFFF FF"
};


resource 'ics4' (128, sysheap, purgeable) {
	$"FFFF FFFF FFFF FFF0 F888 8888 8888 88F0"
	$"F888 8888 8888 88F0 F888 88F8 8888 88F0"
	$"FF88 FFF8 8888 88FF FF8F FCFF FFFF F8FF"
	$"0FFF CCCC CCCC CFFF 0FFF CCCC CCCC CCFF"
	$"0F8F CCCC CCCC CCFF 0F88 FFCC CCCC CCFF"
	$"FF88 88FC CCCC CFFF FF88 88FF FFFF FFFF"
	$"F888 88FF 8888 FFF0 F888 88FF 8888 FFF0"
	$"F888 8888 8888 88F0 FFFF FFFF FFFF FFF0"
};

