///\\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//																		   \\
//	HandySpeechTools.c							    					   \\
//																		   \\
//	Some handy speech tools that make dealing with  					   \\
//	the Speech Manager even simpler !!!									   \\
//																		   \\
//																		   \\
///\\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\

void ShowString(char *text)
{
	char		blank[1] = ' ';
	
	TESetSelect ( 32767, 32767, gTextEditHandle );
	TEInsert ( ( Ptr ) text, (long) strlen(text), gTextEditHandle );
	TESetSelect ( 32767, 32767, gTextEditHandle );
	TEInsert (blank, 1, gTextEditHandle );
}

void ShowText(char *text, int length)
{
	char		blank[1] = ' ';
	
	TESetSelect ( 32767, 32767, gTextEditHandle );
	TEInsert ( ( Ptr ) text, (long) length, gTextEditHandle );
	TESetSelect ( 32767, 32767, gTextEditHandle );
	TEInsert (blank, 1, gTextEditHandle );
}

void ReverseString(char *s)
{
	int c, i, j;
	
	for (i=0, j=strlen(s)-1; i<j; i++, j--) {
		c = s[i];
		s[i] = s[j];
		s[j] = c;
	}
}

void LongToString(long n, char *s)
{
	int i = 0, sign;
	
	if ((sign = n) < 0)	/* record sign */
		n = -n;
		
	do {
		s[i++] = n % 10 + '0';
	} while ((n /= 10) > 0);
	
	if (sign < 0)
		s[i++] = '-';
	s[i] = '\0';
	
	ReverseString(s);
}

// speech error handling

void ErrorAlert(char *TheMessage)
{	
	if(gChannel) 
	{
		SpeakText(gChannel, TheMessage, strlen(TheMessage));
	}
	paramtext(TheMessage, "","","");
	Alert(AlertBoxID, NULL);
}

void ReportSpeechError(OSErr TheError)
{
	switch(TheError)
	{
		case 0:
			break;
		case -50:
			ErrorAlert(" Parameter error");
			break;
		case -108:
			ErrorAlert(" Not enough memory to speak");
			break;
		case -109:
			ErrorAlert(" Handle argument is NIL");
			break;
		case -192:
			ErrorAlert(" resource not found");
			break;
		case -240:
			ErrorAlert(" No TextToSpeechType component available");
			break;
		case -241:
			ErrorAlert(" Could not open speech synthesizer channel");
			break;
		case -242:
			ErrorAlert(" Synth not ready");
			break;
		case -243:
			ErrorAlert(" Output buffer is too small to hold result");
			break;
		case -244:
			ErrorAlert(" Voice not found");
			break;
		case -245:
			ErrorAlert(" Incompatible voice");
			break;
		case -246:
			ErrorAlert(" Format problem with pronounciation dictionary");
			break;
		case -247:
			ErrorAlert(" Raw phoneme text contains invalid characters");
			break;
		case -3000:
			ErrorAlert(" Component Manager error");
			break;
		case -32767:
			ErrorAlert(" weird error that sneaks in all the time");
			break;
		default:
			ErrorAlert(" Not A Speech Manager Error ");
			break;
	}
}

void FinishSpeaking()
{
	while(SpeechBusy())
	{ }
}

void speak(char *s)
{
	OSErr	TheError;
	
	if(gChannel)
	{
		ShowString(s);
		TheError = SpeakText(gChannel, s, strlen(s));
		if(TheError)
		{
			ReportSpeechError(TheError);
		}
		else
		{
			FinishSpeaking();
		}
	}
}

void SpeakAsync(char *s)
{
	OSErr	TheError;
	
	if(gChannel)
	{
		ShowString(s);
		TheError = SpeakText(gChannel, s, strlen(s));
		if(TheError)
		{
			ReportSpeechError(TheError);
		}
	}
}

void SpeakNoShowAsync(char *s)
{
	OSErr	TheError;
	
	if(gChannel)
	{
		TheError = SpeakText(gChannel, s, strlen(s));
		if(TheError)
		{
			ReportSpeechError(TheError);
		}
	}
}

void SpeakNumber(long n)
{
	char	string[256];

	if(gChannel)
	{
		LongToString( n, string);
		ShowString(string);
		SpeakText( gChannel, string, strlen(string) );
		FinishSpeaking();
	}
}

#define Fixed1			(1 <<16)	/* 1.0 in Fixed-Point format */

void SpeakFixed(Fixed TheFixedValue)
{
	long	number;
	long	holder;
	
	number = (long) TheFixedValue;
	if(number >= Fixed1)
	{
		holder = (number / Fixed1);
		SpeakNumber(holder);
		number = (number - (holder * Fixed1));
	}
	SpeakText(gChannel, "point" , 5);
	FinishSpeaking();
	ShowString(".");
	if(number >= (Fixed1 / 10))
	{
		holder = (number / (Fixed1 / 10));
		SpeakNumber(holder);
		number = (number - (holder * (Fixed1 / 10)));
	}
	else
	{
		SpeakNumber(0);
	}
	SpeakNumber(number / (Fixed1 / 100));
}

void OSTypeToString(OSType N, char *Buf)
{
	Buf[0] = (char) ((N & 0xFF000000) >> 24);
	Buf[1] = (char) ((N & 0x00FF0000) >> 16);
	Buf[2] = (char) ((N & 0x0000FF00) >> 8);
	Buf[3] = (char) ((N & 0x000000FF) >> 0);
	Buf[4] = '\0';
}

void SpeakOSType(OSType TheType)
{
	char	string[256];
	OSType	mode = modeLiteral;
	
	OSTypeToString(TheType, string);
	gOSError = SetSpeechInfo( gChannel, soCharacterMode, &mode);
	ReportSpeechError(gOSError);	
	speak(string);
	mode = modeNormal;
	gOSError = SetSpeechInfo( gChannel, soCharacterMode, &mode);
	ReportSpeechError(gOSError);	
}

void SpeakNumVersion(NumVersion TheVersionNum)
{
	speak(" major rev ");
	SpeakNumber((long) TheVersionNum.majorRev);
	speak(" minor rev ");
	SpeakNumber((long) TheVersionNum.minorRev);
	speak(" bug fix rev ");
	SpeakNumber((long) TheVersionNum.bugFixRev);
	switch(TheVersionNum.stage)
	{
		case 0x20:
		speak(" development ");
		break;
		
		case 0x40:
		speak(" alpha ");
		break;
		
		case 0x60:
		speak(" beta ");
		break;
		
		case 0x80:
		speak(" final ");
		break;
	}
	speak(" stage ");
	speak(" not released ");
	SpeakNumber((long) TheVersionNum.nonRelRev);
}
			
void PToCString(char *str)
{
	short i, len = str[0];
	
	for (i=0; i<len; i++) {
		str[i] = str[i+1];
	}
	str[i] = 0;
}

void SetVoice(short index)
{
	VoiceSpec	TheVoice;
	short		NumVoices;

	gOSError = CountVoices(&NumVoices);
	ReportSpeechError(gOSError);
	if(index <= NumVoices)
	{
		gOSError = GetIndVoice(index, &TheVoice);
		ReportSpeechError(gOSError);		
		gOSError = SetSpeechInfo( gChannel, soCurrentVoice, &TheVoice);
		ReportSpeechError(gOSError);
		CheckItem(GetMHandle(voicesMenu), gCurrentVoice + makeVoiceSpecItem, false);
		gCurrentVoice = index;
		CheckItem(GetMHandle(voicesMenu), gCurrentVoice + makeVoiceSpecItem, true);
	}
}


/* 
	Boldface a button. 
*/
void
StdHiliteButton(DialogPtr TheDialogPtr, short TheItemID)
{
	short		ItemType;
	Handle		ItemHndl;
	Rect		ItemBox;
	PenState	SavePen;
	GrafPtr		SavePort;
	
	GetPort(&SavePort);
	SetPort(TheDialogPtr);
	
	GetPenState(&SavePen);
	PenSize(3,3);

	GetDItem(TheDialogPtr, TheItemID, &ItemType, &ItemHndl, &ItemBox);
	InsetRect(&ItemBox,-4,-4);
	FrameRoundRect(&ItemBox,16,16);	
	
	SetPenState(&SavePen);
	
	SetPort(SavePort);
}


pascal Boolean
StdGetNumLongFilterProc(DialogPtr /*TheDialog*/, EventRecord *AnEventPtr, short *ItemHitPtr)
{
	Boolean	ReturnVal = false;
	long	TheEventMessageCharCode = AnEventPtr->message & charCodeMask;
	
	if ((AnEventPtr->what == keyDown) && 
		((TheEventMessageCharCode == charEnterKey) || (TheEventMessageCharCode == charReturnKey))) 
	{
		*ItemHitPtr = stdGetNumOKID;
		ReturnVal = true;
	}
	
	return ReturnVal;
}

/*
	Get the text from an editText item.
*/
char *
StdGetEditTextStr(char *TheString, DialogPtr TheDialog, short TheItemNum)
{
	char 	*RetVal = NULL;
	short	ItemType;
	Handle	ItemHndl;
	Rect	ItemBox;
	

	if (TheItemNum > 0)
	{
		GetDItem(TheDialog, TheItemNum, &ItemType, &ItemHndl, &ItemBox);	/* get the item */
		
		if (ItemHndl != NULL)
		{
			// lock it down
			HLock(ItemHndl);
	
			getitext(ItemHndl, TheString);		/* get the String */
			HUnlock(ItemHndl);					/* unlock item */
			RetVal = TheString;					/* for futur error checking */
		}
	}
	
	return(RetVal);						/* return the pointer to the String */									
}

long
StrToLong(char s[])
{
	int i;
	long n = 0;
	
	for (i = 0; s[i] >= '0' && s[i] <= '9'; i++) {
		n = 10 * n + s[i] - '0';
	}
	return(n);
}

long
StdGetEditTextLong(DialogPtr TheDialog, short TheItemNum)
{
	char StrBuf[256]; 
	long RetVal;
	
	RetVal = StrToLong( StdGetEditTextStr(StrBuf, TheDialog, TheItemNum) );
	return( RetVal );
}

Boolean
IsNumStr(char *TheStringBuf)
{
	Boolean RetVal = true;
	
	while( *TheStringBuf != '\0' ) {
		if (  !((*TheStringBuf >= '0') && (*TheStringBuf <= '9')) ) { 
			RetVal = false;
			break;
		}
		else {
			TheStringBuf++;
		}
	}
	
	return(RetVal);
}

void 
StdSetText(char *text, DialogPtr dialog, short itemNo)
{
	short	itemType;
	Handle	itemHndl;
	Rect	itemBox;    
	
	GetDItem(dialog, itemNo, &itemType, &itemHndl, &itemBox);
	setitext (itemHndl,text);
}


long 
StdGetNumLong(char *ThePrompt, char *DefaultNumStr)
{
	short 		ItemHit, Done = false;
	DialogPtr 	TheDialog;
	char		TheStringBuf[256];
	long		TheReturnValue;
	GrafPtr		SavePort;
	
	TheDialog = GetNewDialog(stdGetNumDialogID, NULL, (WindowPtr) -1);
	GetPort(&SavePort);
	SetPort(TheDialog); 
	
	StdSetText(ThePrompt, TheDialog, stdGetNumPromptID);
	StdSetText(DefaultNumStr, TheDialog, stdGetNumUserInputID);
	
	/* Highlight the ENTIRE default input text */
	SelIText(TheDialog, stdGetNumUserInputID, startOfSelText, maxEndOfSelText);
	/* Highlight the OK button */
	StdHiliteButton(TheDialog, stdGetNumOKID);
		
	while (!Done)
	{
		StdHiliteButton(TheDialog, stdGetNumOKID);
		
		ModalDialog((ModalFilterProcPtr)StdGetNumLongFilterProc, &ItemHit);

		if (ItemHit == stdGetNumOKID) 
		{
			Done = true;
			StdGetEditTextStr(TheStringBuf, TheDialog, stdGetNumUserInputID);
			if ( !IsNumStr(TheStringBuf) ) 
			{
				StdSetText("Please Enter A NUMBER Damn-It!", 
							TheDialog, stdGetNumPromptID);
				Done = false;
			}
		}
	}
	
	SetPort(SavePort);
			
	TheReturnValue = StdGetEditTextLong(TheDialog, stdGetNumUserInputID);
	
	DisposDialog(TheDialog);
	
	return(TheReturnValue);
}

