///\\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//																		   \\
//	SpeechManagerTests.c												   \\
//																		   \\
//	Put up a set of menus which execute tests							   \\
//	which use each Speech Manager feature								   \\
//																		   \\
//																		   \\
///\\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\

#pragma segment SMT

#pragma load  ":AllToolBoxIncludes:AllToolBoxIncludes.h.o"

#define SystemSevenOrLater  TRUE

#include <stdio.h>
#include "string.h"
#include "math.h"
#include "FixMath.h"
#include "errno.h"
#include "ctype.h"
#include "stdlib.h"
#include "GestaltEqu.h"
#include "Speech.h"
#include "SpeechTests.h"

//////////////// globals ///////////////////

Boolean			gQuitting;
SpeechChannel 	gChannel;
OSErr			gOSError;
DialogPtr		gDialog;
TEHandle		gTextEditHandle;
int				gCurrentVoice;
PicHandle		gOpenMouthPict, gClosedMouthPict;

Boolean			gTextDoneCallbackInstalled;
Boolean			gSpeechDoneCallbackInstalled;
Boolean			gSyncCallbackInstalled;
Boolean			gErrorCallbackInstalled;
Boolean			gPhonemeCallbackInstalled;
Boolean			gWordCallbackInstalled;

long			gTextDoneCallbackCount;
long			gSpeechDoneCallbackCount;
long			gSyncCallbackCount;
long			gErrorCallbackCount;
long			gPhonemeCallbackCount;
long			gWordCallbackCount;

#define AlertBoxID 130

// Standard Get User Number Dialog Box____________
#define stdGetNumDialogID			130	
#define stdGetNumOKID				1
#define stdGetNumPromptID			2
#define stdGetNumUserInputID		3	
#define startOfSelText		0
#define maxEndOfSelText		32767
#define charEnterKey		0x03
#define charReturnKey		0x0D

void AdjustMenus()
{
	Fixed  TheModulation;

	gOSError = GetSpeechInfo( gChannel, soPitchMod, &TheModulation );
	if(TheModulation != 0)
	{
		CheckItem(GetMHandle(channelsMenu), pitchModulationItem, true);
	}
	else
	{
		CheckItem(GetMHandle(channelsMenu), pitchModulationItem, false);
	}

	CheckItem(GetMHandle(channelsMenu), acceptPhonemesItem, false);
	CheckItem(GetMHandle(channelsMenu), acceptTextItem, true);
	CheckItem(GetMHandle(channelsMenu), literalCharactersItem, false);
	CheckItem(GetMHandle(channelsMenu), literalNumbersItem, false);
	gTextDoneCallbackInstalled = false;
	gSpeechDoneCallbackInstalled = false;
	gSyncCallbackInstalled = false;
	gErrorCallbackInstalled = false;
	gPhonemeCallbackInstalled = false;
	gWordCallbackInstalled = false;
	CheckItem(GetMHandle(callBacksMenu), setA5Item, false);
	CheckItem(GetMHandle(callBacksMenu), refConItem, false);
	CheckItem(GetMHandle(callBacksMenu), textDoneItem, false);
	CheckItem(GetMHandle(callBacksMenu), speechDoneItem, false);
	CheckItem(GetMHandle(callBacksMenu), syncItem, false);
	CheckItem(GetMHandle(callBacksMenu), errorItem, false);
	CheckItem(GetMHandle(callBacksMenu), phonemeItem, false);
	CheckItem(GetMHandle(callBacksMenu), wordItem, false);
}


#include "HandySpeechTools.c"
#include "TheActualTests.c"


void SMTInitToolBox(void)
{
	MaxApplZone();
	FlushEvents(everyEvent, 0);		/* empty any stray events.			*/
	InitGraf(&qd.thePort);			/* HALInitialize quickdraw.			*/
	InitFonts();					/* HALInitialize Font Manager.			*/
	InitWindows();					/* HALInitialize Window Manager.		*/
	InitMenus();					/* HALInitialize Menu Manager.			*/
	TEInit();						/* HALInitialize Text Edit.			*/
	InitDialogs(NULL);				/* HALInitialize the Dialog Manager.	*/
	InitCursor();					/* Turn cursor back to arrow.		*/
	SetEventMask(everyEvent); // so's we can get keyUp events too!
}

void AddVoicesToVoiceMenu()
{
	short				i, limit;
	VoiceSpec			TheVoice;
	VoiceDescription	info;

	gOSError = CountVoices(&limit);
	ReportSpeechError(gOSError);

	for(i = 1; i <= limit; i++)
	{
		gOSError = GetIndVoice(i, &TheVoice);
		ReportSpeechError(gOSError);
		gOSError = GetVoiceDescription( &TheVoice, &info, sizeof(VoiceDescription));
		ReportSpeechError(gOSError);
		AppendMenu(GetMHandle(voicesMenu), info.name);
	}
	gCurrentVoice = limit;
	CheckItem(GetMHandle(voicesMenu), gCurrentVoice + makeVoiceSpecItem, true);
}

void SMTSetUpMenus()		
{	
	Handle 		MenuBar;
		
	MenuBar = GetNewMBar(menuBar);		
	SetMenuBar(MenuBar);				
	DisposHandle(MenuBar);
	AddVoicesToVoiceMenu();
	DrawMenuBar();
}

void OpenTextEditDialog()
{
	GrafPtr				SavePort;
	
	gDialog = GetNewDialog(kTextEditDialogID, NULL, (WindowPtr) -1);
	GetPort(&SavePort);
	SetPort(gDialog);
	TextFont(monaco);
	TextSize(9);
	TextFace ( 0 );
	gTextEditHandle = TENew(&gDialog->portRect, &gDialog->portRect);
	SetPort(SavePort);
}

void ClearTextEditDialog()
{
	GrafPtr	SavePort;

	GetPort(&SavePort);
	SetPort(gDialog);
	TESetSelect(startOfSelText, maxEndOfSelText, gTextEditHandle);
	TEDelete(gTextEditHandle);
	InvalRect(&gDialog->portRect);
	TEUpdate( &gDialog->portRect, gTextEditHandle );	
	SetPort(SavePort);
}

void SMTStartUp()
{	
	SMTInitToolBox();
	SMTSetUpMenus();
	OpenTextEditDialog();
	gOpenMouthPict = (PicHandle) GetResource('PICT', 62);
	gClosedMouthPict = (PicHandle) GetResource('PICT', 63);

	gOSError = NewSpeechChannel(0, &gChannel);
	if(gOSError)
	{
		ReportSpeechError(gOSError);
	}
	else
	{
		SpeakAsync("\n Speech Manager Tester. Opened a speech channel.");
	}
	AdjustMenus();
}

void AboutSpeechTests()
{
	GrafPtr	SavePort;
	Rect	PictRect, PortRect;
	short	toggle = 0;
	Fixed 	RandomNumber;
	long	OtherRandom;

	GetPort(&SavePort);
	SetPort(gDialog);
	SetRect(&PortRect,0, 0, 510, 302);
	InvertRect(&PortRect);
	SetRect(&PictRect,180, 76, 330, 226);
	OtherRandom = Random();
	SetVoice((OtherRandom % 12) + 1);
	OtherRandom =  25 + (Random() & 255);
	RandomNumber = Long2Fix(OtherRandom);
	gOSError = SetSpeechRate(gChannel, RandomNumber);
	ReportSpeechError(gOSError);
	OtherRandom = 127 & Random(); 
	RandomNumber = Long2Fix(OtherRandom);
	gOSError = SetSpeechPitch(gChannel, RandomNumber);
	ReportSpeechError(gOSError);
	SpeakNoShowAsync("\n Speech Tests, by Braz. Sample code and test program for the Speech Manager and the Macintalk 2 Speech Engine");
	while(SpeechBusy())
	{
		if(toggle)
		{
			DrawPicture(gOpenMouthPict, &PictRect);
		}
		else
		{
			DrawPicture(gClosedMouthPict, &PictRect);		
		}
		toggle = !toggle;
	}
	EraseRect(&PortRect);
	InvalRect(&PortRect);
	SetPort(SavePort);
}

void DoMenuCommand(long menuResult)
{
	short		menuID;				/* the resource ID of the selected menu */
	short		menuItem;			/* the item number of the selected menu */

	menuID = HiWord(menuResult);	
	menuItem = LoWord(menuResult);	
	switch ( menuID ) 
	{
		case appleMenu:
			switch ( menuItem ) 
			{
				case aboutMenuItem:	
					AboutSpeechTests();
					break;
				default:			
					break;
			}
			break;
		case speakMenu:
			switch ( menuItem ) 
			{
				case speakStringItem:
				SpeakStringTest();
				break;
				case speakTextItem:
				SpeakTextTest();
				break;
				case speakBufferItem:
				SpeakBufferTest();
				break;
				case stopSpeechItem:
				StopSpeechTest();
				break;
				case stopSpeechAtItem:
				StopSpeechAtTest();
				break;
				case pauseContinueItem:
				PauseContinueTest();
				break;
				case speechBusyItem:
				SpeechBusyTest();
				break;
				case quitMenuItem:
				gQuitting = true;
				break;
			}
			break;
		case voicesMenu:
			switch ( menuItem ) 
			{
				case countVoicesItem:
				CountVoicesTest();
				break;
				case getIndVoiceItem:
				GetIndVoiceTest();
				break;
				case getVoiceDescriptionItem:
				GetVoiceDescriptionTest();
				break;
				case getVoiceInfoItem:
				GetVoiceInfoTest();
				break;
				case makeVoiceSpecItem:
				MakeVoiceSpecTest();
				break;
				default:
				SetVoice(menuItem - makeVoiceSpecItem);
				GetVoiceDescriptionFromIndex(menuItem - makeVoiceSpecItem);			
				break;
			}
			break;
		case channelsMenu:
			switch ( menuItem ) 
			{
				case newSpeechChannelItem:
				NewSpeechChannelTest();
				break;
				case disposeSpeechChannelItem:
				DisposeSpeechChannelTest();
				break;
				case setSpeechRateItem:
				UserSetRate();
				break;
				case setSpeechPitchItem:
				UserSetPitch();
				break;
				case resetChannelItem:
				SpeechReset();
				break;
				case acceptTextItem:
				TextInputMode();
				break;
				case acceptPhonemesItem:
				PhonemecInputMode();
				break;
				case literalCharactersItem:
				LiteralCharacters();
				break;
				case literalNumbersItem:
				LiteralNumbers();
				break;
				case pitchModulationItem:
				PitchModulationToggle();
				break;
				case rateTestItem:
				SetSpeechRateTest();
				break;
				case pitchTestItem:
				SetSpeechPitchTest();
				break;
				case volumeItem:
				VolumeTest();
				break;
				case newDelimitersItem:
				NewDelimiters();
				break;
			}
			break;
		case infoMenu:
			switch ( menuItem ) 
			{
				case gestaltItem:
				GestaltTest();
				break;
				case versionItem:
				VersionTest();
				break;
				case getSpeechRateItem:
				GetSpeechRateTest();
				break;
				case getSpeechPitchItem:
				GetSpeechPitchTest();
				break;
				case getSpeechStatusItem:
				GetSpeechStatusTest();
				break;
				case getSpeechErrorItem:
				GetSpeechErrorTest();
				break;
				case getSpeechInputModeItem:
				GetInputModeTest();
				break;
				case getCharacterModeItem:
				GetCharacterModeTest();
				break;
				case getNumberModeItem:
				GetNumberModeTest();
				break;
				case getPitchModModeItem:
				GetSpeechPitchModTest();
				break;
				case getVolumeItem:
				GetSpeechVolumeTest();
				break;
				case getVersionInfoItem:
				GetSpeechVersionInfoTest();
				break;
				case getRecentSyncItem:
				GetSpeechSyncTest();
				break;
			}
			break;
		case callBacksMenu:
			switch ( menuItem ) 
			{
				case callBackCountItem:
				CallBackCount();
				break;
				case callBacksOffItem:
				CallBacksOff();
				break;
				case setA5Item:
				SetA5Test();
				break;
				case refConItem:
				SetRefConTest();
				break;
				case textDoneItem:
				NewTextDoneCallback();
				break;
				case speechDoneItem:
				NewSpeechDoneCallback();
				break;
				case syncItem:
				NewSyncCallback();
				break;
				case errorItem:
				NewErrorCallback();
				break;
				case phonemeItem:
				NewPhonemeCallback();
				break;
				case wordItem:
				NewWordCallback();
				break;
			}
			break;
		case otherMenu:
			switch ( menuItem ) 
			{
				case speakFileItem:
				SpeakFile();
				break;
				case textToPhonemesItem:
				TextToPhonemesTest();
				break;
				case phonemeSymbolsItem:
				GetPhonemeSymbolsTest();
				break;
				case useDictionaryItem:
				UseDictionaryTest();
				break;
				case clearTextEditItem:
				ClearTextEditDialog();
				break;
			}
			break;
	}
	HiliteMenu(0);					/* unhighlight what MenuSelect (or MenuKey) hilited */
} 

void SMTEventLoop(void)
{
	OSErr		TheErr = noErr;
	EventRecord	AnEvent;
	WindowPtr	WhichWindow;
	Rect		Boundry;
	Point		APoint;
	GrafPtr		SavePort;

	while(!gQuitting)
	{
		TEIdle(gTextEditHandle);
		if ( WaitNextEvent(everyEvent, &AnEvent, 2, NULL) ) 
		{
			switch (AnEvent.what) 
			{
				case mouseDown:
					switch( FindWindow(AnEvent.where, &WhichWindow) ) 
					{
						case inMenuBar:
							APoint = AnEvent.where;
							
							DoMenuCommand( MenuSelect(APoint) );
							break;
	
						case inSysWindow:
							SystemClick(&AnEvent, WhichWindow);
							break;
	
						case inDrag:
							SetRect(&Boundry, 4, 24, 
									qd.screenBits.bounds.right - 4, 
									qd.screenBits.bounds.bottom - 4
							);
							DragWindow(WhichWindow, AnEvent.where, &Boundry);
							break;
	
						default:
							break;
					}
					break;
				case keyDown:
					WhichWindow = FrontWindow();
					
					if ((AnEvent.modifiers & cmdKey) &&		// Only check for menukeys on key downs
						(MenuKey(AnEvent.message & charCodeMask) & 0xFFFF0000))
					{
						DoMenuCommand(MenuKey(AnEvent.message & charCodeMask));
						break;
					}
					// Otherwise fall through...
				case autoKey:
					TEKey(AnEvent.message & charCodeMask, gTextEditHandle);
					break;
				case keyUp:
					break;				
				case activateEvt:       
					  if(AnEvent.modifiers & activeFlag)
					  {
						GetPort(&SavePort);
						SetPort(gDialog);
					  	TEActivate(gTextEditHandle);
					  	TEUpdate( &gDialog->portRect, gTextEditHandle );	
						SetPort(SavePort);
					  }
					break;
				case updateEvt:			
					BeginUpdate((WindowPtr) AnEvent.message);					
					if ( !EmptyRgn(((WindowPtr) AnEvent.message)->visRgn) )		
					{		
						GetPort(&SavePort);
						SetPort(gDialog);
					  	TEActivate(gTextEditHandle);
					  	TEUpdate( &gDialog->portRect, gTextEditHandle );	
						SetPort(SavePort);
					}
					EndUpdate((WindowPtr) AnEvent.message);				
					break;
				case diskEvt:
					if ( HiWord(AnEvent.message) != noErr) 
					{
						SetPt(&APoint, 50, 50);
						TheErr = DIBadMount(APoint, AnEvent.message);
					}
					break;
				default:
					break;
			}				
		}				
	}
}

void SMTShutDown()
{	
	DisposeDialog(gDialog);
}

void main() 
{
	SMTStartUp();
	SMTEventLoop();
	SMTShutDown();
}
