///\\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\
//																		   \\
//	TheActualTests.c							    					   \\
//																		   \\
//	The ACTUAL subroutines that will be called 							   \\
//	from each menu to make Speech Manager calls							   \\
//																		   \\
//																		   \\
///\\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\/\

void GestaltTest()
{
	long response;
	
	gOSError = Gestalt(gestaltSpeechAttr, &response);
	speak("gestalt result is");
	SpeakNumber(response);
	ReportSpeechError(gOSError);
}

void SpeakStringTest()
{
	ShowString("Speak string seems to be working");
	gOSError = SpeakString("\pSpeak string seems too be working.");
	ReportSpeechError(gOSError);
}

void SpeechBusyTest()
{
	if(SpeechBusy())
	{
		FinishSpeaking();
		speak("Speech busy was true, then"); 
	}
	speak("Speech busy was false, then"); 
	speak("speech busy was true."); 
}
void MakeVoiceSpecTest()
{
	VoiceSpec	TheVoice;
	OSType 		creator = 'ABCD'; 
	OSType 		id = 'EFGH';
	
	gOSError = MakeVoiceSpec ( creator, id, &TheVoice );
	ReportSpeechError(gOSError);
	ShowText( (Ptr) &TheVoice, 8);
	SpeakText( gChannel, (Ptr) &TheVoice, 8);
}

void CountVoicesTest()
{
	short	NumVoices;
	
	gOSError = CountVoices(&NumVoices);
	ReportSpeechError(gOSError);
	speak("There are");
	SpeakNumber((long) NumVoices); 
	speak("voices.");
}

void NewSpeechChannelTest()
{
	if(gChannel)
	{
		DisposeSpeechChannel(gChannel);
	}
	gOSError = NewSpeechChannel(0, &gChannel);
	if(gOSError)
	{
		ReportSpeechError(gOSError);
	}
	else
	{
		speak("Opened a speech channel.");
		AdjustMenus();
	}
}

void GetIndVoiceTest()
{
	VoiceSpec	TheVoice;

	gOSError = GetIndVoice(1, &TheVoice);
	speak("Get Ind Voice returned");
	ShowText( (Ptr) &TheVoice, 8);
	gOSError = SpeakText(gChannel, (Ptr) &TheVoice, 8 );
	ReportSpeechError(gOSError);
}

void SpeakTextTest()
{
	ShowString("Speaking text.");
	gOSError = SpeakText(gChannel, "Speaking text.", 14);
	ReportSpeechError(gOSError);
}

void GetVoiceDescriptionFromIndex(short index)
{
	VoiceSpec			TheVoice;
	VoiceDescription	info;
	
	gOSError = GetIndVoice(index, &TheVoice);
	ReportSpeechError(gOSError);
	gOSError = GetVoiceDescription( &TheVoice, &info, sizeof(VoiceDescription));
	ReportSpeechError(gOSError);
	PToCString(&info.name);
	speak(&info.name);
}

void GetVoiceDescriptionTest()
{
	GetVoiceDescriptionFromIndex(1);
}

void GetVoiceInfoTest()
{
	VoiceDescription	info;
	void 				*voiceInfo = &info;
	OSType 				selector= 'info';
	VoiceSpec 			TheVoice;
	VoiceSpec 			*voice = &TheVoice;
	
	info.length = sizeof(VoiceDescription);
	gOSError = GetIndVoice(1, &TheVoice);
	ReportSpeechError(gOSError);	
	gOSError = GetVoiceInfo(voice, selector, voiceInfo);
	ReportSpeechError(gOSError);	
	PToCString(&info.name);
	speak(&info.name);
}

void StopSpeechTest()
{
	Boolean		UserStopped = false;

	SpeakAsync("Press the mouse button to test stop speech blah blah blah." );
	while(SpeechBusy())
	{
		if(Button())
		{
			gOSError = StopSpeech(gChannel);
			ReportSpeechError(gOSError);
			UserStopped = true;
		}
	}
	if(!UserStopped)
	{
		speak("I guess I'll do it" );
		SpeakAsync("myself." );
		gOSError = StopSpeech(gChannel);
		ReportSpeechError(gOSError);	
	}
}

void SetSpeechRateTest()
{
	Fixed		TheRate;
	long		i;
	
	for( i = 0; i <= 10 ; i++)
	{
		TheRate = Long2Fix(i*50);
		gOSError = SetSpeechRate( gChannel, TheRate);
		ReportSpeechError(gOSError);	
		SpeakNumber( Fix2Long(TheRate));
	}
}

void GetSpeechRateTest()
{
	Fixed		TheRate;
	Fixed		*TheRatePtr = &TheRate;
	
	gOSError = GetSpeechRate( gChannel, TheRatePtr);
	ReportSpeechError(gOSError);	
	SpeakNumber( Fix2Long(TheRate));
}

void SetSpeechPitchTest()
{
	Fixed		ThePitch;
	Fixed		*ThePitchPtr = &ThePitch;
	long		i;
	
	for( i = 0; i != 10 ; i++)
	{
		ThePitch = Long2Fix(i*10);
		gOSError = SetSpeechPitch( gChannel, ThePitch);
		ReportSpeechError(gOSError);	
		gOSError = GetSpeechPitch( gChannel, ThePitchPtr);
		ReportSpeechError(gOSError);	
		SpeakNumber( Fix2Long(ThePitch));
	}
}

void GetSpeechPitchTest()
{
	Fixed		ThePitch;
	
	gOSError = GetSpeechPitch( gChannel, &ThePitch);
	ReportSpeechError(gOSError);	
	SpeakNumber( Fix2Long(ThePitch));
}

void StopSpeechAtTest()
{
	Boolean		UserStopped = false;

	SpeakAsync(" Press the mouse button to test stop speech at immediate." );
	UserStopped = false;
	while(SpeechBusy())
	{
		if(Button())
		{
			gOSError = StopSpeechAt(gChannel, kImmediate);
			ReportSpeechError(gOSError);
			UserStopped = true;
		}
	}
	if(!UserStopped)
	{
		speak(" I guess I'll do it " );
		SpeakAsync(" myself." );
		gOSError = StopSpeechAt(gChannel, kImmediate);
		ReportSpeechError(gOSError);	
	}
	SpeakAsync("Press the mouse button to test stop speech at end of word looongonefortomakeagoodtarget." );
	UserStopped = false;
	while(SpeechBusy())
	{
		if(Button())
		{
			gOSError = StopSpeechAt(gChannel, kEndOfWord);
			ReportSpeechError(gOSError);
			UserStopped = true;
		}
	}
	if(!UserStopped)
	{
		speak("I guess I'll do it" );
		SpeakAsync("myself." );
		gOSError = StopSpeechAt(gChannel, kEndOfWord);
		ReportSpeechError(gOSError);	
	}
	SpeakAsync("Press the mouse button to test stop speech at End Of Sentence." );
	UserStopped = false;
	while(SpeechBusy())
	{
		if(Button())
		{
			gOSError = StopSpeechAt(gChannel, kEndOfSentence);
			ReportSpeechError(gOSError);
			UserStopped = true;
		}
	}
	if(!UserStopped)
	{
		speak("I guess I'll do it " );
		SpeakAsync("myself." );
		gOSError = StopSpeechAt(gChannel, kEndOfSentence);
		ReportSpeechError(gOSError);	
	}
}

void PauseContinueTest()
{
	SpeakAsync("Hold the mouse button down to test pause speech at immediate." );
	while(SpeechBusy())
	{
		if(Button())
		{
			gOSError = PauseSpeechAt(gChannel, kImmediate);
			ReportSpeechError(gOSError);
			while(Button())
			{ }
			gOSError = ContinueSpeech(gChannel);
			ReportSpeechError(gOSError);	
		}
	}
	SpeakAsync("Hold the mouse button down to test pause speech at end of word looongonefortomakeagoodtarget.");
	while(SpeechBusy())
	{
		if(Button())
		{
			gOSError = PauseSpeechAt(gChannel, kEndOfWord);
			ReportSpeechError(gOSError);
			while(Button())
			{ }
			gOSError = ContinueSpeech(gChannel);
			ReportSpeechError(gOSError);	
		}
	}
	SpeakAsync("Hold the mouse button down to test pause speech at End Of Sentence. Next sentence.");
	while(SpeechBusy())
	{
		if(Button())
		{
			gOSError = PauseSpeechAt(gChannel, kEndOfSentence);
			ReportSpeechError(gOSError);
			while(Button())
			{ }
			gOSError = ContinueSpeech(gChannel);
			ReportSpeechError(gOSError);	
		}
	}
}
	
void SpeakBufferTest()
{
	char	string[256];
	
	sprintf(string, "Testing speak buffer with no flags.");
	ShowString(string);
	gOSError = SpeakBuffer(gChannel, string,  strlen(string), 0);
	ReportSpeechError(gOSError);	
	FinishSpeaking();
	sprintf(string, "Testing speak buffer with no ending prosody.");
	ShowString(string);
	gOSError = SpeakBuffer(gChannel, string,  strlen(string), kNoEndingProsody);
	ReportSpeechError(gOSError);	
	FinishSpeaking();
	sprintf(string, "Testing speak buffer with no interruptions.");
	ShowString(string);
	gOSError = SpeakBuffer(gChannel, string,  strlen(string), 0);
	ReportSpeechError(gOSError);	
	sprintf(string, "We should get a channel busy error here.");
	ShowString(string);
	gOSError = SpeakBuffer(gChannel, string,  strlen(string), kNoSpeechInterrupt);
	ReportSpeechError(gOSError);	
	FinishSpeaking();
	sprintf(string, "Press the mouse button");
	ShowString(string);
	gOSError = SpeakBuffer(gChannel, string,  strlen(string), 0);
	ReportSpeechError(gOSError);	
	FinishSpeaking();
	sprintf(string, "to test speak buffer preflight.");
	ShowString(string);
	gOSError = SpeakBuffer(gChannel, string,  strlen(string), kPreflightThenPause);
	ReportSpeechError(gOSError);	
	while(!Button())
	{ }
	ContinueSpeech(gChannel);
	FinishSpeaking();
}

void TextToPhonemesTest()
{
	Handle	PhonemeHandle;
	long 	PhonemeBytes;
	char	string[256];
	OSType	mode = modePhonemes;

	sprintf(string, "Braz loves willa.");
	PhonemeHandle = NewHandle(1024); 
	if(PhonemeHandle)
	{
		gOSError = TextToPhonemes(gChannel, &string, strlen(string), PhonemeHandle, &PhonemeBytes);
	}
	ReportSpeechError(gOSError);	
	gOSError = SetSpeechInfo( gChannel, soInputMode, &mode);
	ReportSpeechError(gOSError);	
	ShowText( *PhonemeHandle, PhonemeBytes );
	gOSError = SpeakText( gChannel, *PhonemeHandle, PhonemeBytes );
	ReportSpeechError(gOSError);	
	FinishSpeaking();
	mode = modeText;
	gOSError = SetSpeechInfo( gChannel, soInputMode, &mode);
	ReportSpeechError(gOSError);	
}

void VersionTest()
{
	NumVersion		TheVersion;
	
	TheVersion = SpeechManagerVersion();
	speak("Speech Manager version"); 
	SpeakNumVersion(TheVersion);
}
	
void UseDictionaryTest()
{
	Handle 		dictionary;
	OSType		DictionaryType = 'dict';
	
	dictionary = GetResource( DictionaryType, 0 );
	gOSError = UseDictionary( gChannel, dictionary);
	ReportSpeechError(gOSError);	
	speak("FELIX"); 
}

void DisposeSpeechChannelTest()
{
	if(gChannel)
	{
		gOSError = DisposeSpeechChannel(gChannel);
		ReportSpeechError(gOSError);
		ShowString("Disposed the channel.");
		SpeakString("\pDisposed the channel.");
	}
	else 
	{
		ShowString("No speech channel to dispose.");
		SpeakString("\pNo speech channel to dispose.");
	}
	FinishSpeaking();
	gOSError = NewSpeechChannel(0,  &gChannel);
	ReportSpeechError(gOSError);
	AdjustMenus();
}

void GetSpeechStatusTest()
{
	SpeechStatusInfo	TheInfo, AnotherInfo;
	
	gOSError = GetSpeechInfo( gChannel, soStatus, &TheInfo );
	ReportSpeechError(gOSError);	
	gOSError = GetSpeechInfo( gChannel, soStatus, &AnotherInfo );
	ReportSpeechError(gOSError);	
	while(AnotherInfo.outputBusy)
	{
		gOSError = GetSpeechInfo( gChannel, soStatus, &AnotherInfo );
	}
	ReportSpeechError(gOSError);	
	if(!TheInfo.outputBusy)
	{
		speak("not");
	}
	speak( "busy");
	if(!TheInfo.outputPaused)
	{
		speak("not");
	}
	speak( "paused");
	SpeakNumber(TheInfo.inputBytesLeft);
	speak("bytes left");
	SpeakNumber((long) TheInfo.phonemeCode);
	speak("is the phoneme code.");
}

void GetSpeechErrorTest()
{
	SpeechErrorInfo  TheInfo;
	
	gOSError = GetSpeechInfo( gChannel, soErrors, &TheInfo );
	ReportSpeechError(gOSError);	
	speak( "error count is");
	SpeakNumber((long) TheInfo.count);
	speak( "oldest error is");
	SpeakNumber((long) TheInfo.oldest);
	speak( "position");
	SpeakNumber((long) TheInfo.oldPos);
	speak( "newest error is");
	SpeakNumber((long) TheInfo.newest);
	speak( "position");
	SpeakNumber(TheInfo.newPos);
	
}

void GetInputModeTest()
{
	OSType  TheInfo;

	gOSError = GetSpeechInfo( gChannel, soInputMode, &TheInfo );
	ReportSpeechError(gOSError);	
	speak("Input mode is"); 
	if((TheInfo == modeText) || (TheInfo == modeTX))
	{
		speak("text.");
	}
	else 
	{
		if((TheInfo == modePhonemes) || (TheInfo == modePH))
		{
			speak("phonemic.");
		}
		else
		{
			speak("not recognised.");
		}
	}
}

void GetCharacterModeTest()
{
	OSType  TheInfo;

	gOSError = GetSpeechInfo( gChannel, soCharacterMode, &TheInfo );
	ReportSpeechError(gOSError);	
	speak("Character mode is"); 
	if(TheInfo == modeNormal)
	{
		speak("normal.");
	}
	else 
	{
		if(TheInfo == modeLiteral)
		{
			speak("literal.");
		}
		else
		{
			speak("not recognised.");
		}
	}
}
void GetNumberModeTest()
{
	OSType  TheInfo;

	gOSError = GetSpeechInfo( gChannel, soNumberMode, &TheInfo );
	ReportSpeechError(gOSError);	
	speak("Number mode is"); 
	if(TheInfo == modeNormal)
	{
		speak("normal.");
	}
	else 
	{
		if(TheInfo == modeLiteral)
		{
			speak("literal.");
		}
		else
		{
			speak("not recognised.");
		}
	}
}

void GetSpeechPitchModTest()
{
	Fixed  TheInfo;

	speak("Speech pitch modulation is"); 
	gOSError = GetSpeechInfo( gChannel, soPitchMod, &TheInfo );
	ReportSpeechError(gOSError);	
	SpeakNumber( Fix2Long(TheInfo));
}

void GetSpeechVolumeTest()
{
	Fixed  TheInfo;

	speak("Speech volume is"); 
	gOSError = GetSpeechInfo( gChannel, soVolume, &TheInfo );
	ReportSpeechError(gOSError);	
	SpeakFixed(TheInfo);
}

void GetSpeechVersionInfoTest()
{
	SpeechVersionInfo  TheInfo;

	gOSError = GetSpeechInfo( gChannel, soSynthType, &TheInfo );
	ReportSpeechError(gOSError);	
	speak("Synth type is");
	SpeakOSType(TheInfo.synthType);
	speak(" synth sub type is"); 
	SpeakOSType(TheInfo.synthSubType); 
	speak(" synth manufacturer is"); 
	SpeakOSType(TheInfo.synthManufacturer); 
	speak(" synth flags are"); 
	SpeakNumber(TheInfo.synthFlags);
	speak(" synth version is"); 
	SpeakNumVersion(TheInfo.synthVersion);
}

void GetSpeechSyncTest()
{
	OSType  TheInfo;

	speak("[[ sync 69 ]]"); 
	gOSError = GetSpeechInfo( gChannel, soRecentSync, &TheInfo );
	ReportSpeechError(gOSError);	
	speak("Most recent sync message is"); 
	SpeakNumber((long) TheInfo);
}

void GetPhonemeSymbolsTest()
{
	Handle  	TheInfoHandle;
	PhonemeDescriptor	*TheInfoPtr;
	short	i;

	gOSError = GetSpeechInfo( gChannel, soPhonemeSymbols, &TheInfoHandle );
	ReportSpeechError(gOSError);	
	TheInfoPtr = (PhonemeDescriptor *) *TheInfoHandle;
	for( i = 0; i != TheInfoPtr->phonemeCount; i++)
	{
		PToCString(TheInfoPtr->thePhonemes[i].phStr);
		speak((char *) TheInfoPtr->thePhonemes[i].phStr);
		PToCString(TheInfoPtr->thePhonemes[i].exampleStr);
		speak((char *) TheInfoPtr->thePhonemes[i].exampleStr);
	}
}

void UserSetRate()
{
	long	NewRate;
	short	SavedVoice;
	
	Fixed	OldPitch;
	
	gOSError = GetSpeechPitch(gChannel, &OldPitch);
	ReportSpeechError(gOSError);
	SavedVoice = gCurrentVoice;
	SetVoice(3);
	SpeakAsync("Enter a new rate please");
	NewRate = StdGetNumLong("Enter the new rate", "200");
	gOSError = SetSpeechRate(gChannel, Long2Fix(NewRate));
	ReportSpeechError(gOSError);
	speak("new rate is");
	SpeakNumber(NewRate);
	SetVoice(SavedVoice);
	gOSError = SetSpeechRate(gChannel, Long2Fix(NewRate));
	ReportSpeechError(gOSError);
	gOSError = SetSpeechPitch(gChannel, OldPitch);
	ReportSpeechError(gOSError);
}

void UserSetPitch()
{
	long	NewPitch;
	short	SavedVoice;
	Fixed	OldRate;
	
	gOSError = GetSpeechRate(gChannel, &OldRate);
	ReportSpeechError(gOSError);
	SavedVoice = gCurrentVoice;
	SetVoice(3);
	SpeakAsync("Enter a new pitch please");
	NewPitch = StdGetNumLong("Enter the new pitch", "30");
	gOSError = SetSpeechPitch(gChannel, Long2Fix(NewPitch));
	ReportSpeechError(gOSError);
	speak("new pitch is");
	SpeakNumber(NewPitch);
	SetVoice(SavedVoice);
	gOSError = SetSpeechPitch(gChannel, Long2Fix(NewPitch));
	ReportSpeechError(gOSError);
	gOSError = SetSpeechRate(gChannel, OldRate);
	ReportSpeechError(gOSError);

}

void TextInputMode()
{
	OSType	mode = modeText;

	gOSError = SetSpeechInfo( gChannel, soInputMode, &mode);
	ReportSpeechError(gOSError);
	CheckItem(GetMHandle(channelsMenu), acceptPhonemesItem, false);
	CheckItem(GetMHandle(channelsMenu), acceptTextItem, true);
	SpeakAsync("Input mode is text.");
}

void PhonemecInputMode()
{
	OSType	mode = modePhonemes;

	gOSError = SetSpeechInfo( gChannel, soInputMode, &mode);
	ReportSpeechError(gOSError);		
	CheckItem(GetMHandle(channelsMenu), acceptTextItem, false);
	CheckItem(GetMHandle(channelsMenu), acceptPhonemesItem, true);
	SpeakAsync("fOWnIYmz.");
}

void LiteralCharacters()
{
	OSType	mode; 

	speak("Character Mode is");
	gOSError = GetSpeechInfo( gChannel, soCharacterMode, &mode);
	ReportSpeechError(gOSError);		
	if(mode == modeLiteral)
	{
		mode = modeNormal;
		gOSError = SetSpeechInfo( gChannel, soCharacterMode, &mode);
		SpeakAsync("normal");
		CheckItem(GetMHandle(channelsMenu), literalCharactersItem, false);
	}
	else
	{
		mode = modeLiteral;
		gOSError = SetSpeechInfo( gChannel, soCharacterMode, &mode);
		SpeakAsync("literal");
		CheckItem(GetMHandle(channelsMenu), literalCharactersItem, true);
	}
	ReportSpeechError(gOSError);		
}

void LiteralNumbers()
{
	OSType	mode; 

	speak("Number Mode is");
	gOSError = GetSpeechInfo( gChannel, soNumberMode, &mode);
	ReportSpeechError(gOSError);		
	if(mode == modeLiteral)
	{
		mode = modeNormal;
		gOSError = SetSpeechInfo( gChannel, soNumberMode, &mode);
		speak("normal");
		CheckItem(GetMHandle(channelsMenu), literalNumbersItem, false);
	}
	else
	{
		mode = modeLiteral;
		gOSError = SetSpeechInfo( gChannel, soNumberMode, &mode);
		speak("literal");
		CheckItem(GetMHandle(channelsMenu), literalNumbersItem, true);
	}
	ReportSpeechError(gOSError);		
	SpeakAsync("123");
}

void PitchModulationToggle()
{
	Fixed  TheModulation;

	gOSError = GetSpeechInfo( gChannel, soPitchMod, &TheModulation );
	ReportSpeechError(gOSError);	

	if(TheModulation == 0)
	{
		TheModulation = Long2Fix(1);
		gOSError = SetSpeechInfo( gChannel, soPitchMod, &TheModulation);
		ReportSpeechError(gOSError);		
		CheckItem(GetMHandle(channelsMenu), pitchModulationItem, true);
		speak("Pitch modulation is on."); 
	}
	else
	{
		TheModulation = 0;
		gOSError = SetSpeechInfo( gChannel, soPitchMod, &TheModulation);
		ReportSpeechError(gOSError);		
		CheckItem(GetMHandle(channelsMenu), pitchModulationItem, false);
		speak("Pitch modulation is off."); 
	}
}

void SpeechReset()
{
	long	ResetValue = 0;
	
	gOSError = SetSpeechInfo( gChannel, soReset, &ResetValue);
	ReportSpeechError(gOSError);		
	SpeakAsync("Channel has been reset.");
}
	
void SetA5Test()
{
	Ptr	MyA5 = SetCurrentA5();

	gOSError = SetSpeechInfo( gChannel, soCurrentA5, MyA5);
	ReportSpeechError(gOSError);		
	CheckItem(GetMHandle(callBacksMenu), setA5Item, true);
}

void SetRefConTest()
{
	long	RefCon = 69;

	gOSError = SetSpeechInfo( gChannel, soRefCon, (void *) RefCon);
	ReportSpeechError(gOSError);		
	CheckItem(GetMHandle(callBacksMenu), refConItem, true);
}

void NewDelimiters()
{
	OSType	delimiters = '%%&&';

	gOSError = SetSpeechInfo( gChannel, soCommandDelimiter, &delimiters);
	ReportSpeechError(gOSError);
	SpeakAsync(" %% rate 350 && did this set the rate faster?");
}

pascal void TextDoneCallback(SpeechChannel chan, unsigned long refCon, 
							Ptr *NextBuffer, unsigned long *ByteLength, unsigned long *ControlFlags)
{
#pragma unused(chan, refCon)

	*NextBuffer = 0;
	*ByteLength = 0;
	*ControlFlags = 0;

	gTextDoneCallbackCount += 1;
}

pascal void SpeechDoneCallback(SpeechChannel chan, unsigned long refCon)
{
#pragma unused(chan, refCon)

	gSpeechDoneCallbackCount += 1;
}

pascal void SyncCallback(SpeechChannel chan, unsigned long refCon, OSType selector)
{
#pragma unused(chan, refCon, selector)

	gSyncCallbackCount += 1;
}

pascal void ErrorCallback(SpeechChannel chan, unsigned long refCon, OSErr selector, long bytePos)
{
#pragma unused(chan, refCon, selector, bytePos)

	gErrorCallbackCount += 1;
}

pascal void PhonemeCallback(SpeechChannel chan, unsigned long refCon, short opcode)
{
#pragma unused(chan, refCon, opcode)

	gPhonemeCallbackCount += 1;
}

pascal void WordCallback(SpeechChannel chan, unsigned long refCon, long wordPos, short wordLen)
{
#pragma unused(chan, refCon, wordPos, wordLen)

	gWordCallbackCount += 1;
}

ProcPtr			gDummyCallback = NULL;


void NewTextDoneCallback()
{
	if(!gTextDoneCallbackInstalled)
	{
		gOSError = SetSpeechInfo( gChannel, soTextDoneCallBack, &TextDoneCallback);
	}
	else
	{
		gOSError = SetSpeechInfo( gChannel, soTextDoneCallBack, gDummyCallback);
		gTextDoneCallbackCount   = 0;
	}
	ReportSpeechError(gOSError);		
	gTextDoneCallbackInstalled = !gTextDoneCallbackInstalled;
	CheckItem(GetMHandle(callBacksMenu), textDoneItem, gTextDoneCallbackInstalled);
}

void NewSpeechDoneCallback()
{
	if(!gSpeechDoneCallbackInstalled)
	{
		gOSError = SetSpeechInfo( gChannel, soSpeechDoneCallBack, &SpeechDoneCallback);
		gSpeechDoneCallbackCount = 0;
	}
	else
	{
		gOSError = SetSpeechInfo( gChannel, soSpeechDoneCallBack, gDummyCallback);
	}
	ReportSpeechError(gOSError);		
	gSpeechDoneCallbackInstalled = !gSpeechDoneCallbackInstalled;
	CheckItem(GetMHandle(callBacksMenu), speechDoneItem, gSpeechDoneCallbackInstalled);
}

void NewSyncCallback()
{
	if(!gSyncCallbackInstalled)
	{
		gOSError = SetSpeechInfo( gChannel, soSyncCallBack, &SyncCallback);
	}
	else
	{
		gOSError = SetSpeechInfo( gChannel, soSyncCallBack, gDummyCallback);
		gSyncCallbackCount = 0;
	}
	ReportSpeechError(gOSError);		
	gSyncCallbackInstalled = !gSyncCallbackInstalled;
	CheckItem(GetMHandle(callBacksMenu), syncItem, gSyncCallbackInstalled);
}

void NewErrorCallback()
{
	if(!gErrorCallbackInstalled)
	{
		gOSError = SetSpeechInfo( gChannel, soErrorCallBack, &ErrorCallback);
	}
	else
	{
		gOSError = SetSpeechInfo( gChannel, soErrorCallBack, gDummyCallback);
		gErrorCallbackCount = 0;
	}
	ReportSpeechError(gOSError);		
	gErrorCallbackInstalled = !gErrorCallbackInstalled;
	CheckItem(GetMHandle(callBacksMenu), errorItem, gErrorCallbackInstalled);
}

void NewPhonemeCallback()
{
	if(!gPhonemeCallbackInstalled)
	{
		gOSError = SetSpeechInfo( gChannel, soPhonemeCallBack, &PhonemeCallback);
	}
	else
	{
		gOSError = SetSpeechInfo( gChannel, soPhonemeCallBack, gDummyCallback);
		gPhonemeCallbackCount = 0;
	}
	ReportSpeechError(gOSError);		
	gPhonemeCallbackInstalled = !gPhonemeCallbackInstalled;
	CheckItem(GetMHandle(callBacksMenu), phonemeItem, gPhonemeCallbackInstalled);
}

void NewWordCallback()
{
	if(!gWordCallbackInstalled)
	{
		gOSError = SetSpeechInfo( gChannel, soWordCallBack, &WordCallback);
	}
	else
	{
		gOSError = SetSpeechInfo( gChannel, soWordCallBack, gDummyCallback);
		gWordCallbackCount = 0;
	}
	ReportSpeechError(gOSError);		
	gWordCallbackInstalled = !gWordCallbackInstalled;
	CheckItem(GetMHandle(callBacksMenu), wordItem, gWordCallbackInstalled);
}

void CallBacksOff()
{
	gOSError = SetSpeechInfo( gChannel, soTextDoneCallBack, gDummyCallback);
	gOSError = SetSpeechInfo( gChannel, soSpeechDoneCallBack, gDummyCallback);
	gOSError = SetSpeechInfo( gChannel, soSyncCallBack, gDummyCallback);
	gOSError = SetSpeechInfo( gChannel, soErrorCallBack, gDummyCallback);
	gOSError = SetSpeechInfo( gChannel, soPhonemeCallBack, gDummyCallback);
	gOSError = SetSpeechInfo( gChannel, soWordCallBack, gDummyCallback);
	gTextDoneCallbackInstalled = false;
	gSpeechDoneCallbackInstalled = false;
	gSyncCallbackInstalled = false;
	gErrorCallbackInstalled = false;
	gPhonemeCallbackInstalled = false;
	gWordCallbackInstalled = false;
	ReportSpeechError(gOSError);		
	CheckItem(GetMHandle(callBacksMenu), textDoneItem, false);
	CheckItem(GetMHandle(callBacksMenu), speechDoneItem, false);
	CheckItem(GetMHandle(callBacksMenu), syncItem, false);
	CheckItem(GetMHandle(callBacksMenu), errorItem, false);
	CheckItem(GetMHandle(callBacksMenu), phonemeItem, false);
	CheckItem(GetMHandle(callBacksMenu), wordItem, false);
}

void CallBackCount()
{
	SpeakNumber(gTextDoneCallbackCount);
	speak("Text Done Callbacks");
	SpeakNumber(gSpeechDoneCallbackCount);
	speak("Speech Done Callbacks");
	SpeakNumber(gSyncCallbackCount);
	speak("Sync Callbacks");
	SpeakNumber(gErrorCallbackCount);
	speak("Error Callbacks");
	SpeakNumber(gPhonemeCallbackCount);
	speak("Phoneme Callbacks");
	SpeakNumber(gWordCallbackCount);
	speak("Word Callbacks");
	gTextDoneCallbackCount   = 0;
	gSpeechDoneCallbackCount = 0;
	gSyncCallbackCount 		 = 0;
	gErrorCallbackCount 	 = 0;
	gPhonemeCallbackCount	 = 0;
	gWordCallbackCount 		 = 0;
}

#define NumSteps		10
#define FixedIncrement	(Fixed1/NumSteps)

void VolumeTest()
{
	Fixed		TheVolume;
	long		i;
	
	for( i = Fixed1; i >= 0; i -= FixedIncrement)
	{
		TheVolume = (Fixed) i;
		gOSError = SetSpeechInfo( gChannel, soVolume, &TheVolume);
		ReportSpeechError(gOSError);	
		SpeakFixed(TheVolume);
	}
}


void SpeakFile()
{
	SFReply			theReply;
	Point 			where = {20, 90};
	SFTypeList		typeList = {'TEXT'};
	OSErr			err;
	short			fRefNum = 0;
	long 			fileLen;
	Handle			textHandle = nil;
	
	SFGetFile(where, 0, 0, 1, typeList, 0, &theReply); /* get a text file */
	
	if (theReply.good)
	{
		err = FSOpen(theReply.fName, theReply.vRefNum, &fRefNum); /* Open the file */
		if (err) {							/* failed on open */
			fRefNum = 0;
			goto BOMB_OUT;
		}
		
		err = GetEOF(fRefNum, &fileLen);	/* find out how long file is */
		if (err) goto BOMB_OUT;
		
		textHandle = NewHandle(fileLen); 	/* get some memory to hold text */
		if (!textHandle) {
			err = MemError();
			goto BOMB_OUT;
		}
		
		MoveHHi(textHandle);
		HLock(textHandle);
		
		err = FSRead(fRefNum, &fileLen, *textHandle); /* read in the text */
		if (err) goto BOMB_OUT;
					
		FSClose(fRefNum);					/* close the disk file */
	}
		
	if (textHandle)
	{
		SpeakText(gChannel, *textHandle, fileLen);
		ShowString("Press the mouse button to stop");
		while(SpeechBusy())
		{
			if(Button())
			{
				gOSError = StopSpeech(gChannel);
				ReportSpeechError(gOSError);
			}
		}
		DisposeHandle(textHandle);
	}
	
	return;									// everything worked

	//______________ ERROR HANDLERS ________________________________________

	BOMB_OUT:
		SysBeep(10);						// should put up some alert here!
		
		if (fRefNum != 0)					// opened a file
			FSClose(fRefNum);				// close it
		
		if (textHandle)						// allocated some RAM
			DisposeHandle(textHandle);		// free it up
}
