/* Note: anything marked ?? needs to be changed */
/*									AOCEErrorESTR.r								*/
/*
 * Error resource strings for AOCE 1.0
 * Copyright  1993 Apple Computer Inc. All rights reserved.
 * Based on <OCEErrors.h> Created: Friday, June 4, 1993 at 10:05AM
 *
 * This is based on <OCEErrors.h>. Unfortunately,  Rez does not understand enum's,
 * so this requires "hard-coded" resource values. This may be incorporated into
 * Think Class Library applications by adding these resources to your project file.
 * In other circumstances, you should do something such as the following to present
 * text messages to your users:
 *
 *	void
 *	MyErrorAlert(
 *			OSErr			errorValue
 *		)
 *	{
 *		Str15				errorValue;
 *		Handle				errorHdl;
 *		StringPtr			errorText;
 *		
 *		NumToString(errorValue, errorValueText);
 *		errorHdl = GetResource('Estr', errorValue);
 *		if (errorHdl == NULL)
 *			errorText = "\p";
 *		else {
 *			HLock(errorHdl);
 *			errorText = (StringPtr) *errorHdl;
 *		}
 *		ParamText(errorValue, errorText, "\p", "\p");
 *		Alert(kMyALRTResourceID, NULL);
 *		if (errorHdl != NULL)
 *			ReleaseResource(errorHdl);
 *	}
 *
 * Error text resources have the following format.
 *		<OCEErrors.h symbol> ": " explanatory text
 *	Explanatory text begins with a capital letter. There is no trailing
 *	punctuation. The <OCEErrors.h> symbole is for the programmer's benefit,
 *	while the text after ": " might be better understood by application users.
 *	Only the text after ": " should be translated when these resources are
 *	localized.
 *
 * Error text marked ?? is not documented in <OCEErrors.h>
 * Parts of this text won't fit on an A4 or 8 1/2 inch vertical page.
 *
 * To decrease the size of the resource fork created by this module, you can
 * define one or more pre-processor symbols to exclude message components. The
 * General toolbox and authentication manager errors should normally not be
 * excluded, but, for example, the Digital Signature errors could be omitted
 * if your application does not use Digital Signatures. The following may be
 * defined to omit a component:
 *	__AOCE_Exclude_General_Errors
 *	__AOCE_Exclude_Authentication_Errors
 *	__AOCE_Exclude_CatalogManager_Errors
 *	__AOCE_Exclude_StandardMail_Errors
 *	__AOCE_Exclude_DigitalSignature_Errors
 *	__AOCE_Exclude_Template_Errors
 *	__AOCE_Exclude_Mail_Errors
 *	__AOCE_Exclude_Messaging_Errors
 *
 * For example, in MPW, you might specify the following in your MakeFile:
 *	Rez													
 *		AOCEErrorEstr.r									
 *		-d "__AOCE_Exclude_DigitalSignature_Errors"		
 *		-append											
 *		-t APPL											
 *		-c DPEK											
 *		-i "{CIncludes}"								
 *		-i "{RIncludes}"								
 *		-o "MyApplication"
 * In Symantec C or C++, you would define the symbol in the Rez prefix dialog.
 */
 
#include "Types.r"
type 'Estr' as 'STR ';

#ifndef __AOCE_Exclude_General_Errors
/*
 * General toolbox errors (not AOCE specific)
 */
resource 'Estr' (-50)   { "paramErr: Invalid parameter passed to toolbox" };
resource 'Estr' (-108)	{ "memFullErr: Not enough free memory to allocate storage" };

/*
 * General errors (-1500 to -1539)
 */
resource 'Estr' (-1500)	{ "kOCEToolboxNotOpen: Toolbox not open, perhaps it's closing" };
resource 'Estr' (-1501) { "kOCEInvalidCommand: Invalid Toolbox request code" };
resource 'Estr' (-1502) { "kOCEInvalidRef: Obsolete or invalid object reference" };
resource 'Estr' (-1503) { "kOCEBufferTooSmall: Buffer too small for toolbox request" };
resource 'Estr' (-1504) { "kOCEVersionError: Software version mismatch" };
resource 'Estr' (-1505) { "kOCESyncAsyncErr: Toolbox request requires synch (or asynch) call" };
resource 'Estr' (-1506) { "kOCEInternalErr: Toolbox internal error" };
resource 'Estr' (-1507) { "kOCENotAnOwner: Operation can only be done by object's owner" };
resource 'Estr' (-1508) { "kOCENotImplemented: Feature not implemented" };
resource 'Estr' (-1509) { "kOCENotAuthenticated: User must be authenticated for this request" };
resource 'Estr' (-1510) { "kOCEAlreadyExists: The object being added already exists" };
resource 'Estr' (-1511) { "kOCEDoesntExist: The object doesn't exist or cannot be found" };
resource 'Estr' (-1512) { "kOCEConnectionErr: Unable to open or maintain a network connection" };
resource 'Estr' (-1513) { "kOCEConnectionClosed: Network connection has closed" };
resource 'Estr' (-1514) { "kOCEInvalidRecipient: Invalid recipient or address for this request" };
resource 'Estr' (-1515) { "kOCEInvalidIndex: Invalid index (out of range)" };
resource 'Estr' (-1516) { "kOCERefIsClosing: Object reference is being closed; cannot complete command" };
resource 'Estr' (-1517) { "kOCEHasPendingMsgs: ??" };
resource 'Estr' (-1518) { "kOCEInvalidConfiguration: ??" };
resource 'Estr' (-1519) { "kOCERequiredServicesNotAvailable: ??" };
resource 'Estr' (-1520) { "kOCEServerInactive: ??" };
resource 'Estr' (-1521) { "kOCEMoreDiskSpaceNeeded: Inadequate disk space to safely complete command" };
#endif /* __AOCE_Exclude_General_Errors */

#ifndef __AOCE_Exclude_Authentication_Errors
/*
 * AOCE Authentication Errors (-1540 to -1609)
 */
resource 'Estr' (-1540) { "kOCEReadAccessDenied: Read access denied" };
resource 'Estr' (-1541) { "kOCEWriteAccessDenied: Write access denied" };
resource 'Estr' (-1542) { "kOCEAccessRightsInsufient: User not authenticated or authorized" };
resource 'Estr' (-1543) { "kOCEUnsupportedCredentialsVersion: Cannot read that version of credentials" };
resource 'Estr' (-1544) { "kOCECredentialsProblem: Cannot validate credentials" };
resource 'Estr' (-1545) { "kOCECredentialsImmature: Current time prior to valid time" };
resource 'Estr' (-1546) { "kOCECredentialsExpired: Current time past expiration time" };
resource 'Estr' (-1547) { "kOCEProxyImmature: Current time prior to proxy valid time" };
resource 'Estr' (-1548) { "kOCEProxyExpired: Current time past proxy expiration time" };
resource 'Estr' (-1549) { "kOCEDisallowRecipient: Recipient not specified in proxy" };
resource 'Estr' (-1550) { "kOCENoKeyFound: Found no key for entity in question" };
resource 'Estr' (-1551) { "kOCEPrincipalKeyNotFound: Couldn't validate proxy because principal has no key" };
resource 'Estr' (-1552) { "kOCERecipientKeyNotFound: Recipient has no key" };
resource 'Estr' (-1553) { "kOCEAgentKeyNotFound: In TPPC (??), agent has no key" };
resource 'Estr' (-1554) { "kOCEKeyAlreadyRegistered: Entity already has a key" };
resource 'Estr' (-1555) { "kOCEMalformedKey: Unknown key format or key doesn't match password" };
resource 'Estr' (-1556) { "kOCEUndesireableKey: Password too short, key too simple, or trying to change cluster key to the same value" };
resource 'Estr' (-1557) { "kOCEWrongIdentityOrKey: Name doesn't exist or key incorrect" };
resource 'Estr' (-1558) { "kOCEInitiatorKeyProblem: No initiator's key or key cannot be processedt" };
resource 'Estr' (-1559) { "kOCEBadEncryptionMethod: Specified encryption method not supportedt" };
resource 'Estr' (-1560) { "kOCELocalIdentityDoesNotExist: Local identity has not been specified" };
resource 'Estr' (-1561) { "kOCELocalAuthenticationFail: User must login for this operation" };
resource 'Estr' (-1562) { "kOCELocalIdentitySetupExists: Local identity setup exists; use change instead of setup" };
resource 'Estr' (-1563) { "kOCEDirectoryIdentitySetupExists: DirectoryIdentity setup exists; use change instead of setup" };
resource 'Estr' (-1564) { "kOCEDirectoryIdentitySetupDoesNotExist: Directory identity does not exist" };
resource 'Estr' (-1565) { "kOCENotLocalIdentity: Use local identity toolbox calls (??)" };
resource 'Estr' (-1566) { "kOCENoMoreIDs: Identity table is full" };
resource 'Estr' (-1567) { "kOCEUnknownID: Invalid identity password" };
resource 'Estr' (-1568) { "kOCEOperationDenied: Local identity operation fails" };
resource 'Estr' (-1569) { "kOCEAmbiguousMatches: Ambiguous match; creation id was not unique"};
resource 'Estr' (-1570) { "kOCENoASDSPWorkSpace: No workspace passed to Open Secure Stream toolbox call"};
resource 'Estr' (-1571) { "kOCEAuthenticationTrouble: Authentication failed"};
#endif /* __AOCE_Exclude_Authentication_Errors */

#ifndef __AOCE_Exclude_DirectoryManager_Errors
/*
 * AOCE Catalog Errors (-1610 to -1680)
 */
resource 'Estr' (-1610) { "kOCENotLocal: The server does not have the requested catalog item"};
resource 'Estr' (-1611) { "kOCETooBusy: The server cannot process this request at this time"};
resource 'Estr' (-1612) { "kOCEDatabaseFull: Device is full"};
resource 'Estr' (-1613) { "kOCETargetDirectoryInaccessable: Catalog server did not respond to the request"};
resource 'Estr' (-1614) { "kOCEBogusArgs: Arguments incorrect in request message"};
resource 'Estr' (-1615) { "kOCENoSuchDNode: The requested catalog item does not exist"};
resource 'Estr' (-1616) { "kOCENodeUnavailable: Could not find any server that provides the requested catalog item"};
resource 'Estr' (-1617) { "kOCEBadRecordID: Invalid record identifier (usually name and/or type do not match creation id)"};
resource 'Estr' (-1618) { "kOCENoSuchRecord: Specified record does not exist"};
resource 'Estr' (-1619) { "kOCENoSuchAttributeValue: Attribute value does not exist"};
resource 'Estr' (-1620) { "kOCENoSuchPseudonym: Specified pseudonym does not exist"};
resource 'Estr' (-1621) { "kOCEAttributeValueTooBig: Cannot store attribute value"};
resource 'Estr' (-1622) { "kOCETypeExists: Specified attribute type already exists in the record"};
resource 'Estr' (-1623) { "kOCEMoreData: More data available for request - not an error"};
resource 'Estr' (-1624) { "kOCERefNumBad: Invalid catalog reference number"};
resource 'Estr' (-1625) { "kOCEStreadCreationErr: Cannot create stream"};
resource 'Estr' (-1626) { "kOCEOperationNotSupported: Specified catalog does not support requested operation"};
resource 'Estr' (-1627) { "kOCEPABNotOpen: Specified Personal Catalog not open for requested operation"};
resource 'Estr' (-1628) { "kOCEDSAMInstallErr: Specified Catalog Service Access Modle could not be installed"};
resource 'Estr' (-1629) { "kOCEDirListFullErr: Catalog list is full; try removing an entry"};
resource 'Estr' (-1630) { "kOCEDirectoryNotFoundErr: Catalog not found in the list" };
resource 'Estr' (-1631) { "kOCEAbortNotSupportedForThisCall: This request cannot be aborted" };
resource 'Estr' (-1632) { "kOCEAborted: This request was aborted" };
resource 'Estr' (-1633) { "kOCEOCESetupRequired: Local identity setup is required for this request" };
resource 'Estr' (-1634) { "kOCEDSAMRecordNotFound: Catalog Service Access Module was not found" };
resource 'Estr' (-1635) { "kOCEDSAMNotInstantiated: Catalog Service Access Module was not initialized" };
resource 'Estr' (-1636) { "kOCEDSAMRecordExists: Catalog Service Access Module already exists" };
resource 'Estr' (-1637) { "kOCELengthError: Supplied buffer was too small" };
resource 'Estr' (-1638) { "kOCEBadStartingRecord: Lookup starting record was not within range" };
resource 'Estr' (-1639) { "kOCEBadStartingAttribute: Lookup starting attribute was not within range " };
resource 'Estr' (-1640) { "kOCEMoreAttrValue: Attribute value could not fit into supplied buffer (even though it was the only value)" };
resource 'Estr' (-1641) { "kOCENoDupAllowed: No duplicate allowed" };
resource 'Estr' (-1642) { "kOCENoSuchAttributeType: Attribute type not present" };
resource 'Estr' (-1643) { "kOCEMiscError: Catalog Manager error" };
resource 'Estr' (-1644) { "kOCENoSuchIcon: Requested icon type not available" };
resource 'Estr' (-1645) { "kOCERLIsDontMatch: Record location identifiers do not match in lookup" };
resource 'Estr' (-1646) { "kOCEDirectoryCorrupt: Serious catalog corruption problem" };
#endif /* __AOCE_Exclude_DirectoryManager_Errors */

#ifndef __AOCE_Exclude_StandardMail_Errors
/*
 * AOCE Standard Mail Errors (-1900 to -1949)
 */
resource 'Estr' (-1900) { "kSMPNotEnoughMemoryForAllRecips: Not enough memory for all recipients" };
resource 'Estr' (-1901) { "kSMPCopyInProgress: Copy in progress - operation cannot be done at this time" };
resource 'Estr' (-1902) { "kSMPMailerNotInitialized: Mailer not initialized" };
resource 'Estr' (-1903) { "kSMPShouldNotAddContent: Should not add content" };
resource 'Estr' (-1904) { "kSMPMailboxNotFound: Mailbox not found" };
resource 'Estr' (-1905) { "kSMPNoNextLetter: No next letter" };
resource 'Estr' (-1906) { "kSMPHasOpenAttachments: Message has open attachments" };
resource 'Estr' (-1907) { "kSMPFinderNotRunning: Finder not running" };
resource 'Estr' (-1908) { "kSMPCommandDisabled: Command disabled" };
resource 'Estr' (-1909) { "kSMPNoMailerInWindow: No mailer in this window" };
resource 'Estr' (-1910) { "kSMPNoSuchAddress: No such address" };
resource 'Estr' (-1911) { "kSMPMailerAlreadyInWindow: Mailer already in window" };
resource 'Estr' (-1912) { "kSMPMailerUneditable: Mailer not editable" };
resource 'Estr' (-1913) { "kSMPNoMatchingBegin: No matching begin" };
resource 'Estr' (-1914) { "kSMPCannotSendReceivedLetter: Cannot send received letter" };
resource 'Estr' (-1915) { "kSMPIllegalForDraftLetter: Illegal for draft letter" };
resource 'Estr' (-1916) { "kSMPMailerCannotExpandOrContract: Mailer cannot expand or contract" };
resource 'Estr' (-1917) { "kSMPMailerAlreadyExpandedOrContracted: Mailer already expanded or contracted" };
resource 'Estr' (-1918) { "kSMPIllegalComponent: Illegal component" };
resource 'Estr' (-1919) { "kSMPMailerAlreadyNotTarget: Mailer already not target" };
resource 'Estr' (-1920) { "kSMPComponentIsAlreadyTarget: Component is already target" };
resource 'Estr' (-1921) { "kSMPRecordDoesNotContainAddress: Record does not contain an address" };
resource 'Estr' (-1922) { "kSMPAddressAlreadyInList: Address already in list (warning)" };
resource 'Estr' (-1923) { "kSMPIllegalSendFormats: Illegal send format" };
resource 'Estr' (-1924) { "kSMPInvalidAddressString: Invalid address string" };
resource 'Estr' (-1925) { "kSMPSubjectTooBig: Subject too big" };
resource 'Estr' (-1926) { "kSMPParamCountErr: Parameter count error (enclosure count not one, but send enclosure TRUE)" };
resource 'Estr' (-1927) { "kSMPTooManyPages: Too many pages" };
resource 'Estr' (-1928) { "kSMPTooManyEnclosures: Too many enclosures" };
#endif /* __AOCE_Exclude_StandardMail_Errors */

#ifndef __AOCE_Exclude_CatalogManager_Errors
/*
 * AOCE Standard Catalog Errors (-1950 to -1969)
 */
resource 'Estr' (-1950) { "kSDPNoSearchText: No search text" };
resource 'Estr' (-1951) { "kSDPTooManyLoginAttempts: Too many login attempts" };
resource 'Estr' (-1952) { "kSDPNoSelection: No selection" };
resource 'Estr' (-1953) { "kSDPPersonalDirectoryRepairFailed: Could not repair personal catalog" };
#endif /* __AOCE_Exclude_CatalogManager_Errors */

#ifndef __AOCE_Exclude_DigitalSignature_Errors

/*
 * AOCE Digital Signature Errors (-1970 to -1999)
 */
resource 'Estr' (-1970) { "kSIGOperationIncompatibleErr: Context in use for different type of operation" };
resource 'Estr' (-1971) { "kSIGCertificateQueryDenied: Can't query certificates with this context" };
resource 'Estr' (-1972) { "kSIGVerifyFailedErr: Verification failed" };
resource 'Estr' (-1973) { "kSIGInvalidCredentialErr: Verified OK but credential out of date, either pending or expired" };
resource 'Estr' (-1974) { "kSIGIndexErr: The index given is outside the range of allowable values" };
resource 'Estr' (-1975) { "kSIGSignerErr: Problem with the signer or signature itself" };
resource 'Estr' (-1976) { "kSIGPasswordErr: Password is incorrect" };
resource 'Estr' (-1977) { "kSIGInternalsErr: An internal error occurred: bad digest, context or signature" };
resource 'Estr' (-1978) { "kSIGToolboxNotPresentErr: Digital Signature Manager not present" };	/* Not returned by Toolbox	*/
resource 'Estr' (-1979) { "kSIGContextPrepareErr: Context corrupted or incompatible with this operation" };
resource 'Estr' (-1980) { "kSIGNoDigestErr: No digest in the signature" };
resource 'Estr' (-1981) { "kSIGConversionErr: Cannot convert signature attribute to Macintosh format" };
#endif /* __AOCE_Exclude_DigitalSignature_Errors */

#ifndef __AOCE_Exclude_Template_Errors
/*
 * AOCE Templates Errors (-15000 to -15039)
 */
resource 'Estr' (-15000) { "kDETInvalidTargetAspectName: Could not locate an aspect by name" };
resource 'Estr' (-15001) { "kDETInvalidTargetItemNumber: Invalid (out of range) target item number " };
resource 'Estr' (-15002) { "kDETInvalidTargetFromNonAspect: This target selection requires an aspect" };
resource 'Estr' (-15003) { "kDETInvalidTargetDSSpec: Could not resolve target Catalog Specifier (DSSpec)" };
resource 'Estr' (-15004) { "kDETUnknownTargetSelector: Unknown selector type for target specification" };
resource 'Estr' (-15005) { "kDETInvalidTarget: Invalid Target specification" };
resource 'Estr' (-15006) { "kDETTargetNotAnAspect: Specified target object was not an aspect" };
resource 'Estr' (-15007) { "kDETInvalidCommandItemNumber: Invalid (out of range) command item number" };
resource 'Estr' (-15008) { "kDETUnableToGetCommandItemSpec: Unable to retrieve item specification (possibly out of memory)" };
resource 'Estr' (-15009) { "kDETRequestedTypeUnavailable: Could not represent command item in the form requested" };
resource 'Estr' (-15010) { "kDETInvalidDSSpec: Could not resolve Catalog Specifier (DSSpec)" };
resource 'Estr' (-15011) { "kDETUnableToAccessProperty: Could not find and/or create property" };
resource 'Estr' (-15012) { "kDETInfoPageNotOpen: This request requires an open info-page" };
resource 'Estr' (-15013) { "kDETNoSuchView: No view found with specified property number" };
resource 'Estr' (-15014) { "kDETCouldNotAddMenuItem: Could not add item to dynamic menu" };
resource 'Estr' (-15015) { "kDETCouldNotRemoveMenuItem: Could not remove item from dynamic menu" };
resource 'Estr' (-15016) { "kDETCouldNotFindMenuItem: Could not find menu item" };
resource 'Estr' (-15017) { "kDETCouldNotFindCustomView: Could not find custom view in info-page" };
resource 'Estr' (-15018) { "kDETInvalidReqFunction: Invalid call-back request function" };
resource 'Estr' (-15019) { "kDETInvalidCallBack: Invalid call-back" };
resource 'Estr' (-15020) { "kDETPropertyBusy: This property is currently being edited" };
#endif /* __AOCE_Exclude_Template_Errors */

#ifndef __AOCE_Exclude_Mail_Errors
/*
 * AOCE Mail Errors (-15040 to -15089)
 */
resource 'Estr' (-15040) { "kMailInvalidOrder: This object has already been written to and cannot be written again" };
resource 'Estr' (-15041) { "kMailInvalidSeqNum: Invalid letter sequence or reference to a nonexistent letter" };
resource 'Estr' (-15042) { "kMailAttrNotInHdr: Attribute does not exist in this letter header" };
resource 'Estr' (-15043) { "kMailHdrAttrMissing: Required attribute not written into letter header" };
resource 'Estr' (-15044) { "kMailBadEnclLengthErr: Incorrect number of bytes has been written after memForm of PutEnclosure" };
resource 'Estr' (-15045) { "kMailInvalidRequest: Invalid request; cannot putAttr for report, putRcptReport for letter" };
resource 'Estr' (-15046) { "kMailInvalidPostItVersion: Cannot create new postit with this version" };
resource 'Estr' (-15047) { "kMailNotASlotInQ: Queue identifier does not refer to a mail slot's incoming queue" };
resource 'Estr' (-15048) { "kMailLtrNotOpen: Letter is not open, probably not cached in incoming queue" };
resource 'Estr' (-15049) { "kMailMissingMailCat: Trying to create message in incoming queue, with no postIt" };
resource 'Estr' (-15050) { "kMailCannotWriteMailCat: Cannot update a mail catentry" };
resource 'Estr' (-15051) { "kMailTooManyTagsPerLetter: Trying to add too many tags to a letter" };
resource 'Estr' (-15052) { "kMailGwyDead: Message gateway (MSAM) died" };
resource 'Estr' (-15053) { "kMailIgnoredErr: ePPC message was ignored" };		/* ?? */
resource 'Estr' (-15054) { "kMailLengthErr: ePPC message did not complete" };	/* ?? */
resource 'Estr' (-15055) { "kMailTooManyErr: Too many requests outstanding" };
resource 'Estr' (-15056) { "kMailNoMSAMErr: No Message gateway (MSAM) found in mailbox or for command" };
resource 'Estr' (-15057) { "kMailCancelled: Mail request cancelled" };
resource 'Estr' (-15058) { "kMailSlotSuspended: Mail slot suspended" };
resource 'Estr' (-15059) { "kMailMSAMSuspended: Message gateway suspended" };
resource 'Estr' (-15060) { "kMailBadSlotInfo: Bad mail slot data" };
resource 'Estr' (-15061) { "kMailMalformedContent: Incorrectly formed mail structure" };
resource 'Estr' (-15062) { "kMailNoSuchSlot: No such maiil slot" };
resource 'Estr' (-15063) { "kMailBadLtrInfo: Bad letter information; cannot change it" };
resource 'Estr' (-15064) { "kMailSetFailed: Could not set letter information" };
resource 'Estr' (-15065) { "kMailBadConfig: Message gateway (MSAM) incorrectly configured" };
resource 'Estr' (-15066) { "kMailBadMSAM: Message gateway (MSAM) is unusable" };
resource 'Estr' (-15067) { "kMailCannotLkupServerSlots: Cannot locate server mail slots" };
resource 'Estr' (-15068) { "kMailBadState: Mailer is in the wrong state" };
#endif /* __AOCE_Exclude_Mail_Errors */

#ifndef __AOCE_Exclude_Messaging_Errors
/*
 * AOCE Messaging Errors (-15090 to -15169)
 */
resource 'Estr' (-15090) { "kIPMCantCreateIPMCatEntry: ??" };
resource 'Estr' (-15091) { "kIPMInvalidMsgType: ??" };
resource 'Estr' (-15092) { "kIPMInvalidProcHint: ??" };
resource 'Estr' (-15093) { "kIPMInvalidOffset: ??" };
resource 'Estr' (-15094) { "kIPMUpdateCatFailed: ??" };
resource 'Estr' (-15095) { "kIPMMsgTypeReserved: ??" };
resource 'Estr' (-15096) { "kIPMNotInABlock: ??" };
resource 'Estr' (-15097) { "kIPMNestedMsgOpened: ??" };
resource 'Estr' (-15098) { "kIPMA1HdrCorrupt: ??" };
resource 'Estr' (-15099) { "kIPMCorruptDataStructures: ??" };
resource 'Estr' (-15100) { "kIPMAbortOfNestedMsg: ??" };
resource 'Estr' (-15101) { "kIPMBlockIsNotNestedMsg: ??" };
resource 'Estr' (-15102) { "kIPMCacheFillError: ??" };
resource 'Estr' (-15103) { "kIPMInvalidSender: ??" };
resource 'Estr' (-15104) { "kIPMNoRecipientsYet: ??" };
resource 'Estr' (-15105) { "kIPMInvalidFilter: ??" };
resource 'Estr' (-15106) { "kIPMAttrNotInHdr: ??" };
resource 'Estr' (-15107) { "kIPMBlkNotFound: ??" };
resource 'Estr' (-15108) { "kIPMStreamErr: ??" };
resource 'Estr' (-15109) { "kIPMPortClosed: ??" };
resource 'Estr' (-15110) { "kIPMBinBusy: ??" };
resource 'Estr' (-15111) { "kIPMCorruptedBin: ??" };
resource 'Estr' (-15112) { "kIPMBadQName: ??" };
resource 'Estr' (-15113) { "kIPMEndOfBin: ??" };
resource 'Estr' (-15114) { "kIPMBinNeedsConversion: ??" };
resource 'Estr' (-15115) { "kIPMMgrInternalErr: ??" };
resource 'Estr' (-15116) { "kIPMEltBusy: ??" };
resource 'Estr' (-15117) { "kIPMEltClosedNotDeleted: ??" };
resource 'Estr' (-15118) { "kIPMBadContext: ??" };
resource 'Estr' (-15119) { "kIPMContextIsClosing: ??" };
resource 'Estr' (-15120) { "kIPMeoQ: ??" };
resource 'Estr' (-15121) { "kIPMQOutOfSpace: ??" };
resource 'Estr' (-15122) { "kIPMEltNotFound: ??" };
resource 'Estr' (-15123) { "kIPMBinOutOfSpace: ??" };
resource 'Estr' (-15124) { "kIPMeoCatEntry: ??" };
resource 'Estr' (-15125) { "kIPMeoElt: ??" };
resource 'Estr' (-15126) { "kIPMQBusy: ??" };
resource 'Estr' (-15127) { "kIPMNotSameBin: ??" };
resource 'Estr' (-15128) { "kIPMNoMoreServices: ??" };
resource 'Estr' (-15129) { "kIPMLookupAttrTooBig: ??" };
resource 'Estr' (-15130) { "kIPMUnknownUAM: ??" };
resource 'Estr' (-15131) { "kIPMUnsupportedLogin: ??" };
resource 'Estr' (-15132) { "kIPMLogInUnknown: ??" };
resource 'Estr' (-15133) { "kIPMNoMoreFilters: ??" };
resource 'Estr' (-15134) { "kIPMCouldNotReadInfoReply: ??" };
resource 'Estr' (-15135) { "kIPMInvalidUser: ??" };
resource 'Estr' (-15136) { "kIPMNoMoreSessions: ??" };
resource 'Estr' (-15137) { "kIPMNilUserProc: ??" };
resource 'Estr' (-15138) { "kIPMBadCatSize: ??" };
resource 'Estr' (-15139) { "kIPMIncompleteMsg: ??" };
resource 'Estr' (-15140) { "kIPMNoResponsibleTunnel: ??" };
resource 'Estr' (-15141) { "kIPMAccessDenied: ??" };
resource 'Estr' (-15142) { "kIPMReplyParamUnderrun: Too few reply parameters" };
resource 'Estr' (-15143) { "kIPMReplyParamOverrun: Too many reply parameters" };
resource 'Estr' (-15144) { "kIPMRequestParamUnderrun: Too few request parameters" };
resource 'Estr' (-15145) { "kIPMOutOfSync: Protocol synchronization error" };
resource 'Estr' (-15146) { "kIPMNoAttrsFound: Could not find attributes in lookup" };
resource 'Estr' (-15147) { "kIPMWrongAttrTag: Wrong attribute tag in server lookup" };
resource 'Estr' (-15148) { "kIPMNullCID: No creation id in enumeration" };
resource 'Estr' (-15149) { "kIPMBadMailSlotAttrVal: Invalid mail slot attribute value" };
#endif /* __AOCE_Exclude_Messaging_Errors */
