/* file: BigEasy2.c
 *
 * Started 30 June 1988, more or less.
 *
 * A set of routines to allow the quick development
 * of simple Macintosh applications.
 *
 */


#define BigEasy2

#include "colortoolbox.h"
#include "EventMgr.h"
#include "MenuMgr.h"
#include "BigEasy2.h"

/************************************
* Limits
************************************/
#define kMMax 15			/* Number of menus in menubar */
#define kMIMax 15			/* Number of items per menu */

#define kStagStepX 7
#define kStagStepY 7
#define kStagLimX 97
#define kStagLimY 37

/************************************
* Types and globals
************************************/

Rect gResizeLim = {50,50,1000,1000};
tWindowObject **gWindowObjectListH;		/* Handle to array of window objects		*/
tWindowObject *gWindowObjectList;			/* Pointer to array of window objects		*/
s16 gWindowObjectListSize;				/* Number of window objects allocated		*/

/*** Menus ***/
MenuHandle gMenuHandleList[kMMax];		/* MenuHandles indexed by MenuID's 		*/
s16 gMenuLengths[kMMax];				/* MenuHandles indexed by MenuID's 		*/
FP gMenuProcs[kMMax][kMIMax];			/* Array of procedures to call from menus	*/
s16 gMenuRefs[kMMax][kMIMax];			/* Array of menu reference numbers		*/
s16 gMenuCount;						/* How many menus defined				*/
s16 gMenuItemCount;					/* How many items in last menu			*/
MenuHandle gMenuEdit;					/* Which one is the edit menu?				*/

/*** About ***/
FP gAboutProc;
s8 gAboutS0[256];

/*** Application Edit Commands ***/
FP gAppUndoProc,gAppCutProc,gAppCopyProc,gAppPasteProc,gAppClearProc;

/*** Global Status ***/
FP gMasterLoopyProc;
s16 gStagX,gStagY;
WindowPeek gLastFrontLoopy;
s16 gLastFrontWindowNumber;

/************************************
* Prototypes
************************************/
static void InitToolbox(void);
static void EventLoop(void);
static void InitBEVars(void);
static void SetGrowRect(WindowPeek,Rect*);
static void EnoughWindowObjects(s16);
static void EnDis(MenuHandle,s16,s16);
static void FrontDAEdits(void);
void EnDisEdits(s16 Eundo,s16 Ecut,s16 Ecopy,s16 Epaste,s16 Eclear);

/************************************
* Some useful routines
************************************/

void InitToolbox()
	{
	InitGraf(&thePort);
	InitFonts();
	FlushEvents(0xffff,0);
	InitWindows();
	InitMenus();
	InitDialogs(0);
	TEInit();
	InitCursor();
	}


void SetGrowRect(w,r)
	WindowPeek w;
	Rect *r;
	{
	register Rect *gr;

	gr = r;
	*gr = w->port.portRect;
	gr->left = gr->right - 15;
	gr->top = gr->bottom - 15;
	}

static void EnoughWindowObjects(n)
/*
  * If there are less than n window objects in the list,
  * grow the handle appropriately.
  */
	s16 n;
	{
	s16 i;

	n++;
	if(gWindowObjectListSize < n)
		{
		HUnlock(gWindowObjectListH);
		SetHandleSize(gWindowObjectListH,sizeof(tWindowOdoect) * (long)n);
		HLock(gWindowObjectListH);
		gWindowObjectList = *gWindowObjectListH;

		for(i = gWindowObjectListSize; i < n; i++)
			gWindowObjectList[i].wUsed = 0;

		gWindowObjectListSize = n;
		}
	}

static tWindowObject *GoodWNum(n)
/*
  * return true if n is the number of an
  * existing, used window.
  */
	s16 n;
	{
	register tWindowObject *thisWindow;
	if(n<1 || n>=gWindowObjectListSize)
		return 0;
	thisWindow = &gWindowObjectList[n];
	if(thisWindow->wUsed)
		return thisWindow;
	else
		return 0;
	}

static void EnoughSubwindows(wNum,n)
/*
  * If there are less than n Subwindows in window wNum,
  * make some more.
  */
	s16 wNum,n;
	{
	s16 i;
	register s16 swCount;
	register tWindowObject *thisWindowObject;
	tSubwindow *thisSubwindow;

	if(!(thisWindowObject = GoodWNum(wNum)))		/* Find the window			*/
		return;

	n++;
	swCount = thisWindowObject->wSubwindowCount;
	if(!swCount)								/* No subwindows yet?		*/
		{
		thisWindowObject->wSubwindowsH = (tSubwindow **)NewHandle(sizeof(tSubwindow));
		(*thisWindowObject->wSubwindowsH)[0].vUsed = 0;
		swCount = 0;
		}

	if(swCount < n)
		{
		HUnlock(thisWindowObject->wSubwindowsH);
		SetHandleSize(thisWindowObject->wSubwindowsH,sizeof(tSubwindow) * (long)n);
		HLock(thisWindowObject->wSubwindowsH);

		thisSubwindow = &(*thisWindowObject->wSubwindowsH)[swCount];
		for(i = swCount; i < n; i++)
			thisSubwindow++->vUsed = 0;

		thisWindowObject->wSubwindowCount = n;
		}
	}

void SetSubport(wNum,aSub,outR)
/*
  * set the clip and origin to the subport passed.
  * if outRnil, set it to the local rect bounds.
  * If aSub < 0 then set port to the whole window,
  * and leave outR alone.
  */

	s16 wNum;
	s16 aSub;
	Rect *outR;
	{
	register tSubwindow *thisSubwindow;
	register tWindowObject *thisWindow;
	Rect r;

	if(!(thisWindow = GoodWNum(wNum)))
		return;								/* find the window			*/

	if(aSub>thisWindow->wSubwindowCount)			/* too high a subport?		*/
		return;

	SetPort(thisWindow->wWindow);
	SetOrigin(0,0);

	if(aSub >= 0)								/* whole window?			*/
		{
		thisSubwindow = &(*thisWindow->wSubwindowsH)[aSub];
		if(!thisSubwindow->vUsed)
			return;
	
		r = thisSubwindow->vRect;
		SetOrigin(-r.left,-r.top);
		OffsetRect(&r,-r.left,-r.top);
		ClipRect(&r);
		if(outR)
			*outR = r;
		}
	else
		ClipRect(&gBigRect);					/* whole window: big clip		*/
	}

void InvalSubport(wNum,iNum)
/*
  * Force a redraw of just one subport.
  */
	s16 wNum;
	s16 iNum;
	{
	register tWindowObject *thisWindow;
	register tSubwindow *thisSubwindow;
	short i;

	if(wNum >= gWindowObjectListSize)
		return;

	thisWindow = &(*gWindowObjectListH)[wNum];
	if(iNum >= thisWindow->wSubwindowCount)
		return;

	thisSubwindow = &(*thisWindow->wSubwindowsH)[iNum];

	if(!thisSubwindow->vUsed)
		return;

	SetPort(thisWindow->wWindow);
	SetOrigin(0,0);
	InvalRect(&thisSubwindow->vRect);
	}

void InvalSubports(wNum,ref)
/*
  * Force a redraw of all subports of the
  * specified window with the specified refnum.
  */
	s16 wNum;
	s32 ref;
	{
	register tWindowObject *thisWindow;
	register tSubwindow *thisSubwindow;
	short i;

	if(wNum >= gWindowObjectListSize)
		return;

	thisWindow = &(*gWindowObjectListH)[wNum];
	if(!thisWindow->wSubwindowCount)
		return;

	SetPort(thisWindow->wWindow);
	SetOrigin(0,0);

	thisSubwindow = &(*thisWindow->wSubwindowsH)[0];
	for(i = 0; i < thisWindow->wSubwindowCount; i++)
		{
		if(thisSubwindow->vUsed && thisSubwindow->vRefcon == ref)
			InvalRect(&thisSubwindow->vRect);
		thisSubwindow++;
		}
	}

static void EnDis(m,i,f)
/*
 * Enable or disable item i of menu m.
 */
	MenuHandle m;
	s16 i,f;
	{
	if (f==0)
		DisableItem(m,i);
	else if (f > 0)
		EnableItem(m,i);
	}

static void FrontDAEdits()
/*
 * Check the front window. If it's a DA,
 * enable all edit menu options.
 */
	{
	WindowRecord *w;

	w = (WindowRecord*)FrontWindow();
	if(w->windowKind < 0)									/* Any click in a DA window: */
		EnDisEdits(1,1,1,1,1);								/* enable edit menu. */
	}


DoMyAbout()
	{
	DialogPtr d;
	Handle h;

	if(gAboutProc)
		(*gAboutProc)();
	else
		{
		h = GetResource('DLOG',1962);							/* does it exist? */
		if( (ResErr==0) && (h!=nil) )
			{
			ParamText(gAboutS0,0,0,0);
			d = GetNewDialog(1962,0,-1);
			DrawDialog(d);
			while(!Button())
				EventLoop();
			DisposDialog(d);
			}
		}
	}

SetMasterLoopy(r)
	FP r;
	{
	gMasterLoopyProc = r;
	}

SetAbout(progName,s0)
/*
 * Sets the program's name in the about box, and
 * sets the second string as ParamText 0 (^0)
 * for the about dialog, 1962.
 */
	char *progName,s0;
	{
	s16 i;
	
	SetItem(gMenuHandleList[1],1,progName);

	for(i=0; i<256; i++)
		gAboutS0[i] = s0++;
	} 

/************************************
* Error Handling
************************************/

void FailNil(x)
/*
  * If x is zero, post an error
  * alert and quit the program.
  */
	long x;
	{
	if(!x)
		{
		Alert(200,nil);
		ExitToShell();
		}
	}

/************************************
* Some inherent methods
************************************/

BEUndo(n)
	short n;
	{
	if(!SystemEdit(0))
		(*gAppUndoProc)(n);
	}

BECut()
	{
	if(!SystemEdit(2))
		(*gAppCutProc)();
	}

BECopy()
	{
	if(!SystemEdit(3))
		(*gAppCopyProc)();
	}

BEPaste()
	{
	if(!SystemEdit(4))
		(*gAppPasteProc)();
	}

BEClear()
	{
	if(!SystemEdit(5))
		(*gAppClearProc)();
	}

BENull(){}


/************************************
* Menu Action Routines
************************************/

DoKeyPress(k)
	long k;
	{
	MenuPoint(MenuKey((char)k));
	}
	
MenuClick(p)
	Point p;
	{
	MenuPoint(MenuSelect(p));
	}

MenuPoint(theclick)
	long theclick;
	{
	s16 mID,mItem;
	char DAname[30];
	WindowPeek w;
	short wNum;
	register FP aProc;

	mID = theclick>>16;
	mItem = theclick & 0xffff;

	if(mID == 0) return;

	if(mID == 1)							/* Apple menu */
		{
		if(mItem == 1)						/* Its either the about box */
			DoMyAbout();
		else	
			{									/* Or a DA */
			GetItem(gMenuHandleList[1],mItem,DAname);
			OpenDeskAcc(DAname);
			}
		}
	else
		{
		w = (WindowPeek)FrontWindow();
		wNum = ScanWindowList(w);
		aProc = gMenuProcs[mID][0];
		if(aProc)										/* call the menu's routine, if any		*/
			(*aProc)(wNum,mItem,gMenuRefs[mID][0]);
		aProc = gMenuProcs[mID][mItem];
		if(aProc)										/* call the item's routine, if any		*/
			(*aProc)(wNum,mItem,gMenuRefs[mID][mItem]);
		}

	HiliteMenu(0);	
	}

StartMenus()
/*
 * Start with just an About box, and some DA's.
 */
	{
	MenuHandle applemenu;
	applemenu = NewMenu(1,"\p");				/* Apple menu: ID 1					*/
	AppendMenu(applemenu,"\pAbout...;(-");
	AddResMenu(applemenu,'DRVR');
	InsertMenu(applemenu,0);
	DrawMenuBar();
	gMenuHandleList[1] = applemenu;

	gMenuCount = 1;							/* Next menu added will be ID 2		*/
	gMenuLengths[1] = 0;						/* No items, really, in Apple menu		*/
	}

void InstallMenu(s,action,ref)
/*
 * Start a new menu with name s
 */
	char s[];
	FP action;
	s16 ref;
	{
	gMenuCount++;								/* new menu						*/
	gMenuItemCount = 0;							/* with no entries					*/

	gMenuLengths[gMenuCount] = 0;
	gMenuHandleList[gMenuCount] = NewMenu(gMenuCount,s);
	InsertMenu(gMenuHandleList[gMenuCount],0);
	gMenuProcs[gMenuCount][gMenuItemCount] = action;
	gMenuRefs[gMenuCount][gMenuItemCount] = ref;
	gMenuItemCount++;
	DrawMenuBar();
	}

void InstallItem(s,action,ref)
/*
 * Add an item to the last menu, and associate a routine with it.
 */
	char s[];
	FP action;
	s16 ref;
	{
	register MenuHandle mh;

	mh = gMenuHandleList[gMenuCount];

	if (ref < 0)
		{
		DisableItem(mh,gMenuItemCount);
		ref =- ref;
		}

	AppendMenu(mh,s);
	gMenuProcs[gMenuCount][gMenuItemCount] = action;
	gMenuRefs[gMenuCount][gMenuItemCount] = ref;
	gMenuLengths[gMenuCount] = ++gMenuItemCount;
	}

InstallEditMenu(Xundo,Xcut,Xcopy,Xpaste,Xclear)
/*
 * Start an edit menu, and put the first six things in.
 */
	FP Xundo,Xcut,Xcopy,Xpaste,Xclear;
	{
	gAppUndoProc = Xundo;
	gAppCutProc = Xcut;
	gAppCopyProc = Xcopy;
	gAppPasteProc = Xpaste;
	gAppClearProc = Xclear;

	InstallMenu("\pEdit",nil,0);
	InstallItem("\px/Z",BEUndo,-1);
	InstallItem("\p(-",BENull,0);
	InstallItem("\px/X",BECut,-2);
	InstallItem("\px/C",BECopy,-3);
	InstallItem("\px/V",BEPaste,-4);
	InstallItem("\px/B",BEClear,-5);

	gMenuEdit = gMenuHandleList[gMenuCount];

	}

void SetMenuItem(ref,enable,isMarked,mark,s)
/*
  * Set all items with the reference number to the
  * enable/disable state, with mark, and name s.
  * pass enable+ to enable, zero to disable, and
  * enable - to leave alone.
  */
	s16 ref;
	boolean enable;
	boolean isMarked;
	char mark;
	char *s;
	{
	s16 m,i;
	MenuHandle mh;

	for(m=1; m<=gMenuCount; m++)
		{
		mh = gMenuHandleList[m];
		for(i=gMenuLengths[m]; i>=1; i--)
			if(gMenuRefs[m][i] == ref)
				{
				if(enable > 0)
					EnableItem(mh,i);
				else if (enable == 0)
					DisableItem(mh,i);

				if(isMarked)
					SetItemMark(mh,i,mark);
				else
					SetItemMark(mh,i,' ');
				if(s)
					SetItem(mh,i,s);
				}
		}
	}






void EnDisEdits(Eundo,Ecut,Ecopy,Epaste,Eclear)
/*
 * For each edit menu entry,
 * 0=disable, 1=enable, -1=leave alone.
 */
	s16 Eundo,Ecut,Ecopy,Epaste,Eclear;
	{
	SetMenuItem(1,Eundo,false,0,"\pUndo");
	SetMenuItem(2,Ecut,false,0,"\pCut");
	SetMenuItem(3,Ecopy,false,0,"\pCopy");
	SetMenuItem(4,Epaste,false,0,"\pPaste");
	SetMenuItem(5,Eclear,false,0,"\pClear");
	}

s16 ScanWindowList(w)
	WindowPeek(w);
	{
	s16 i;
	tWindowObject *wo;

	wo= &gWindowObjectList[1];

	for(i=1; i<gWindowObjectListSize; i++)
		{
		if (wo->wUsed && (wo->wWindow==w))
			return i;
		wo++;
		}
	return 0;
	}


/************************************
* Mouse Action Routines
************************************/
void DoMouseClick(event)
	EventRecord *event;
	{
	WindowPeek w;
	s16 part;
	s16 wNum;
	s16 i;
	tWindowObject *thisWindowObject;
	tSubwindow *thisSubwindow;
	Point p,subP;
	Rect r;
	boolean subFound;

	p = event->where;
	part = FindWindow (p, &w);
	wNum = ScanWindowList(w);
	thisWindowObject = &gWindowObjectList[wNum];

	switch (part)
		{
		case 0:		/* click on desk */
			break;

		case 1:		/* menubar */
			MenuClick(p);
			break;
		
		case 2:		/* system window */
			SystemClick(event,w);
			break;
		
		case 3:		/* content region */
		contentClick:
			if (w != (WindowPeek)FrontWindow())				/* If clicked on a non-front window, */
				SelectWindow(w);							/* bring it to the front. */
			else if(wNum)									/* Click on front window: give click */
				{										/* to window's click routine. */
				SetPort(w);
				GlobalToLocal(&p);
				subFound = 0;
				for(i=thisWindowObject->wSubwindowCount-1; i>=0; i--)
					{
					thisSubwindow = &(*thisWindowObject->wSubwindowsH)[i];
					if(thisSubwindow->vUsed && PtInRect(p,&thisSubwindow->vRect))
						{
						subFound = 1;
						subP = p;
						subP.v -= thisSubwindow->vRect.top;
						subP.h -= thisSubwindow->vRect.left;
						SetSubport(wNum,i,&r);
						(*thisSubwindow->vClickProc)(wNum,i,&r,
								thisSubwindow->vRefcon,subP,event->modifiers);
						}
					}

				ClipRect(&gBigRect);
				SetOrigin(0,0);

				if(!subFound)								/* only click in main window		*/
					(*thisWindowObject->wClickProc)(wNum,p,
							event->modifiers);				/* if no subwindows found		*/
				}
			break;

		case 5:		/* growregion */
			if(wNum && thisWindowObject->wGrowable)
				{
				long newSize;

				SetPort(w);
				SetGrowRect(w,&r);
				InvalRect(&r);
				newSize = GrowWindow(w,event->where,&gResizeLim);
				SizeWindow(w,newSize&0xffff,newSize>>16,1);
				SetGrowRect(w,&r);
				InvalRect(&r);
				}
			else
				goto contentClick;
			break;

		case 4:		/* drag */
			DragWindow (w,event->where,&gBigRect);
			break;

		case 6:		/* goAway */
			if(wNum)
				if (TrackGoAway(w,event->where))
					{
					(*thisWindowObject->wGoAwayProc)(wNum);
					}
			break;
		}
	}

/************************************
* Window Action Routines
************************************/

void InstallWindow(iNum,iTitle,iRect,iType,iGrow,
		iUpdate,iClick,iKey,iGoAway,
		iActivate,iDeactivate,iLoopy)
/*
 * Add a window to BigEasy's list. If the window is
 * already up somewhere, bring it to the front and
 * visualize it.
 */
 	s16 iNum;				/* internal window slot to use			*/
	s8 *iTitle;
	Rect *iRect;			/* bounds of window					*/
	s16 iType;			/* window type					*/
	s16 iGrow;
	
	FP iUpdate;
	FP iClick;
	FP iKey;
	FP iGoAway;
	FP iActivate;
	FP iDeactivate;
	FP iLoopy;
	{
	tWindowObject *thisWindow;
	Rect stagRect;

	EnoughWindowObjects(iNum);			/* Make sure there's space for new one		*/
	thisWindow = &gWindowObjectList[iNum];

	if (thisWindow->wUsed)				/* something already assigned to this window? */
		{
		if(!thisWindow->wWindow->visible)	/* If so, just show it, and bring it to the front. */
			ShowWindow(thisWindow->wWindow);
		SelectWindow(thisWindow->wWindow);
		return;
		}

	stagRect = *iRect;
	gStagX = (gStagX + kStagStepX)%kStagLimX;
	gStagY = (gStagY + kStagStepY)%kStagLimY;
	OffsetRect(&stagRect,gStagX,gStagY);

	thisWindow->wUsed = 1;
	thisWindow->wWindow = (WindowPeek)NewCWindow(0,&stagRect,iTitle,TRUE,iType,-1,iGoAway!=0,0);
	thisWindow->wGrowable = iGrow;
	thisWindow->wUpdateProc = iUpdate;
	thisWindow->wClickProc = iClick;
	thisWindow->wKeyProc = iKey;
	thisWindow->wGoAwayProc = iGoAway;
	thisWindow->wActivateProc = iActivate;
	thisWindow->wDeactivateProc = iDeactivate;
	thisWindow->wLoopyProc = iLoopy;
	thisWindow->wSubwindowCount = 0;
	}

void UninstallWindow(iNum)
	s16 iNum;
	{
	tWindowObject *thisWindow;

	if(!(thisWindow = GoodWNum(iNum)))
		return;

	(*thisWindow->wDeactivateProc)(iNum);
	CloseWindow(thisWindow->wWindow);
	thisWindow->wUsed = 0;

	}

void BringToFront(iNum)
	s16 iNum;
	{
	tWindowObject *thisWindow;

	if(!(thisWindow = GoodWNum(iNum)))
		return;

	SelectWindow(thisWindow->wWindow);
	}

void InstallSubwindow(wNum,iNum,iRect,iRefcon,
		iUpdateProc,iClickProc)
	s16 wNum;
	s16 iNum;
	Rect *iRect;
	s32 iRefcon;
	FP iUpdateProc;
	FP iClickProc;
	{
	register tSubwindow *thisSubwindow;
	tWindowObject *thisWindow;

	thisWindow = &gWindowObjectList[wNum];
	if(!thisWindow->wUsed)
		return;

	EnoughSubwindows(wNum,iNum);
	thisSubwindow = &(*thisWindow->wSubwindowsH)[iNum];
	if(thisSubwindow->vUsed)								/* this spot already taken?	*/
		return;

	thisSubwindow->vUsed = 1;
	thisSubwindow->vOwner = wNum;
	thisSubwindow->vRect = *iRect;
	thisSubwindow->vRefcon = iRefcon;
	thisSubwindow->vUpdateProc = iUpdateProc;
	thisSubwindow->vClickProc = iClickProc;

	}

void GetWindowRect(n,r)
/*
  * return the window's current rectangle
  * in global cordinate space.
  */
	s16 n;
	Rect *r;
	{
	tWindowObject *thisWindow;
	s16 xo,yo;
	GrafPort *g;

	if(!(thisWindow = GoodWNum(n)))
		return;

	if(thisWindow->wUsed)
		{
		g = &(thisWindow->wWindow)->port;
		*r = g->portRect;
		xo = g->portBits.bounds.left;
		yo = g->portBits.bounds.top;
		r->top -= xo;
		r->bottom -= xo;
		r->left -= yo;
		r->right -= yo;
		}
	}
		


/************************************
* Event Routines
************************************/

void EventLoop()
	{
	EventRecord ER;
	s16 i;
	s16 wNum;
	WindowPeek w;
	tWindowObject *thisWindowObject;
	tSubwindow *thisSubwindow;
	Rect r;

	if (GetNextEvent(0xffff, &ER) )
		{

		if(ER.what > 5)									/* update event or higher: in message	*/
			w = (WindowPeek)ER.message;
		else
			w = (WindowPeek)FrontWindow();				/* else, use FrontW					*/

		if(ER.what > 1)
			{
			wNum = ScanWindowList(w);					/* other than null or click, scan list		*/
			thisWindowObject = &gWindowObjectList[wNum];	/* and get record					*/
			}

		switch (ER.what)
			{
			case 0:	/* null event */
				break;

			case 1:	/* mouse down */
				FrontDAEdits();
				DoMouseClick(&ER);
				break;

			case 3:
			case 5:
				FrontDAEdits();
				if( (!gMenuNeedsCmdKey) || (ER.modifiers & 256) )
					DoKeyPress(ER.message);
				else
					if(wNum)
						{
						SetSubport(wNum,-1,nil);						/* Set port to window	*/
						(*thisWindowObject->wKeyProc)(wNum,			/* Call key proc		*/
								(s16)(ER.message&0xff),
								(s16)((ER.message>>8)&0xff),
								ER.modifiers);
						}
				break;

			case 6:	/* update */
				if(wNum)
					{
					Rect visBounds,dummyRect;

					BeginUpdate(&w->port);
					visBounds = (**w->port.visRgn).rgnBBox;
					SetSubport(wNum,-1,nil);							/* Set port to window	*/
					(*thisWindowObject->wUpdateProc)(wNum);			/* Call drawing proc		*/

					for(i = thisWindowObject->wSubwindowCount-1; i>=0; i--)
						{
						thisSubwindow = &(*thisWindowObject->wSubwindowsH)[i];
						if(thisSubwindow->vUsed &&
								SectRect(&visBounds,&thisSubwindow->vRect,&dummyRect) )
							{
							SetSubport(wNum,i,&r);
							(*thisSubwindow->vUpdateProc)(wNum,i,&r,thisSubwindow->vRefcon);
							}
						}

					SetOrigin(0,0);
					ClipRect(&gBigRect);

					if (thisWindowObject->wGrowable)					/* Draw growbox, if...	*/
						{
						SetGrowRect(w,&r);
						ForeColor(blackColor);
						PaintRect(&r);
						InsetRect(&r,2,2);
						InvertOval(&r);
						}

					EndUpdate(ER.message);
					}
				break;

			case 8:	/* activate */
				FrontDAEdits();
				if(wNum)
					if(ER.modifiers & 1)
						(*thisWindowObject->wActivateProc)(wNum);
					else
						{
						EnDisEdits(0,0,0,0,0);
						(*thisWindowObject->wDeactivateProc)(wNum);
						}
				break;
			}
		}

	SystemTask();
	w = (WindowPeek)FrontWindow();

	if ( w == gLastFrontLoopy)				/* Minor optimization, wins if the same		*/
		wNum = gLastFrontWindowNumber;		/* window is frontmost for a while.			*/
	else
		{
		wNum = ScanWindowList(w);
		gLastFrontLoopy = w;
		gLastFrontWindowNumber = wNum;
		}

	if(wNum)
		(*gWindowObjectList[wNum].wLoopyProc)();

	if (gMasterLoopyProc)
		(*gMasterLoopyProc)();
	}

void InitBEVars()
	{
	s16 i;

	gMasterLoopyProc = 0;
	gAboutProc = 0;
	gQuitApp = 0;
	gMenuNeedsCmdKey = 1;

	SetRect(&gBigRect,-16000,-16000,16000,16000);


	gWindowObjectListH = (tWindowObject**)NewHandle(sizeof(tWindowObject));
	HLock(gWindowObjectListH);
	gWindowObjectList = *gWindowObjectListH;
	gWindowObjectList[0].wUsed = 0;
	gWindowObjectListSize = 1;

	gLastFrontLoopy = (WindowPeek) 0;
	gLastFrontWindowNumber = 0;
	}

main()
	{
	InitToolbox();
	StartMenus();
	
	InitBEVars();
	Bootstrap();
	
	while (!gQuitApp)
		EventLoop();

	FlushEvents(0xffff,0);
	ExitToShell();
	}
