//
//	CMIntf.h
//	C Interface to the Connection Manager
//	
//	Copyright  Apple Computer, Inc. 1988, 1989
//	All rights reserved
//
//

#ifndef	__CMINTF__
#define __CMINTF__

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __CTBUTILS__
#include <CTBUtils.h>
#endif

//	current Connection Manager version 
#define	curCMVersion 		1

//	current Connection Manager Environment Record version 
#define	curConnEnvRecVers	0
	
// CMErr 
typedef	OSErr CMErr;

#define cmGenericError 		-1
#define	cmNoErr				0
#define cmRejected			1
#define cmFailed			2
#define cmTimeOut			3
#define cmNotOpen			4
#define cmNotClosed			5
#define cmNoRequestPending	6
#define cmNotSupported 		7
#define cmNoTools			8	


// CMRecFlags and CMChannel
//		Low word of CMRecFlags is same as CMChannel
typedef long CMRecFlags;
typedef short CMChannel;

enum {
 	cmData 			= 1 << 0,
	cmCntl 			= 1 << 1,
	cmAttn 			= 1 << 2,
	
	cmDataClean 	= 1 << 8,		
	cmCntlClean		= 1 << 9,
	cmAttnClean		= 1 << 10,

//		Only for CMRecFlags (not CMChannel) in the rest of this enum
	cmNoMenus 		= 1 << 16,	
	cmQuiet			= 1 << 17
};

//	connection status flags
typedef long CMStatFlags;

enum {
	cmStatusOpening		= 1 << 0,
	cmStatusOpen 		= 1 << 1,
	cmStatusClosing 	= 1 << 2,
	cmStatusDataAvail 	= 1 << 3,
	cmStatusCntlAvail 	= 1 << 4,
	cmStatusAttnAvail 	= 1 << 5,
	
	cmStatusDRPend 		= 1 << 6,		// data read pending
	cmStatusDWPend 		= 1 << 7,		// data write pending
	cmStatusCRPend 		= 1 << 8,		// cntl read pending
	cmStatusCWPend 		= 1 << 9,		// cntl write pending
	cmStatusARPend 		= 1 << 10,		// attn read pending
	cmStatusAWPend 		= 1 << 11,		// attn write pending

	cmStatusBreakPend 	= 1 << 12,
	cmStatusListenPend 	= 1 << 13,
	cmStatusIncomingCallPresent 
						= 1 << 14
};

//	CMBuffers	fields
typedef short CMBufFields;

enum {
	cmDataIn,
	cmDataOut,
	
	cmCntlIn,
	cmCntlOut,
	
	cmAttnIn,
	cmAttnOut,
	
	cmRsrvIn,
	cmRsrvOut
};

typedef Ptr CMBuffers[8];
typedef long CMBufferSizes[8];

//	CMSearchFlags
typedef short CMSearchFlags;

enum {
	cmSearchSevenBit =	1 << 0
};

//	CMFlags (CMRead/Write deliminator flags)
typedef short CMFlags;
enum {
	cmFlagsEOM		=	1 << 0
};

struct ConnEnvironRec {
	short		version;
	long		baudRate;
	short		dataBits;
	CMChannel	channels;
	Boolean		swFlowControl;
	Boolean		hwFlowControl;
	CMFlags		flags;
};
#ifndef __cplusplus
typedef struct ConnEnvironRec ConnEnvironRec;
#endif

typedef ConnEnvironRec *ConnEnvironRecPtr;

struct ConnRecord {
	short		procID;
	
	CMRecFlags	flags;
	CMErr		errCode;
	
	long		refCon;
	long		userData;
	
	ProcPtr		defProc;
	
	Ptr			config;
	Ptr			oldConfig;
	
	long		reserved0;
	long		reserved1;
	long		reserved2;

	Ptr			cmPrivate;

	CMBuffers		bufferArray;
	CMBufferSizes	bufSizes;

	long		mluField;
	
	CMBufferSizes	asyncCount;
};

#ifndef __cplusplus
typedef struct ConnRecord ConnRecord;
#endif
typedef ConnRecord *ConnPtr, **ConnHandle;


#ifdef __safe_link
extern "C" {
#endif	
extern pascal CMErr		InitCM(void);
extern pascal Handle 	CMGetVersion(ConnHandle hConn);
extern pascal short 	CMGetCMVersion(void);

extern pascal ConnHandle	CMNew(short procID, CMRecFlags flags, CMBufferSizes desiredSizes, 
								long refCon, long userData);

extern pascal void 		CMDispose(ConnHandle hConn);

extern pascal CMErr 	CMListen(ConnHandle hConn, Boolean async, ProcPtr completor, long timeout);
extern pascal CMErr 	CMAccept(ConnHandle hConn, Boolean accept);

extern pascal CMErr 	CMOpen(ConnHandle hConn, Boolean async, ProcPtr completor, long timeout);
extern pascal CMErr 	CMClose(ConnHandle hConn, Boolean async, ProcPtr completor, long timeout, Boolean now);

extern pascal CMErr 	CMAbort(ConnHandle hConn);

extern pascal void 		CMIdle(ConnHandle hConn);
extern pascal CMErr 	CMStatus(ConnHandle hConn, CMBufferSizes sizes, CMStatFlags *flags);

extern pascal void 		CMReset(ConnHandle hConn);

extern pascal void 		CMBreak(ConnHandle hConn, long duration, Boolean async, 
							ProcPtr completor);

extern pascal CMErr 	CMRead(ConnHandle hConn, Ptr theBuffer, long *toRead, 
							CMChannel theChannel, Boolean async, ProcPtr completor, long timeout, 
							CMFlags *flags);

extern pascal CMErr 	CMWrite(ConnHandle hConn, Ptr theBuffer, long *toWrite,
							CMChannel theChannel, Boolean async, ProcPtr completor, long timeout, 
							CMFlags flags);

extern pascal CMErr 	CMIOKill(ConnHandle hConn, short which);

extern pascal void 		CMActivate(ConnHandle hConn, Boolean activate);
extern pascal void 		CMResume(ConnHandle hConn, Boolean resume);
extern pascal Boolean 	CMMenu(ConnHandle hConn, short menuID, short item);

extern pascal Boolean 	CMValidate(ConnHandle hConn);
extern pascal void 		CMDefault(Ptr *theConfig, short procID, Boolean allocate);

extern pascal Handle 	CMSetupPreflight(short procID, long *magicCookie);
extern pascal Boolean 	CMSetupFilter(short procID, Ptr theConfig, short count, 
							DialogPtr theDialog, EventRecord *theEvent, short *theItem, 
							long *magicCookie);
extern pascal void 		CMSetupSetup(short procID, Ptr theConfig, short count, 
							DialogPtr theDialog, long *magicCookie);
extern pascal void 		CMSetupItem(short procID, Ptr theConfig, short count, 
							DialogPtr theDialog, short *theItem, long *magicCookie);
extern pascal void 		CMSetupCleanup(short procID, Ptr theConfig, short count, 
							DialogPtr theDialog, long *magicCookie);
extern pascal void 		CMSetupPostflight(short procID);

extern pascal Ptr 		CMGetConfig(ConnHandle hConn);
extern pascal short 	CMSetConfig(ConnHandle hConn, Ptr thePtr);

extern pascal OSErr 	CMIntlToEnglish(ConnHandle hConn, 
							Ptr inputPtr, Ptr *outputPtr, short language);
extern pascal OSErr 	CMEnglishToIntl(ConnHandle hConn, 
							Ptr inputPtr, Ptr *outputPtr, short language);

extern pascal long 		CMAddSearch(ConnHandle hConn, const Str255 theString, 
							CMSearchFlags flags, ProcPtr callBack);
extern pascal void		CMRemoveSearch(ConnHandle hConn, long refnum);
extern pascal void		CMClearSearch(ConnHandle hConn);

extern pascal CMErr 	CMGetConnEnvirons(ConnHandle hConn, ConnEnvironRec *theEnvirons);
		
extern pascal short 	CMChoose(ConnHandle *hConn, Point where, ProcPtr idleProc);
extern pascal void		CMEvent(ConnHandle hConn, const EventRecord *theEvent);

extern pascal void 		CMGetToolName(short procID, Str255 name);
extern pascal short 	CMGetProcID(const Str255 name);

extern pascal void		CMSetRefCon(ConnHandle hConn, long refCon);
extern pascal long 		CMGetRefCon(ConnHandle hConn);

extern pascal long 		CMGetUserData(ConnHandle hConn);
extern pascal void 		CMSetUserData(ConnHandle hConn, long userData);

#ifdef __safe_link
}
#endif

#endif