/*
	FilterTables.cp
	
	An MPW Tool to modify CFront's virtual tables to be
	simple declarations.  This is the first pass.  The second
	pass reads the tables and writes a function to initialize them,
	which is appended to the output of this program.
	
	1990 by Patrick Beard.  All rights reserved.
 */

#include <stdio.h>
#include <string.h>
#include <Types.h>
#include <ctype.h>

char line[512];			// a line of input.
char temp[512];			// temporary buffer.
char className[512];	// the current class name (mangled).

// filtering for multiple inheritance classes.

void MIFilterTables()
{
	char *table_starts = "struct __mptr __vtbl__";	// begins each table declaration.
	int starts_match_len = strlen(table_starts);
	char *table_ends = "0,0,0};";					// ends each table.
	int ends_match_len = strlen(table_ends);
	int index;
	int found_table = 0;
	char *s, *d;
	
	while(fgets(line, sizeof(line), stdin)) {
		if(!found_table) {
			if(strncmp(table_starts, line, starts_match_len) == 0) {
				// we've started parsing a table.
				s = &line[starts_match_len]; d = className;
				while( *s != '[') *d++ = *s++;
				*d = 0;
				fprintf(stdout, "/* className = %s */\n", className);
				index = 1;
				found_table = 1;
			} else {
				fputs(line, stdout);
			}
		} else {
			// we're processing a table.
			if(strncmp(table_ends, line, ends_match_len) != 0) {
				index++;
			} else {
				fprintf(stdout, "struct __mptr __vtbl__%s[%d];\n", className, index + 1);
				fprintf(stdout, "struct __mptr *__ptbl__%s;\n", className);
				fgets(line, sizeof(line), stdin); 	// eat __ptbl__ line.
				found_table = 0;	// start over and search for another table.
			}
		}
	}
}

// filtering for single-inheritance classes.

void SIFilterTables()
{
	char *table_starts = "__vptp __vtbl__";		// begins each table declaration.
	int starts_match_len = strlen(table_starts);
	char *table_ends = "0};";					// ends each table.
	int ends_match_len = strlen(table_ends);
	int index;
	int found_table = 0;
	char *s, *d;
	
	while(fgets(line, sizeof(line), stdin)) {
		if(!found_table) {
			if(strncmp(table_starts, line, starts_match_len) == 0) {
				// we've started parsing a table.
				s = &line[starts_match_len]; d = className;
				while( *s != '[') *d++ = *s++;
				*d = 0;
				fprintf(stdout, "/* className = %s */\n", className);
				index = 1;
				found_table = 1;
			} else {
				fputs(line, stdout);
			}
		} else {
			// we're processing a table.
			if(strncmp(table_ends, line, ends_match_len) != 0) {
				index++;
			} else {
				fprintf(stdout, "__vptp __vtbl__%s[%d];\n", className, index + 1);
				fprintf(stdout, "__vptp *__ptbl__%s;\n", className);
				fgets(line, sizeof(line), stdin); 	// eat __ptbl__ line.
				found_table = 0;	// start over and search for another table.
			}
		}
	}
}

Boolean UsingMultipleInheritance(int argc, char** argv)
{
	// by default, we chew on the tables assuming multiple inheritance.
	Boolean useMI = true;
	while(--argc) {
		char *arg = *++argv;
		if(arg[0] == '-') {
			switch(tolower(arg[1])) {
			case 'm':
				useMI = true;
				break;
			case 's':
				useMI = false;
				break;
			}
		}
	}
	
	return useMI;
}

int main(int argc, char** argv)
{	
	if(UsingMultipleInheritance(argc, argv))
		MIFilterTables();
	else
		SIFilterTables();
	return 0;
}
